classdef (StrictDefaults, TunablesDetermineInactiveStatus) SpectrumAnalyzerBase < ...
    matlabshared.scopes.mixin.CommandLineAutoscale & ...
    matlabshared.scopes.UnifiedSystemScope & dynamicprops
    
%SpectrumAnalyzerBase Display frequency-domain signals and the frequency
%spectrum of time-domain signals

%   Copyright 2011-2019 The MathWorks, Inc.


% Declare MATLAB Compiler dependencies for extension manager based files.
% See g2060669. To get a list of the pragmas, call the method
% printExtensionPragmas on the System Object.

%#function matlabshared.scopes.GeneralUIOptions
%#function matlabshared.scopes.SourceUIOptions
%#function matlabshared.scopes.source.SystemObjectSource
%#function dsp.scopes.SpectrumVisual
%#function matlabshared.scopes.tool.PlotNavigation
%#function matlabshared.scopes.tool.Measurements

properties (Nontunable)
  %NumInputPorts Number of input signals
  %   Specify the number of input signals to the Spectrum Analyzer as a
  %   positive integer. You must invoke the step method with the same
  %   number of inputs as the value of this property. The default is 1.
  NumInputPorts = 1;
end

properties (Nontunable, Dependent)
  %InputDomain Input signal domain
  %   Specify the input signal domain as one of 'Time' | 'Frequency'. The
  %   default is 'Time'.
  InputDomain
end

properties (Dependent)
%SpectrumType Choose to visualize power, power density, or RMS
  %   Specify the spectrum type as one of 'Power' | 'Power density' |
  %   'RMS'. The default is 'Power'. This property is tunable.
  SpectrumType
  %ViewType Choose to view spectrum, spectrogram, or both
  %   Specify the view type as one of 'Spectrum' | 'Spectrogram' | 
  %   'Spectrum and spectrogram'. The default is 'Spectrum'. This property
  %   is tunable.
  ViewType
end

properties (Nontunable, Dependent)
  %SampleRate Sample rate of input
  %   Specify the sampling rate of the input in Hertz as a finite numeric
  %   scalar. The default is 10e3.
  SampleRate
  %Method Spectrum estimation method
  %   Specify the spectrum estimation method as one of 'Welch' |
  %   'Filter bank'. The filter bank method provides better resolution,
  %   lower spectral leakage, a more accurate noise floor and more
  %   precise peaks at the expense of increased computation and slower
  %   tracking. The default is 'Welch'.
  Method
end

properties (Nontunable, Dependent)
  %PlotAsTwoSidedSpectrum Two sided spectrum flag
  %   Set this property to true so that the Spectrum Analyzer computes two
  %   sided spectral estimates. Set this property to false to obtain one
  %   sided spectral estimates. In this case, the input must be real. The
  %   default is true. 
  PlotAsTwoSidedSpectrum
end

properties (Dependent)
  %FrequencyScale Frequency scale
  %   Specify the frequency scale as one of 'Linear' or 'Log'. You cannot
  %   set the frequency scale to 'Log' when the PlotAsTwoSidedSpectrum
  %   property is set to true. The default is 'Linear'. This property is
  %   tunable.
  FrequencyScale
  %FrequencySpan Frequency span mode
  %   Specify the frequency span mode as one of 'Full' | 'Span and center
  %   frequency' | 'Start and stop frequencies'. The default is 'Full'.
  %   When you set this property to 'Full' and if the
  %   PlotAsTwoSidedSpectrum property is set to true, the Spectrum Analyzer
  %   computes the spectrum over the entire Nyquist range which equals
  %   [-SampleRate/2, SampleRate/2] Hz  and plots in the frequency range
  %   [-(SampleRate/2)+min(FrequencyOffset),
  %   (SampleRate/2)+max(FrequencyOffset)] Hz. When you set this property
  %   to 'Full' and if the PlotAsTwoSidedSpectrum property is set to false,
  %   the Spectrum Analyzer computes the spectrum over the one-sided
  %   Nyquist range which equals [0, SampleRate/2] Hz and plots in the
  %   frequency range [min(FrequencyOffset),
  %   (SampleRate/2)+max(FrequencyOffset)] Hz. When you set this property
  %   to 'Span and center frequency', the Spectrum Analyzer computes and
  %   plots the spectrum over the frequency interval specified by the Span
  %   and CenterFrequency properties. When you set this property to 'Start
  %   and stop frequencies' the Spectrum Analyzer computes and plots the
  %   spectrum over the frequency interval specified by the StartFrequency
  %   and StopFrequency properties. This property is tunable.
  %
  %   See also: dsp.SpectrumAnalyzer.Span,
  %             dsp.SpectrumAnalyzer.CenterFrequency,
  %             dsp.SpectrumAnalyzer.StartFrequency,
  %             dsp.SpectrumAnalyzer.StopFrequency.
  FrequencySpan
  %Span Frequency span over which spectrum is computed
  %   Specify the frequency span over which the Spectrum Analyzer computes
  %   and plots the spectrum as a real, positive scalar in Hertz. This
  %   property applies when you set the FrequencySpan property to 'Span and
  %   center frequency'. The overall span defined by the Span and
  %   CenterFrequency properties must fall within the full span frequency
  %   range which equals [-(SampleRate/2)+min(FrequencyOffset),
  %   (SampleRate/2)+max(FrequencyOffset)] Hz when the
  %   PlotAsTwoSidedSpectrum property is set to true, and
  %   [min(FrequencyOffset), (SampleRate/2)+max(FrequencyOffset)] Hz when
  %   the PlotAsTwoSidedSpectrum property is set to false. The default is
  %   10 kHz. This property is tunable.
  %
  %   See also: dsp.SpectrumAnalyzer.CenterFrequency,
  %             dsp.SpectrumAnalyzer.PlotAsTwoSidedSpectrum
  Span  
  %CenterFrequency Frequency over which span is centered
  %   Specify the center frequency of the span over which the Spectrum
  %   Analyzer computes and plots the spectrum. CenterFrequency must be a
  %   real scalar in Hertz. This property applies when you set the
  %   FrequencySpan property to 'Span and center frequency'. The overall
  %   span defined by the Span and CenterFrequency properties must fall
  %   within the full span frequency range which equals
  %   [-(SampleRate/2)+min(FrequencyOffset),
  %   (SampleRate/2)+max(FrequencyOffset)] Hz when the
  %   PlotAsTwoSidedSpectrum property is set to true, and
  %   [min(FrequencyOffset), (SampleRate/2)+max(FrequencyOffset)] Hz when
  %   the PlotAsTwoSidedSpectrum property is set to false. The default is 0
  %   Hz. This property is tunable.
  %
  %   See also: dsp.SpectrumAnalyzer.Span,
  %             dsp.SpectrumAnalyzer.PlotAsTwoSidedSpectrum  
  CenterFrequency
  %StartFrequency Start frequency over which spectrum is computed
  %   Specify the start frequency over which the Spectrum Analyzer computes
  %   and plots the spectrum as a real scalar in Hertz. This property
  %   applies when you set the FrequencySpan property to 'Start and stop
  %   frequencies'. The overall span defined by the StartFrequency and
  %   StopFrequency properties must fall within the full span frequency
  %   range which equals [-(SampleRate/2)+min(FrequencyOffset),
  %   (SampleRate/2)+max(FrequencyOffset)] Hz when the
  %   PlotAsTwoSidedSpectrum property is set to true, and
  %   [min(FrequencyOffset), (SampleRate/2)+max(FrequencyOffset)] Hz when
  %   the PlotAsTwoSidedSpectrum property is set to false. This property is
  %   tunable.
  %
  %   See also: dsp.SpectrumAnalyzer.StopFrequency, 
  %             dsp.SpectrumAnalyzer.PlotAsTwoSidedSpectrum  
  StartFrequency
  %StopFrequency Stop frequency over which spectrum is computed
  %   Specify the stop frequency over which the Spectrum Analyzer computes
  %   and plots the spectrum as a real scalar in Hertz. This property
  %   applies when you set the FrequencySpan property to 'Start and stop
  %   frequencies'. The overall span defined by the StartFrequency and
  %   StopFrequency properties must fall within the full span frequency
  %   range which equals [-(SampleRate/2)+min(FrequencyOffset),
  %   (SampleRate/2)+max(FrequencyOffset)] Hz when the
  %   PlotAsTwoSidedSpectrum property is set to true, and
  %   [min(FrequencyOffset), (SampleRate/2)+max(FrequencyOffset)] Hz when
  %   the PlotAsTwoSidedSpectrum property is set to false. This property is
  %   tunable.
  %
  %   See also: dsp.SpectrumAnalyzer.StartFrequency, 
  %             dsp.SpectrumAnalyzer.PlotAsTwoSidedSpectrum    
  StopFrequency
  %FrequencyResolutionMethod Control resolution using RBW, window length or
  %number of frequency bands
  %   Specify how you want to control the frequency resolution of the
  %   spectrum analyzer as one of 'RBW' | 'WindowLength' |
  %   'NumFrequencyBands'. The default is 'RBW'. When you set the property
  %   to 'RBW' you control the frequency resolution of the analyzer
  %   directly in Hertz using the RBWSource and RBW properties. When you
  %   set the property to 'WindowLength' you control the frequency
  %   resolution of the analyzer indirectly using the WindowLength
  %   property. When you set the property to 'NumFrequencyBands' you
  %   control the frequency resolution of the analyzer indirectly using the
  %   FFTLengthSource and FFTLength properties. 'NumFrequencyBands' applies
  %   when the Method property is set to 'Filter bank'. 'WindowLength'
  %   applies when the Method property is set to 'Welch'.The frequency
  %   resolution of the analyzer is given as NENBW*Fs/WindowLength where
  %   NENBW is the normalized effective noise bandwidth of the window
  %   currently specified in the Window property. This property is tunable.
  FrequencyResolutionMethod
  %RBWSource Source of resolution bandwidth value
  %   Specify the source of the resolution bandwidth (RBW) as one of 'Auto'
  %   | 'Property'. The default is 'Auto'. This property applies when
  %   FrequencyResolutionMethod is set to 'RBW'. If the InputDomain is set
  %   to 'Time' and you set this property to 'Auto', the Spectrum Analyzer
  %   adjusts the spectral estimation resolution to ensure that there are
  %   1024 RBW intervals over the defined frequency span. If the
  %   InputDomain is set to 'Frequency' and you set this property to
  %   'Auto', the Spectrum Analyzer computes the resolution bandwidth as
  %   the minimum frequency separation in the frequency vector. If you set
  %   this property to 'Property', then you specify the resolution
  %   bandwidth directly on the RBW property. This property is tunable.
  %
  %   See also: dsp.SpectrumAnalyzer.RBW
  RBWSource
  %RBW Resolution bandwidth
  %   Specify the resolution bandwidth (RBW) of the Spectrum Analyzer as a
  %   real, positive, scalar in Hertz. This property applies when you set
  %   the FrequencyResolutionMethod property to 'RBW' and the RBWSource
  %   property to 'Property'. The ratio of the specified span and RBW
  %   values must be at least two (i.e. there must be at least two RBW
  %   intervals over the specified frequency span). The default is 9.76 Hz.
  %   This property is tunable.
  RBW
  %WindowLength Window length
  %   Specify the window length, in samples, used to compute spectral
  %   estimates as an integer scalar greater than two. This property
  %   applies when you set the FrequencyResolutionMethod property to
  %   'WindowLength'. The default is 1024. The frequency resolution of the
  %   analyzer is given as NENBW*Fs/WindowLength where NENBW is the
  %   normalized effective noise bandwidth of the window currently
  %   specified in the Window property. This property is tunable.
  WindowLength   
  %FFTLengthSource Source of the FFT length value
  %   Specify the source of the FFT length value as one of 'Auto' |
  %   'Property'. The default is 'Auto'. This property applies when you set
  %   the FrequencyResolutionMethod property to 'WindowLength'. When you
  %   set this property to 'Auto' the Spectrum Analyzer sets the FFT length
  %   to max(1024,WindowLength). When you set this property to 'Property'
  %   then you specify the number of FFT points using the FFTLength
  %   property. The number of FFT points must be equal to or larger than
  %   the value specified in the WindowLength property. This property is
  %   tunable.
  %
  %   See also: dsp.SpectrumAnalyzer.FFTLength
  FFTLengthSource
  %FFTLength FFT length
  %   Specify the length of the FFT that the Spectrum Analyzer uses to
  %   compute spectral estimates as a positive, integer scalar. This
  %   property applies when you set the FrequencyResolutionMethod property
  %   to 'WindowLength' or 'NumFrequencyBands' and the FFTLengthSource
  %   property to 'Property'. The default is 1024. FFTLength must be equal
  %   to or larger than WindowLength. This property is tunable.
  FFTLength
  %NumTapsPerBand Number of filter taps per frequency band
  %   Specify the number of filter coefficients or taps for each
  %   frequency band. This also corresponds to the number of filter
  %   coefficients per polyphase branch. The total number of filter
  %   coefficients is given by NumTapsPerBand times FFTLength. This
  %   property applies when you set Method to 'Filter bank'. The
  %   default is 12. This property is tunable.
  NumTapsPerBand
  % FrequencyVectorSource Source of frequency vector value
  %   Specify the source of the frequency vector as one of 'Auto' |
  %   'Property'. The default is 'Auto'. This property applies when
  %   InputDomain is set to 'Frequency'. When you set this property to
  %   'Auto', the Spectrum Analyzer computes the frequency vector based on
  %   the input signal frame size and the specified sampling rate. When
  %   you set this property to 'Property', then you specify the frequency
  %   vector directly on the FrequencyVector property. This property is
  %   tunable.
  FrequencyVectorSource  
  % FrequencyVector Input frequency vector
  %   Specify the input frequency vector as a real, positive, sorted,
  %   column vector in Hertz. The frequency vector must have the same
  %   length as the input signal frame size. This property applies when you
  %   set the InputDomain property to 'Frequency' and the
  %   FrequencyVectorSource property to 'Property'. The default is [-5000
  %   5000] Hz. This property is tunable.
  FrequencyVector
end

properties (Nontunable, Dependent)
  %InputUnits Input signal units
  %   Specify the input signal units as one of 'dBm' | 'dBW' | 'Watts' |
  %   'Vrms' | 'dBV'. This property applies when InputDomain is set to
  %   'Frequency'. The default is 'dBm'.
  InputUnits  
end

properties (Dependent)
  %OverlapPercent Overlap percentage
  %   Specify the percentage overlap between the current and previous
  %   buffered window segments as a real, scalar value, greater than or
  %   equal to zero and less than 100. The default is 0. This property is
  %   tunable.
  OverlapPercent 
  %Window Window function
  %   Specify a window function for the spectral estimator as one of
  %   'Rectangular' | 'Chebyshev' | 'Flat Top' | 'Hamming' | 'Hann' |
  %   'Kaiser' | 'Blackman-Harris' | 'Custom'. The default is 'Hann'. This
  %   property is tunable.
  Window
  %CustomWindow
  %   Specify a custom window function name as an array of characters. The
  %   window function must be on the path and must have exactly one input
  %   and one output. This property applies when you set the Window
  %   property to 'Custom'.  The default is 'hann'. This property is
  %   tunable.
  CustomWindow(1,:) char
  %SidelobeAttenuation Sidelobe attenuation of window
  %   Specify the sidelobe attenuation of the window as a real, positive
  %   scalar in decibels (dB). This property must be greater than or equal 
  %   to 45 dB. This property applies when you set the Window property to 
  %   'Chebyshev' or 'Kaiser'. The default is 60 dB. This property is 
  %   tunable.
  SidelobeAttenuation
  %SpectrumUnits Spectrum units
  %   Specify the units in which the Spectrum Analyzer displays power or
  %   voltage values as one of 'Auto' | 'dBm' | 'dBW' | 'dBFS' | 'Watts' |
  %   'Vrms' | 'dBV'. 'Vrms' and 'dBV' apply when the SpectrumType is set
  %   to 'RMS'. 'dBFS' applies when the InputDomain is set to 'Time'.
  %   'Auto' applies when the InputDomain is set to 'Frequency'. If the
  %   InputDomain is set to 'Frequency' and SpectrumUnits is set to 'Auto',
  %   the Spectrum Analyzer assumes the spectrum units to be equal to input
  %   units specified on the 'InputUnits' property. If the InputDomain is
  %   set to 'Time' and SpectrumUnits is set to any option other than
  %   'Auto', the Spectrum Analyzer converts from the units specified on
  %   InputUnits to the units specified on SpectrumUnits. The default is
  %   'dBm'. This property is tunable.
  SpectrumUnits
  %FullScaleSource Source of full scale value
  %   Specify the source for the full scale of the input signal as one of
  %   'Auto' | 'Property'. The default is 'Auto'. This property applies
  %   when the SpectrumUnits property is set to 'dBFS'. When set to 'Auto',
  %   the full scale is computed based on the input data. When set to
  %   'Property', you specify the full scale of the input signal using the
  %   'FullScale' property. This property is tunable.
  FullScaleSource
  %FullScale Full scale value
  %   Specify the full scale value as a positive, scalar integer. The
  %   Property applies when you set the SpectrumUnits property to 'dBFS'.
  %   This property is tunable.
  FullScale
  % AveragingMethod Averaging method 
  %   Specify the averaging method as 'Running' or 'Exponential'. In the
  %   running averaging method, the object computes an equally weighted
  %   average of specified number of spectrum estimates defined by
  %   'SpectralAverages' property. In the exponential method, the object
  %   computes the average over samples weighted by an exponentially
  %   decaying forgetting factor. The default is 'Running'. This property
  %   is tunable.
  AveragingMethod
  %SpectralAverages Number of spectral averages
  %   Specify the number of spectral averages as a positive, integer
  %   scalar. The Spectrum Analyzer computes the current power spectrum
  %   estimate by averaging the last N power spectrum estimates, where N is
  %   the number of spectral averages defined in the SpectralAverages
  %   property. This property applies when the ViewType property is set to
  %   'Spectrum'. The default is 1. This property is tunable.
  SpectralAverages
  %ForgettingFactor Forgetting factor
  %   Specify the exponential weighting forgetting factor as a scalar value
  %   greater than zero and smaller than or equal to one. This property
  %   applies when 'AveragingMethod' is equal to 'Exponential'. The default
  %   is 0.9. This property is tunable.
  ForgettingFactor;
  %ReferenceLoad Reference load
  %   Specify the load that the Spectrum Analyzer uses as a reference to
  %   compute power values as a real, positive scalar in Ohms. This
  %   property applies when the SpectrumType property is set to 'Power' or
  %   'Power density'. The default is 1 Ohm. This property is tunable.
  ReferenceLoad
  %FrequencyOffset Frequency offset
  %   Specify the frequency offset to apply to the frequency axis (x-axis)
  %   in units of Hertz. This property can either be set to a numeric
  %   scalar, or a vector of length equal to the number of input channels.
  %   If this property is set to a scalar, the same frequency offset is
  %   used for all channels. If this property is set to a vector, each
  %   element is used as the frequency offset for the corresponding
  %   channel.The overall span must fall within the full span frequency
  %   range. The full span frequency range is the union of the valid
  %   Nyquist frequency intervals for all channels after applying frequency
  %   offsets. When the PlotAsTwoSidedSpectrum property is set to true, the
  %   full span frequency range equals [-(SampleRate/2) +
  %   min(FrequencyOffset), (SampleRate/2) + max(FrequencyOffset)] Hz. When
  %   the PlotAsTwoSidedSpectrum property is set to false, the full span
  %   frequency range equals [min(FrequencyOffset), (SampleRate/2) +
  %   max(FrequencyOffset)] Hz. The default is 0 Hz. This property is
  %   tunable.
  FrequencyOffset
end

properties (Dependent)
  %SpectrogramChannel Channel for which spectrogram settings apply
  %   Specify the channel number as a real, positive scalar integer. This
  %   property applies when you set the ViewType property to 'Spectrogram'
  %   or 'Spectrum and spectrogram'. The default is 1. This property is
  %   tunable.
  SpectrogramChannel
  %TimeResolutionSource Source of the time resolution value
  %   Specify the source for the time resolution of each spectrogram line
  %   as one of 'Auto' | 'Property'. This property applies when you set the
  %   SpectrumType property to 'Spectrogram'. The default is 'Auto'. 
  %
  %   When FrequencyResolutionMethod is 'RBW' the following rules apply:
  %
  %      When both RBWSource and TimeResolutionSource are set to 'Auto'
  %      then RBW is set such that there are 1024 RBW intervals in one
  %      frequency span. The time resolution is set to 1/RBW.
  %      
  %      When RBWSource is set to 'Auto' and TimeResolutionSource is set to
  %      'Property' then time resolution becomes the main control and RBW
  %      is set to 1/TimeResolution Hz.
  %
  %      When RBWSource is set to 'Property' and TimeResolutionSource is
  %      set to 'Auto' then RBW becomes the main control and the time
  %      resolution is set 1/RBW s.
  %
  %      When both RBWSource and TimeResolutionSource are set to 'Property'
  %      then the specified time resolution value must be equal to or
  %      larger than the minimum attainable time resolution which is
  %      defined by 1/RBW. Several spectral estimates are combined into one
  %      spectrogram line to obtain the desired time resolution.
  %
  %   When FrequencyResolutionMethod is 'WindowLength' the following rules
  %   apply:
  %
  %      When TimeResolutionSource is set to 'Auto' then the time
  %      resolution is set to 1/RBW. In this case RBW is computed from the
  %      window length value as RBW = NENBW*Fs/WindowLength where NENBW is
  %      the normalized effective noise bandwidth of the window currently
  %      specified in the Window property.
  %
  %      When TimeResolutionSource is set to 'Property' then the time
  %      resolution value must be equal to or larger than the minimum
  %      attainable time resolution which equals 1/(NENBW*Fs/WindowLength).
  %      In this case several spectral estimates are combined into one
  %      spectrogram line to obtain the desired time resolution.
  %
  %   This property is tunable.
  TimeResolutionSource
  %TimeResolution Time resolution
  %   Specify the time resolution of each spectrogram line as a positive
  %   scalar in seconds. This property applies when you set the ViewType
  %   property to 'Spectrogram' or 'Spectrum and spectrogram', and  the
  %   TimeResolutionSource property to 'Property'. The default is 1 ms.
  %   This property is tunable.
  TimeResolution  
  %TimeSpanSource Source of time span value
  %   Specify the source for the time span of the spectrogram as one of
  %   'Auto' | 'Property'. The default is 'Auto'. This property applies
  %   when the ViewType property is set to 'Spectrogram' or 'Spectrum and
  %   spectrogram'. When set to 'Auto' the spectrogram displays 100
  %   spectrogram lines at any given time. When set to 'Property' you
  %   specify the time duration of the spectrogram in seconds using the
  %   'TimeSpan' property. The specified time span must be at least two
  %   times larger than the duration of the number of samples required for
  %   a spectral update. This property is tunable.
  TimeSpanSource
  %TimeSpan Time span
  %   Specify the time span of the spectrogram display as a positive scalar
  %   in seconds. This property applies when you set the ViewType property
  %   to 'Spectrogram' or 'Spectrum and spectrogram', and  the
  %   TimeSpanSource property to 'Property'. The specified time span must
  %   be at least two times larger than the duration of the number of
  %   samples required for a spectral update. The default is 100 ms. This
  %   property is tunable.
  TimeSpan
end

properties (Dependent)
  %MeasurementChannel Channel for which measurement will be performed
  %   Specify the measurement channel number as a real, positive scalar
  %   integer. The default is 1. This property is tunable.
  MeasurementChannel
  %SpectralMask Spectral mask specification
  %   Enable and specify the spectral mask to draw upper and/or lower limit
  %   mask lines on the Power and Power Density plots. The SpectralMask
  %   property uses the SpectralMaskSpecification properties: EnabledMasks,
  %   UpperMask, LowerMask, ReferenceLevel, CustomReferenceLevel,
  %   MaskFrequencyOffset and SelectedChannel. See <a href="matlab:helpview(fullfile(docroot,'dsp','dsp.map'),'dsp.SpectrumAnalyzer.SpectralMask')">SpectralMask</a>
  %   documentation for more details. The SpectralMask property is tunable.
  %
  %   % Example:
  %     sine = dsp.SineWave('Frequency',[98 100],'SampleRate',1000);
  %     sine.SamplesPerFrame = 1024;
  %     scope = dsp.SpectrumAnalyzer('SampleRate',sine.SampleRate, ...
  %         'PlotAsTwoSidedSpectrum',false,'ShowLegend',true);
  %     for i=1:100
  %         scope(sine() + 0.05*randn(1024,2));
  %     end
  %     % Set SpectralMask properties
  %     upperMask = [0 -10; 90 -10; 90 40; 110 40; 110 -10; 500 -10];
  %     scope.SpectralMask.EnabledMasks = 'Upper and lower';
  %     set(scope.SpectralMask,'UpperMask',upperMask,'LowerMask',-60);
  %     release(scope);
  SpectralMask
end

properties (SetAccess = protected, Dependent)
  %PeakFinder Peak finder specification
  %   Enable peak finder to compute and display the largest calculated peak
  %   values. The PeakFinder property uses the PeakFinderSpecification
  %   properties: MinHeight, NumPeaks, MinDistance, Threshold,
  %   LabelFormat and Enable. See <a
  %   href="matlab:helpview(fullfile(docroot,'dsp','dsp.map'),'dsp.SpectrumAnalyzer.PeakFinder')">PeakFinder</a>
  %   documentation for more details. The PeakFinder property is tunable.
  %
  %   % Example:
  %     sine = dsp.SineWave('Frequency',[98 100],'SampleRate',1000);
  %     sine.SamplesPerFrame = 1024;
  %     scope = dsp.SpectrumAnalyzer('SampleRate',sine.SampleRate, ...
  %         'PlotAsTwoSidedSpectrum',false,'ShowLegend',true);
  %
  %    % Enable peak finder measurement
  %    scope.PeakFinder.Enable = true;
  %
  %    % Set peak finder NumPeaks property
  %    scope.PeakFinder.NumPeaks = 4;
  %
  %    for i=1:100
  %        scope(sine() + 0.05*randn(1024,2));
  %    end
  %
  %    release(scope);  
  PeakFinder
  %CursorMeasurementsSpecification Cursor measurements specification
  %    Enable cursor measurements to display screen or waveform cursors.
  %    The CursorMeasurements property uses the
  %    CursorMeasurementsSpecification properties: Type,
  %    ShowHorizontal, ShowVertical, Cursor1TraceSource,
  %    Cursor2TraceSource, LockSpacing, SnapToData, XLocation,
  %    YLocation and Enable. See <a href="matlab:helpview(fullfile(docroot,'dsp','dsp.map'),'dsp.SpectrumAnalyzer.CursorMeasurements')">CursorMeasurements</a>
  %    documentation for more details. The CursorMeasurements property is
  %    tunable.
  %
  %    % Example:
  %     sine = dsp.SineWave('Frequency',[98 100],'SampleRate',1000);
  %     sine.SamplesPerFrame = 1024;
  %     scope = dsp.SpectrumAnalyzer('SampleRate',sine.SampleRate, ...
  %         'PlotAsTwoSidedSpectrum',false,'ShowLegend',true);
  %
  %    % Enable cursor measurements
  %    scope.CursorMeasurements.Enable = true;
  %
  %    % Set CursorType property
  %    scope.CursorMeasurements.CursorType = 'Screen cursors';
  %
  %    for i=1:100
  %        scope(sine() + 0.05*randn(1024,2));
  %    end
  %
  %    release(scope);
  CursorMeasurements
  %ChannelMeasurements Channel measurements specification
  %    Enable channel measurements to compute and display the occupied
  %    bandwidth or adjacent channel power ratio. The ChannelMeasurements
  %    property uses the ChannelMeasurementsSpecification properties:
  %    Algorithm, FrequencySpan, Span, CenterFrequency, StartFrequency,
  %    StopFrequency, PercentOccupiedBW, NumOffsets, AdjacentBW,
  %    FilterShape, FilterCoeff, ACPROffsets and Enable. See <a href="matlab:helpview(fullfile(docroot,'dsp','dsp.map'),'dsp.SpectrumAnalyzer.ChannelMeasurements')">ChannelMeasurements</a>
  %    documentation for more details. The CursorMeasurements property is
  %    tunable.
  %
  %    % Example:
  %     sine = dsp.SineWave('Frequency',[98 100],'SampleRate',1000);
  %     sine.SamplesPerFrame = 1024;
  %     scope = dsp.SpectrumAnalyzer('SampleRate',sine.SampleRate, ...
  %         'PlotAsTwoSidedSpectrum',false,'ShowLegend',true);
  %
  %    % Enable channel measurements
  %    scope.ChannelMeasurements.Enable = true;
  %
  %    % Set Algorithm property
  %    scope.ChannelMeasurements.Algorithm = 'ACPR';
  %
  %    for i=1:100
  %        scope(sine() + 0.05*randn(1024,2));
  %    end
  %
  %    release(scope);
  ChannelMeasurements
  %DistortionMeasurements Distortion measurements specification
  %    Enable distortion measurements to compute and display the harmonic
  %    distortion and intermodulation distortion. The
  %    DistortionMeasurements property uses the
  %    DistortionMeasurementsSpecification properties: Algorithm,
  %    NumHarmonics and Enable. See <a href="matlab:helpview(fullfile(docroot,'dsp','dsp.map'),'dsp.SpectrumAnalyzer.DistortionMeasurements')">DistortionMeasurements</a>
  %    documentation for more details. The DistortionMeasurements property
  %    is tunable.
  %
  %    % Example:
  %     sine = dsp.SineWave('Frequency',[98 100],'SampleRate',1000);
  %     sine.SamplesPerFrame = 1024;
  %     scope = dsp.SpectrumAnalyzer('SampleRate',sine.SampleRate, ...
  %         'PlotAsTwoSidedSpectrum',false,'ShowLegend',true);
  %
  %    % Enable distortion measurements
  %    scope.DistortionMeasurements.Enable = true;
  %
  %    % Set NumHarmonics property
  %    scope.DistortionMeasurements.NumHarmonics = 4;
  %
  %    for i=1:100
  %        scope(sine() + 0.05*randn(1024,2));
  %    end
  %
  %    release(scope);
  DistortionMeasurements 
end

properties
  %Name Caption to display on Spectrum Analyzer window
  %   Specify the caption to display on the Spectrum Analyzer window as any
  %   string. The default is 'Spectrum Analyzer'. This property is tunable.
  Name = 'Spectrum Analyzer';  
end

properties (Dependent)
  %PlotType Choose the plot type
  %   Specify the type of plot to be used for the normal traces (i.e. the
  %   traces displaying the free running spectral estimates). Valid types
  %   are 'Line' and 'Stem'. This property applies when the ViewType
  %   property is set to 'Spectrum' or 'Spectrum and spectrogram', and the
  %   PlotNormalTrace property is set to true. The default is 'Line'. This
  %   property is tunable.
  PlotType
  %PlotNormalTrace Normal trace flag
  %   Set this property to false to remove the normal traces (i.e. the
  %   traces displaying the free running spectral estimates) from the
  %   Spectrum Analyzer display. Note that even though the traces are
  %   removed from the display, the spectral computations continue. This
  %   property applies when the ViewType property is set to 'Spectrum' or
  %   'Spectrum and spectrogram'. The default is true. This property is
  %   tunable.
  %
  %   See also: dsp.SpectrumAnalyzer.PlotMaxHoldTrace,
  %             dsp.SpectrumAnalyzer.PlotMinHoldTrace      
  PlotNormalTrace
  %PlotMaxHoldTrace Max-hold trace flag
  %   Set this property to true so that the Spectrum Analyzer computes and
  %   plots the max-hold spectrum of each input channel. This property
  %   applies when the ViewType property is set to 'Spectrum' or 'Spectrum
  %   and spectrogram'. The max-hold spectrum is computed by keeping, at
  %   each frequency bin, the maximum value of all the power spectrum
  %   estimates. Toggling the PlotMaxHoldTrace control resets the max-hold
  %   computations. The default is false. This property is tunable.
  %
  %   See also: dsp.SpectrumAnalyzer.PlotMinHoldTrace,
  %             dsp.SpectrumAnalyzer.PlotNormalTrace  
  PlotMaxHoldTrace
  %PlotMinHoldTrace Min-hold trace flag
  %   Set this property to true so that the Spectrum Analyzer computes and
  %   plots the min-hold spectrum of each input channel. This property
  %   applies when the ViewType property is set to 'Spectrum' or 'Spectrum
  %   and spectrogram'. The min-hold spectrum is computed by keeping, at
  %   each frequency bin, the minimum value of all the power spectrum
  %   estimates. Toggling the PlotMinHoldTrace control resets the min-hold
  %   computations. The default is false. This property is tunable.
  %
  %   See also: dsp.SpectrumAnalyzer.PlotMaxHoldTrace,
  %             dsp.SpectrumAnalyzer.PlotNormalTrace    
  PlotMinHoldTrace
  %ReducePlotRate Reduce plot rate to improve performance
  %   Set this property to false to update the plot after each PSD
  %   computation when the ViewType property is set to 'Spectrum' or after
  %   each spectrogram line computation when the ViewType property is set
  %   to 'Spectrogram' or 'Spectrum and spectrogram'. Setting this
  %   property to false negatively impacts performance. The default is
  %   true. This property is tunable.
  ReducePlotRate
  %Title Display title
  %   Specify the display title as a string. The default value is ''. This
  %   property is tunable.  
  Title(:,:) char
  %YLabel Y-axis label
  %   Specify the y-axis label as a string. The default value is ''. This
  %   property applies when the ViewType property is set to 'Spectrum' or
  %   'Spectrum and spectrogram'. This property is tunable.
  YLabel(1,:) char
  %ShowLegend Show or hide legend
  %   When this property is set to false, no legend is displayed. When this
  %   property is set to true, a legend with automatic string labels for
  %   each input channel is displayed. You can use the legend to toggle
  %   signal visibility in the scope. In the legend, click on a signal name
  %   to hide the signal in the scope. To show the signal, click on the
  %   signal name again. To show only one signal, right-click the signal
  %   name. This property applies when the ViewType property is set to
  %   'Spectrum' or 'Spectrum and spectrogram'. The default is false. This
  %   property is tunable.
  ShowLegend
  %ChannelNames Names for the input data channels
  %   Set this property to a cell array of strings to represent the input
  %   channels. These strings are used to label the data channels in the
  %   legend, in the Style dialog and in various measurement tools. The
  %   default is an empty cell array. When this property is an empty cell
  %   array, the channels will be named by default as Channel 1, Channel 2
  %   and so on. This property is tunable.
  ChannelNames
  %ShowGrid Show or hide grid
  %   Set this property to true to turn on the grid. The default is true.
  %   This property is tunable.
  ShowGrid
  %YLimits Y-axis limits
  %   Specify the y-axis limits as a two-element numeric vector: [ymin
  %   ymax]. This property applies when the ViewType property is set to
  %   'Spectrum' or 'Spectrum and spectrogram'. This property is tunable.
  YLimits
  %ColorLimits Color limits
  %   Specify the color limits as a two-element numeric vector: [cmin
  %   cmax]. This property applies when the ViewType property is set to
  %   'Spectrogram' or 'Spectrum and spectrogram'. This property is tunable.
  ColorLimits
  %AxesScaling Axes scaling mode
  %   Specify the scaling mode for the axes as one of 'Auto' | 'Manual' |
  %   'Updates' | 'OnceAtStop'.  When this property is set to 'Auto', the
  %   scope attempts to keep the data in the display while minimizing the
  %   number of axes limit updates.  When this property is set to 'Manual',
  %   the scope takes no action unless specified by the user.  When this
  %   property is set to 'Updates', the scope updates the axes limits after
  %   a set number of visual updates.  When this property is set to
  %   'OnceAtStop', the limits will be updated once at the end of the
  %   simulation (when release is called).  The default is 'Auto'. This
  %   property is tunable.
  AxesScaling
  %AxesLayout Choose the axes layout
  %   Specify the layout type as one of 'Horizontal' | 'Vertical'. This
  %   property applies when the ViewType is set to 'Spectrum and
  %   spectrogram'. The default is 'Vertical'. This property is tunable.
  AxesLayout
end

properties(SetAccess=protected, Hidden, Dependent)
  %CCDFMeasurements CCDF measurements specification
  CCDFMeasurements
end

properties(Transient,Hidden)
   SpectrumUnitsSet = matlab.system.StringSet({'dBm','dBW','Watts','dBFS'}); 
   FrequencyResolutionMethodSet = matlab.system.StringSet({'RBW','WindowLength'});
end

properties (Constant, Hidden)
  WindowSet = matlab.system.StringSet({...
    'Rectangular', ...
    'Blackman-Harris',...
    'Chebyshev', ...
    'Flat Top', ...
    'Hamming', ...
    'Hann', ...
    'Kaiser',...
    'Custom'});
  FFTLengthSourceSet = matlab.system.StringSet({'Auto','Property'});
  TimeSpanSourceSet = matlab.system.StringSet({'Auto','Property'});
  TimeResolutionSourceSet = matlab.system.StringSet({'Auto','Property'});
  RBWSourceSet = matlab.system.StringSet({'Auto','Property'});
  SpectrumTypeSet = dsp.scopes.SpectrumTypeSet({...
    'Power','Power density','RMS'},{'Spectrogram'},{'Power'});
  ViewTypeSet = matlab.system.StringSet({...
      'Spectrum','Spectrogram','Spectrum and spectrogram'});
  FrequencyScaleSet =  matlab.system.StringSet({'Linear','Log'});
  PlotTypeSet = matlab.system.StringSet({'Line','Stem'});
  AxesLayoutSet = matlab.system.StringSet({'Vertical','Horizontal'});
  % PowerUnits Set used when SpectrumType is Power or Power density
  PowerUnitsSet = matlab.system.StringSet({'dBm','dBW','dBFS','Watts'});
  % PowerUnits Set used when SpectrumType is RMS
  RMSUnitsSet = matlab.system.StringSet({'Vrms','dBV'});
  % Full Scale SourceSet is used when SpectrumUnits is dBFS
  FullScaleSourceSet = matlab.system.StringSet({'Auto','Property'});
  AxesScalingSet = matlabshared.scopes.mixin.CommandLineAutoscale.getAxesScalingSet;
  MethodSet = matlab.system.StringSet({'Welch','Filter bank'});
  % FrequencyResolutionMethodWelchSet is used when Method is Welch
  FrequencyResolutionMethodWelchSet = matlab.system.StringSet({'RBW','WindowLength'});
  % FrequencyResolutionMethodFilterBankSet is used when Method is Filter
  % bank
  FrequencyResolutionMethodFilterBankSet = matlab.system.StringSet({'RBW','NumFrequencyBands'});
  InputDomainSet = matlab.system.StringSet({'Frequency','Time'});
  % InputUnitsSet is used when InputDomain is Frequency 
  InputUnitsSet = matlab.system.StringSet({'dBm','dBV','dBW','Vrms','Watts'});
  FrequencySpanSet = matlab.system.StringSet({'Full',...
      'Span and center frequency','Start and stop frequencies'});
  FrequencyVectorSourceSet = matlab.system.StringSet({'Auto','Property'});
  FrequencyInputSpectrumUnitsSet = matlab.system.StringSet({'Auto','dBm','dBV','dBW','Vrms','Watts'});
  % Averaging methods
  AveragingMethodSet =  matlab.system.StringSet({'Running','Exponential'});
end

properties (Access = private)
  pNumChannels;
  % Listener for NumInputPorts
  NumInputPortsListener;
  %Legacy Mode used for R2016b and earlier releases
  pSpectrumTypeLegacySetting = false;
  % Script generator
  pScriptGenerator;
end

properties (Dependent, Hidden)  
  %PowerUnits Power units
  %   Specify the units in which the Spectrum Analyzer displays power
  %   values as one of 'dBm' | 'dBW' | 'Watts'. The default is 'dBm'.  
  PowerUnits
  
  % RMSUnits used by RMSUnitsSet
  RMSUnits
  % CombinedUnits used by CombinedUnitsSet
  FrequencyInputSpectrumUnits
  % FrequencyResolutionMethodWelch used by
  % FrequencyResolutionMethodWelchSet
  FrequencyResolutionMethodWelch
  % FrequencyResolutionMethodFilterBank used by
  % FrequencyResolutionMethodFilterBankSet
  FrequencyResolutionMethodFilterBank
end

%--------------------------------------------------------------------------
% Public methods
%--------------------------------------------------------------------------
methods
  
  %------------------------------------------------------------------------
  function obj = SpectrumAnalyzerBase(varargin)
    %SpectrumAnalyzer   Construct the SpectrumAnalyzer class.    
      
    obj@matlabshared.scopes.UnifiedSystemScope();
    
    % Specify the default size for Spectrum Analyzer window to be [800,450]
    % (16:9 aspect ratio) as against the default MATLAB figure size of
    % [560,420].
    obj.Position = uiscopes.getDefaultPosition([800 450]);
    
    obj.NumInputPortsListener = event.proplistener(obj.pSource, ...
          obj.pSource.findprop('NumInputPorts'), 'PostSet', ...
          @(~,~) updateNumInputPorts(obj));
    
    % Allow NumInputPorts to be specified as a value-only input argument
    setProperties(obj, nargin, varargin{:}, 'NumInputPorts');
    obj.pNumChannels = -1;
    % Instantiate Script generator
    obj.pScriptGenerator = dsp.scopes.ScopesScriptGenerator();
    setup(obj.pScriptGenerator, obj, obj.pFramework.Specification);
  end
  
%--------------------------------------------------------------------------
% Public user visible methods
%--------------------------------------------------------------------------
  
  %------------------------------------------------------------------------
  function s = getSpectralMaskStatus(obj)
    %getSpectralMaskStatus Get the current results of spectral mask testing
    %   S = getSpectralMaskStatus(specanlyz) returns a structure S whose
    %   fields contain information about the current results of spectral
    %   mask testing. The structure S contains the following fields:
    %   IsCurrentlyPassing  - Whether the mask test is currently passing.
    %   NumTotalTests       - Total number of spectral mask tests in the
    %                         current simulation run. A test is performed
    %                         for every spectral update.
    %   NumPassedTests      - Number of passed tests in the current
    %                         simulation.
    %   SuccessRate         - Percentage success rate of spectral mask
    %                         testing.
    %   FailingMasks        - Currently failing masks.
    %   FailingChannels     - Currently failing input channels.
    %   SimulationTime      - Simulation time when this statistics was
    %                         generated.
    s = struct([]);
    if ~isScopeLaunched(obj)
      launchScope(obj);
    end
    hVisual = obj.pFramework.Visual;
    if ~isempty(hVisual) && ~isempty(hVisual.MaskTesterObject)
      % Before obtaining the mask status from the MaskTesterObject, force
      % update the visual if the scope is in ReducePlotRate mode.
      forceUpdateVisual(obj); 
      s = getMaskStatus(hVisual.MaskTesterObject);
    end
  end
  %------------------------------------------------------------------------
  function data = getSpectrumData(obj, allFlag)
    %getSpectrumData Get the current spectrum data displayed on the
    %Spectrum Analyzer
    %   DATA = getSpectrumData(scope) returns DATA table about the current
    %   spectrum or spectrogram displayed on the Spectrum Analyzer. 
    %
    %   DATA = getSpectrumData(scope, 'all') returns a DATA table with all
    %   spectrum information for the current time step.
    %
    %   DATA table contains the following fields:
    %   SimulationTime        - Simulation time 
    %   Spectrum              - Current spectrum data 
    %   Spectrogram           - Current spectrogram data
    %   MinHoldTrace          - Minimum-hold trace 
    %   MaxHoldTrace          - Maximum-hold trace 
    %   FrequencyVector       - Frequency vector
    %
    %   Example: Get the spectrum data displayed on the spectrum analyzer.
    %       sine = dsp.SineWave('Frequency',100, 'SampleRate', 1000);
    %       sine.SamplesPerFrame = 1000;
    %       scope = dsp.SpectrumAnalyzer('SampleRate', swv.SampleRate,...
    %                                    'ReducePlotRate',false);
    %       x = sine() + 0.05*randn(1000,1);
    %       san(x);
    %       data = getSpectrumData(scope);
    %       release(scope);
    
    % Check the number of input arguments
    narginchk(1,2);
    if nargin == 2
        % String support
        allFlag = convertStringsToChars(allFlag);
        % Validate with valid strings
        validatestring(allFlag,{'all'});
        allFlag = true;
    else
        allFlag = false;
    end
    hVisual = obj.pFramework.Visual;
    data = getSpectrumData(hVisual, allFlag);
  end
  %------------------------------------------------------------------------  
  function data = getMeasurementsData(obj, allFlag)
    %getMeasurementsData Get the current measurement data displayed on
    %the Spectrum Analyzer
    %   DATA = getMeasurementsData(scope) returns data table about the
    %   current spectrum analyzer measurements in use.
    %
    %   DATA = getMeasurementsData(scope, 'all') returns data table about
    %   all spectrum analyzer measurements for the current time step.
    %
    %   DATA contains the following fields:
    %   SimulationTime           - Simulation time
    %   PeakFinder               - Peak finder data
    %   CursorMeasurements       - Cursor measurements data
    %   ChannelMeasurements      - Channel measurements data
    %   DistortionMeasurements   - Distortion measurements data
    %
    %   Example: Get Peak finder data
    %       sine = dsp.SineWave('Frequency',[98 100],'SampleRate',1000);
    %       sine.SamplesPerFrame = 1024;
    %       scope = dsp.SpectrumAnalyzer('SampleRate',sine.SampleRate, ...
    %         'PlotAsTwoSidedSpectrum',false,'ShowLegend',true);
    %
    %       % Enable peak finder measurement
    %       scope.PeakFinder.Enable = true;
    %
    %       % Set peak finder NumPeaks property
    %       scope.PeakFinder.NumPeaks = 4;
    %       scope(sine() + 0.05*randn(1024,2));
    %       data = getMeasurementsData(scope);
    %
    %       release(scope);
    
    % Check the number of input arguments
    narginchk(1,2);
    if nargin == 2
      % String support
      allFlag = convertStringsToChars(allFlag);
      % Validate with valid strings
      validatestring(allFlag,{'all'});
      allFlag = true;
    else
      allFlag = false;
    end
    hVisual = obj.pFramework.Visual;
    data = getMeasurementsData(hVisual, allFlag);
  end
  %------------------------------------------------------------------------
  function flag = isNewDataReady(obj)
    %isNewDataReady Return whether or not the Spectrum Analyzer is
    %displaying new spectrum data
    %   FLAG = isNewDataReady(scope) returns a logical to indicate if the
    %   Spectrum Analyzer plotted new spectrum data
    flag = false;
    hVisual = obj.pFramework.Visual;
    if ~isempty(hVisual)
        flag = hVisual.IsNewDataReady;
    end
  end
  %------------------------------------------------------------------------
  function varargout = generateScript(obj)
      % generateScript(obj) Generate MATLAB(R) script to construct
      % dsp.SpectrumAnalyzer with current settings.
      if nargout > 0
          varargout{1} = obj.pScriptGenerator.generateScript();
      else
          obj.pScriptGenerator.generateScript();
      end
  end
%--------------------------------------------------------------------------
% Main Properties Set/Get methods
%--------------------------------------------------------------------------  

  %------------------------------------------------------------------------        
  function set.NumInputPorts(obj, value)
    validateattributes(value, ...
      {'numeric'}, {'integer', 'scalar', '>=', 1, '<=', 96}, '', 'NumInputPorts');
    obj.NumInputPorts = value;
    if ~isScopeLaunched(obj)
      launchScope(obj);
    end
    obj.pSource.NumInputPorts = value;
  end
  function value = get.NumInputPorts(obj)
    if isScopeLaunched(obj)
      value = obj.pSource.NumInputPorts;
    else
      value = obj.NumInputPorts;
    end
  end
  %------------------------------------------------------------------------
  function set.InputDomain(obj, value)
    validateattributes(value,{'char'},{},'','InputDomain');
    setVisualProperty(obj, 'InputDomain', value, false);
  end
  function value = get.InputDomain(obj)    
    value = getVisualProperty(obj, 'InputDomain', false);
  end 
  %------------------------------------------------------------------------
  function set.SpectrumType(obj, value)
      % Handle spectrum type cases prior to R2017a
      % check if the property value is grandfathered and moved to a new
      % property
        flag = isGrandFathered(obj.SpectrumTypeSet);
        if strcmpi(value,'Power') && flag
            setVisualProperty(obj,'SpectrumType','Power', false);
            setVisualProperty(obj,'ViewType','Spectrogram',false);
            obj.pSpectrumTypeLegacySetting = true;
        elseif (strcmpi(value,'Power') || strcmpi(value,'Power density')) && obj.pSpectrumTypeLegacySetting
            setVisualProperty(obj, 'SpectrumType', value, false);
            setVisualProperty(obj, 'ViewType', 'Spectrum', false);
            obj.pSpectrumTypeLegacySetting = false;
        else
            setVisualProperty(obj, 'SpectrumType', value, false);
        end
  end
  function value = get.SpectrumType(obj)    
    value = getVisualProperty(obj, 'SpectrumType', false);
  end
  %------------------------------------------------------------------------
  function set.ViewType(obj, value)    
    setVisualProperty(obj, 'ViewType', value, false);
  end
  function value = get.ViewType(obj)    
    value = getVisualProperty(obj, 'ViewType', false);
  end 
  %------------------------------------------------------------------------        
  function set.SampleRate(obj, value)    
    validateattributes(value,{'double'}, {'positive','real','scalar','finite','nonnan'},'','SampleRate');
    if strcmp(obj.InputDomain,'Time')
       setVisualProperty(obj, 'SampleRate', value, true);
    else
       setVisualProperty(obj, 'FrequencyInputSampleRate', value, true); 
    end
  end
  function value = get.SampleRate(obj)
    if strcmp(obj.InputDomain,'Time')  
       value = getVisualProperty(obj, 'SampleRate', true);
    else
       value = getVisualProperty(obj, 'FrequencyInputSampleRate', true); 
    end
  end
  %------------------------------------------------------------------------
  function set.Method(obj, value)
    validateattributes(value,{'char'},{},'','Method');
    setVisualProperty(obj, 'Method', value, false);
  end
  function value = get.Method(obj)    
    value = getVisualProperty(obj, 'Method', false);
  end
  %------------------------------------------------------------------------
  function set.PlotAsTwoSidedSpectrum(obj, value)
    validateattributes(value,{'logical'}, {'scalar'},'','PlotAsTwoSidedSpectrum');
    setVisualProperty(obj, 'TwoSidedSpectrum', value, false);    
    if ~value
      obj.FrequencyScale = 'Linear';
    end        
  end
  function value = get.PlotAsTwoSidedSpectrum(obj)
    value = getVisualProperty(obj, 'TwoSidedSpectrum', false);
  end
  %------------------------------------------------------------------------
  function set.FrequencyScale(obj, value)    
    if (obj.PlotAsTwoSidedSpectrum && strcmp(value,'Log') && strcmp(obj.InputDomain,'Time')) || ...
            (strcmp(obj.FrequencyVectorSource,'Auto') && strcmp(value,'Log') && strcmp(obj.InputDomain,'Frequenecy'))
      sendError(obj,'dspshared:SpectrumAnalyzer:InvalidFreqScale')   
    end            
    setVisualProperty(obj, 'FrequencyScale', value, false);     
  end
  function value = get.FrequencyScale(obj)
    value = getVisualProperty(obj, 'FrequencyScale', false);
  end
  
%--------------------------------------------------------------------------
% Advanced Properties Set/Get methods
%--------------------------------------------------------------------------   

  %------------------------------------------------------------------------
  function set.FrequencySpan(obj, value)    
    setVisualProperty(obj, 'FrequencySpan', value, false);       
    if isLocked(obj) && ~isCorrectionMode(obj)
      validatePropertiesImpl(obj);
    end                    
  end
  function value = get.FrequencySpan(obj)
    value = getVisualProperty(obj, 'FrequencySpan', false);
  end
  %------------------------------------------------------------------------
  function set.Span(obj, value)
    validateattributes(value,{'double'}, {'positive','real','scalar','finite','nonnan'},'','Span');    
    if isLocked(obj) && ~isCorrectionMode(obj) && ~isInactivePropertyImpl(obj,'Span')
      % Check valid span using the new span value. Do not do cross checking
      % with other properties if property is inactive or if visual is in
      % correction mode.
      span = value;
      Fstart = obj.CenterFrequency - span/2;
      Fstop = obj.CenterFrequency + span/2;
      validateSpan(obj,Fstart,Fstop);      
    end
    setVisualProperty(obj, 'Span', value, true);
  end
  function value = get.Span(obj)
    value = getVisualProperty(obj, 'Span', true);
  end
  %------------------------------------------------------------------------
  function set.CenterFrequency(obj, value)
    validateattributes(value,{'double'}, {'real','scalar','finite','nonnan'},'','CenterFrequency');
    if isLocked(obj) && ~isCorrectionMode(obj) && ~isInactivePropertyImpl(obj, 'CenterFrequency')
      % Check valid span using the new CF value. Do not do cross checking
      % with other properties if property is inactive or if visual is in
      % correction mode.
      CF = value;
      Fstart = CF - obj.Span/2;
      Fstop = CF + obj.Span/2;
      validateSpan(obj,Fstart,Fstop);      
    end
    setVisualProperty(obj, 'CenterFrequency', value, true);
  end
  function value = get.CenterFrequency(obj)
    value = getVisualProperty(obj, 'CenterFrequency', true);    
  end  
  %------------------------------------------------------------------------
  function set.StartFrequency(obj, value)
    validateattributes(value,{'double'}, {'real','scalar','finite','nonnan'},'','StartFrequency');
    if isLocked(obj) && ~isCorrectionMode(obj) && ~isInactivePropertyImpl(obj,'StartFrequency')
      % Check valid span using the new Fstart value. Do not do cross
      % checking with other properties if property is inactive or if visual
      % is in correction mode.
      Fstart = value;
      Fstop = obj.StopFrequency;
      validateSpan(obj,Fstart,Fstop);      
    end
    setVisualProperty(obj, 'StartFrequency', value, true);
  end
  function value = get.StartFrequency(obj)
    value = getVisualProperty(obj, 'StartFrequency', true);    
  end   
  %------------------------------------------------------------------------
  function set.StopFrequency(obj, value)
    validateattributes(value,{'double'}, {'real','scalar','finite','nonnan'},'','StopFrequency');
    if isLocked(obj) && ~isCorrectionMode(obj) && ~isInactivePropertyImpl(obj, 'StopFrequency')
      % Check valid span using the new Fstop value. Do not do cross
      % checking with other properties if property is inactive or if visual
      % is in correction mode.
      Fstart = obj.StartFrequency;
      Fstop = value;
      validateSpan(obj,Fstart,Fstop);
    end
    setVisualProperty(obj, 'StopFrequency', value, true);
  end
  function value = get.StopFrequency(obj)
    value = getVisualProperty(obj, 'StopFrequency', true);    
  end     
  %------------------------------------------------------------------------
  function value = get.FrequencyResolutionMethodSet(obj)
    if strcmpi(obj.Method,'Welch')
        value = obj.FrequencyResolutionMethodWelchSet;
    else
        value = obj.FrequencyResolutionMethodFilterBankSet;
    end
  end
  function set.FrequencyResolutionMethod(obj, value)
      if strcmp(obj.Method,'Welch')
         setVisualProperty(obj,'FrequencyResolutionMethodWelch', value, false);
      else
         setVisualProperty(obj,'FrequencyResolutionMethodFilterBank',value,false); 
      end
      setVisualProperty(obj,'FrequencyResolutionMethod',value,false);
  end
  function value = get.FrequencyResolutionMethod(obj)
      if strcmp(obj.Method,'Welch')
          value = getVisualProperty(obj, 'FrequencyResolutionMethodWelch', false);
      else
          value = getVisualProperty(obj, 'FrequencyResolutionMethodFilterBank', false);
      end
  end
  %------------------------------------------------------------------------
  function set.RBWSource(obj, value)            
    if strcmp(obj.InputDomain,'Time')
        setVisualProperty(obj, 'RBWSource', value, false);
        if isLocked(obj) && strcmp(value,'Property')  && ~isCorrectionMode(obj) ...
            && ~isInactivePropertyImpl(obj, 'RBW')
            Fstart = getFstart(obj);
            Fstop = getFstop(obj);
            validateSpan(obj,Fstart,Fstop, obj.RBW, value);
        end
    else
        setVisualProperty(obj,'FrequencyInputRBWSource',value, false);
    end
        
  end
  function value = get.RBWSource(obj)
    if strcmp(obj.InputDomain,'Time')
        value = getVisualProperty(obj, 'RBWSource', false);
    else
        value = getVisualProperty(obj, 'FrequencyInputRBWSource', false);
    end
  end    
  %------------------------------------------------------------------------
  function set.RBW(obj, value)
    validateattributes(value,{'double'}, {'positive','real','scalar','finite','nonnan'},'','RBW');   
    if isLocked(obj) && ~isCorrectionMode(obj) && ~isInactivePropertyImpl(obj, 'RBW')
        % Check validity of RBW only if locked, not in correction mode
        % and if property is active.
        Fstart = getFstart(obj);
        Fstop = getFstop(obj);
        if strcmp(obj.InputDomain,'Time')
            validateSpan(obj,Fstart,Fstop,value);
        end
    end
    setVisualProperty(obj, 'RBW', value, true); 
  end
  function value = get.RBW(obj)  
    value = getVisualProperty(obj, 'RBW', true);
  end   
  %------------------------------------------------------------------------
  function set.WindowLength(obj, value)
    validateattributes(value,{'double'}, {'positive','real','integer','scalar','>',2,'finite','nonnan'},'','WindowLength');
    if isLocked(obj) && ~isCorrectionMode(obj) && ~isInactivePropertyImpl(obj, 'FFTLength')
      NFFT = getVisualProperty(obj, 'FFTLength', true);
      if NFFT < value
        sendError(obj,'dspshared:SpectrumAnalyzer:InvalidFFTLength')
      end
    end    
    setVisualProperty(obj, 'WindowLength', value, true);
  end
  function value = get.WindowLength(obj)
    value = getVisualProperty(obj, 'WindowLength', true);    
  end
  %------------------------------------------------------------------------
  function set.FFTLengthSource(obj, value)    
    if strcmp(value,'Property') && ~isInactivePropertyImpl(obj, 'FFTLengthSource')
      % Try to get the visual property FFTLength to ensure it can be
      % evaluated (it might be set to a variable undefined in the
      % workspace).
     getVisualProperty(obj, 'FFTLength',true);
    end
    setVisualProperty(obj, 'FFTLengthSource', value, false);
  end
  function value = get.FFTLengthSource(obj)
    value = getVisualProperty(obj, 'FFTLengthSource', false);
  end  
  %------------------------------------------------------------------------
  function set.FFTLength(obj, value)
    validateattributes(value,{'double'}, {'positive','real','scalar','integer','finite','nonnan'},'','FFTLength');
    if isLocked(obj) && ~isCorrectionMode(obj) && ~isInactivePropertyImpl(obj, 'FFTLength')
        if strcmp(getVisualProperty(obj,'FrequencyResolutionMethod'),'WindowLength')
            % Window length as frequency resolution methods
            WL = getVisualProperty(obj, 'WindowLength', true);
            if value < WL
                sendError(obj,'dspshared:SpectrumAnalyzer:InvalidFFTLength')
            end
        else
            if value < 2
                sendError(obj,'dspshared:SpectrumAnalyzer:InvalidNumFrequencyBands')
            end
        end
    end
    setVisualProperty(obj, 'FFTLength', value, true);
  end
  function value = get.FFTLength(obj)
    value = getVisualProperty(obj, 'FFTLength', true);    
  end
  %------------------------------------------------------------------------
  function set.NumTapsPerBand(obj,value)
    validateattributes(value,...
      {'numeric'},...
      {'finite','real','scalar','>',0,'integer','even'},...
      '','NumTapsPerBand');
    setVisualProperty(obj, 'NumTapsPerBand', value, true);
  end
  function value = get.NumTapsPerBand(obj)
   value = getVisualProperty(obj, 'NumTapsPerBand', true); 
  end   
  %------------------------------------------------------------------------
  function set.FrequencyVectorSource(obj, value)    
    if strcmp(value,'Property') && ~isInactivePropertyImpl(obj, 'FrequencyVectorSource')
      % Try to get the visual property FrequencyVector to ensure it can be
      % evaluated (it might be set to a variable undefined in the
      % workspace).
     getVisualProperty(obj, 'FrequencyVector',true);
    end
    setVisualProperty(obj, 'FrequencyVectorSource', value, false);
  end
  function value = get.FrequencyVectorSource(obj)
    value = getVisualProperty(obj, 'FrequencyVectorSource', false);
  end
  %------------------------------------------------------------------------
  function set.FrequencyVector(obj, value)
    validateattributes(value,{'double'}, {'real','vector','finite','nonnan','nonempty'},'','FrequencyVector');
    if isscalar(value) || ~issorted(value) || abs(min(diff(value))) == 0
       % if the frequency vector is a scalar or is not monotonically
       % increasing or it has repeating values throw an error
       sendError(obj,'dspshared:SpectrumAnalyzer:InvalidFrequencyVectorUnlocked') 
    else
       setVisualProperty(obj, 'FrequencyVector', value, true); 
    end
  end
  function value = get.FrequencyVector(obj)
    value = getVisualProperty(obj, 'FrequencyVector', true);    
  end
  %------------------------------------------------------------------------
  function set.InputUnits(obj, value)
    setVisualProperty(obj, 'InputUnits', value, false);
  end
  function value = get.InputUnits(obj)
    value = getVisualProperty(obj, 'InputUnits', false);
  end 
  %------------------------------------------------------------------------
  function set.OverlapPercent(obj, value)
   validateattributes(value,{'double'}, {'real','scalar','>=',0,'<',100,'nonnan'},'','OverlapPercent');               
   setVisualProperty(obj, 'OverlapPercent', value, true);
  end
  function value = get.OverlapPercent(obj)
    value = getVisualProperty(obj, 'OverlapPercent', true);    
  end    
  %------------------------------------------------------------------------
  function set.Window(obj, value)    
    if any(strcmp({'Chebyshev','Kaiser'},value))
      % Try to get the visual property SidelobeAttenuation to ensure it can
      % be evaluated (it might be set to a variable undefined in the
      % workspace).
      getVisualProperty(obj, 'SidelobeAttenuation',true);
    end        
    setVisualProperty(obj, 'Window', value, false);    
  end
  function value = get.Window(obj)
    value = getVisualProperty(obj, 'Window', false);
  end
  %------------------------------------------------------------------------
  function set.CustomWindow(obj, value)
      % Add String Support
      value = convertStringsToChars(value);
      % Validate the provided function name is a character
      validateattributes(value,{'char'},{'nonempty'},'','CustomWindow')
      % validate the CustomWindow existence on the MATLAB path and validity
      % as a MATLAB function
      validateCustomWindow(obj, value);
      setVisualProperty(obj, 'CustomWindow', value, false);
  end
  function value = get.CustomWindow(obj)
     value = getVisualProperty(obj, 'CustomWindow'); 
  end
  %------------------------------------------------------------------------
  function set.SidelobeAttenuation(obj, value)
   validateattributes(value,{'double'}, {'positive','real','scalar','>=',45,'finite','nonnan'},'','SidelobeAttenuation');               
   setVisualProperty(obj, 'SidelobeAttenuation', value, true);
  end
  function value = get.SidelobeAttenuation(obj)
    value = getVisualProperty(obj, 'SidelobeAttenuation', true);    
  end  
  %------------------------------------------------------------------------
  function value = get.SpectrumUnitsSet(obj)
      if strcmpi(obj.InputDomain,'Time')
          if strcmpi(obj.SpectrumType,'Power')|| strcmpi(obj.SpectrumType,'Power density')
              value = obj.PowerUnitsSet;
          else
              value = obj.RMSUnitsSet;
          end
      else
         value = obj.FrequencyInputSpectrumUnitsSet; 
      end
  end
  function set.SpectrumUnits(obj, value)
      if strcmpi(obj.InputDomain, 'Time')
          if strcmpi(getVisualProperty(obj,'SpectrumType'),'Power') || strcmpi(getVisualProperty(obj,'SpectrumType'),'Power density')
              setVisualProperty(obj, 'PowerUnits', value, false);
          else
              setVisualProperty(obj, 'RMSUnits', value, false);
          end
      else
         setVisualProperty(obj,'FrequencyInputSpectrumUnits', value, false); 
      end
  end
  function value = get.SpectrumUnits(obj)
      if strcmpi(obj.InputDomain, 'Time')
      if strcmpi(getVisualProperty(obj,'SpectrumType'),'Power') || strcmpi(getVisualProperty(obj,'SpectrumType'),'Power density')
          value = getVisualProperty(obj, 'PowerUnits', false);
      else
          value = getVisualProperty(obj, 'RMSUnits', false);
      end
      else
         value =  getVisualProperty(obj, 'FrequencyInputSpectrumUnits', false);
      end
  end   
  %------------------------------------------------------------------------
  function set.PowerUnits(obj, value)
    setVisualProperty(obj, 'PowerUnits', value, false);
  end
  function value = get.PowerUnits(obj)
    value = getVisualProperty(obj, 'PowerUnits', false);
  end
  %------------------------------------------------------------------------
  function set.FullScaleSource(obj, value)            
    setVisualProperty(obj, 'FullScaleSource', value, false);
  end
  function value = get.FullScaleSource(obj)
    value = getVisualProperty(obj, 'FullScaleSource', false);
  end
  %------------------------------------------------------------------------
  function set.FullScale(obj, value)
   validateattributes(value,{'double'}, {'positive','real','scalar','finite','nonnan'},'','FullScale')
   setVisualProperty(obj, 'FullScale', value, true);
  end   
  function value = get.FullScale(obj)    
    value = getVisualProperty(obj, 'FullScale', true);
  end
  %------------------------------------------------------------------------
  function set.AveragingMethod(obj, value)
    validateattributes(value,{'char'},{},'','AveragingMethod');
    setVisualProperty(obj, 'AveragingMethod', value, false);
  end
  function value = get.AveragingMethod(obj)    
    value = getVisualProperty(obj, 'AveragingMethod', false);
  end 
  %------------------------------------------------------------------------
  function set.SpectralAverages(obj, value)
    validateattributes(value,{'double'}, {'positive','real','scalar','integer','finite','nonnan'},'','SpectralAverages');
    setVisualProperty(obj, 'SpectralAverages', value, true);
  end
  function value = get.SpectralAverages(obj)
    value = getVisualProperty(obj, 'SpectralAverages', true);
  end
  %------------------------------------------------------------------------
  function set.ForgettingFactor(obj, value)
    validateattributes(value,{'double'}, {'real','positive','scalar','>',0,'<=',1,'finite','nonnan'},'','ForgettingFactor');
    setVisualProperty(obj, 'ForgettingFactor', value, true);
  end
  function value = get.ForgettingFactor(obj)
    value = getVisualProperty(obj, 'ForgettingFactor', true);
  end  
  %------------------------------------------------------------------------
  function set.ReferenceLoad(obj, value)
    validateattributes(value,{'double'}, {'positive','real','scalar','finite','nonnan'},'','ReferenceLoad');
    setVisualProperty(obj, 'ReferenceLoad', value, true);
  end
  function value = get.ReferenceLoad(obj)
    value = getVisualProperty(obj, 'ReferenceLoad', true);
  end  
  %------------------------------------------------------------------------
  function set.FrequencyOffset(obj, value)
    validateattributes(value,{'double'}, {'real','vector','finite','nonnan'},'','FrequencyOffset');
    setVisualProperty(obj, 'FrequencyOffset', value, true);
  end
  function value = get.FrequencyOffset(obj)
    value = getVisualProperty(obj, 'FrequencyOffset', true);
  end
    
%--------------------------------------------------------------------------
% Spectrogram Properties Set/Get methods
%--------------------------------------------------------------------------   
  
  %------------------------------------------------------------------------
  function set.SpectrogramChannel(obj, value)
    validateattributes(value,{'double'},{'real','scalar','integer','finite','nonnan'},'','SpectrogramChannel');
    if ~isLocked(obj)
        if value <= 0 || value > 100
            % if the channel number is less than 0 or greater than 100 throw an
            % error
            sendError(obj,'dspshared:SpectrumAnalyzer:InvalidSpectrogramChannelUnlocked');
        end
    else
        if value <= 0 || value > obj.pNumChannels
            sendError(obj,'dspshared:SpectrumAnalyzer:InvalidSpectrogramChannel', obj.pNumChannels)
        end
    end
    setVisualProperty(obj, 'ChannelNumber', value, true);
  end
  function value = get.SpectrogramChannel(obj)
    value = getVisualProperty(obj, 'ChannelNumber', true);
  end
  %------------------------------------------------------------------------
  function set.TimeResolutionSource(obj, value)            
    setVisualProperty(obj, 'TimeResolutionSource', value, false);
  end
  function value = get.TimeResolutionSource(obj)
    value = getVisualProperty(obj, 'TimeResolutionSource', false);
  end    
  %------------------------------------------------------------------------
  function set.TimeResolution(obj, value)
    validateattributes(value,{'double'}, {'positive','real','scalar','finite','nonnan'},'','TimeResolution');
    setVisualProperty(obj, 'TimeResolution', value, true);
  end
  function value = get.TimeResolution(obj)
    value = getVisualProperty(obj, 'TimeResolution', true);    
  end         
  %------------------------------------------------------------------------
  function set.TimeSpanSource(obj, value)            
    setVisualProperty(obj, 'TimeSpanSource', value, false);
  end
  function value = get.TimeSpanSource(obj)
    value = getVisualProperty(obj, 'TimeSpanSource', false);
  end    
  %------------------------------------------------------------------------
  function set.TimeSpan(obj, value)
    validateattributes(value,{'double'}, {'positive','real','scalar','finite','nonnan'},'','TimeSpan'); 
    setVisualProperty(obj, 'TimeSpan', value, true);
  end
  function value = get.TimeSpan(obj)
    value = getVisualProperty(obj, 'TimeSpan', true);    
  end
  
%--------------------------------------------------------------------------
% Measurements Properties Set/Get methods
%--------------------------------------------------------------------------  
  
  %------------------------------------------------------------------------
  function set.MeasurementChannel(obj, value)
       validateattributes(value,{'double'},{'real','scalar','integer','finite','nonnan'},'','MeasurementChannel');
    if value <= 0 || value > 100
        % if the channel number is less than 0 or greater than 100 throw an
        % error
        sendError(obj,'dspshared:SpectrumAnalyzer:InvalidMeasurementChannelUnlocked');
    else
        setVisualProperty(obj, 'MeasurementChannelNumber', value, true);
    end
  end
  function value = get.MeasurementChannel(obj)
    value = getVisualProperty(obj, 'MeasurementChannelNumber', true);
  end 
  %------------------------------------------------------------------------
  function set.SpectralMask(obj, value)
    if isa(value, 'SpectralMaskSpecification') && isvalid(value)
      if ~isScopeLaunched(obj)
        launchScope(obj);
      end
      setSpectralMask(obj.pFramework.Visual, value);
    else
      sendError(obj,'dspshared:SpectrumAnalyzer:invalidMaskSpecification','SpectralMask');
    end
  end
  function value = get.SpectralMask(obj)
    if ~isScopeLaunched(obj)
      launchScope(obj);
    end
    value = getSpectralMask(obj.pFramework.Visual);
  end
  %------------------------------------------------------------------------
  function value = get.PeakFinder(obj)
      if ~isScopeLaunched(obj)
          launchScope(obj);
      end
      value = getPeakFinder(obj.pFramework.Visual);
  end
  %------------------------------------------------------------------------
  function value = get.CursorMeasurements(obj)
      if ~isScopeLaunched(obj)
          launchScope(obj);
      end
      value = getCursorMeasurements(obj.pFramework.Visual);
  end
  %------------------------------------------------------------------------
  function value = get.ChannelMeasurements(obj)
      if ~isScopeLaunched(obj)
          launchScope(obj);
      end
      value = getChannelMeasurements(obj.pFramework.Visual);
  end
  %------------------------------------------------------------------------
  function value = get.DistortionMeasurements(obj)
      if ~isScopeLaunched(obj)
          launchScope(obj);
      end
      value = getDistortionMeasurements(obj.pFramework.Visual);
  end
  %------------------------------------------------------------------------ 
  function value = get.CCDFMeasurements(obj)
      warning(message('dspshared:SpectrumAnalyzer:CCDFMeasurementsObsolete'));
      if ~isScopeLaunched(obj)
          launchScope(obj);
      end
      value = getCCDFMeasurements(obj.pFramework.Visual);
  end

%--------------------------------------------------------------------------
% Visualization Properties Set/Get methods
%--------------------------------------------------------------------------   
  
  %------------------------------------------------------------------------        
  function set.Name(obj, value)
    % Add String Support
    value = convertStringsToChars(value);
    % Set the name in the scope.
    setScopeName(obj, value);
    obj.Name = value;
  end  
  %------------------------------------------------------------------------
  function set.PlotNormalTrace(obj, value)
    validateattributes(value,{'logical'}, {'scalar'},'','PlotNormalTrace');
    setVisualProperty(obj, 'NormalTrace', value, false);
  end
  function value = get.PlotNormalTrace(obj)
    value = getVisualProperty(obj, 'NormalTrace', false);
  end 
  %------------------------------------------------------------------------
  function set.PlotMaxHoldTrace(obj, value)
      validateattributes(value,{'logical'}, {'scalar'},'','PlotMaxHoldTrace');
      setVisualProperty(obj, 'MaxHoldTrace', value, false);
  end
  function value = get.PlotMaxHoldTrace(obj)
      value = getVisualProperty(obj, 'MaxHoldTrace', false);
  end
  %------------------------------------------------------------------------
  function set.PlotMinHoldTrace(obj, value)
    validateattributes(value,{'logical'}, {'scalar'},'','PlotMinHoldTrace');
    setVisualProperty(obj, 'MinHoldTrace', value, false);
  end
  function value = get.PlotMinHoldTrace(obj)
    value = getVisualProperty(obj, 'MinHoldTrace', false);
  end
  %------------------------------------------------------------------------
  function set.PlotType(obj, value)
    setVisualProperty(obj, 'PlotType', value);
  end
  function value = get.PlotType(obj)
    value = getVisualProperty(obj, 'PlotType');
  end
  %------------------------------------------------------------------------        
  function set.ReducePlotRate(obj, value)
    validateattributes(value,{'logical'}, {'scalar'},'','ReducePlotRate');
    setVisualProperty(obj, 'ReduceUpdates', value);
  end  
  function value = get.ReducePlotRate(obj)
    value = getVisualProperty(obj, 'ReduceUpdates');
  end
  %------------------------------------------------------------------------
  function set.Title(obj, value)
    % Add String Support
    value = convertStringsToChars(value);
    validateattributes(value,{'char'}, {},'','Title');   
    setVisualProperty(obj, 'Title', value, false);
  end  
  function value = get.Title(obj)
    value = getVisualProperty(obj,'Title', false);
  end
  %------------------------------------------------------------------------
  function set.YLabel(obj, value)
    % Add String Support
    value = convertStringsToChars(value);    
    validateattributes(value,{'char'}, {},'','YLabel');
    setVisualProperty(obj, 'YLabel', value);
  end  
  function value = get.YLabel(obj)
    value = getVisualProperty(obj,'YLabel');
  end
  %------------------------------------------------------------------------        
  function set.ShowLegend(obj, value)
    validateattributes(value,{'logical'}, {'scalar'},'','ShowLegend');
    setVisualProperty(obj, 'Legend', value);
  end  
  function value = get.ShowLegend(obj)
    value = getVisualProperty(obj, 'Legend');
  end
  %------------------------------------------------------------------------        
  function set.ChannelNames(obj, value)
    if ~iscell(value) || (~isempty(value) && (~isvector(value) || ~iscellstr(cellstr(value))))
      error(message('dspshared:SpectrumAnalyzer:InvalidChannelNames', 'ChannelNames'));
    end
    value = cellstr(value);
    setVisualProperty(obj, 'UserDefinedChannelNames', value);
  end
  function value = get.ChannelNames(obj)
    value = getVisualProperty(obj, 'UserDefinedChannelNames');
  end
  %------------------------------------------------------------------------        
  function set.ShowGrid(obj, value)
    validateattributes(value,{'logical'}, {'scalar'},'','ShowGrid');
    setVisualProperty(obj, 'Grid', value);
  end  
  function value = get.ShowGrid(obj)
    value = getVisualProperty(obj, 'Grid');
  end 
  %------------------------------------------------------------------------
  function set.YLimits(obj, value)
    % Check that value is a 2-element numeric vector
    if ~all(isnumeric(value)) || ~all(isfinite(value)) || ...
        numel(value)~=2 || value(1) >= value(2) ||any(~isreal(value))
       sendError(obj,'dspshared:SpectrumAnalyzer:InvalidLimits','YLimits');
    end
    if value(1) > obj.YLimits(1)
        % If MinYLim is increased, update MaxYLim followed by MinYLim.
        setVisualProperty(obj, 'MaxYLim', value(2), true);
        setVisualProperty(obj, 'MinYLim', value(1), true);
    else
        % If MinYLim is decreased, update MinYLim followed by MaxYLim.
        setVisualProperty(obj, 'MinYLim', value(1), true);
        setVisualProperty(obj, 'MaxYLim', value(2), true);
    end
    
     % Turn off autoscaling if the YLimits are set directly.
      fevalOnExtension(obj.pFramework, 'Tools', 'Plot Navigation', ...
          @(h) setPropertyValue(h, 'AutoscaleMode', 'Manual', 'OnceAtStop', false));
  end
  function value = get.YLimits(obj)
    value = ...
      [getVisualProperty(obj, 'MinYLim', true) ...
      getVisualProperty(obj, 'MaxYLim', true)];
  end
  %------------------------------------------------------------------------      
  function set.ColorLimits(obj, value)
    % Check that value is a 2-element numeric vector
    if ~all(isnumeric(value)) || ~all(isfinite(value)) || ...
        numel(value)~=2 || value(1) >= value(2) || any(~isreal(value))
       sendError(obj,'dspshared:SpectrumAnalyzer:InvalidLimits','ColorLimits');
    end
    if value(1) > obj.ColorLimits(1)
        % If MinColorLim is increased, update MaxColorLim followed by MinColorLim.
        setVisualProperty(obj, 'MaxColorLim', value(2), true);
        setVisualProperty(obj, 'MinColorLim', value(1), true);
    else
        % If MinColorLim is decreased, update MinColorLim followed by MaxColorLim.
        setVisualProperty(obj, 'MinColorLim', value(1), true);
        setVisualProperty(obj, 'MaxColorLim', value(2), true);
    end
     % Turn off autoscaling if the YLimits are set directly.
      fevalOnExtension(obj.pFramework, 'Tools', 'Plot Navigation', ...
          @(h) setPropertyValue(h, 'AutoscaleMode', 'Manual', 'OnceAtStop', false));
  end
  function value = get.ColorLimits(obj)
    value = ...
      [getVisualProperty(obj, 'MinColorLim', true) ...
      getVisualProperty(obj, 'MaxColorLim', true)];
  end
  %------------------------------------------------------------------------
  function set.AxesScaling(obj, newScaling)
      setAxesScaling(obj, newScaling)
  end
  function scaling = get.AxesScaling(obj)
      scaling = getAxesScaling(obj);
  end
  %------------------------------------------------------------------------
  function set.AxesLayout(obj, value)
    validateattributes(value,{'char'},{},'','AxesLayout');
    setVisualProperty(obj, 'AxesLayout', value, false);
  end
  function value = get.AxesLayout(obj)    
    value = getVisualProperty(obj, 'AxesLayout', false);
  end 
end

%--------------------------------------------------------------------------
% Public, hidden methods
%--------------------------------------------------------------------------
methods (Hidden)
  %------------------------------------------------------------------------  
  function st = getInputSampleTime(obj)
    % Return frame time
    if obj.NumInputPorts > 1 && isscalar(obj.SampleRate)
      st = 1./obj.SampleRate*ones(1,obj.NumInputPorts);
    else
      st = 1./obj.SampleRate;
    end
    dims = ones(1,obj.NumInputPorts);
    for ii=1:obj.NumInputPorts
      dm = getMaxDimensions(obj.pSource,ii);
      dims(ii) = dm(1);
    end
    st = st.*dims;
  end
end

%--------------------------------------------------------------------------
% Protected methods
%--------------------------------------------------------------------------
methods (Access = protected)
  %------------------------------------------------------------------------
  function hScopeCfg = getScopeCfg(~)
    hScopeCfg = spbscopes.SpectrumAnalyzerSystemObjectCfg;
  end
  %------------------------------------------------------------------------          
  function num = getNumInputsImpl(obj)
    num = obj.NumInputPorts;
  end
  %------------------------------------------------------------------------          
  function validateInputsImpl(obj,varargin)
    % All inputs must have same frame size
    if obj.NumInputPorts > 1
      [nrows, ~] = cellfun(@size, varargin);
      if numel(unique(nrows))~=1
        sendError(obj,'Spcuilib:scopes:FrameSizesDiffer');
      end
    end
    % Concatenate all inputs
    x = horzcat(varargin{:});
    
    % Complex Inputs are not allowed for frequency-domain data
    if (strcmp(obj.InputDomain, 'Frequency') && ~isreal(x))
      sendError(obj,'dspshared:SpectrumAnalyzer:InvalidInputDomainComplexity')  
    end
    
    % All inputs must be matrices
    if ~ismatrix(x)
      sendError(obj,'MATLAB:system:inputDimsMoreThanTwo',class(obj));
    end
    % Number of Channels
    channLim = 100;
    numChannels = size(x,2);
    if obj.pNumChannels~=-1 && obj.pNumChannels ~= numChannels
      sendError(obj,'dsp:system:varSizeChannelsNotSupported');
    end
    if numChannels > channLim
      sendError(obj,'dspshared:SpectrumAnalyzer:TooManyInputChannels',channLim);
    end
    if numChannels == 0
      sendError(obj,'dspshared:SpectrumAnalyzer:ZeroChannels');
    end
    
    % If the Frequency Resoltion Method is NumFrequencyBands and FFTLength
    % is Auto and the input is scalar throw an error
    if (size(x,1) == 1 && strcmp(obj.FrequencyResolutionMethod,'NumFrequencyBands') ...
            && strcmp(obj.FFTLengthSource,'Auto')) || (size(x,1) == 1 && strcmp(obj.InputDomain, 'Frequency'))
       sendError(obj,'dspshared:SpectrumAnalyzer:ScalarInput'); 
    end
    % Validate selected channel for spectrogram
    if obj.SpectrogramChannel > numChannels
        % if the channel number is greater than the number of input
        % channels throw an error
        sendError(obj,'dspshared:SpectrumAnalyzer:InvalidSpectrogramChannel', numChannels);
    end
    % Validate selected channel for spectral mask
    validateSpectralMaskSelectedChannel(obj, numChannels);
  end
  %------------------------------------------------------------------------          
  function validatePropertiesImpl(obj)    
    % Verify correct span            

    % Since properties can be set to variables in the visual, we need to
    % check the validity of the types and values.
    
    % validate properties common to the two input domain modes
    validateattributes(obj.SampleRate,{'double'}, {'positive','real','scalar'},'','SampleRate');
    validateattributes(obj.ReferenceLoad,{'double'}, {'positive','real','scalar'},'','ReferenceLoad');
    validateattributes(obj.FrequencyOffset,{'double'}, {'real','vector'},'','FrequencyOffset');
    if strcmp(obj.RBWSource,'Property')
        validateattributes(obj.RBW,{'double'}, {'positive','real','scalar'},'','RBW');
    end
    % validate properties valid only for time-domain data
    if strcmp(obj.InputDomain,'Time')
        if strcmp(obj.FrequencySpan, 'Span and center frequency')
            validateattributes(obj.Span,{'double'}, {'positive','real','scalar'},'','Span');
            validateattributes(obj.CenterFrequency,{'double'}, {'real','scalar'},'','CenterFrequency');
        elseif strcmp(obj.FrequencySpan, 'Start and stop frequencies')
            validateattributes(obj.StartFrequency,{'double'}, {'real','scalar'},'','StartFrequency');
            validateattributes(obj.StopFrequency,{'double'}, {'real','scalar'},'','StopFrequency');
        end
                
        if strcmp(obj.FFTLengthSource,'Property')
            validateattributes(obj.FFTLength,{'double'}, {'positive','real','scalar','integer'},'','FFTLength');
        end
        
        validateattributes(obj.FullScale,{'double'}, {'positive','real','scalar'},'','FullScale');
        
        if any(strcmp({'Chebyshev','Kaiser'},obj.Window))
            validateattributes(obj.SidelobeAttenuation,{'double'}, {'positive','real','scalar'},'','SidelobeAttenuation');
        end
        
        if strcmp(obj.Window,'Custom')
            validateattributes(obj.CustomWindow,{'char'},{'nonempty'},'','CustomWindow');
            validateCustomWindow(obj,obj.CustomWindow);
        end
        
        % Validate properties relevant when spectrum type is power or power
        % density or RMS
        if strcmp(obj.ViewType,'Spectrum')
            validateattributes(obj.SpectralAverages,{'double'}, {'positive','real','scalar','integer'},'','SpectralAverages');
            validateattributes(obj.OverlapPercent,{'double'}, {'real','scalar','>=',0,'<',100,'nonnan'},'','OverlapPercent');
        end
        % Check overall span --> must be within Nyquist interval
        if ~isCorrectionMode(obj)
            Fstart = getFstart(obj);
            Fstop = getFstop(obj);
            validateSpan(obj, Fstart, Fstop);
        end
    else
        if strcmp(obj.FrequencyVectorSource,'Property')
            validateattributes(obj.FrequencyVector,{'double'}, {'real','vector'},'','FrequencyVector');
        end
    end
    % Validate properties relevant when view type is spectrogram or
    % spectrum and spectrogram
    if any(strcmp(obj.ViewType,{'Spectrogram','Spectrum and spectrogram'}))
      if strcmp(obj.TimeSpanSource,'Property')
        validateattributes(obj.TimeSpan,{'double'}, {'positive','real','scalar'},'','TimeSpan');
      end
      if strcmp(obj.TimeResolutionSource,'Property')
        validateattributes(obj.TimeResolution,{'double'}, {'positive','real','scalar'},'','TimeResolution');
      end
    else
      % All traces cannot be off simultaneously
      if ~obj.PlotMaxHoldTrace && ~obj.PlotMinHoldTrace && ~obj.PlotNormalTrace
        sendError(obj,'dspshared:SpectrumAnalyzer:AllTracesOff')
      end
    end
    % Validate properties relevant when averaging method is exponential
    if strcmpi(obj.AveragingMethod,'Exponential')
      validateattributes(obj.ForgettingFactor,{'double'}, {'positive','real','scalar','>',0,'<=',1,'finite','nonnan'},'','ForgettingFactor');  
    end
  end
  %------------------------------------------------------------------------
  function setupImpl(obj, varargin)
    validateInputs(obj,varargin{:});
    % if NumInputPorts > 1 convert all inputs to double to avoid precision
    % loss by horzcat. 
    if obj.NumInputPorts > 1
       for idx = 1:numel(varargin)
          varargin{idx} = double(varargin{idx}); 
       end
    end
    % Concatenate all inputs
    x = horzcat(varargin{:});   
    obj.pNumChannels = size(x, 2);
    setupImpl@matlabshared.scopes.UnifiedSystemScope(obj, varargin{:});
    onSourceRun(obj.pFramework.Visual);
    % Add a tooltip to the Simulation time in status bar
    hFrame = obj.pFramework.Handles.frameStatus;
    hFrame.Tooltip = obj.pSource.getTimeStatusTooltip; 
  end
  %------------------------------------------------------------------------
  function updateImpl(obj, varargin)
    % If the scope does not exist, create one and post new data
    if isempty(obj.pFramework)
      launchScope(obj);
    end
    if obj.pSource.NumInputPorts == 1
      update(obj.pSource, varargin{:});
    else
      update(obj.pSource, horzcat(varargin{:}));
    end
  end
  %------------------------------------------------------------------------
  function S = saveObjectImpl(obj)
    S = saveObjectImpl@matlabshared.scopes.UnifiedSystemScope(obj);    
    S.CurrentLineProperties = getPropertyValue(obj.pFramework.Visual, 'LineProperties');
    % Properties that are not part of the Scope Configuration should be
    % saved explicitly in the serialized object.
    S.NumInputPorts = obj.NumInputPorts;
  end  
  %------------------------------------------------------------------------  
  function loadObjectImpl(obj, S, wasLocked)
    
    if isfield(S, 'NumInputPorts')
      obj.NumInputPorts = S.NumInputPorts;
    end
    loadObjectImpl@matlabshared.scopes.UnifiedSystemScope(obj, S, wasLocked);
    % Handle loading saved SpectrumAnalyzer objects with SpectrumType as
    % Spectrogram
    if strcmpi(obj.SpectrumType,'Spectrogram')
        setPropertyValue(obj.pFramework.Visual,'SpectrumType','Power');
        setPropertyValue(obj.pFramework.Visual,'ViewType','Spectrogram');
    end
    if wasLocked
      onSourceRun(obj.pFramework.Visual); 
      if strcmp(obj.ViewType,'Spectrogram')
        % Reset to ensure we blank the screen
        reset(obj.pFramework.Visual);
      end
    else
      % IsValidSettingsDialogReadouts is no longer stored in the
      % PropertySet. This is to ensure backward compatibility.
      hVisual = obj.pFramework.Visual;
      if ~isempty(hVisual.Config) && ~isempty(hVisual.Config.PropertySet) && ...
          isValidProperty(hVisual.Config.PropertySet,'IsValidSettingsDialogReadouts')
        setPropertyValue(hVisual,'IsValidSettingsDialogReadouts',false);
      end
    end
    
    setPropertyValue(obj.pFramework.Visual, 'LineProperties',S.CurrentLineProperties);
    loadLineProperties(obj.pFramework.Visual);
    % Load measurements properties saved on the propertyset
    updateMeasurementsPropertyValues(obj.pFramework.Visual);
  end            
  %------------------------------------------------------------------------
  function flag = isInactivePropertyImpl(obj, prop)
      flag = false;
      switch prop
          case {'SpectrumType','Method','FrequencyResolutionMethod','FrequencySpan'}
              flag = strcmp(obj.InputDomain,'Frequency');
          case {'Span','CenterFrequency'}
              flag = any(strcmp({'Full','Start and stop frequencies'},obj.FrequencySpan)) || strcmp(obj.InputDomain,'Frequency');
          case {'StartFrequency','StopFrequency'}
              flag = any(strcmp({'Full','Span and center frequency'},obj.FrequencySpan)) || strcmp(obj.InputDomain,'Frequency');
          case 'RBWSource'
              flag = ~strcmp(obj.FrequencyResolutionMethod,'RBW');
          case 'RBW'
              flag = ~strcmp(obj.FrequencyResolutionMethod,'RBW') || strcmp(obj.RBWSource,'Auto');
          case 'FrequencyVectorSource'
              flag = strcmp(obj.InputDomain,'Time');
          case 'FrequencyVector'
              flag = strcmp(obj.InputDomain,'Time') || strcmp(obj.FrequencyVectorSource,'Auto');
          case 'WindowLength'
              flag = ~strcmp(obj.FrequencyResolutionMethod,'WindowLength') ...
                  ||  strcmp(obj.Method,'Filter bank') || strcmp(obj.InputDomain,'Frequency');
          case 'FFTLengthSource'
              flag = strcmp(obj.FrequencyResolutionMethod,'RBW') || strcmp(obj.InputDomain,'Frequency');
          case 'FFTLength'
              flag = strcmp(obj.FrequencyResolutionMethod,'RBW') ...
                  || strcmp(obj.FFTLengthSource,'Auto') || strcmp(obj.InputDomain,'Frequency');
          case 'SidelobeAttenuation'
              flag = ~any(strcmp({'Chebyshev','Kaiser'},obj.Window)) ||  strcmp(obj.Method,'Filter bank') || strcmp(obj.InputDomain,'Frequency');
          case {'PlotMaxHoldTrace', 'PlotMinHoldTrace', 'PlotNormalTrace'}
              flag = strcmp(obj.ViewType,'Spectrogram');
          case {'YLimits','YLabel','ShowLegend'}
              flag = strcmp(obj.ViewType,'Spectrogram');
          case 'AveragingMethod'
              flag = strcmp(obj.InputDomain,'Frequency');
          case 'SpectralAverages'
              flag = strcmp(obj.AveragingMethod,'Exponential') || strcmp(obj.ViewType,'Spectrogram') || strcmp(obj.ViewType,'Spectrum and spectrogram') || strcmp(obj.InputDomain,'Frequency');
          case 'ForgettingFactor'
              flag = strcmp(obj.AveragingMethod,'Running') || strcmp(obj.InputDomain,'Frequency');
          case {'PlotType'}
              flag = strcmp(obj.ViewType,'Spectrogram') || ~obj.PlotNormalTrace;
          case 'TimeSpanSource'
              flag = strcmp(obj.ViewType,'Spectrum');
          case 'TimeSpan'
              flag = strcmp(obj.ViewType,'Spectrum') || strcmp(obj.TimeSpanSource,'Auto');
          case 'TimeResolutionSource'
              flag = strcmp(obj.ViewType,'Spectrum') || strcmp(obj.InputDomain,'Frequency');
          case 'TimeResolution'
              flag = strcmp(obj.ViewType,'Spectrum') || strcmp(obj.TimeResolutionSource,'Auto') || strcmp(obj.InputDomain,'Frequency');
          case 'AxesLayout'
              flag = strcmp(obj.ViewType,'Spectrum') || strcmp(obj.ViewType,'Spectrogram') || strcmp(obj.InputDomain,'Frequency');
          case 'ReferenceLoad'
              flag = strcmp(obj.SpectrumType,'RMS') || strcmp(obj.SpectrumUnits,'dBFS');
          case 'ColorLimits'
              flag = strcmp(obj.ViewType,'Spectrum') || strcmp(obj.InputDomain,'Frequency');
          case 'CustomWindow'
              flag = ~strcmp(obj.Window,'Custom') ||  strcmp(obj.Method,'Filter bank') || strcmp(obj.InputDomain,'Frequency');
          case 'FullScale'
              flag = ~strcmp(obj.SpectrumUnits,'dBFS') || strcmp(obj.FullScaleSource,'Auto');
          case  'FullScaleSource'
              flag = ~strcmp(obj.SpectrumUnits,'dBFS');
          case 'NumTapsPerBand'
              flag = strcmp(obj.Method,'Welch') || strcmp(obj.InputDomain,'Frequency');
          case {'Window','OverlapPercent'}
              flag = strcmp(obj.Method,'Filter bank') || strcmp(obj.InputDomain,'Frequency');
          case 'InputUnits'
              flag = strcmp(obj.InputDomain,'Time');
          case 'PlotAsTwoSidedSpectrum'
              flag = ~strcmp(obj.FrequencyVectorSource,'Auto');
          case 'SpectrogramChannel'
              flag = strcmp(obj.ViewType,'Spectrum');
          case 'SpectralMask'
              flag = strcmp(obj.ViewType,'Spectrogram') || any(strcmp(obj.SpectrumUnits, {'Watts','Vrms','dBV'}));
          case {'PeakFinder','ChannelMeasurements','DistortionMeasurements'}
              flag = strcmp(obj.ViewType,'Spectrogram');
          case {'CCDFMeasurements'}
              flag = strcmp(obj.InputDomain,'Frequency');
      end
  end 
  %------------------------------------------------------------------------
  function resetImpl(obj)
    resetImpl@matlabshared.scopes.UnifiedSystemScope(obj);
    hVisual = obj.pFramework.Visual;
    reset(hVisual.SpectrumObject);
    flush(hVisual.DataBuffer);
    % Reset visual public properties used with get spectrum data
    hVisual.SpectrumData = struct('SimulationTime',[], ...
        'Spectrum', [], ...
        'Spectrogram', [], ...
        'MinHoldTrace', [], ...
        'MaxHoldTrace', [], ...
        'FrequencyVector', [], ...
        'SpectrumUnits', []);
    hVisual.IsNewDataReady = false;
  end 
  %------------------------------------------------------------------------
  function releaseImpl(obj)
    % Flush buffer only if framework has not been cleared
    if isvalid(obj.pFramework)
      forceUpdateVisual(obj);
      flush(obj.pFramework.Visual.DataBuffer);
    end
    releaseImpl@matlabshared.scopes.UnifiedSystemScope(obj);
    obj.pNumChannels = -1;
  end 
  %------------------------------------------------------------------------
  function flag = isInputSizeMutableImpl(~,~)
     flag = true; 
  end
end

%--------------------------------------------------------------------------
% Private methods
%--------------------------------------------------------------------------
methods(Access = private)
  %------------------------------------------------------------------------     
  function sendError(~,id,varargin)
    throwAsCaller(MException(message(id, varargin{:})));
  end
  %------------------------------------------------------------------------       
  function forceUpdateVisual(obj)
    % Force update the visual if in ReducePlotRate mode. This function is
    % called when release() or getSpectralMaskStatus() is called.
    if obj.ReducePlotRate
      while ~obj.pFramework.Visual.DataBuffer.isTimeUp
        % No op, let the timer time out
      end
      if obj.pFramework.Visual.DataBuffer.IsReady
        % If buffer has data, update display and sim time one more time.
        % This ensures correct simulation time when we are in asynchronous
        % mode.
        update(obj.pFramework.DataSource.Updater);
      end
    end
  end
  %------------------------------------------------------------------------
  function span = getSpan(obj)    
    span = getFstop(obj) - getFstart(obj);
  end
  %------------------------------------------------------------------------       
  function fstart = getFstart(obj)
    Fs = obj.SampleRate;
    if strcmp(obj.FrequencySpan,'Full')
      fstart = -Fs/2*obj.PlotAsTwoSidedSpectrum;        
    elseif strcmp(obj.FrequencySpan,'Span and center frequency') 
        if ~obj.PlotAsTwoSidedSpectrum && obj.CenterFrequency == 0
            fstart = 0;
        else
            fstart = obj.CenterFrequency - obj.Span/2;
        end
    else
      fstart = obj.StartFrequency;
    end      
  end
  %------------------------------------------------------------------------       
  function fstop = getFstop(obj)
    Fs = obj.SampleRate;
    if strcmp(obj.FrequencySpan,'Full')
      fstop = Fs/2;
    elseif strcmp(obj.FrequencySpan,'Span and center frequency') 
        if ~obj.PlotAsTwoSidedSpectrum && obj.CenterFrequency == 0
            fstop = obj.Span;
        else
            fstop = obj.CenterFrequency + obj.Span/2;
        end
    else
      fstop = obj.StopFrequency;
    end      
  end      
  %------------------------------------------------------------------------         
  function flag = isCorrectionMode(obj)
    flag = isCorrectionMode(obj.pFramework.Visual);
  end
  % -----------------------------------------------------------------------
  function validateSpectralMaskSelectedChannel(obj, inputNumChannels)
    if ~(strcmp(obj.ViewType, 'Spectrogram') || strcmp(obj.SpectrumUnits, 'Watts') || ...
        strcmp(obj.SpectralMask.EnabledMasks, 'None') || isCCDFMode(obj.pFramework.Visual)) && ...
        strcmp(obj.SpectralMask.ReferenceLevel, 'Spectrum peak')
      % Validate SelectedChannel
      maskSelectedChannel = obj.SpectralMask.SelectedChannel;
      if isnumeric(maskSelectedChannel) && maskSelectedChannel > inputNumChannels
        sendError(obj, 'dspshared:SpectrumAnalyzer:invalidMaskChannelNumber');
      end
    end
  end
end

%--------------------------------------------------------------------------
% Static Hidden methods
%--------------------------------------------------------------------------
methods(Static, Hidden)
  %------------------------------------------------------------------------
  function a = getAlternateBlock
   a = 'dspsnks4/Spectrum Analyzer';
  end
end

%--------------------------------------------------------------------------
% Static Protected methods
%--------------------------------------------------------------------------
methods(Static, Access=protected)
  function groups = getPropertyGroupsImpl
        % Only the properties listed in mainProps are displayed in the
    % command-line by default
     mainProps = {'NumInputPorts', ...
        'InputDomain', ...
        'SpectrumType', ...
        'ViewType', ...
        'SampleRate',  ...
        'Method', ...
        'PlotAsTwoSidedSpectrum', ...
        'FrequencyScale'};
    
    advancedProps =  {'FrequencySpan', ...
        'Span', ...
        'CenterFrequency', ...
        'StartFrequency', ...
        'StopFrequency', ...
        'FrequencyResolutionMethod',...
        'RBWSource', ...
        'RBW', ...
        'WindowLength', ...
        'FFTLengthSource', ...
        'FFTLength',...
        'NumTapsPerBand', ...
        'FrequencyVectorSource', ...
        'FrequencyVector', ...
        'InputUnits',  ...
        'OverlapPercent', ...
        'Window', ...
        'CustomWindow', ...
        'SidelobeAttenuation', ...
        'SpectrumUnits', ...
        'FullScaleSource', ...
        'FullScale'...
        'AveragingMethod', ...
        'SpectralAverages', ...
        'ForgettingFactor', ...
        'ReferenceLoad', ...
        'FrequencyOffset'};
    
    spectrogramProps = {'SpectrogramChannel', ...
        'TimeResolutionSource', ...
        'TimeResolution', ...
        'TimeSpanSource', ...
        'TimeSpan'};
    
    measurementsProps = {'MeasurementChannel', ...
        'SpectralMask', ...
        'PeakFinder', ...
        'CursorMeasurements', ...
        'ChannelMeasurements', ...
        'DistortionMeasurements'};
        
    visualizationProps = {'Name', ...
        'Position', ...
        'PlotType', ...
        'PlotNormalTrace', ...
        'PlotMaxHoldTrace', ...
        'PlotMinHoldTrace', ...
        'ReducePlotRate', ...
        'Title', ...
        'YLabel', ...
        'ShowLegend', ...
        'ChannelNames', ...
        'ShowGrid', ...
        'YLimits', ...
        'ColorLimits', ...
        'AxesScaling', ...
        'AxesLayout'};
    
    advancedGroupTitle = getString(message('dspshared:SpectrumAnalyzer:AdvancedProperties'));
    spectrogramGroupTitle = getString(message('dspshared:SpectrumAnalyzer:SpectrogramProperties'));
    measurementsGroupTitle = getString(message('dspshared:SpectrumAnalyzer:MeasurementsProperties'));
    visualizationGroupTitle = getString(message('dsp:system:Shared:VisualizationProperties'));
    
    mainGroup = matlab.system.display.SectionGroup('TitleSource', ...
      'Auto', 'PropertyList', mainProps);
    
    advancedGroup = matlab.system.display.SectionGroup('Title', ...
      advancedGroupTitle, 'PropertyList', advancedProps);
  
    spectrogramGroup = matlab.system.display.SectionGroup('Title', ...
      spectrogramGroupTitle, 'PropertyList', spectrogramProps);
        
    measurementsGroup = matlab.system.display.SectionGroup('Title', ...
      measurementsGroupTitle, 'PropertyList', measurementsProps);
    
    visualizationGroup = matlab.system.display.SectionGroup('Title', ...
      visualizationGroupTitle, 'PropertyList', visualizationProps);
    
    groups = [mainGroup advancedGroup spectrogramGroup measurementsGroup visualizationGroup];
  end
end
end

%-------------------------------------------------------------------------
% Helper functions
%-------------------------------------------------------------------------
function validateSpan(obj,Fstart,Fstop, RBW, rbwSrc)

if nargin < 4
  RBW = obj.RBW;
end

if nargin < 5
  rbwSrc = obj.RBWSource;
end

if Fstart >= Fstop
  sendError(obj,'dspshared:SpectrumAnalyzer:FstartGreaterThanFstop')
end

FO = obj.FrequencyOffset;
NyquistRange = [(-obj.SampleRate/2)*obj.PlotAsTwoSidedSpectrum obj.SampleRate/2] + [min(FO), max(FO)];
if (Fstart < NyquistRange(1)) || (Fstop > NyquistRange(2))
  % When setting invalid frequency span values explicitly, always error out
  [NyquistRange, ~, unitsNyquistRange] = engunits(NyquistRange);
  [spanRange, ~, unitsSpanRange] = engunits([Fstart, Fstop]);
  switch obj.FrequencySpan
    case 'Span and center frequency'
      sendError(obj,'dspshared:SpectrumAnalyzer:InvalidSpanAndCenterFrequency',...
        num2str(spanRange(1)),num2str(spanRange(2)),unitsSpanRange, ...
        num2str(NyquistRange(1)),num2str(NyquistRange(2)),unitsNyquistRange);
    case 'Start and stop frequencies'
      sendError(obj,'dspshared:SpectrumAnalyzer:InvalidStartAndStopFrequencies',...
        num2str(spanRange(1)),num2str(spanRange(2)),unitsSpanRange, ...
        num2str(NyquistRange(1)),num2str(NyquistRange(2)),unitsNyquistRange);
  end
end

% Span/RBW must be greater than 2
if  strcmp(rbwSrc,'Property')
  span = Fstop - Fstart;
  if (span/RBW) <= 2
    sendError(obj,'dspshared:SpectrumAnalyzer:InvalidSpanRBW')
  end
end
end

function updateNumInputPorts(obj)
obj.NumInputPorts = obj.pSource.NumInputPorts;
end

function validateCustomWindow(obj,value)
switch value
    case {'blackman','blackmanharris','flattopwin','hamming','hann','nuttallwin'}
        % try evaluating the function with 'periodic' attribute. This is
        % mainly done to make sure that if the user is using window from
        % the signal processing toolbox, it is periodic.
        try
            feval(value,100,'periodic');
        catch
            % if an exception is caught that means the user has a
            % function with the same name on the MATLAB path, and the
            % function does not take periodic attribute. We should never
            % let the user overload the existing signal processing
            % win functions, because in the spectral estimator class we
            % call them also with the periodic attribute, so if their
            % function does not take periodic attribute we will get a run time error.
            sendError(obj,'dspshared:SpectrumAnalyzer:InvalidCustomWindowShadowing',value)
        end
    otherwise
        try
            % Evaluate the custom window function and check if the return
            % type is double and if the number of elements returned is
            % equal to 100
            feval_value = feval(value,100);
            if ~isa(feval_value,'double') && numel(feval_value)~= 100
                sendError(obj,'dspshared:SpectrumAnalyzer:InvalidCustomWindowName',value)
            end
            
            if numel(feval_value)~= 100 || isrow(feval_value)
                sendError(obj,'dspshared:SpectrumAnalyzer:InvalidCustomWindowDimensions',value)
            end
        catch ME
            switch ME.identifier
                case 'MATLAB:UndefinedFunction'
                    % The custom function is not on the MATLAB path
                    sendError(obj,'dspshared:SpectrumAnalyzer:UndefinedCustomWindowName',value);
                case {'MATLAB:minrhs','MATLAB:TooManyInputs'}
                    % The custom function is either called with the
                    % wrong number of arguments
                    sendError(obj,'dspshared:SpectrumAnalyzer:InvalidCustomWindowNotEnoughArgs',value);
                case 'dspshared:SpectrumAnalyzer:InvalidCustomWindowDimensions'
                    % The custom function returns a row vector
                    % instead of a column vector, or a vector with
                    % wrong number of elements
                    sendError(obj,'dspshared:SpectrumAnalyzer:InvalidCustomWindowDimensions',value)
                otherwise
                    % the custom window function is a matlab function
                    % that does not take any input
                    sendError(obj,'dspshared:SpectrumAnalyzer:InvalidCustomWindowName',value);
            end
        end
end
end
