classdef SpectrumAnalyzerBase< matlabshared.scopes.mixin.CommandLineAutoscale & matlabshared.scopes.UnifiedSystemScope & dynamicprops
%SpectrumAnalyzerBase Display frequency-domain signals and the frequency
%spectrum of time-domain signals

 
%   Copyright 2011-2019 The MathWorks, Inc.

    methods
        function out=SpectrumAnalyzerBase
            %SpectrumAnalyzer   Construct the SpectrumAnalyzer class.
        end

        function out=generateScript(~) %#ok<STOUT>
            % generateScript(obj) Generate MATLAB(R) script to construct
            % dsp.SpectrumAnalyzer with current settings.
        end

        function out=getMeasurementsData(~) %#ok<STOUT>
            %getMeasurementsData Get the current measurement data displayed on
            %the Spectrum Analyzer
            %   DATA = getMeasurementsData(scope) returns data table about the
            %   current spectrum analyzer measurements in use.
            %
            %   DATA = getMeasurementsData(scope, 'all') returns data table about
            %   all spectrum analyzer measurements for the current time step.
            %
            %   DATA contains the following fields:
            %   SimulationTime           - Simulation time
            %   PeakFinder               - Peak finder data
            %   CursorMeasurements       - Cursor measurements data
            %   ChannelMeasurements      - Channel measurements data
            %   DistortionMeasurements   - Distortion measurements data
            %
            %   Example: Get Peak finder data
            %       sine = dsp.SineWave('Frequency',[98 100],'SampleRate',1000);
            %       sine.SamplesPerFrame = 1024;
            %       scope = dsp.SpectrumAnalyzer('SampleRate',sine.SampleRate, ...
            %         'PlotAsTwoSidedSpectrum',false,'ShowLegend',true);
            %
            %       % Enable peak finder measurement
            %       scope.PeakFinder.Enable = true;
            %
            %       % Set peak finder NumPeaks property
            %       scope.PeakFinder.NumPeaks = 4;
            %       scope(sine() + 0.05*randn(1024,2));
            %       data = getMeasurementsData(scope);
            %
            %       release(scope);
        end

        function out=getNumInputsImpl(~) %#ok<STOUT>
        end

        function out=getPropertyGroupsImpl(~) %#ok<STOUT>
            % Only the properties listed in mainProps are displayed in the
            % command-line by default
        end

        function out=getScopeCfg(~) %#ok<STOUT>
        end

        function out=getSpectralMaskStatus(~) %#ok<STOUT>
            %getSpectralMaskStatus Get the current results of spectral mask testing
            %   S = getSpectralMaskStatus(specanlyz) returns a structure S whose
            %   fields contain information about the current results of spectral
            %   mask testing. The structure S contains the following fields:
            %   IsCurrentlyPassing  - Whether the mask test is currently passing.
            %   NumTotalTests       - Total number of spectral mask tests in the
            %                         current simulation run. A test is performed
            %                         for every spectral update.
            %   NumPassedTests      - Number of passed tests in the current
            %                         simulation.
            %   SuccessRate         - Percentage success rate of spectral mask
            %                         testing.
            %   FailingMasks        - Currently failing masks.
            %   FailingChannels     - Currently failing input channels.
            %   SimulationTime      - Simulation time when this statistics was
            %                         generated.
        end

        function out=getSpectrumData(~) %#ok<STOUT>
            %getSpectrumData Get the current spectrum data displayed on the
            %Spectrum Analyzer
            %   DATA = getSpectrumData(scope) returns DATA table about the current
            %   spectrum or spectrogram displayed on the Spectrum Analyzer. 
            %
            %   DATA = getSpectrumData(scope, 'all') returns a DATA table with all
            %   spectrum information for the current time step.
            %
            %   DATA table contains the following fields:
            %   SimulationTime        - Simulation time 
            %   Spectrum              - Current spectrum data 
            %   Spectrogram           - Current spectrogram data
            %   MinHoldTrace          - Minimum-hold trace 
            %   MaxHoldTrace          - Maximum-hold trace 
            %   FrequencyVector       - Frequency vector
            %
            %   Example: Get the spectrum data displayed on the spectrum analyzer.
            %       sine = dsp.SineWave('Frequency',100, 'SampleRate', 1000);
            %       sine.SamplesPerFrame = 1000;
            %       scope = dsp.SpectrumAnalyzer('SampleRate', swv.SampleRate,...
            %                                    'ReducePlotRate',false);
            %       x = sine() + 0.05*randn(1000,1);
            %       san(x);
            %       data = getSpectrumData(scope);
            %       release(scope);
        end

        function out=isInactivePropertyImpl(~) %#ok<STOUT>
        end

        function out=isInputSizeMutableImpl(~) %#ok<STOUT>
        end

        function out=isNewDataReady(~) %#ok<STOUT>
            %isNewDataReady Return whether or not the Spectrum Analyzer is
            %displaying new spectrum data
            %   FLAG = isNewDataReady(scope) returns a logical to indicate if the
            %   Spectrum Analyzer plotted new spectrum data
        end

        function out=loadObjectImpl(~) %#ok<STOUT>
        end

        function out=releaseImpl(~) %#ok<STOUT>
            % Flush buffer only if framework has not been cleared
        end

        function out=resetImpl(~) %#ok<STOUT>
        end

        function out=saveObjectImpl(~) %#ok<STOUT>
        end

        function out=setupImpl(~) %#ok<STOUT>
        end

        function out=updateImpl(~) %#ok<STOUT>
            % If the scope does not exist, create one and post new data
        end

        function out=validateInputsImpl(~) %#ok<STOUT>
            % All inputs must have same frame size
        end

        function out=validatePropertiesImpl(~) %#ok<STOUT>
            % Verify correct span
        end

    end
    properties
        % AveragingMethod Averaging method 
        %   Specify the averaging method as 'Running' or 'Exponential'. In the
        %   running averaging method, the object computes an equally weighted
        %   average of specified number of spectrum estimates defined by
        %   'SpectralAverages' property. In the exponential method, the object
        %   computes the average over samples weighted by an exponentially
        %   decaying forgetting factor. The default is 'Running'. This property
        %   is tunable.
        AveragingMethod;

        %AxesLayout Choose the axes layout
        %   Specify the layout type as one of 'Horizontal' | 'Vertical'. This
        %   property applies when the ViewType is set to 'Spectrum and
        %   spectrogram'. The default is 'Vertical'. This property is tunable.
        AxesLayout;

        %AxesScaling Axes scaling mode
        %   Specify the scaling mode for the axes as one of 'Auto' | 'Manual' |
        %   'Updates' | 'OnceAtStop'.  When this property is set to 'Auto', the
        %   scope attempts to keep the data in the display while minimizing the
        %   number of axes limit updates.  When this property is set to 'Manual',
        %   the scope takes no action unless specified by the user.  When this
        %   property is set to 'Updates', the scope updates the axes limits after
        %   a set number of visual updates.  When this property is set to
        %   'OnceAtStop', the limits will be updated once at the end of the
        %   simulation (when release is called).  The default is 'Auto'. This
        %   property is tunable.
        AxesScaling;

        %CenterFrequency Frequency over which span is centered
        %   Specify the center frequency of the span over which the Spectrum
        %   Analyzer computes and plots the spectrum. CenterFrequency must be a
        %   real scalar in Hertz. This property applies when you set the
        %   FrequencySpan property to 'Span and center frequency'. The overall
        %   span defined by the Span and CenterFrequency properties must fall
        %   within the full span frequency range which equals
        %   [-(SampleRate/2)+min(FrequencyOffset),
        %   (SampleRate/2)+max(FrequencyOffset)] Hz when the
        %   PlotAsTwoSidedSpectrum property is set to true, and
        %   [min(FrequencyOffset), (SampleRate/2)+max(FrequencyOffset)] Hz when
        %   the PlotAsTwoSidedSpectrum property is set to false. The default is 0
        %   Hz. This property is tunable.
        %
        %   See also: dsp.SpectrumAnalyzer.Span,
        %             dsp.SpectrumAnalyzer.PlotAsTwoSidedSpectrum  
        CenterFrequency;

        %ChannelMeasurements Channel measurements specification
        %    Enable channel measurements to compute and display the occupied
        %    bandwidth or adjacent channel power ratio. The ChannelMeasurements
        %    property uses the ChannelMeasurementsSpecification properties:
        %    Algorithm, FrequencySpan, Span, CenterFrequency, StartFrequency,
        %    StopFrequency, PercentOccupiedBW, NumOffsets, AdjacentBW,
        %    FilterShape, FilterCoeff, ACPROffsets and Enable. See <a href="matlab:helpview(fullfile(docroot,'dsp','dsp.map'),'dsp.SpectrumAnalyzer.ChannelMeasurements')">ChannelMeasurements</a>
        %    documentation for more details. The CursorMeasurements property is
        %    tunable.
        %
        %    % Example:
        %     sine = dsp.SineWave('Frequency',[98 100],'SampleRate',1000);
        %     sine.SamplesPerFrame = 1024;
        %     scope = dsp.SpectrumAnalyzer('SampleRate',sine.SampleRate, ...
        %         'PlotAsTwoSidedSpectrum',false,'ShowLegend',true);
        %
        %    % Enable channel measurements
        %    scope.ChannelMeasurements.Enable = true;
        %
        %    % Set Algorithm property
        %    scope.ChannelMeasurements.Algorithm = 'ACPR';
        %
        %    for i=1:100
        %        scope(sine() + 0.05*randn(1024,2));
        %    end
        %
        %    release(scope);
        ChannelMeasurements;

        %ChannelNames Names for the input data channels
        %   Set this property to a cell array of strings to represent the input
        %   channels. These strings are used to label the data channels in the
        %   legend, in the Style dialog and in various measurement tools. The
        %   default is an empty cell array. When this property is an empty cell
        %   array, the channels will be named by default as Channel 1, Channel 2
        %   and so on. This property is tunable.
        ChannelNames;

        %ColorLimits Color limits
        %   Specify the color limits as a two-element numeric vector: [cmin
        %   cmax]. This property applies when the ViewType property is set to
        %   'Spectrogram' or 'Spectrum and spectrogram'. This property is tunable.
        ColorLimits;

        %CursorMeasurementsSpecification Cursor measurements specification
        %    Enable cursor measurements to display screen or waveform cursors.
        %    The CursorMeasurements property uses the
        %    CursorMeasurementsSpecification properties: Type,
        %    ShowHorizontal, ShowVertical, Cursor1TraceSource,
        %    Cursor2TraceSource, LockSpacing, SnapToData, XLocation,
        %    YLocation and Enable. See <a href="matlab:helpview(fullfile(docroot,'dsp','dsp.map'),'dsp.SpectrumAnalyzer.CursorMeasurements')">CursorMeasurements</a>
        %    documentation for more details. The CursorMeasurements property is
        %    tunable.
        %
        %    % Example:
        %     sine = dsp.SineWave('Frequency',[98 100],'SampleRate',1000);
        %     sine.SamplesPerFrame = 1024;
        %     scope = dsp.SpectrumAnalyzer('SampleRate',sine.SampleRate, ...
        %         'PlotAsTwoSidedSpectrum',false,'ShowLegend',true);
        %
        %    % Enable cursor measurements
        %    scope.CursorMeasurements.Enable = true;
        %
        %    % Set CursorType property
        %    scope.CursorMeasurements.CursorType = 'Screen cursors';
        %
        %    for i=1:100
        %        scope(sine() + 0.05*randn(1024,2));
        %    end
        %
        %    release(scope);
        CursorMeasurements;

        %CustomWindow
        %   Specify a custom window function name as an array of characters. The
        %   window function must be on the path and must have exactly one input
        %   and one output. This property applies when you set the Window
        %   property to 'Custom'.  The default is 'hann'. This property is
        %   tunable.
        CustomWindow;

        %DistortionMeasurements Distortion measurements specification
        %    Enable distortion measurements to compute and display the harmonic
        %    distortion and intermodulation distortion. The
        %    DistortionMeasurements property uses the
        %    DistortionMeasurementsSpecification properties: Algorithm,
        %    NumHarmonics and Enable. See <a href="matlab:helpview(fullfile(docroot,'dsp','dsp.map'),'dsp.SpectrumAnalyzer.DistortionMeasurements')">DistortionMeasurements</a>
        %    documentation for more details. The DistortionMeasurements property
        %    is tunable.
        %
        %    % Example:
        %     sine = dsp.SineWave('Frequency',[98 100],'SampleRate',1000);
        %     sine.SamplesPerFrame = 1024;
        %     scope = dsp.SpectrumAnalyzer('SampleRate',sine.SampleRate, ...
        %         'PlotAsTwoSidedSpectrum',false,'ShowLegend',true);
        %
        %    % Enable distortion measurements
        %    scope.DistortionMeasurements.Enable = true;
        %
        %    % Set NumHarmonics property
        %    scope.DistortionMeasurements.NumHarmonics = 4;
        %
        %    for i=1:100
        %        scope(sine() + 0.05*randn(1024,2));
        %    end
        %
        %    release(scope);
        DistortionMeasurements;

        %FFTLength FFT length
        %   Specify the length of the FFT that the Spectrum Analyzer uses to
        %   compute spectral estimates as a positive, integer scalar. This
        %   property applies when you set the FrequencyResolutionMethod property
        %   to 'WindowLength' or 'NumFrequencyBands' and the FFTLengthSource
        %   property to 'Property'. The default is 1024. FFTLength must be equal
        %   to or larger than WindowLength. This property is tunable.
        FFTLength;

        %FFTLengthSource Source of the FFT length value
        %   Specify the source of the FFT length value as one of 'Auto' |
        %   'Property'. The default is 'Auto'. This property applies when you set
        %   the FrequencyResolutionMethod property to 'WindowLength'. When you
        %   set this property to 'Auto' the Spectrum Analyzer sets the FFT length
        %   to max(1024,WindowLength). When you set this property to 'Property'
        %   then you specify the number of FFT points using the FFTLength
        %   property. The number of FFT points must be equal to or larger than
        %   the value specified in the WindowLength property. This property is
        %   tunable.
        %
        %   See also: dsp.SpectrumAnalyzer.FFTLength
        FFTLengthSource;

        %ForgettingFactor Forgetting factor
        %   Specify the exponential weighting forgetting factor as a scalar value
        %   greater than zero and smaller than or equal to one. This property
        %   applies when 'AveragingMethod' is equal to 'Exponential'. The default
        %   is 0.9. This property is tunable.
        ForgettingFactor;

        %FrequencyOffset Frequency offset
        %   Specify the frequency offset to apply to the frequency axis (x-axis)
        %   in units of Hertz. This property can either be set to a numeric
        %   scalar, or a vector of length equal to the number of input channels.
        %   If this property is set to a scalar, the same frequency offset is
        %   used for all channels. If this property is set to a vector, each
        %   element is used as the frequency offset for the corresponding
        %   channel.The overall span must fall within the full span frequency
        %   range. The full span frequency range is the union of the valid
        %   Nyquist frequency intervals for all channels after applying frequency
        %   offsets. When the PlotAsTwoSidedSpectrum property is set to true, the
        %   full span frequency range equals [-(SampleRate/2) +
        %   min(FrequencyOffset), (SampleRate/2) + max(FrequencyOffset)] Hz. When
        %   the PlotAsTwoSidedSpectrum property is set to false, the full span
        %   frequency range equals [min(FrequencyOffset), (SampleRate/2) +
        %   max(FrequencyOffset)] Hz. The default is 0 Hz. This property is
        %   tunable.
        FrequencyOffset;

        %FrequencyResolutionMethod Control resolution using RBW, window length or
        %number of frequency bands
        %   Specify how you want to control the frequency resolution of the
        %   spectrum analyzer as one of 'RBW' | 'WindowLength' |
        %   'NumFrequencyBands'. The default is 'RBW'. When you set the property
        %   to 'RBW' you control the frequency resolution of the analyzer
        %   directly in Hertz using the RBWSource and RBW properties. When you
        %   set the property to 'WindowLength' you control the frequency
        %   resolution of the analyzer indirectly using the WindowLength
        %   property. When you set the property to 'NumFrequencyBands' you
        %   control the frequency resolution of the analyzer indirectly using the
        %   FFTLengthSource and FFTLength properties. 'NumFrequencyBands' applies
        %   when the Method property is set to 'Filter bank'. 'WindowLength'
        %   applies when the Method property is set to 'Welch'.The frequency
        %   resolution of the analyzer is given as NENBW*Fs/WindowLength where
        %   NENBW is the normalized effective noise bandwidth of the window
        %   currently specified in the Window property. This property is tunable.
        FrequencyResolutionMethod;

        %FrequencyScale Frequency scale
        %   Specify the frequency scale as one of 'Linear' or 'Log'. You cannot
        %   set the frequency scale to 'Log' when the PlotAsTwoSidedSpectrum
        %   property is set to true. The default is 'Linear'. This property is
        %   tunable.
        FrequencyScale;

        %FrequencySpan Frequency span mode
        %   Specify the frequency span mode as one of 'Full' | 'Span and center
        %   frequency' | 'Start and stop frequencies'. The default is 'Full'.
        %   When you set this property to 'Full' and if the
        %   PlotAsTwoSidedSpectrum property is set to true, the Spectrum Analyzer
        %   computes the spectrum over the entire Nyquist range which equals
        %   [-SampleRate/2, SampleRate/2] Hz  and plots in the frequency range
        %   [-(SampleRate/2)+min(FrequencyOffset),
        %   (SampleRate/2)+max(FrequencyOffset)] Hz. When you set this property
        %   to 'Full' and if the PlotAsTwoSidedSpectrum property is set to false,
        %   the Spectrum Analyzer computes the spectrum over the one-sided
        %   Nyquist range which equals [0, SampleRate/2] Hz and plots in the
        %   frequency range [min(FrequencyOffset),
        %   (SampleRate/2)+max(FrequencyOffset)] Hz. When you set this property
        %   to 'Span and center frequency', the Spectrum Analyzer computes and
        %   plots the spectrum over the frequency interval specified by the Span
        %   and CenterFrequency properties. When you set this property to 'Start
        %   and stop frequencies' the Spectrum Analyzer computes and plots the
        %   spectrum over the frequency interval specified by the StartFrequency
        %   and StopFrequency properties. This property is tunable.
        %
        %   See also: dsp.SpectrumAnalyzer.Span,
        %             dsp.SpectrumAnalyzer.CenterFrequency,
        %             dsp.SpectrumAnalyzer.StartFrequency,
        %             dsp.SpectrumAnalyzer.StopFrequency.
        FrequencySpan;

        % FrequencyVector Input frequency vector
        %   Specify the input frequency vector as a real, positive, sorted,
        %   column vector in Hertz. The frequency vector must have the same
        %   length as the input signal frame size. This property applies when you
        %   set the InputDomain property to 'Frequency' and the
        %   FrequencyVectorSource property to 'Property'. The default is [-5000
        %   5000] Hz. This property is tunable.
        FrequencyVector;

        % FrequencyVectorSource Source of frequency vector value
        %   Specify the source of the frequency vector as one of 'Auto' |
        %   'Property'. The default is 'Auto'. This property applies when
        %   InputDomain is set to 'Frequency'. When you set this property to
        %   'Auto', the Spectrum Analyzer computes the frequency vector based on
        %   the input signal frame size and the specified sampling rate. When
        %   you set this property to 'Property', then you specify the frequency
        %   vector directly on the FrequencyVector property. This property is
        %   tunable.
        FrequencyVectorSource;

        %FullScale Full scale value
        %   Specify the full scale value as a positive, scalar integer. The
        %   Property applies when you set the SpectrumUnits property to 'dBFS'.
        %   This property is tunable.
        FullScale;

        %FullScaleSource Source of full scale value
        %   Specify the source for the full scale of the input signal as one of
        %   'Auto' | 'Property'. The default is 'Auto'. This property applies
        %   when the SpectrumUnits property is set to 'dBFS'. When set to 'Auto',
        %   the full scale is computed based on the input data. When set to
        %   'Property', you specify the full scale of the input signal using the
        %   'FullScale' property. This property is tunable.
        FullScaleSource;

        %InputDomain Input signal domain
        %   Specify the input signal domain as one of 'Time' | 'Frequency'. The
        %   default is 'Time'.
        InputDomain;

        %InputUnits Input signal units
        %   Specify the input signal units as one of 'dBm' | 'dBW' | 'Watts' |
        %   'Vrms' | 'dBV'. This property applies when InputDomain is set to
        %   'Frequency'. The default is 'dBm'.
        InputUnits;

        %MeasurementChannel Channel for which measurement will be performed
        %   Specify the measurement channel number as a real, positive scalar
        %   integer. The default is 1. This property is tunable.
        MeasurementChannel;

        %Method Spectrum estimation method
        %   Specify the spectrum estimation method as one of 'Welch' |
        %   'Filter bank'. The filter bank method provides better resolution,
        %   lower spectral leakage, a more accurate noise floor and more
        %   precise peaks at the expense of increased computation and slower
        %   tracking. The default is 'Welch'.
        Method;

        %Name Caption to display on Spectrum Analyzer window
        %   Specify the caption to display on the Spectrum Analyzer window as any
        %   string. The default is 'Spectrum Analyzer'. This property is tunable.
        Name;

        %NumInputPorts Number of input signals
        %   Specify the number of input signals to the Spectrum Analyzer as a
        %   positive integer. You must invoke the step method with the same
        %   number of inputs as the value of this property. The default is 1.
        NumInputPorts;

        %NumTapsPerBand Number of filter taps per frequency band
        %   Specify the number of filter coefficients or taps for each
        %   frequency band. This also corresponds to the number of filter
        %   coefficients per polyphase branch. The total number of filter
        %   coefficients is given by NumTapsPerBand times FFTLength. This
        %   property applies when you set Method to 'Filter bank'. The
        %   default is 12. This property is tunable.
        NumTapsPerBand;

        %OverlapPercent Overlap percentage
        %   Specify the percentage overlap between the current and previous
        %   buffered window segments as a real, scalar value, greater than or
        %   equal to zero and less than 100. The default is 0. This property is
        %   tunable.
        OverlapPercent;

        %PeakFinder Peak finder specification
        %   Enable peak finder to compute and display the largest calculated peak
        %   values. The PeakFinder property uses the PeakFinderSpecification
        %   properties: MinHeight, NumPeaks, MinDistance, Threshold,
        %   LabelFormat and Enable. See <a
        %   href="matlab:helpview(fullfile(docroot,'dsp','dsp.map'),'dsp.SpectrumAnalyzer.PeakFinder')">PeakFinder</a>
        %   documentation for more details. The PeakFinder property is tunable.
        %
        %   % Example:
        %     sine = dsp.SineWave('Frequency',[98 100],'SampleRate',1000);
        %     sine.SamplesPerFrame = 1024;
        %     scope = dsp.SpectrumAnalyzer('SampleRate',sine.SampleRate, ...
        %         'PlotAsTwoSidedSpectrum',false,'ShowLegend',true);
        %
        %    % Enable peak finder measurement
        %    scope.PeakFinder.Enable = true;
        %
        %    % Set peak finder NumPeaks property
        %    scope.PeakFinder.NumPeaks = 4;
        %
        %    for i=1:100
        %        scope(sine() + 0.05*randn(1024,2));
        %    end
        %
        %    release(scope);  
        PeakFinder;

        %PlotAsTwoSidedSpectrum Two sided spectrum flag
        %   Set this property to true so that the Spectrum Analyzer computes two
        %   sided spectral estimates. Set this property to false to obtain one
        %   sided spectral estimates. In this case, the input must be real. The
        %   default is true. 
        PlotAsTwoSidedSpectrum;

        %PlotMaxHoldTrace Max-hold trace flag
        %   Set this property to true so that the Spectrum Analyzer computes and
        %   plots the max-hold spectrum of each input channel. This property
        %   applies when the ViewType property is set to 'Spectrum' or 'Spectrum
        %   and spectrogram'. The max-hold spectrum is computed by keeping, at
        %   each frequency bin, the maximum value of all the power spectrum
        %   estimates. Toggling the PlotMaxHoldTrace control resets the max-hold
        %   computations. The default is false. This property is tunable.
        %
        %   See also: dsp.SpectrumAnalyzer.PlotMinHoldTrace,
        %             dsp.SpectrumAnalyzer.PlotNormalTrace  
        PlotMaxHoldTrace;

        %PlotMinHoldTrace Min-hold trace flag
        %   Set this property to true so that the Spectrum Analyzer computes and
        %   plots the min-hold spectrum of each input channel. This property
        %   applies when the ViewType property is set to 'Spectrum' or 'Spectrum
        %   and spectrogram'. The min-hold spectrum is computed by keeping, at
        %   each frequency bin, the minimum value of all the power spectrum
        %   estimates. Toggling the PlotMinHoldTrace control resets the min-hold
        %   computations. The default is false. This property is tunable.
        %
        %   See also: dsp.SpectrumAnalyzer.PlotMaxHoldTrace,
        %             dsp.SpectrumAnalyzer.PlotNormalTrace    
        PlotMinHoldTrace;

        %PlotNormalTrace Normal trace flag
        %   Set this property to false to remove the normal traces (i.e. the
        %   traces displaying the free running spectral estimates) from the
        %   Spectrum Analyzer display. Note that even though the traces are
        %   removed from the display, the spectral computations continue. This
        %   property applies when the ViewType property is set to 'Spectrum' or
        %   'Spectrum and spectrogram'. The default is true. This property is
        %   tunable.
        %
        %   See also: dsp.SpectrumAnalyzer.PlotMaxHoldTrace,
        %             dsp.SpectrumAnalyzer.PlotMinHoldTrace      
        PlotNormalTrace;

        %PlotType Choose the plot type
        %   Specify the type of plot to be used for the normal traces (i.e. the
        %   traces displaying the free running spectral estimates). Valid types
        %   are 'Line' and 'Stem'. This property applies when the ViewType
        %   property is set to 'Spectrum' or 'Spectrum and spectrogram', and the
        %   PlotNormalTrace property is set to true. The default is 'Line'. This
        %   property is tunable.
        PlotType;

        %RBW Resolution bandwidth
        %   Specify the resolution bandwidth (RBW) of the Spectrum Analyzer as a
        %   real, positive, scalar in Hertz. This property applies when you set
        %   the FrequencyResolutionMethod property to 'RBW' and the RBWSource
        %   property to 'Property'. The ratio of the specified span and RBW
        %   values must be at least two (i.e. there must be at least two RBW
        %   intervals over the specified frequency span). The default is 9.76 Hz.
        %   This property is tunable.
        RBW;

        %RBWSource Source of resolution bandwidth value
        %   Specify the source of the resolution bandwidth (RBW) as one of 'Auto'
        %   | 'Property'. The default is 'Auto'. This property applies when
        %   FrequencyResolutionMethod is set to 'RBW'. If the InputDomain is set
        %   to 'Time' and you set this property to 'Auto', the Spectrum Analyzer
        %   adjusts the spectral estimation resolution to ensure that there are
        %   1024 RBW intervals over the defined frequency span. If the
        %   InputDomain is set to 'Frequency' and you set this property to
        %   'Auto', the Spectrum Analyzer computes the resolution bandwidth as
        %   the minimum frequency separation in the frequency vector. If you set
        %   this property to 'Property', then you specify the resolution
        %   bandwidth directly on the RBW property. This property is tunable.
        %
        %   See also: dsp.SpectrumAnalyzer.RBW
        RBWSource;

        %ReducePlotRate Reduce plot rate to improve performance
        %   Set this property to false to update the plot after each PSD
        %   computation when the ViewType property is set to 'Spectrum' or after
        %   each spectrogram line computation when the ViewType property is set
        %   to 'Spectrogram' or 'Spectrum and spectrogram'. Setting this
        %   property to false negatively impacts performance. The default is
        %   true. This property is tunable.
        ReducePlotRate;

        %ReferenceLoad Reference load
        %   Specify the load that the Spectrum Analyzer uses as a reference to
        %   compute power values as a real, positive scalar in Ohms. This
        %   property applies when the SpectrumType property is set to 'Power' or
        %   'Power density'. The default is 1 Ohm. This property is tunable.
        ReferenceLoad;

        %SampleRate Sample rate of input
        %   Specify the sampling rate of the input in Hertz as a finite numeric
        %   scalar. The default is 10e3.
        SampleRate;

        %ShowGrid Show or hide grid
        %   Set this property to true to turn on the grid. The default is true.
        %   This property is tunable.
        ShowGrid;

        %ShowLegend Show or hide legend
        %   When this property is set to false, no legend is displayed. When this
        %   property is set to true, a legend with automatic string labels for
        %   each input channel is displayed. You can use the legend to toggle
        %   signal visibility in the scope. In the legend, click on a signal name
        %   to hide the signal in the scope. To show the signal, click on the
        %   signal name again. To show only one signal, right-click the signal
        %   name. This property applies when the ViewType property is set to
        %   'Spectrum' or 'Spectrum and spectrogram'. The default is false. This
        %   property is tunable.
        ShowLegend;

        %SidelobeAttenuation Sidelobe attenuation of window
        %   Specify the sidelobe attenuation of the window as a real, positive
        %   scalar in decibels (dB). This property must be greater than or equal 
        %   to 45 dB. This property applies when you set the Window property to 
        %   'Chebyshev' or 'Kaiser'. The default is 60 dB. This property is 
        %   tunable.
        SidelobeAttenuation;

        %Span Frequency span over which spectrum is computed
        %   Specify the frequency span over which the Spectrum Analyzer computes
        %   and plots the spectrum as a real, positive scalar in Hertz. This
        %   property applies when you set the FrequencySpan property to 'Span and
        %   center frequency'. The overall span defined by the Span and
        %   CenterFrequency properties must fall within the full span frequency
        %   range which equals [-(SampleRate/2)+min(FrequencyOffset),
        %   (SampleRate/2)+max(FrequencyOffset)] Hz when the
        %   PlotAsTwoSidedSpectrum property is set to true, and
        %   [min(FrequencyOffset), (SampleRate/2)+max(FrequencyOffset)] Hz when
        %   the PlotAsTwoSidedSpectrum property is set to false. The default is
        %   10 kHz. This property is tunable.
        %
        %   See also: dsp.SpectrumAnalyzer.CenterFrequency,
        %             dsp.SpectrumAnalyzer.PlotAsTwoSidedSpectrum
        Span;

        %SpectralAverages Number of spectral averages
        %   Specify the number of spectral averages as a positive, integer
        %   scalar. The Spectrum Analyzer computes the current power spectrum
        %   estimate by averaging the last N power spectrum estimates, where N is
        %   the number of spectral averages defined in the SpectralAverages
        %   property. This property applies when the ViewType property is set to
        %   'Spectrum'. The default is 1. This property is tunable.
        SpectralAverages;

        %SpectralMask Spectral mask specification
        %   Enable and specify the spectral mask to draw upper and/or lower limit
        %   mask lines on the Power and Power Density plots. The SpectralMask
        %   property uses the SpectralMaskSpecification properties: EnabledMasks,
        %   UpperMask, LowerMask, ReferenceLevel, CustomReferenceLevel,
        %   MaskFrequencyOffset and SelectedChannel. See <a href="matlab:helpview(fullfile(docroot,'dsp','dsp.map'),'dsp.SpectrumAnalyzer.SpectralMask')">SpectralMask</a>
        %   documentation for more details. The SpectralMask property is tunable.
        %
        %   % Example:
        %     sine = dsp.SineWave('Frequency',[98 100],'SampleRate',1000);
        %     sine.SamplesPerFrame = 1024;
        %     scope = dsp.SpectrumAnalyzer('SampleRate',sine.SampleRate, ...
        %         'PlotAsTwoSidedSpectrum',false,'ShowLegend',true);
        %     for i=1:100
        %         scope(sine() + 0.05*randn(1024,2));
        %     end
        %     % Set SpectralMask properties
        %     upperMask = [0 -10; 90 -10; 90 40; 110 40; 110 -10; 500 -10];
        %     scope.SpectralMask.EnabledMasks = 'Upper and lower';
        %     set(scope.SpectralMask,'UpperMask',upperMask,'LowerMask',-60);
        %     release(scope);
        SpectralMask;

        %SpectrogramChannel Channel for which spectrogram settings apply
        %   Specify the channel number as a real, positive scalar integer. This
        %   property applies when you set the ViewType property to 'Spectrogram'
        %   or 'Spectrum and spectrogram'. The default is 1. This property is
        %   tunable.
        SpectrogramChannel;

        %SpectrumType Choose to visualize power, power density, or RMS
        %   Specify the spectrum type as one of 'Power' | 'Power density' |
        %   'RMS'. The default is 'Power'. This property is tunable.
        SpectrumType;

        %SpectrumUnits Spectrum units
        %   Specify the units in which the Spectrum Analyzer displays power or
        %   voltage values as one of 'Auto' | 'dBm' | 'dBW' | 'dBFS' | 'Watts' |
        %   'Vrms' | 'dBV'. 'Vrms' and 'dBV' apply when the SpectrumType is set
        %   to 'RMS'. 'dBFS' applies when the InputDomain is set to 'Time'.
        %   'Auto' applies when the InputDomain is set to 'Frequency'. If the
        %   InputDomain is set to 'Frequency' and SpectrumUnits is set to 'Auto',
        %   the Spectrum Analyzer assumes the spectrum units to be equal to input
        %   units specified on the 'InputUnits' property. If the InputDomain is
        %   set to 'Time' and SpectrumUnits is set to any option other than
        %   'Auto', the Spectrum Analyzer converts from the units specified on
        %   InputUnits to the units specified on SpectrumUnits. The default is
        %   'dBm'. This property is tunable.
        SpectrumUnits;

        %StartFrequency Start frequency over which spectrum is computed
        %   Specify the start frequency over which the Spectrum Analyzer computes
        %   and plots the spectrum as a real scalar in Hertz. This property
        %   applies when you set the FrequencySpan property to 'Start and stop
        %   frequencies'. The overall span defined by the StartFrequency and
        %   StopFrequency properties must fall within the full span frequency
        %   range which equals [-(SampleRate/2)+min(FrequencyOffset),
        %   (SampleRate/2)+max(FrequencyOffset)] Hz when the
        %   PlotAsTwoSidedSpectrum property is set to true, and
        %   [min(FrequencyOffset), (SampleRate/2)+max(FrequencyOffset)] Hz when
        %   the PlotAsTwoSidedSpectrum property is set to false. This property is
        %   tunable.
        %
        %   See also: dsp.SpectrumAnalyzer.StopFrequency, 
        %             dsp.SpectrumAnalyzer.PlotAsTwoSidedSpectrum  
        StartFrequency;

        %StopFrequency Stop frequency over which spectrum is computed
        %   Specify the stop frequency over which the Spectrum Analyzer computes
        %   and plots the spectrum as a real scalar in Hertz. This property
        %   applies when you set the FrequencySpan property to 'Start and stop
        %   frequencies'. The overall span defined by the StartFrequency and
        %   StopFrequency properties must fall within the full span frequency
        %   range which equals [-(SampleRate/2)+min(FrequencyOffset),
        %   (SampleRate/2)+max(FrequencyOffset)] Hz when the
        %   PlotAsTwoSidedSpectrum property is set to true, and
        %   [min(FrequencyOffset), (SampleRate/2)+max(FrequencyOffset)] Hz when
        %   the PlotAsTwoSidedSpectrum property is set to false. This property is
        %   tunable.
        %
        %   See also: dsp.SpectrumAnalyzer.StartFrequency, 
        %             dsp.SpectrumAnalyzer.PlotAsTwoSidedSpectrum    
        StopFrequency;

        %TimeResolution Time resolution
        %   Specify the time resolution of each spectrogram line as a positive
        %   scalar in seconds. This property applies when you set the ViewType
        %   property to 'Spectrogram' or 'Spectrum and spectrogram', and  the
        %   TimeResolutionSource property to 'Property'. The default is 1 ms.
        %   This property is tunable.
        TimeResolution;

        %TimeResolutionSource Source of the time resolution value
        %   Specify the source for the time resolution of each spectrogram line
        %   as one of 'Auto' | 'Property'. This property applies when you set the
        %   SpectrumType property to 'Spectrogram'. The default is 'Auto'. 
        %
        %   When FrequencyResolutionMethod is 'RBW' the following rules apply:
        %
        %      When both RBWSource and TimeResolutionSource are set to 'Auto'
        %      then RBW is set such that there are 1024 RBW intervals in one
        %      frequency span. The time resolution is set to 1/RBW.
        %      
        %      When RBWSource is set to 'Auto' and TimeResolutionSource is set to
        %      'Property' then time resolution becomes the main control and RBW
        %      is set to 1/TimeResolution Hz.
        %
        %      When RBWSource is set to 'Property' and TimeResolutionSource is
        %      set to 'Auto' then RBW becomes the main control and the time
        %      resolution is set 1/RBW s.
        %
        %      When both RBWSource and TimeResolutionSource are set to 'Property'
        %      then the specified time resolution value must be equal to or
        %      larger than the minimum attainable time resolution which is
        %      defined by 1/RBW. Several spectral estimates are combined into one
        %      spectrogram line to obtain the desired time resolution.
        %
        %   When FrequencyResolutionMethod is 'WindowLength' the following rules
        %   apply:
        %
        %      When TimeResolutionSource is set to 'Auto' then the time
        %      resolution is set to 1/RBW. In this case RBW is computed from the
        %      window length value as RBW = NENBW*Fs/WindowLength where NENBW is
        %      the normalized effective noise bandwidth of the window currently
        %      specified in the Window property.
        %
        %      When TimeResolutionSource is set to 'Property' then the time
        %      resolution value must be equal to or larger than the minimum
        %      attainable time resolution which equals 1/(NENBW*Fs/WindowLength).
        %      In this case several spectral estimates are combined into one
        %      spectrogram line to obtain the desired time resolution.
        %
        %   This property is tunable.
        TimeResolutionSource;

        %TimeSpan Time span
        %   Specify the time span of the spectrogram display as a positive scalar
        %   in seconds. This property applies when you set the ViewType property
        %   to 'Spectrogram' or 'Spectrum and spectrogram', and  the
        %   TimeSpanSource property to 'Property'. The specified time span must
        %   be at least two times larger than the duration of the number of
        %   samples required for a spectral update. The default is 100 ms. This
        %   property is tunable.
        TimeSpan;

        %TimeSpanSource Source of time span value
        %   Specify the source for the time span of the spectrogram as one of
        %   'Auto' | 'Property'. The default is 'Auto'. This property applies
        %   when the ViewType property is set to 'Spectrogram' or 'Spectrum and
        %   spectrogram'. When set to 'Auto' the spectrogram displays 100
        %   spectrogram lines at any given time. When set to 'Property' you
        %   specify the time duration of the spectrogram in seconds using the
        %   'TimeSpan' property. The specified time span must be at least two
        %   times larger than the duration of the number of samples required for
        %   a spectral update. This property is tunable.
        TimeSpanSource;

        %Title Display title
        %   Specify the display title as a string. The default value is ''. This
        %   property is tunable.  
        Title;

        %ViewType Choose to view spectrum, spectrogram, or both
        %   Specify the view type as one of 'Spectrum' | 'Spectrogram' | 
        %   'Spectrum and spectrogram'. The default is 'Spectrum'. This property
        %   is tunable.
        ViewType;

        %Window Window function
        %   Specify a window function for the spectral estimator as one of
        %   'Rectangular' | 'Chebyshev' | 'Flat Top' | 'Hamming' | 'Hann' |
        %   'Kaiser' | 'Blackman-Harris' | 'Custom'. The default is 'Hann'. This
        %   property is tunable.
        Window;

        %WindowLength Window length
        %   Specify the window length, in samples, used to compute spectral
        %   estimates as an integer scalar greater than two. This property
        %   applies when you set the FrequencyResolutionMethod property to
        %   'WindowLength'. The default is 1024. The frequency resolution of the
        %   analyzer is given as NENBW*Fs/WindowLength where NENBW is the
        %   normalized effective noise bandwidth of the window currently
        %   specified in the Window property. This property is tunable.
        WindowLength;

        %YLabel Y-axis label
        %   Specify the y-axis label as a string. The default value is ''. This
        %   property applies when the ViewType property is set to 'Spectrum' or
        %   'Spectrum and spectrogram'. This property is tunable.
        YLabel;

        %YLimits Y-axis limits
        %   Specify the y-axis limits as a two-element numeric vector: [ymin
        %   ymax]. This property applies when the ViewType property is set to
        %   'Spectrum' or 'Spectrum and spectrogram'. This property is tunable.
        YLimits;

    end
end
