function [s,g] = fos2sos(B,A)
%fos2sos Utility function to convert fourth-order section transfer function
%to second-order sections transfer function.
%
% B and A are M x 5 matrices, where M is the number of fourth order
% sections. 
%
% s is N x 6 matrix, where N is the number of second order sections. g is
% N+1 length column vector.
%
% Used in iirparameqDesign

% Copyright 2014-2017 The MathWorks, Inc.

%#codegen

% Check if already second-order (lowpass or highpass case or 2nd order overall design)
if all(all(B(:,4:5)==0,1),2) && all(all(A(:,4:5)==0,1),2)
    s = [B(:,1:3)./repmat(B(:,1),1,3),A(:,1:3)];
    g = [B(:,1);1];
else
    % Initialize outputs
    N = size(B,1)*4;
    if all(B(1,4:5)==0) && all(A(1,4:5)==0)
        % First row contains a SOS
        N = N-2;
    end
    s = zeros(N/2,6,'like',B);
    g = ones(N/2+1,1,'like',B);
    
    g(1:2:N/2) = B(:,1);
    
    if rem(N/2,2) 
        % Remove 2nd order section
        s(1,1:6) = [B(1,1:3)./B(1,1) A(1,1:3)];
        nextidx = 2;
        
        Br = B(2:end,:);
        Ar = A(2:end,:);
    else
        nextidx = 1;
       
        Br = B;
        Ar = A;
    end
           
        
    for k = 1:size(Br,1)
        Br(k,:) = Br(k,:)/Br(k,1);
    end
    
    % Form SOS numerators    
    if all(all(Br(:,[2,4])==0,1),2) && all(all(A(:,[2,4])==0,1),2)
        % Special case, Wo = 0.5 - treat differently for codegen reasons
        for k = nextidx:2:N/2-1
            r  = roots(Br(ceil(k/2),1:2:5));
            r2 = roots(Ar(ceil(k/2),1:2:5));
            
            rs  = sort(sqrt(r));
            rs2 = sort(sqrt(r2));
            
            p1 = poly([rs(1),conj(rs(1))]);
            p2 = poly([-rs(2),-conj(rs(2))]);
            p3 = poly([rs2(1),conj(rs2(1))]);
            p4 = poly([-rs2(2),-conj(rs2(2))]);
            
            s(k,1:3)   = real(p1(1:3));
            s(k,4:6)   = real(p3(1:3));
            s(k+1,1:3) = real(p2(1:3));
            s(k+1,4:6) = real(p4(1:3));
        end
    else
        for k = nextidx:2:N/2-1
            r  = roots(Br(ceil(k/2),:));
            r2 = roots(Ar(ceil(k/2),:));
            
            rs  = sort(r);
            rs2 = sort(r2);
            
            p1 = poly(rs(1:2));
            p2 = poly(rs(3:4));
            p3 = poly(rs2(1:2));
            p4 = poly(rs2(3:4));
            
            s(k,1:3)   = real(p1(1:3));
            s(k,4:6)   = real(p3(1:3));
            s(k+1,1:3) = real(p2(1:3));
            s(k+1,4:6) = real(p4(1:3));
        end
    end

end
end
