classdef (StrictDefaults)BinaryFileReader < dsp.BinaryFileReader & ...
        matlab.system.mixin.FiniteSource

% This class is used in the binary file reader block

% Copyright 2016 The MathWorks, Inc.

%#codegen
%#ok<*EMCLS>

properties (Nontunable)
    %SampleTime Sample time (s)
    SampleTime   = 1;

   % OutputEOF Output end-of-file indicator
   OutputEOF (1, 1) logical = false 
end

properties (Nontunable, Access = private)
    % pDataType Actual datatype 
    pDataType
end

methods
    % Constructor for the System object
    function obj = BinaryFileReader(varargin)
        setProperties(obj, nargin, varargin{:});
    end
    
end

    % Overridden implementation methods
    methods(Access = protected)
 
        function setpDataType(obj)
            dt = overrideDataType(obj, numerictype(obj.DataType));

            coder.internal.errorIf(~isempty(strfind(dt.DataTypeMode,'Scaled double')), ['dsp:binaryfileio',...
                    ':NoScaledDouble']);
                
            if strcmp(dt.DataTypeMode,'Double')
                obj.pDataType = 'double';
            elseif strcmp(dt.DataTypeMode,'Single')
                obj.pDataType = 'single';
            else
                obj.pDataType = obj.DataType;
            end
        end
        
        function n = getNumOutputsImpl(obj)
           n = 1+ double(obj.OutputEOF); 
        end
        
        function setupImpl(obj)
            setpDataType(obj);
            setupImpl@dsp.BinaryFileReader(obj);
        end
        
        function varargout = stepImpl(obj)
            if (obj.OutputEOF)
               varargout{2} = obj.pIsDone(obj.pIndex); 
            end
            y = stepImpl@dsp.BinaryFileReader(obj);
            varargout{1} = cast(y,obj.pDataType);
        end
        
       %%Propagators
        function varargout = isOutputComplexImpl(obj)
            varargout{1} = obj.IsDataComplex;
            varargout{2} = false;
        end
        
        function varargout = getOutputSizeImpl(obj)
            varargout{1} = [obj.SamplesPerFrame obj.NumChannels];
            varargout{2} = [1 1];
        end
        
        function varargout = getOutputDataTypeImpl(obj)
           dt = overrideDataType(obj, numerictype(obj.DataType));
           varargout{1} = dt;
           varargout{2} = 'logical';
        end
        
        function varargout = isOutputFixedSizeImpl(~)
            varargout{1} = true;
            varargout{2} = true;
        end
        
       function sts = getSampleTimeImpl(obj)
            Ts = obj.SampleTime * obj.SamplesPerFrame;
            sts = createSampleTime(obj, 'Type', 'Discrete', 'SampleTime', Ts);
       end 
        
       function varargout = getOutputNamesImpl(obj)
            if ~obj.OutputEOF
               varargout{1} = ''; 
            else
               varargout{1} = 'data'; 
               varargout{2} = 'EOF'; 
            end
       end
        
       function icon = getIconImpl(~)
          icon = getString(message('dsp:binaryfileio:BinaryFileReaderIcon'));
       end

       function resolvedName = resolveFileName(obj)
           coder.extrinsic('dsp.simulink.BinaryFileReader.resolve');
           coder.extrinsic('dsp.BinaryFileReader.checkFileName');
           resolvedName = coder.const(@dsp.simulink.BinaryFileReader.resolve,obj.pFilename);
           [~,exists] = coder.const(@dsp.BinaryFileReader.checkFileName,resolvedName);
           if isempty(coder.target)
                coder.internal.errorIf(exists~=2, ['dsp:binaryfileio',...
                    ':FileDoesNotExist']);
            end
       end

    end
            
    %% Block header - widget grouping
    methods(Static, Access=protected)
        function group = getPropertyGroupsImpl
            % Modify order of display
            group =  matlab.system.display.Section(...
                'Title', getString(message('dsp:system:Shared:Parameters')), ...
                'PropertyList', {'Filename',...
                                 'HeaderStructure',...
                                 'DataType','IsDataComplex',...
                                 'SamplesPerFrame',...
                                 'NumChannels',...
                                 'OutputEOF',...
                                 'SampleTime'},...
                                 'DependOnPrivatePropertyList',{'Filename','HeaderStructure','DataType','HeaderStructure'});
                             
        end
        %------------------------------------------------------------------
        function header = getHeaderImpl
            % MATLAB System block header
            header = matlab.system.display.Header(...
                'dsp.simulink.BinaryFileReader', ...
                'ShowSourceLink', true, ...
                'Title', getString(message('dsp:binaryfileio:BinaryFileReaderTitle')),...
                'Text',  getString(message('dsp:binaryfileio:BinaryFileReaderHeader')));
        end
        
    end
    
    methods(Static, Hidden)
        function resolvedName = resolve(filename)
            
            resolvedName = filename;
            
            if exist(filename,'file')
                return;
            end

            try %#ok
                 % Try evaluating as a variable
                filename = slResolve(filename,gcbh);
                
                if ~ischar(filename) || isempty(filename)
                    return;
                end

                name = strrep(filename,'\','/');
                
                % Check if file exists
                if ~exist(name,'file')
                    return;
                end
                
                resolvedName = name;
    
            catch e %#ok
            end
        end
    end
    
end
