classdef (StrictDefaults)IIRHalfbandDecimator < dsp.IIRHalfbandDecimator
    % This class is used for the IIR Halfband Decimator block in dspfdesign
    
    % Copyright 2014-2020 The MathWorks, Inc.
    
    %#codegen
    %#ok<*EMCA>
    properties (Nontunable)
        %FilterBankOutputPort Output highpass subband
        %   Specify if the block should be an analysis filter bank, as a
        %   logical scalar. When this property is set to false, the block
        %   acts as a decimator. If the property is set to true, the object
        %   acts as an analysis filter bank, producing two
        %   power-complementary outputs. The default of this property is
        %   false.
        FilterBankOutputPort (1, 1) logical = false;
        
        %CoefficientsPort Specify coefficients from input port
        CoefficientsPort (1, 1) logical = false;
    end
    
    properties (Constant, Hidden)
        AllpassCoefficients1Set = matlab.system.SourceSet( ...
            dsp.simulink.internal.IIRHalfbandPolicy('SystemBlock', 'CoefficientsPort', 1, 'coeffs1'), ...
            {'Property','MATLAB','CoefficientsPort'})
        AllpassCoefficients2Set = matlab.system.SourceSet( ...
            dsp.simulink.internal.IIRHalfbandPolicy('SystemBlock', 'CoefficientsPort', 2, 'coeffs2'), ...
            {'Property','MATLAB','CoefficientsPort'})
    end
    %----------------------------------------------------------------------
    % Public Methods
    %----------------------------------------------------------------------
    methods
        function obj = IIRHalfbandDecimator(varargin)
            obj@dsp.IIRHalfbandDecimator(varargin{:});
        end
    end
    %----------------------------------------------------------------------
    % Protected Methods
    %----------------------------------------------------------------------
    methods (Access = protected)
        
        function validateInputsImpl(obj,varargin)
            validateInputsImpl@dsp.IIRHalfbandDecimator(obj,varargin{:});
        end
        %------------------------------------------------------------------
        function num = getNumOutputsImpl(obj)
            num = 1+obj.FilterBankOutputPort;
        end
        %------------------------------------------------------------------
        function varargout = getInputNamesImpl(obj)
            % Name of input port (shows on block icon)
            % Input port name to show on the block
            if obj.CoefficientsPort
                varargout{1} = 'x';
            else
                varargout{1} = '';
            end
        end
        %------------------------------------------------------------------
        function varargout = getOutputNamesImpl(obj)
            % Name of output ports (shows on block icon)
            if ~obj.FilterBankOutputPort
                varargout{1} =   '';
            else % power-complementary outputs requested
                varargout{1} =   'LP';
                varargout{2} =   'HP';
            end
        end
        %------------------------------------------------------------------
        function icon = getIconImpl(obj)
            % MATLAB system block icon
            if ~obj.FilterBankOutputPort
                icon = getString(message(...
                    'dsp:system:IIRHalfbandDecimator:decimatorIconString'));
            else % power-complementary outputs requested
                icon = getString(message(...
                    'dsp:system:IIRHalfbandDecimator:analysisIconString'));
            end
        end
        %------------------------------------------------------------------
        function val = supportsMultipleInstanceImpl(~)
            % Support in for-each subsystem
            val = true;
        end
        % PROPAGATOR METHODS: (used by Simulink)
        %------------------------------------------------------------------
        function varargout = getOutputSizeImpl(obj)
            inputSize = propagatedInputSize(obj,1);
            coder.internal.errorIf(mod(inputSize(1),2)~=0, ...
                'dsp:system:FIRHalfbandDecimator:invalidInputDimsForHalfbandDecim');
            varargout{1} = inputSize;
            varargout{1}(1) = varargout{1}(1)/2;
            if obj.FilterBankOutputPort
                varargout{2} = varargout{1};
            end
        end
        %------------------------------------------------------------------
        function varargout = getOutputDataTypeImpl(obj)
            varargout{1} = propagatedInputDataType(obj,1);
            if obj.FilterBankOutputPort
                varargout{2} = varargout{1};
            end
        end
        %------------------------------------------------------------------
        function varargout = isOutputComplexImpl(obj)
            varargout{1} = propagatedInputComplexity(obj,1);
            if obj.FilterBankOutputPort
                varargout{2} = varargout{1};
            end
        end
        %------------------------------------------------------------------
        function varargout = isOutputFixedSizeImpl(obj)
            varargout{1} = propagatedInputFixedSize(obj,1);
            if obj.FilterBankOutputPort
                varargout{2} = varargout{1};
            end
        end
    end
    %----------------------------------------------------------------------
    % Static hidden methods
    %----------------------------------------------------------------------
    methods(Static,Hidden)
        function props = getDisplayPropertiesImpl()
            props = {...
                'Specification',...
                'FilterOrder', ...
                'TransitionWidth', ...
                'StopbandAttenuation', ...
                'DesignMethod', ...
                'Structure', ...
                'HasPureDelayBranch', ...
                'Delay', ...
                'CoefficientsPort', ...
                'AllpassCoefficients1', ...
                'AllpassCoefficients2', ...
                'WDFCoefficients1', ...
                'WDFCoefficients2', ...
                'HasTrailingFirstOrderSection', ...
                'FilterBankOutputPort', ...
                'InheritSampleRate', ...
                'SampleRate'};
        end
    end
    %----------------------------------------------------------------------
    % Static protected methods
    %----------------------------------------------------------------------
    methods(Static,Access=protected)
        function mainS = getPropertyGroupsImpl
            props = dsp.simulink.IIRHalfbandDecimator.getDisplayPropertiesImpl;
            
            mainS = matlab.system.display.Section(...
                'Title', getString(message('dsp:system:Shared:Parameters')), ...
                'PropertyList', props);
            
            filterVis = dsp.internal.FilterVisualizationButton;
            mainS.Actions = filterVis.fvtoolAction;
        end
        %------------------------------------------------------------------
        function header = getHeaderImpl
            % MATLAB System block header
            header = matlab.system.display.Header(...
                'dsp.simulink.IIRHalfbandDecimator', ...
                'ShowSourceLink', true, ...
                'Title', getString(message('dsp:system:IIRHalfbandDecimator:titleString')),...
                'Text',  getString(message('dsp:system:IIRHalfbandDecimator:descriptionString')));
        end
    end
    %----------------------------------------------------------------------
    % Hidden methods
    %----------------------------------------------------------------------
    methods (Hidden)
        function [flag,errID] = isFVtoolSupported(obj)
            % Specify if fvtool plot is supported, and what error should be
            % used when it is not supported.
            [flag,errID] = isFVtoolSupported@dsp.internal.FilterAnalysis(obj);
            if (strcmp(obj.Specification,'Coefficients') &&  ...
                    strcmp(obj.Structure,'Minimum multiplier') && obj.CoefficientsPort)
                flag = false;
                errID = 'dsp:system:Shared:fvtoolButtonSpecifyCoefficientsText';
            end

        end
    end
end
