classdef VariableBandwidthFIRFilter < dsp.VariableBandwidthFIRFilter
    % This class is used for Variable Bandwidth FIR Filter block in
    % dspfdesign
    
    %   Copyright 2017-2021 The MathWorks, Inc.
    
    %#codegen
    %#ok<*EMCLS>
    properties(Nontunable)
        %BandwidthPort Specify bandwidth from input port
        BandwidthPort (1, 1) logical = false;
        %CenterFrequencyPort Specify center frequency from input port
        CenterFrequencyPort (1, 1) logical = false;
        %CutoffFrequencyPort Specify cutoff frequency from input port
        CutoffFrequencyPort (1, 1) logical = false;
    end
    
    properties (Constant, Hidden)
        CenterFrequencySet = matlab.system.SourceSet( ...
            dsp.simulink.internal.VariableBandwidthFilterPolicy('SystemBlock', 'CenterFrequencyPort', 1, 'Fc'), ...
            {'Property','MATLAB','CenterFrequencyPort'})
        BandwidthSet = matlab.system.SourceSet( ...
            dsp.simulink.internal.VariableBandwidthFilterPolicy('SystemBlock', 'BandwidthPort', 2, 'BW'), ...
            {'Property','MATLAB','BandwidthPort'})
        CutoffFrequencySet = matlab.system.SourceSet( ...
            dsp.simulink.internal.VariableBandwidthFilterPolicy('SystemBlock', 'CutoffFrequencyPort', 3, 'Fcut'), ...
            {'Property','MATLAB','CutoffFrequencyPort'})
    end
    
%--------------------------------------------------------------------------
% Public methods
%--------------------------------------------------------------------------
    methods
        function obj = VariableBandwidthFIRFilter(varargin)
            % Constructor
            obj@dsp.VariableBandwidthFIRFilter(varargin{:})
        end
    end
%--------------------------------------------------------------------------
% Protected methods
%--------------------------------------------------------------------------
    methods (Access = protected)
        
        function validateInputsImpl(obj,varargin)
            validateInputsImpl@dsp.internal.VariableBandwidthFilterBase(obj,varargin{:});
        end
        %------------------------------------------------------------------
        function icon = getIconImpl(~)
            % MATLAB system block icon
            icon = getString(message(...
                'dsp:system:VariableBandwidthFilter:BlockIconFIR'));
        end
        %------------------------------------------------------------------
        function varargout = getInputNamesImpl(obj)
            if (obj.CutoffFrequencyPort && any(strcmp(obj.FilterType,{'Lowpass','Highpass'}))) || ...
                    ((obj.CenterFrequencyPort || obj.BandwidthPort) && any(strcmp(obj.FilterType,{'Bandpass','BandStop'})))
                varargout{1} = 'x';
            else
                varargout{1} = '';
            end
        end
        %------------------------------------------------------------------
        function processTunedPropertiesImpl(obj)
            % Validate the filter specs and values if they are specified
            % from the input port
            if (obj.CutoffFrequencyPort && any(strcmp(obj.FilterType,{'Lowpass','Highpass'}))) || ...
                    ((obj.CenterFrequencyPort || obj.BandwidthPort) && any(strcmp(obj.FilterType,{'Bandpass','BandStop'})))
                validateFilterSpec(obj);
            end
            % Tune the filter coefficients
            processTunedPropertiesImpl@dsp.VariableBandwidthFIRFilter(obj);
        end
        
        %------------------------------------------------------------------
        function crossValidateProperties(obj)
            % Validate the filter specs and values if they are specified on
            % the dialog
            if (~obj.CutoffFrequencyPort && any(strcmp(obj.FilterType,{'Lowpass','Highpass'}))) || ...
                    ((~obj.CenterFrequencyPort || ~obj.BandwidthPort) && any(strcmp(obj.FilterType,{'Bandpass','BandStop'})))
                validateFilterSpec(obj);
            end
        end
        %------------------------------------------------------------------
        function validateFilterSpec(obj)
            Fs = obj.SampleRate;
            % Cross-validation of filter specifications
            if strcmp(obj.FilterType,'Bandpass') || ...
                    strcmp(obj.FilterType,'Bandstop')
                coder.internal.errorIf(obj.CenterFrequency >= Fs/2, 'dsp:system:VariableBandwidthFilter:CharacteristicTooLarge','CenterFrequency', floor(Fs), round((Fs - floor(Fs)) * 10));
                coder.internal.errorIf(obj.Bandwidth >= Fs/2, 'dsp:system:VariableBandwidthFilter:CharacteristicTooLarge','Bandwidth' , floor(Fs), round((Fs - floor(Fs)) * 10));
                coder.internal.errorIf(obj.CenterFrequency + obj.Bandwidth/2 >= Fs/2, 'dsp:system:VariableBandwidthFilter:CharacteristicTooLarge','CenterFrequency + Bandwidth/2' , floor(Fs), round((Fs - floor(Fs)) * 10));
                coder.internal.errorIf(obj.CenterFrequency - obj.Bandwidth/2 <= 0, 'dsp:system:VariableBandwidthFilter:CFMinusBandwidthNegative');
            else % highpass/lowpass validation
                validateFrequencyRange(obj);
            end
        end
    end
%--------------------------------------------------------------------------
% Static, hidden methods
%--------------------------------------------------------------------------
    methods (Static, Hidden)
        %------------------------------------------------------------------
        function props = getDisplayPropertiesImpl()
            props = {'FilterOrder','FilterType','CutoffFrequencyPort','CutoffFrequency',...
                'CenterFrequencyPort','CenterFrequency','BandwidthPort','Bandwidth', ...
                'Window','KaiserWindowParameter','SidelobeAttenuation','InheritSampleRate','SampleRate'};
        end
        %------------------------------------------------------------------
        function checkVal(value,prop)
            validateattributes(value,{'double','single'},...
                {'real','scalar','>=',0,'finite'},...
                '',prop);
        end
    end
%--------------------------------------------------------------------------
% Static, Protected methods
%--------------------------------------------------------------------------
    methods(Static, Access=protected)
        %------------------------------------------------------------------
        function mainS = getPropertyGroupsImpl
            % Modify order of display
            props = dsp.simulink.VariableBandwidthFIRFilter.getDisplayPropertiesImpl;
            mainS =  matlab.system.display.Section(...
                'Title', getString(message('dsp:system:Shared:Parameters')), ...
                'PropertyList', props);
            
            filterVis = dsp.simulink.internal.FilterResponseVisualizationButton;
            mainS.Actions = filterVis.visualizeAction;
        end
        %------------------------------------------------------------------
        function header = getHeaderImpl
            % MATLAB System block header
            title = getString(message('dsp:system:VariableBandwidthFilter:VariableFIRTop'));
            text = getString(message('dsp:system:VariableBandwidthFilter:VariableFIRHeader'));
            header = matlab.system.display.Header('dsp.simulink.VariableBandwidthFIRFilter', ...
                'ShowSourceLink', true, 'Title', title, 'Text', text);
        end
    end
%--------------------------------------------------------------------------
% Hidden methods
%--------------------------------------------------------------------------
    methods (Hidden)
        %------------------------------------------------------------------
        function filters = getVisualizationFilters(obj)
            % Used to return filters that will be shown when the visualize
            % filter response button is clicked.
            filters = obj;
        end
        %------------------------------------------------------------------
        function legend = getVisualizationLegend(~,blkName)
            % Used to return the legend that will be shown when the
            % visualize filter response plot is shown
            legend = {blkName};
        end
    end
end
