classdef (StrictDefaults)ZoomFFT < dsp.ZoomFFT
    
    % ZOOMFFT Implements zoom FFT
    % This class is used in the Zoom FFT block
    
    % Copyright 2016 The MathWorks, Inc.
    
    %#codegen
    %#ok<*EMCLS>
    
    properties (Nontunable)
        % CenterFrequencyPort Specify center frequency from input port
        CenterFrequencyPort (1, 1) logical = false;
    
        % InheritFFTLength Inherit FFT Length from input dimensions
        %  Specify how to determine the FFT length. When this property is
        %  set to true, the FFT length is equal to the number of rows of
        %  the input signal divided by the decimation factor. when it is
        %  set to false, the FFT length is equal to the value of the
        %  FFtLength property.
        InheritFFTLength (1, 1) logical = true;
    end
    
    properties (Constant, Hidden)
        % CenterFrequencySet is used to setup the choices for CenterFrequency
        CenterFrequencySet = matlab.system.SourceSet({'PropertyOrInput', ...
            'SystemBlock', 'CenterFrequencyPort', 1, 'Fc'});
    end
    
    methods
        %CONSTRUCTOR
        function obj = ZoomFFT(varargin)
            setProperties(obj, nargin, varargin{:});
        end
    end

    methods(Access = protected)

        function cacheFFTLength(obj)
            if isFFTAutoMode(obj)
              obj.pNFFT = [];
            else
              obj.pNFFT = obj.FFTLength; 
            end
        end
        
        function flag = isFFTAutoMode(obj)
            flag = obj.InheritFFTLength;
        end
        
        %%Propagators
        
        function o = isOutputComplexImpl(~)
            o = true;
        end
        
        function sz = getOutputSizeImpl(obj)
            
            sz_in = propagatedInputSize(obj, 1);
            
            coder.internal.errorIf(...
                mod(sz_in(1),obj.DecimationFactor)~=0, ...
                'dsp:system:ZoomFFT:InvalidSize',obj.DecimationFactor);
            
            if isFFTAutoMode(obj)
                nfft = sz_in(1) / obj.DecimationFactor;
            else
                nfft = obj.FFTLength;
            end
            sz = [nfft sz_in(2)];
        end
        
        function dt = getOutputDataTypeImpl(obj)
            dt = propagatedInputDataType(obj, 1);
        end
        
        function f = isOutputFixedSizeImpl(obj)
            f = propagatedInputFixedSize(obj,1);
        end
        
        function name = getInputNamesImpl(obj)
            if obj.CenterFrequencyPort
                name = 'x';
            else
                name = '';
            end
        end
        
        function name = getOutputNamesImpl(~)
            name = '';
        end
        
        function validateInputsImpl(obj,varargin)
            u = varargin{1};
            validateInputsImpl@dsp.ZoomFFT(obj,u);
        end
        
        function flag = isInactivePropertyImpl(obj,prop)
           flag = false;
           if strcmp(prop,'FFTLength')
              flag =obj.InheritFFTLength;
           end
        end
        
        function dt = getTunablePropertiesSpecificationImpl(obj,~)
            dt = propagatedInputDataType(obj,1);
        end
        
        function flag = supportsMultipleInstanceImpl(~)
           flag = true; 
        end
        
        function flag = isInputSizeMutableImpl(~,~)
            % input arg may change size after locked
            flag = true;
        end
                
        function flag = isInputComplexityMutableImpl(~,~)
            % input arg may change complexity after locked
            flag = true;
        end
        
        %% Block Icon
        function icon = getIconImpl(~)
            icon = getString(message('dsp:system:ZoomFFT:BlockIcon'));
        end
    end
    
    %% Block header - widget grouping
    methods(Static, Access=protected)
        
        function group = getPropertyGroupsImpl
            % Modify order of display
            DecimationS =  matlab.system.display.Section(...
                'Title', getString(message('dsp:system:Shared:Parameters')), ...
                'PropertyList', {'DecimationFactor'});
            
             CFS =  matlab.system.display.Section(...
                'Title', '', ...
                'PropertyList', {'CenterFrequencyPort','CenterFrequency'});
            
             FFTS =  matlab.system.display.Section(...
                'Title', '', ...
                'PropertyList', {'InheritFFTLength','FFTLength'});
            
            SampleRateS  =  matlab.system.display.Section(...
                'Title', '','PropertyList',{'InheritSampleRate','SampleRate'});

            group = [DecimationS CFS FFTS SampleRateS];
        end
        
        function header = getHeaderImpl
            % MATLAB System block header
            header = matlab.system.display.Header(...
                'dsp.simulink.ZoomFFT', ...
                'ShowSourceLink', true, ...
                'Title', getString(message('dsp:system:ZoomFFT:BlockTitle')),...
                'Text',  getString(message('dsp:system:ZoomFFT:BlockHeader')));
        end
    end
    
    
end