classdef (StrictDefaults)ArrayVectorSubtractor < matlab.System
%ArrayVectorSubtractor Subtract vector from array along specified dimension
%   AVS = dsp.ArrayVectorSubtractor returns an array vector subtraction
%   System object, AVS, that subtracts the elements of vector V from input
%   array A along the specified dimension.
%
%   AVS = dsp.ArrayVectorSubtractor('PropertyName', PropertyValue, ...)
%   returns an array vector subtraction object, AVS, with each specified
%   property set to the specified value.
%
%   Step method syntax:
%
%   Y = step(AVS, A, V) returns Y, the result of subtracting the elements
%   of input vector V from the input array A along the specified dimension
%   when the VectorSource property is 'Input port'. The length of the input
%   V must be the same as the length of the specified dimension of A.
%
%   Y = step(AVS, A) returns Y, the result of subtracting the elements of
%   the vector specified in the Vector property from the input array A
%   along the specified dimension when the VectorSource property is
%   'Property'. The length of the vector specified in the Vector property
%   must be the same as the length of the specified dimension of A.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   ArrayVectorSubtractor methods:
%
%   step     - See above description for use of this method
%   release  - Allow property value and input characteristics changes
%   clone    - Create array vector subtraction object with same property
%              values
%   isLocked - Locked status (logical)
%
%   ArrayVectorSubtractor properties: 
%
%   Dimension    - Dimension along which to subtract vector elements from
%                  input
%   VectorSource - Source of vector
%   Vector       - Vector values
%
%   This System object supports fixed-point operations. For more
%   information, type dsp.ArrayVectorSubtractor.helpFixedPoint.
%
%   % EXAMPLE: Use ArrayVectorSubtractor System object to subtract a vector
%   %          from a matrix
%       arrVecSub = dsp.ArrayVectorSubtractor;
%       a = ones(2);
%       x = [1 2]';
%       y = arrVecSub(a, x) 
%
%   See also dsp.ArrayVectorSubtractor.helpFixedPoint.

%   Copyright 2008-2021 The MathWorks, Inc.

%#codegen
    properties
        %Vector Vector values 
        %   Specify the vector values. This property is applicable when the
        %   VectorSource property is 'Property'. The default value is
        %   [0.5 0.25]. This property is tunable.
        Vector = [0.5 0.25];
    end

    properties (Nontunable)
        %Dimension Dimension along which to subtract vector elements from input
        %   Specify the dimension along which to subtract the elements of the
        %   vector from the input array as an integer-valued scalar greater
        %   than 0. The default value is 1.
        Dimension = 1;
        %VectorSource Source of vector  
        %   Specify the source of the vector values as one of ['Input port' |
        %   'Property']. The default is 'Input port'.
        VectorSource = 'Input port';
        
        %RoundingMethod Rounding method for fixed-point operations
        %   Specify the rounding method as one of ['Ceiling' | 'Convergent' |
        %   'Floor' | 'Nearest' | 'Round' | 'Simplest' | 'Zero']. This
        %   property is applicable when the object is not in a full precision
        %   configuration. The default is 'Floor'.
        RoundingMethod = 'Floor';
        %OverflowAction Overflow action for fixed-point operations
        %   Specify the overflow action as one of ['Wrap' | 'Saturate']. This
        %   property is applicable when the object is not in a full precision
        %   configuration. The default is 'Wrap'.
        OverflowAction = 'Wrap';
        
        %VectorDataType Vector word- and fraction-length designations
        %   Specify the vector fixed-point data type as one of ['Same word
        %   length as input' | 'Custom']. This property is applicable when the
        %   VectorSource property is 'Property'. The default is 'Same word
        %   length as input'.
        VectorDataType = 'Same word length as input';
        %CustomVectorDataType Vector word and fraction lengths
        %   Specify the vector fixed-point type as a numerictype
        %   object. This property is applicable when the VectorSource property
        %   is 'Property' and the VectorDataType property is 'Custom'. The
        %   default value of this property is numerictype([],16,15).
        %
        %   See also numerictype.
        CustomVectorDataType = numerictype([], 16, 15);
        
        %AccumulatorDataType Accumulator word- and fraction-length designations
        %   Specify the accumulator fixed-point data type as one of ['Full
        %   precision' | 'Same as first input' | 'Custom']. This
        %   property is applicable when the FullPrecisionOverride property is
        %   false. The default is 'Full precision'.
        AccumulatorDataType = 'Full precision';
        %CustomAccumulatorDataType Accumulator word and fraction lengths
        %   Specify the accumulator fixed-point type as an auto-signed scaled
        %   numerictype object. This property is applicable when the
        %   FullPrecisionOverride property is false and when the
        %   AccumulatorDataType property is 'Custom'. The default value of this
        %   property is numerictype([],32,30).
        %
        %   See also numerictype.
        CustomAccumulatorDataType = numerictype([], 32, 30);
        
        %OutputDataType Output word- and fraction-length designations
        %   Specify the output fixed-point data type as one of ['Same as
        %   accumulator' | 'Same as first input' | 'Custom']. This
        %   property is applicable when the FullPrecisionOverride property is
        %   false. The default is 'Same as accumulator'.
        OutputDataType = 'Same as accumulator';
        %CustomOutputDataType Output word and fraction lengths
        %   Specify the output fixed-point type as an auto-signed scaled
        %   numerictype object. This property is applicable when the
        %   FullPrecisionOverride property is false and when the
        %   OutputDataType property is 'Custom'. The default value of this
        %   property is numerictype([],16,15).
        %
        %   See also numerictype.
        CustomOutputDataType = numerictype([], 16, 15);

        
        % FIXED-POINT

        %FullPrecisionOverride Full precision override for fixed-point arithmetic
        %   Specify whether to use full precision rules. If you set
        %   FullPrecisionOverride to true the object computes all
        %   internal arithmetic and output data types using full
        %   precision rules. These rules guarantee that no quantization
        %   occurs within the object. Bits are added as needed to ensure
        %   that no round-off or overflow occurs. If you set
        %   FullPrecisionOverride to false, fixed-point data types
        %   are controlled through individual property settings.
        FullPrecisionOverride (1, 1) logical = true;
    end

    properties(Access = private)
        pVector;
        pOutput;
        pFimath;
    end % properties(Access = private)
    
    properties(Access=private,Nontunable)
        pVectorConst
    end
    
    properties(Constant, Hidden)
        VectorSourceSet = dsp.CommonSets.getSet('PropertyOrInputPort');
        % fixpt enum props
        RoundingMethodSet       = dsp.CommonSets.getSet('RoundingMethod');
        OverflowActionSet       = dsp.CommonSets.getSet('OverflowAction');
        VectorDataTypeSet = dsp.CommonSets.getSet('FixptModeEitherScale');
        AccumulatorDataTypeSet = dsp.CommonSets.getSet('FixptModeInheritFirst');
        OutputDataTypeSet = dsp.CommonSets.getSet('FixptModeAccumFirst');
    end % properties(Constant, Hidden)
    
    properties(Nontunable,Access=private)
        privVector
    end
        
    methods
        function obj = ArrayVectorSubtractor(varargin)
            coder.internal.warning('dsp:system:ArrayVectorSubtractor_NotSupported');
            if length(varargin) > 1
                % Support name-value pair arguments
                for i=coder.unroll(1:2:nargin)
                    if strcmp(varargin{i}, 'Vector')
                        obj.pVectorConst = varargin{i+1};
                    end
                end
            end
            setProperties(obj,nargin,varargin{:});
        end % ArrayVectorSubtractor constructor
        
        function set.CustomAccumulatorDataType(obj, val)
            validateCustomDataType(obj, 'CustomAccumulatorDataType', val, ...
                {'AUTOSIGNED','SCALED'});
            obj.CustomAccumulatorDataType = val;
        end
        
        function set.CustomOutputDataType(obj, val)
            validateCustomDataType(obj, 'CustomOutputDataType', val, ...
                {'AUTOSIGNED','SCALED'});
            obj.CustomOutputDataType = val;
        end
    end % Public methods
    
    methods (Access=protected)
        
        function processTunedPropertiesImpl(obj)
            if strcmp(obj.VectorSource, 'Property')
                obj.pVector(:) = obj.Vector;
            end
        end % processTunedPropertiesImpl
        
        function setupImpl(obj, A, varargin)
            % Vector type can be 'Same as first input'
            vectorT = obj.vectorType(A,varargin{:});
            %
            % Accumulator type can be 'Same as first input' and depends on
            % vector type if "Full Precision".
            accT = obj.accumulatorType(A,vectorT);
            %
            % Output type can be 'Same as first input' or 'Same as
            % accumulator'
            outT = obj.outputType(A,accT);
            %
            % Create private properties
            if isfi(accT)
                obj.pFimath=fimath('RoundingMethod',obj.RoundingMethod, ...
                    'OverflowAction',obj.OverflowAction,...
                    'SumMode','SpecifyPrecision',...
                    'SumWordLength',accT.WordLength,...
                    'SumFractionLength',accT.FractionLength);
                obj.pOutput=setfimath(zeros(size(A),'like',outT),obj.pFimath);
            else
                obj.pFimath=[];
                obj.pOutput=zeros(size(A),'like',outT);
            end
            if strcmp(obj.VectorSource, 'Property')
                obj.pVector = cast(obj.Vector,'like',vectorT);
            end
        end % setupImpl
        
        function N = getNumInputsImpl(obj)
            if strcmp(obj.VectorSource, 'Input port')
                N=2;
            else % Property
                % Specify number of System inputs
                N = 1; % Because stepImpl has one argument beyond obj
            end
        end % getNumInputsImpl
        
        function N = getNumOutputsImpl(~)
            % Specify number of System outputs
            N = 1; % Because stepImpl has one output
        end % getNumOutputsImpl
        
        function vectorT=vectorType(obj,A,varargin)
            if obj.getNumInputsImpl()==2
                % step(obj, A, v)
                vectorT = cast(0,'like',varargin{1});
            else
                % step(obj, A)
                % VectorSource = 'Property'
                if isfi(A) && isscaledtype(A)
                    if ~strcmp(obj.VectorDataType, 'Custom')
                        switch obj.VectorDataType
                            case 'Same word length as input'
                                % Best precision of value of
                                % obj.Vector (or obj.pVector?)
                                vectorT = cast(0,'like',fi(coder.const(obj.pVectorConst),true,A.WordLength));
                        end
                    else % Custom
                        vectorT = fi(obj.Vector,obj.CustomVectorDataType);
                    end
                else % Same as A
                    vectorT = cast(obj.Vector,'like',real(A));
                end
            end
        end % vectorType
        
        function accT = accumulatorType(obj,A,vectorT)
            if ~isreal(A) || ~isreal(vectorT)
                tmp = 1i;
            else
                tmp = 0;
            end
            if isfi(A) && isscaledtype(A)
                if ~strcmp(obj.AccumulatorDataType, 'Custom')
                    switch obj.AccumulatorDataType
                        case 'Same as first input'
                            accT = cast(tmp,'like',A);
                        case 'Full precision'
                            if isfi(vectorT)  && isscaledtype(vectorT)
                                fpnt = fixed.aggregateType(A.numerictype, vectorT.numerictype);
                            else
                                fpnt = A.numerictype;
                            end
                            accT = fi(tmp,fpnt.Signed,fpnt.WordLength+1,fpnt.FractionLength);
                    end
                else % 'Custom'
                    accT = fi(tmp,obj.CustomAccumulatorDataType);
                end
            else
                accT = cast(tmp,'like',A);
            end
        end % accumulatorType
        
        function outT = outputType(obj,A,accT)
            if isfi(A) && isscaledtype(A)
                if ~strcmp(obj.OutputDataType, 'Custom')
                    switch obj.OutputDataType
                        case 'Same as first input'
                            outT = cast(accT,'like',A);
                        case 'Same as accumulator'
                            outT = accT;
                    end
                else % 'Custom'
                    outT = fi(accT,obj.CustomOutputDataType);
                end
            else
                outT = cast(accT,'like',A);
            end
        end % outputType
        
        function flag = isInactivePropertyImpl(obj, prop)
            flag = false;
            switch prop
                case 'CustomVectorDataType'
                    if ( strcmp(obj.VectorSource, 'Property') && ...
                            ~matlab.system.isSpecifiedTypeMode(obj.VectorDataType) ) || ...
                            ~strcmp(obj.VectorSource, 'Property')
                        flag = true;
                    end
                case {'Vector', 'VectorDataType'}
                    if ~strcmp(obj.VectorSource, 'Property')
                        flag = true;
                    end
                case {'RoundingMethod','OverflowAction'}
                    if obj.FullPrecisionOverride || ...
                            (strcmpi(obj.AccumulatorDataType, 'Full precision') &&  ...
                            strcmpi(obj.OutputDataType, 'Same as accumulator'))
                        % Either FullPrecisionOverride is TRUE OR
                        % Due to the individual signal property settings, the
                        % object is effectively in FullPrecisionOverride mode.
                        flag = true;
                    end
                case {'AccumulatorDataType','OutputDataType'}
                    if obj.FullPrecisionOverride
                        flag = true;
                    end
                case 'CustomAccumulatorDataType'
                    if obj.FullPrecisionOverride || ...
                            (strcmpi(obj.AccumulatorDataType, 'Full precision') &&  ...
                            strcmpi(obj.OutputDataType, 'Same as accumulator')) || ...
                            ~matlab.system.isSpecifiedTypeMode(obj.AccumulatorDataType)
                        % Either FullPrecisionOverride is TRUE OR
                        % Due to the individual signal property settings, the
                        % object is effectively in FullPrecisionOverride mode OR
                        % the property is NOT in 'custom' mode
                        flag = true;
                    end
                case 'CustomOutputDataType'
                    if obj.FullPrecisionOverride || ...
                            (strcmpi(obj.AccumulatorDataType, 'Full precision') &&  ...
                            strcmpi(obj.OutputDataType, 'Same as accumulator')) || ...
                            ~matlab.system.isSpecifiedTypeMode(obj.OutputDataType)
                        % Either FullPrecisionOverride is TRUE OR
                        % Due to the individual signal property settings, the
                        % object is effectively in FullPrecisionOverride mode OR
                        % the property is NOT in 'custom' mode
                        flag = true;
                    end
            end
        end % isInactivePropertyImpl
        
        function s = saveObjectImpl(obj)
            % Call the base class method
            s = saveObjectImpl@matlab.System(obj);
            % Save the private data only if we are locked
            if isLocked(obj)
                s.pVector = obj.pVector;
                s.pOutput = obj.pOutput;
                s.pFimath = obj.pFimath;
            end
        end % saveObjectImpl
        
        function loadObjectImpl(obj, s, wasLocked)
            % Restore the private data only if we were locked
            if wasLocked
                % If the object was initially saved in s-function form,
                % then these fields are not present
                if isfield(s,'pVector')
                    obj.pVector = s.pVector;
                    obj.pOutput = s.pOutput;
                    obj.pFimath = s.pFimath;
                end
            end
            
            % Call the base class method
            loadObjectImpl@matlab.System(obj, s);
        end % loadObjectImpl
        
        function Y = stepImpl(obj, A, varargin)
            tmpY = coder.nullcopy(zeros(size(A),'like',obj.pOutput));

            if isempty(coder.target)
                isFixPtA = isfi(A) || isinteger(A);
                if isFixPtA
                    tmpA = setfimath(A, obj.pFimath);
                else
                    tmpA = A;
                end
            else
                tmpA = setfimath(A, obj.pFimath);
            end
            
            if strcmp(obj.VectorSource, 'Input port')
                V = varargin{1};
            else
                V = obj.pVector;
            end
            V = V(:);
            
            if isempty(coder.target)
                isFixPtV = isfi(V) || isinteger(V);
                if isFixPtV
                    tmpV = setfimath(V, obj.pFimath);
                else
                    tmpV = V;
                end
            else
                tmpV = setfimath(V, obj.pFimath);
            end
            
            dim = obj.Dimension;
            
            if dim==1
                % Sum over leading dimension of matrix or N-D array
                [~,n]=size(tmpA);
                for j=1:n
                    tmpY(:,j) = tmpA(:,j) - tmpV;
                end
            elseif dim==2 && ismatrix(tmpA)
                % Sum over matrix rows
                for j=1:size(tmpA,2)
                    tmpY(:,j) = tmpA(:,j) - tmpV(j);
                end
            else
                % Sum over arbitrary dimension of N-D array
                siz = size(tmpA);
                if dim>1
                    prodLowDims=prod(siz(1:dim-1)); 
                else
                    prodLowDims=1; 
                end
                if dim<ndims(tmpA)
                    prodHighDims=prod(siz(dim+1:end));
                else
                    prodHighDims=1; 
                end
                n = 1;
                for i=1:prodHighDims
                    for j=1:length(tmpV)
                        for k=1:prodLowDims
                            tmpY(n) = tmpA(n) - tmpV(j);
                            n = n + 1;
                        end
                    end
                end
            end
            
            if ~isempty(coder.target) || isFixPtA || isFixPtV
                Y = removefimath(tmpY);
            else
                Y = tmpY;
            end
        end % stepImpl
        
    end % methods (Access=protected)
    
    methods(Static, Access=protected)
        function groups = getPropertyGroupsImpl
            %getPropertyGroupsImpl   Return property groups for object display
            
            main = matlab.system.display.SectionGroup('TitleSource', 'Auto', ...
                'PropertyList', {'Dimension', 'VectorSource', 'Vector'});
            datatypes = matlab.system.display.SectionGroup(...
                'Title', 'Data Types', ...
                'PropertyList', {'FullPrecisionOverride', 'RoundingMethod', ...
                'OverflowAction', 'VectorDataType', 'CustomVectorDataType', ...
                'AccumulatorDataType', 'CustomAccumulatorDataType'...
                'OutputDataType', 'CustomOutputDataType'});
            
            groups = [main, datatypes];
        end
    end
   methods(Static)
        function helpFixedPoint
            %helpFixedPoint Display dsp.ArrayVectorSubtractor System object fixed-point
            %               information
            %   dsp.ArrayVectorSubtractor.helpFixedPoint displays information about
            %   fixed-point properties and operations of the
            %   dsp.ArrayVectorSubtractor System object.
            
            matlab.system.dispFixptHelp('dsp.ArrayVectorSubtractor', ...
                dsp.ArrayVectorSubtractor.getDisplayFixedPointPropertiesImpl);
        end
    end % methods, static
        
    methods(Static, Hidden)
        function a = getAlternateBlock
            a = 'dspmtrx3/Array-Vector Subtract';
        end
        
        function  props = getDisplayPropertiesImpl()
            props = {...
                'Dimension' ...
                'VectorSource' ...
                'Vector' ...
                };
        end
        
        function props = getDisplayFixedPointPropertiesImpl()
            props = {...
                'FullPrecisionOverride', ...
                'RoundingMethod',  'OverflowAction' ...
                'VectorDataType', 'CustomVectorDataType' ...
                'AccumulatorDataType',  'CustomAccumulatorDataType' ...
                'OutputDataType', 'CustomOutputDataType' ...
                };
        end
    end % methods static, hidden
end % classdef ArrayVectorSubtractor
