classdef (StrictDefaults)FrequencyDomainFIRFilter < matlab.System...
     & dsp.enhancedsim.UpdateSysObjBuildInfo
%   FrequencyDomainFIRFilter Filter input signal in the frequency domain.
%   fdf = dsp.FrequencyDomainFIRFilter returns a System object, fdf, which
%   independently filters each channel of the input over time using the 
%   overlap-save or overlap-add method.
%
%   fdf = dsp.FrequencyDomainFIRFilter('Name', Value, ...) returns a
%   frequency domain filter System object, fdf, with each specified
%   property name set to the specified value. You can specify additional
%   name-value pair arguments in any order as (Name1,Value1,...,NameN,
%   ValueN).
%
%   fdf = dsp.FrequencyDomainFIRFilter(num) returns a frequency domain
%   filter object, fdf, with the Numerator property set to num.
%
%   Step method syntax:
%
%   Y = step(fdf, X) computes the filtered output, Y, of the input X. Each
%   column of X is a channel. The System object filters each channel of the
%   input signal independently over time.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   FrequencyDomainFIRFilter methods:
%
%   step                     - Filter the input signal in the frequency
%                              domain
%   release                  - Allow property value and input 
%                              characteristics changes, and release filter
%                              resources
%   clone                    - Create filter object with same
%                              property values
%   isLocked                 - Display locked status (logical)
%   reset                    - Reset the states of the filter
%   fvtool                   - Visualize the numerator's frequency response
%
%   FrequencyDomainFIRFilter properties:
%
%   Method                     - Frequency-domain filter method
%   NumeratorDomain            - Numerator domain (Time or Frequency)
%   Numerator                  - Filter coefficients
%   FrequencyResponse          - Filter frequency response
%   FFTLength                  - FFT length
%   NumeratorLength            - Time-domain numerator length
%   FilterIsReal               - Filter is real
%   PartitionForReducedLatency - Partition numerator to reduce latency
%   PartitionLength            - Numerator partition length
%   Latency                    - Filter latency (read-only)
%
%   % Example 1: Filter input signal using overlap-add, and compare to the
%   % output of a time-domain FIR filter.
%
%   % Design FIR lowpass filter coefficients:
%   impL    = 400;
%   Fs      = 8000;
%   Fcutoff = 2000;
%   imp     = fir1(impL,2 * Fcutoff / Fs);
% 
%   oa  = dsp.FrequencyDomainFIRFilter(imp,'Method','overlap-add');
%   fprintf('Frequency domain filter latency is %d samples\n',oa.Latency);                            
%   fir = dsp.FIRFilter('Numerator',imp);
%   % Delay the FIR output by the frequency-domain filter's latency.
%   dly = dsp.Delay('Length',oa.Latency);
% 
%   numFrames = 1e4;
%   frameLen  = 256;
% 
%   sin_100Hz = dsp.SineWave('Frequency',100,'SampleRate',Fs,...
%                            'SamplesPerFrame',frameLen);
%   sin_3KHz  = dsp.SineWave('Frequency',3e3,'SampleRate',Fs,...
%                            'SamplesPerFrame',frameLen);
%
%   ts = timescope('TimeSpanSource','property',...
%                  'TimeSpan',5 * frameLen/Fs,...
%                  'YLimits',[-1.1 1.1],...
%                  'ShowLegend',true,...
%                  'SampleRate',Fs,...
%                  'ChannelNames',{'Overlap-add','Direct-form FIR'});
%
%   for idx = 1:numFrames
%     x = sin_100Hz() + sin_3KHz() + 0.01 * randn(frameLen,1);
%     y1 = oa(x);
%     y2 = fir(dly(x));
%     ts([y1,y2]);
%   end
%
%   % Example 2: Use overlap-save to pass an input signal through an FIR 
%   % filter of length 3000. Partition numerator to reduce latency. 
%   % Compare to the output of a time-domain FIR filter.
%
%   % Design FIR lowpass filter coefficients:
%   impL    = 2999;
%   Fs      = 8000;
%   Fcutoff = 2000;
%   imp     = fir1(impL,2 * Fcutoff / Fs);
% 
%   os  = dsp.FrequencyDomainFIRFilter(imp,'Method','overlap-save',...
%                                    'PartitionForReducedLatency',true,...
%                                    'PartitionLength',256);
%   fprintf('Frequency domain filter latency is %d samples\n',os.Latency);                            
%   fir = dsp.FIRFilter('Numerator',imp);
%   dly = dsp.Delay('Length',os.Latency);
% 
%   numFrames = 1e4;
%   frameLen  = 256;
% 
%   sin_100Hz = dsp.SineWave('Frequency',100,'SampleRate',Fs,...
%                            'SamplesPerFrame',frameLen);
%   sin_3KHz  = dsp.SineWave('Frequency',3e3,'SampleRate',Fs,...
%                            'SamplesPerFrame',frameLen);
%
%   ts = timescope('TimeSpanSource','property',...
%                  'TimeSpan',5 * frameLen/Fs,...
%                  'YLimits',[-1.1 1.1],...
%                  'ShowLegend',true,...
%                  'SampleRate',Fs,...
%                  'ChannelNames',{'Overlap-save','Direct-form FIR'});
%
%   for idx = 1:numFrames
%       x = sin_100Hz() + sin_3KHz() + .1 * randn(frameLen,1);
%       y1 = os(x);
%       y2 = fir(dly(x));
%       ts([y1,y2]);
%   end
%
%   % Example 3: Specify the filter's frequency response. Filter input
%   % signal using overlap-add, and compare to the output of a 
%   % time-domain FIR filter.
%
%   % Design FIR lowpass filter coefficients:
%   impL    = 400;
%   Fs      = 8000;
%   Fcutoff = 2000;
%   imp     = fir1(impL,2 * Fcutoff / Fs);
% 
%   H   = fft(imp , 2 * numel(imp));
%   oa  = dsp.FrequencyDomainFIRFilter('NumeratorDomain','Frequency',...
%                                      'FrequencyResponse', H,...
%                                      'NumeratorLength',numel(imp),...
%                                      'Method','overlap-add');
%   fprintf('Frequency domain filter latency is %d samples\n',oa.Latency);                            
%   fir = dsp.FIRFilter('Numerator',imp);
%   % Delay the FIR output by the frequency-domain filter's latency.
%   dly = dsp.Delay('Length',oa.Latency);
% 
%   numFrames = 1e4;
%   frameLen  = 256;
% 
%   sin_100Hz = dsp.SineWave('Frequency',100,'SampleRate',Fs,...
%                            'SamplesPerFrame',frameLen);
%   sin_3KHz  = dsp.SineWave('Frequency',3e3,'SampleRate',Fs,...
%                            'SamplesPerFrame',frameLen);
%
%   ts = timescope('TimeSpanSource','property',...
%                  'TimeSpan',5 * frameLen/Fs,...
%                  'YLimits',[-1.1 1.1],...
%                  'SampleRate',Fs,...
%                  'ChannelNames',{'Overlap-add','Direct-form FIR'});
%
% 
%   for idx = 1:numFrames
%     x = sin_100Hz() + sin_3KHz() + 0.01 * randn(frameLen,1);
%     y1 = oa(x);
%     y2 = fir(dly(x));
%     ts([y1,y2]);
%   end
%
%   See also dsp.FIRFilter
    
% Copyright 2017-2022 The MathWorks, Inc.
    
%#codegen
%#ok<*EMCLS>
  
    properties
        % Numerator Filter coefficients
        %  Specify the filter coefficients as a real or complex numeric row
        %  vector. This property is tunable. This property is applicable
        %  when NumeratorDomain is 'Time'. The default value of this
        %  property is fir1(100,0.3).
        Numerator = fir1(100,0.3);
        % FrequencyResponse Frequency response 
        % Specify the filter frequency response. When
        % PartitionForReducedLatency is false, FrequencyResponse must be a
        % row vector (the FFT length is equal to the length of the vector).
        % When PartitionForReducedLatency is true, FrequencyResponse must
        % be a 2*P-by-N, where P is the partition size, and N is the number
        % partitions. This property is applicable when NumeratorDomain is
        % 'Frequency'. The default value of this property is
        % fft(fir1(100,0.3) , 202). This property is tunable.
        FrequencyResponse = fft(fir1(100,0.3) , 202);
    end

    properties (Nontunable)
        % NumeratorLength Time-domain numerator length
        % Specify the time-domain numerator length as a positive scalar.
        % This property is applicable when NumeratorDomain is 'Frequency'.
        % The default value of this property is 101.
        NumeratorLength (1, 1) {mustBePositive, mustBeInteger} = 101; 
    
        % NumeratorDomain Numerator domain
        % Specify the numerator domain as 'Time' or 'Frequency'. When
        % NumeratorDomain is 'Time', you specify the time-domain filter
        % numerator in the Numerator property. When NumeratorDomain is
        % 'Frequency', you specify the filter's frequency response in the
        % FrequencyResponse property. The default is 'Time'.
        NumeratorDomain = 'Time';
        % Method Frequency-domain filter method
        %  Specify the frequency-domain filter method as 'Overlap-save' or
        %  'Overlap-add'. The default is 'Overlap-save'.
        Method = 'Overlap-save';
        % FFTLength FFT length
        %  Specify the FFT length as a positive scalar integer. The default
        %  value of this property is [], which means that the FFT length is
        %  equal to twice the numerator length. This property applies when
        %  PartitionForReducedLatency is false and NumeratorDomain is
        %  'Time'.
        FFTLength       = [];
    
        % FilterIsReal Filter is real
        % Specify if the filter coefficients are all real. This property
        % applies when NumeratorDomain is 'Frequency'. The default is true.
        FilterIsReal (1, 1) logical = true;
    
        % PartitionLength Numerator partition length
        %  Specify the numerator partition length as a positive scalar
        %  integer less than or equal to the length of the numerator. The
        %  default is 32. This property applies when
        %  PartitionForReducedLatency is true and NumeratorDomain is
        %  'Time'.
        PartitionLength (1, 1) {mustBePositive, mustBeInteger}   = 32; 
    end
    
    properties (Nontunable, Dependent, SetAccess = private)
        % Latency Filter latency
        %   Use this read-only property to access the filter latency, in
        %   samples. When PartitionForReducedLatency is false, the latency
        %   is equal to FFTLength - length(Numerator) + 1. When
        %   PartitionForReducedLatency is true, the latency is equal to
        %   the partition length.
       Latency; 
    end
    
    properties(Access = protected, Nontunable)
        % pNumeratorLength Numerator length
        pNumeratorLength 
    end
    
    properties (Nontunable)
        % PartitionForReducedLatency Partition numerator to reduce latency
        %   When PartitionForReducedLatency is false, the filter uses the
        %   traditional overlap-save or overlap-add method. The latency in
        %   that case is FFTLength - length(Numerator) + 1. To reduce the
        %   latency, set PartitionForReducedLatency to true. In this mode,
        %   the numerator is partitioned into segments with length
        %   specified by the PartitionLength property. The filter performs
        %   overlap-save or overlap-add on each partition, and combines the
        %   partial results to form the overall output. The latency is now
        %   reduced to the partition length.
       PartitionForReducedLatency (1, 1) logical = false; 
    end
    
    properties(Constant, Hidden)
        MethodSet = matlab.system.StringSet( { ...
            'Overlap-save', ...
            'Overlap-add'} );
        
       NumeratorDomainSet = matlab.system.StringSet( { ...
            'Time', ...
            'Frequency'} );
    end
    
    properties(Access = protected)
        % pFilter Frequency-domain filter
        pFilter
    end
    
    properties(Access = private)
        % NumChannels Cache number of input channels
        NumChannels = int32(-1);
    end
    
     properties(Access = private, Nontunable)
        % pFFTLength Cache FFT length
        pFFTLength
        % pDatatype Cache datatype
        pDatatype;
     end

    methods
        function obj = FrequencyDomainFIRFilter(varargin)
            setProperties(obj,nargin,varargin{:},'Numerator');
        end
        
        function set.FFTLength(obj,val)
            if isempty(val)
                validateattributes(val, {'numeric'},{},...
                    '','FFTLength');
            else
                validateattributes(val, {'numeric'},{'finite','scalar','integer','real','positive'},...
                    '','FFTLength');
            end
            obj.FFTLength = double(val);
        end
        
        function set.PartitionLength(obj,val)
            validateattributes(val, {'numeric'},{'finite','scalar','integer','real','positive'},...
                '','PartitionLength');
            obj.PartitionLength = double(val);
        end
        
        function set.Numerator(obj, value)
            validateattributes(value, ...
                {'numeric'}, {'finite', 'nonempty', 'row'}, '', 'Numerator');
            obj.Numerator = value;
        end
        
        function set.FrequencyResponse(obj, value)
            validateattributes(value, ...
                {'numeric'}, {'finite', 'nonempty','2d'}, '', 'FrequencyResponse');
            obj.FrequencyResponse = value;
        end
        
        function lat = get.Latency(obj)
            if strcmp(obj.NumeratorDomain,'Time')
                if obj.PartitionForReducedLatency
                    l = obj.PartitionLength;
                else
                    if isempty(obj.FFTLength)
                        fftlen =  2*numel(obj.Numerator);
                    else
                        fftlen = obj.FFTLength;
                    end
                    l = fftlen - numel(obj.Numerator) + 1;
                end
            else
                if obj.PartitionForReducedLatency
                    l = size(obj.FrequencyResponse , 1) / 2;
                else
                    fftlen = size(obj.FrequencyResponse , 2);
                    l      = fftlen - obj.NumeratorLength + 1;
                end
            end
            lat = uint32(l);
        end
        
        function varargout = fvtool(obj,varargin)
            % Visualize the filter's frequency response. Visualize the
            % magnitude response, phase response, group delay, impulse 
            % response, or pole-zero plot.
            %
            % Example
            % h    = firpm(25,[0 .4 .5 1],[0 0 1 1],'h');
            % filt = dsp.FrequencyDomainFIRFilter(h);
            % fvtool(filt)
            
            % Convert string inputs to char inputs
            [varargin{:}] = convertStringsToChars(varargin{:});
      
            if strcmp(obj.NumeratorDomain,'Time') 
               coeffs = obj.Numerator;
            else
                if obj.PartitionForReducedLatency
                     coeffs = ifft(obj.FrequencyResponse);
                     coeffs = coeffs(1:size(obj.FrequencyResponse,1)/2,:);
                     coeffs = coeffs(:).';
                else
                    coeffs = ifft(obj.FrequencyResponse);
                    coeffs = coeffs(1:obj.NumeratorLength);
                end
            end
            
            filt = dsp.FIRFilter('Numerator',coeffs);
            
            if isLocked(obj)
                filt(cast(1,obj.pDatatype));
            end
            
            if nargout > 0
                varargout{1} = fvtool(filt,varargin{:});
            else
                fvtool(filt,varargin{:});
            end
        end
    end
    
    methods(Access = protected)
        
        function flag = isPartitioned(obj)
             flag = obj.PartitionForReducedLatency;
        end
        
        function validateProps(obj)
            if strcmp(obj.NumeratorDomain,'Time')
                coder.internal.errorIf(obj.PartitionForReducedLatency && obj.PartitionLength > numel(obj.Numerator), ...
                    'dsp:system:FrequencyDomainFIRFilter:InvalidPartitionLength');
                coder.internal.errorIf(~obj.PartitionForReducedLatency && ~obj.inheritFFT && (isempty(obj.FFTLength) || obj.FFTLength < numel(obj.Numerator)), ...
                    'dsp:system:FrequencyDomainFIRFilter:InvalidFFTLength');
            else
                coder.internal.errorIf(...
                    obj.PartitionForReducedLatency && mod(size(obj.FrequencyResponse,1),2)==1, ...
                    'dsp:system:FrequencyDomainFIRFilter:FrequencyResponseSizePartition');
                
                coder.internal.errorIf(~obj.PartitionForReducedLatency && numel(obj.FrequencyResponse) < obj.NumeratorLength, ...
                    'dsp:system:FrequencyDomainFIRFilter:InvalidFFTLengthFreqDomain');
                coder.internal.errorIf(~obj.PartitionForReducedLatency && (size(obj.FrequencyResponse,1) > 1), ...
                    'dsp:system:FrequencyDomainFIRFilter:FrequencyResponseSizeNoPartition');

            end
        end
        
        function validatePropertiesImpl(obj)
            validateProps(obj);
        end
        
        function flag = inheritFFT(obj)
            flag = isempty(obj.FFTLength);
        end

        function numchans = getNumChannels(~,u)
            % GETNUMCHANNELS Return the number of input channels
            numchans = size(u,2);
        end
        
        function pnum = getNumeratorPrototype(obj)
            if strcmp(obj.NumeratorDomain,'Time')
                pnum =  obj.Numerator;
            else
                pnum =  obj.FrequencyResponse;
            end
        end
        
        function setupImpl(obj,u)
            
            %  Get the number of input channels
            numchans = getNumChannels(obj,u);
            
            if strcmp(obj.NumeratorDomain,'Time')
                obj.pNumeratorLength = numel(obj.Numerator);
            else
                obj.pNumeratorLength = numel(obj.FrequencyResponse);
            end
            
            % Disallow a first step with zero channels
            coder.internal.errorIf(numchans == 0,'dsp:system:Shared:EmptyFirstStep');
            
            obj.NumChannels = int32(numchans);

            obj.pDatatype = class(u);
            
            num = getNumeratorPrototype(obj);
            if strcmp(obj.NumeratorDomain,'Time')
                if ~obj.PartitionForReducedLatency
                    if  inheritFFT(obj)
                        obj.pFFTLength =  2 * numel(num);
                    else
                        obj.pFFTLength =  obj.FFTLength;
                    end
                end
            else
                obj.pFFTLength = size(num , 2);
            end
            
            % Select frequency-domain filter object
            if strcmp(obj.Method,'Overlap-save')
                if ~isPartitioned(obj)
                    obj.pFilter = dsp.internal.NoPartitionOS('Domain',obj.NumeratorDomain,...
                        'Numerator',num,...
                        'FFTLength',obj.pFFTLength,...
                        'NumeratorLength',obj.NumeratorLength,...
                        'FilterIsReal',obj.FilterIsReal);
                else
                    obj.pFilter = dsp.internal.PartitionOS('Domain',obj.NumeratorDomain,...
                        'Numerator',num,...
                        'PartitionLength',obj.PartitionLength,...
                        'FilterIsReal',obj.FilterIsReal);
                end
            else
                if ~isPartitioned(obj)
                    obj.pFilter = dsp.internal.NoPartitionOA('Domain',obj.NumeratorDomain,...
                        'Numerator',num,...
                        'FFTLength',obj.pFFTLength,...
                        'NumeratorLength',obj.NumeratorLength,...
                        'FilterIsReal',obj.FilterIsReal);
                else
                    obj.pFilter = dsp.internal.PartitionOA('Domain',obj.NumeratorDomain,...
                        'Numerator',num,...
                        'PartitionLength',obj.PartitionLength,...
                        'FilterIsReal',obj.FilterIsReal);
                end
            end
           
           setNumchans(obj.pFilter , numchans);
           setup(obj.pFilter,u);
        end

        function y = stepImpl(obj,u)
            y = obj.pFilter(u);
        end

        function resetImpl(obj)
            reset(obj.pFilter);
        end
        
        function releaseImpl(obj)
            release(obj.pFilter);
            obj.NumChannels = int32(-1);
        end
        

        function flag = isInputSizeMutableImpl(~,~)
            flag = true;
        end

        function flag = isInactivePropertyImpl(obj, prop)
            flag = false;
            switch prop
                case 'FFTLength'
                    flag = strcmp(obj.NumeratorDomain,'Frequency') || isPartitioned(obj);
                case 'PartitionLength'
                    flag = ~obj.PartitionForReducedLatency || strcmp(obj.NumeratorDomain,'Frequency') ;
                case {'FilterIsReal','FrequencyResponse'}
                    flag =  strcmp(obj.NumeratorDomain,'Time');
                case 'NumeratorLength'
                    flag =  strcmp(obj.NumeratorDomain,'Time') || isPartitioned(obj);
                 case 'Numerator'
                     flag =  strcmp(obj.NumeratorDomain,'Frequency');
            end
        end
        
        function processTunedPropertiesImpl(obj)
            coder.internal.errorIf(strcmp(obj.NumeratorDomain,'Time') && obj.pNumeratorLength ~= numel(obj.Numerator),'dsp:system:FrequencyDomainFIRFilter:ChangedNumLength');
            if strcmp(obj.NumeratorDomain,'Time')
                obj.pFilter.Numerator = obj.Numerator;
            else
                obj.pFilter.Numerator = obj.FrequencyResponse;
            end
        end
        
        function s = saveObjectImpl(obj)
            s = saveObjectImpl@matlab.System(obj);
            if isLocked(obj)
                s.pFilter      = matlab.System.saveObject(obj.pFilter);
                s.NumChannels  = obj.NumChannels;
                s.pFFTLength   = obj.pFFTLength;
                s.pDatatype    = obj.pDatatype;
                s.pNumeratorLength = obj.pNumeratorLength;
            end
        end
        
        function s = loadObjectImpl(obj, s, wasLocked)
            if wasLocked
               obj.pFilter      = matlab.System.loadObject(s.pFilter);
               obj.NumChannels  = s.NumChannels;
               obj.pFFTLength   = s.pFFTLength;
               obj.pDatatype    = s.pDatatype;
               obj.pNumeratorLength = s.pNumeratorLength;
            end
            loadObjectImpl@matlab.System(obj, s, wasLocked);
        end
        
        function validateInputsImpl(obj,u)
            validateattributes(u, {'single', 'double'}, {'2d'},'','')
            if obj.NumChannels == -1 || obj.NumChannels == 0 || size(u,2) == 0
                return;
            end
            coder.internal.errorIf(size(u,2) ~= obj.NumChannels, ...
                'dsp:system:Shared:numChannels');
        end
    end
    
    methods(Static, Access=protected)
        function group = getPropertyGroupsImpl
            % Modify order of display
            group =  matlab.system.display.Section(...
                'Title', 'Parameters', ...
                'PropertyList', {'Method','NumeratorDomain','Numerator',...
                'FrequencyResponse','NumeratorLength','FilterIsReal'...
                'PartitionForReducedLatency','FFTLength',...
                'PartitionLength','Latency'},...
                'DependOnPrivatePropertyList', {'Latency'});
        end
    end 
    
    methods (Static)
        function H = partitionNumerator(NUM,P)
            % partitionNumerator Partition numerator.
            % H = partitionNumerator(NUM,P) partitions the numerator, NUM,
            % using a partition length P. H is a 2*P-by-N matrix, where N
            % is the number of partitions. NUM must be a row vector.
            %
            % % Example:
            % h = fir1(299,.3); 
            % H = dsp.FrequencyDomainFIRFilter.partitionNumerator(h,30);
            N = ceil(length(NUM) / P);
            pad = (N * P) - length(NUM);
            NUM = [NUM  zeros(1,pad,'like',NUM)];
            H = dsp.internal.PartitionOAOSBase.getIRFreqDomain(P, N, NUM, 2*P , class(NUM));
        end
    end
end
