classdef (StrictDefaults)PeakToPeak < matlab.System 
%PeakToPeak Compute peak-to-peak values in an input or sequence of inputs
%   PTP = dsp.PeakToPeak returns a System object, PTP, that computes the 
%   difference between the maximum and minimum value in an input or a
%   sequence of inputs.
%
%   PTP = dsp.PeakToPeak('PropertyName', PropertyValue, ...) returns a
%   PeakToPeak System object, PTP, with each specified property set to
%   the specified value.
%
%   Step method syntax:
%
%   Y = step(PTP, X) computes the peak-to-peak value, Y, of real floating
%   point input vector X. When the RunningPeakToPeak property is true, Y
%   corresponds to the peak-to-peak value of the input elements over
%   successive calls to the step method.
%
%   Y = step(PTP, X, R) computes the peak-to-peak value of the input
%   elements over successive calls to the step method. The object
%   optionally resets its state based on the reset input signal, R, and the
%   ResetCondition property. This is possible when you set both the
%   RunningPeakToPeak and the ResetInputPort properties to true.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   PeakToPeak methods:
%
%   step     - See above description for use of this method
%   release  - Allow property value and input characteristics changes
%   clone    - Create PeakToPeak object with same property values
%   isLocked - Locked status (logical)
%   reset    - Reset the states of running peak-to-peak ratio
%
%   PeakToPeak properties:
%
%   RunningPeakToPeak    - Calculation over successive calls to step method
%   ResetInputPort       - Enables resetting in running peak-to-peak mode
%   ResetCondition       - Reset condition for running peak-to-peak mode
%   Dimension            - Dimension to operate along
%   CustomDimension      - Numerical dimension to operate along
%
%   % EXAMPLE #1: Determine the peak-to-peak value of a vector input
%      in1 = (1:10)';
%      pp1 = dsp.PeakToPeak;
%      y1  = pp1(in1)
%
%   % EXAMPLE #2: Determine the peak-to-peak value of a matrix input
%      in2 = magic(4);
%      pp2 = dsp.PeakToPeak;
%      pp2.Dimension = 'All';
%      y2  = pp2(in2)
%
%   See also dsp.PeakToRMS

%   Copyright 2008-2021 The MathWorks, Inc.

%#codegen
%#ok<*EMCLS>

  properties (Nontunable)
    %ResetCondition Reset condition for running peak-to-peak mode
    %   Specify the event to reset the running peak-to-peak as one of
    %   'Rising edge' | 'Falling edge' | 'Either edge' | 'Non-zero', where
    %   the default is 'Non-zero'. This property is applicable when the
    %   ResetInputPort property is true. 
    ResetCondition = 'Non-zero'
    %Dimension Dimension to operate along    
    %   Specify the dimension along which to calculate the peak-to-peak
    %   ratio as one of 'All' | 'Row' | 'Column' | 'Custom', where the
    %   default is 'Column'. This property is applicable when the
    %   RunningPeakToPeak property is false. 
    Dimension = 'Column'
  
    %CustomDimension Numerical dimension to operate along
    %   Specify the dimension of the input signal along which the
    %   peak-to-peak ratio is computed as a positive integer. The value of
    %   this property cannot exceed the number of dimensions in the input
    %   signal. This property is applicable when Dimension property is
    %   'Custom'. The default value of this property is 1.
    CustomDimension (1, 1) {mustBePositive, mustBeInteger} = 1

    %RunningPeakToPeak Calculation over successive calls to step method
    %   Set this property to true to enable the calculation of the
    %   peak-to-peak ratio over successive calls to the step method. The
    %   default value of this property is false.
    RunningPeakToPeak (1, 1) logical = false
    %ResetInputPort Enables resetting in running peak-to-peak mode 
    %   Set this property to true to enable resetting the running
    %   peak-to-peak. When the property is set to true, a reset input must
    %   be specified to the step method to reset the running peak-to-peak
    %   value. This property is applicable when the RunningPeakToPeak
    %   property is true. The default value of this property is false.
    ResetInputPort (1, 1) logical = false
  end


  properties(Constant, Hidden)
    ResetConditionSet = dsp.CommonSets.getSet('ResetCondition');
    DimensionSet = dsp.CommonSets.getSet('Dimension');
  end

  properties(Access = private)
    cMax
    cMin
  end
  
  methods
    function obj = PeakToPeak(varargin)
      coder.internal.warning('dsp:system:PeakToPeak_NotSupported');
      setProperties(obj, nargin, varargin{:});
    end

  end

  methods(Access=protected)

    function flag = isInactivePropertyImpl(obj, prop)
      if obj.RunningPeakToPeak
        flag = strcmp(prop,'Dimension') ...
            || strcmp(prop,'CustomDimension') ...
            || ~obj.ResetInputPort && strcmp(prop,'ResetCondition');
      else
        flag = strcmp(prop,'ResetInputPort') ...
            || strcmp(prop,'ResetCondition') ...
            || ~strcmp(obj.Dimension, 'Custom') ...
                && strcmp(prop,'CustomDimension');
      end
    end
    

    function flag = isInputSizeMutableImpl(~,~)
      flag = true;
    end
    
    function num = getNumInputsImpl(obj)
      num = 1 + (obj.RunningPeakToPeak && obj.ResetInputPort);
    end
    
    function validateInputsImpl(obj, varargin)
        
      coder.internal.errorIf(~isreal(varargin{1}),...
                             'measure:PeakToPeak:inDataTypeNotReal');

      coder.internal.errorIf(~isfloat(varargin{1}),...
                             'measure:PeakToPeak:inDataTypeNotFloat');

      coder.internal.errorIf(~isInactivePropertyImpl(obj,'CustomDimension') && ndims(varargin{1}) < obj.CustomDimension   ,...
                             'measure:PeakToPeak:inSigDimNotSetting');
    
      coder.internal.errorIf(nargin>2 && (~isscalar(varargin{2}) || ~isreal(varargin{2}))   ,...
                             'measure:PeakToPeak:inputMustBeRealScalar','R');

     coder.internal.errorIf( obj.RunningPeakToPeak && numel(size(varargin{1})) > 2,...
                             'measure:PeakToPeak:inSigMustBe2DForFrames');
              
    end
    
    function setupImpl(obj, varargin)      
      if obj.RunningPeakToPeak
        if obj.ResetInputPort
          obj.cMax = dsp.internal.Maximum( ...
                 'RunningMaximum', true, ...
                 'ResetInputPort', true, ...
                 'ResetCondition', obj.ResetCondition);
          obj.cMin = dsp.internal.Minimum( ...
                 'RunningMinimum', true, ...
                 'ResetInputPort', true, ...
                 'ResetCondition', obj.ResetCondition);
        else
          obj.cMax = dsp.internal.Maximum( ...
                 'RunningMaximum', true);
          obj.cMin = dsp.internal.Minimum( ...
                 'RunningMinimum', true);
        end
      else
        if strcmp(obj.Dimension,'Custom')
          obj.cMax = dsp.internal.Maximum('Dimension',obj.Dimension, ...
                                 'CustomDimension',obj.CustomDimension);
          obj.cMin = dsp.internal.Minimum('Dimension',obj.Dimension, ...
                                 'CustomDimension',obj.CustomDimension);
        else
          obj.cMax = dsp.internal.Maximum('Dimension',obj.Dimension);
          obj.cMin = dsp.internal.Minimum('Dimension',obj.Dimension);
        end
      end
    end

    function y = stepImpl(obj, varargin)
      max = step(obj.cMax, varargin{:});
      min = step(obj.cMin, varargin{:});
      y = max - min;
    end
    
    function resetImpl(obj)
        reset(obj.cMax);
        reset(obj.cMin);
    end
    
    function releaseImpl(obj)
        release(obj.cMax);
        release(obj.cMin);
    end
    
    function s = saveObjectImpl(obj)
        s = saveObjectImpl@matlab.System(obj);
        if isLocked(obj)
            s.cMax      = matlab.System.saveObject(obj.cMax);
            s.cMin      = matlab.System.saveObject(obj.cMin);
        end
    end
    
    function loadObjectImpl(obj, s, wasLocked)
        if wasLocked
            obj.cMax     =  matlab.System.loadObject(s.cMax);
            obj.cMin      =  matlab.System.loadObject(s.cMin);
        end
        % Call the base class method
        loadObjectImpl@matlab.System(obj, s);
    end
    
  end

  methods(Static,Hidden,Access=protected)
    function groups = getPropertyGroupsImpl
      props = { ...
        'RunningPeakToPeak', ...
        'ResetInputPort', ...
        'ResetCondition', ...
        'Dimension', ...
        'CustomDimension', ...
        };
      groups = matlab.system.display.Section('Title', 'Parameters',...
                                             'PropertyList', props);
    end
  end
end

