classdef AsyncBuffer< handle & matlab.mixin.SetGet
%AsyncBuffer FIFO buffer
%   Q = dsp.AsyncBuffer returns an object, Q, that implements first in
%   first out (FIFO) buffering.
%
%   Q = dsp.AsyncBuffer(Cap) returns an AsyncBuffer object, Q, with
%   capacity set to Cap.
%
%   Q = dsp.AsyncBuffer('Name',Value, ...) returns an AsyncBuffer
%   object, Q, with each specified property name set to the specified
%   value. You can specify additional name-value pair arguments in any
%   order as (Name1,Value1,...,NameN,ValueN).
%
%
%   AsyncBuffer methods:
%
%   write    - Write to buffer
%   read     - Read from buffer
%   peek     - Peek at buffer
%   reset    - Reset the internal states to initial conditions
%   release  - Allow property values and input characteristics 
%              to change
%   clone    - Create AsyncBuffer object with same property values
%   isLocked - Locked status (logical)
%   info     - Get cumulative overrun and underrun
%
%   AsyncBuffer properties:
%
%   Capacity           - Number of writable/readable samples in buffer
%   NumUnreadSamples   - Number of unread samples in buffer (read only)
%
%
%   % EXAMPLE 1: Read variable frame sizes from buffer
%     % Create AsyncBuffer object
%     Q = dsp.AsyncBuffer;
%
%     % Create input signal for buffer
%     signal = randn(512,1);
%
%     % Write input signal to buffer
%     write(Q,signal);
%
%     outTotal = [];
%     plot(signal);hold on
%
%     % Read from buffer until all samples are read
%     while Q.NumUnreadSamples ~= 0
%         % Determine random number of samples to read
%         numToRead = randi([1,64]);
%
%         % Read from buffer
%         out = read(Q,numToRead);
%
%         % Visualize data read from buffer
%         outTotal = [outTotal;out];
%         plot(outTotal,'r','LineWidth',2);
%         pause(0.2) % The pause is for illustrative purposes only
%     end
%
%   % EXAMPLE 2: Write variable frame sizes to buffer
%   % Create AsyncBuffer object
%   Q = dsp.AsyncBuffer;
%
%   % Create ArrayPlot object
%   plotter = dsp.ArrayPlot();
%
%   start = 1;
%   tic
%   while toc < 5
%       % Determine random number of samples to write
%       numToWrite = randi([200,800]);
%       finish = start + numToWrite;
%
%       % Create input data
%       inputData = sin(start:finish)';
%       start = finish + 1;
%
%       % Write data to buffer
%       write(Q,inputData);
%
%       % If enough data is cached, read from buffer and perform FFT
%       while Q.NumUnreadSamples >= 256
%           x = read(Q,256);
%           X = abs(fft(x));
%           plotter(log(X));
%       end
%   end
%
% See also dsp.Delay.

     
    % Copyright 2016-2021 The MathWorks, Inc.

    methods
        function out=AsyncBuffer
        end

        function out=clone(~) %#ok<STOUT>
        end

        function out=info(~) %#ok<STOUT>
            %INFO Get cumulative overrun and underrun
            %   CumulativeOverrun - Number of samples overrun per channel since
            %       last call to reset. The number of samples overrun is the
            %       number of unread samples overwritten.
            %
            %   CumulativeUnderrun - Number of samples underrun per channel
            %       since last call to reset. Underrun occurs if you attempt to
            %       read more samples than available.
        end

        function out=peek(~) %#ok<STOUT>
            %PEEK Read from buffer without changing the number of unread 
            %     samples
            % out = peek(Q) returns all unread samples in buffer.
            %
            % out = peek(Q,NumRows) returns NumRows samples from each
            %   channel (column) of buffer. If the requested number of
            %   samples is greater than the number of unread samples, the
            %   output is zero-padded.
            %
            % out = peek(Q,NumRows,Overlap) returns NumRows samples from
            %   each channel and overlaps previously read samples by
            %   Overlap. The total number of samples read is NumRows *
            %   (number of channels). The total number of new samples read
            %   is (NumRows - Overlap) * (number of channels). If the
            %   overlap portion read contains samples that are
            %   overwritten, and are therefore not contiguously written,
            %   the output is zero-padded.
            %
            % [out,nUnderrun] = peek(Q,...) additionally returns the
            %   number of rows zero-padded if underrun occurred. Underrun
            %   occurs if you attempt to read more samples than available.
            %   Samples zero-padded in overlapped portions are not counted
            %   as underrun.
            %
            % % Example: Peek at contents of buffer
            % % Create AsyncBuffer object
            %   Q = dsp.AsyncBuffer;
            % 
            % % Create input signal
            % signal = (1:100).';
            % 
            % %% Write Signal to buffer
            % write(Q,signal);
            % 
            % %% Peek at first 3 samples
            % out1  = peek(Q,3); % out1 = [1; 2; 3];
            % 
            % %% Read first 50 samples
            % out2  = read(Q,50); % out2 = (1:50).';
            % 
            % %% Now peek again at first 3 samples
            % out1  = peek(Q,3); % out1 = [51; 52; 53];
            % 
            % %% Read remaining samples
            % out2  = read(Q); % out2 = (51:100).';
        end

        function out=read(~) %#ok<STOUT>
            %READ Read from buffer
            % out = read(Q) returns all unread samples in buffer.
            %
            % out = read(Q,NumRows) returns NumRows samples from each
            %   channel (column) of buffer. If the requested number of
            %   samples is greater than the number of unread samples, the
            %   output is zero-padded.
            %
            % out = read(Q,NumRows,Overlap) returns NumRows samples from
            %   each channel and overlaps previously read samples by
            %   Overlap. The total number of samples read is NumRows *
            %   (number of channels). The total number of new samples read
            %   is (NumRows - Overlap) * (number of channels). If the
            %   overlap portion read contains samples that are
            %   overwritten, and are therefore not contiguously written,
            %   the output is zero-padded.
            %
            % [out,nUnderrun] = read(Q,...) additionally returns the
            %   number of rows zero-padded if underrun occurred. Underrun
            %   occurs if you attempt to read more samples than available.
            %   Samples zero-padded in overlapped portions are not counted
            %   as underrun.
        end

        function out=release(~) %#ok<STOUT>
            %  Allow property value and input characteristics to change
        end

        function out=reset(~) %#ok<STOUT>
            % Reset the buffer internal states
        end

        function out=setup(~) %#ok<STOUT>
        end

        function out=step(~) %#ok<STOUT>
        end

        function out=write(~) %#ok<STOUT>
            %WRITE Write to buffer
            % nOverrun = write(Q,x) writes the samples, x, to the
            %   buffering object, Q. It also returns the number of samples
            %   overrun in the current call to write. The number of samples
            %   overrun is the number of unread samples overwritten.
        end

    end
    properties
        Cache;

        %Capacity Number of writable/readable samples per channel in buffer
        % The default is 192000.
        Capacity;

        CumulativeOverrun;

        CumulativeUnderrun;

        LastReadUnderrun;

        NumChannels;

        %NumUnreadSamples Number of unread samples per channel in buffer
        % The total number of unread samples in the buffer is
        % NumUnreadSamples*(number of channels). This is a read-only
        % property.
        NumUnreadSamples;

        ReadPointer;

        WritePointer;

        isInitialized;

        % Save internal capacity as int32.
        pCapacity;

        % Cache number of channels
        pCompiledNumChannels;

    end
end
