classdef PeakToPeak< matlab.System
%PeakToPeak Compute peak-to-peak values in an input or sequence of inputs
%   PTP = dsp.PeakToPeak returns a System object, PTP, that computes the 
%   difference between the maximum and minimum value in an input or a
%   sequence of inputs.
%
%   PTP = dsp.PeakToPeak('PropertyName', PropertyValue, ...) returns a
%   PeakToPeak System object, PTP, with each specified property set to
%   the specified value.
%
%   Step method syntax:
%
%   Y = step(PTP, X) computes the peak-to-peak value, Y, of real floating
%   point input vector X. When the RunningPeakToPeak property is true, Y
%   corresponds to the peak-to-peak value of the input elements over
%   successive calls to the step method.
%
%   Y = step(PTP, X, R) computes the peak-to-peak value of the input
%   elements over successive calls to the step method. The object
%   optionally resets its state based on the reset input signal, R, and the
%   ResetCondition property. This is possible when you set both the
%   RunningPeakToPeak and the ResetInputPort properties to true.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   PeakToPeak methods:
%
%   step     - See above description for use of this method
%   release  - Allow property value and input characteristics changes
%   clone    - Create PeakToPeak object with same property values
%   isLocked - Locked status (logical)
%   <a href="matlab:help matlab.System/reset   ">reset</a>    - Reset the states of running peak-to-peak ratio
%
%   PeakToPeak properties:
%
%   RunningPeakToPeak    - Calculation over successive calls to step method
%   ResetInputPort       - Enables resetting in running peak-to-peak mode
%   ResetCondition       - Reset condition for running peak-to-peak mode
%   Dimension            - Dimension to operate along
%   CustomDimension      - Numerical dimension to operate along
%
%   % EXAMPLE #1: Determine the peak-to-peak value of a vector input
%      in1 = (1:10)';
%      pp1 = dsp.PeakToPeak;
%      y1  = pp1(in1)
%
%   % EXAMPLE #2: Determine the peak-to-peak value of a matrix input
%      in2 = magic(4);
%      pp2 = dsp.PeakToPeak;
%      pp2.Dimension = 'All';
%      y2  = pp2(in2)
%
%   See also dsp.PeakToRMS

 
%   Copyright 2008-2021 The MathWorks, Inc.

    methods
        function out=PeakToPeak
        end

        function out=getNumInputsImpl(~) %#ok<STOUT>
        end

        function out=isInactivePropertyImpl(~) %#ok<STOUT>
        end

        function out=isInputSizeMutableImpl(~) %#ok<STOUT>
        end

        function out=loadObjectImpl(~) %#ok<STOUT>
        end

        function out=releaseImpl(~) %#ok<STOUT>
        end

        function out=resetImpl(~) %#ok<STOUT>
        end

        function out=saveObjectImpl(~) %#ok<STOUT>
        end

        function out=setupImpl(~) %#ok<STOUT>
        end

        function out=stepImpl(~) %#ok<STOUT>
        end

        function out=validateInputsImpl(~) %#ok<STOUT>
        end

    end
    properties
        %CustomDimension Numerical dimension to operate along
        %   Specify the dimension of the input signal along which the
        %   peak-to-peak ratio is computed as a positive integer. The value of
        %   this property cannot exceed the number of dimensions in the input
        %   signal. This property is applicable when Dimension property is
        %   'Custom'. The default value of this property is 1.
        CustomDimension;

        %Dimension Dimension to operate along    
        %   Specify the dimension along which to calculate the peak-to-peak
        %   ratio as one of 'All' | 'Row' | 'Column' | 'Custom', where the
        %   default is 'Column'. This property is applicable when the
        %   RunningPeakToPeak property is false. 
        Dimension;

        %ResetCondition Reset condition for running peak-to-peak mode
        %   Specify the event to reset the running peak-to-peak as one of
        %   'Rising edge' | 'Falling edge' | 'Either edge' | 'Non-zero', where
        %   the default is 'Non-zero'. This property is applicable when the
        %   ResetInputPort property is true. 
        ResetCondition;

        %ResetInputPort Enables resetting in running peak-to-peak mode 
        %   Set this property to true to enable resetting the running
        %   peak-to-peak. When the property is set to true, a reset input must
        %   be specified to the step method to reset the running peak-to-peak
        %   value. This property is applicable when the RunningPeakToPeak
        %   property is true. The default value of this property is false.
        ResetInputPort;

        %RunningPeakToPeak Calculation over successive calls to step method
        %   Set this property to true to enable the calculation of the
        %   peak-to-peak ratio over successive calls to the step method. The
        %   default value of this property is false.
        RunningPeakToPeak;

    end
end
