classdef PeakToRMS< matlab.System
%PeakToRMS Compute peak-to-root-mean-square of the vector elements
%   PTR = dsp.PeakToRMS returns a System object, PTR, that
%   computes the ratio of the maximum magnitude (peak) to the
%   root-mean-square (RMS) value in an input or a sequence of inputs.
%
%   PTR = dsp.PeakToRMS('PropertyName', PropertyValue, ...) returns a
%   PeakToRMS System object, PTR, with each specified property set to
%   the specified value.
%
%   Step method syntax:
%
%   Y = step(PTR, X) computes the peak-to-RMS ratio, Y, of floating point
%   input vector X. When the RunningPeakToRMS property is true, Y
%   corresponds to the peak-to-RMS ratio of the input elements over
%   successive calls to the step method.
%
%   Y = step(PTR, X, R) computes the peak-to-RMS ratio of the input
%   elements over successive calls to the step method. The object
%   optionally resets its state based on the reset input signal, R, and the
%   ResetCondition property. This is possible when you set both the
%   RunningPeakToRMS and the ResetInputPort properties to true.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   PeakToRMS methods:
%
%   step     - See above description for use of this method
%   release  - Allow property value and input characteristics changes
%   clone    - Create PeakToRMS object with same property values
%   isLocked - Locked status (logical)
%   <a href="matlab:help matlab.System/reset   ">reset</a>    - Reset the states of running peak-to-RMS ratio
%
%   PeakToRMS properties:
%
%   RunningPeakToRMS     - Calculation over successive calls to step method
%   ResetInputPort       - Enables resetting in running peak-to-RMS mode
%   ResetCondition       - Reset condition for running peak-to-RMS mode
%   Dimension            - Dimension to operate along
%   CustomDimension      - Numerical dimension to operate along
%   DecibelScaledOutput  - Report output in decibels (dB)
%
%   % EXAMPLE #1: Determine the peak-to-RMS ratio of a vector input
%      in1 = (1:10)';
%      pr1 = dsp.PeakToRMS;
%      y1  = pr1(in1)
%
%   % EXAMPLE #2: Determine the peak-to-RMS ratio of a matrix input
%      in2 = magic(4);
%      pr2 = dsp.PeakToRMS;
%      pr2.Dimension = 'All';
%      y2  = pr2(in2)
%
%   See also dsp.PeakToRMS

 
%   Copyright 2008-2021 The MathWorks, Inc.

    methods
        function out=PeakToRMS
        end

        function out=getNumInputsImpl(~) %#ok<STOUT>
        end

        function out=isInactivePropertyImpl(~) %#ok<STOUT>
        end

        function out=isInputSizeMutableImpl(~) %#ok<STOUT>
        end

        function out=loadObjectImpl(~) %#ok<STOUT>
        end

        function out=releaseImpl(~) %#ok<STOUT>
        end

        function out=resetImpl(~) %#ok<STOUT>
        end

        function out=saveObjectImpl(~) %#ok<STOUT>
        end

        function out=setupImpl(~) %#ok<STOUT>
        end

        function out=stepImpl(~) %#ok<STOUT>
        end

        function out=validateInputsImpl(~) %#ok<STOUT>
        end

    end
    properties
        %CustomDimension Numerical dimension to operate along
        %   Specify the dimension as a positive integer, along which the
        %   peak-to-RMS ratio is computed. The value of this property cannot
        %   exceed the number of dimensions in the input signal. This property
        %   is applicable when Dimension property is 'Custom'. The default
        %   value of this property is 1.
        CustomDimension;

        %DecibelScaledOutput report output in decibels (dB)
        %   Set this property to true to enable output in dB. Set this
        %   property to false to report output as a ratio. The default value
        %   of this property is false.
        DecibelScaledOutput;

        %Dimension Dimension to operate along    
        %   Specify the dimension along which to calculate the peak-to-RMS
        %   ratio as one of 'All' | 'Row' | 'Column' | 'Custom', where the
        %   default is 'Column'. This property is applicable when the
        %   RunningPeakToRMS property is false.
        Dimension;

        %ResetCondition Reset condition for running peak-to-RMS mode
        %   Specify the event to reset the running peak-to-RMS as one of
        %   'Rising edge' | 'Falling edge' | 'Either edge' | 'Non-zero', where
        %   the default is 'Non-zero'. This property is applicable when the
        %   ResetInputPort property is true. 
        ResetCondition;

        %ResetInputPort Enables resetting in running peak-to-RMS mode 
        %   Set this property to true to enable resetting the running RMS. When
        %   the property is set to true, a reset input must be specified to the
        %   step method to reset the running peak-to-RMS ratio. This property
        %   is applicable when the RunningPeakToRMS property is true. The
        %   default value of this property is false.
        ResetInputPort;

        %RunningPeakToRMS Calculation over successive calls to step method
        %   Set this property to true to enable the calculation of the
        %   peak-to-RMS ratio over successive calls to the step method. The
        %   default value of this property is false.
        RunningPeakToRMS;

    end
end
