classdef Convolver< matlab.system.SFunSystem
%Convolver Compute convolution of two inputs
%   CNV = dsp.Convolver returns a convolution System object, CNV, that
%   convolves two inputs in the time domain or frequency domain.
%
%   CNV = dsp.Convolver('PropertyName',PropertyValue,...) returns a
%   convolution object, CNV, with each specified property set to the
%   specified value.
%
%   Step method syntax:
%
%   Y = step(CNV, A, B) convolves inputs A and B, and returns Y as the
%   convolved output.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   Convolver methods:
%
%   step     - See above description for use of this method
%   release  - Allow property value and input characteristics changes
%   clone    - Create convolution object with same property values
%   isLocked - Locked status (logical)
%
%   Convolver properties:
%
%   Method   - Domain for computing convolutions, time or frequency
%
%   This System object supports fixed-point operations when the property
%   Method is set to 'Time Domain'. For more information, type
%   dsp.Convolver.helpFixedPoint.
%
%   % EXAMPLE: Use the Convolver System object to convolve two rectangular
%   % sequences to generate a triangular sequence.
%   CONVOLVER = dsp.Convolver;
%   x = ones(10,1);
%   y = CONVOLVER(x,x);
%   figure, plot(y);    % triangular sequence
%
%   See also dsp.Crosscorrelator, dsp.Convolver.helpFixedPoint.

 
%   Copyright 2008-2021 The MathWorks, Inc.

    methods
        function out=Convolver
        end

        function out=helpFixedPoint(~) %#ok<STOUT>
            %helpFixedPoint Display dsp.Convolver System object
            %               fixed-point information
            %   dsp.Convolver.helpFixedPoint displays information about
            %   fixed-point properties and operations of the dsp.Convolver
            %   System object.
        end

        function out=isInactivePropertyImpl(~) %#ok<STOUT>
        end

        function out=setPortDataTypeConnections(~) %#ok<STOUT>
        end

    end
    properties
        %AccumulatorDataType Accumulator word- and fraction-length designations
        %   Specify the accumulator fixed-point data type as one of ['Full
        %   precision' | 'Same as product' | 'Same as first input' |
        %   'Custom']. This property is applicable when the Method property is
        %   'Time Domain' and the FullPrecisionOverride property is false. The
        %   default is 'Full precision'.
        AccumulatorDataType;

        %CustomAccumulatorDataType Accumulator word and fraction lengths
        %   Specify the accumulator fixed-point type as an auto-signed scaled
        %   numerictype object. This property is applicable when the Method
        %   property is 'Time Domain', the FullPrecisionOverride property is
        %   false, and the AccumulatorDataType property is 'Custom'. The
        %   default value of this property is numerictype([],32,30).
        %
        %   See also numerictype.
        CustomAccumulatorDataType;

        %CustomOutputDataType Output word and fraction lengths
        %   Specify the output fixed-point type as an auto-signed, scaled
        %   numerictype object. This property is applicable when the Method
        %   property is 'Time Domain', the FullPrecisionOverride property  is
        %   false, and the OutputDataType property is 'Custom'.
        %   The default value of this property is numerictype([],16,15).
        %
        %   See also numerictype.
        CustomOutputDataType;

        %CustomProductDataType Product word and fraction lengths              
        %   Specify the product fixed-point type as an auto-signed, scaled
        %   numerictype object. This property is applicable when the Method
        %   property is 'Time Domain', the FullPrecisionOverride property is
        %   false, and the ProductDataType property is 'Custom'.
        %   The default value of this property is numerictype([],32,30).
        %
        %   See also numerictype.
        CustomProductDataType;

        %FullPrecisionOverride Full precision override for fixed-point arithmetic
        %   Specify whether to use full precision rules. If you set
        %   FullPrecisionOverride to true the object computes all
        %   internal arithmetic and output data types using full
        %   precision rules. These rules guarantee that no quantization
        %   occurs within the object. Bits are added as needed to ensure
        %   that no round-off or overflow occurs. If you set
        %   FullPrecisionOverride to false, fixed-point data types
        %   are controlled through individual property settings.
        FullPrecisionOverride;

        %Method Domain for computing convolutions, time or frequency
        %   Specify the domain in which the System object computes convolutions
        %   as one of ['Time Domain' | 'Frequency Domain' | 'Fastest'].
        %   Computing convolutions in the time domain minimizes memory use.
        %   Computing convolutions in the frequency domain might require fewer
        %   computations than computing in the time domain, depending on the
        %   input length. If this property is set to 'Fastest', the object
        %   computes convolutions in the domain which minimizes the number of
        %   computations. The default is 'Time Domain'.
        Method;

        %OutputDataType Output word- and fraction-length designations
        %   Specify the output fixed-point data type as one of ['Same as
        %   accumulator' | 'Same as product' | 'Same as first input' |
        %   'Custom']. This property is applicable when the Method property is
        %   'Time Domain' and the FullPrecisionOverride property is false. The
        %   default is 'Same as accumulator'.
        OutputDataType;

        %OverflowAction Overflow action for fixed-point operations
        %   Specify the overflow action as one of ['Wrap' | 'Saturate']. This
        %   property is applicable when the Method property is 'Time Domain'
        %   and the object is not in a full precision configuration. The
        %   default is 'Wrap'.
        OverflowAction;

        %ProductDataType Product word- and fraction-length designations
        %   Specify the product fixed-point data type as one of ['Full
        %   precision' | 'Same as first input' | 'Custom']. This property is
        %   applicable when the Method property is 'Time Domain' and the
        %   FullPrecisionOverride property is false. The default is 'Full
        %   precision'.
        ProductDataType;

        %RoundingMethod Rounding method for fixed-point operations
        %   Specify the rounding method as one of ['Ceiling' | 'Convergent' |
        %   'Floor' | 'Nearest' | 'Round' | 'Simplest' | 'Zero']. This
        %   property is applicable when the Method property is 'Time Domain'
        %   and the object is not in a full precision configuration. The
        %   default is 'Floor'.
        RoundingMethod;

    end
end
