classdef Crosscorrelator< matlab.system.SFunSystem
%Crosscorrelator Cross-correlation of two inputs
%   XCR = dsp.Crosscorrelator returns a System object, XCR, that computes
%   the cross-correlation of two inputs in the time domain or frequency
%   domain.
%
%   XCR = dsp.Crosscorrelator('PropertyName', PropertyValue, ...) returns
%   a correlation object, XCR, with each specified property set to the
%   specified value.
%
%   Step method syntax:
%
%   Y = step(XCR, A,B) computes Y as the cross-correlation of A and B.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   Crosscorrelator methods:
%
%   step     - See above description for use of this method
%   release  - Allow property value and input characteristics changes
%   clone    - Create cross-correlation object with same property values
%   isLocked - Locked status (logical)
%
%   Crosscorrelator properties:
%
%   Method - Domain for computing correlations, time or frequency
%
%   This System object supports fixed-point operations when the property
%   Method is set to 'Time Domain'. For more information, type
%   dsp.Crosscorrelator.helpFixedPoint.
%
%   % EXAMPLE: Compute correlation between two signals.
%   XCR = dsp.Crosscorrelator;
%   t   = 0:0.001:1;
%   x1  = sin(2*pi*2*t) + 0.05*sin(2*pi*50*t);
%   x2  = sin(2*pi*2*t);
%   y   = XCR(x1',x2');  % computes cross-correlation of x1 and x2
%   figure, plot(t,x1,'b',t,x2,'g');
%   legend('Input signal 1', 'Input signal 2')
%   figure, plot(y);
%   title('Correlated output')
%
%   See also dsp.Crosscorrelator.helpFixedPoint.

 
%   Copyright 2008-2021 The MathWorks, Inc.

    methods
        function out=Crosscorrelator
        end

        function out=helpFixedPoint(~) %#ok<STOUT>
            %helpFixedPoint Display dsp.Crosscorrelator System object
            %               fixed-point information
            %   dsp.Crosscorrelator.helpFixedPoint displays information
            %   about fixed-point properties and operations of the
            %   dsp.Crosscorrelator System object.
        end

        function out=isInactivePropertyImpl(~) %#ok<STOUT>
        end

        function out=setPortDataTypeConnections(~) %#ok<STOUT>
        end

    end
    properties
        %AccumulatorDataType Accumulator word- and fraction-length designations
        %   Specify the accumulator fixed-point data type as one of ['Full
        %   precision' | 'Same as product' | 'Same as first input' |
        %   'Custom']. This property is applicable when the Method property is
        %   'Time Domain' and the FullPrecisionOverride property is false. The
        %   default is 'Full precision'.
        AccumulatorDataType;

        %CustomAccumulatorDataType Accumulator word and fraction lengths
        %   Specify the accumulator fixed-point type as an auto-signed scaled
        %   numerictype object. This property is applicable when the Method
        %   property is 'Time Domain', the FullPrecisionOverride property is
        %   false, and the AccumulatorDataType property is 'Custom'. The
        %   default value of this property is numerictype([],32,30).
        %
        %   See also numerictype.
        CustomAccumulatorDataType;

        %CustomOutputDataType Output word and fraction lengths
        %   Specify the output fixed-point type as an auto-signed, scaled
        %   numerictype object. This property is applicable when the Method
        %   property is 'Time Domain', the FullPrecisionOverride property  is
        %   false, and the OutputDataType property is 'Custom'.
        %   The default value of this property is numerictype([],16,15).
        %
        %   See also numerictype.
        CustomOutputDataType;

        %CustomProductDataType Product word and fraction lengths              
        %   Specify the product fixed-point type as an auto-signed, scaled
        %   numerictype object. This property is applicable when the Method
        %   property is 'Time Domain', the FullPrecisionOverride property is
        %   false, and the ProductDataType property is 'Custom'.
        %   The default value of this property is numerictype([],32,30).
        %
        %   See also numerictype.
        CustomProductDataType;

        %FullPrecisionOverride Full precision override for fixed-point arithmetic
        %   Specify whether to use full precision rules. If you set
        %   FullPrecisionOverride to true the object computes all
        %   internal arithmetic and output data types using full
        %   precision rules. These rules guarantee that no quantization
        %   occurs within the object. Bits are added as needed to ensure
        %   that no round-off or overflow occurs. If you set
        %   FullPrecisionOverride to false, fixed-point data types
        %   are controlled through individual property settings.
        FullPrecisionOverride;

        %Method Domain for computing correlations, time or frequency
        %   Specify the domain in which the System object computes correlation
        %   as one of ['Time Domain' | 'Frequency Domain' | 'Fastest'].
        %   Computing correlations in the time domain minimizes memory use.
        %   Computing correlations in the frequency domain might require fewer
        %   computations than computing in the time domain, depending on the
        %   input length. If this property is set to 'Fastest', the object
        %   computes in the domain which minimizes the number of computations.
        %   The default is 'Time Domain'.
        Method;

        %OutputDataType Output word- and fraction-length designations
        %   Specify the output fixed-point data type as one of ['Same as
        %   accumulator' | 'Same as product' | 'Same as first input' |
        %   'Custom']. This property is applicable when the Method property is
        %   'Time Domain' and the FullPrecisionOverride property is false. The
        %   default is 'Same as accumulator'.
        OutputDataType;

        %OverflowAction Overflow action for fixed-point operations
        %   Specify the overflow action as one of ['Wrap' | 'Saturate']. This
        %   property is applicable when the Method property is 'Time Domain'
        %   and the object is not in a full precision configuration. The
        %   default is 'Wrap'.
        OverflowAction;

        %ProductDataType Product word- and fraction-length designations
        %   Specify the product fixed-point data type as one of ['Full
        %   precision' | 'Same as first input' | 'Custom']. This property is
        %   applicable when the Method property is 'Time Domain' and the
        %   FullPrecisionOverride property is false. The default is 'Full
        %   precision'.
        ProductDataType;

        %RoundingMethod Rounding method for fixed-point operations
        %   Specify the rounding method as one of ['Ceiling' | 'Convergent' |
        %   'Floor' | 'Nearest' | 'Round' | 'Simplest' | 'Zero']. This
        %   property is applicable when the Method property is 'Time Domain'
        %   and the object is not in a full precision configuration. The
        %   default is 'Floor'.
        RoundingMethod;

    end
end
