classdef CumulativeSum< matlab.system.SFunSystem
%CumulativeSum Compute cumulative sum of channel, column, or row elements
%   CUMSUM = dsp.CumulativeSum returns a System object, CUMSUM, that
%   computes the cumulative sum of input matrix or input vector elements
%   along a specified dimension.
%
%   CUMSUM = dsp.CumulativeSum('PropertyName', PropertyValue, ...) returns
%   a cumulative sum object, CUMSUM, with each specified property set to
%   the specified value.
%
%   Step method syntax:
%
%   Y = step(CUMSUM, X) computes the cumulative sum along the specified
%   dimension for the input X.
%
%   Y = step(CUMSUM, X, R) computes the cumulative sum of the input
%   elements over time, and optionally resets the System object's state
%   based on the ResetCondition property value, and the value of the reset
%   signal, R. This occurs when the ResetInputPort property is true.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   CumulativeSum methods:
%
%   step     - See above description for the use of this method
%   release  - Allow property value and input characteristics changes
%   clone    - Create cumulative sum object with same property values
%   isLocked - Locked status (logical)
%   <a href="matlab:help matlab.System/reset   ">reset</a>    - Reset the cumulative sum to zero
%
%   CumulativeSum properties:
%
%   Dimension            - Computation dimension for cumulative sum
%   ResetInputPort       - Enables resetting the cumulative sum via an
%                          input port
%   ResetCondition       - Reset condition for cumulative sum
%
%   This System object supports fixed-point operations. For more
%   information, type dsp.CumulativeSum.helpFixedPoint.
%
%   % EXAMPLE: Use CumulativeSum System object to compute cumulative sum of
%   % a matrix.
%   csum = dsp.CumulativeSum;
%   x = magic(2);
%   y = csum(x);
%   y = csum(x)
%
%   See also cumsum, dsp.CumulativeSum.helpFixedPoint.

 
%   Copyright 1995-2021 The MathWorks, Inc.

    methods
        function out=CumulativeSum
        end

        function out=helpFixedPoint(~) %#ok<STOUT>
            %helpFixedPoint Display dsp.CumulativeSum System object
            %               fixed-point information
            %   dsp.CumulativeSum.helpFixedPoint displays information
            %   about fixed-point properties and operations of the
            %   dsp.CumulativeSum System object.
        end

        function out=isInactivePropertyImpl(~) %#ok<STOUT>
            % Hide the reset and sampling mode properties unless we're in
            % running mode
        end

        function out=loadObjectImpl(~) %#ok<STOUT>
        end

        function out=setPortDataTypeConnections(~) %#ok<STOUT>
        end

    end
    properties
        %AccumulatorDataType  Accumulator word- and fraction-length designations
        %   Specify the accumulator fixed-point data type as one of ['Same
        %   as input' | 'Custom']. The default is 'Same as input'.
        AccumulatorDataType;

        %CustomAccumulatorDataType  Accumulator word and fraction lengths
        %   Specify the accumulator fixed-point type as an auto-signed,
        %   scaled numerictype object. This property is applicable when the
        %   AccumulatorDataType property is 'Custom'. The default value of
        %   this property is numerictype([], 32, 30).
        %
        %   See also numerictype.
        CustomAccumulatorDataType;

        %CustomOutputDataType  Output word and fraction lengths
        %   Specify the output fixed-point type as an auto-signed, scaled
        %   numerictype object. This property is applicable when the
        %   OutputDataType property is 'Custom'. The default value of this
        %   property is numerictype([], 16, 15).
        %
        %   See also numerictype.
        CustomOutputDataType;

        %Dimension  Computation dimension for cumulative sum
        %   Specify the computation dimension as one of ['Channels (running
        %   sum)' | 'Rows' | 'Columns']. The default is 'Channels (running
        %   sum)'.
        Dimension;

        %OutputDataType  Output word- and fraction-length designations
        %   Specify the output fixed-point data type as one of ['Same as
        %   accumulator' | 'Same as input' | 'Custom']. The default is
        %   'Same as accumulator'.
        OutputDataType;

        %OverflowAction  Overflow action for fixed-point operations
        %   Specify the overflow action as one of ['Wrap' | 'Saturate'].
        %   The default is 'Wrap'.
        OverflowAction;

        %ResetCondition Reset condition for cumulative sum
        %   Specify the event on the reset input port that causes the
        %   cumulative sum to be reset as one of ['Rising edge' | 'Falling
        %   edge' | 'Either edge' | 'Non-zero']. This property is applicable
        %   when the Dimension property is set to 'Channels (running sum)'
        %   and the ResetInputPort property is true. The default is 'Rising
        %   edge'.
        ResetCondition;

        %ResetInputPort Enables resetting the cumulative sum via an input
        %               port
        %   Set this property to true to enable resetting the cumulative
        %   sum. When the property is set to true, a reset input must be
        %   specified to the step method to reset the cumulative sum. This
        %   property is relevant when the Dimension property is set to
        %   'Channels (running sum)'. The default value of this property is
        %   false.
        ResetInputPort;

        %RoundingMethod  Rounding method for fixed-point operations
        %   Specify the rounding method as one of ['Ceiling' | 'Convergent'
        %   | 'Floor' | 'Nearest' | 'Round' | 'Simplest' | 'Zero']. The
        %   default is 'Floor'.
        RoundingMethod;

    end
end
