classdef Interpolator< matlab.system.SFunSystem
%Interpolator Interpolate values of real input samples
%   INTERPOLATOR  = dsp.Interpolator returns an interpolation System
%   object, INTERPOLATOR , to interpolate values between real-valued input
%   samples using linear or FIR interpolation.
%
%   INTERPOLATOR  = dsp.Interpolator('PropertyName', PropertyValue, ...)
%   returns an interpolation System object, INTERPOLATOR , with each
%   specified property set to the specified value.
%
%   Specify which values to interpolate by providing a vector of
%   interpolation points. An interpolation point of 1 refers to the first
%   sample in the input. To interpolate the value half-way between the
%   second and third sample in the input, specify an interpolation point of
%   2.5. Interpolation points that are not within the valid range are
%   replaced with the closest value in the valid range.
%
%   Step method syntax:
%
%   Y = step(INTERPOLATOR, X) outputs the interpolated sequence, Y, of the
%   input vector or matrix X as specified in the InterpolationPoints
%   property. Each column of X is treated as an independent channel of
%   input.
%
%   Y = step(INTERPOLATOR, X, IPTS) outputs the interpolated sequence as
%   specified by the input argument IPTS when the InterpolationPointsSource
%   property is set to 'Input port'. IPTS is a column vector or a matrix
%   with the same number of channels as the input.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   Interpolator methods:
%
%   step     - See above description for use of this method
%   release  - Allow property value and input characteristics changes
%   clone    - Create interpolation object with same property values
%   isLocked - Locked status (logical)
%
%   Interpolator properties:
%
%   InterpolationPointsSource    - Source of interpolation points
%   InterpolationPoints          - Interpolation points
%   Method                       - Interpolation method
%   FilterHalfLength             - Interpolation filter half-length
%   InterpolationPointsPerSample - Interpolation points per input sample
%   Bandwidth                    - Normalized input bandwidth
%
%   % EXAMPLE: Use Interpolator System object to interpolate a subsampled
%   % signal.
%       t = 0:.0001:.0511;
%       x = sin(2*pi*20*t);
%       x1 = x(1:50:end);
%       I = 1:0.1:length(x1);
%       interpolator = ...
%           dsp.Interpolator('InterpolationPointsSource', 'Input port');
%       y = interpolator(x1',I');
%       stem(I',y,'r'); title('Original and Interpolated Signal');
%       hold on;
%       stem(x1,'Linewidth',2);
%       legend('Interpolated','Original');
%       hold off;
%
%   See also interp, dsp.FIRInterpolator.

 
%   Copyright 1995-2021 The MathWorks, Inc.

    methods
        function out=Interpolator
        end

        function out=isInactivePropertyImpl(~) %#ok<STOUT>
            % if in Linear mode hide filter properties
        end

        function out=setPortDataTypeConnections(~) %#ok<STOUT>
        end

    end
    properties
        %Bandwidth Normalized input bandwidth
        %   Specify the bandwidth to which the interpolated output samples
        %   should be constrained as numeric scalar between 0 and 1. This
        %   property can be used to take advantage of the bandlimited frequency
        %   content of the input. For example, if the input signal does not
        %   have frequency content above Fs/4 (where Fs is the sampling
        %   frequency), a value of 0.5 can be specified. A value of 1 specifies
        %   half the sample frequency. This property is applicable when the
        %   InterpolationMethod property is 'FIR'. The default value is 0.5.
        Bandwidth;

        %FilterHalfLength Interpolation filter half-length
        %   See help on the Method property for more information. This property
        %   is applicable when the Method property is 'FIR'. The default value
        %   is 3.
        %
        %   See also dsp.Interpolator.Method.
        FilterHalfLength;

        %InterpolationPoints Interpolation points
        %   Specify the interpolation points as a column vector or a matrix
        %   with the same number of channels as the input. The valid range of
        %   the values in the interpolation vector is from 1 to the number of
        %   samples in each channel of the input. This property applies when
        %   you set the InterpolationPointsSource property to 'Property'. The
        %   default value of this property is [1.1;4.8;2.67;1.6;3.2]. This
        %   property is tunable.
        InterpolationPoints;

        %InterpolationPointsPerSample Interpolation points per input sample
        %   See help on the Method property for more information. This property
        %   is applicable when the Method property is 'FIR'. The default value
        %   is 3.
        %
        %   See also dsp.Interpolator.Method.
        InterpolationPointsPerSample;

        %InterpolationPointsSource Source of interpolation points
        %   This property can be set to one of ['Property' | 'Input port'].
        %   The default is 'Property'.
        InterpolationPointsSource;

        %Method Interpolation method
        %   Specify the interpolation method as one of ['Linear' | 'FIR'].
        %   When this property is 'Linear', the System object interpolates data
        %   values by assuming that the data varies linearly between samples
        %   taken at adjacent sample times. When this property is 'FIR', the
        %   System object computes a value at the desired interpolation point
        %   by applying an FIR filter of order 2P to the input samples on
        %   either side of the desired interpolation point, where P is the
        %   Interpolation filter half-length. A vector of 2P filter tap weights
        %   is precomputed for each Q discrete points between input samples,
        %   where you specify Q in the InterpolationPointsPerSample property.
        %   For an interpolation point corresponding to one of the Q points,
        %   the unique filter computed for that point is applied to obtain a
        %   value for the sample at the specified interpolation point. For
        %   interpolation points that fall between Q points, the value computed
        %   at the nearest Q point is used. For interpolation points less than
        %   P-1, the output is computed using linear interpolation. The default
        %   is 'Linear'.
        Method;

    end
end
