classdef LMSFilter< matlab.system.SFunSystem
%LMSFilter Compute output, error, and weights using LMS adaptive algorithm
%   LMSFILT = dsp.LMSFilter returns an adaptive FIR filter System object,
%   LMSFILT, that computes the filtered output, filter error and the filter
%   weights for a given input and desired signal using the Least Mean
%   Squares (LMS) algorithm.
%
%   LMSFILT = dsp.LMSFilter('PropertyName', PropertyValue, ...) returns an
%   LMS filter System object, LMSFILT, with each specified property set to
%   the specified value.
%
%   LMSFILT = dsp.LMSFilter(LEN, 'PropertyName', PropertyValue, ...)
%   returns an LMS filter System object, LMSFILT, with the Length property
%   set to LEN, and other specified properties set to the specified values.
%
%   Step method syntax:
%
%   [Y, ERR, WTS] = step(LMSFILT, X, D) filters the input X, using D as the
%   desired signal, and returns the filtered output in Y, the filter error
%   in ERR, and the estimated filter weights in WTS. The System object
%   estimates the filter weights needed to minimize the error between the
%   output signal and the desired signal.
%
%   [Y, ERR] = step(LMSFILT, X, D) filters the input X, using D as the
%   desired signal, and returns the filtered output in Y and the filter
%   error in ERR when the WeightsOutput property is 'None'.
%
%   [...] = step(LMSFILT, X, D, MU) filters the input X, using D as the
%   desired signal and MU as the step size, when the StepSizeSource
%   property is 'Input port'.
%
%   [...] = step(LMSFILT, X, D, A) filters the input X, using D as the
%   desired signal and A as the adaptation control, when the AdaptInputPort
%   property is true. When A is non-zero, the System object continuously
%   updates the filter weights. When A is zero, the filter weights remain
%   constant.
%
%   [...] = step(LMSFILT, X, D, R) filters the input X, using D as the
%   desired signal and R as a reset signal when the WeightsResetInputPort
%   property is true. The WeightsResetCondition property can be used to set
%   the reset trigger condition. If a reset event occurs, the System object
%   resets the filter weights to their initial values.
%
%   The above operations can be used simultaneously, provided the System
%   object properties are set appropriately. One example of providing all
%   possible inputs and returning all possible outputs is shown below: [Y,
%   ERR, WTS] = step(LMSFILT, X, D, MU, A, R) filters input X using D as
%   the desired signal, MU as the step size, A as the adaptation control,
%   and R as the reset signal, and returns the filtered output in Y, the
%   filter error in ERR, and the adapted filter weights in WTS.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   LMSFilter methods:
%
%   step     - See above description for use of this method
%   release  - Allow property value and input characteristics changes
%   clone    - Create LMS filter object with same property values
%   isLocked - Locked status (logical)
%   <a href="matlab:help matlab.System/reset   ">reset</a>    - Reset filter states
%   maxstep  - Maximum step size for LMS adaptive filter convergence
%   msepred  - Predicted mean-square error for LMS filter
%   msesim   - Mean-squared error for LMS filter
%
%   LMSFilter properties:
%
%   Method                - Method to calculate filter weights
%   Length                - Length of FIR filter weights vector
%   StepSizeSource        - How to specify adaptation step size
%   StepSize              - Adaptation step size
%   LeakageFactor         - Leakage factor used in Leaky LMS method
%   InitialConditions     - Initial conditions of filter weights
%   AdaptInputPort        - Enables weight adaptation
%   WeightsResetInputPort - Enables weight reset
%   WeightsResetCondition - Reset trigger setting for filter weights
%   WeightsOutput         - Enables returning filter weights
%
%   This System object supports fixed-point operations. For more
%   information, type dsp.LMSFilter.helpFixedPoint.
%
%   % EXAMPLE #1: System identification of an FIR filter
%   % Initialize
%   N = 20; FrameSize = 100; NIter = 2000;
%   lmsfilt1 = dsp.LMSFilter(N+1, 'StepSize', 0.01);
%   b = fir1(N,0.25); firfilt1 = dsp.FIRFilter('Numerator',b);
%   AP = dsp.ArrayPlot('YLimits',[-0.1 .3],'ShowLegend',true, ...
%       'Position',[0 0 560 420],'ChannelNames', ...
%       {'Actual Coefficients', 'Estimated Coefficients'});
%   TS = timescope('TimeSpanSource','property','TimeSpan',FrameSize*NIter,...
%        'YLimits',[-50 0],'Title','Learning curve','YLabel','dB');
%   signalmean = dsp.MovingAverage('SpecifyWindowLength',false);
%   % Stream
%   for k = 1:NIter
%       x = randn(FrameSize,1);                     % input signal
%       d = firfilt1(x) + 0.01*randn(FrameSize,1);  % desired signal
%       [y,e,w] = lmsfilt1(x,d);
%       AP([b.',w]);        % Actual = channel 1; Estimate = channel 2
%       TS(10*log10(signalmean(abs(e).^2)));
%   end
%
%   % EXAMPLE #2: Noise cancellation
%   % Initialize
%   FrameSize = 100; NIter = 10;
%   lmsfilt2 = dsp.LMSFilter('Length',11,'Method','Normalized LMS', ...
%             'StepSize',0.05);
%   firfilt2 = dsp.FIRFilter('Numerator', fir1(10,[.5, .75]));
%   sinewave = dsp.SineWave('Frequency',0.01, ...
%       'SampleRate',1,'SamplesPerFrame',FrameSize);
%   TS = timescope('TimeSpanSource','property','TimeSpan',FrameSize*NIter,...
%        'YLimits',[-3 3],'ShowLegend',true,'ChannelNames', ...
%        {'Noisy signal', 'Filtered signal'});
%   % Stream
%   for k = 1:NIter
%       x = randn(FrameSize,1);       % Input signal
%       d = firfilt2(x) + sinewave(); % Noise + Signal
%       [y,e,w] = lmsfilt2(x,d);
%       TS([d,e]);          % Noisy = channel 1; Filtered = channel 2
%   end
%
%   % EXAMPLE #3: LMS with full adaptive weights history output
%   % Initialize
%   FrameSize = 15000;
%   lmsfilt3 = dsp.LMSFilter('Length', 63, 'Method', 'LMS', ...
%             'StepSize', 0.001, 'LeakageFactor', 0.99999, ...
%             'WeightsOutput', 'All'); % full Weights history
%   w_actual = fir1(64,[.5, .75]);
%   firfilt3 = dsp.FIRFilter('Numerator',w_actual);
%   sinewave = dsp.SineWave('Frequency', 0.01, ...
%       'SampleRate',1,'SamplesPerFrame',FrameSize);
%   TS = timescope('TimeSpanSource','property','TimeSpan',FrameSize,...
%        'YLimits',[-0.25 0.75],'ShowLegend', true, 'ChannelNames', ...
%        {'Coeff 33 Estimate','Coeff 34 Estimate','Coeff 35 Estimate', ...
%         'Coeff 33 Actual',  'Coeff 34 Actual',  'Coeff 35 Actual'});
%   % Run one frame and plot full adaptive weights history
%   x = randn(FrameSize,1);       % Input signal
%   d = firfilt3(x) + sinewave(); % Noise + Signal
%   [~,~,w] = lmsfilt3(x,d);
%   % Plot convergence of Weights indices 33, 34, 35
%   idxBeg = 33; idxEnd = 35;
%   TS([w(:,idxBeg:idxEnd), repmat(w_actual(idxBeg:idxEnd),FrameSize,1)]);
%
%   See also dsp.BlockLMSFilter, dsp.LMSFilter.helpFixedPoint.

 
%   Copyright 1995-2020 The MathWorks, Inc.

    methods
        function out=LMSFilter
        end

        function out=helpFixedPoint(~) %#ok<STOUT>
            %helpFixedPoint Display dsp.LMSFilter System object
            %               fixed-point information
            %   dsp.LMSFilter.helpFixedPoint displays information about
            %   fixed-point properties and operations of the dsp.LMSFilter
            %   System object.
        end

        function out=isInactivePropertyImpl(~) %#ok<STOUT>
        end

        function out=maxstep(~) %#ok<STOUT>
            %MAXSTEP  Maximum step size for LMS adaptive filter convergence.
            %
            %   MUMAX = MAXSTEP(LMSFILT,X) predicts a bound on the step size to
            %   provide convergence of the mean values of the coefficients of
            %   the adaptive filter defined by LMSFILT.
            %
            %   The columns of the matrix X contain individual input signal
            %   sequences. The signal set is assumed to have zero mean or
            %   nearly so.
            %
            %   [MUMAX,MUMAXMSE] = MAXSTEP(LMSFILT,X) predicts a bound on the
            %   adaptive filter step size to provide convergence of the
            %   adaptive filter coefficients in mean square.
        end

        function out=msepred(~) %#ok<STOUT>
            %MSEPRED Predicted mean-squared error for LMS adaptive filter.
            %
            %   [MMSE,EMSE] = MSEPRED(LMSFILT,X,D) predicts the steady-state
            %   values at convergence of the minimum mean-squared error (MMSE)
            %   and the excess mean-squared error (EMSE) given the input and
            %   desired response signal sequences in X and D and the quantities
            %   in the adaptive filter LMSFILT.
            %
            %   [MMSE,EMSE,MEANW,MSE,TRACEK] = MSEPRED(LMSFILT,X,D) calculates
            %   three sequences corresponding to the analytical behavior of the
            %   LMS adaptive filter defined by LMSFILT:
            %
            %   MEANW   - Sequence of coefficient vector means. The columns of
            %             this matrix contain predictions of the mean values of
            %             the LMS adaptive filter coefficients at each time
            %             instant. The dimensions of MEANW are (SIZE(X,1)) x
            %             (LMSFILT.length).
            %   MSE     - Sequence of mean-square errors. This column vector
            %             contains predictions of the mean-square error of the
            %             LMS adaptive filter at each time instant. The length
            %             of MSE is equal to SIZE(X,1).
            %   TRACEK  - Sequence of total coefficient error powers. This
            %             column vector contains predictions of the total
            %             coefficient error power of the LMS adaptive filter at
            %             each time instant. The length of TRACEK is equal to
            %             SIZE(X,1).
            %
            %   [MMSE,EMSE,MEANW,MSE,TRACEK] = MSEPRED(LMSFILT,X,D,M) specifies
            %   an optional decimation factor for computing MEANW, MSE, and
            %   TRACEK. If M > 1, every Mth predicted value of each of these
            %   sequences is saved. If omitted, the value of M defaults to one.
        end

        function out=msesim(~) %#ok<STOUT>
            %MSESIM Mean-squared error for LMS adaptive filter.
            %   [MSE,MEANW,W,TRACEK] = MSESIM(LMSFILT,X,D,M) calculates the
            %   sequence of mean-square errors and three other parameters
            %   corresponding to the simulated behavior of the LMS adaptive
            %   filter defined by LMSFILT.
            %
            %   Inputs:
            %   X and D are matrices of input signal sequences and desired
            %   response signal sequences respectively. M specifies an
            %   optional decimation factor for computing outputs MSE,
            %   MEANW, and TRACEK. If M > 1, every Mth predicted value of
            %   each of these sequences is saved. If omitted, the value of
            %   M defaults to one.
            %
            %   Outputs:
            %   MSE     - Sequence of mean-square errors. This column
            %             vector contains estimates of the mean-square
            %             error of the adaptive filter at each time
            %             instant. The length of MSE is equal to SIZE(X,1).
            %             The columns of the matrix X contain individual
            %             input signal sequences, and the columns of the
            %             matrix D contain corresponding desired response
            %             signal sequences.
            %   MEANW   - Sequence of coefficient vector means. The columns
            %             of this matrix contain estimates of the mean
            %             values of the LMS adaptive filter coefficients at
            %             each time instant. The dimensions of MEANW are
            %             (SIZE(X,1)) x (LMSFILT.length).
            %   W       - Estimate of the final values of the adaptive
            %             filter coefficients for the algorithm
            %             corresponding to LMSFILT.
            %   TRACEK  - Sequence of total coefficient error powers. This
            %             column vector contains estimates of the total
            %             coefficient error power of the LMS adaptive
            %             filter at each time instant. The length of TRACEK
            %             is equal to SIZE(X,1).
        end

        function out=setPortDataTypeConnections(~) %#ok<STOUT>
            % WeightsOutput can be 'None', 'Last', or 'All'
        end

    end
    properties
        %AdaptInputPort Enables weight adaptation
        %   Specify when the System object should adapt the filter weights. By
        %   default, the value of this property is false, and the object
        %   continuously updates the filter weights. When this property is set to
        %   true, an adaptation control input is provided to the step method. If
        %   the value of this input is non-zero, the object continuously updates
        %   the filter weights. If the input is zero, the filter weights remain
        %   at their current value.
        AdaptInputPort;

        %ConvolutionAccumulatorDataType Convolution accumulator word- and fraction-length designations
        %   Specify the convolution accumulator fixed-point data
        %   type as one of ['Same as first input' | 'Custom'].
        %   The default is 'Same as first input'.
        ConvolutionAccumulatorDataType;

        %ConvolutionProductDataType Convolution product word- and fraction-length designations
        %   Specify the convolution product fixed-point data type as
        %   one of ['Same as first input' | 'Custom']. The default is
        %   'Same as first input'.
        ConvolutionProductDataType;

        %CustomConvolutionAccumulatorDataType Convolution accumulator word and fraction lengths
        %   Specify the convolution accumulator fixed-point type as an auto-signed
        %   scaled numerictype object. This property is applicable when the
        %   ConvolutionAccumulatorDataType property is 'Custom'. The default
        %   value of this property is numerictype([],32,20).
        %
        %   See also numerictype.
        CustomConvolutionAccumulatorDataType;

        %CustomConvolutionProductDataType Convolution product word and fraction lengths
        %   Specify the convolution product fixed-point type as an auto-signed
        %   scaled numerictype object. This property is applicable when the
        %   ConvolutionProductDataType property is 'Custom'. The default value of
        %   this property is numerictype([],32,20).
        %
        %   See also numerictype.
        CustomConvolutionProductDataType;

        %CustomEnergyAccumulatorDataType Energy accumulator word and fraction lengths
        %   Specify the energy accumulator fixed-point type as an auto-signed
        %   scaled numerictype object. This property is applicable when the
        %   Method property is 'Normalized LMS' and the EnergyAccumulatorDataType
        %   property is 'Custom'. The default value of this property is
        %   numerictype([],32,20).
        %
        %   See also numerictype.
        CustomEnergyAccumulatorDataType;

        %CustomEnergyProductDataType Energy product word and fraction lengths
        %   Specify the energy product fixed-point type as an auto-signed scaled
        %   numerictype object. This property is applicable when the Method
        %   property is 'Normalized LMS' and the EnergyProductDataType property
        %   is 'Custom'. The default value of this property is
        %   numerictype([],32,20).
        %
        %   See also numerictype.
        CustomEnergyProductDataType;

        %CustomLeakageFactorDataType Leakage factor word and fraction lengths
        %   Specify the leakage factor fixed-point type as an auto-signed
        %   numerictype object. This property is applicable when the
        %   LeakageFactorDataType property is 'Custom'. The default value of this
        %   property is numerictype([],16,15).
        %
        %   See also numerictype.
        CustomLeakageFactorDataType;

        %CustomQuotientDataType Quotient word and fraction lengths
        %   Specify the quotient fixed-point type as an auto-signed scaled
        %   numerictype object. This property is applicable when the Method
        %   property is 'Normalized LMS' and the QuotientDataType property is
        %   'Custom'. The default value of this property is
        %   numerictype([],32,20).
        %
        %   See also numerictype.
        CustomQuotientDataType;

        %CustomStepSizeDataType Step size word and fraction lengths
        %   Specify the step size fixed-point type as an auto-signed numerictype
        %   object. This property is applicable when the StepSizeSource property
        %   is 'Property' and the StepSizeDataType property is 'Custom'. The
        %   default value of this property is numerictype([],16,15).
        %
        %   See also numerictype.
        CustomStepSizeDataType;

        %CustomStepSizeErrorProductDataType Step size error product word and fraction lengths
        %   Specify the step size error product fixed-point type as an auto-signed
        %   scaled numerictype object. This property is applicable when the
        %   StepSizeErrorProductDataType property is 'Custom'. The default value
        %   of this property is numerictype([],32,20).
        %
        %   See also numerictype.
        CustomStepSizeErrorProductDataType;

        %CustomWeightsDataType Weights word and fraction lengths
        %   Specify the filter weights fixed-point type as an auto-signed scaled
        %   numerictype object. This property is applicable when the
        %   WeightsDataType property is 'Custom'. The default value of this
        %   property is numerictype([],16,15).
        %
        %   See also numerictype.
        CustomWeightsDataType;

        %CustomWeightsUpdateProductDataType Weight update product word and fraction lengths
        %   Specify the weight update product fixed-point type as an auto-signed
        %   scaled numerictype object. This property is applicable when the
        %   WeightsUpdateProductDataType property is 'Custom'. The default value
        %   of this property is numerictype([],32,20).
        %
        %   See also numerictype.
        CustomWeightsUpdateProductDataType;

        %EnergyAccumulatorDataType Energy accumulator word- and fraction-length designations
        %   Specify the energy accumulator fixed-point data type as one of
        %   ['Same as first input' | 'Custom']. The default is 'Same as first input'. This
        %   property is applicable when the Method property is 'Normalized LMS'.
        EnergyAccumulatorDataType;

        %EnergyProductDataType Energy product word- and fraction-length designations
        %   Specify the energy product fixed-point data type as one of ['Same as
        %   first input' | 'Custom']. The default is 'Same as first input'. This
        %   property is applicable when the Method property is 'Normalized LMS'.
        EnergyProductDataType;

        %InitialConditions Initial conditions of filter weights
        %   Specify the initial values of the FIR filter weights as a scalar or
        %   vector of length equal to the Length property value. The default
        %   value of this property is 0.
        InitialConditions;

        %LeakageFactor Leakage factor used in Leaky LMS method
        %   Specify the leakage factor used in Leaky LMS method as a scalar
        %   numeric value between 0 and 1, both inclusive. When the value is less
        %   than 1, the System object implements a leaky LMS method. The default
        %   value of this property is 1, providing no leakage in the adapting
        %   method. This property is tunable.
        LeakageFactor;

        %LeakageFactorDataType Leakage factor word- and fraction-length designations
        %   Specify the leakage factor fixed-point data type as one of ['Same
        %   word length as first input' | 'Custom']. The default is 'Same word
        %   length as first input'.
        LeakageFactorDataType;

        %Length Length of FIR filter weights vector
        %   Specify the length of the FIR filter weights vector as a scalar
        %   positive integer value. The default value of this property is 32.
        Length;

        %Method Method to calculate filter weights
        %   Specify the method used to calculate filter weights as one of
        %   ['LMS' | 'Normalized LMS' | 'Sign-Error LMS' | 'Sign-Data LMS' |
        %   'Sign-Sign LMS']. The default is 'LMS'.
        Method;

        %OverflowAction Overflow action for fixed-point operations
        %   Specify the overflow action as one of ['Wrap' | 'Saturate'].
        %   The default is 'Wrap'.
        OverflowAction;

        %QuotientDataType Quotient word- and fraction-length designations
        %   Specify the quotient fixed-point data type as one of ['Same as first
        %   input' | 'Custom']. The default is 'Same as first input'. This
        %   property is applicable when the Method property is 'Normalized LMS'.
        QuotientDataType;

        %RoundingMethod Rounding method for fixed-point operations
        %   Specify the rounding method as one of ['Ceiling' | 'Convergent' |
        %   'Floor' | 'Nearest' | 'Round' | 'Simplest' | 'Zero']. The default
        %   is 'Floor'.
        RoundingMethod;

        %StepSize Adaptation step size
        %   Specify the adaptation step size factor as a scalar non-negative
        %   numeric value. For convergence of the normalized LMS method, the step
        %   size should be greater than 0 and less than 2. The default value of
        %   this property is 0.1. This property is applicable when the
        %   StepSizeSource property is 'Property'. This property is tunable.
        StepSize;

        %StepSizeDataType Step size word- and fraction-length designations
        %   Specify the step size fixed-point data type as one of ['Same word
        %   length as first input' | 'Custom']. The default is 'Same word length
        %   as first input'. This property is applicable when the StepSizeSource
        %   property is 'Property'.
        StepSizeDataType;

        %StepSizeErrorProductDataType Step size error product word- and fraction-length designations
        %   Specify the step size error product fixed-point data type as one of
        %   ['Same as first input' | 'Custom'].
        %   The default is 'Same as first input'.
        StepSizeErrorProductDataType;

        %StepSizeSource How to specify adaptation step size
        %   Choose how to specify the adaptation step size factor as one of
        %   ['Property' | 'Input port']. The default is 'Property'.
        StepSizeSource;

        %WeightsDataType Weights word- and fraction-length designations
        %   Specify the filter weights fixed-point data type as one of ['Same as
        %   first input' | 'Custom']. The default is 'Same as first input'.
        WeightsDataType;

        %WeightsOutput Enables returning filter weights
        %   Specify how to output the adapted filter weights as one of
        %   ['None' | 'Last' | 'All']. By default, the value of this property is
        %   'Last'. The 'None' setting disables the weights output. The 'Last'
        %   setting enables the weights output to return a column vector of
        %   weights with dimensions [Length 1], where Length is the number of LMS
        %   filter weights. The 'All' setting enables the weights output to return
        %   a matrix of weights values with dimensions [FrameLength Length],
        %   corresponding to the full sample-by-sample history of weights values
        %   for all FrameLength samples of the input values.
        WeightsOutput;

        %WeightsResetCondition Reset trigger setting for filter weights
        %   Specify the event to reset the filter weights as one of [ 'Rising
        %   edge' | 'Falling edge' | 'Either edge' | 'Non-zero' ]. The default is
        %   'Non-zero'. The System object resets the filter weights based on the
        %   values of this property and the reset input to the step method. This
        %   property is applicable when the WeightsResetInputPort property is
        %   true.
        WeightsResetCondition;

        %WeightsResetInputPort Enables weight reset 
        %   Specify when the System object should reset the filter weights. By
        %   default, the value of this property is false, and the object does not
        %   reset the weights. When this property is set to true, a reset control
        %   input is provided to the step method, and the WeightsResetCondition
        %   property becomes applicable. The object resets the filter weights
        %   based on the values of the WeightsResetCondition property and the
        %   reset input to the step method.
        WeightsResetInputPort;

        %WeightsUpdateProductDataType Weight update product word- and fraction-length designations
        %   Specify the weight update product fixed-point data type as one of 
        %   ['Same as first input' | 'Custom'].
        %   The default is 'Same as first input'
        WeightsUpdateProductDataType;

    end
end
