classdef LevinsonSolver< matlab.system.SFunSystem
%LevinsonSolver Solve linear system of equations using Levinson-Durbin
%recursion
%   LEV = dsp.LevinsonSolver returns a System object, LEV, that solves a
%   Hermitian Toeplitz system of equations using the Levinson-Durbin
%   recursion.
%
%   LEV = dsp.LevinsonSolver('PropertyName', PropertyValue, ...) returns a
%   Levinson-Durbin object, LEV, with each specified property set to the
%   specified value.
%
%   Step method syntax:
%
%   K = step(LEV, X) returns reflection coefficients K corresponding to the
%   columns of input X. X is typically a column or matrix of
%   autocorrelation coefficients with lag 0 as the first element.
%
%   A = step(LEV, X) returns polynomial coefficients A when the AOutputPort
%   property is true and KOutputPort property is false.
%
%   [A, K] = step(LEV, X) returns polynomial coefficients A and reflection
%   coefficients K when both the AOutputPort and KOutputPort properties are
%   true.
%
%   [..., P] = step(LEV, X) also returns the error power P when the
%   PredictionErrorOutputPort property is true.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   LevinsonSolver methods:
%
%   step     - See above description for use of this method
%   release  - Allow property value and input characteristics changes
%   clone    - Create Levinson-Durbin object with same property values
%   isLocked - Locked status (logical)
%
%   LevinsonSolver properties:
%
%   AOutputPort               - Enables polynomial coefficients output
%   KOutputPort               - Enables reflection coefficients output
%   PredictionErrorOutputPort - Enables prediction error output
%   ZerothLagZeroAction       - Action when value of lag zero is zero
%
%   This System object supports fixed-point operations. For more
%   information, type dsp.LevinsonSolver.helpFixedPoint.
%
%   % EXAMPLE:
%       lev = dsp.LevinsonSolver;
%       lev.AOutputPort = true;
%       lev.KOutputPort = false;
%       x = (1:100)';
%       acorr = dsp.Autocorrelator(...
%               'MaximumLagSource', 'Property', ...
%               'MaximumLag', 10);
%       a = acorr(x);
%       c = lev(a);      % Compute polynomial coefficients
%
%   See also dsp.LevinsonSolver.helpFixedPoint.

 
%   Copyright 2008-2021 The MathWorks, Inc.

    methods
        function out=LevinsonSolver
        end

        function out=helpFixedPoint(~) %#ok<STOUT>
            %helpFixedPoint Display dsp.LevinsonSolver System object 
            %               fixed-point information
            %   dsp.LevinsonSolver.helpFixedPoint displays information
            %   about fixed-point properties and operations of the
            %   dsp.LevinsonSolver System object.
        end

        function out=isInactivePropertyImpl(~) %#ok<STOUT>
        end

        function out=setPortDataTypeConnections(~) %#ok<STOUT>
        end

    end
    properties
        %ACoefficientDataType A coefficient word- and fraction-length
        %                     designations
        %   This is a constant property with value 'Custom'.
        ACoefficientDataType;

        %AOutputPort Enables polynomial coefficients output
        %   Set this property to true to output the polynomial coefficients A.
        %   Both AOutputPort and KOutputPort properties cannot be false at the
        %   same time. For scalar inputs AOutputPort property must be true. The
        %   default value of this property is false.
        AOutputPort;

        %AccumulatorDataType Accumulator word- and fraction-length designations
        %   Specify the Accumulator fixed-point data type as one of ['Same as
        %   input' | 'Same as product' | 'Custom']. The default is 'Custom'.
        AccumulatorDataType;

        %CustomACoefficientDataType A coefficient word and fraction lengths
        %   Specify the A coefficient fixed-point type as an auto-signed scaled
        %   numerictype object. The default value of this property is
        %   numerictype([],16,15).
        %
        %   See also numerictype.
        CustomACoefficientDataType;

        %CustomAccumulatorDataType Accumulator word and fraction lengths
        %   Specify the accumulator fixed-point type as an auto-signed scaled
        %   numerictype object. This property is applicable when the
        %   AccumulatorDataType property is 'Custom'. The default value of this
        %   property is numerictype([],32,30).
        %
        %   See also numerictype.
        CustomAccumulatorDataType;

        %CustomKCoefficientDataType K coefficient word and fraction lengths
        %   Specify the K coefficient fixed-point type as an auto-signed scaled
        %   numerictype object. The default value of this property is
        %   numerictype([],16,15).
        %
        %   See also numerictype.
        CustomKCoefficientDataType;

        %CustomPredictionErrorDataType Prediction error power word and fraction
        %                              lengths
        %   Specify the prediction error power fixed-point type as an
        %   auto-signed scaled numerictype object. This property is applicable
        %   when the PredictionErrorDataType property is 'Custom'. The default
        %   value of this property is numerictype([],16,15).
        %
        %   See also numerictype.
        CustomPredictionErrorDataType;

        %CustomProductDataType Product word and fraction lengths
        %   Specify the product fixed-point type as an auto-signed scaled
        %   numerictype object. This property is applicable when the
        %   ProductDataType property is 'Custom'. The default value of this
        %   property is numerictype([],32,30).
        %
        %   See also numerictype.
        CustomProductDataType;

        %KCoefficientDataType K coefficient word- and fraction-length
        %                     designations
        %   This is a constant property with value 'Custom'.
        KCoefficientDataType;

        %KOutputPort Enables reflection coefficients output
        %   Set this property to true to output the reflection coefficients K.
        %   Both AOutputPort and KOutputPort properties cannot be false at the
        %   same time. For scalar inputs KOutputPort property must be false.
        %   The default value of this property is true.
        KOutputPort;

        %OverflowAction Overflow action for fixed-point operations
        %   Specify the overflow action as one of ['Wrap' | 'Saturate'].
        %   The default is 'Wrap'.
        OverflowAction;

        %PredictionErrorDataType Prediction error power word- and 
        %                        fraction-length designations
        %   Specify the prediction error power fixed-point data type as one of
        %   ['Same as input' | 'Custom']. The default is 'Same as input'.
        PredictionErrorDataType;

        %PredictionErrorOutputPort Enables prediction error output
        %   Set this property to true to output the prediction error. The
        %   default value of this property is false.
        PredictionErrorOutputPort;

        %ProductDataType Product word- and fraction-length designations
        %   Specify the product fixed-point data type as one of ['Same as
        %   input' | 'Custom']. The default is 'Custom'.
        ProductDataType;

        %RoundingMethod Rounding method for fixed-point operations
        %   Specify the rounding method as one of ['Ceiling' | 'Convergent' |
        %   'Floor' | 'Nearest' | 'Round' | 'Simplest' | 'Zero']. The default
        %   is 'Floor'.
        RoundingMethod;

        %ZerothLagZeroAction Action when value of lag zero is zero
        %   Specify the output for inputs with first coefficient as zero as one
        %   of ['Ignore' | 'Use zeros']. The default is 'Use zeros'.
        ZerothLagZeroAction;

    end
end
