classdef PhaseUnwrapper< matlab.system.SFunSystem
%PhaseUnwrapper Unwrap signal phase
%   PUNWRAP = dsp.PhaseUnwrapper returns a System object, PUNWRAP, that
%   adds or subtracts appropriate multiples of 2pi to each input element to
%   remove phase discontinuities (unwrap). Inputs should be radian phase
%   values.
%
%   PUNWRAP = dsp.PhaseUnwrapper('PropertyName', PropertyValue, ...)
%   returns an unwrap System object, PUNWRAP, with each specified property
%   set to the specified value.
%
%   Step method syntax:
%
%   Y = step(PUNWRAP, X) unwraps the input signal X. Each column of X is
%   treated as an independent channel of input.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   PhaseUnwrapper methods:
%
%   step     - See above description for use of this method
%   release  - Allow property value and input characteristics changes
%   clone    - Create unwrap object with same property values
%   isLocked - Locked status (logical)
%   <a href="matlab:help matlab.System/reset   ">reset</a>    - Resets the states when the InterFrameUnwrap property is true
%
%   PhaseUnwrapper properties:
%
%   InterFrameUnwrap     - Enables unwrapping of phase discontinuities
%                          between successive frames
%   Tolerance            - Jump size as a true phase discontinuity
%
%   %EXAMPLE: Unwrap input phase data.
%       punwrap = dsp.PhaseUnwrapper;
%       p = [0 2/5 4/5 -4/5 -2/5 0 2/5 4/5 -4/5 -2/5 0 2/5 4/5 -4/5, ...
%           -2/5]*pi;
%       y = punwrap(p'); 
%       figure,stem(p); hold 
%       stem(y, 'r');
%
%   See also unwrap.

 
%   Copyright 2008-2017 The MathWorks, Inc.

    methods
        function out=PhaseUnwrapper
        end

        function out=loadObjectImpl(~) %#ok<STOUT>
        end

        function out=setPortDataTypeConnections(~) %#ok<STOUT>
        end

    end
    properties
        %InterFrameUnwrap Enables unwrapping of phase discontinuities between
        %                 successive frames
        %   Set this property to false to unwrap phase discontinuities only
        %   within the frame.  Set this property to true to also unwrap phase
        %   discontinuities between successive frames. The default value of
        %   this property is true.
        InterFrameUnwrap;

        %Tolerance Jump size as a true phase discontinuity
        %   Specify the jump size that the System object recognizes as a true
        %   phase discontinuity. The default value of this property is set to
        %   pi (rather than a smaller value) to avoid altering legitimate
        %   signal features. To increase the System object's sensitivity, set
        %   Tolerance to a value slightly less than pi.
        Tolerance;

    end
end
