classdef SOSFilter< matlab.system.SFunSystem & dsp.internal.FilterAnalysis
%SOSFilter Second Order Section Biquadratic IIR filter structures
%   SOS = dsp.SOSFilter returns a biquadratic IIR filter
%   System object, SOS, which independently filters each channel of the
%   input over time using a specified biquadratic structure.
%
%   SOS = dsp.SOSFilter(Name=Value, ...) returns a biquadratic filter
%   object, SOS, with each specified property name set to the specified
%   property value.
%
%   SOS = dsp.SOSFilter(NUM, DEN, Name=Value, ...) returns a biquadratic
%   filter object, SOS, with the Numerator property set to NUM, the
%   Denominator property set to DEN, and other specified property names set
%   to the specified property values.
%
%   Step method syntax:
%
%   Y = step(SOS, X) filters the real or complex input signal X, and
%   outputs the filtered values, Y. The System object filters each column
%   of the input signal independently over successive calls to step method.
%   This syntax is valid only when CoefficientSource is 'Property'.
%
%   Y = step(SOS, X, NUM, DEN) filters the real or complex input signal X,
%   with Numerator values NUM, Denominator values DEN, and outputs the
%   filtered values, Y. The System object filters each column of the input
%   signal independently over successive calls to step method. This syntax
%   is valid only when CoefficientSource is 'Input port' and HasScaleValues
%   is false.
%
%   Y = step(SOS, X, NUM, DEN, G) filters the real or complex
%   input signal X, with Numerator values NUM, Denominator values DEN,
%   Scale values G, and outputs the filtered values, Y. The System object
%   filters each column of the input signal independently over successive
%   calls to step method. This syntax is valid only when CoefficientSource
%   is 'Input port' and HasScaleValues is true.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, Y = step(SOS, X) and Y = SOS(X) are
%   equivalent syntax.
%
%   SOSFilter methods:
%
%   step        - See above description for use of this method
%   release     - Allow property value and input characteristics changes
%   clone       - Create another SOSFilter with same property values
%   isLocked    - Locked status (logical)
%   <a href="matlab:help matlab.System/reset   ">reset</a>       - Reset filter states to initial conditions
%
%   SOSFilter properties:
%
%   Structure         - Filter structure
%   CoefficientSource - 'Property' or 'Input port'
%   Numerator         - Numerator coefficients of filter
%   Denominator       - Denominator coefficients of filter
%   HasScaleValues    - Specify whether or not object has ScaleValues
%   ScaleValues       - Scale values for each second order section
%
%   This System object supports several filter analysis methods. For more
%   information, type dsp.SOSFilter.helpFilterAnalysis.
%
%   This System object supports fixed-point operations. For more
%   information, type dsp.SOSFilter.helpFixedPoint.
%
%   % EXAMPLE: Remove high-frequency sinusoid using a biquad filter.
%   % Initialize
%   f1 = 1000; f2 = 3000; Fs = 8000; Fcutoff = 2000;
%   % Input composed of 1 kHz and 3 kHz sinusoids.
%   SR = dsp.SineWave(Frequency=[f1,f2],SampleRate=Fs,...
%       SamplesPerFrame=1024); 
%   [z,p,k] = butter(10,Fcutoff/(Fs/2));
%   [s, g]  = zp2sos(z,p,k);
%   num = s(:,1:3);
%   den = s(:,4:6);
%   sosFilter = ...
%       dsp.SOSFilter(num, den, HasScaleValues=true, ScaleValues=g);
%   SA = spectrumAnalyzer(SampleRate=Fs,...
%       Method="welch", ...
%       PlotAsTwoSidedSpectrum=false,ShowLegend=true,...
%       YLimits=[-350 50],...
%       ChannelNames=["Input signal", "Output signal"]);
%   % Stream processing loop
%   for k = 1:100
%       input = sum(SR(), 2); % Add the two sinusoids together
%       filteredOutput = sosFilter(input);
%       SA([input,filteredOutput]);
%   end
%
%   See also dsp.FourthOrderSectionFilter, dsp.FIRFilter, dsp.IIRFilter

 
%   Copyright 1995-2022 The MathWorks, Inc.

    methods
        function out=SOSFilter
        end

        function out=cumsec(~) %#ok<STOUT>
            %CUMSEC  Cumulative second-order section of SOSFilter System object
            %   H = cumsec(H) returns a cell array, H, of SOSFilter
            %   System objects containing the cumulative sections of the
            %   SOSFilter System object, H.
            %
            %   H = cumsec(H, INDICES) returns a cell array of
            %   SOSFilter Systems whose indices into the original
            %   filter are in INDICES.
            %
            %   H = cumsec(H, INDICES, SECONDARY) uses the secondary scaling
            %   points to determine where the sections should be split when
            %   SECONDARY is true. SECONDARY is false by default. This option
            %   only applies for SOSFilter objects with 'Direct
            %   form II' and 'Direct form I transposed' structures. For these
            %   structures, the secondary scaling points refer to the location
            %   between the recursive and the nonrecursive part (i.e. the
            %   "middle" of the section).
            %
            %   H = cumsec(H,'Arithmetic',ARITH,...) analyzes the filter System
            %   object, H, based on the arithmetic specified in the ARITH input.
            %   ARITH can be set to one of 'double', 'single', or 'fixed'. The
            %   analysis tool assumes a double precision filter when the
            %   arithmetic input is not specified and the filter System object is
            %   in an unlocked state.
            %
            %   cumsec(H,...) with no output arguments plots the magnitude
            %   response of the cumulative sections using FVTool.
        end

        function out=helpFixedPoint(~) %#ok<STOUT>
            %helpFixedPoint Display dsp.SOSFilter System object
            %               fixed-point information
            %   dsp.SOSFilter.helpFixedPoint displays information
            %   about fixed-point properties and operations of the
            %   dsp.SOSFilter System object.
        end

        function out=reorder(~) %#ok<STOUT>
            %REORDER Reorder second-order sections of SOSFilter System object
            %   reorder(H, ORDER) reorders the sections of the
            %   SOSFilter System object, H, using the vector of
            %   double precision indices in ORDER. ORDER does not need to contain
            %   all of the indices of the filter, the sections that correspond to
            %   missing indices will be removed from the filter.
            %
            %   reorder(H, ORDER), when ORDER is a vector of logicals, removes
            %   the sections of the filter that correspond to the index values
            %   where ORDER is equal to false.
            %
            %   HNEW = reorder(H, ORDER) generates a new SOSFilter
            %   System object, HNEW, that contains the re-ordered sections. In
            %   this case, the original filter, H, is not changed.
            %
            %   reorder(H, NUMORDER, DENORDER) reorders the numerator and
            %   denominator of the filter separately using the vector of indices
            %   in NUMORDER and DENORDER respectively. These vectors must be of
            %   the same length.
            %
            %   reorder(H, NUMORDER, DENORDER, SVORDER) independently reorders
            %   the scale values of the SOSFilter System object.
            %   If SVORDER is not specified, the ScaleValues will be reordered in
            %   the same way as the numerator. The output scale value will always
            %   remain at the end when NUMORDER is used to reorder the scale values.
            %
            %   reorder(H, FILTER_TYPE) where FILTER_TYPE is one of:
            %   'auto','lowpass', 'highpass', 'bandpass', or 'bandstop', reorders
            %   the SOSFilter System object, H, in a way suitable
            %   for the given filter type. This mode is intended for fixed-point
            %   implementations where the ordering of the sections can have a
            %   significant impact in the filter performance. Automatic reordering
            %   only applies when H was obtained using FDESIGN. The sections will
            %   be automatically reordered depending on the response type of the
            %   design (lowpass, highpass, etc.).
            %
            %   reorder(H, DIR_FLAG) reorders the filter sections such that the
            %   first section will contain the poles closest to the origin, and
            %   the last section will contain the poles closest to the unit
            %   circle when DIR_FLAG is 'UP'. When DIR_FLAG is 'DOWN', the
            %   sections are ordered in the opposite direction. The zeros are
            %   always paired with the poles closest to them.
            %
            %   reorder(H, DIR_FLAG, SV) reorders the scale values following the
            %   reordering of the poles when SV is set to 'poles', or following
            %   the reordering of the zeros when SV is set to 'zeros'. The scale
            %   values are not reordered when using the DIR_FLAG option unless SV
            %   is specified.
            %
            %   reorder(H,'Arithmetic',ARITH,...) assumes that the filter
            %   arithmetic is equal to ARITH. ARITH can be set to one of
            %   'double', 'single', or 'fixed'. The method assumes a double
            %   precision filter when the arithmetic input is not specified and
            %   the filter System object is in an unlocked state. The overflow
            %   action used in the reorder operation will be set to the one
            %   specified in the filter System object when ARITH is 'fixed'.
        end

        function out=scale(~) %#ok<STOUT>
            %SCALE   Scale second-order sections of SOSFilter
            %   scale(H) scales the SOSFilter System object, H,
            %   using peak magnitude response scaling (L-infinity), in order to
            %   reduce the possibility of overflow when H operates with
            %   fixed-point inputs.
            %
            %   HNEW = scale(H) generates a new filter System object, Hnew, with
            %   scaled second-order sections. The original filter System object,
            %   H, is not changed.
            %
            %   scale(H,Pnorm) specifies the norm used to scale the filter. Pnorm
            %   can be either a discrete-time-domain norm or a frequency-domain
            %   norm. Valid time-domain norms are 'l1','l2', and 'linf'. Valid
            %   frequency-domain norms are 'L1','L2', and 'Linf'. Note that
            %   L2-norm is equal to l2-norm (Parseval's theorem) but the same is
            %   not true for other norms.
            %
            %   The different norms can be ordered in terms of how stringent they
            %   are as follows: 'l1' >= 'Linf' >= 'L2' = 'l2' >= 'L1' >= 'linf'.
            %
            %   Using the most stringent scaling, 'l1', the filter is the least
            %   prone to overflow, but also has the worst signal-to-noise ratio.
            %   Linf-scaling is the most commonly used scaling in practice.
            %
            %   scale(H,Pnorm,P1,V1,P2,V2,...) specifies optional scaling
            %   parameters via parameter-value pairs. Valid pairs are:
            %
            %   Parameter               Default     Description/Valid values
            %   ---------               -------     ------------------------
            %   'sosReorder'            'auto'      Reorder section prior to scaling.
            %                                       {'auto','none','up','down','lowpass',
            %                                       'highpass','bandpass','bandstop'}
            %   'MaxNumerator'          2           Maximum value for numerator
            %                                       coefficients
            %   'NumeratorConstraint'   'none'      {'none', 'unit', 'normalize','po2'}
            %   'OverflowMode'          'wrap'      {'wrap','saturate'}
            %   'ScaleValueConstraint'  'unit'      {'unit','none','po2'}
            %   'MaxScaleValue'         'Not used'  Maximum value for scale values
            %
            %   Automatic reordering will only take effect when H was obtained as a
            %   result from a design using FDESIGN. The sections will be
            %   automatically reordered depending on the response type of the
            %   design (lowpass, highpass, etc.).
            %
            %   Note that 'MaxScaleValue' will only be used when
            %   'ScaleValueConstraint' is set to something other than 'unit'. If
            %   'MaxScaleValue' is set to a number, the 'ScaleValueConstraint' will
            %   be changed to 'none'.
            %
            %   scale(H,Pnorm,OPTS) uses an options object to specify the optional
            %   scaling parameters in lieu of specifying parameter-value pairs. The
            %   OPTS object can be created using the SCALEOPTS method: OPTS =
            %   scaleopts(H).
            %
            %   scale(H,'Arithmetic',ARITH,...) assumes that the filter arithmetic
            %   is equal to ARITH. ARITH can be set to one of 'double', 'single',
            %   or 'fixed'. The scale method assumes a double precision filter when
            %   the arithmetic input is not specified and the filter System object
            %   is in an unlocked state. If 'Arithmetic' is 'double' or 'single',
            %   the default values shown in the table above will be used for all
            %   scaling options that are not specified as an input to the scale
            %   method. If 'Arithmetic' is 'fixed', the values used for the scaling
            %   options will be set according to the settings in the filter System
            %   object, H. However, if a scaling option is specified that differs
            %   from the settings in H, this option will be used for scaling
            %   purposes but will not change the setting in H. For example, if you
            %   do not specify the 'OverflowMode' scaling option, the scale method
            %   will assume that the 'OverflowMode' is equal to the value in the
            %   OverflowAction property of the System object, H. If you do specify
            %   an 'OverflowMode' scaling option, then the scale method will use
            %   this overflow mode value regardless of the value in the
            %   OverflowAction property of the System object.
        end

        function out=scalecheck(~) %#ok<STOUT>
            %SCALECHECK Check scaling of SOSFilter System object
            %   S = scalecheck(H,PNORM) for a SOSFilter System
            %   object, H, and 'Direct form I' or 'Direct form II transposed'
            %   structures, returns a row vector S that computes the p-norm of
            %   the filter from its input to the output of each second-order
            %   section. Note that this computation does not include the trailing
            %   scale value H.ScaleValues(end).
            %
            %   Input PNORM can be either frequency-domain norms: 'L1', 'L2',
            %   'Linf' or discrete-time-domain norms: 'l1', 'l2', 'linf'. Note
            %   that the L2-norm of a filter is equal to its l2-norm (Parseval's
            %   theorem), but this is not true for other norms. PNORM defaults to
            %   'Linf' when omitted.
            %
            %   For 'Direct form II' and 'Direct form I transposed' structures, S
            %   is a row vector containing the p-norm from the filter input to
            %   the input of the recursive part of each second-order section.
            %   This corresponds to the input to the multipliers in these
            %   structures and are the points where overflow should be avoided.
            %   If H has non-trivial scale values, i.e. if not all scale values
            %   are equal to one, S is a two-row matrix containing in the second
            %   row the p-norm from the input of the filter to the input of each
            %   scale value between the sections. Note that for these structures,
            %   the last numerator and the trailing scale value are not included
            %   when checking the scale.
            %
            %   S = scalecheck(H,'Arithmetic',ARITH,...) analyzes the System
            %   object, H, based on the arithmetic specified in the ARITH input.
            %   ARITH can be set to one of 'double', 'single', or 'fixed'. The
            %   analysis tool assumes a double precision filter when the
            %   arithmetic input is not specified and the filter System object is
            %   in an unlocked state.
            %
            %   For a given p-norm, an optimally scaled filter will have partial
            %   norms equal to one, so S will contain all ones.
        end

        function out=scaleopts(~) %#ok<STOUT>
            %SCALEOPTS Create an options object for second-order section scaling
            %   OPTS = scaleopts(H) uses the current settings in the
            %   SOSFilter System object, H, to create an options
            %   object OPTS that contains scaling options for second-order
            %   section scaling. The OPTS object can be passed in as an argument
            %   to the SCALE method.
            %
            %   OPTS = scaleopts(H,'Arithmetic',ARITH,...) assumes that the
            %   filter arithmetic is equal to ARITH. ARITH can be set to one of
            %   'double', 'single', or 'fixed'. The method assumes a double
            %   precision filter when the arithmetic input is not specified and
            %   the filter System object is in an unlocked state. The scaleopts
            %   method will choose the default values of the scaling options
            %   according to the 'Arithmetic' value and the System object, H,
            %   settings.
        end

    end
    properties
        %CoefficientSource Source of Numerator, Denominator, and ScaleValues
        %   Specify the source of the Numerator, Denominator, and ScaleValues
        %   as one of ['Property' | 'Input port']. The default is 'Property'.
        CoefficientSource;

        %Denominator Denominator coefficients of filter
        %   Specify the denominator coefficients of size Lx3, or Lx2, where L
        %   is the number of sections. The leading denominator coefficients are
        %   assumed to be 1 always - if the denominator is of size Lx3, the
        %   first column values are ignored and replaced with one(s). If the
        %   denominator is of size Lx2, one(s) are appended to make it size
        %   Lx3. The size of this property cannot be modified once STEP method
        %   is called. However, size Lx3 denominator values may be modified.
        %   This property is applicable when the CoefficientSource property is
        %   'Property'. The default value is [1 -0.942809041582063 0.3333333333].
        Denominator;

        %DenominatorAccumulatorDataType Denominator accumulator numerictype
        %   Specify the denominator accumulator fixed-point data type as a
        %   numerictype object. The default is numerictype(1,64,48).
        DenominatorAccumulatorDataType;

        %DenominatorDataType Denominator Coefficients word- and fraction-length designations
        %   Specify the denominator coefficients fixed-point data type as
        %   ['Same word length as input'] or as a numerictype object. This
        %   property is applicable when the CoefficientSource property is
        %   'Property'. The default is 'Same word length as input'.
        DenominatorDataType;

        %HasScaleValues Specify if object has scale values for each section
        %   If true, allow specifying scale values to be applied before and
        %   after each section of a biquadratic filter. If false, do not use
        %   scale values. The default value is false.
        HasScaleValues;

        %MultiplicandDataType Multiplicand word- and fraction-length
        %                     designations
        %   Specify the multiplicand fixed-point data type as ['Same as
        %   output'] or as a numerictype object. This property is applicable
        %   when the Structure property is 'Direct form I transposed'.
        %   The default is 'Same as output'.
        MultiplicandDataType;

        %Numerator Numerator coefficients of filter
        %   Specify the numerator coefficients of size Lx3, where L is the
        %   number of sections. The size of this property cannot be modified
        %   once STEP method is called. However, the values may be modified.
        %   This property is applicable when the CoefficientSource property is
        %   'Property'. The default value is [0.0975 0.195 0.0975].
        Numerator;

        %NumeratorDataType Numerator coefficients word- and
        %                              fraction-length designations
        %   Specify the numerator coefficients fixed-point data type as 
        %   ['Same word length as input'] or as a numerictype object. This
        %   property is applicable when the CoefficientSource property is
        %   'Property'. The default is 'Same word length as input'.
        NumeratorDataType;

        %OutputDataType Output word- and fraction-length designations
        %   Specify the output fixed-point data type as ['Full precision'] or
        %   as a numerictype object. The default is 'Full precision'.
        OutputDataType;

        %OverflowAction Overflow action for fixed-point operations
        %   Specify the overflow action as one of ['Wrap' | 'Saturate'].
        %   The default is 'Wrap'.
        OverflowAction;

        %RoundingMethod Rounding method for fixed-point operations
        %   Specify the rounding method as one of ['Ceiling' | 'Convergent' |
        %   'Floor' | 'Nearest' | 'Round' | 'Simplest' | 'Zero'].
        %   The default is 'Floor'.
        RoundingMethod;

        %ScaleValues Scale values for each biquad second order section
        %   Specify the scale values to be applied before and after each
        %   section of a biquadratic filter. The scale values must be a
        %   real-valued scalar or a vector of length L+1, where L is the number
        %   of sections. If this property is set to a scalar, the value is used
        %   as the gain value before the first section of the second-order
        %   filter. The rest of the gain values are set to 1. If this property
        %   is set to a vector of L+1 values, each value is used for a separate
        %   section of the filter. This property is applicable when the
        %   CoefficientSource property is 'Property', and the HasScaleValues
        %   property is true. The default value is [1 1].
        ScaleValues;

        %ScaleValuesDataType Scale Values word- and fraction-length
        %                    designations
        %   Specify the scale values fixed-point data type as ['Same word
        %   length as input'] or as a numerictype object. This property is
        %   applicable when the CoefficientSource property is 'Property' and
        %   the HasScaleValues property is true.
        %   The default is 'Same word length as input'.
        ScaleValuesDataType;

        %SectionInputDataType Section input word- and fraction-length
        %                     designations
        %   Specify the section input fixed-point data type as ['Same as
        %   input'] or as a numerictype object.
        %   The default is 'Same as input'.
        SectionInputDataType;

        %SectionOutputDataType Section output word- and fraction-length
        %                      designations
        %   Specify the section output fixed-point data type as ['Same as
        %   section input'] or as a numerictype object.
        %   The default is 'Same as section input'.
        SectionOutputDataType;

        %StateDataType State word- and fraction-length designations
        %   Specify the state fixed-point data type as one of ['Full
        %   precision'] or as a numerictype object. This
        %   property is applicable when the Structure property is
        %   'Direct form II'. The default is 'Full precision'.
        StateDataType;

        %Structure Filter structure
        %   Specify the filter structure as one of ['Direct form I' | 'Direct
        %   form I transposed' | 'Direct form II' | 'Direct form II
        %   transposed']. The default is 'Direct form II transposed'.
        Structure;

    end
end
