classdef VariableIntegerDelay< matlab.system.SFunSystem
%VariableIntegerDelay Delay input by time-varying integer number of sample
%periods
%   VID = dsp.VariableIntegerDelay returns a variable integer delay System
%   object, VID, that delays discrete-time input by a time-varying integer
%   number of sample periods.
%
%   VID = dsp.VariableIntegerDelay('PropertyName', PropertyValue, ...)
%   returns a variable integer delay object, VID, with each specified
%   property set to the specified value.
%
%   Step method syntax:
%
%   Y = step(VID, X, D) delays the input X by D samples, where D should be
%   less than or equal to the value specified in the MaximumDelay property
%   and greater than or equal to 0. Delay values outside this range are
%   clipped appropriately and non-integer delays are rounded to the nearest
%   integer values. Each column of X is treated as an independent channel
%   of input.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   VariableIntegerDelay methods:
%
%   step     - See above description for use of this method
%   release  - Allow property value and input characteristics changes
%   clone    - Create variable integer delay object with same property
%              values
%   isLocked - Locked status (logical)
%   <a href="matlab:help matlab.System/reset   ">reset</a>    - Reset the states to initial conditions
%
%   VariableIntegerDelay properties:
%
%   MaximumDelay         - Maximum delay
%   InitialConditions    - Initial values in the memory
%
%   % EXAMPLE: Use a VariableIntegerDelay System object to delay a signal by 
%   %          a varying integer number of sample periods. 
%       VID = dsp.VariableIntegerDelay;
%       x = 1:100;
%       index = 0;
%       k = 0;
%       yout = [];
%       while(index+10 <= 100)
%           y = VID( x(index+1:index+10)',k);
%           yout = [yout;y];
%           index = index+10;
%           k = k+1;
%       end
%       stem(x,'b');
%       hold on; stem(yout,'r');
%       legend('Original Signal', 'Variable Integer Delayed Signal')
% 
%   See also dsp.VariableFractionalDelay, dsp.Delay, designFracDelayFIR.

 
%   Copyright 2008-2021 The MathWorks, Inc.

    methods
        function out=VariableIntegerDelay
        end

        function out=loadObjectImpl(~) %#ok<STOUT>
        end

        function out=setPortDataTypeConnections(~) %#ok<STOUT>
        end

    end
    properties
        %InitialConditions Initial values in memory
        %   Specify the values with which the object's memory is initialized.
        %   The dimensions of this property can vary depending on whether you
        %   want fixed or time-varying initial conditions. The default value of
        %   this property is 0.
        %   The object treats each of the N input columns as a frame containing
        %   M sequential time samples from an independent channel. For an
        %   M-by-N frame-based input matrix U, you can set the
        %   InitialConditions property as follows:
        %   * To specify fixed initial conditions, set the InitialConditions
        %   property to a scalar value. The object initializes every sample of
        %   every channel in memory using the value you specify.
        %   * To specify different time-varying initial conditions for each
        %   channel, set the InitialConditions property to an array of size
        %   1-by-N-by-D, where D is the value of the MaximumDelay property.
        InitialConditions;

        %MaximumDelay Maximum delay
        %   Specify the maximum delay the object can produce for any input
        %   sample. The maximum delay must be a positive scalar integer value.
        %   The object clips input delay values greater than the MaximumDelay
        %   to the MaximumDelay. The default value of this property is 100.
        MaximumDelay;

    end
end
