classdef audioDeviceWriter< matlab.System
%audioDeviceWriter Play audio data using the computer's audio device
%   ADW = audioDeviceWriter returns a System object, ADW, that plays audio
%   samples using an audio output device in real-time.
%
%   ADW = audioDeviceWriter('PropertyName', PropertyValue, ...) returns an
%   audio device writer System object, ADW, with each specified property
%   set to the specified value.
% 
%   ADW = audioDeviceWriter(SAMPLERATE, 'PropertyName', PropertyValue, ...)
%   returns an audio device writer System object, ADW, with the SampleRate
%   property set to SAMPLERATE and other specified properties set to the
%   specified values.
%
%   step method syntax:
%
%   NUMUNDERRUN = step(ADW, AUDIO) or NUMUNDERRUN = play(ADW, AUDIO) writes
%   one frame of audio samples to the selected audio output device, and
%   returns the number of samples NUMUNDERRUN by which the audio device
%   writer's queue was underrun since the last call to STEP/PLAY method.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   audioDeviceWriter methods:
%
%   step             - See above description for use of this method
%   play             - Works exactly like step() method
%   getAudioDevices  - List available audio output devices
%   info             - List properties of the selected audio device
%   release          - Allow property values and input characteristics to 
%                      change, and release audio device writer resources
%   clone            - Create audio device writer object with same property
%                      values
%   isLocked         - Locked status (logical)
%
%   audioDeviceWriter properties:
%
%   Driver                   - Driver used to access the device
%   Device                   - Device used to play audio data
%   SampleRate               - Number of samples per second sent to device
%   BitDepth                 - Data type used by the device
%   SupportVariableSizeInput - Allow input frame size to change
%   BufferSize               - Buffer size
%   ChannelMappingSource     - Source of output device channel mapping 
%   ChannelMapping           - Data-to-device channel mapping 
%
%   % EXAMPLE: Read in an MP3 audio file and play it back using the
%   %          standard audio output device.
%      fileReader = dsp.AudioFileReader;
%      devWriter = audioDeviceWriter('SampleRate',fileReader.SampleRate);
%      fileInfo = audioinfo(fileReader.Filename);
%      setup(devWriter, zeros(fileReader.SamplesPerFrame, ...
%                       fileInfo.NumChannels));  % Initialize audio device
%      while ~isDone(fileReader)
%          audioSamples = fileReader();
%          play(devWriter, audioSamples);
%      end
%      release(fileReader);             % close the input file
%      release(devWriter);              % close the audio output device
%
%   % EXAMPLE: Play an audio file and print information about number of 
%   %          samples underrun in the queue if that number is not zero.
%   %          Also, display the latency due to the output buffer on audio 
%   %          device.
%      fileReader = dsp.AudioFileReader;
%      devWriter = audioDeviceWriter('SampleRate',fileReader.SampleRate);
%      fileInfo = audioinfo(fileReader.Filename);
%      setup(devWriter, zeros(fileReader.SamplesPerFrame, ...
%                       fileInfo.NumChannels));  % Initialize audio device
%      while ~isDone(fileReader)
%          audioIn = fileReader();
%          nUnderrun = play(devWriter, audioIn);
%          if nUnderrun > 0
%            fprintf('Audio writer queue was underrun by %d samples.\n',...
%                      nUnderrun);
%          end
%      end
%      fprintf('Latency due to sound card''s output buffer: %f seconds.\n', ...
%               fileReader.SamplesPerFrame/devWriter.SampleRate);
%      release(fileReader);            % close the input file
%      release(devWriter);             % release the audio device
%
%   See also audioDeviceReader, audioPlayerRecorder, dsp.AudioFileReader,
%   dsp.AudioFileWriter, asiosettings, audiodevreset.

 
%   Copyright 2015-2021 The MathWorks, Inc.

    methods
        function out=audioDeviceWriter
        end

        function out=getAudioDevices(~) %#ok<STOUT>
            %GETAUDIODEVICES List available audio output devices
            % devs = getAudioDevices(ADW) returns a list of audio output
            % devices supported by the audio driver listed under 'Driver'
            % property of audioDeviceWriter object.
        end

        function out=getPropertyGroupsImpl(~) %#ok<STOUT>
            % Only the properties listed in mainProps are displayed in the
            % command-line by default
        end

        function out=infoImpl(~) %#ok<STOUT>
            %INFO Information about audio device
            % devInfo = info(ADW) returns a structure of name of driver
            % and device associated with ADW, and maximum number of output
            % channels that the device supports. If the 'Device' property
            % of ADW is 'Default', this structure will provide the full
            % name of the device that is taken as default.
            %
            % % Example: List device information
            % devWriter = audioDeviceWriter;
            % info(devWriter)
        end

        function out=isInactivePropertyImpl(~) %#ok<STOUT>
        end

        function out=isInputSizeMutableImpl(~) %#ok<STOUT>
        end

        function out=loadObjectImpl(~) %#ok<STOUT>
        end

        function out=play(~) %#ok<STOUT>
            %PLAY Stream audio data to output device
            % underrun = play(ADW, u) plays audio samples to computer's
            % audio input device. It returns the number of samples by which
            % audio queue was underrun since the last call to play.
        end

        function out=releaseImpl(~) %#ok<STOUT>
        end

        function out=resetImpl(~) %#ok<STOUT>
        end

        function out=saveObjectImpl(~) %#ok<STOUT>
        end

        function out=setupImpl(~) %#ok<STOUT>
        end

        function out=stepImpl(~) %#ok<STOUT>
        end

        function out=validateInputsImpl(~) %#ok<STOUT>
        end

        function out=validatePropertiesImpl(~) %#ok<STOUT>
        end

    end
    properties
        %BitDepth Data type used by the device
        %   Specify the data type used by the device to acquire audio data
        %   as one of '8-bit integer'|'16-bit integer'|'24-bit
        %   integer'|'32-bit float'. The default is '16-bit integer'.
        BitDepth;

        %BufferSize Buffer size
        %   Specify the size of the buffer that the System object uses to
        %   communicate with the audio device as a scalar integer. The
        %   default value of this property is 4096.
        BufferSize;

        %ChannelMapping Mapping of input data to device channels
        %   Specify a vector of valid channel indices to represent the
        %   mapping between data and device output channels as a scalar or
        %   vector of integers. This property applies when the
        %   ChannelMappingSource is 'Property'. The default value of this
        %   property depends on the audio device, and is 1:maxChannels.
        ChannelMapping;

        %ChannelMappingSource Source of output channel mapping
        %   Specify how to determine the channel mapping as one of
        %   'Auto'|'Property'. The default is 'Auto'.
        ChannelMappingSource;

        %Device Device to which to send audio data
        %   Specify the device to which the audio data should be sent.
        Device;

        %Driver Audio device driver
        %   Specify the driver used to communicate with the audio device as
        %   one of 'ASIO'|'DirectSound'|'WASAPI'. This property applies
        %   only on a Windows machine. The driver on Linux machine is
        %   always ALSA and on a Mac machine is always CoreAudio. The
        %   default value of this property is 'DirectSound'.
        %   Note: When using ASIO driver with SupportVariableSizeInput set
        %   to false, use the ASIOSETTINGS function to match the driver's
        %   buffer size with the input frame size to minimize latency. When
        %   using ASIO driver with SupportVariableSizeInput set to true,
        %   use the ASIOSETTINGS function to match the driver's buffer size
        %   with the BufferSize property value to minimize latency.
        Driver;

        %SampleRate Number of samples per second sent to audio device
        %   Specify the number of samples per second in the signal as a
        %   scalar value. The default value of this property is 44100.
        SampleRate;

        %SupportVariableSizeInput Support variable sized input
        %   Specify if the object should support variable sized input as a
        %   logical scalar. When this property is set to false, the input
        %   has to have same frame size at each call to step() or play()
        %   method and the buffer size for audio device is same as the
        %   input frame size. When this property is set to true, the input
        %   frame size can change at each call to step() or play() method
        %   and the buffer size is specified through BufferSize property.
        %   The default value of this property is false.
        SupportVariableSizeInput;

    end
end
