function dspunfold(varargin)
% DSPUNFOLD Generates a multi-threaded MEX file from a MATLAB function
%
%   Generates a multi-threaded MEX file from a MATLAB function using DSP
%   unfolding technology. DSP unfolding is a technique to improve
%   throughput through parallelization.
%
%   Specify the data type of the inputs the generated code can accept. The
%   generated multi-threaded MEX file is specialized to the class, size,
%   and complexity of the inputs.
%
%   The general syntax of the dspunfold command is
%      dspunfold [-options] [file]
%
%
%   FILE:
%
%   Specify the name of the main (entry point) MATLAB file you want to
%   compile on the command line.
%
%
%   OPTIONS:
%
%   -args ARGS      Specify the types that the generated code accept. ARGS
%                   is a cell array specifying the type of each function 
%                   argument from the input MATLAB file. By default, the 
%                   value of ARGS is empty {}.
%
%   -o OUTPUT       Specify the name of the output multi-threaded MEX
%                   file. If no OUTPUT name is provided, the name of the
%                   generated multi-threaded MEX file is inherited from the
%                   input MATLAB file with a '_mt' suffix. DSPUNFOLD also
%                   adds a platform-specific extension to this name. In
%                   addition, a single-threaded MEX file with a '_st'
%                   suffix, and a test bench file with a '_analyzer'
%                   suffix, are generated. The test bench file is used to
%                   test the output results and measure speedup between the
%                   multi-threaded and single-threaded MEX files.
%                    
%   -s STATELENGTH  Specify the state length (aka Impulse Response Length)
%                   of the algorithm from the input MATLAB file (a value 0
%                   represents a stateless algorithm). Specifying 'auto'
%                   for the STATELENGTH triggers state length auto
%                   detection, which can significantly increase the time to
%                   generate the MEX files. If FRAMEINPUTS argument is
%                   'false', the STATELENGTH unit is frames, otherwise the
%                   unit is samples. By default, the value of the
%                   STATELENGTH is 0(zero).
%
%   -r REPETITION   Specify the repetition factor used to generate the
%                   multi-threaded MEX file (see the documentation for
%                   details on this option). Increasing this value can
%                   improve the multi-threaded MEX speedup at the cost of a
%                   larger output latency. Likewise, decreasing this value
%                   can reduce the output latency, with a multi-threaded
%                   MEX speedup decrease. By default, the value of
%                   REPETITION is 1.
%
%   -t THREADS      Specify the number of threads to be used by the
%                   multi-threaded MEX file. Increasing the THREADS value
%                   can improve the multi-threaded MEX speedup at the cost
%                   of a larger output latency. Decreasing the THREADS
%                   value reduces the output latency, but can also reduce
%                   the multi-threaded MEX speedup. The default value for
%                   the THREADS is the number of the physical CPU cores
%                   present.
%
%   -f FRAMEINPUTS  Specify the frameness of the MATLAB file input
%                   arguments. Set frameness to true for each input that
%                   can be divided without altering the algorithm behavior.
%                   This can increase the speedup of the generated
%                   multi-threaded MEX file. A scalar logical value can be
%                   used to set the frameness of all inputs simultaneously.
%                   If STATELENGTH>0, the inputs marked as frames must have
%                   the same dimensions. By default, the value of the
%                   FRAMEINPUTS is 'false'.
%
%   -v VERBOSE      Show verbose output during code generation. Default
%                   value for VERBOSE is true.
%
%
%   EXAMPLES:
%
%    % EXAMPLE 1: Generate multi-threaded MEX file for the FFTUnfoldExample
%    % function. This function accepts one input argument of type double 
%    % and size 4096x1. The algorithm from the FFTUnfoldExample function is
%    % stateless. Use dspunfold to generate the multi-threaded MEX file 
%    % named FFTUnfoldExample_mt, optimized to run on 4 threads.
%       dspunfold FFTUnfoldExample -args {rand(4096,1)} -t 4 -s 0
%
%
%    % EXAMPLE 2: Generate multi-threaded MEX file for the FIRUnfoldExample
%    % function. This function accepts two input arguments. The first input
%    % is of type double and size 4096x1, representing the input to the 
%    % FIR filter. The second input is of type double and size 1x128, 
%    % representing the FIR Filter coefficients. The algorithm from the 
%    % FIRUnfoldExample function has a state length of 127 samples. Use 
%    % dspunfold to generate the multi-threaded MEX file named 
%    % FIRUnfoldExample_mt, which uses repetition 10.
%       dspunfold FIRUnfoldExample -args {rand(4096,1),fir1(127,0.4)} ...
%                                  -f [true,false] -s 127 -r 10 
%
%  See also codegen.


