classdef powermeter< dsp.internal.SupportScalarVarSize
%powermeter Measure power, peak-to-average power ratio (PAPR), and the
%complementary cumulative distribution function (CCDF) of the power of
%voltage signal
%   METER = POWERMETER returns a System object, METER, that computes the
%   power, peak-to-average power ratio (PAPR), and the complementary
%   cumulative distribution function (CCDF) of the power of a voltage signal.
%   The CCDF helps to find the probability that the instantaneous signal
%   power exceeds a specified level above the average signal power.
%
%   METER = POWERMETER(Name=Value, ...) returns a power meter
%   System object, METER, with each specified property name set to the
%   specified value. You can specify additional name-value pair arguments
%   in any order as (Name1=Value1, ..., NameN=ValueN).
%
%   METER = POWERMETER(Length, Overlap, Name=Value, ...) returns a power
%   meter object, METER, with the WindowLength property set to Length,
%   OverlapLength property set to Overlap, and other properties set to the
%   specified values. This syntax is valid if the ComputeCCDF property is
%   set to false, which is the default.
%
%   Step method syntax:
%
%   AVGPWR = step(METER, X) computes the average power, AVGPWR, of the
%   input X when the Measurement property is set to "Average power". Each
%   column of X is a channel. The System object computes the power of each
%   channel of the input signal independently.
%
%   PEAKPWR = step(METER, X) computes the peak power, PEAKPWR, of the
%   input X when the Measurement property is set to "Peak power". Each
%   column of X is a channel. The System object computes the power of each
%   channel of the input signal independently.
%
%   PAPR = step(METER, X) computes the peak-to-average power ratio, PAPR,
%   of the input X when the Measurement property is set to "Peak-to-average
%   power ratio". Each column of X is a channel. The System object computes
%   the power of each channel of the input signal independently.
%
%   [AVGPWR, PEAKPWR, PAPR] = step(METER, X) computes the average power,
%   AVGPWR, the peak power, PEAKPWR, and the peak-to-average power ratio,
%   PAPR, of the input X when the Measurement property is set to "All".
%   Each column of X is a channel. The System object computes the power of
%   each channel of the input signal independently.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   POWERMETER methods:
%
%   step           - Measure power of the input
%   release        - Allow property value and input characteristics
%                    changes, and release sliding window resources
%   clone          - Create power meter object with same property values
%   isLocked       - Display locked status (logical)
%   <a href="matlab:help matlab.System/reset   ">reset</a>          - Reset the states of the object
%   plotCCDF       - Plot CCDF curves. Must set ComputeCCDF to true beforehand.
%                    The x-axis represents relative power, the ratio of
%                    instantaneous signal power to average signal power in dB.
%                    The y-axis represents the probability (in percent) that
%                    the relative power exceeds a specified dB level
%   ccdf           - Get the coordinates of CCDF curves. Must set ComputeCCDF
%                    to true beforehand
%   relativePower  - Use CCDF to find the relative power for a specified
%                    probability. Must set ComputeCCDF to true beforehand
%   probability    - Use CCDF to find the probability for a specified
%                    relative power. Must set ComputeCCDF to true beforehand
%
%   POWERMETER properties:
%
%   Measurement      - Desired power measurement
%   ReferenceLoad    - Reference load
%   ComputeCCDF      - Enable calculation of CCDF. All input samples since object
%                      creation or reset are used to collect statistics
%   PowerRange       - The x-axis range of the computed CCDF curves in dB.
%                      The computed CCDF curves end at the maximum relative power,
%                      i.e. PAPR of the signal, and start at PAPR minus PowerRange
%   PowerResolution  - The x-axis resolution of the computed CCDF curves in dB
%   WindowLength     - Sliding window length. Used if ComputeCCDF is false
%   OverlapLength    - Overlap length between windows. Used if ComputeCCDF is false
%   OutputPowerUnits - Power units
%
%  % EXAMPLE 1: Compute the average power of a 256-QAM signal using
%  % a sliding window of 16 samples with default overlap, and a sliding 
%  % window of 16 samples with an overlap of 10 samples.
%  FrameLength = 512;
%  Fs = 1000;
%  pm  = powermeter(16, Measurement="Average power", ...
%                       ReferenceLoad=50, ...
%                       OutputPowerUnits="dBm");
%  pm_overlap  = powermeter(16, 10, Measurement="Average power", ...
%                       ReferenceLoad=50, ...
%                       OutputPowerUnits="dBm");
%  scope  = timescope(SampleRate=[Fs,Fs/(16-10)],...
%                     TimeSpanSource="property",...
%                     TimeSpan=20,...
%                     YLabel="dBm",...
%                     YLimits=[0 50], ...
%                     Title="Average Power", ...
%                     ChannelNames = ["Sliding window of 16 samples with default overlap",...
%      "Sliding window of 16 samples with overlap of 10 samples"]);
%  tic;
%  while (toc < 5)
%    x = randi([0 255], FrameLength, 1);
%    y = qammod(x, 256);
%    averagePower = pm(y);
%    averagePower_overlap = pm_overlap(y);
%    scope(averagePower,averagePower_overlap);
%  end
%
%  % EXAMPLE 2: Find the probability that the instantaneous power of a signal
%  % is 3 dB above the average power.
%  x = complex(rand(10000,1)-0.5,rand(10000,1)-0.5);
%  pm = powermeter(ComputeCCDF=true);
%  averagePower = pm(x)            % Power in dBm. Default ReferenceLoad = 1 ohm 
%  prob = probability(pm,3)        % Probability in percent
%  plotCCDF(pm,GaussianReference=true)
%  relpwr = relativePower(pm,prob) % Find relative power for a given probability
%
%  See also dsp.MovingAverage, dsp.MovingRMS, dsp.MovingVariance,
%  dsp.MovingMaximum.

 
%   Copyright 2020-2022 The MathWorks, Inc.

    methods
        function out=powermeter
        end

        function out=ccdf(~) %#ok<STOUT>
            % ccdf Get the coordinates of the CCDF curves
            %   [RELPOWER, PROB] = ccdf(METER) returns the coordinates of the CCDF curves. ComputeCCDF must
            %   be set to true before this method is called.
            %
            %   RELPOWER and PROB are double precision matrices of the same dimensions. The number of
            %   columns is the number of columns in the signals provided to the step method. The number
            %   of rows is ceil(PowerRange/PowerResolution)+1. The probability that the instantaneous signal
            %   power of the j-th channel is above its average signal power by RELPOWER(i,j) dB is PROB(i)/100.
        end

        function out=getDataType(~) %#ok<STOUT>
        end

        function out=getFrameLength(~) %#ok<STOUT>
        end

        function out=getNumChans(~) %#ok<STOUT>
        end

        function out=getNumOutputsImpl(~) %#ok<STOUT>
        end

        function out=getPropertyGroups(~) %#ok<STOUT>
        end

        function out=isComputingCCDF(~) %#ok<STOUT>
        end

        function out=isFixedSizeOutput(~) %#ok<STOUT>
            % Varsize output if input is varsize or arbitrary input is allowed
        end

        function out=isInactivePropertyImpl(~) %#ok<STOUT>
        end

        function out=isInputSizeMutableImpl(~) %#ok<STOUT>
        end

        function out=loadObjectImpl(~) %#ok<STOUT>
        end

        function out=plotCCDF(~) %#ok<STOUT>
            % plotCCDF Plot CCDF curves
            %   H = plotCCDF(METER) plots CCDF measurements in the powermeter System object, METER,
            %   and returns the plot handles as an output, H. This method plots as
            %   many curves as the number of input channels.
            %
            %   H = plotCCDF(METER, AX) plots CCDF measurements using the axes object AX.
            %
            %   H = plotCCDF(..., Name, Value) specifies options using one or more name-value arguments.
            %   Name-value arguments accepted by MATLAB's plot function can be used. For example,
            %   plotCCDF(METER,'LineWidth',2) will plot CCDF curves with line widths of 2 points.
            %   An additional name-value argument is supported:
            %
            %   'GaussianReference' - Set this property to true to plot a reference CCDF curve.
            %                         The reference CCDF curve represents the power of a complex
            %                         white Gaussian noise, calculated as a chi-squared distribution.
            %                         The default is false.
        end

        function out=probability(~) %#ok<STOUT>
            % probability Use CCDF to find the probability for a specified relative power
            %   PROB = probability(METER, RELPOWER) finds the probability PROB in percent that the instantaneous
            %   signal power is above the average signal power by RELPOWER dB. ComputeCCDF must be set to true
            %   before this method is called.
            %
            %   RELPOWER must be a double or single precision column vector. PROB is a double precision matrix that
            %   has the same number of rows as RELPOWER and has the same number of columns as the signals provided to the
            %   step method. The probability that the instantaneous signal power of the j-th channel is above its average
            %   signal power by RELPOWER(i,j) dB is PROB(i)/100.
            %
            %   If the PowerRange property is not set sufficiently large, the computed CCDF curves may not reach
            %   some relative power values in RELPOWER. In those cases, the corresponding values in PROB will be NaN.
        end

        function out=relativePower(~) %#ok<STOUT>
            % relativePower Use CCDF to find the relative power for a specified probability
            %   RELPOWER = relativePower(METER, PROB) finds the relative power, RELPOWER, for the specified probability
            %   PROB, such that the instantaneous signal power is above the average signal power by RELPOWER dB with
            %   probability PROB/100. ComputeCCDF must be set to true before this method is called.
            %
            %   PROB must be a double or single precision non-negative column vector. RELPOWER is a double precision
            %   matrix that has the same number of rows as PROB and has the same number of columns as the signals
            %   provided to the step method. The probability that the instantaneous signal power of the j-th channel
            %   is above its average signal power by RELPOWER(i,j) dB is PROB(i)/100.
            %
            %   If the PowerRange property is not set sufficiently large, the computed CCDF curves may not reach
            %   some probability values in PROB. In those cases, the corresponding values in RELPOWER will be NaN.
            %
            %   If PROB is not provided, relativePower will return results for PROB = [10 1 0.1 0.01 0.001 0.0001]'.
        end

        function out=releaseImpl(~) %#ok<STOUT>
        end

        function out=resetImpl(~) %#ok<STOUT>
        end

        function out=saveObjectImpl(~) %#ok<STOUT>
        end

        function out=setAutoOverlapLength(~) %#ok<STOUT>
            % If the user did not set the overlap length, it should
            % be equal to WindowLength - 1
        end

        function out=setUserDefinedOverlapLength(~) %#ok<STOUT>
            % If the user set the overlap length, set the overlap 
            % length factor source to 'property'
        end

        function out=setupImpl(~) %#ok<STOUT>
            % Get the number of input channels
        end

        function out=stepImpl(~) %#ok<STOUT>
            % Convert datatype of reference load to preserve input datatype
        end

        function out=updateCCDF(~) %#ok<STOUT>
        end

        function out=validateInputsImpl(~) %#ok<STOUT>
        end

        function out=validatePropertiesImpl(~) %#ok<STOUT>
            % validate properties
        end

    end
    properties
        % ComputeCCDF Compute CCDF
        % Specify whether to calculate CCDF. The default is false.
        % If this property is set to true, all input samples since
        % object creation or reset are used to collect statistics, and
        % WindowLength and OverlapLength become read-only properties and are
        % set to Inf and 0 respectively.
        ComputeCCDF;

        % Measurement Measurement
        % Specify the desired power measurement as one of "Average power" |
        % "Peak power" | "Peak-to-average power ratio" | "All". The
        % default is "Average power".
        Measurement;

        % OutputPowerUnits Output power units
        % Specify the output power units as one of "dBm" | "dBW" | "Watts". The
        % default is "dBm".
        OutputPowerUnits;

        % OverlapLength Overlap length
        % Specify the overlap length between sliding windows as a positive
        % integer. The default is WindowLength-1. If ComputeCCDF is true, this
        % property is set to 0 and becomes read-only.
        OverlapLength;

        % PowerRange Power range (dB)
        % Specify the x-axis range of the computed CCDF curves as a real,
        % positive scalar in dB. The default is 50 dB. The computed CCDF
        % curves end at the maximum relative power, i.e. PAPR of the signal,
        % and start at (PAPR - PowerRange). This property can be used only
        % when ComputeCCDF is true.
        PowerRange;

        % PowerResolution Power resolution (dB)
        % Specify the x-axis resolution of the computed CCDF curves as a real,
        % positive scalar in dB. The default is 0.1 dB. This property can be
        % used only when ComputeCCDF is true.
        PowerResolution;

        % ReferenceLoad Reference load (ohms)
        % Specify the load that the power meter uses as a reference to
        % compute power values as a real, positive scalar in ohms. The
        % default is 1 ohm. This property is tunable.
        ReferenceLoad;

        % WindowLength Window length
        % Specify the sliding window length as a positive integer.
        % The default is 256. If ComputeCCDF is true, this property is set to
        % Inf and becomes read-only.
        WindowLength;

        pMovingAvg;

        pMovingMax;

    end
end
