classdef (StrictDefaults)powermeter < dsp.internal.SupportScalarVarSize
%powermeter Measure power, peak-to-average power ratio (PAPR), and the
%complementary cumulative distribution function (CCDF) of the power of
%voltage signal
%   METER = POWERMETER returns a System object, METER, that computes the
%   power, peak-to-average power ratio (PAPR), and the complementary
%   cumulative distribution function (CCDF) of the power of a voltage signal.
%   The CCDF helps to find the probability that the instantaneous signal
%   power exceeds a specified level above the average signal power.
%
%   METER = POWERMETER(Name=Value, ...) returns a power meter
%   System object, METER, with each specified property name set to the
%   specified value. You can specify additional name-value pair arguments
%   in any order as (Name1=Value1, ..., NameN=ValueN).
%
%   METER = POWERMETER(Length, Overlap, Name=Value, ...) returns a power
%   meter object, METER, with the WindowLength property set to Length,
%   OverlapLength property set to Overlap, and other properties set to the
%   specified values. This syntax is valid if the ComputeCCDF property is
%   set to false, which is the default.
%
%   Step method syntax:
%
%   AVGPWR = step(METER, X) computes the average power, AVGPWR, of the
%   input X when the Measurement property is set to "Average power". Each
%   column of X is a channel. The System object computes the power of each
%   channel of the input signal independently.
%
%   PEAKPWR = step(METER, X) computes the peak power, PEAKPWR, of the
%   input X when the Measurement property is set to "Peak power". Each
%   column of X is a channel. The System object computes the power of each
%   channel of the input signal independently.
%
%   PAPR = step(METER, X) computes the peak-to-average power ratio, PAPR,
%   of the input X when the Measurement property is set to "Peak-to-average
%   power ratio". Each column of X is a channel. The System object computes
%   the power of each channel of the input signal independently.
%
%   [AVGPWR, PEAKPWR, PAPR] = step(METER, X) computes the average power,
%   AVGPWR, the peak power, PEAKPWR, and the peak-to-average power ratio,
%   PAPR, of the input X when the Measurement property is set to "All".
%   Each column of X is a channel. The System object computes the power of
%   each channel of the input signal independently.
%
%   System objects may be called directly like a function instead of using
%   the step method. For example, y = step(obj, x) and y = obj(x) are
%   equivalent.
%
%   POWERMETER methods:
%
%   step           - Measure power of the input
%   release        - Allow property value and input characteristics
%                    changes, and release sliding window resources
%   clone          - Create power meter object with same property values
%   isLocked       - Display locked status (logical)
%   reset          - Reset the states of the object
%   plotCCDF       - Plot CCDF curves. Must set ComputeCCDF to true beforehand.
%                    The x-axis represents relative power, the ratio of
%                    instantaneous signal power to average signal power in dB.
%                    The y-axis represents the probability (in percent) that
%                    the relative power exceeds a specified dB level
%   ccdf           - Get the coordinates of CCDF curves. Must set ComputeCCDF
%                    to true beforehand
%   relativePower  - Use CCDF to find the relative power for a specified
%                    probability. Must set ComputeCCDF to true beforehand
%   probability    - Use CCDF to find the probability for a specified
%                    relative power. Must set ComputeCCDF to true beforehand
%
%   POWERMETER properties:
%
%   Measurement      - Desired power measurement
%   ReferenceLoad    - Reference load
%   ComputeCCDF      - Enable calculation of CCDF. All input samples since object
%                      creation or reset are used to collect statistics
%   PowerRange       - The x-axis range of the computed CCDF curves in dB.
%                      The computed CCDF curves end at the maximum relative power,
%                      i.e. PAPR of the signal, and start at PAPR minus PowerRange
%   PowerResolution  - The x-axis resolution of the computed CCDF curves in dB
%   WindowLength     - Sliding window length. Used if ComputeCCDF is false
%   OverlapLength    - Overlap length between windows. Used if ComputeCCDF is false
%   OutputPowerUnits - Power units
%
%  % EXAMPLE 1: Compute the average power of a 256-QAM signal using
%  % a sliding window of 16 samples with default overlap, and a sliding 
%  % window of 16 samples with an overlap of 10 samples.
%  FrameLength = 512;
%  Fs = 1000;
%  pm  = powermeter(16, Measurement="Average power", ...
%                       ReferenceLoad=50, ...
%                       OutputPowerUnits="dBm");
%  pm_overlap  = powermeter(16, 10, Measurement="Average power", ...
%                       ReferenceLoad=50, ...
%                       OutputPowerUnits="dBm");
%  scope  = timescope(SampleRate=[Fs,Fs/(16-10)],...
%                     TimeSpanSource="property",...
%                     TimeSpan=20,...
%                     YLabel="dBm",...
%                     YLimits=[0 50], ...
%                     Title="Average Power", ...
%                     ChannelNames = ["Sliding window of 16 samples with default overlap",...
%      "Sliding window of 16 samples with overlap of 10 samples"]);
%  tic;
%  while (toc < 5)
%    x = randi([0 255], FrameLength, 1);
%    y = qammod(x, 256);
%    averagePower = pm(y);
%    averagePower_overlap = pm_overlap(y);
%    scope(averagePower,averagePower_overlap);
%  end
%
%  % EXAMPLE 2: Find the probability that the instantaneous power of a signal
%  % is 3 dB above the average power.
%  x = complex(rand(10000,1)-0.5,rand(10000,1)-0.5);
%  pm = powermeter(ComputeCCDF=true);
%  averagePower = pm(x)            % Power in dBm. Default ReferenceLoad = 1 ohm 
%  prob = probability(pm,3)        % Probability in percent
%  plotCCDF(pm,GaussianReference=true)
%  relpwr = relativePower(pm,prob) % Find relative power for a given probability
%
%  See also dsp.MovingAverage, dsp.MovingRMS, dsp.MovingVariance,
%  dsp.MovingMaximum.

%   Copyright 2020-2022 The MathWorks, Inc.

%#codegen

    properties (Nontunable)
        % Measurement Measurement
        % Specify the desired power measurement as one of "Average power" |
        % "Peak power" | "Peak-to-average power ratio" | "All". The
        % default is "Average power".
        Measurement = 'Average power'
    end

    properties
        % ReferenceLoad Reference load (ohms)
        % Specify the load that the power meter uses as a reference to
        % compute power values as a real, positive scalar in ohms. The
        % default is 1 ohm. This property is tunable.
        ReferenceLoad = 1
    end

    properties (Nontunable,Hidden,Dependent)
        % PowerUnits Output power units
        % Specify the output power units as one of "dBm" | "dBW" | "Watts".
        % The default is "dBm".
        PowerUnits
    end

    properties (Nontunable, Logical)
        % ComputeCCDF Compute CCDF
        % Specify whether to calculate CCDF. The default is false.
        % If this property is set to true, all input samples since
        % object creation or reset are used to collect statistics, and
        % WindowLength and OverlapLength become read-only properties and are
        % set to Inf and 0 respectively.
        ComputeCCDF (1,1) = false
    end

    properties (Nontunable)
        % PowerRange Power range (dB)
        % Specify the x-axis range of the computed CCDF curves as a real,
        % positive scalar in dB. The default is 50 dB. The computed CCDF
        % curves end at the maximum relative power, i.e. PAPR of the signal,
        % and start at (PAPR - PowerRange). This property can be used only
        % when ComputeCCDF is true.
        PowerRange (1,1) = 50
        % PowerResolution Power resolution (dB)
        % Specify the x-axis resolution of the computed CCDF curves as a real,
        % positive scalar in dB. The default is 0.1 dB. This property can be
        % used only when ComputeCCDF is true.
        PowerResolution (1,1) = 0.1
        % WindowLength Window length
        % Specify the sliding window length as a positive integer.
        % The default is 256. If ComputeCCDF is true, this property is set to
        % Inf and becomes read-only.
        WindowLength (1, 1) {mustBePositive, mustBeInteger} = 256
        % OverlapLength Overlap length
        % Specify the overlap length between sliding windows as a positive
        % integer. The default is WindowLength-1. If ComputeCCDF is true, this
        % property is set to 0 and becomes read-only.
        OverlapLength;
        % OutputPowerUnits Output power units
        % Specify the output power units as one of "dBm" | "dBW" | "Watts". The
        % default is "dBm".
        OutputPowerUnits = 'dBm'
    end

    properties(Constant, Hidden)
        OutputPowerUnitsSet = matlab.system.StringSet( { ...
            'dBm', ...
            'dBW', ...
            'Watts'} );
        MeasurementSet = matlab.system.StringSet( { ...
            'Average power', ...
            'Peak power', ...
            'Peak-to-average power ratio', ...
            'All'} );
    end

    properties(Access = protected)
        pMovingAvg
        pMovingMax
    end

    properties(Access = private, Nontunable)
        % pNumChannels Cached number of channels (used to allocate state)
        pNumChannels
        %pOverlapLengthSource OverlapLength source
        pOverlapLengthSource = 'auto';
        %pAutoOverlapLength Auto OverlapLength value
        pAutoOverlapLength = 255;
        %pUserDefinedOverlapLength User-defined OverlapLength
        pUserDefinedOverlapLength = 255;
        %pDataType Input data type
        pDataType;
        % numBins number of bins in histogram for CCDF
        numBins;
    end

    properties(Access=private)
        counts         % histogram for computing CCDF
        maxpwr         % maximum signal power based on an infinite window
        meanpwr        % mean signal power based on an infinite window
        numPastSamples % number of finite-power samples received so far
        N0             % bin location of the maximum signal power
        NumChannels    % Cached number of channels (used for error checking)
        FrameLength    % Cached input frame length (used for error checking)
    end
    %----------------------------------------------------------------------
    % Public methods
    %----------------------------------------------------------------------
    methods
        %------------------------------------------------------------------
        function [x,y] = ccdf(obj)
        % ccdf Get the coordinates of the CCDF curves
        %   [RELPOWER, PROB] = ccdf(METER) returns the coordinates of the CCDF curves. ComputeCCDF must
        %   be set to true before this method is called.
        %
        %   RELPOWER and PROB are double precision matrices of the same dimensions. The number of
        %   columns is the number of columns in the signals provided to the step method. The number
        %   of rows is ceil(PowerRange/PowerResolution)+1. The probability that the instantaneous signal
        %   power of the j-th channel is above its average signal power by RELPOWER(i,j) dB is PROB(i)/100.

            coder.internal.errorIf(~obj.ComputeCCDF, ...
                'dsp:system:powermeter:ComputeCCDFNotTrue');
            coder.internal.errorIf(~isLocked(obj) || min(obj.numPastSamples) == 0, ...
                'dsp:system:powermeter:NoCCDFData');
            dt = getDataType(obj);
            x = coder.nullcopy(zeros(ceil(obj.PowerRange/obj.PowerResolution)+1,obj.pNumChannels, dt));
            y = coder.nullcopy(zeros(ceil(obj.PowerRange/obj.PowerResolution)+1,obj.pNumChannels, dt));
            for i = 1:obj.pNumChannels
                myBinEdges = ((-obj.numBins:0)+obj.N0(i))*obj.PowerResolution;
                avgpwr = real(10*log10(obj.meanpwr(:,i)));
                x(:,i) = (myBinEdges - avgpwr)';
                y(:,i) = [cumsum(obj.counts(:,i),'reverse'); 0]/obj.numPastSamples(i)*100;
            end
        end
        %------------------------------------------------------------------
        function prob = probability(obj, relPower)
        % probability Use CCDF to find the probability for a specified relative power
        %   PROB = probability(METER, RELPOWER) finds the probability PROB in percent that the instantaneous
        %   signal power is above the average signal power by RELPOWER dB. ComputeCCDF must be set to true
        %   before this method is called.
        %
        %   RELPOWER must be a double or single precision column vector. PROB is a double precision matrix that
        %   has the same number of rows as RELPOWER and has the same number of columns as the signals provided to the
        %   step method. The probability that the instantaneous signal power of the j-th channel is above its average
        %   signal power by RELPOWER(i,j) dB is PROB(i)/100.
        %
        %   If the PowerRange property is not set sufficiently large, the computed CCDF curves may not reach
        %   some relative power values in RELPOWER. In those cases, the corresponding values in PROB will be NaN.

            coder.internal.errorIf(~obj.ComputeCCDF, ...
                'dsp:system:powermeter:ComputeCCDFNotTrue');
            coder.internal.errorIf(~isLocked(obj) || min(obj.numPastSamples) == 0, ...
                'dsp:system:powermeter:NoCCDFData');

            validateattributes(relPower,{'single','double'},{'real','finite','nonnan','column'},'powermeter','relative power');
            dt = getDataType(obj);
            prob = coder.nullcopy(zeros(size(relPower,1),obj.pNumChannels, dt));
            avgpwr = real(10*log10(obj.meanpwr));
            for i = 1:obj.pNumChannels
                instpwr = relPower + avgpwr(i);
                localCounts = cumsum(obj.counts(:,i),'reverse');
                loc = floor(instpwr/obj.PowerResolution) - obj.N0(i) + obj.numBins;
                for j = 1:size(relPower,1)
                    if loc(j) < 0
                        prob(j,i) = NaN;
                    elseif loc(j) >= obj.numBins
                        prob(j,i) = 0;
                    else
                        p1 = localCounts(loc(j)+1);
                        if loc(j) == (obj.numBins -1)
                            p2 = cast(0, dt);
                        else
                            p2 = localCounts(loc(j)+2);
                        end
                        leftEdge = (obj.N0(i)-obj.numBins+loc(j))*obj.PowerResolution;
                        prob(j,i) = (p1+(p2-p1)/obj.PowerResolution*(instpwr(j)-leftEdge))/obj.numPastSamples(i)*100;
                    end
                end
            end
        end
        %------------------------------------------------------------------
        function relPower = relativePower(obj, varargin)
        % relativePower Use CCDF to find the relative power for a specified probability
        %   RELPOWER = relativePower(METER, PROB) finds the relative power, RELPOWER, for the specified probability
        %   PROB, such that the instantaneous signal power is above the average signal power by RELPOWER dB with
        %   probability PROB/100. ComputeCCDF must be set to true before this method is called.
        %
        %   PROB must be a double or single precision non-negative column vector. RELPOWER is a double precision
        %   matrix that has the same number of rows as PROB and has the same number of columns as the signals
        %   provided to the step method. The probability that the instantaneous signal power of the j-th channel
        %   is above its average signal power by RELPOWER(i,j) dB is PROB(i)/100.
        %
        %   If the PowerRange property is not set sufficiently large, the computed CCDF curves may not reach
        %   some probability values in PROB. In those cases, the corresponding values in RELPOWER will be NaN.
        %
        %   If PROB is not provided, relativePower will return results for PROB = [10 1 0.1 0.01 0.001 0.0001]'.

            coder.internal.errorIf(~obj.ComputeCCDF, ...
                'dsp:system:powermeter:ComputeCCDFNotTrue');
            coder.internal.errorIf(~isLocked(obj) || min(obj.numPastSamples) == 0, ...
                'dsp:system:powermeter:NoCCDFData');
            
            narginchk(1,2);
            if nargin == 2
                prob = varargin{1};
            else
                prob = 10.^(1:-1:-4)'; % [10 1 0.1 0.01 0.001 0.0001]'
            end
            validateattributes(prob,{'single','double'},{'real','nonnegative','finite','nonnan','column','<=',100},'powermeter','probability');

            relPower = coder.nullcopy(zeros(size(prob,1),obj.pNumChannels,getDataType(obj)));
            avgpwr = real(10*log10(obj.meanpwr));
            for i = 1:obj.pNumChannels
                p = [cumsum(obj.counts(:,i),'reverse'); 0];
                for j = 1:size(prob,1)
                    if prob(j) == 0
                        relPower(j,i) = obj.N0(i)*obj.PowerResolution - avgpwr(i);
                    else
                        count = prob(j)/100*obj.numPastSamples(i);
                        loc = find(count>p,1);
                        if loc == 1
                            relPower(j,i) = NaN;
                        else
                            p1 = p(loc-1);
                            if p1 ~= count
                                % prob(j) does not hit a corner point or a
                                % horizontal segment of the CCDF curve
                                p2 = p(loc);
                                rightEdge = (obj.N0(i)-obj.numBins+loc-1)*obj.PowerResolution;
                                relPower(j,i) = rightEdge - (count-p2)/(p1-p2)*obj.PowerResolution - avgpwr(i);
                            else
                                % prob(j) hits a corner point or a
                                % horizontal segment of the CCDF curve
                                loc = find(count==p,1); % Get the minimum power level
                                relPower(j,i) = (obj.N0(i)-obj.numBins+loc-1)*obj.PowerResolution - avgpwr(i);
                            end
                        end
                    end
                end
            end
        end
        %------------------------------------------------------------------
        function varargout = plotCCDF(obj,varargin)
        % plotCCDF Plot CCDF curves
        %   H = plotCCDF(METER) plots CCDF measurements in the powermeter System object, METER,
        %   and returns the plot handles as an output, H. This method plots as
        %   many curves as the number of input channels.
        %
        %   H = plotCCDF(METER, AX) plots CCDF measurements using the axes object AX.
        %
        %   H = plotCCDF(..., Name, Value) specifies options using one or more name-value arguments.
        %   Name-value arguments accepted by MATLAB's plot function can be used. For example,
        %   plotCCDF(METER,'LineWidth',2) will plot CCDF curves with line widths of 2 points.
        %   An additional name-value argument is supported:
        %
        %   'GaussianReference' - Set this property to true to plot a reference CCDF curve.
        %                         The reference CCDF curve represents the power of a complex
        %                         white Gaussian noise, calculated as a chi-squared distribution.
        %                         The default is false.

            coder.internal.errorIf(~isempty(coder.target), ...
                'dsp:system:powermeter:NoCodegenForPlotCCDF');
            coder.internal.errorIf(~obj.ComputeCCDF, ...
                'dsp:system:powermeter:ComputeCCDFNotTrue');
            coder.internal.errorIf(~isLocked(obj) || min(obj.numPastSamples) == 0, ...
                'dsp:system:powermeter:NoCCDFData');

            useCustomAxes = false;
            plotGaussianRef = false;
            if nargin > 1
                ax = varargin{1};
                useCustomAxes = isa(ax,'matlab.graphics.axis.Axes');
                if useCustomAxes
                    argList = varargin(2:end);
                else
                    argList = varargin;
                end
                k = [];
                for i = 1:length(argList)
                    if strcmpi(argList{i},'GaussianReference')
                        if i < length(argList)
                            validateattributes(argList{i+1}, {'logical'}, ...
                                {'scalar'}, 'powermeter', 'GaussianReference');
                            plotGaussianRef = argList{i+1};
                            k = [i i+1];
                        else
                            k = i;
                        end
                        break;
                    end
                end
                argList(k) = [];
            else
                argList = {};
            end

            [x,y] = ccdf(obj);
            y(end,:) = 1e-300;
            if useCustomAxes
                plotHandle = semilogy(ax,x,y,argList{:});
                actualAx = ax;                
            else
                plotHandle = semilogy(x,y,argList{:});
                actualAx = gca;
            end

            minProb=min(y(end-1,:)); % The smallest positive probability
            exponent = log10(minProb);
            if exponent == floor(exponent)
                exponent = exponent - 1; % This ensures that the last "step" of the curve does not coincide with the x-axis
            else
                exponent = floor(exponent);
            end
            minProb = 10^(exponent);
            if minProb == 100
                minProb = 0.01;
            end
            if plotGaussianRef
                % Plot Gaussian reference
                hold(actualAx,'on')
                xMin = min(0,min(x(1,:)));
                xMax = max(max(x(end,:)),10*log10(gammaincinv(minProb/100,1,'upper')));
                xData = xMin:0.01:xMax;
                yData = 100 * gammainc(10 .^ (xData/10), 1, 'upper');
                semilogy(actualAx,xData,yData,'Color','black','LineStyle','--');
                hold(actualAx,'off')
            end

            set(actualAx,'YLim',[minProb 100])
            xLim = get(actualAx,'Xlim');
            set(actualAx,'XLim',[0 xLim(2)])
            legendStr = cell(1,obj.pNumChannels);
            for i = 1:obj.pNumChannels
                legendStr{i} = getString(message('dsp:system:powermeter:PlotCCDFLegend',num2str(i)));
            end
            if plotGaussianRef
                legendStr{end+1} = getString(message('dsp:system:powermeter:PlotCCDFGaussianRefLegend'));
            end
            legend(actualAx, legendStr{:})
            title(actualAx,getString(message('dsp:system:powermeter:PlotCCDFTitle')))
            xlabel(actualAx,getString(message('dsp:system:powermeter:PlotCCDFXLabel')))
            ylabel(actualAx,getString(message('dsp:system:powermeter:PlotCCDFYLabel')))
            grid(actualAx,'on')

            if nargout > 0
                varargout{1} = plotHandle;
            end
            set(plotHandle,'Tag','CCDF Plot');
        end
    end
    %----------------------------------------------------------------------
    % Public properties set/get
    %----------------------------------------------------------------------
    methods
        %------------------------------------------------------------------
        % Constructor
        function obj = powermeter(varargin)
            setProperties(obj, nargin, varargin{:}, 'WindowLength','OverlapLength');            
            obj.NumChannels = int32(-1);
            obj.FrameLength = int32(-1);
        end
        %------------------------------------------------------------------
        function set.ReferenceLoad(obj, value)
            validateattributes(value,{'single','double'},{'real','positive','scalar','finite','nonnan'},'powermeter','ReferenceLoad');
            obj.ReferenceLoad = value;
        end
        %------------------------------------------------------------------
        function set.PowerUnits(obj, P)
            obj.OutputPowerUnits = P;
        end
        function P = get.PowerUnits(obj)
            P = obj.OutputPowerUnits;
        end
        %------------------------------------------------------------------
        function set.WindowLength(obj,L)
            if isComputingCCDF(obj) && getExecPlatformIndex(obj) ~= 1
                % In MATLAB
                error(message('dsp:system:powermeter:CCDFReadOnlyProperty','WindowLength'));
            else
                validateattributes(L,...
                    {'numeric'},...
                    {'real','scalar','positive','integer','finite'},...
                    'powermeter','WindowLength');
                obj.WindowLength = L;
                setAutoOverlapLength(obj);
            end
        end
        function L = get.WindowLength(obj)
            if obj.ComputeCCDF
                L = Inf;
            else
                L = obj.WindowLength;
            end
        end
        %------------------------------------------------------------------
        function set.OverlapLength(obj, L)
            if isComputingCCDF(obj) && getExecPlatformIndex(obj) ~= 1
                % In MATLAB
                error(message('dsp:system:powermeter:CCDFReadOnlyProperty','OverlapLength'));
            else
                validateattributes(L,...
                    {'numeric'},...
                    {'real','scalar','nonnegative','integer','finite'},...
                    'powermeter','OverlapLength');
                setUserDefinedOverlapLength(obj,L);
            end
        end 
        function L = get.OverlapLength(obj)
            if obj.ComputeCCDF
                L = 0;
            else
                if strcmpi(obj.pOverlapLengthSource,'property')
                    L = obj.pUserDefinedOverlapLength;
                else
                    L = obj.pAutoOverlapLength;
                end
            end
        end
        %------------------------------------------------------------------
        function set.PowerRange(obj, value)
            validateattributes(value,{'single','double'},{'real','positive','scalar','finite','nonnan'},'powermeter','PowerRange');
            obj.PowerRange = value;
        end
        function set.PowerResolution(obj, value)
            validateattributes(value,{'single','double'},{'real','positive','scalar','finite','nonnan'},'powermeter','PowerResolution');
            obj.PowerResolution = value;
        end
    end
    %----------------------------------------------------------------------
    % Protected methods
    %----------------------------------------------------------------------
    methods(Access = protected)
        %------------------------------------------------------------------
        function flag = isComputingCCDF(obj)
            flag = obj.ComputeCCDF;
        end
        %------------------------------------------------------------------
        function setupImpl(obj, u)
            % Get the number of input channels
            numchans = getNumChans(obj,u);
            dt = class(u);
            obj.NumChannels  = int32(numchans);
            obj.FrameLength = int32(getFrameLength(obj,u));
            obj.pNumChannels = numchans;
            obj.pDataType = dt;
            % Disallow a first step with zero channels
            coder.internal.errorIf(numchans == 0,'dsp:system:Shared:EmptyFirstStep');

            if obj.ComputeCCDF
                obj.numBins = ceil(obj.PowerRange/obj.PowerResolution);
                obj.counts = zeros(obj.numBins,numchans, dt);
                obj.numPastSamples = zeros(1,numchans, dt);
                obj.N0 = cast(-ones(1,numchans), dt);
                obj.meanpwr = cast(NaN(1,numchans), dt);
                obj.maxpwr = cast(NaN(1,numchans), dt);
            else
                switch obj.Measurement
                    case {'Average power'}
                        if isempty(coder.target)
                            obj.pMovingAvg =  dsp.internal.SlidingWindowAverageSim('WindowLength',obj.WindowLength,'HopSize',(obj.WindowLength - obj.OverlapLength));
                        else
                            obj.pMovingAvg =  dsp.internal.SlidingWindowAverageCG('WindowLength',int32(obj.WindowLength),'HopSize',int32(obj.WindowLength - obj.OverlapLength),'isFixedOutput',obj.isFixedSizeOutput());
                        end
                        obj.pMovingAvg.NumChans = obj.pNumChannels;
                    case {'Peak power'}
                        if isempty(coder.target)
                            obj.pMovingMax = dsp.internal.SlidingWindowMaximumOverlapSim('WindowLength',obj.WindowLength,'HopSize',(obj.WindowLength - obj.OverlapLength));
                        else
                            obj.pMovingMax = dsp.internal.SlidingWindowMaximumOverlapCG('WindowLength',obj.WindowLength,'HopSize',int32(obj.WindowLength - obj.OverlapLength),'isFixedOutput',obj.isFixedSizeOutput());
                        end
                        obj.pMovingMax.NumChans = obj.pNumChannels;
                    case {'Peak-to-average power ratio', 'All'}
                        if isempty(coder.target)
                            obj.pMovingAvg =  dsp.internal.SlidingWindowAverageSim('WindowLength',obj.WindowLength,'HopSize',(obj.WindowLength - obj.OverlapLength));
                            obj.pMovingMax = dsp.internal.SlidingWindowMaximumOverlapSim('WindowLength',obj.WindowLength,'HopSize',(obj.WindowLength - obj.OverlapLength));
                        else
                            obj.pMovingAvg =  dsp.internal.SlidingWindowAverageCG('WindowLength',int32(obj.WindowLength),'HopSize',int32(obj.WindowLength - obj.OverlapLength),'isFixedOutput',obj.isFixedSizeOutput());
                            obj.pMovingMax = dsp.internal.SlidingWindowMaximumOverlapCG('WindowLength',obj.WindowLength,'HopSize',int32(obj.WindowLength - obj.OverlapLength),'isFixedOutput',obj.isFixedSizeOutput());
                        end
                        obj.pMovingAvg.NumChans = obj.pNumChannels;
                        obj.pMovingMax.NumChans = obj.pNumChannels;
                end
            end
        end
        function flag = isFixedSizeOutput(obj)
            % Varsize output if input is varsize or arbitrary input is allowed
            fixedinput = propagatedInputFixedSize(obj,1);
            sz = propagatedInputSize(obj, 1);
            WL = obj.WindowLength;
            OL = obj.OverlapLength;
            nonArbInputLength = mod(sz, WL - OL)==0;
            flag = fixedinput(1) && nonArbInputLength(1);       
        end
        %------------------------------------------------------------------
        function n = getNumChans(~,u)
            n = size(u,2);
        end
        function n = getFrameLength(obj,u)
            if obj.getExecPlatformIndex()
                thisSize = propagatedInputSize(obj, 1);
                n = thisSize(1);
            else
                n = size(u,1);
            end
        end
        %------------------------------------------------------------------
        function updateCCDF(obj,x)
            for j=1:obj.pNumChannels
                p = abs(x(isfinite(x(:,j)),j)).^2;
                if isempty(p)
                    continue;
                end
                if isnan(obj.meanpwr(j))
                    obj.meanpwr(j) = 0;
                end
                obj.meanpwr(j) = (obj.meanpwr(j)*obj.numPastSamples(j)+sum(p))/(obj.numPastSamples(j)+size(p,1));
                obj.numPastSamples(j) = obj.numPastSamples(j) + size(p,1);

                maxpwr0 = max(p);
                if isnan(obj.maxpwr(j))
                    obj.maxpwr(j) = maxpwr0;
                else
                    if maxpwr0 > obj.maxpwr(j)
                        obj.maxpwr(j) = maxpwr0;
                    end
                end

                N = real(floor(10*log10(obj.maxpwr(j))/obj.PowerResolution)+1);
                if obj.N0(j) == -1
                    obj.N0(j) = N;
                end

                d = N - obj.N0(j);
                if d > 0
                    if d >= obj.numBins
                        obj.counts(:,j) = 0;
                    else
                        obj.counts(1:(end-d),j) = obj.counts((d+1):end,j);
                        obj.counts((end-d+1):end,j) = 0;
                    end
                    obj.N0(j) = N;
                end

                instpwr = 10*log10(p);
                loc = floor(instpwr/obj.PowerResolution) - obj.N0(j) + obj.numBins;
                for i = 1:length(loc)
                    if loc(i) >= 0 && loc(i) < size(obj.counts,1)
                        obj.counts(loc(i)+1,j) = obj.counts(loc(i)+1,j)+1;
                    end
                end
            end
        end
        %------------------------------------------------------------------
        function varargout = stepImpl(obj, x)
            % Convert datatype of reference load to preserve input datatype
            referenceLoad = cast(obj.ReferenceLoad, getDataType(obj));
            if obj.ComputeCCDF
                updateCCDF(obj,x);
                switch obj.Measurement
                    case {'Average power'}
                        avgPwr = obj.meanpwr/referenceLoad;
                        switch obj.OutputPowerUnits
                            case 'dBm'
                                avgPwr = 10*log10(avgPwr) + 30;
                            case 'dBW'
                                avgPwr = 10*log10(avgPwr);
                        end
                        varargout{1} = avgPwr;

                    case {'Peak power'}
                        peakPwr = obj.maxpwr/referenceLoad;
                        switch obj.OutputPowerUnits
                            case 'dBm'
                                peakPwr = 10*log10(peakPwr) + 30;
                            case 'dBW'
                                peakPwr = 10*log10(peakPwr);
                        end
                        varargout{1} = peakPwr;

                    case {'Peak-to-average power ratio'}
                        papr = obj.maxpwr./obj.meanpwr;
                        if ~strcmp(obj.OutputPowerUnits, 'Watts')
                            papr = 10*log10(papr);
                        end
                        varargout{1} = papr;

                    case {'All'}
                        avgPwr = obj.meanpwr/referenceLoad;
                        peakPwr = obj.maxpwr/referenceLoad;
                        papr = peakPwr./avgPwr;
                        switch obj.OutputPowerUnits
                            case 'dBm'
                                avgPwr = 10*log10(avgPwr) + 30;
                                peakPwr = 10*log10(peakPwr) + 30;
                                papr = 10*log10(papr);
                            case 'dBW'
                                avgPwr = 10*log10(avgPwr);
                                peakPwr = 10*log10(peakPwr);
                                papr = 10*log10(papr);
                        end
                        varargout{1} = avgPwr;
                        varargout{2} = peakPwr;
                        varargout{3} = papr;
                end
            else
                power = real(x.*conj(x));
                switch obj.Measurement
                    case {'Average power'}
                        avgPwr = obj.pMovingAvg(power)/referenceLoad;
                        switch obj.OutputPowerUnits
                            case 'dBm'
                                avgPwr = 10*log10(avgPwr) + 30;
                            case 'dBW'
                                avgPwr = 10*log10(avgPwr);
                        end
                        varargout{1} = avgPwr;

                    case {'Peak power'}
                        peakPwr = obj.pMovingMax(power)/referenceLoad;
                        switch obj.OutputPowerUnits
                            case 'dBm'
                                peakPwr = 10*log10(peakPwr) + 30;
                            case 'dBW'
                                peakPwr = 10*log10(peakPwr);
                        end
                        varargout{1} = peakPwr;

                    case {'Peak-to-average power ratio'}
                        papr = obj.pMovingMax(power)./obj.pMovingAvg(power);
                        if ~strcmp(obj.OutputPowerUnits, 'Watts')
                            papr = 10*log10(papr);
                        end
                        varargout{1} = papr;

                    case {'All'}
                        avgPwr = obj.pMovingAvg(power)/referenceLoad;
                        peakPwr = obj.pMovingMax(power)/referenceLoad;
                        papr = peakPwr./avgPwr;
                        switch obj.OutputPowerUnits
                            case 'dBm'
                                avgPwr = 10*log10(avgPwr) + 30;
                                peakPwr = 10*log10(peakPwr) + 30;
                                papr = 10*log10(papr);
                            case 'dBW'
                                avgPwr = 10*log10(avgPwr);
                                peakPwr = 10*log10(peakPwr);
                                papr = 10*log10(papr);
                        end
                        varargout{1} = avgPwr;
                        varargout{2} = peakPwr;
                        varargout{3} = papr;
                end
            end
        end
        %------------------------------------------------------------------
        function resetImpl(obj)
            if ~obj.ComputeCCDF
                switch obj.Measurement
                    case {'Average power'}
                        reset(obj.pMovingAvg);
                    case {'Peak power'}
                        reset(obj.pMovingMax);
                    case {'Peak-to-average power ratio', 'All'}
                        reset(obj.pMovingAvg);
                        reset(obj.pMovingMax);
                end
            else
                obj.counts(:) = 0;
                obj.numPastSamples(:) = 0;
                obj.N0(:) = -1;
                obj.meanpwr(:) = NaN;
                obj.maxpwr(:) = NaN;
            end
        end
        %------------------------------------------------------------------
        function releaseImpl(obj)
            if ~obj.ComputeCCDF
                switch obj.Measurement
                    case {'Average power'}
                        release(obj.pMovingAvg);
                    case {'Peak power'}
                        release(obj.pMovingMax);
                    case {'Peak-to-average power ratio', 'All'}
                        release(obj.pMovingAvg);
                        release(obj.pMovingMax);
                end
            end
            obj.NumChannels = int32(-1);
            obj.FrameLength = int32(-1);
        end
        %------------------------------------------------------------------
        % Backup/restore functions
        function s = saveObjectImpl(obj)
            s = saveObjectImpl@matlab.System(obj);
            s.pOverlapLengthSource = obj.pOverlapLengthSource;
            s.pAutoOverlapLength = obj.pAutoOverlapLength;
            s.pUserDefinedOverlapLength = obj.pUserDefinedOverlapLength;
            if isLocked(obj)
                if obj.ComputeCCDF
                    s.counts = obj.counts;
                    s.meanpwr = obj.meanpwr;
                    s.maxpwr = obj.maxpwr;
                    s.numPastSamples = obj.numPastSamples;
                    s.N0 = obj.N0;
                    s.numBins = obj.numBins;
                else
                    switch obj.Measurement
                        case {'Average power'}
                            s.pMovingAvg = matlab.System.saveObject(obj.pMovingAvg);
                        case {'Peak power'}
                            s.pMovingMax = matlab.System.saveObject(obj.pMovingMax);
                        case {'Peak-to-average power ratio', 'All'}
                            s.pMovingAvg = matlab.System.saveObject(obj.pMovingAvg);
                            s.pMovingMax = matlab.System.saveObject(obj.pMovingMax);
                    end
                end
                s.NumChannels = obj.NumChannels;
                s.pNumChannels = obj.pNumChannels;
                s.pDataType = obj.pDataType;
            end
        end
        %------------------------------------------------------------------
        function loadObjectImpl(obj,s,wasLocked)
            if wasLocked
                if isfield(s,'ComputeCCDF') && s.ComputeCCDF
                    obj.counts = s.counts;
                    obj.meanpwr = s.meanpwr;
                    obj.maxpwr = s.maxpwr;
                    obj.numPastSamples = s.numPastSamples;
                    obj.N0 = s.N0;
                    obj.numBins = s.numBins;
                else
                    switch s.Measurement
                        case {'Average power'}
                            obj.pMovingAvg = matlab.System.loadObject(s.pMovingAvg);
                        case {'Peak power'}
                            obj.pMovingMax = matlab.System.loadObject(s.pMovingMax);
                        case {'Peak-to-average power ratio', 'All'}
                            obj.pMovingAvg = matlab.System.loadObject(s.pMovingAvg);
                            obj.pMovingMax = matlab.System.loadObject(s.pMovingMax);
                    end
                end
                obj.NumChannels = s.NumChannels;
                obj.pNumChannels = s.pNumChannels;
                if isfield(s,'pDataType')
                    obj.pDataType = s.pDataType;
                end
            end
            if isfield(s,'ComputeCCDF') && s.ComputeCCDF
                s = rmfield(s,'WindowLength');
                s = rmfield(s,'OverlapLength');
            end
            if (isfield(s,'pOverlapLengthSource'))
                obj.pOverlapLengthSource = s.pOverlapLengthSource;
            end
            if (isfield(s,'pAutoOverlapLength'))
                obj.pAutoOverlapLength = s.pAutoOverlapLength;
            end
            if (isfield(s,'pUserDefinedOverlapLength'))
                obj.pUserDefinedOverlapLength = s.pUserDefinedOverlapLength;
            end 
            loadObjectImpl@matlab.System(obj,s,wasLocked);
        end
        %------------------------------------------------------------------
        function validateInputsImpl(obj, u)
            validateattributes(u, {'single', 'double'}, {'2d'}, '', '')
            if obj.NumChannels ~= -1
                coder.internal.errorIf(size(u,2) ~= obj.NumChannels, ['dsp:system',...
                    ':Shared:numChannels']);
            end
            if obj.FrameLength ~= -1
                ArbInputLength = mod(size(u,1), obj.WindowLength - obj.OverlapLength)~=0;
                inputLengthChanged = (size(u,1) ~= obj.FrameLength);
                coder.internal.errorIf(obj.isFixedSizeOutput && inputLengthChanged && ArbInputLength, 'dsp:system:MovingStatistic:inputSizeChangeUndefined');
            end
        end
        %------------------------------------------------------------------
        function validatePropertiesImpl(obj)
            % validate properties
            O = obj.OverlapLength;
            L = obj.WindowLength;
            % Throw an error if the specified OverlapLength is not 
            % less than the WindowLength
            coder.internal.errorIf(L <= O,['dsp:system',...
                ':MovingStatistic:invalidOverlapLength'],L);
        end
        %------------------------------------------------------------------
        function flag = isInactivePropertyImpl(obj,prop)
            if strcmpi(prop,'WindowLength2')
                flag = true;
            elseif obj.ComputeCCDF
                if any(strcmpi(prop, {'PowerResolution','PowerRange'}))
                    flag = false;
                elseif strcmpi(prop,'ProbVec') && ~strcmpi(obj.CCDFOutput,'Relative power (dB above average power)')
                    flag = true;
                elseif strcmpi(prop,'RelPowerVec') && ~strcmpi(obj.CCDFOutput,'Probability (%)')
                    flag = true;
                else
                    flag = false;
                end
            else
                if any(strcmpi(prop, {'PowerResolution','PowerRange','CCDFOutput','ProbVec','RelPowerVec','ResetPort'}))
                    flag = true;
                else
                    flag = false;
                end
            end
        end
        %------------------------------------------------------------------
        function dt = getDataType(obj)
            dt = obj.pDataType;
        end
        %------------------------------------------------------------------
        function num = getNumOutputsImpl(obj)
            if strcmp(obj.Measurement, 'All')
                num = 3;
            else
                num = 1;
            end
        end
        %------------------------------------------------------------------
        function flag = isInputSizeMutableImpl(~,~)
            flag = true;
        end
        %------------------------------------------------------------------
        function setUserDefinedOverlapLength(obj,L)
            % If the user set the overlap length, set the overlap 
            % length factor source to 'property'            
            obj.pOverlapLengthSource = 'property';
            obj.pUserDefinedOverlapLength = L;            
        end
        %------------------------------------------------------------------
        function setAutoOverlapLength(obj)
            % If the user did not set the overlap length, it should
            % be equal to WindowLength - 1 
            obj.pAutoOverlapLength = obj.WindowLength - 1; 
        end 
        %------------------------------------------------------------------
        function groups = getPropertyGroups(obj)
            propName  = properties(obj);
            propVal = cell(numel(propName),1);
            activeIdx = true(size(propName));
            for n = 1:numel(propName)
                if isInactiveProperty(obj,propName{n}) || strcmp(propName{n},'PowerUnits')
                    % If it is inactive then do not add it to the property list
                    activeIdx(n) = false;
                else
                    propVal{n} = obj.(propName{n});
                end
            end
            if obj.ComputeCCDF
                readOnlyIdx = strcmp(propName,'WindowLength') | strcmp(propName,'OverlapLength');
                readOnlyPropList = cell2struct(propVal(activeIdx&readOnlyIdx),propName(activeIdx&readOnlyIdx));
                normalPropList = cell2struct(propVal(activeIdx&~readOnlyIdx),propName(activeIdx&~readOnlyIdx));
                groups = [matlab.mixin.util.PropertyGroup(normalPropList) matlab.mixin.util.PropertyGroup(readOnlyPropList,'Read-only properties:')];
            else
                normalPropList = cell2struct(propVal(activeIdx),propName(activeIdx));
                groups = matlab.mixin.util.PropertyGroup(normalPropList);
            end
        end
    end
end
