classdef ArrayPlotVisual < matlabshared.scopes.visual.DisplayVisual
    %ArrayPlotVisual ArrayPlotVisual Define the ArrayPlotVisual class.
    
    %   Copyright 2012-2018 The MathWorks, Inc.
    
    properties
        DialogFunction
        % Below are properties that bridge the display to the properties
        % dialog. The properties dialog has widgets that sync with each of
        % these, and updating a widget will result in the corresponding
        % property being set. In that case, the display specified in
        % SelectedDisplay will be updated. However, when SelectedDisplay
        % changes, we want to refresh the dialog with the new selection's
        % values. In that case, we want to update these properties but
        % without updating the display. The SelectedDisplayUpdateEnabled
        % property is used for this.
        SelectedDisplayXLabel
        SelectedDisplayMinYLimReal
        SelectedDisplayMaxYLimReal
        SelectedDisplayYLabelReal
        SelectedDisplayMinYLimMag
        SelectedDisplayMaxYLimMag
        SelectedDisplayTitle
        SelectedDisplayLegend
        SelectedDisplayGrid
        SelectedDisplayPlotMagPhase
    end
    
    properties (Hidden)
        DialogMgr
        StyleDialog
        SelectedDialogTab = 'main'
        IsOptionsDialogOpen = false
        IsStyleDialogOpen = false
        MissingDataWarningThrown = false
        FrameProcessingAllowed = true
        IsUpdatingStyle = false
    end
    
    properties (Access=protected)
        XOffset
        % Cache the frame length of the current inputs to handle varsize
        CurrentFrameLength = 10;
        MarkerEnabledMessageShown = false;
        CustomXDataInsufficientMessageShown = false;
        % SourceDataCache Cache for source data properties.  The cache is
        % used so that there is no need to re-compute values when
        % updateDisplays is called.
        SourceDataCache
        TotalTraceCount
        DisplayTraceCount
        SynchronousChangedListener
        SourceListeners
        % ScopeCloseListener - For deleting style dialog when Scope is
        % deleted
        ScopeCloseListener
        % ScopeVisibleListener - For hiding style dialog when Scope is
        % hidden
        ScopeVisibleListener
        % ScopeTitleListener - For updating the title of the Style Dialog
        % when Scope title changes
        ScopeTitleListener
        % FigureColorListener - For updating the color of display
        % containers when figure color changes
        FigureColorListener
        % Listeners for compact menus and toolbars
        ViewMenuListener
        PropertiesButtonListener
        Buttons
        Menus
        %NegYValuesIgnoredMessageShown - For showing a warning dialog that
        %negative values are ignored when YScale is set to log
        NegYValuesIgnoredMessageShown = false;
        %DisabledConfigurationProperties - For disabling irrelevant
        %configuration settings for dsp.ArrayPlot if used as a plotter for
        %different scopes. dsp.DynamicFilterVisualizer is an example
        DisabledConfigurationProperties = {''};
    end
    
    methods
        function this = ArrayPlotVisual(varargin)
            %ArrayPlotVisual    Construct a ArrayPlotVisual object.
            
            % Avoid clearing the visual when user clears the workspace
            mlock;
            
            % Call the superclass constructor
            this@matlabshared.scopes.visual.DisplayVisual(varargin{:});
            
            % Add listeners
            hApp = this.Application;
            this.SourceListeners = [addlistener(hApp, 'SourceRun', @this.onSourceRun), ...
                addlistener(hApp, 'SourceStopping', @this.onSourcePauseStop), ...
                addlistener(hApp, 'SourcePause', @this.onSourcePauseStop), ...
                addlistener(hApp, 'SourceContinue', @this.onSourceContinue), ...
                addlistener(hApp, 'DataSourceChanged', @this.onDataSourceChanged)];
            this.ViewMenuListener = addlistener(hApp, 'ViewMenuOpening', @this.onViewMenuOpening);
        end
        function set.SelectedDisplayXLabel(this,val)
            this.SelectedDisplayXLabel = val;
            updateSelectedDisplayProperty(this, 'XLabel', val);
        end
        function set.SelectedDisplayMinYLimReal(this,val)
            this.SelectedDisplayMinYLimReal = val;
            updateSelectedDisplayProperty(this, 'MinYLimReal', val);
        end
        function set.SelectedDisplayMaxYLimReal(this,val)
            this.SelectedDisplayMaxYLimReal = val;
            updateSelectedDisplayProperty(this, 'MaxYLimReal', val);
        end
        function set.SelectedDisplayYLabelReal(this,val)
            this.SelectedDisplayYLabelReal = val;
            updateSelectedDisplayProperty(this, 'YLabelReal', val);
        end
        function set.SelectedDisplayMinYLimMag(this,val)
            this.SelectedDisplayMinYLimMag = val;
            updateSelectedDisplayProperty(this, 'MinYLimMag', val);
        end
        function set.SelectedDisplayMaxYLimMag(this,val)
            this.SelectedDisplayMaxYLimMag = val;
            updateSelectedDisplayProperty(this, 'MaxYLimMag', val);
        end
        function set.SelectedDisplayTitle(this,val)
            this.SelectedDisplayTitle = val;
            updateSelectedDisplayProperty(this, 'Title', val);
        end
        function set.SelectedDisplayLegend(this,val)
            this.SelectedDisplayLegend = logical(val);
            updateSelectedDisplayProperty(this, 'LegendVisibility', uiservices.logicalToOnOff(val));
        end
        function set.SelectedDisplayGrid(this,val)
            this.SelectedDisplayGrid = logical(val);
            updateSelectedDisplayProperty(this, 'XGrid', val);
            updateSelectedDisplayProperty(this, 'YGrid', val);
        end
        function set.SelectedDisplayPlotMagPhase(this,val)
            % The SET method of SelectedDisplayPlotMagPhase uses
            % SelectedDisplay property. There are cases when
            % SelectedDisplayPlotMagPhase is set in a state where
            % SelectedDisplay is not yet correctly set - e.g. during set
            % method of SelectedDisplay (see updateSelectedDisplay method).
            % setSelectedDisplayPlotMagPhase method is intended to handle
            % this situation and performs all the tasks required during SET
            % of SelectedDisplayPlotMagPhase property.
            this.SelectedDisplayPlotMagPhase = logical(val);
            setSelectedDisplayPlotMagPhase(this, val, str2double(this.SelectedDisplay));
            refreshPropertiesDialog(this);
        end
        function set.CurrentFrameLength(this,val)
            this.CurrentFrameLength = val;
            updateMarkers(this,val);
        end
    end
    
    methods (Hidden)
        displayTab = createDisplayTab(this,hDlg)
        mainTab = createMainTab(this,hDlg)
        propsSchema = getPropsSchema(this,hDlg)
        onAutoscale(this,auto)
        onFigureZoom(this,e)
        onPanEnter(this,~)
        onPanRelease(this,~)
        onEditOptions(this,displayNum)
        [success,msg] = postOptionsDialogApply(this,~)
        postReset(~)
        varargout = validate(this,hDlg)
        refreshPropertiesDialog(this)
        optionsDialogTitle = getOptionsDialogTitle(~,~)
        onOptionsDialogClose(this,~)
        varargout = validateSource(this,hSource)
        dataType = getPropDataType(~,propName)
        setDisabledConfigurationProperties(obj, val)
    end
    
    methods (Static)
        hPropDb = getPropertySet
        onDialogTabChange(hDlg,~,tabInd)
    end
    
    methods (Access=protected)
        cacheTotalTraceCount(this)
        closeVisual(this)
        hInstall = createGUI(this)
        onViewMenuOpening(this, hScope, ~)
        delegate = getDisplayClass(~)
        hideVisual(this)
        onDataSourceChanged(this,~,~)
        onFigureColorChange(this)
        onLayoutChange(this,newDisplays)
        onScopeTitleChange(this)
        onSynchronousChanged(this,hSource)
        onSourceContinue(this,~,~)
        onSourcePauseStop(this,~,~)
        onSourceRun(this,~,~)
        onYLimChange(this,hDisplay)
        propertyChangedVisual(this,ev)
        setupListeners(this,~)
        cb = makeCallback(this,fcn,varargin)
        updateDisplays(this)
        updateMaximizeAxes(this)
        onAnnotationChange(this)
        onScopeVisibleChange(this)
        onPrintDisplay(this,hDisplay)
        onChannelNamesChanged(this,hDisplay)
        onChannelVisibilityChanged(this,hDisplay)
        updatePrintFigure(this,printFig,gridContainer)
        updateSelectedDisplay(this,newSelectedDisplay)
        updateXDataMode(this)
        updateCustomXData(this,showErrors)
        updateXOffset(this,newDisplays,showErrors)
        updateSampleIncrement(this)
        updateXScale(this)
        updateXSpan(this,hDisplays)
        updatePlotType(this)
    end
end
