function update(this,startTime,endTime)
%update   Update the graphics

%   Copyright 2012-2017 The MathWorks, Inc.

% Return early if in the middle of reshaping the layout
if this.ReshapingLayout
    return
end
source = this.Application.DataSource;
if nargin<3
    % Get the end time from the source.
    endTime = getTimeOfDisplayData(source);
    isSync = isSynchronous(source);
else
    isSync = true;
end

% Array Plot has only one display.
hDisplay = this.Displays{1};
if nargin<2
    startTime = hDisplay.StartTime;
end

% When the frame size is larger than the time span, make sure that the
% start time does not exceed the end time due to wrapping.
startTime = min(startTime, endTime);
% Format all the data.
allData = getData(source, startTime, endTime);
% Return early if there is no data to send to displays
if isempty(allData)
    return
end

if ~isSync
    % If in asynchronous update, the end time calculation is different.
    endTime = calculateAsynchronousEndTime(hDisplay, endTime);
end

% Determine the current frame length (trailing NaNs are ignored)
currDataFrameLength = 0;
allMaxDims = hDisplay.MaxDimensions;
isSimulink = strcmp(source.Type, 'Simulink');
thisDisplayData = allData(1:end);
for indx = 1:numel(thisDisplayData)
    if ~isempty(thisDisplayData(indx).values)
        % Draw only the last frame of data.
        thisDisplayData(indx).values = thisDisplayData(indx).values(:, end);
        thisDisplayData(indx).time = thisDisplayData(indx).time(:, end);
        % The number of channels is equal to the product of all dimensions
        % after the first. The first is the actual values of the channels.
        nChannels = prod(allMaxDims(indx, 2:end));
        % Reshape the data column vector back out so that channels
        % are in the second dimension and time in the third.
        currDataFrame = reshape(thisDisplayData(indx).values, ...
            allMaxDims(indx), nChannels, numel(thisDisplayData(indx).time));
        % Trailing NaNs represent varsize input in Simulink. Trim to find
        % the actual frame length.
        if isSimulink
            portFrameLength = find(~all(isnan(currDataFrame),2),1,'last');
            if isempty(portFrameLength)
                portFrameLength  = 0;
            end
        else
            portFrameLength = size(currDataFrame,1);
        end
        currDataFrameLength = max(currDataFrameLength,portFrameLength);
    end
end

% Update the XSpan if the frame-length has changed
if currDataFrameLength > 0 && currDataFrameLength ~= this.CurrentFrameLength
    this.CurrentFrameLength = currDataFrameLength;
    updateXSpan(this);
end

% Data has changed. Reset the cached YExtents and update the EndTime
% property of the display object.
resetYExtents(hDisplay);
hDisplay.EndTime = endTime;

% Draw the new lines
newFrameLength = validateFrameLength(this,hDisplay,currDataFrameLength);
draw(hDisplay,thisDisplayData,newFrameLength);
end

function newFrameLength = validateFrameLength(this,hDisplay,currFrameLength)
% validate CustomXData vector for the current input frame length 
if strcmp(hDisplay.XDataMode, 'Custom') && currFrameLength > 0 && ...
        currFrameLength > numel(hDisplay.CustomXData)
    newFrameLength = numel(hDisplay.CustomXData);
    if ~this.CustomXDataInsufficientMessageShown
        warningDialog(this.Application, ...
            getString(message('dsp:system:ArrayPlot:CustomXDataInsufficient')), ...
            'CustomXDataInsufficient');
        this.CustomXDataInsufficientMessageShown = true;
    end
else
    newFrameLength = currFrameLength;
    % remove any warnings for CustomXData, since the values are now valid
    if this.CustomXDataInsufficientMessageShown
        warningDialog(this.Application,'-remove','CustomXDataInsufficient');
        this.CustomXDataInsufficientMessageShown = false;
    end
end
end
