classdef ArrayPlotDisplay < matlabshared.scopes.visual.TimeDomainDisplay
%ArrayPlotDisplay  ArrayPlotDisplay  class    
    
% Copyright 2012-2018 The MathWorks, Inc.
    
    properties
        DefaultMarker;
        XScale;
        YScale;
        XDataMode = 'Sample increment and X-offset';
        SampleIncrement = 1;
    end
    
    properties (Dependent)
        CustomXData;
    end
    
    properties (Hidden)
        CurrentFrameLength = 0;
    end
    
    properties (Access=private)
        privCustomXData;
    end
    
    methods
        function this = ArrayPlotDisplay(hContainer, placementNum)
            % Call the superclass CTOR
            this = this@matlabshared.scopes.visual.TimeDomainDisplay(hContainer, placementNum);
            
            this.DefaultMarker = 'o';
            
            % This gets (sec) out of the lower right  display dialog
            % element.
            this.TimeUnits = '';
            
            % For the initializing and settingtimeproperties to false to
            % avoid issues with xlim.
            this.Initializing = false;
            this.SettingTimeProperties = false;
        end
        
        function set.DefaultMarker(this, defaultMarker)                                    
            this.DefaultMarker = defaultMarker;
            this.LinePropertyDefaults.Marker = this.DefaultMarker;
        end
        
        function set.XScale(this,value)
            if this.TimeDisplayOffset < 0 && strcmpi(value,'Log')
                error(message('dsp:system:ArrayPlot:invalidXScale'));
            end
            set(this.Axes,'XScale',value);
            this.XScale = value;
        end
        
        function set.CustomXData(this,xData)
            this.privCustomXData = xData;
        end
        
        function xData = get.CustomXData(this)
            if ~isempty(this.privCustomXData)
                xData = this.privCustomXData;
            else
                xData = 0:this.CurrentFrameLength-1;
            end
        end
        
        function set.YScale(this,value)
            %  % Suppress negative Y values ignored warning, and restore
            %  its status on cleanup
            onState = warning('off', 'MATLAB:Axes:NegativeLimitsInLogAxis');
            c = onCleanup(@() warning(onState));
            set(this.Axes,'YScale',value);
            this.YScale = value;
        end
        
        function updateCachedLinePropertiesMarkers(this)            
            for i=1:length(this.LinePropertiesCache)
                this.LinePropertiesCache{i}.Marker = this.DefaultMarker;
            end            
        end
        
        function anyLineMarkerOverridden = enableMarkers(this,dataFrameLength)
            % Temporary override to the line marker if the framelength == 1
            anyLineMarkerOverridden = false;
            hLines = this.Lines;
            if isempty(hLines)
                return;
            end
            if nargin < 2
                % All inputs have the same framelength in ArrayPlot
                dataFrameLength = numel(get(hLines(1),'XData'));
                if dataFrameLength == 1 && isnan(get(hLines(1),'YData'))
                    % scope is bring setup, no need to add markers
                    return;
                end
            end
            if any(strcmp(this.PlotType,{'Line','Stairs'}))
                for indx = 1:numel(hLines)
                    hLine = hLines(indx);
                    ud = get(hLine,'UserData');
                    if dataFrameLength == 1 && strcmp(get(hLine,'Marker'),'none')
                        % Override marker if framelength = 1 and plottype
                        % is line or stairs
                        set(hLine,'Marker','o');
                        ud.MarkerOverridden = true;
                        anyLineMarkerOverridden = (anyLineMarkerOverridden || true);
                        set(hLine,'UserData',ud);
                    elseif ~isempty(ud) && isfield(ud,'MarkerOverridden')
                        % Remove the overridden marker if framelength > 1
                        if dataFrameLength > 1 && ud.MarkerOverridden
                            set(hLine,'Marker','none');
                            ud.MarkerOverridden = false;
                            set(hLine,'UserData',ud);
                        end
                        anyLineMarkerOverridden = (anyLineMarkerOverridden || ud.MarkerOverridden);
                    end
                end
            end
        end
        
        function newXLim = calculateXLim(this, ~)
            frameLength = this.CurrentFrameLength;
            if strcmp(this.XDataMode, 'Custom')
                xData = this.CustomXData;
                if frameLength == 1 || numel(xData) == 1
                    % Show the single data point in the middle of the xspan
                    timeSpan = this.TimeSpan;
                    newXLim = xData(1) + [-timeSpan/2, timeSpan/2];
                    if strcmpi(this.XScale,'Log') && any(newXLim<0)
                        newXLim = newXLim + abs(min(newXLim));
                    end
                else
                    lastIdx = min(frameLength,numel(xData));
                    newXLim = [xData(1), xData(lastIdx)];
                end
            else
                timeDisplayOffset = this.TimeDisplayOffset;
                timeSpan          = this.TimeSpan;
                % Consider the offset value to set the limits
                if frameLength == 1 && ~(strcmpi(this.XScale,'Log') && max(timeDisplayOffset) == 0)
                    % Show the single data point in the middle of the xspan
                    newXLim = max(timeDisplayOffset) + [-timeSpan/2, timeSpan/2];
                else
                    newXLim = max(timeDisplayOffset) + [0, timeSpan];
                end
            end
        end
        
        function extents = getDataExtents(this, ~)
            % Get the data extents. This function is called by the
            % autoscaler code.
            xlim = calculateXLim(this);
            ylim1 = this.YExtents(1,:);
            ylim2 = this.YExtents(2,:);
            
            extents(1).X = xlim;
            extents(1).Y = ylim1;
            extents(2).X = xlim;
            extents(2).Y = ylim2;
        end
        
        function propStruct = getCloneProperties(this)
            %getCloneProperties   Get properties struct to create clone
            %   PROPS = getCloneProperties(THIS) returns a structure of
            %   properties PROPS that may be passed into ArrayPlotDisplay
            %   in order to create a clone of THIS display.  The clone will
            %   have the same display properties as THIS display but will
            %   not have the same content.
            
            propStruct = getCloneProperties@matlabshared.scopes.visual.TimeDomainDisplay(this);
            
            % Add XLabel property
            propStruct.XLabel = this.XLabel;
        end
        
        function updateTimeOffsetReadout(~,~)
            % Overrides updateTimeOffsetReadout in TimeDomainPlotter.
            % Does not update the readout because it is disabled.
        end
        
        function draw(this,allData,currDataFrameLength)
            % Draw the data.
            % Assumes that allData has only the latest data frame.
            lineCount = 0;
            yExtents = this.YExtents;
            allMaxDims = this.MaxDimensions;
            nAllChannels = sum(this.NumberOfChannels);
            
            % Determine xdata based on currFrameLength
            if strcmp(this.XDataMode, 'Custom')
                xData = this.CustomXData(1:currDataFrameLength);
            else
                sampleIncrement = this.SampleIncrement;
                xValues = (0:sampleIncrement:(sampleIncrement*currDataFrameLength - sampleIncrement))';
                xOffset = this.TimeDisplayOffset;
                xData = xValues + xOffset;
            end
            
            for indx = 1:numel(allData)
                d = allData(indx);
                % The number of channels is equal to the product of all
                % dimensions after the first.  The first is the actual
                % values of the channels.
                nChannels = prod(allMaxDims(indx, 2:end));
                % Reshape the data column vector back out so that channels
                % are in the second dimension and time in the third.
                d.values = reshape(d.values, allMaxDims(indx,1), nChannels, numel(d.time));
                % Trim the values up to the current frame length
                d.values = d.values(1:currDataFrameLength,:,:);
                x = repmat(xData,numel(d.time),1);
                
                % Loop over the channel count.
                for jndx = 1:nChannels
                    % Pull the frames out of the second dimension.
                    values = d.values(:, jndx, :);
                    % Reshape to align the time down the rows and get a
                    % single column to be passed to YData.
                    values = values(:);
                    lineCount = lineCount + 1;
                    if this.PlotMagPhase
                        absValues = abs(values);
                        phaseValues = angle(values)*180/pi;
                        updateLineData(this, lineCount, x, absValues);
                        updateLineData(this, nAllChannels + lineCount, x, phaseValues);
                        if ~isempty(values)
                            yExtents(1,:) = [min(yExtents(1, 1),min(absValues)), max(yExtents(1, 2),max(absValues))];
                            yExtents(2,:) = [min(yExtents(2, 1),min(phaseValues)), max(yExtents(2, 2),max(phaseValues))];
                        end
                    elseif this.IsComplex
                        if isreal(values)
                            updateLineData(this, lineCount, x, values);
                            updateLineData(this, nAllChannels + lineCount, x, zeros(length(values),1));
                            if ~isempty(values)
                                yExtents(1,:) = [min([yExtents(1, 1),min(values),0]), max([yExtents(1, 2),max(values),0])];
                            end
                        else
                            realValues = real(values);
                            imagValues = imag(values);
                            updateLineData(this, lineCount, x, realValues);
                            updateLineData(this, nAllChannels + lineCount, x, imagValues);
                            if ~isempty(values)
                                yExtents(1,:) = [min([yExtents(1,1),min(realValues),min(imagValues)]), max([yExtents(1,2),max(realValues),max(imagValues)])];
                            end
                        end
                        if ~isempty(values)
                            yExtents(2,:) = [-1, 1];
                        end
                    else
                        updateLineData(this, lineCount, x, values);
                        if ~isempty(values)
                            yExtents(1,:) = [min(yExtents(1, 1), min(values(:))), max(yExtents(1, 2), max(values))];
                            yExtents(2,:) = [-1, 1];
                        end
                    end
                end
            end
            this.YExtents = yExtents;
        end
        
    end
    
    methods(Static)
        function hDisplay = create(displayContainer, displayPlacement)
            %create   Factory method to create new ArrayPlotDisplay
            %   DISPLAY = create(CONTAINER, PLACEMENT) creates new display
            %   with parent CONTAINER and placement PLACEMENT.
            hDisplay = dsp.scopes.ArrayPlotDisplay(displayContainer, displayPlacement);
        end
        
        function hDisplay = deserialize(gridContainer, serializedDisplay)
            %deserialize   Reconstruct display
            %   [DISPLAY, PROPS] = ArrayPlotDisplay.deserialize(HPARENT, SERIALIZED)
            %   returns ArrayPlotDisplay object DISPLAY that is parented
            %   to HPARENT and reconstructed from serialized display
            %   SERIALIZED.
            displayPlacement = serializedDisplay.Placement;
            hDisplay = dsp.scopes.ArrayPlotDisplay(gridContainer, displayPlacement);
        end
    end
    
    methods (Access=protected)
        function updateLineData(this,lineCount,xData,values)
            hLine = this.Lines(lineCount);
            set(hLine, 'XData', xData, 'YData', values);
        end
        
        function updateXLabel(this)
            % Delete existing label texts, since these will either be unnecessary (if
            % axes not maximized) or need to be replaced (if axes maximized).
            if ~isempty(this.XLabelText)
                delete(this.XLabelText);
                this.XLabelText = [];
            end
            
            % Return early if not showing labels
            if ~this.ShowTimeAxisLabels
                return;
            end
            
            xLabel = this.privXLabel;
            if this.MaximizeAxes
                setXLabelAndVisibility(this, xLabel, 'off');
                if ~isempty(this.privXLabel)
                    hAxes = getInsideLabelsAxes(this);
                    % Create xlabel text in lower-right corner
                    [~, textColor] = getLegendColors(this);
                    this.XLabelText = text(1, 0, [this.privXLabel ' '], ...
                        'Interpreter', 'none', ...
                        'Tag', 'XTextLabel', ...
                        'Units', 'norm', ...
                        'Color', textColor, ...
                        'HorizontalAlignment', 'right', ...
                        'VerticalAlignment', 'bottom', ...
                        'FontName', get(hAxes, 'FontName'), ...
                        'FontSize', get(hAxes, 'FontSize'), ...
                        'Parent', hAxes);
                end                                
            else
                setXLabelAndVisibility(this, xLabel, 'on');
                axLabelColor = this.AxesTickColor;
                for ax = this.Axes
                    set(get(ax, 'XLabel'), 'Color', axLabelColor);
                end
            end
        end
        
        function setXLabelAndVisibility(this, newLabel, visibility)
            if this.PlotMagPhase
                % Font size of labels must be consistent with axes font size
                xlabel(this.Axes(1),newLabel, 'Visible', visibility, ...
                    'Interpreter', 'none');
                xlabel(this.Axes(2),newLabel, 'Visible', visibility, ...
                    'Interpreter', 'none');
            else
                xlabel(this.Axes(1),newLabel, 'Visible', visibility, ...
                    'Interpreter', 'none');
            end
        end
        
        function [xdata,ydata] = getXYData(this,varargin)
            % Overload superclass method so the StartTime property is not
            % used to adjust XData values. StartTime does not have any
            % significance in Array Plot XData.
            xdata = get(this.Lines, 'XData');
            ydata = get(this.Lines, 'YData');
            if ~iscell(xdata)
                xdata = {xdata};
            end
            if ~iscell(ydata)
                ydata = {ydata};
            end
        end
    end
end
