classdef SpectrumAnalyzerSystemObjectCfg < matlabshared.scopes.SystemObjectScopeSpecification
    %SpectrumAnalyzerSystemObjectCfg   Define the SpectrumAnalyzerSystemObjectCfg class.
    
    %   Copyright 2011-2019 The MathWorks, Inc.
    
    properties (Access=protected)
        FileMenuOpeningListener;
        MoreInputsCalled = false;
    end
    
    events
        % Event thrown when the generate script toolbar button is clicked
        % or when the menu item is selected
        GenerateScriptClicked;
    end
    
    methods
        
        function this = SpectrumAnalyzerSystemObjectCfg(varargin)
            
            mlock
            this@matlabshared.scopes.SystemObjectScopeSpecification(varargin{:});
        end
        
        function b = getShowWaitbar(~)
            b = false;
        end
        
        function appname = getScopeTag(~)
            appname = 'Spectrum Analyzer';
        end
        
        function b = shouldShowControls(~, id)
            b = ~strcmpi(id, 'Snapshot');
        end
        
        function b = isToolbarCompact(~, toolbarName)
            % Use compact collapsible toolbars for all toolbar items except
            % measurements and autoscale.
            b = ~(strcmp(toolbarName, 'measurements'));
        end
        
        function b = isMenuCompact(~, menuName)
            % Use compact autoscale menu
            b = strcmp(menuName, 'autoscale');
        end
        
        function hiddenExts = getHiddenExtensions(~)
            %getHiddenExtensions Returns the extensions to hide.
            hiddenExts = {'Core:Source UI', ...
                'Visuals', ...
                'Tools:Instrumentation Sets', ...
                'Tools:Image Tool', ...
                'Tools:Pixel Region', ...
                'Tools:Image Navigation Tools', ...
                };
        end
        
        function b = showConfiguration(~)
            % Return false to hide the configuration controls menu item. No
            % real value added by this dialog.
            b = false;
        end
        
        function measurementTags = getSupportedMeasurements(~)
            measurementTags = {'fcursors','peaks','channel','distortion','ccdf'};
        end
        
        function renderToolbars(this,hScope)
            if ~isdeployed()
                % Generate script button is not available under deployment
                hToolbar = hScope.Handles.mainToolbar;
                hGenerateMatlabScriptButton = findobj(hToolbar, 'Tag', 'uimgr.uitoggletool_GenerateMatlabScript');
                if isempty(hGenerateMatlabScriptButton)
                    hGenerateMatlabScriptButton = uipushtool(hToolbar, ...
                        'Tag', 'uimgr.uitoggletool_GenerateMatlabScript', ...
                        'CData', getIcon(hScope, 'export_to_script'), ...
                        'TooltipString', getString(message('dspshared:SpectrumAnalyzer:GenerateMatlabScriptToolTip')), ...
                        'Interruptible', 'off', ...
                        'Separator','on',...
                        'BusyAction','cancel', ...
                        'ClickedCallback', @this.generateMatlabScriptCB); %#ok<NASGU>
                end
            end
        end
        
        function renderMenus(this, hScope)
            % Create a listener on the file menu opening callback to render
            % the NumInputPorts menu
            this.FileMenuOpeningListener = addlistener(hScope, ...
                'FileMenuOpening', @this.onFileMenuOpening);
        end
        
        function [mApp, mExample, mAbout] = createHelpMenuItems(~, mHelp)
            if ~isdeployed()
                % docroot and demo not available under deployment
                
                % Default Mode
                mapFileLocation = fullfile(docroot,'toolbox','dsp','dsp.map');
                
                mApp(1) = uimenu(mHelp, ...
                    'Tag', 'uimgr.uimenu_dsp.SpectrumAnalyzer', ...
                    'Label', getString(message('dspshared:SpectrumAnalyzer:SpectrumAnalyzerSystemObjectHelp')), ...
                    'Callback', @(hco,ev) helpview(mapFileLocation, 'sigblksspectrumanalyzer'));
                
                mApp(2) = uimenu(mHelp, ...
                    'Tag', 'uimgr.uimenu_DSP System Toolbox', ...
                    'Label', getString(message('dspshared:SpectrumAnalyzer:DSPSystemToolboxHelp')), ...
                    'Callback', @(hco,ev) helpview(mapFileLocation,'dspinfo'));
                
                
                % NOTE: The Keyboard Command and Message Log menus are added by the
                % Framework in: ...\@uiscopes\@Framework\Framework.m
                
                mExample = uimenu(mHelp, ...
                    'Tag', 'uimgr.uimenu_DSP System Toolbox Demos', ...
                    'Label', getString(message('dspshared:SpectrumAnalyzer:DSPSystemToolboxDemos')), ...
                    'Callback', @(hco,ev) demo('toolbox', 'dsp'));
                
            else
                mApp = [];
                mExample = [];
            end
          % Want the "About" option separated, so we group everything above
          % into a menugroup and leave "About" as a singleton menu
            mAbout = uimenu(mHelp, ...
            'Tag', 'uimgr.uimenu_About', ...
            'Label', getString(message('Spcuilib:scopes:AboutDSPSystemToolbox')), ...
            'Callback', @(hco,ev) aboutdspsystbx);  
        end
        
        function cfgFile = getConfigurationFile(~)
            cfgFile = 'spectrumanalyzersysobj.cfg';
        end
        
        function helpArgs = getHelpArgs(~,key)
            if strcmp(key, 'PlotNavigationSuffix')
                helpArgs = 'SA';
            else
                helpArgs = [];
            end
        end
        
        function b = showPrintAction(~, type)
            b = strcmp(type, 'menu');
        end
        
        function b = showKeyboardCommand(~)
            %showKeyboardCommand - Returns true when the keyboard
            %command help menu item should be shown
            b = false;
        end
        
        function b = showMessageLog(~)
            %showMessageLog - Returns true when the message log help
            %menu item should be shown
            b = false;
        end
    end
    
    methods (Hidden)
        function numInputsCallback(this, h, ~)
            %NUMINPUTSCALLBACK Callback to the various number of inputs menus
            
            hScope = get(get(h, 'Parent'), 'UserData');
            
            % Get the id from the tag of the widget.
            tag = get(h, 'Tag');
            [~, id] = strtok(tag, '_');
            id(1) = [];
            
            % Create an input dialog for the "more" case.
            pSource = hScope.getExtInst('Sources','Streaming');
            if strcmpi(id, 'CustomInputPorts')
                if this.MoreInputsCalled
                    return;
                end
                this.MoreInputsCalled = true;
                options.Resize = 'off';
                options.WindowStyle = 'normal';
                options.Interpreter = 'none';
                
                if ~isLocked(pSource.DataSpecsLocked)
                    numInputPorts = inputdlg(getString(message('Spcuilib:scopeblock:NumInputPorts')),...
                        getString(message('Spcuilib:scopes:DialogTitleSourceUIOptions')), ...
                        1, {num2str(pSource.NumInputPorts)}, options);
                    
                    evalAndSetNumInputPorts(hScope, numInputPorts);
                end
                this.MoreInputsCalled = false;
            else
                if ~isLocked(pSource.DataSpecsLocked)
                    pSource.NumInputPorts = str2double(id(1));
                end
            end
        end
        
        function onFileMenuOpening(this, hScope, ~)
            %ONFILEMENUOPENING Respond to the FileMenuOpening event.
            
            % Create the menus
            hFileMenu = findobj(hScope.Parent, 'Tag', 'uimgr.uimenugroup_File');
            % Num Input ports menu item            
            hNumInputs = findobj(hFileMenu, 'Tag', 'uimgr.uimenugroup_NumberOfInputPorts');
            if isempty(hNumInputs)
                
                hNumInputs = uimenu('Parent', hFileMenu, ...
                    'Tag', 'uimgr.uimenugroup_NumberOfInputPorts', ...
                    'Position', 1, ...
                    'UserData', hScope, ...
                    'Label', getString(message('Spcuilib:scopes:NumInputPortsWithKBShortcut')), ...
                    'Callback', @this.numInputPortsCB);
                
                cb = @this.numInputsCallback;
                
                uimenu('Parent', hNumInputs, ...
                    'Tag', 'uimgr.spctogglemenu_1InputPort', ...
                    'Label', '&1', ...
                    'Callback', cb);
                uimenu('Parent', hNumInputs, ...
                    'Label', '&2', ...
                    'Tag', 'uimgr.spctogglemenu_2InputPort', ...
                    'Callback', cb);
                uimenu('Parent', hNumInputs, ...
                    'Label', '&3', ...
                    'Tag', 'uimgr.spctogglemenu_3InputPort', ...
                    'Callback', cb);
                uimenu('Parent', hNumInputs, ...
                    'Interruptible', 'off', ...
                    'BusyAction', 'cancel', ...
                    'Label', getString(message('Spcuilib:scopes:MenuMore')), ...
                    'Tag', 'uimgr.spctogglemenu_CustomInputPorts', ...
                    'Callback', cb);
                
                % Add a separator to any preexisting menu items.
                set(findobj(hFileMenu, 'Parent', hFileMenu, 'Position', 2), 'Separator', 'on');
            end
            
            % Update the states of the menu.
            isStopped = ~hScope.DataSource.DataSpecsLocked;
            set(hNumInputs, 'Enable', uiservices.logicalToOnOff(isStopped), ...
                'Visible', uiservices.logicalToOnOff(~hScope.DataSource.Floating));
            if ~isdeployed()
                % Generate MATLAB script menu item is not available under
                % deployment
                hGenerateMatlabScriptMenu = findobj(hFileMenu, 'Tag', 'uimgr.spctogglemenu_GenerateMatlabScript');
                if isempty(hGenerateMatlabScriptMenu)
                    % Render generate MATLAB script menu item
                    hGenerateMatlabScriptMenu = uimenu(hFileMenu, ...
                        'Tag', 'uimgr.spctogglemenu_GenerateMatlabScript', ...
                        'Separator', 'on', ...
                        'Position',2,...
                        'Label', getString(message('dspshared:SpectrumAnalyzer:GenerateMatlabScript')), ...
                        'Callback', @this.generateMatlabScriptCB); %#ok<NASGU>
                end
            end
        end
        
        function numInputPortsCB(~, h, ~)
            % Callback for File->Number of Input Ports submenu
            
            hScope = get(h, 'UserData');
            children = get(h, 'Children');
            
            pSource = hScope.getExtInst('Sources','Streaming');
            numInputPorts = pSource.NumInputPorts;
            
            if numInputPorts > 3
                child = findobj(children, 'tag', 'uimgr.spctogglemenu_CustomInputPorts');
            else
                child = findobj(children, 'tag', ['uimgr.spctogglemenu_' num2str(numInputPorts) 'InputPort']);
            end
            
            siblings = children(children ~= child);
            
            set(child, 'Checked', 'on');
            for idx = 1:length(siblings)
                set(siblings(idx), 'Checked', 'off');
            end
        end
        
        function generateMatlabScriptCB(this,~,~)
            % Callback for File->Generate Script and Generate Script
            % toolbar button
            notify(this,'GenerateScriptClicked');
        end
                
        function b = useUIMgr(~)
            b = false;
        end
        
        function b = useMCOSExtMgr(~)
            b = true;
        end  
    end
end

% -------------------------------------------------------------------------
% Local functions
% -------------------------------------------------------------------------
function evalAndSetNumInputPorts(hScope, numInputPorts)

if ~isempty(numInputPorts) && ~isempty(numInputPorts{1})
    % isempty(numInputPorts) = true, if 'Cancel' button is hit
    % isempty(numInputPorts{1}) = true, if the edit field is left empty
    % and OK is hit
    % numInputPorts is a string - it could be a valid numerical value or a
    % variable name
    
    numInputPortsVal = str2num(numInputPorts{1}); %#ok<ST2NM>
    
    if isempty(numInputPortsVal)
        % numInputPorts contains a variable name
        % make sure variable name is valid
        if ~isvarname(numInputPorts{1})
            uiscopes.errorHandler(DAStudio.message(...
                'Spcuilib:scopes:InvalidVariableName', numInputPorts{1}));
            return;
        end
    end
    
    % validate the value before setting it
    if ~isscalar(numInputPortsVal) ...
            || ~isnumeric(numInputPortsVal) ...
            ||  ~isfinite(numInputPortsVal) ...
            || ~isreal(numInputPortsVal) ...
            || (floor(numInputPortsVal) ~= numInputPortsVal) ...
            || (numInputPortsVal < 1) ...
            || (numInputPortsVal > 96)
        uiscopes.errorHandler(DAStudio.message(...
            'Spcuilib:scopeblock:invalidNumberOfInputPorts'));
    else
        % valid value - set System object property
        pSource = hScope.getExtInst('Sources','Streaming');
        pSource.NumInputPorts = numInputPortsVal;
    end
    
    %else
    % 'Cancel' button hit or edit field left empty - do not set block's
    % NumInputPorts parameter i.e. do nothing
end
end

% -------------------------------------------------------------------------
function value = isLocked(value)
if value
    uiscopes.errorHandler(getString(message('Spcuilib:scopes:PropertySetWhenLocked')));
end 
end

% [EOF]
