classdef TimeScopeSystemObjectCfg < matlabshared.scopes.SystemObjectScopeSpecification
  %TimeScopeSystemObjectCfg   Define the TimeScopeSystemObjectCfg class.
  
  %   Copyright 2009-2018 The MathWorks, Inc.
 
  properties (Access = protected)
      FileMenuOpeningListener
      ViewMenuOpeningListener
      MoreInputsCalled = false;
  end
  
  properties (SetAccess=public, Hidden, Transient)
      OptionsDialog
  end
  
  methods
    function this = TimeScopeSystemObjectCfg(varargin)
      %TimeScopeSystemObjectCfg   Construct the TimeScopeSystemObjectCfg class.
      
      % Prevent clear classes warnings
      mlock;
      this@matlabshared.scopes.SystemObjectScopeSpecification(varargin{:});
    end
    
    function appName = getScopeTag(~)
      %getScopeTag Returns the simple application name.
      appName = 'Time Scope';
    end
    
    function b = isToolbarCompact(~, toolbarName)
        b = ~strcmp(toolbarName, 'measurements');
    end
    
    function b = isMenuCompact(~, menuName)
        b = strcmp(menuName, 'autoscale');
    end
    
    function [success, errMessage] = checkMeasurementLicense(~, ~)
      [success, errMessage] = builtin('license','checkout','Signal_Blocks');
    end
    
    function hiddenExts = getHiddenExtensions(~)
      %getHiddenExtensions Returns the extensions to hide.
      hiddenExts = {'Core:Source UI', ...
        'Visuals', ...
        'Tools:Instrumentation Sets', ...
        'Tools:Image Tool', ...
        'Tools:Pixel Region', ...
        'Tools:Image Navigation Tools', ...
        };
    end
    
    function measurementTags = getSupportedMeasurements(~)
      measurementTags = {'triggers','tcursors','signalstats','peaks','bilevel'};
    end
    
    function renderToolbars(~, hScope)
        hButton = renderLayoutButton(hScope.Visual);
        set(hButton, 'Separator', 'on');
    end
    
    function renderMenus(this, hScope)    
      
      % Create a listener on the file menu opening callback to render the
      % NumInputPorts and OpenAtMdlStart menus
      this.FileMenuOpeningListener = addlistener(hScope, 'FileMenuOpening', @this.onFileMenuOpening);
      this.ViewMenuOpeningListener = addlistener(hScope, 'ViewMenuOpening', @this.onViewMenuOpening);
    end
    
    function [mApp, mExample, mAbout] = createHelpMenuItems(~, mHelp)
        % Help menus
      if ~isdeployed
          mapFileLocation = fullfile(docroot,'toolbox','dsp','dsp.map');

          mApp(1) = uimenu(mHelp, ...
              'Tag', 'uimgr.uimenu_dsp.TimeScope', ...
              'Label', getString(message('Spcuilib:scopes:TimeScopeSystemObjectHelp')), ...
              'Callback', uiservices.makeCallback(@helpview, mapFileLocation, 'sigblkstimescope'));
          
          mApp(2) = uimenu(mHelp, ...
              'Tag', 'uimgr.uimenu_DSP System Toolbox', ...
              'Label', getString(message('Spcuilib:scopes:DSPSystemToolboxHelp')), ...
              'Callback', uiservices.makeCallback(@helpview, mapFileLocation,'dspinfo'));
          
          mExample = uimenu(mHelp, ...
              'Tag', 'uimgr.uimenu_DSP System Toolbox Demos', ...
              'Label', getString(message('Spcuilib:scopes:DSPSystemToolboxDemos')), ...
              'Callback', uiservices.makeCallback(@demo, 'toolbox', 'dsp'));
      else
          mApp = [];
          mExample = [];
      end
      
      % Want the "About" option separated, so we group everything above
      % into a menugroup and leave "About" as a singleton menu
      mAbout = uimenu(mHelp, ...
          'Tag', 'uimgr.uimenu_About', ...
          'Label', getString(message('Spcuilib:scopes:AboutDSPSystemToolbox')), ...
          'Callback', uiservices.makeCallback(@aboutdspsystbx));
      
    end
    
    function cfgFile = getConfigurationFile(~)
      cfgFile = 'timescopesysobj.cfg';
    end        
    
    function helpArgs = getHelpArgs(this,key) %#ok
      helpArgs = [];
    end 
    
    function b = showConfiguration(~)
      b = false;
    end
    
    function b = showPrintAction(~, type)
      b = strcmp(type, 'menu');
    end
    
    function b = showKeyboardCommand(~)
      b = false;
    end
  end

  methods (Hidden)
      
    function b = useUIMgr(~)
      b = false;
    end
    function b = useMCOSExtMgr(~)
      b = true;
    end
    
    function numInputsCallback(this, h, ~)
    %NUMINPUTSCALLBACK Callback to the various number of inputs menus
        
      hScope = get(get(h, 'Parent'), 'UserData');
      
      % Get the id from the tag of the widget.
      tag = get(h, 'Tag');
      [~, id] = strtok(tag, '_');
      id(1) = [];
      
      % Create an input dialog for the "more" case.
      pSource = hScope.getExtInst('Sources','Streaming');
      if strcmpi(id, 'CustomInputPorts')
        if this.MoreInputsCalled
            return;
        end
        this.MoreInputsCalled = true;
        options.Resize = 'off';
        options.WindowStyle = 'normal';
        options.Interpreter = 'none';
        
        if ~isLocked(pSource.DataSpecsLocked)
          numInputPorts = inputdlg(getString(message('Spcuilib:scopeblock:NumInputPorts')),...
            getString(message('Spcuilib:scopes:DialogTitleSourceUIOptions')), ...
            1, {num2str(pSource.NumInputPorts)}, options);
          
          evalAndSetNumInputPorts(hScope, numInputPorts);
        end
        this.MoreInputsCalled = false;
      else
        if ~isLocked(pSource.DataSpecsLocked)
          pSource.NumInputPorts = str2double(id(1));
        end
      end
    end
    
    function onViewMenuOpening(this, hScope, ~)
      this.ViewMenuOpeningListener = [];
      hView = findobj(hScope.Parent, 'Tag', 'uimgr.uimenugroup_View');
      
      uimenu(hView, ...
          'Separator', 'on', ...
          'Label', getString(message('Spcuilib:scopes:TitleDatHistOpt')), ...
          'Tag',   'uimgr.uimenu_DataHistoryOptions', ...
          'Callback', uiservices.makeCallback(@onEditOptions, hScope));
    end
    
    function onFileMenuOpening(this, hScope, ~)
      %ONFILEMENUOPENING Respond to the FileMenuOpening event.
      
      % Create the menus
      hFileMenu = findobj(hScope.Parent, 'Tag', 'uimgr.uimenugroup_File');
      
      hNumInputs = findobj(hFileMenu, 'Tag', 'uimgr.uimenugroup_NumberOfInputPorts');
      if isempty(hNumInputs)
          
        hNumInputs = uimenu('Parent', hFileMenu, ...
          'Tag', 'uimgr.uimenugroup_NumberOfInputPorts', ...
          'Position', 1, ...
          'UserData', hScope, ...
          'Label', getString(message('Spcuilib:scopes:NumInputPortsWithKBShortcut')), ...
          'Callback', @this.numInputPortsCB);
        
        cb = @this.numInputsCallback;
        
        uimenu('Parent', hNumInputs, ...
          'Tag', 'uimgr.spctogglemenu_1InputPort', ...
          'Label', '&1', ...
          'Callback', cb);
        uimenu('Parent', hNumInputs, ...
          'Label', '&2', ...
          'Tag', 'uimgr.spctogglemenu_2InputPort', ...
          'Callback', cb);
        uimenu('Parent', hNumInputs, ...
          'Label', '&3', ...
          'Tag', 'uimgr.spctogglemenu_3InputPort', ...
          'Callback', cb);
        uimenu('Parent', hNumInputs, ...
          'Interruptible', 'off', ...
          'BusyAction', 'cancel', ...
          'Label', getString(message('Spcuilib:scopes:MenuMore')), ...
          'Tag', 'uimgr.spctogglemenu_CustomInputPorts', ...
          'Callback', cb);
        
        % Add a separator to any preexisting menu items.
        set(findobj(hFileMenu, 'Parent', hFileMenu, 'Position', 2), 'Separator', 'on');
      end
      
      % Update the states of the menu.
      isStopped = ~hScope.DataSource.DataSpecsLocked;
      set(hNumInputs, 'Enable', uiservices.logicalToOnOff(isStopped), ...
        'Visible', uiservices.logicalToOnOff(~hScope.DataSource.Floating));
    end
    
    function numInputPortsCB(~, h, ~)
        % Callback for File->Number of Input Ports submenu
        
        hScope = get(h, 'UserData');
        children = get(h, 'Children');
        
        pSource = hScope.getExtInst('Sources','Streaming');
        numInputPorts = pSource.NumInputPorts;
        
        if numInputPorts > 3
            child = findobj(children, 'tag', 'uimgr.spctogglemenu_CustomInputPorts');
        else
            child = findobj(children, 'tag', ['uimgr.spctogglemenu_' num2str(numInputPorts) 'InputPort']);
        end
        
        siblings = children(children ~= child);
        
        set(child, 'Checked', 'on');
        for idx = 1:length(siblings)
            set(siblings(idx), 'Checked', 'off');
        end
    end
  end
  
  methods (Hidden, Static)
    function varargout = useMCOS(flag)
      persistent usemcos;
      
      if nargin
        usemcos = flag;
      end
      
      if nargout
        if isempty(usemcos)
          usemcos = true;
        end
        varargout = {usemcos};
      end
    end
    function this = loadobj(s)
        this = loadobj@matlabshared.scopes.SystemObjectScopeSpecification(s);

        configSet = this.CurrentConfiguration;
        if isempty(configSet)
            return;
        end
        configs = configSet.Children;
        if isempty(configs)
            return;
        end
        allTypes = {configs.Type};
        allNames = {configs.Name};
        if ~isfield(s, 'Version') % 13a or earlier
            cfg = configs(strcmp(allTypes, 'Visuals') & strcmp(allNames, 'Time Domain'));
            if ~isempty(cfg) && ~isempty(cfg.PropertySet)
                prop = cfg.PropertySet.findProp('SerializedDisplays');
                if ~isempty(prop)
                    value = prop.Value;
                    for indx = 1:numel(value)
                        
                        % Check the axes color for old defaults
                        if isfield(value{indx}, 'AxesColor') && isequal(value{indx}.AxesColor, [0 0 0])
                            value{indx} = rmfield(value{indx}, 'AxesColor');
                        end
                        
                        % Check the axestickcolor for old defaults
                        defaultATColor = [.3137 .3137 .3137];
                        if isfield(value{indx}, 'AxesTickColor') && all(value{indx}.AxesTickColor - defaultATColor < 1e-4)
                            value{indx} = rmfield(value{indx}, 'AxesTickColor');
                        end
                        
                        % Check the color order for old defaults
                        if isfield(value{indx}, 'ColorOrder') && isequal(value{indx}.ColorOrder, [1 1 0; 1 0 1; 0 1 1; 1 0 0; 0 1 0; .75 .75 .75])
                            value{indx} = rmfield(value{indx}, 'ColorOrder');
                        end
                    end
                    prop.Value = value;
                end
            end
            cfg = configs(strcmp(allTypes, 'Core') & strcmp(allNames, 'General UI'));
            if ~isempty(cfg) && ~isempty(cfg.PropertySet)
                prop = cfg.PropertySet.findProperty('FigureColor');
                if ~isempty(prop)
                    
                    % Check the figure color for old defaults.  Check
                    % for both linux windows and mac.
                    c = [ ...
                        get(0, 'DefaultUIControlBackgroundColor'); ...
                        repmat(0.941176, 1, 3); ...
                        repmat(0.929411, 1, 3)];
                    v = repmat(prop.Value, 3, 1);
                    if any(all(abs(c-v) < 1e-6, 2))
                        remove(cfg.PropertySet, prop);
                    end
                end
            end
        end
    end
  end
end

function onEditOptions(hScope)
editOptions(hScope.ExtDriver, hScope.DataSource);
end

% -------------------------------------------------------------------------
function evalAndSetNumInputPorts(hScope, numInputPorts)

if ~isempty(numInputPorts) && ~isempty(numInputPorts{1})
  % isempty(numInputPorts) = true, if 'Cancel' button is hit
  % isempty(numInputPorts{1}) = true, if the edit field is left empty
  % and OK is hit
  % numInputPorts is a string - it could be a valid numerical value or a
  % variable name
  
  numInputPortsVal = str2num(numInputPorts{1}); %#ok<ST2NM>
  
  if isempty(numInputPortsVal)
    % numInputPorts contains a variable name
    % make sure variable name is valid
    if ~isvarname(numInputPorts{1})
      uiscopes.errorHandler(DAStudio.message(...
        'Spcuilib:scopes:InvalidVariableName', numInputPorts{1}));
      return;
    end    
  end
  
  % validate the value before setting it
  if ~isscalar(numInputPortsVal) ...
      || ~isnumeric(numInputPortsVal) ...
      ||  ~isfinite(numInputPortsVal) ...
      || ~isreal(numInputPortsVal) ...
      || (floor(numInputPortsVal) ~= numInputPortsVal) ...
      || (numInputPortsVal < 1) ...
      || (numInputPortsVal > 96)
    uiscopes.errorHandler(DAStudio.message(...
      'Spcuilib:scopeblock:invalidNumberOfInputPorts'));
  else
    % valid value - set System object property 
    pSource = hScope.getExtInst('Sources','Streaming');  
    pSource.NumInputPorts = numInputPortsVal;
  end
  
  %else
  % 'Cancel' button hit or edit field left empty - do not set block's
  % NumInputPorts parameter i.e. do nothing
end
end

% -------------------------------------------------------------------------
function value = isLocked(value)
if value
    uiscopes.errorHandler(getString(message('Spcuilib:scopes:PropertySetWhenLocked')));
end 
end

% [EOF]
