function hMainFigure = HelperCreateParamTuningUI(varargin)
%HelperCreateParamTuningUI Interactively tune parameters of System
%                           objects
% Function to create a user interface (UI) to tune System objects
% properties in a MATLAB simulation or block parameters in a Simulink
% model. Sliders and drop down lists can be created for the parameters and
% can be used to change the parameter values while streaming data. Edit
% boxes placed to the right of the sliders show the current value of the
% slider and can also be used to type in the new values. Labels to the left
% of sliders and drop down lists denote the parameter name. The UI also has
% buttons to reset the simulation state or to stop the simulation.
%
% The UI can also synchronize with a MIDI control. The sliders and buttons
% have context menus, available when you right-click on them. When the user
% selects the context menu, a dialog is opened to let the user choose a
% MIDI control.  The chosen MIDI control is then cross-wired with the
% uicontrol so that operating one control is tracked by the other control.
%
% Inputs:
%       parameters    - Array of structures for parameters that need to be
%                       tuned. Each structure can have the following
%                       fields:
%                       * Type - Type of the widget. May be set to either 
%                         'slider' or 'dropdown'
%                       * Name - String which will be displayed to the left
%                         of the widget. Default values are 'Parameter 1',
%                         'Parameter 2', and so on
%                       * Limits - Vector of two elements that correspond
%                         to minimum and maximum values for the sliders.
%                         Default value is [0, 1]. This parameter should
%                         only be specified for sliders.
%                       * Values - Cell array for the values of the drop
%                         down list.  This parameter should
%                         only be specified for drop down lists.
%                       * InitialValue - Scalar that is the initial
%                         position of the widget. For sliders, this must be 
%                         a value  within the limits. Default value is the 
%                         average of the two limit values. For drop down 
%                         lists, the value must be a positive integer less 
%                         than or equal to the number of elements in Values.
%                       * SliderScale - Type of scaling for the slider
%                         widget. Must be set to either 'Linear' (default)
%                         or 'Log'. This parameter must be specified only
%                         when the Type parameter is 'slider'.
%                       * BlockName (for Simulink simulations only) -
%                         String corresponding to the full path of the 
%                         block to be tuned
%                       * ParameterName (for Simulink simulations only) -
%                         String corresponding to the parameter name to be
%                         tuned on the block
%       title          -  String that is displayed as the title of the UI.
%                         This input is optional
%       reset callback -  Handle to the function that executes when the
%                         reset button is clicked. If the function handle
%                         is empty or invalid, the Reset button will be
%                         hidden.
% Output:
%       hMainFigure -  Handle to the UI figure
%
% Example 1:
%         % Create a UI with two sliders
%         param = struct([]);
%         param(1).Name = 'First parameter';
%         param(1).Type = 'slider';
%         param(1).InitialValue = 20;
%         param(1).Limits = [0, 400];
%         param(2).Name = 'Second parameter (Log scaled)';
%         param(2).Type = 'slider';
%         param(2).InitialValue = 0.3;
%         param(2).SliderScale = 'Log';
%         HelperCreateParamTuningUI(param, ...
%                                   'Example to show parameter tuning UI')
% 
% Example 2:
%         % Create a UI with one slider and one drop down list
%         param = struct([]);
%         param(1).Name = 'First parameter';
%         param(1).Type = 'slider';
%         param(1).InitialValue = 20;
%         param(1).Limits = [0, 400];
%         param(2).Name = 'Second parameter';
%         param(2).Type = 'dropdown';
%         param(2).Values = {'Dropdown Option 1','Dropdown Option 2'};
%         param(2).InitialValue = 1;
%         HelperCreateParamTuningUI(param, ...
%                                  'Example to show parameter tuning UI')
% Example 3:
%         % Create a UI for a Simulink model with two sliders
%         param = struct([]);
%         param(1).Name = 'First parameter';
%         param(1).Type = 'slider';
%         param(1).InitialValue = 20;
%         param(1).Limits = [0, 400];
%         param(1).BlockName = 'modelname/blockname1';
%         param(1).ParameterName = 'myparamname1';
%         param(2).Name = 'Second parameter';
%         param(2).Type = 'slider';
%         param(2).InitialValue = 0.3;
%         param(2).BlockName = 'modelname/blockname2';
%         param(2).ParameterName = 'myparamname2';
%         % Create UI - when reset button is clicked, the function
%         % resetFunctionHandle is executed
%         HelperCreateParamTuningUI(param, ...
%                                   'Simulink Example',...
%                                   'ResetCallback',@()resetHandle);
%
% A complete working example with a System object is present in the help
% for the helper function HelperUnpackUIData. For usage of the UI in a
% Simulink model, refer to the Simulink example "System Identification
% Using RLS Adaptive Filtering".

% Copyright 2013-2017 The MathWorks, Inc.

% Validate inputs
% Find out number of parameters
narginchk(1, 6);    % title input is optional
parameters = varargin{1};
if nargin >= 2
    title = varargin{2};
else
    title = getString(message('dsp:demo:GUITitle'));
end
nParams = length(parameters);
% Check context in which UI is used:
% 0: MATLAB
% 1: Simulink - each slider linked to a block parameter
% 2: Simulink - each slider linked to a function handle
if isfield(parameters(1),'Callback')
    UIMode = 2;
elseif isfield(parameters(1),'BlockName')
    UIMode = 1;
else
    UIMode = 0;
end
isSimulink = (UIMode ~= 0);
parameters = checkInputs(parameters,UIMode); % validate inputs and add default values
% get Simulink model name
systemName = '';
if UIMode == 1
    % model name
    blkName = parameters{1}.BlockName;
    ind = strfind(blkName,'/');
    systemName = blkName(1:ind(1)-1);
elseif UIMode == 2
    systemName = varargin{nargin};
end

% Pause button is hidden in Simulink;
showPauseButton = ~isSimulink;

% Reset button is shown by default in MATLAB, and hidden by default in
% Simulink.
showResetButton = ~isSimulink;
resetCallback = '';
if nargin>=3 && strcmp(varargin{3},'ResetCallback')% reset handle was provided
    % Function handle to be executed when reset button is clicked (Simulink)
    if isa(varargin{4},'function_handle')
        resetCallback = varargin{4};
        showResetButton = true;
    else
        showResetButton = false;
    end
end

userDataCounter = 0;
if isSimulink
     clear HelperUnpackUDP
end

% Handles for UI objects
% Declare and create all the UI objects in this UI here so that they can
% be used in any functions
hMainFigure    =   figure(...       % the main UI figure
                        'Name', title, ...
                        'MenuBar','none', ...
                        'Toolbar','none', ...
                        'HandleVisibility','callback', ...
                        'NumberTitle','off', ...
                        'IntegerHandle','off', ...
                        'Color', get(0, 'defaultuicontrolbackgroundcolor'),...
                        'DeleteFcn', @hFigDeleteCallback);
hWidget = zeros(nParams,1);
hParamNames = zeros(nParams,1);
hParamValues = zeros(nParams,1);
sliderVal = zeros(nParams, 1);
widgetHeight = min(0.85/(2*nParams+1), 0.08);
for idx = 1:nParams
    param = parameters{idx};
    if strcmp(param.Type,'slider') % slider
        if strcmp(param.SliderScale,'Log')
            sliderLimits = log10(param.Limits + eps);
        else
            sliderLimits = param.Limits;
        end
        hWidget(idx)    =   uicontrol(...    % Slider values for parameters
            'Parent', hMainFigure, ...
            'Units','normalized',...
            'Position',[0.2, 1-2*idx*widgetHeight, 0.6, widgetHeight],...
            'HandleVisibility','callback',...
            'Style','slider', ...
            'Min', sliderLimits(1),...
            'Max', sliderLimits(2),...
            'Tag', ['Widget ', int2str(idx)],...
            'Callback', @hSliderCallback);
        setWidgetValue(idx, param.InitialValue);  % Set the initial value for the slider
        
        learnMIDIControl(hWidget(idx)); % Register the slider for MIDI control
        
        hParamNames(idx)=   uicontrol(...   % Label for parameter names to the left of sliders
            'Parent', hMainFigure, ...
            'Units','normalized',...
            'Position',[0,   1-(2*idx+0.3)*widgetHeight, ...
            0.2, widgetHeight],...
            'HandleVisibility','callback', ...
            'String',param.Name,...
            'FontUnits', 'normalized',...
            'FontSize', 0.4,...
            'Style','text');
        hParamValues(idx)=   uicontrol(...   % Edit box for parameter values to the right of sliders
            'Parent', hMainFigure, ...
            'Units','normalized',...
            'Position',[0.82, 1-(2*idx)*widgetHeight, ...
            0.16, widgetHeight],...
            'HandleVisibility','callback', ...
            'BackgroundColor', [1 1 1], ...
            'String',num2str(param.InitialValue),...
            'FontUnits', 'normalized',...
            'FontSize', 0.4,...
            'Style','edit', ...
            'Tag', ['Edit ', int2str(idx)],...
            'Callback', @hParamValuesCallback);
        sliderVal(idx) =   param.InitialValue; % Used to roll back value when edit box value is not in limits
        addlistener(hWidget(idx),'ContinuousValueChange',@hSliderCallback);
        
    else % drop down
            hWidget(idx)    =   uicontrol(...  
            'Parent', hMainFigure, ...
            'Units','normalized',...
            'Position',[0.2, 1-2*idx*widgetHeight, ...
             0.4, widgetHeight],...
            'HandleVisibility','callback',...
            'FontUnits', 'normalized',...
            'FontSize', 0.4,...
            'Style','popupmenu', ...
            'Value', param.InitialValue,...
            'String',param.Values,...
            'Tag', ['Widget ', int2str(idx)],...
            'Callback', @hDropDownCallback);

        hParamNames(idx)=   uicontrol(...
            'Parent', hMainFigure, ...
            'Units','normalized',...
            'Position',[0,   1-(2*idx)*widgetHeight, ...
            0.2, widgetHeight],...
            'HandleVisibility','callback', ...
            'String',param.Name,...
            'FontUnits', 'normalized',...
            'FontSize', 0.4,...
            'Style','text');
        
    end
end

% Reset button
if showResetButton
    if showPauseButton
        pos = [0.1 0.05 0.1 widgetHeight];
    else
        pos = [0.1 0.05 0.3 widgetHeight];
    end
    hReset = uicontrol(...    % button to reset the System object
        'Parent', hMainFigure, ...
        'Units','normalized',...
        'HandleVisibility','callback', ...
        'Position',pos,...
        'String',getString(message('dsp:demo:GUIReset')),...
        'FontUnits', 'normalized',...
        'FontSize', 0.4,...
        'Tag', 'ButtonReset',...
        'Callback', @hResetButtonCallback);
    learnMIDIControl(hReset);   % Register the button for MIDI control
end

% Pause button: For Simulink applications, the PAUSE button is hidden
if showPauseButton
    if showResetButton
        pos = [0.25 0.05 0.3 widgetHeight];
    else
        pos = [0.1 0.05 0.3 widgetHeight];
    end
    hPauseSim = uicontrol(...    % button to reset the System object
        'Parent', hMainFigure, ...
        'Units','normalized',...
        'HandleVisibility','callback', ...
        'Position',pos,...
        'String',getString(message('dsp:demo:GUIPauseSim')),...
        'FontUnits', 'normalized',...
        'FontSize', 0.4,...
        'Tag', 'ButtonPause',...
        'Callback', @hPauseSimButtonCallback);
    learnMIDIControl(hPauseSim);   % Register the button for MIDI control
end

if showPauseButton || showResetButton       % 2 or 3 buttons
    pos = [0.6 0.05 0.3 widgetHeight];
else                                        % Only stop button
    pos = [0.35 0.05 0.3 widgetHeight];
end
hStopSim = uicontrol(...    % button to stop the simulation
    'Parent', hMainFigure, ...
    'Units','normalized',...
    'HandleVisibility','callback', ...
    'Position',pos,...
    'String',getString(message('dsp:demo:GUIStopSim')),...
    'FontUnits', 'normalized',...
    'FontSize', 0.4,...
    'Tag', 'ButtonStop',...
    'Callback', @hStopSimButtonCallback);
learnMIDIControl(hStopSim); % Register the button for MIDI control


switchReset = false;         % Can be changed to reset the System object
flagPauseSim = false;        % Can be set to pause the simulation
flagStopSim = false;         % Can be set to stop the simulation


updateUserData()
drawnow

% Callback functions as nested functions

    function val = getWidgetValue(index)
        % Get the value of the widget. If the slider is in Log mode,
        % convert the value to linear scale.
        if isfield(parameters{index}, 'SliderScale') && ...
                strcmp(parameters{index}.SliderScale, 'Log')
            logval = get(hWidget(index),'Value');
            val = 10^logval-eps;
        else
            val = get(hWidget(index),'Value');
        end
    end

    function setWidgetValue(index, val)
        % Set the value of the widget. If the slider is in Log mode,
        % convert the value to log scale before setting the slider value.
        if isfield(parameters{index}, 'SliderScale') && ...
                strcmp(parameters{index}.SliderScale, 'Log')
            set(hWidget(index), 'Value', log10(val + eps));
        else
            set(hWidget(index), 'Value', val);
        end
    end

    function updateUserData()
            data = zeros(1,nParams+4);
            
            for index = 1:nParams
                data(index) = getWidgetValue(index);
            end
            userDataCounter = userDataCounter + 1;
            data(end-3) = userDataCounter;
            data(end-2) = switchReset;
            data(end-1) = logical(flagPauseSim);
            data(end)   = logical(flagStopSim);
            
            set(hMainFigure,'UserData',data);
    end

function hSliderCallback(hObject,~,~)
    
    updateUserData();
    
% Called when a slider value changes
    TagStr = get(hObject,'Tag');
    TagStr = strrep(TagStr,'Widget','');
    index = str2double(TagStr); % This identifies the parameter number
    val = getWidgetValue(index);
    set(hParamValues(index), 'String', num2str(val));
    sliderVal(index) = val;
   
    tuneParameters(index);
end

function hDropDownCallback(hObject,~,~)
    updateUserData();
    TagStr = get(hObject,'Tag');
    TagStr = strrep(TagStr,'Widget','');
    index = str2double(TagStr); % This identifies the parameter number
    tuneParameters(index);
end

function hParamValuesCallback(hObject,~,~)
% Called when value in edit box changes

    TagStr = get(hObject,'Tag');
    TagStr = strrep(TagStr,'Edit','');
    index = str2double(TagStr); % This identifies the parameter number
    try
        val = evalin('base',get(hObject,'String'));
        validateattributes(val, {'double', 'scalar'}, ... % check for new value to be in limit
            {'real', '>=',parameters{index}.Limits(1),'<=',parameters{index}.Limits(2)})
    catch ME
        % Throw error on message box and revert to previous value on edit
        % box
        msgbox(ME.message, 'Error');
        set(hParamValues(index), 'String', num2str(sliderVal(index)));
        return;
    end
    
    set(hParamValues(index), 'String', num2str(val));
    setWidgetValue(index, val);
    sliderVal(index) = val;
    
    tuneParameters(index);
    
    updateUserData();
end

    function hResetButtonCallback(~,~,~)
        % Called when "Reset" button is pressed
        if isa(resetCallback,'function_handle')
            % try-catch in case model is not open
            try
                resetCallback();
            catch e %#ok
            end
        else
            switchReset = ~switchReset;
            updateUserData();
            tuneParameters();
        end
    end

function hPauseSimButtonCallback(~,~,~)
% Called when "Stop Simulation" button is pressed
    if ~flagPauseSim
        set(hPauseSim, 'String', getString(message('dsp:demo:GUIResumeSim')));
    else
        set(hPauseSim, 'String', getString(message('dsp:demo:GUIPauseSim')));
    end
    flagPauseSim = ~flagPauseSim;
    updateUserData();
    tuneParameters();
end

function hStopSimButtonCallback(~,~,~)
% Called when "Stop Simulation" button is pressed
    if isSimulink
        % Try-catch in case model is not open
        try
            set_param(systemName,'SimulationCommand','stop');
        catch e %#ok
        end
    else
        flagStopSim = true;
        updateUserData();
         tuneParameters();
    end
end

function hFigDeleteCallback(~,~,~)
    % Called when the UI is closed
    flagStopSim = true;   % Stop the simulation when UI is closed
    tuneParameters();
end

    function tuneParameters(varargin)
            if isSimulink
                tuneSimulinkParameters(varargin{:});
            end
    end

    function tuneSimulinkParameters(index)
        try
            % try-catch in case model is not open
            if UIMode == 1
                
                % Set block parameters
                
                set_param( parameters{index}.BlockName, parameters{index}.ParameterName,num2str(getWidgetValue(index)));
                
            else
                parameters{index}.Callback(getWidgetValue(index), index, systemName);
            end
        catch e %#ok
            
        end
    end

end

% validate inputs and add default values
function checkedInputs = checkInputs(inputs,UIMode)
    
    nInputs = length(inputs);
    checkedInputs = cell(nInputs); 
    
    for index = 1:nInputs
        
        validateattributes(inputs(index), {'struct'}, {'nonempty'});
        
        % The type is slider by default
        checkedInputs{index}.Type = 'slider';
        % Overwrite the type if specified by user
        if isfield(inputs(index), 'Type') && ~isempty(inputs(index).Type)
             checkedInputs{index}.Type = inputs(index).Type;
        end
        
        if isfield(inputs(index), 'Name') && ...
                ~isempty(inputs(index).Name)
            % Name should be a string
            validateattributes(inputs(index).Name,...
                {'char'}, {'vector'}); 
            checkedInputs{index}.Name = inputs(index).Name;
        else
            % Default value for Name
            checkedInputs{index}.Name = ['Parameter ',int2str(index)];
        end
        
        if strcmp(checkedInputs{index}.Type , 'slider')
            % Validate and set SliderScale
            if isfield(inputs(index), 'SliderScale') && ~isempty(inputs(index).SliderScale)
                validateattributes(inputs(index).SliderScale, {'char'}, {'vector'}); 
                if strcmpi(inputs(index).SliderScale, 'Log')
                    checkedInputs{index}.SliderScale = 'Log';
                elseif strcmpi(inputs(index).SliderScale, 'Linear')
                    checkedInputs{index}.SliderScale = 'Linear';
                else
                    checkedInputs{index}.SliderScale = 'Linear';
                    error(message('dsp:demo:GUIInvalidSliderScale', 'SliderScale'));
                end
            else
                checkedInputs{index}.SliderScale = 'Linear';
            end
            
            % Validate and set Limits
            if isfield(inputs(index), 'Limits') && ...
                    ~isempty(inputs(index).Limits)
                
                % Limits should be a vector of two numbers, and min should
                % be less than max. For Log scale, Limits should be non
                % negative.
                if strcmp(checkedInputs{index}.SliderScale, 'Log')
                    validateattributes(inputs(index).Limits,...
                        {'numeric'}, {'real', 'nonnegative', 'increasing', 'vector', 'numel', 2});
                else
                    validateattributes(inputs(index).Limits,...
                        {'numeric'}, {'real', 'increasing', 'vector', 'numel', 2});
                end
                checkedInputs{index}.Limits = inputs(index).Limits;
            else
                % Default value for Limits
                if isfield(inputs(index), 'InitialValue') &&...
                        ~isempty(inputs(index).InitialValue)
                    % if InitialValue is present
                    if strcmp(checkedInputs{index}.SliderScale, 'Log')
                        checkedInputs{index}.Limits = ...
                            [inputs(index).InitialValue/100, inputs(index).InitialValue*100];
                    else
                        checkedInputs{index}.Limits = ...
                            [inputs(index).InitialValue-0.5, inputs(index).InitialValue+0.5];
                    end
                else
                    if ~isfield(inputs(index), 'Values')
                        checkedInputs{index}.Limits = [0, 1];
                    end
                end
            end
        end
        
        if isfield(inputs(index), 'InitialValue') &&...
                ~isempty(inputs(index).InitialValue)
            % Initial value should be a scalar number, and should be
            % between non-negative if the slider is in Log mode.
            if isfield(checkedInputs{index}, 'SliderScale') && strcmp(checkedInputs{index}.SliderScale, 'Log')
                validateattributes(inputs(index).InitialValue,{'numeric'}, {'real', 'scalar', 'nonnegative'});
            else
                validateattributes(inputs(index).InitialValue,{'numeric'}, {'real', 'scalar'});
            end
                
            % Initial value must be within the limits.
            if isfield(checkedInputs{index}, 'Limits') && ~isempty(checkedInputs{index}.Limits)
                validateattributes(inputs(index).InitialValue, {'numeric'}, ...
                    {'>=', checkedInputs{index}.Limits(1), ...
                    '<=', checkedInputs{index}.Limits(2)});
            end
            checkedInputs{index}.InitialValue = inputs(index).InitialValue;
        else
            % Default value for InitialValue is average of min and max
            checkedInputs{index}.InitialValue = ...
               (checkedInputs{index}.Limits(1)+checkedInputs{index}.Limits(2))/2;
        end
        
         if isfield(inputs(index), 'Values')
             checkedInputs{index}.Values =inputs(index).Values;
         end
        
        if UIMode == 1
            if isfield(inputs(index), 'BlockName')
                checkedInputs{index}.BlockName = inputs(index).BlockName;
                checkedInputs{index}.ParameterName = inputs(index).ParameterName;
            end
           
        end
        
        if UIMode == 2
            if isfield(inputs(index), 'Callback')
                checkedInputs{index}.Callback = inputs(index).Callback;
            end
        end
    end
    
end

% Functions for MIDI control
function learnMIDIControl(uihandle)
    % LEARNMIDICONTROL Enable uicontrol to synchronize with a MIDI control
    %
    % This will attach a context menu to uicontrol UIHANDLE.
    % When the user selects the context menu, a dialog is opened to let the
    % user choose a MIDI control.  The chosen MIDI control is then
    % cross-wired with the uicontrol so that operating one control is
    % tracked by the other control.
    %
    % Caveats: 
    %   1) Only bidirection MIDI controls can track a uicontrol.
    %
    %   2) Currently only sliders, togglebuttons, pushbuttons, and
    %      checkboxes are supported.
    %
    %   3) The cross-wiring hijacks the uicontrol's callback.  For this to
    %      work correctly, the uicontrol callback MUST be set before
    %      calling learnMIDIControl()
    %
    %   4) The MIDI control will invoke the uicontrol's callback with []
    %      for the 2nd argument (eventdata).  This is correct for genuine
    %      uicontrols (only the KeyPressFcn is passed non-empty eventdata).
    %      This may be an issue for a uibuttongroup if its callback relies
    %      on eventdata. 
    %

    
    type = get(uihandle,'Type');
    style = get(uihandle,'Style');
    
    assert(strcmpi(type,'uicontrol'), 'only supports uicontrols');

    assert(strcmpi(style,'slider') || ...
        strcmpi(style,'togglebutton') || ...
        strcmpi(style,'pushbutton') || ...
        strcmpi(style,'checkbox'), ...
        'only supports sliders, togglebuttons, pushbuttons, and checkboxes');
    
    % Save this asap before it has a chance to get lost.
    if isempty(getappdata(uihandle,'savedcallback'))
        % Setting this twice => inifinite recursion
        setappdata(uihandle,'savedcallback',get(uihandle, 'callback'));
    end
    
    prefname = getPrefName(uihandle);
    if ispref('midi', prefname)
        pref = getpref('midi',prefname);
        s = warning('off','dsp:midi:specifiedDeviceOpenFailed');
        crossWireMidicontrolToUicontrol(uihandle, pref.controlNumber, pref.deviceName);
        warning(s.state,s.identifier);
    end
    
    attachContextMenu(uihandle, prefname);
    
end

function prefname = getPrefName(uihandle)
    fig = ancestor(uihandle,'figure');
    prefname = genvarname([get(fig,'Name') '_' get(uihandle,'Tag')]); 
end
    
function attachContextMenu(uihandle, prefname)

    m = get(uihandle, 'uicontextmenu');
    if isempty(m)
        fig = ancestor(uihandle,'figure');
        m = uicontextmenu('Parent',fig);
        set(uihandle, 'uicontextmenu',m)
    end
    
    m1 = uimenu(m,'Label','Synchronize to MIDI control...');
    set(m1, 'callback', @(hObj,eventdata)contextMenuCallback(hObj,eventdata, uihandle, prefname));

    function contextMenuCallback(hObj,eventdata,uihandle,prefname) %#ok<INUSL>
        [controlNumber, deviceName] = chooseControlViaDialog;
        if ~isempty(controlNumber)
            pref.controlNumber = controlNumber;
            pref.deviceName = deviceName;
            setpref('midi', prefname, pref);
            crossWireMidicontrolToUicontrol(uihandle, controlNumber, deviceName);
        end
    end
end

function crossWireMidicontrolToUicontrol(uihandle, controlNumber, deviceName)
    assert(~isempty(getappdata(uihandle,'savedcallback')),...
        'we are toast: setting callbacks without saving the original');
    mval = getMIDIValFromGUI(uihandle);
    mc = midicontrols(controlNumber, mval, 'MIDIDevice', deviceName);
    midisync(mc);
    if strcmp(get(uihandle,'Style'), 'pushbutton')
        midicallback(mc, @(mc)midiControlPushbuttonCallback(mc,uihandle));
    else
        midicallback(mc, @(mc)midiControlCallback(mc,uihandle));
    end        

    setappdata(uihandle,'midicontrol',mc);
    set(uihandle,'callback',@uicontrolCallback);
end

% invoked when midicontrol changes
function midiControlCallback(mc, uihandle)
    if isgraphics(uihandle)
        val = getGUIValFromMIDI(uihandle,mc);
        set(uihandle,'Value',val);
        cb = getappdata(uihandle,'savedcallback');
        cb(uihandle,[]);   % BEWARE: callback is getting empty eventdata
    end
end

% invoked when midicontrol changes
function midiControlPushbuttonCallback(mc, uihandle)
    if isgraphics(uihandle)
        if midiread(mc) == 0
            % Only fire callback on buttonup, ie, when value => 0
            % (same as uicontrol pushbutton)
            set(uihandle,'Value',get(uihandle,'Min'));
            cb = getappdata(uihandle,'savedcallback');
            cb(uihandle,[]);   % BEWARE: callback is getting empty eventdata
        end
    end
end

% invoked when uicontrol changes
function uicontrolCallback(uihandle,dummyeventdata)
    mc = getappdata(uihandle,'midicontrol');
    mval = getMIDIValFromGUI(uihandle);
    midisync(mc,mval);
    cb = getappdata(uihandle,'savedcallback');
    cb(uihandle,dummyeventdata);
end

function mval = getMIDIValFromGUI(uihandle)
    val = get(uihandle,'Value');
    min = get(uihandle,'Min');
    max = get(uihandle,'Max');
    mval = (val-min)/(max-min);
end

function val = getGUIValFromMIDI(uihandle,mc)
    mval = midiread(mc);
    min = get(uihandle,'Min');
    max = get(uihandle,'Max');
    val = min + (max-min)*mval;
end

function [controlNumber, deviceName] = chooseControlViaDialog
    fig = launchDialog;
    setControlAndDevice(fig, [], '');
    listenForAnyControlOnAnyDevice(fig);
    uiwait(fig);
    
    [controlNumber, deviceName] = getControlAndDevice(fig);
    delete(fig);
end

function listenForAnyControlOnAnyDevice(fig)
    data = guidata(fig);
    devinfo = midicontrols.devices;
    devinfo = devinfo([devinfo.input]);
    data.ctls = {};
    for d = devinfo
        % InitialValue helps detect togglebuttons' 1->0 transition
        h = midicontrols('MIDIDevice',d.name,'InitialValue',0.5);
        midicallback(h, @(ctl)midicb(ctl,d.name,fig));
        data.ctls{end+1} = h;
    end
    guidata(fig, data);

    
    function midicb(ctl,device,hObject)
        [~,lastctl] = read(ctl); % hidden method on midicontrols
        if ~isempty(lastctl)

            handles = guihandles(hObject);
            set(handles.textDisplay, 'String', ...
                sprintf('control %d on ''%s''', lastctl, device));
            
            setControlAndDevice(hObject, lastctl, device)
        end
    end

end

function setControlAndDevice(obj, control, device)
    data = guidata(obj);
    data.controlNumber = control;
    data.deviceName = device;
    guidata(obj, data);
end
    
function [control, device] = getControlAndDevice(obj)
    data = guidata(obj);
    control = data.controlNumber;
    device = data.deviceName;
end
    
function fig = launchDialog
    
    fig = figure(...
        'Units','characters',...
        'Color',[0.701960784313725 0.701960784313725 0.701960784313725],...
        'Colormap',[0 0 0.5625;0 0 0.625;0 0 0.6875;0 0 0.75;0 0 0.8125;0 0 0.875;0 0 0.9375;0 0 1;0 0.0625 1;0 0.125 1;0 0.1875 1;0 0.25 1;0 0.3125 1;0 0.375 1;0 0.4375 1;0 0.5 1;0 0.5625 1;0 0.625 1;0 0.6875 1;0 0.75 1;0 0.8125 1;0 0.875 1;0 0.9375 1;0 1 1;0.0625 1 1;0.125 1 0.9375;0.1875 1 0.875;0.25 1 0.8125;0.3125 1 0.75;0.375 1 0.6875;0.4375 1 0.625;0.5 1 0.5625;0.5625 1 0.5;0.625 1 0.4375;0.6875 1 0.375;0.75 1 0.3125;0.8125 1 0.25;0.875 1 0.1875;0.9375 1 0.125;1 1 0.0625;1 1 0;1 0.9375 0;1 0.875 0;1 0.8125 0;1 0.75 0;1 0.6875 0;1 0.625 0;1 0.5625 0;1 0.5 0;1 0.4375 0;1 0.375 0;1 0.3125 0;1 0.25 0;1 0.1875 0;1 0.125 0;1 0.0625 0;1 0 0;0.9375 0 0;0.875 0 0;0.8125 0 0;0.75 0 0;0.6875 0 0;0.625 0 0;0.5625 0 0],...
        'IntegerHandle','off',...
        'InvertHardcopy',get(0,'defaultfigureInvertHardcopy'),...
        'MenuBar','none',...
        'Name','Choose a MIDI control',...
        'NumberTitle','off',...
        'PaperPosition',get(0,'defaultfigurePaperPosition'),...
        'Position',[103.833333333333 45.75 58.3333333333333 15.75],...
        'Resize','off',...
        'HandleVisibility','callback',...
        'UserData',[],...
        'Tag','figure1',...
        'Visible','on',...
        'WindowStyle','modal' );

    % Instruction text
    uicontrol(...
        'Parent',fig,...
        'Units','characters',...
        'HorizontalAlignment','left',...
        'Position',[4.5 12.4166666666667 41.6666666666667 1.08333333333333],...
        'String','Operate the MIDI control to use.',...
        'Style','text',...
        'Tag','textInstructions' );
    
    % Control display
    uicontrol(...
        'Parent',fig,...
        'Units','characters',...
        'BackgroundColor',[1 1 1],...
        'HorizontalAlignment','left',...
        'Position',[4.5 9.75 45.3333333333333 1.08333333333333],...
        'String','',...
        'Style','text',...
        'Tag','textDisplay' );
    
%     % Checkbox to remember control
%     uicontrol(...
%         'Parent',fig,...
%         'Units','characters',...
%         'Position',[4.5 6.5 28.8333333333333 1.41666666666667],...
%         'String','Remember this choice',...
%         'Style','checkbox',...
%         'Value',0,...
%         'Tag','checkboxRemember',...
%         'Enable','off' );
    
    % OK button
    uicontrol(...
        'Parent',fig,...
        'Units','characters',...
        'Callback',@pushbuttonOKCallback,...
        'Position',[8.16666666666667 1.41666666666667 12 1.75],...
        'String','OK',...
        'Tag','pushbuttonOK' );
    
    % Cancel button
    uicontrol(...
        'Parent',fig,...
        'Units','characters',...
        'Callback',@pushbuttonCancelCallback,...
        'Position',[24.8333333333333 1.41666666666667 12 1.75],...
        'String','Cancel',...
        'Tag','pushbuttonCancel' );
    
    % Help button
    uicontrol(...
        'Parent',fig,...
        'Units','characters',...
        'Callback',@pushbuttonHelpCallback,...
        'Position',[41.5 1.41666666666667 12 1.75],...
        'String','Help',...
        'Tag','pushbuttonHelp' );
    

    function pushbuttonOKCallback(hObject, eventdata) %#ok<INUSD>
        % if get(handles.checkboxRemember,'Value')
        %     disp('Saving control in pref.');
        % else
        %     disp('Not saving control in pref.');
        % end
        uiresume;
    end
    
    function pushbuttonCancelCallback(hObject, eventdata) %#ok<INUSD>
        setControlAndDevice(hObject, [], '')
        uiresume;
    end
    
    function pushbuttonHelpCallback(hObject, eventdata) %#ok<INUSD>
        helpstr = {
            ['Operate the MIDI control that you wish to synchronize to this UI control. '...
            'As you operate a control, its device name and control number will be displayed. '...
            'Click OK when you are satisfied with your choice. ' ...
            'The UI control will then be synchronized with the MIDI control: '...
            'when you operate the MIDI control, the UI control will follow; '...
            'when you operate the UI control, the MIDI control will follow (if it has that capability).']
            ''
            'Click cancel to close the dialog without making a choice.'
            ''
            };
        helpdlg(helpstr,'Choose a MIDI control');
    end
end