
/* Copyright 2019-2021 The MathWorks, Inc. */

/* decim_dbl.c

    This file contains the implementation of FIRDecimation using polyPhase
*/
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "decim_dbl_avx2_blk.h"
#include <immintrin.h>

/* Forcing the states buffer to upadate once per step(),
   this implementation will take more memory for state_buffer.
*/
#define DELAY_STATES_ONCE_PER_PROCESS

/*-----------------Error codes-------------------*/
#define NO_ERR            (0)
#define ERR_NULL_POINTER  (-1)
#define ERR_FUNCTIONALITY (-2)

#define NULL_PTR_CHECK(ptr, str) \
    if(ptr == NULL) {                 \
        printf("NULL pointer received in %s()\n", str); \
        return (ERR_NULL_POINTER); \
    }

#define VECTOR_UNITS_FOR_256   (256/64)
#define UNROLL_FACTOR          (4)
#define UNROLL_VECTOR_ELEMENTS (UNROLL_FACTOR * VECTOR_UNITS_FOR_256)
#define UF1     (1 * VECTOR_UNITS_FOR_256)
#define UF2     (2 * VECTOR_UNITS_FOR_256)
#define UF3     (3 * VECTOR_UNITS_FOR_256)
#define UF4     (4 * VECTOR_UNITS_FOR_256)

static size_t modulus(size_t nNumerator, size_t nDenominator)
{
    while(nNumerator >= nDenominator)
    {
        nNumerator -= nDenominator;
    }
    return nNumerator;
}

/* converting the coefficients to polyphase form */
static int convInputPortToPoly(
    const double *coefficients,
    size_t actualLen,
    tFIRDecimatorDblRealHandle *S)
{
    size_t nConvertedFilterLength;
    size_t anDecimateFactor;
    size_t nPolyFilterLen;
    size_t i = 0;
    size_t k = 0;
    size_t j = 0;
    double *outCoefficients;

    NULL_PTR_CHECK(coefficients, "convInputPortToPoly");
    NULL_PTR_CHECK(S, "convInputPortToPoly");

    nConvertedFilterLength = S->nFilterLen;
    anDecimateFactor = S->nDecimateFactor;
    nPolyFilterLen = nConvertedFilterLength/ anDecimateFactor;

    outCoefficients = S->pfCoeff;
    memset(outCoefficients, 0, nConvertedFilterLength * sizeof(double));
    for (k = 0; (k < nPolyFilterLen)  && (i <actualLen); k++)
    {
        for (j=0; (j < anDecimateFactor) && (i <actualLen);j++)
        {
            outCoefficients[(j * nPolyFilterLen)+k] = coefficients[i];
            i++;
        }
    }

    return NO_ERR;
}

void MWDSPCRL_avx2_decim_dbl_real_init(tFIRDecimatorDblRealHandle *S,
    size_t nDecimateFactor,
    size_t nFilterLen,
    size_t nFrameLen,
    size_t nNumChannels,
    size_t StateSize,
    double *convCoefficients,
    double *arrState,
    double *arrAccum)
{

    size_t nOutputLenPerCh;
    size_t index;
    size_t nRoundedInFrameLen;

    nOutputLenPerCh = (size_t)(ceil(nFrameLen /(float) nDecimateFactor));
    nRoundedInFrameLen = nOutputLenPerCh * nDecimateFactor;

    S->nDecimateFactor = nDecimateFactor;
    S->nFilterLen = nFilterLen;
    S->nNumChannels = nNumChannels;
    S->nFrameLen = nFrameLen;
    S->nStatesLen = nRoundedInFrameLen+nFilterLen; /*Length per Channel*/
    S->nPhaseIdx = nDecimateFactor;

    S->pfCoeff = convCoefficients;
    S->pfStates = arrState;
    S->pfAccumulator = arrAccum;
    for (index =0; index < StateSize; index++)
    {
        arrState[index] = 0;
    }

    for (index =0; index < nNumChannels; index++)
    {
        arrAccum[index] = 0;
    }
}

int MWDSPCRL_avx2_decim_dbl_real_portcoeffs_process(
    const double *afInput,
    double *afOutput,
    tFIRDecimatorDblRealHandle *psHandle,
    const double *afCoeffs,
    size_t length)
{
    convInputPortToPoly(afCoeffs,length,psHandle);
    MWDSPCRL_avx2_decim_dbl_real_process(afInput,afOutput,psHandle);

    return NO_ERR;
}

/* function for stepProcess*/
int MWDSPCRL_avx2_decim_dbl_real_process(const double *afInput,
    double *afOutput,
    tFIRDecimatorDblRealHandle *psHandle)
{
    size_t nPolyFilterLen = 0;
    size_t nOutputLenPerCh = 0;
    size_t nPolyFilterStateLen = 0;
    size_t nPhaseIdxBeforeStep;
    size_t nPhaseIdxAfterStep;
    size_t nVector_elements;
    size_t polyFilterLen_vLen;
    size_t polyFilterLen_sLenRem;
    size_t nDecimateFactor = psHandle->nDecimateFactor;
    size_t nFilterLen = psHandle->nFilterLen;
    size_t nNumChannels = psHandle->nNumChannels;
    size_t nFrameLen = psHandle->nFrameLen;
    size_t nStatesLen = psHandle->nStatesLen;

    double *pfCoeff = psHandle->pfCoeff;
    double *pfStates = psHandle->pfStates;
    double *pfAccumulator = psHandle->pfAccumulator;

    size_t k= 0;
    size_t i= 0;
    size_t j= 0;

    /* Determine the valid functional parameters*/
    if ((nDecimateFactor <= 0) || \
        (nFilterLen <= 0) || \
        (nNumChannels <=0))
    {
        return ERR_FUNCTIONALITY;
    }

    /* Validate the pointers*/
    NULL_PTR_CHECK(pfCoeff, "MWDSPCRL_avx2_decim_dbl_real_process");
    NULL_PTR_CHECK(pfStates, "MWDSPCRL_avx2_decim_dbl_real_process");
    NULL_PTR_CHECK(pfAccumulator, "MWDSPCRL_avx2_decim_dbl_real_process");
    NULL_PTR_CHECK(afInput, "MWDSPCRL_avx2_decim_dbl_real_process");
    NULL_PTR_CHECK(afOutput, "MWDSPCRL_avx2_decim_dbl_real_process");

    /* Determine the polyphase index*/
    nPolyFilterLen = nFilterLen / nDecimateFactor;
    nPolyFilterStateLen = nStatesLen / nDecimateFactor;
    nOutputLenPerCh = (size_t)(ceil(nFrameLen /(float) nDecimateFactor));
    nPhaseIdxBeforeStep = psHandle->nPhaseIdx;
    nPhaseIdxAfterStep = nPhaseIdxBeforeStep;

    nVector_elements = UNROLL_VECTOR_ELEMENTS;
    polyFilterLen_vLen = nPolyFilterLen/nVector_elements;
    polyFilterLen_sLenRem = nPolyFilterLen - (polyFilterLen_vLen * nVector_elements);

    for (k = 0; k < nNumChannels; k++)
    {
        /* Adjust the states, output and input */
        double *pStates = pfStates + (k * nStatesLen);
        double *pOutput = afOutput + (k * nOutputLenPerCh);
        const double *pInput = afInput + (k * nFrameLen);
        double fAccumulator = *(pfAccumulator+k);
        size_t idx = 0;

        size_t nPhaseIdx = nPhaseIdxBeforeStep;
        size_t nStateOffset = nOutputLenPerCh;

        __m256d vfResult = _mm256_set_pd(fAccumulator, 0.0, 0.0, 0.0);
        double fScalarResult = 0.0;
        size_t nStateOffsetIdx = 0;

        /* per channel*/
        for (i = 0; i < nFrameLen; i++)
        {
            __m256d vfCoeff;
            __m256d vfState;
            __m256d vfCoeff_2;
            __m256d vfState_2;
            __m256d vfCoeff_3;
            __m256d vfState_3;
            __m256d vfCoeff_4;
            __m256d vfState_4;
            __m256d vfResult_2;
            __m256d vfResult_3;
            __m256d vfResult_4;
            double *pCurrCoeff;
            double *pCurrStates;
            double *ptempCoeff;
            double *ptempStates;

            if (nStateOffsetIdx == nDecimateFactor)
            {
                nStateOffset--;
                nStateOffsetIdx = 1;
            }
            else
            {
                nStateOffsetIdx++;
            }
            idx = nDecimateFactor - nPhaseIdx;

            /* Add to appropriate poly filter vfState*/
            pCurrCoeff = &pfCoeff[idx * nPolyFilterLen];
#ifdef DELAY_STATES_ONCE_PER_PROCESS
            pCurrStates = &pStates[(idx * nPolyFilterStateLen) + nStateOffset];
#else
            pCurrStates = &pStates[(idx * nPolyFilterLen)];
#endif /* DELAY_STATES_ONCE_PER_PROCESS */
            pCurrStates[0] = pInput[i];

            /* polyphase filtering */

            /* Base C code for the entire SIMD code
             *
             * for (size_t j = 0; j < nPolyFilterLen; j++)
             * {
             *      fAccumulator += pCurrCoeff[j] * pCurrStates[j];
             * }
             *
             */

            vfResult_2 = _mm256_setzero_pd();
            vfResult_3 = _mm256_setzero_pd();
            vfResult_4 = _mm256_setzero_pd();

            ptempCoeff = pCurrCoeff;
            ptempStates = pCurrStates;

            /* Unrolled 4 times*/
            for (j = 0; j < polyFilterLen_vLen; j++)
            {
                /* Load the coefficients in unrolled manner*/
                vfCoeff = _mm256_loadu_pd(ptempCoeff);
                vfCoeff_2 = _mm256_loadu_pd(ptempCoeff + UF1);
                vfCoeff_3 = _mm256_loadu_pd(ptempCoeff + UF2);
                vfCoeff_4 = _mm256_loadu_pd(ptempCoeff + UF3);

                /* Load the states in unrolled manner*/
                vfState = _mm256_loadu_pd(ptempStates);
                vfState_2 = _mm256_loadu_pd(ptempStates + UF1);
                vfState_3 = _mm256_loadu_pd(ptempStates + UF2);
                vfState_4 = _mm256_loadu_pd(ptempStates + UF3);

                /* Perform MAC operation for each unrolled vector*/
                vfResult = _mm256_fmadd_pd(vfCoeff, vfState, vfResult);
                vfResult_2 = _mm256_fmadd_pd(vfCoeff_2, vfState_2, vfResult_2);
                vfResult_3 = _mm256_fmadd_pd(vfCoeff_3, vfState_3, vfResult_3);
                vfResult_4 = _mm256_fmadd_pd(vfCoeff_4, vfState_4, vfResult_4);

                /* Update the states and coefficients for next vectors*/
                ptempCoeff = ptempCoeff + UF4;
                ptempStates = ptempStates + UF4;
            }

            /* Add the resultant from four vectors to final 'vfResult' */
            vfResult = _mm256_add_pd(vfResult_2, vfResult);
            vfResult_3 = _mm256_add_pd(vfResult_4, vfResult_3);
            vfResult = _mm256_add_pd(vfResult, vfResult_3);

            /* Check for further unprocessed states in above vectorization*/
            if (polyFilterLen_sLenRem)
            {
                /* Do vectorized MAC if the remaining elements are greater than equal to VECTOR_UNITS_FOR_256 */
                j = polyFilterLen_vLen * nVector_elements;
                for (; (j+UF1) < nPolyFilterLen; j=j+UF1)
                {
                    vfCoeff = _mm256_loadu_pd(&pCurrCoeff[j]);
                    vfState = _mm256_loadu_pd(&pCurrStates[j]);
                    vfResult = _mm256_fmadd_pd(vfCoeff, vfState, vfResult);
                }

                /* Process remaining elements if exists*/
                for (; j < nPolyFilterLen; j++)
                {
                    fScalarResult += pCurrCoeff[j] * pCurrStates[j];
                }
            }

            /* Delay the states */

#ifndef DELAY_STATES_ONCE_PER_PROCESS
            memmove((pCurrStates+1),(pCurrStates),(nPolyFilterLen-1)*sizeof(double));
#endif /* DELAY_STATES_ONCE_PER_PROCESS*/

            /* copy the decimated value only*/
            if (!idx)
            {
                double fResult[4];
                _mm256_storeu_pd(fResult, vfResult);
                fAccumulator = fResult[0] + fResult[1] + fResult[2] + fResult[3];
                fAccumulator += fScalarResult;

                vfResult = _mm256_setzero_pd();
                fScalarResult = 0;
                *(pOutput++) = fAccumulator;

                fAccumulator = 0;
                nPhaseIdx = 1;
            }
            else
            {
                /* update the index*/
                nPhaseIdx++;
            }
        }

       /* Accumulator needs to be added to 'fAccumulator' incase of MWDSPCRL_avx2_IMPLEMENTATION
         because intermediate are not added to 'fAccumulator'
        */
        if(idx)
        {
            double fResult[4];
            _mm256_storeu_pd(fResult, vfResult);
            fAccumulator = fResult[0] + fResult[1] + fResult[2] + fResult[3];
            fAccumulator += fScalarResult;
        }

        *(pfAccumulator+k) = fAccumulator;
        nPhaseIdxAfterStep = nPhaseIdx;

#ifdef DELAY_STATES_ONCE_PER_PROCESS
        if((nPhaseIdxAfterStep == nDecimateFactor) && ((nPhaseIdxBeforeStep == nDecimateFactor) || (nPhaseIdxBeforeStep == 0)))
        {
            double* pDst;
            double* pSrc;
            for (j =0; j < nDecimateFactor; j++)
            {
                pDst = &pStates[(j * nPolyFilterStateLen) + nOutputLenPerCh + 1];
                pSrc =  pDst - nOutputLenPerCh;
                memmove(pDst,pSrc,(nPolyFilterLen-1)*sizeof(double));
            }
        }
        else
        {
            /* Determine how many polyfilters the commutator moved after one complete counter clock rotation*/
            size_t nShiftType1 = modulus(nFrameLen,nDecimateFactor);
            /* Number of filters need to be shifted from the computed offset*/
            size_t nShiftsType2 = nDecimateFactor - nShiftType1;
            /* Get the buffer index*/
            size_t nIdx = (nDecimateFactor - nPhaseIdxAfterStep) + 1;

            /* Shifts for the poly filters which didn't produced output
            will be in clock wise of commutator
            */
            for (j = 0 ; j < nShiftType1; j++)
            {
                double* pDst;
                double* pSrc;
                if(nIdx == nDecimateFactor)
                {
                    nIdx = 0;
                }
                pDst = &pStates[(nIdx * nPolyFilterStateLen) + nOutputLenPerCh + 1];
                pSrc = &pStates[(nIdx * nPolyFilterStateLen) + nStateOffset];
                memmove(pDst,pSrc,(nPolyFilterLen-1)*sizeof(double));
                nIdx++;
            }
            /* Shifts for the poly filters which produced output
            will be in clock wise of commutator
            */
            for (j = 0; j < nShiftsType2; j++)
            {
                double* pDst;
                double* pSrc;
                if(nIdx == nDecimateFactor)
                {
                    nIdx = 0;
                }
                pDst = &pStates[(nIdx * nPolyFilterStateLen) + nOutputLenPerCh + 1];
                pSrc = &pStates[(nIdx * nPolyFilterStateLen) + nStateOffset + 1];
                memmove(pDst,pSrc,(nPolyFilterLen-1)*sizeof(double));
                nIdx++;
            }
        }
#endif /* DELAY_STATES_ONCE_PER_PROCESS */
    }

    /* Post-step contexts*/
    psHandle->nPhaseIdx = nPhaseIdxAfterStep;

    return NO_ERR;
}
