
/* Copyright 2020-2021 The MathWorks, Inc. */

/* decim_dbl_cmplx_avx2_blk.c

    This file contains the implementation of FIRDecimation using polyPhase
*/
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "decim_dbl_cmplx_avx2_blk.h"
#include <immintrin.h>

/* Forcing the states buffer to upadate once per step(),
   this implementation will take more memory for state_buffer.
*/
#define DELAY_STATES_ONCE_PER_PROCESS

/*-----------------Error codes-------------------*/
#define NO_ERR            (0)
#define ERR_NULL_POINTER  (-1)
#define ERR_FUNCTIONALITY (-2)

#define NULL_PTR_CHECK(ptr, str) \
    if(ptr == NULL) {                 \
        printf("NULL pointer received in %s()\n", str); \
        return (ERR_NULL_POINTER); \
    }

#define VECTOR_UNITS_FOR_256   (256/64)
#define UNROLL_FACTOR          (4)
#define UNROLL_VECTOR_ELEMENTS (UNROLL_FACTOR * VECTOR_UNITS_FOR_256)
#define UF1     (1 * VECTOR_UNITS_FOR_256)
#define UF2     (2 * VECTOR_UNITS_FOR_256)
#define UF3     (3 * VECTOR_UNITS_FOR_256)
#define UF4     (4 * VECTOR_UNITS_FOR_256)

static size_t modulus(size_t nNumerator, size_t nDenominator)
{
    while(nNumerator >= nDenominator)
    {
        nNumerator -= nDenominator;
    }
    return nNumerator;
}

/* converting the coefficients to polyphase form */
static int convInputPortToPoly(
    const double *coefficients,
    size_t actualLen,
    tFIRDecimatorDblCmplxHandle *S)
{
    size_t nConvertedFilterLength;
    size_t anDecimateFactor;
    size_t nPolyFilterLen;
    size_t i;
    size_t k;
    size_t j;
    size_t idx;
    double *outCoefficients;

    NULL_PTR_CHECK(coefficients, "convInputPortToPoly");
    NULL_PTR_CHECK(S, "convInputPortToPoly");
    nConvertedFilterLength = S->nFilterLen;
    anDecimateFactor = S->nDecimateFactor;
    nPolyFilterLen = nConvertedFilterLength/ anDecimateFactor;
    outCoefficients = S->pfCoeff;

    i = 0;
    memset(outCoefficients, 0, nConvertedFilterLength * sizeof(double));
    if (S->nIsRealCoeff)
    {
        i = 0;
        for (k = 0; (k < nPolyFilterLen)  && (i <actualLen); k++)
        {
            for (j=0; (j < anDecimateFactor) && (i <actualLen);j++)
            {
                outCoefficients[(j*nPolyFilterLen)+k] = coefficients[i];
                i++;
            }
        }
    }
    else
    {
        for (k = 0; (k < nPolyFilterLen)  && (i <actualLen); k += 2)
        {
            for (j=0; (j < anDecimateFactor) && (i <actualLen);j++)
            {
                idx = ((j * nPolyFilterLen) + k);
                outCoefficients[idx] = coefficients[i];
                outCoefficients[idx+1] = coefficients[i+1];
                i += 2;
            }
        }
    }

    return NO_ERR;
}

void MWDSPCRL_avx2_decim_dbl_cmplx_init(tFIRDecimatorDblCmplxHandle *S,
    size_t nDecimateFactor,
    size_t nFilterLen,
    size_t nFrameLen,
    size_t nNumChannels,
    size_t StateSize,
    size_t nIsRealCoeff,
    double *convCoefficients,
    double *arrState,
    double *arrAccum)
{
    size_t nOutEle;
    size_t nRoundedInEle;
    size_t index;
    size_t accumLen;

    nOutEle = (size_t) ceil((nFrameLen >> 1)/(float)nDecimateFactor);
    nRoundedInEle = nOutEle * nDecimateFactor;

    S->nDecimateFactor = nDecimateFactor;
    S->nFilterLen = nFilterLen;
    S->nNumChannels = nNumChannels;
    S->nFrameLen = nFrameLen; /* will be two times because of complex*/

    /* State buffer length per Channel */
    if (nIsRealCoeff)
    {
        /* In case of real coefficients even filter length should be multilpied */
        S->nStatesLen = (nRoundedInEle + nFilterLen) * 2;
    }
    else
    {
        /* In case of complex coefficients filter length is already the real+imaginary
        hence multiply only for the input frame elements*/
        S->nStatesLen = (nRoundedInEle * 2) + nFilterLen;
    }
    S->nPhaseIdx = nDecimateFactor;

    S->pfCoeff = convCoefficients;
    S->pfStates = arrState;
    S->pfAccumulator = arrAccum;
    S->nIsRealCoeff = nIsRealCoeff;
    for (index =0; index < StateSize; index++)
    {
        arrState[index] = 0;
    }
    /* For real and imaginary accumulators*/
    accumLen = nNumChannels * 2;
    for (index =0; index < accumLen; index++)
    {
        arrAccum[index] = 0;
    }
}

int MWDSPCRL_avx2_decim_dbl_cmplx_portcoeffs_process(
    const double *afInput,
    double *afOutput,
    tFIRDecimatorDblCmplxHandle *psHandle,
    const double *afCoeffs,
    size_t length)
{
    convInputPortToPoly(afCoeffs,length,psHandle);
    MWDSPCRL_avx2_decim_dbl_cmplx_process(afInput,afOutput,psHandle);

    return NO_ERR;
}

static int MWDSPCRL_avx2_decim_dbl_output_real_coeff(const double *afInput,
    double *afOutput,
    tFIRDecimatorDblCmplxHandle *psHandle)
{
    size_t nPolyFilterLen;
    size_t nOutputLenPerCh;
    size_t nPolyFilterStateLen;
    size_t nFrameElements;
    size_t nOutputElePerCh;
    size_t nStatesElements;
    size_t nPhaseIdxBeforeStep;
    size_t nPhaseIdxAfterStep;

    size_t nVector_elements;
    size_t polyFilterLen_vLen;
    size_t polyFilterLen_sLenRem;

    size_t nDecimateFactor = psHandle->nDecimateFactor;
    size_t nFilterLen = psHandle->nFilterLen;
    size_t nNumChannels = psHandle->nNumChannels;
    size_t nFrameLen = psHandle->nFrameLen; /* Len will be two times the elements */
    size_t nStatesLen = psHandle->nStatesLen; /* Len will be two times the elements */

    double *pfCoeff = psHandle->pfCoeff;
    double *pfStates = psHandle->pfStates;
    double *pfAccumulator = psHandle->pfAccumulator;

    size_t k= 0;
    size_t i= 0;
    size_t j= 0;

    /* Determine the valid functional parameters*/
    if ((nDecimateFactor <= 0) || \
        (nFilterLen <= 0) || \
        (nNumChannels <=0))
    {
        return ERR_FUNCTIONALITY;
    }

    /* Validate the pointers*/
    NULL_PTR_CHECK(pfCoeff, "MWDSPCRL_avx2_decim_dbl_cmplx_process");
    NULL_PTR_CHECK(pfStates, "MWDSPCRL_avx2_decim_dbl_cmplx_process");
    NULL_PTR_CHECK(pfAccumulator, "MWDSPCRL_avx2_decim_dbl_cmplx_process");
    NULL_PTR_CHECK(afInput, "MWDSPCRL_avx2_decim_dbl_cmplx_process");
    NULL_PTR_CHECK(afOutput, "MWDSPCRL_avx2_decim_dbl_cmplx_process");

    nFrameElements = nFrameLen >> 1;
    nStatesElements = nStatesLen >> 1;

    /* Determine the polyphase index*/
    nPolyFilterLen = nFilterLen / nDecimateFactor;
    nPolyFilterStateLen = nStatesElements / nDecimateFactor;
    nOutputElePerCh = ((size_t)ceil(nFrameElements /(float) nDecimateFactor));
    nOutputLenPerCh = nOutputElePerCh << 1;

    nPhaseIdxBeforeStep = psHandle->nPhaseIdx;
    nPhaseIdxAfterStep = nPhaseIdxBeforeStep;

    nVector_elements = UNROLL_VECTOR_ELEMENTS;
    polyFilterLen_vLen = nPolyFilterLen/nVector_elements;
    polyFilterLen_sLenRem = nPolyFilterLen - (polyFilterLen_vLen * nVector_elements);

    for (k = 0; k < nNumChannels; k++)
    {
        /* Adjust the states, output and input */
        double *pStates_Re = pfStates + (k * nStatesLen);
        double *pStates_Im = pfStates + (k * nStatesLen) + nStatesElements;
        double *pOutput = afOutput + (k * nOutputLenPerCh);
        const double *pInput = afInput + (k * nFrameLen);
        double fAccumulator_Re = *(pfAccumulator+(k*2));
        double fAccumulator_Im = *(pfAccumulator+(k*2)+1);
        size_t idx = 0;

        size_t nPhaseIdx = nPhaseIdxBeforeStep;
        size_t nStateOffset = nOutputElePerCh;

        __m256d vfResult_Re_1 = _mm256_set_pd(fAccumulator_Re, 0.0, 0.0, 0.0);
        __m256d vfResult_Im_1 = _mm256_set_pd(fAccumulator_Im, 0.0, 0.0, 0.0);
        double fScalarResult_Re = 0.0;
        double fScalarResult_Im = 0.0;
        size_t nStateOffsetIdx = 0;
        /* per channel*/
        for (i = 0; i < nFrameLen; i= i+2)
        {
            __m256d vfCoeff_1;
            __m256d vfCoeff_2;
            __m256d vfCoeff_3;
            __m256d vfCoeff_4;
            __m256d vfState_Re_1;
            __m256d vfState_Re_2;
            __m256d vfState_Re_3;
            __m256d vfState_Re_4;
            __m256d vfState_Im_1;
            __m256d vfState_Im_2;
            __m256d vfState_Im_3;
            __m256d vfState_Im_4;
            __m256d vfResult_Re_2;
            __m256d vfResult_Re_3;
            __m256d vfResult_Re_4;
            __m256d vfResult_Im_2;
            __m256d vfResult_Im_3;
            __m256d vfResult_Im_4;
            double* pCurrCoeff;
            double* pCurrStates_Re;
            double* pCurrStates_Im;
            double* pTmpCoeff;
            double* ptempStates_Re;
            double* ptempStates_Im;

            if (nStateOffsetIdx == nDecimateFactor)
            {
                nStateOffset--;
                nStateOffsetIdx = 1;
            }
            else
            {
                nStateOffsetIdx++;
            }
            idx = nDecimateFactor - nPhaseIdx;

            /* Add to appropriate poly filter vfState*/
            pCurrCoeff = &pfCoeff[idx * nPolyFilterLen];
#ifdef DELAY_STATES_ONCE_PER_PROCESS
            pCurrStates_Re = &pStates_Re[(idx * nPolyFilterStateLen) + nStateOffset];
            pCurrStates_Im = &pStates_Im[(idx * nPolyFilterStateLen) + nStateOffset];
#else
            pCurrStates_Re = &pStates_Re[(idx * nPolyFilterLen)];
            pCurrStates_Im = &pStates_Im[(idx * nPolyFilterLen)];
#endif /* DELAY_STATES_ONCE_PER_PROCESS */
            pCurrStates_Re[0] = pInput[i];
            pCurrStates_Im[0] = pInput[i+1];

            /* polyphase filtering */

            /* Base C code for the entire SIMD code
             *
             * for (size_t j = 0; j < nPolyFilterLen; j++)
             * {
             *      fAccumulator += pCurrCoeff[j] * pCurrStates[j];
             * }
             *
             */

            vfResult_Re_2 = _mm256_setzero_pd();
            vfResult_Re_3 = _mm256_setzero_pd();
            vfResult_Re_4 = _mm256_setzero_pd();
            vfResult_Im_2 = _mm256_setzero_pd();
            vfResult_Im_3 = _mm256_setzero_pd();
            vfResult_Im_4 = _mm256_setzero_pd();

            pTmpCoeff = pCurrCoeff;
            ptempStates_Re = pCurrStates_Re;
            ptempStates_Im = pCurrStates_Im;

            /* Unrolled 4 times*/
            for (j = 0; j < polyFilterLen_vLen; j++)
            {
                /* Load the coefficients in unrolled manner*/
                vfCoeff_1 = _mm256_loadu_pd(pTmpCoeff);
                vfCoeff_2 = _mm256_loadu_pd(pTmpCoeff + UF1);
                vfCoeff_3 = _mm256_loadu_pd(pTmpCoeff + UF2);
                vfCoeff_4 = _mm256_loadu_pd(pTmpCoeff + UF3);

                /* Load the states in unrolled manner*/
                vfState_Re_1 = _mm256_loadu_pd(ptempStates_Re);
                vfState_Re_2 = _mm256_loadu_pd(ptempStates_Re + UF1);
                vfState_Re_3 = _mm256_loadu_pd(ptempStates_Re + UF2);
                vfState_Re_4 = _mm256_loadu_pd(ptempStates_Re + UF3);

                vfState_Im_1 = _mm256_loadu_pd(ptempStates_Im);
                vfState_Im_2 = _mm256_loadu_pd(ptempStates_Im + UF1);
                vfState_Im_3 = _mm256_loadu_pd(ptempStates_Im + UF2);
                vfState_Im_4 = _mm256_loadu_pd(ptempStates_Im + UF3);

                /* Perform MAC operation for each unrolled vector*/
                vfResult_Re_1 = _mm256_fmadd_pd(vfCoeff_1, vfState_Re_1, vfResult_Re_1);
                vfResult_Re_2 = _mm256_fmadd_pd(vfCoeff_2, vfState_Re_2, vfResult_Re_2);
                vfResult_Re_3 = _mm256_fmadd_pd(vfCoeff_3, vfState_Re_3, vfResult_Re_3);
                vfResult_Re_4 = _mm256_fmadd_pd(vfCoeff_4, vfState_Re_4, vfResult_Re_4);

                vfResult_Im_1 = _mm256_fmadd_pd(vfCoeff_1, vfState_Im_1, vfResult_Im_1);
                vfResult_Im_2 = _mm256_fmadd_pd(vfCoeff_2, vfState_Im_2, vfResult_Im_2);
                vfResult_Im_3 = _mm256_fmadd_pd(vfCoeff_3, vfState_Im_3, vfResult_Im_3);
                vfResult_Im_4 = _mm256_fmadd_pd(vfCoeff_4, vfState_Im_4, vfResult_Im_4);

                /* Update the states and coefficients for next vectors*/
                pTmpCoeff = pTmpCoeff + UF4;
                ptempStates_Re = ptempStates_Re + UF4;
                ptempStates_Im = ptempStates_Im + UF4;
            }

            /* Add the resultant from four vectors to final 'vfResult' */
            vfResult_Re_1 = _mm256_add_pd(vfResult_Re_2, vfResult_Re_1);
            vfResult_Re_3 = _mm256_add_pd(vfResult_Re_4, vfResult_Re_3);
            vfResult_Re_1 = _mm256_add_pd(vfResult_Re_1, vfResult_Re_3);

            vfResult_Im_1 = _mm256_add_pd(vfResult_Im_2, vfResult_Im_1);
            vfResult_Im_3 = _mm256_add_pd(vfResult_Im_4, vfResult_Im_3);
            vfResult_Im_1 = _mm256_add_pd(vfResult_Im_1, vfResult_Im_3);

            /* Check for further unprocessed states in above vectorization*/
            if (polyFilterLen_sLenRem)
            {
                /* Do vectorized MAC if the remaining elements are greater than equal to VECTOR_UNITS_FOR_256 */
                j = polyFilterLen_vLen * nVector_elements;
                for (; (j+UF1) < nPolyFilterLen; j=j+UF1)
                {
                    vfCoeff_1 = _mm256_loadu_pd(&pCurrCoeff[j]);
                    vfState_Re_1 = _mm256_loadu_pd(&pCurrStates_Re[j]);
                    vfResult_Re_1 = _mm256_fmadd_pd(vfCoeff_1, vfState_Re_1, vfResult_Re_1);

                    vfState_Im_1 = _mm256_loadu_pd(&pCurrStates_Im[j]);
                    vfResult_Im_1 = _mm256_fmadd_pd(vfCoeff_1, vfState_Im_1, vfResult_Im_1);
                }

                /* Process remaining elements if exists*/
                for (; j < nPolyFilterLen; j++)
                {
                    fScalarResult_Re += pCurrCoeff[j] * pCurrStates_Re[j];
                    fScalarResult_Im += pCurrCoeff[j] * pCurrStates_Im[j];
                }
            }

            /* Delay the states */

#ifndef DELAY_STATES_ONCE_PER_PROCESS
            memmove((pCurrStates_Re+1),(pCurrStates_Re),(nPolyFilterLen-1)*sizeof(double));
            memmove((pCurrStates_Im+1),(pCurrStates_Im),(nPolyFilterLen-1)*sizeof(double));
#endif /* DELAY_STATES_ONCE_PER_PROCESS*/

            /* copy the decimated value only*/
            if (!idx)
            {
                double fResult[4];
                _mm256_storeu_pd(fResult, vfResult_Re_1);
                fAccumulator_Re = fResult[0] + fResult[1] + fResult[2] + fResult[3];
                fAccumulator_Re += fScalarResult_Re;

                vfResult_Re_1 = _mm256_setzero_pd();
                fScalarResult_Re = 0;
                *(pOutput++) = fAccumulator_Re;
                fAccumulator_Re = 0;

                _mm256_storeu_pd(fResult, vfResult_Im_1);
                fAccumulator_Im = fResult[0] + fResult[1] + fResult[2] + fResult[3];
                fAccumulator_Im += fScalarResult_Im;

                vfResult_Im_1 = _mm256_setzero_pd();
                fScalarResult_Im = 0;
                *(pOutput++) = fAccumulator_Im;
                fAccumulator_Im = 0;

                nPhaseIdx = 1;
            }
            else
            {
                /* update the index*/
                nPhaseIdx++;
            }
        }

       /* Accumulator needs to be added to 'fAccumulator' incase of MWDSPCRL_avx2_IMPLEMENTATION
         because intermediate are not added to 'fAccumulator'
        */
        if(idx)
        {
            double fResult[4];
            _mm256_storeu_pd(fResult, vfResult_Re_1);
            fAccumulator_Re = fResult[0] + fResult[1] + fResult[2] + fResult[3];
            fAccumulator_Re += fScalarResult_Re;

            _mm256_storeu_pd(fResult, vfResult_Im_1);
            fAccumulator_Im = fResult[0] + fResult[1] + fResult[2] + fResult[3];
            fAccumulator_Im += fScalarResult_Im;
        }

        *(pfAccumulator+(k*2)) = fAccumulator_Re;
        *(pfAccumulator+(k*2)+1) = fAccumulator_Im;
        nPhaseIdxAfterStep = nPhaseIdx;

#ifdef DELAY_STATES_ONCE_PER_PROCESS
        if((nPhaseIdxAfterStep == nDecimateFactor) && ((nPhaseIdxBeforeStep == nDecimateFactor) || (nPhaseIdxBeforeStep == 0)))
        {
            double* pDst_Re;
            double* pSrc_Re;
            double* pDst_Im;
            double* pSrc_Im;
            for (j =0; j < nDecimateFactor; j++)
            {
                pDst_Re = &pStates_Re[(j * nPolyFilterStateLen) + nOutputElePerCh + 1];
                pSrc_Re =  pDst_Re - nOutputElePerCh;
                memmove(pDst_Re,pSrc_Re,(nPolyFilterLen-1)*sizeof(double));

                pDst_Im = &pStates_Im[(j * nPolyFilterStateLen) + nOutputElePerCh + 1];
                pSrc_Im =  pDst_Im - nOutputElePerCh;
                memmove(pDst_Im,pSrc_Im,(nPolyFilterLen-1)*sizeof(double));
            }
        }
        /* For framelength non multipes of decimation factor*/
        else  /*TODO: This else condition is not triggered in automation tests due to limiation of simulink block*/
        {
            /* Determine how many polyfilters the commutator moved after one complete counter clock rotation*/
            size_t nShiftType1 = modulus(nFrameElements,nDecimateFactor);
            /* Number of filters need to be shifted from the computed offset*/
            size_t nShiftsType2 = nDecimateFactor - nShiftType1;
            /* Get the buffer index*/
            size_t nIdx = (nDecimateFactor - nPhaseIdxAfterStep) + 1;

            /* Shifts for the poly filters which didn't produced output
            will be in clock wise of commutator
            */
            for (j = 0 ; j < nShiftType1; j++)
            {
                double* pDst_Re;
                double* pSrc_Re;
                double* pDst_Im;
                double* pSrc_Im;
                if(nIdx == nDecimateFactor)
                {
                    nIdx = 0;
                }
                pDst_Re = &pStates_Re[(nIdx * nPolyFilterStateLen) + nOutputElePerCh + 1];
                pSrc_Re = &pStates_Re[(nIdx * nPolyFilterStateLen) + nStateOffset];
                memmove(pDst_Re,pSrc_Re,(nPolyFilterLen-1)*sizeof(double));

                pDst_Im = &pStates_Im[(nIdx * nPolyFilterStateLen) + nOutputElePerCh + 1];
                pSrc_Im = &pStates_Im[(nIdx * nPolyFilterStateLen) + nStateOffset];
                memmove(pDst_Im,pSrc_Im,(nPolyFilterLen-1)*sizeof(double));

                nIdx++;
            }
            /* Shifts for the poly filters which produced output
            will be in clock wise of commutator
            */
            for (j = 0; j < nShiftsType2; j++)
            {
                double* pDst_Re;
                double* pSrc_Re;
                double* pDst_Im;
                double* pSrc_Im;
                if(nIdx == nDecimateFactor)
                {
                    nIdx = 0;
                }
                pDst_Re = &pStates_Re[(nIdx * nPolyFilterStateLen) + nOutputElePerCh + 1];
                pSrc_Re = &pStates_Re[(nIdx * nPolyFilterStateLen) + nStateOffset + 1];
                memmove(pDst_Re,pSrc_Re,(nPolyFilterLen-1)*sizeof(double));

                pDst_Im = &pStates_Im[(nIdx * nPolyFilterStateLen) + nOutputElePerCh + 1];
                pSrc_Im = &pStates_Im[(nIdx * nPolyFilterStateLen) + nStateOffset + 1];
                memmove(pDst_Im,pSrc_Im,(nPolyFilterLen-1)*sizeof(double));

                nIdx++;
            }
        }
#endif /* DELAY_STATES_ONCE_PER_PROCESS */
    }

    /* Post-step contexts*/
    psHandle->nPhaseIdx = nPhaseIdxAfterStep;

    return NO_ERR;
}


static int MWDSPCRL_avx2_decim_dbl_output_cmplx_coeff(const double *afInput,
    double *afOutput,
    tFIRDecimatorDblCmplxHandle *psHandle)
{
    size_t nPolyFilterElements;
    size_t nOutputLenPerCh;
    size_t nPolyFilterStateLen;
    size_t nPolyFilterStateElements;
    size_t nFrameElements;
    size_t nOutputElePerCh;
    size_t nStatesElements;
    size_t nPolyFilterLen;
    size_t nPhaseIdxBeforeStep;
    size_t nPhaseIdxAfterStep;

    size_t nVecIterations;
    size_t nScaIterations;
    __m256d vNegFactor;

    size_t nDecimateFactor = psHandle->nDecimateFactor;
    size_t nFilterLen = psHandle->nFilterLen;
    size_t nNumChannels = psHandle->nNumChannels;
    size_t nFrameLen = psHandle->nFrameLen; /* Len will be two times the elements */
    size_t nStatesLen = psHandle->nStatesLen; /* Len will be two times the elements */

    double *pfCoeff = psHandle->pfCoeff;
    double *pfStates = psHandle->pfStates;
    double *pfAccumulator = psHandle->pfAccumulator;

    size_t k= 0;
    size_t i= 0;
    size_t j= 0;

    /* Determine the valid functional parameters*/
    if ((nDecimateFactor <= 0) || \
        (nFilterLen <= 0) || \
        (nNumChannels <=0))
    {
        return ERR_FUNCTIONALITY;
    }

    /* Validate the pointers*/
    NULL_PTR_CHECK(pfCoeff, "MWDSPCRL_avx2_decim_dbl_cmplx_process");
    NULL_PTR_CHECK(pfStates, "MWDSPCRL_avx2_decim_dbl_cmplx_process");
    NULL_PTR_CHECK(pfAccumulator, "MWDSPCRL_avx2_decim_dbl_cmplx_process");
    NULL_PTR_CHECK(afInput, "MWDSPCRL_avx2_decim_dbl_cmplx_process");
    NULL_PTR_CHECK(afOutput, "MWDSPCRL_avx2_decim_dbl_cmplx_process");

    /* Determine the polyphase index*/
    nPolyFilterLen = nFilterLen / nDecimateFactor;
    nPolyFilterStateLen = nStatesLen / nDecimateFactor;

    nFrameElements = nFrameLen >> 1;
    nStatesElements = nStatesLen >> 1;
    nPolyFilterElements = nPolyFilterLen >> 1;
    nPolyFilterStateElements = nPolyFilterStateLen >> 1;

    nOutputElePerCh = ((size_t)ceil(nFrameElements /(float) nDecimateFactor));
    nOutputLenPerCh = nOutputElePerCh << 1;

    nPhaseIdxBeforeStep = psHandle->nPhaseIdx;
    nPhaseIdxAfterStep = nPhaseIdxBeforeStep;

    nVecIterations = nPolyFilterElements >> 3;
    nScaIterations = nPolyFilterElements - (nVecIterations << 3);
    vNegFactor = _mm256_setr_pd(1,-1,1,-1);

    for (k = 0; k < nNumChannels; k++)
    {
        /* Adjust the states, output and input */
        double *pStates = pfStates + (k * nStatesLen);
        double *pOutput = afOutput + (k * nOutputLenPerCh);
        const double *pInput = afInput + (k * nFrameLen);
        double fAccumulator_Re = *(pfAccumulator+(k*2));
        double fAccumulator_Im = *(pfAccumulator+(k*2)+1);
        double fScalarResult_Re = 0.0;
        double fScalarResult_Im = 0.0;
        size_t idx = 0;

        size_t nPhaseIdx = nPhaseIdxBeforeStep;
        size_t nStateOffset = nOutputElePerCh;
        size_t nStateOffsetIdx = 0;
        __m256d vfResult = _mm256_setzero_pd();

        /* per channel*/
        for (i = 0; i < nFrameLen; i= i+2)
        {
            __m256d vCoeff_1;
            __m256d vCoeff_2;
            __m256d vCoeff_3;
            __m256d vCoeff_4;

            __m256d vData_1;
            __m256d vData_2;
            __m256d vData_3;
            __m256d vData_4;

            __m256d vReMul_1;
            __m256d vReMul_2;
            __m256d vReMul_3;
            __m256d vReMul_4;

            __m256d vImMul_1;
            __m256d vImMul_2;
            __m256d vImMul_3;
            __m256d vImMul_4;

            __m256d vRst_1;
            __m256d vRst_2;
            __m256d vRst_3;
            __m256d vRst_4;

            double* pCurrCoeff;
            double* pCurrStates;
            double* pTmpStates;
            double* pTmpCoeff;

            if (nStateOffsetIdx == nDecimateFactor)
            {
                nStateOffset--;
                nStateOffsetIdx = 1;
            }
            else
            {
                nStateOffsetIdx++;
            }
            idx = nDecimateFactor - nPhaseIdx;

            /* Add to appropriate poly filter vfState*/
            pCurrCoeff = &pfCoeff[idx * nPolyFilterLen];
#ifdef DELAY_STATES_ONCE_PER_PROCESS
            pCurrStates = &pStates[(idx * nPolyFilterStateLen) + (nStateOffset << 1)];
#else
            pCurrStates = &pStates[(idx * nPolyFilterLen)];
#endif /* DELAY_STATES_ONCE_PER_PROCESS */
            pCurrStates[0] = pInput[i];
            pCurrStates[1] = pInput[i+1];
            pTmpStates = pCurrStates;
            pTmpCoeff = pCurrCoeff;

            /*
            Plain C code
            for (j = 0; j < nPolyFilterLen; j +=2)
            {
                double coeffRe = pCurrCoeff[j];
                double coeffIm = pCurrCoeff[j+1];
                double dataRe = pCurrStates[j];
                double dataIm = pCurrStates[j+1];
                fAccumulator_Re += (coeffRe * dataRe) - (coeffIm * dataIm);
                fAccumulator_Im += (coeffIm * dataRe) + (coeffRe * dataIm);
            }
            */

            for (j = 0 ; j < nVecIterations;  j++)
            {
                vData_1 = _mm256_loadu_pd(pTmpStates);
                vCoeff_1 = _mm256_loadu_pd(pTmpCoeff);
                vReMul_1 = _mm256_mul_pd(vCoeff_1,vData_1);
                vCoeff_1 = _mm256_permute_pd (vCoeff_1, 0x5);
                vCoeff_1 = _mm256_mul_pd(vNegFactor,vCoeff_1);
                vImMul_1 = _mm256_mul_pd(vCoeff_1,vData_1);
                vRst_1 = _mm256_hsub_pd(vReMul_1, vImMul_1);

                vData_2 = _mm256_loadu_pd(pTmpStates + UF1);
                vCoeff_2 = _mm256_loadu_pd(pTmpCoeff + UF1);
                vReMul_2 = _mm256_mul_pd(vCoeff_2,vData_2);
                vCoeff_2 = _mm256_permute_pd (vCoeff_2, 0x5);
                vCoeff_2 = _mm256_mul_pd(vNegFactor,vCoeff_2);
                vImMul_2 = _mm256_mul_pd(vCoeff_2,vData_2);
                vRst_2 = _mm256_hsub_pd(vReMul_2, vImMul_2);

                vReMul_1 = _mm256_add_pd(vRst_1,vRst_2);
                vfResult = _mm256_add_pd(vfResult,vReMul_1);

                vData_3 = _mm256_loadu_pd(pTmpStates + UF2);
                vCoeff_3 = _mm256_loadu_pd(pTmpCoeff + UF2);
                vReMul_3 = _mm256_mul_pd(vCoeff_3,vData_3);
                vCoeff_3 = _mm256_permute_pd (vCoeff_3, 0x5);
                vCoeff_3 = _mm256_mul_pd(vNegFactor,vCoeff_3);
                vImMul_3 = _mm256_mul_pd(vCoeff_3,vData_3);
                vRst_3 = _mm256_hsub_pd(vReMul_3, vImMul_3);

                vData_4 = _mm256_loadu_pd(pTmpStates + UF3);
                vCoeff_4 = _mm256_loadu_pd(pTmpCoeff + UF3);
                vReMul_4 = _mm256_mul_pd(vCoeff_4,vData_4);
                vCoeff_4 = _mm256_permute_pd (vCoeff_4, 0x5);
                vCoeff_4 = _mm256_mul_pd(vNegFactor,vCoeff_4);
                vImMul_4 = _mm256_mul_pd(vCoeff_4,vData_4);
                vRst_4 = _mm256_hsub_pd(vReMul_4, vImMul_4);

                vReMul_2 = _mm256_add_pd(vRst_4,vRst_3);
                vfResult = _mm256_add_pd(vfResult,vReMul_2);

                pTmpStates += UF4;
                pTmpCoeff +=  UF4;
            }

            if(nScaIterations)
            {

                for (j = 0; j < nScaIterations ; j++)
                {
                    double coeffRe = pTmpCoeff[0];
                    double coeffIm = pTmpCoeff[1];
                    double dataRe = pTmpStates[0];
                    double dataIm = pTmpStates[1];
                    fScalarResult_Re += (coeffRe * dataRe) - (coeffIm * dataIm);
                    fScalarResult_Im += (coeffIm * dataRe) + (coeffRe * dataIm);

                    pTmpCoeff +=2;
                    pTmpStates +=2;
                }

            }
            /* Delay the states */

#ifndef DELAY_STATES_ONCE_PER_PROCESS
            memmove((pCurrStates+2),(pCurrStates),(nPolyFilterLen-2)*sizeof(double));
#endif /* DELAY_STATES_ONCE_PER_PROCESS*/

            /* copy the decimated value only*/
            if (!idx)
            {
                double fResult[4];
                _mm256_storeu_pd(fResult, vfResult);
                fAccumulator_Re += fResult[0] + fResult[2];
                fAccumulator_Re += fScalarResult_Re;

                fAccumulator_Im += fResult[1] + fResult[3];
                fAccumulator_Im += fScalarResult_Im;

                *(pOutput++) = fAccumulator_Re;
                fAccumulator_Re = 0;
                fScalarResult_Re = 0;

                *(pOutput++) = fAccumulator_Im;
                fAccumulator_Im = 0;
                fScalarResult_Im = 0;

                nPhaseIdx = 1;
                vfResult = _mm256_setzero_pd();
            }
            else
            {
                /* update the index*/
                nPhaseIdx++;
            }
        }

        if(idx)
        {
            double fResult[4];
            _mm256_storeu_pd(fResult, vfResult);
            fAccumulator_Re += fResult[0] + fResult[2];
            fAccumulator_Re += fScalarResult_Re;

            fAccumulator_Im += fResult[1] + fResult[3];
            fAccumulator_Im += fScalarResult_Im;
        }

        *(pfAccumulator+(k*2)) = fAccumulator_Re;
        *(pfAccumulator+(k*2)+1) = fAccumulator_Im;
        nPhaseIdxAfterStep = nPhaseIdx;

#ifdef DELAY_STATES_ONCE_PER_PROCESS
        if((nPhaseIdxAfterStep == nDecimateFactor) && ((nPhaseIdxBeforeStep == nDecimateFactor) || (nPhaseIdxBeforeStep == 0)))
        {
            double* pDst;
            double* pSrc;

            for (j =0; j < nDecimateFactor; j++)
            {
                pDst = &pStates[(j * nPolyFilterStateLen) + nOutputLenPerCh + 2];
                pSrc =  pDst - nOutputLenPerCh;
                memmove(pDst,pSrc,(nPolyFilterLen-2)*sizeof(double));
            }
        }
        /* For framelength non multipes of decimation factor*/
        else  /*TODO: This else condition is not triggered in automation tests due to limiation of simulink block*/
        {
            /* Determine how many polyfilters the commutator moved after one complete counter clock rotation*/
            size_t nShiftType1 = modulus(nFrameElements,nDecimateFactor);
            /* Number of filters need to be shifted from the computed offset*/
            size_t nShiftsType2 = nDecimateFactor - nShiftType1;
            /* Get the buffer index*/
            size_t nIdx = (nDecimateFactor - nPhaseIdxAfterStep) + 1;

            /* Shifts for the poly filters which didn't produced output
            will be in clock wise of commutator
            */
            for (j = 0 ; j < nShiftType1; j++)
            {
                double* pDst;
                double* pSrc;
                if(nIdx == nDecimateFactor)
                {
                    nIdx = 0;
                }
                pDst = &pStates[(nIdx * nPolyFilterStateLen) + nOutputLenPerCh + 2];
                pSrc = &pStates[(nIdx * nPolyFilterStateLen) + (nStateOffset <<1)];
                memmove(pDst,pSrc,(nPolyFilterLen-2)*sizeof(double));

                nIdx++;
            }
            /* Shifts for the poly filters which produced output
            will be in clock wise of commutator
            */
            for (j = 0; j < nShiftsType2; j++)
            {
                double* pDst;
                double* pSrc;
                if(nIdx == nDecimateFactor)
                {
                    nIdx = 0;
                }

                pDst = &pStates[(nIdx * nPolyFilterStateLen) + nOutputLenPerCh + 2];
                pSrc = &pStates[(nIdx * nPolyFilterStateLen) + (nStateOffset <<1) + 2];
                memmove(pDst,pSrc,(nPolyFilterLen-2)*sizeof(double));

                nIdx++;
            }
        }
#endif /* DELAY_STATES_ONCE_PER_PROCESS */
    }

    /* Post-step contexts*/
    psHandle->nPhaseIdx = nPhaseIdxAfterStep;

    return NO_ERR;
}



/* function for stepProcess*/
int MWDSPCRL_avx2_decim_dbl_cmplx_process(const double *afInput,
    double *afOutput,
    tFIRDecimatorDblCmplxHandle *psHandle)
{
    if(psHandle->nIsRealCoeff)
    {
        MWDSPCRL_avx2_decim_dbl_output_real_coeff(afInput,afOutput,psHandle);
        return NO_ERR;
    }
    else
    {
        MWDSPCRL_avx2_decim_dbl_output_cmplx_coeff(afInput,afOutput,psHandle);
        return NO_ERR;
    }
}