/* Copyright 2019-2020 The MathWorks, Inc. */

#include "fir_dbl_avx2_blk.h"
#include <immintrin.h>
#define CTRL_BIT 0xd8
/**
 * Reversal of Coefficients using AVX2 intrinsics
 * Called once in MWDSPCRL_avx2_discrete_fir_init_mask
 * Called iteratively in MWDSPCRL_avx2_discrete_fir_port
 * @param coeffs : Coefficients vector
 * @param reversedCoeffs : Reversed Coefficients vector
 * @param coeffs_length : Filter Order
 * Output
 * @return void
 **/

static void ReversalOfCoefficients(const double* coeffs,double* reversedCoeffs,size_t coeffs_length)
{
    /* Reversal of the Coefficients Array - AVX2 SIMD Implementation */
    /* Reversed Coefficients to be used for SIMD Implementation */
    __m256d loadCoefficients;
    size_t startIndex = 0;
    size_t endIndex = coeffs_length-1;
    size_t coefCount = coeffs_length >> 2;                                        /* Vector Loop run count for Reversal of coefficients*/
    size_t scalarCount = coeffs_length & 3;                                        /* Scalar Loop run count for Reversal of coefficients*/
    size_t idx;

    for (idx = 0; idx <coefCount; idx++)
    {
        loadCoefficients = _mm256_loadu_pd(coeffs + startIndex);
        loadCoefficients = _mm256_permute4x64_pd(loadCoefficients, _MM_SHUFFLE(0, 1, 2, 3));  /* Rearrange the coefficients in reverse order */
        startIndex += 4;
        _mm256_storeu_pd((reversedCoeffs + endIndex - 3), loadCoefficients);
        endIndex -= 4;
    }

    for (idx = 0; idx < scalarCount; idx++)
    {
        reversedCoeffs[endIndex--] = coeffs[startIndex++];
    }

}  /* End of Reversal of the Coefficients Array - AVX2 SIMD Implementation*/

/**
 * AVX2 implementation of Discrete FIR
 * Calculates the output data based on the filter coefficients and input data
 * @param coeffs : Coefficients vector
 * @param reversedCoeffs : Reversed coefficients vector
 * @param coeffs_length : Filter order
 * @param in : Input vector
 * @param frame_length : Frame length
 * @param Nchan : Number of Channels
 * @param states : States vector
 * Output
 * @param out : Output vector
 * @return void
 **/

static void DiscreteFIR(const double* coeffs,double* reversedCoeffs, size_t coeffs_length, const double* in, size_t frame_length,size_t num_chan, double* states, double* out)
{
    int ch;
    size_t coef_looplen = coeffs_length - 1;
    size_t numIterations=coef_looplen;
    /* Handling the case when input length or coefficient length is zero */
    if (frame_length == 0 || coeffs_length == 0)
    {
        size_t s;
        for (s = 0; s < num_chan*frame_length; s++)
            out[s] = 0.0;
    }
    if(frame_length<coef_looplen)
        numIterations=frame_length;

    for (ch = 0; ch < num_chan ;ch++)
    {
        size_t j;
        int statesOffset = (int)(ch * (coeffs_length - 1));
        int samplesOffset = (int)(ch * frame_length);
        int i;
        /* Reference C code
         * for (i = 0; i < (coeffs_length - 1); i++)
         * {
         *     acc = 0.0;
         *     for (j = 0; j < i + 1; j++)
         *     {
         *         acc += in[i - j] * coeffs[j];
         *     }
         *
         *     for (j = 0; j < (coeffs_length - 1) - i; j++)
         *     {
         *         acc += coeffs[(i + j) + 1] * states[j];
         *     }
         *     out[i] = acc;
         * }
         */
        for (i = 0; i < numIterations; i++)
        {
            __m256d accumulator = _mm256_setzero_pd();
            __m256d loadIn;
            __m256d loadCoeffs;
            __m256d loadState;
            __m256d vectorAddition;
            double scalarSum = 0.0;
            double vectorSum;
            size_t coefIdx = coef_looplen - i;                                    /* Coefficient offset */
            size_t inIdx = samplesOffset;                                        /* Input offset */
            size_t incr = i + 1;
            size_t vectorLoopCount = (incr) >> 2;
            size_t scalarLoopCount = (incr) & 3;
            size_t stateIdx;
            /* Loads inputs from in array and coefficients from the reversedCoeffs array
             * Stores the product in the accumulator
             */
            for (j = 0; j <vectorLoopCount; j++)
            {
                loadIn = _mm256_loadu_pd(&in[inIdx]);
                loadCoeffs = _mm256_loadu_pd(&reversedCoeffs[coefIdx]);
                accumulator = _mm256_fmadd_pd(loadIn, loadCoeffs, accumulator);
                inIdx += 4;
                coefIdx += 4;
            }

            for (j = 0; j < scalarLoopCount; j++)
                scalarSum += in[inIdx++] * reversedCoeffs[coefIdx++];

            stateIdx = statesOffset;                                            /* States offset */
            coefIdx = incr;                                                    /* Coefficient offset */
            vectorLoopCount = (coef_looplen - i) >> 2;
            scalarLoopCount = (coef_looplen - i) & 3;

            /* Loads coefficients from coeffs array and states from the states array
             * Stores the product in the accumulator
             */
            for (j = 0; j < vectorLoopCount; j++)
            {
                loadCoeffs = _mm256_loadu_pd(&coeffs[coefIdx]);
                loadState =_mm256_loadu_pd(&states[stateIdx]);
                accumulator = _mm256_fmadd_pd(loadCoeffs, loadState, accumulator);
                coefIdx += 4;
                stateIdx += 4;
            }

            for (j = 0; j < scalarLoopCount; j++)
            {
                scalarSum += coeffs[coefIdx++] * states[stateIdx++];
            }
            /* Computing sum of individual elements in a vector */
            vectorAddition = _mm256_hadd_pd(accumulator, accumulator);
            vectorAddition = _mm256_permute4x64_pd(vectorAddition, CTRL_BIT);
            vectorAddition = _mm256_hadd_pd(vectorAddition, vectorAddition);
            vectorSum = _mm_cvtsd_f64(_mm256_castpd256_pd128(vectorAddition));

            out[i + samplesOffset] = scalarSum + vectorSum;
        }


        /* Reference C Loop for iterations when Framesize > coef_looplen
         * for (i = (coeffs_length - 1); i < frame_length; i++) {
         *     acc1 = 0.0;
         *     for (j = 0; j < Lb; j++) {
         *         acc1 += in[i - j] * coeffs[j];
         *     }
         *     out[i] = acc1;
         * }
         */

        /* Loop Unrolled Twice */

        for (i = (int)(coef_looplen + samplesOffset); i < frame_length + samplesOffset; i++)  /* Discrete FIR - AVX2 SIMD Implementation for calculating output when frame_length > coef_looplen*/
        {
            /* Initializing the accumulators to Zero */
            __m256d accumulator = _mm256_setzero_pd();
            __m256d accumulator1 = _mm256_setzero_pd();
            __m256d loadIn;
            __m256d loadIn1;
            __m256d loadCoeffs;
            __m256d loadCoeffs1;
            __m256d vectorAddition;
            double scalarSum = 0.0;
            double vectorSum;
            size_t inIndex = i - coef_looplen;
            size_t coefIndex = 0;
            size_t ScalarLoopCount = coeffs_length & 7;
            size_t VectLoopCount = coeffs_length >> 3;
            for (j= 0; j < VectLoopCount; j++)
            {
                loadIn = _mm256_loadu_pd(&in[inIndex]);
                loadIn1 = _mm256_loadu_pd(&in[inIndex + 4]);
                loadCoeffs = _mm256_loadu_pd(&reversedCoeffs[coefIndex]);
                loadCoeffs1 = _mm256_loadu_pd(&reversedCoeffs[coefIndex + 4]);

                inIndex += 8;                                                   /* Auto-increment by 8 since loop unrolling takes place twice */
                coefIndex += 8;
                accumulator = _mm256_fmadd_pd(loadIn, loadCoeffs, accumulator);
                accumulator1 = _mm256_fmadd_pd(loadIn1, loadCoeffs1, accumulator1);

            }

            for (j = 0; j < ScalarLoopCount; j++)
            {
                scalarSum += in[inIndex++] * reversedCoeffs[coefIndex++];

            }

            /* Summing up the MUL-Accumulation results */
            accumulator = _mm256_add_pd(accumulator, accumulator1);
            /* Computing sum of individual elements in a vector */
            vectorAddition  = _mm256_hadd_pd(accumulator, accumulator);
            vectorAddition = _mm256_permute4x64_pd(vectorAddition, CTRL_BIT);
            vectorAddition = _mm256_hadd_pd(vectorAddition, vectorAddition);
            vectorSum = _mm_cvtsd_f64(_mm256_castpd256_pd128(vectorAddition));  /* Extract the lower 64bits */
            out[i] = vectorSum + scalarSum;
        }  /* End of Discrete FIR - AVX2 SIMD Implementation */

        /* Update delay line for next frame */
        if(frame_length<coef_looplen)
        {
            size_t loopvalue = coeffs_length - 2 - frame_length;
            for (i = (int)(statesOffset+loopvalue); i >= statesOffset; i--)
            {
                states[frame_length + i] = states[i];
            }

            for (i = 0; i < frame_length; i++)
            {
                states[frame_length - 1+(statesOffset - i)] = in[samplesOffset+i];
            }
        }
        else
        {
            for (j = 0; j < coef_looplen; j++)
                states[j + statesOffset] = in[frame_length - 1 - j + samplesOffset];
        }
    }
}

/**
 * Copies the initial states into the states array and does the Reversal of Coefficients
 * @param coeffs : Coefficients vector
 * @param reversedCoeffs : Reversed Coefficients vector
 * @param coeffs_length : Filter Order
 * @param initiaStates : Initial States vector
 * @param states : States vector
 * @param stateSize : States Size
 * Output
 * @return void
 **/
void MWDSPCRL_avx2_fir_dbl_real_init_mask(double* coeffs,double* reversedCoeffs,size_t coeffs_length,double* initialStates,double* states,size_t stateSize,size_t initStateSize,size_t num_chan)
{
    ReversalOfCoefficients(coeffs,reversedCoeffs,coeffs_length);

    /* Copying the InitialStates into a states buffer */

    if((initStateSize!=1) && (stateSize) && (initStateSize!=stateSize))
    {
        size_t i,tmp;
        for (i = 0; i < num_chan; i++) {
            tmp = i * initStateSize;
            memcpy(&states[tmp],&initialStates[0],initStateSize*sizeof(double));
        }
    }
    else if(initStateSize==stateSize)
    {
        memcpy(&states[0],&initialStates[0],stateSize*sizeof(double));
    }
    else
    {
        int i;
        double stateVal = *initialStates;
        for(i = 0; i < stateSize; i++)
            states[i] = stateVal;
    }
}

/**
 * Copies the initial states into the states array
 * @param initiaStates : Initial States vector
 * @param states : States vector
 * @param stateSize : States Size
 * Output
 * @return void
 **/

void MWDSPCRL_avx2_fir_dbl_real_init_port(double* initialStates,double* states,size_t stateSize,size_t initStateSize,size_t num_chan)
{
    /* Copying the InitialStates into a states buffer */

    if((initStateSize!=1) && (stateSize) && (initStateSize!=stateSize))
    {
        size_t i,tmp;
        for (i = 0; i < num_chan; i++) {
            tmp=i * initStateSize;
            memcpy(&states[tmp],&initialStates[0],initStateSize*sizeof(double));
        }
    }
    else if(initStateSize==stateSize){
        memcpy(&states[0],&initialStates[0],stateSize*sizeof(double));
    }
    else
    {
        int i;
        double stateVal = *initialStates;
        for(i = 0; i < stateSize; i++)
            states[i] = stateVal;
    }
}

/**
 *AVX2 implementation of Discrete FIR when Coefficient Source is Dialog Parameters
 *
 **/

void MWDSPCRL_avx2_fir_dbl_real_mask(double* coeffs,double* reversedCoeffs, size_t coeffs_length, const double* in, size_t frame_length,size_t num_chan, double* states, double* out)
{
    DiscreteFIR(coeffs,reversedCoeffs,coeffs_length,in,frame_length,num_chan,states,out);
}

/**
 *AVX2 implementation of Discrete FIR when Coefficient Source is Input Port
 *
 **/

void MWDSPCRL_avx2_fir_dbl_real_port(const double* coeffs,double* reversedCoeffs, size_t coeffs_length, const double* in, size_t frame_length,size_t num_chan, double* states, double* out)
{
    ReversalOfCoefficients(coeffs,reversedCoeffs,coeffs_length);
    DiscreteFIR(coeffs,reversedCoeffs,coeffs_length,in,frame_length,num_chan,states,out);
}
