/* Copyright 2020 The MathWorks, Inc. */
#include "fir_dft_dbl_avx2_blk.h"
#include <immintrin.h>
#define VECTOR_UNITS_FOR_256 (256/64)
#define UF1 (1 * VECTOR_UNITS_FOR_256)
#define UF2 (2 * VECTOR_UNITS_FOR_256)
#define UF3 (3 * VECTOR_UNITS_FOR_256)
#define UF4 (4 * VECTOR_UNITS_FOR_256)

static void firTransposed(DiscreteFIRTransDblReal *S, const double* in, double* out)
{
    size_t k;
    size_t samplesOffset;
    size_t statesOffset;
    size_t filtOrder;
    size_t frameLen;
    size_t nChan;
    double* states;
    const double* coeffs;
    frameLen = S->frameLen;
    nChan = S->nChan;
    filtOrder = S->filtOrder;
    states = S->states;
    coeffs = S->coeffs;

    for (k = 0; k < nChan; k++)
    {
        size_t i;
        size_t vectorLoopCount;
        size_t scalarLoopCount;
        samplesOffset = k * frameLen;
        statesOffset = k * (filtOrder + 1);
        vectorLoopCount = filtOrder >> 4;
        scalarLoopCount = filtOrder & 15;
        for (i = 0; i < frameLen; i++)
        {
            __m256d vCoeffs;
            __m256d vCoeffs_2;
            __m256d vCoeffs_3;
            __m256d vCoeffs_4;
            __m256d vInputs;
            __m256d vProd;
            __m256d vProd_2;
            __m256d vProd_3;
            __m256d vProd_4;
            __m256d vStates;
            __m256d vStates_2;
            __m256d vStates_3;
            __m256d vStates_4;
            __m256d vUpdatedStates;
            __m256d vUpdatedStates_2;
            __m256d vUpdatedStates_3;
            __m256d vUpdatedStates_4;
            double input;
            double prod;
            double *fTemp;
            const double *fTempCoeffs;
            double *pStates = states + statesOffset;
            size_t j ,vIdx;
            input = in[samplesOffset + i];
            prod = 0;
            j = 0;
            vInputs = _mm256_set1_pd(input);

            for (vIdx = 0; vIdx < vectorLoopCount; vIdx++)
            {
                fTempCoeffs = coeffs + j;
                vCoeffs = _mm256_loadu_pd(fTempCoeffs);
                vCoeffs_2 = _mm256_loadu_pd(fTempCoeffs + UF1);
                vCoeffs_3 = _mm256_loadu_pd(fTempCoeffs + UF2);
                vCoeffs_4 = _mm256_loadu_pd(fTempCoeffs + UF3);

                vProd = _mm256_mul_pd(vCoeffs, vInputs);
                vProd_2 = _mm256_mul_pd(vCoeffs_2, vInputs);
                vProd_3 = _mm256_mul_pd(vCoeffs_3, vInputs);
                vProd_4 = _mm256_mul_pd(vCoeffs_4, vInputs);

                fTemp = pStates + j + 1;
                vStates = _mm256_loadu_pd(fTemp );
                vStates_2 = _mm256_loadu_pd(fTemp + UF1);
                vStates_3 = _mm256_loadu_pd(fTemp + UF2);
                vStates_4 = _mm256_loadu_pd(fTemp + UF3);

                vUpdatedStates = _mm256_add_pd(vStates, vProd);
                vUpdatedStates_2 = _mm256_add_pd(vStates_2, vProd_2);
                vUpdatedStates_3 = _mm256_add_pd(vStates_3, vProd_3);
                vUpdatedStates_4 = _mm256_add_pd(vStates_4, vProd_4);

                fTemp = pStates + j ;
                _mm256_storeu_pd(fTemp, vUpdatedStates);
                _mm256_storeu_pd(fTemp + UF1, vUpdatedStates_2);
                _mm256_storeu_pd(fTemp + UF2, vUpdatedStates_3);
                _mm256_storeu_pd(fTemp + UF3, vUpdatedStates_4);
                j += UF4;

            }
            if (scalarLoopCount) {
                for (; j + UF1 < filtOrder; j += UF1)
                {
                    vCoeffs = _mm256_loadu_pd(coeffs + j);
                    vProd = _mm256_mul_pd(vCoeffs, vInputs);
                    vStates = _mm256_loadu_pd(pStates + j + 1);
                    vUpdatedStates = _mm256_add_pd(vStates,vProd);
                    _mm256_storeu_pd(pStates + j, vUpdatedStates);
                }
                for (; j < filtOrder; j++)
                {
                    prod = coeffs[j] * input;
                    pStates[j] = pStates[j + 1] + prod;
                }
            }
            out[i + samplesOffset] = pStates[0];
        }
    }
}

void MWDSPCRL_avx2_fir_dft_dbl_real_init(DiscreteFIRTransDblReal *S, size_t frameLen, size_t nChan, size_t filtOrder, double* states, const double* coeffs,double* initialStates,size_t stateSize,size_t initStateSize)
{
    if((nChan > 1) && (initStateSize != 1))
    {
        if((stateSize) && (initStateSize != stateSize - 2 * nChan))
        {
            size_t i,tmp;
            for (i = 0; i < nChan; i++) {
                tmp = i * (initStateSize + 2);
                memcpy(&states[tmp + 1],&initialStates[0],initStateSize * sizeof(double));
            }
        }
        else if((stateSize) && (initStateSize==stateSize- 2 * nChan))
        {
            size_t i,tmp,tempStateSize;
            tempStateSize = initStateSize/nChan;
            for (i = 0; i < nChan; i++) {
                tmp=i * (tempStateSize + 2);
                memcpy(&states[tmp + 1],&initialStates[i * tempStateSize],tempStateSize * sizeof(double));
            }            
                
        }
    }
    else if(initStateSize==stateSize - 2){
        memcpy(&states[1],&initialStates[0],initStateSize*sizeof(double));
    }
    else
    {
        double stateVal = *initialStates;
        if(nChan==1)
        {
            int i;
            for(i = 1; i < stateSize-1; i++)
                states[i] = stateVal;
        }
        else
        {
            size_t i,j,tmp;
            for (i = 0; i < nChan; i++) {
                tmp=i * (filtOrder+1);
                for(j=0; j < filtOrder-1; j++)
                {
                    states[tmp+1]=stateVal;
                    tmp++;
                }
            }
        }
    }
    S->frameLen = frameLen;
    S->nChan = nChan;
    S->filtOrder = filtOrder;
    S->coeffs=coeffs;
    S->states=states;
}

void MWDSPCRL_avx2_fir_dft_dbl_real_process(DiscreteFIRTransDblReal *S, const double* in, double* out){
    firTransposed(S, in, out);
}
