/* Copyright 2020 The MathWorks, Inc. */
#include "fir_dft_sgl_avx2_blk.h"
#include <immintrin.h>
#define VECTOR_UNITS_FOR_256 (256/32)
#define UF1 (1 * VECTOR_UNITS_FOR_256)
#define UF2 (2 * VECTOR_UNITS_FOR_256)
#define UF3 (3 * VECTOR_UNITS_FOR_256)
#define UF4 (4 * VECTOR_UNITS_FOR_256)

static void firTransposed(DiscreteFIRTransSglReal *S, const float* in, float* out)
{
    size_t k;
    size_t samplesOffset;
    size_t statesOffset;
    size_t filtOrder;
    size_t frameLen;
    size_t nChan;
    float* states;
    const float* coeffs;
    frameLen = S->frameLen;
    nChan = S->nChan;
    filtOrder = S->filtOrder;
    states = S->states;
    coeffs = S->coeffs;

    for (k = 0; k < nChan; k++)
    {
        size_t i;
        size_t vectorLoopCount;
        size_t scalarLoopCount;
        samplesOffset = k * frameLen;
        statesOffset = k * (filtOrder + 1);
        vectorLoopCount = filtOrder >> 5;
        scalarLoopCount = filtOrder & 31;
        for (i = 0; i < frameLen; i++)
        {
            __m256 vCoeffs;
            __m256 vCoeffs_2;
            __m256 vCoeffs_3;
            __m256 vCoeffs_4;
            __m256 vInputs;
            __m256 vProd;
            __m256 vProd_2;
            __m256 vProd_3;
            __m256 vProd_4;
            __m256 vStates;
            __m256 vStates_2;
            __m256 vStates_3;
            __m256 vStates_4;
            __m256 vUpdatedStates;
            __m256 vUpdatedStates_2;
            __m256 vUpdatedStates_3;
            __m256 vUpdatedStates_4;
            float input;
            float prod;
            float *fTemp;
            const float *fTempCoeffs;
            float *pStates = states + statesOffset;
            size_t j ,vIdx;
            input = in[samplesOffset + i];
            prod = 0;
            j = 0;
            vInputs = _mm256_set1_ps(input);

            for (vIdx = 0; vIdx < vectorLoopCount; vIdx++)
            {
                fTempCoeffs = coeffs + j;
                vCoeffs = _mm256_loadu_ps(fTempCoeffs);
                vCoeffs_2 = _mm256_loadu_ps(fTempCoeffs + UF1);
                vCoeffs_3 = _mm256_loadu_ps(fTempCoeffs + UF2);
                vCoeffs_4 = _mm256_loadu_ps(fTempCoeffs + UF3);

                vProd = _mm256_mul_ps(vCoeffs, vInputs);
                vProd_2 = _mm256_mul_ps(vCoeffs_2, vInputs);
                vProd_3 = _mm256_mul_ps(vCoeffs_3, vInputs);
                vProd_4 = _mm256_mul_ps(vCoeffs_4, vInputs);

                fTemp = pStates + j + 1;
                vStates = _mm256_loadu_ps(fTemp );
                vStates_2 = _mm256_loadu_ps(fTemp + UF1);
                vStates_3 = _mm256_loadu_ps(fTemp + UF2);
                vStates_4 = _mm256_loadu_ps(fTemp + UF3);

                vUpdatedStates = _mm256_add_ps(vStates, vProd);
                vUpdatedStates_2 = _mm256_add_ps(vStates_2, vProd_2);
                vUpdatedStates_3 = _mm256_add_ps(vStates_3, vProd_3);
                vUpdatedStates_4 = _mm256_add_ps(vStates_4, vProd_4);

                fTemp = pStates + j ;
                _mm256_storeu_ps(fTemp, vUpdatedStates);
                _mm256_storeu_ps(fTemp + UF1, vUpdatedStates_2);
                _mm256_storeu_ps(fTemp + UF2, vUpdatedStates_3);
                _mm256_storeu_ps(fTemp + UF3, vUpdatedStates_4);
                j += UF4;
            }
            if (scalarLoopCount) {
                for (; j + UF1 < filtOrder; j += UF1)
                {
                    vCoeffs = _mm256_loadu_ps(coeffs + j);
                    vProd = _mm256_mul_ps(vCoeffs, vInputs);
                    vStates = _mm256_loadu_ps(pStates + j + 1);
                    vUpdatedStates = _mm256_add_ps(vStates,vProd);
                    _mm256_storeu_ps(pStates + j, vUpdatedStates);
                }
                for (; j < filtOrder; j++)
                {
                    prod = coeffs[j] * input;
                    pStates[j] = pStates[j + 1] + prod;
                }
            }
            out[i + samplesOffset] = pStates[0];
        }
    }
}

void MWDSPCRL_avx2_fir_dft_sgl_real_init(DiscreteFIRTransSglReal *S, size_t frameLen, size_t nChan, size_t filtOrder, float* states, const float* coeffs,float* initialStates,size_t stateSize,size_t initStateSize)
{
	if((nChan > 1) && (initStateSize != 1))
    {
        if((stateSize) && (initStateSize != stateSize - 2 * nChan))
        {
            size_t i,tmp;
            for (i = 0; i < nChan; i++) {
                tmp = i * (initStateSize + 2);
                memcpy(&states[tmp + 1],&initialStates[0],initStateSize * sizeof(float));
            }
        }
        else if((stateSize) && (initStateSize==stateSize-2*nChan))
        {
            size_t i,tmp,tempStateSize;
            tempStateSize=initStateSize/nChan;
            for (i = 0; i < nChan; i++) {
                tmp=i * (tempStateSize + 2);
                memcpy(&states[tmp + 1],&initialStates[i * tempStateSize],tempStateSize * sizeof(float));
            }            
                
        }
    }

    else if(initStateSize==stateSize - 2){
        memcpy(&states[1],&initialStates[0],initStateSize*sizeof(float));
    }
    else
    {
        float stateVal = *initialStates;
        if(nChan==1)
        {
            int i;
            for(i = 1; i < stateSize-1; i++)
                states[i] = stateVal;
        }
        else
        {
            size_t i,j,tmp;
            for (i = 0; i < nChan; i++) {
                tmp=i * (filtOrder+1);
                for(j=0; j < filtOrder-1; j++)
                {
                    states[tmp+1]=stateVal;
                    tmp++;
                }
            }
        }
    }
    S->frameLen = frameLen;
    S->nChan = nChan;
    S->filtOrder = filtOrder;
    S->coeffs=coeffs;
    S->states=states;
}

void MWDSPCRL_avx2_fir_dft_sgl_real_process(DiscreteFIRTransSglReal *S, const float* in, float* out){
    firTransposed(S, in, out);
}
