/* Copyright 2019-2020 The MathWorks, Inc. */

#include "fir_sgl_avx2_blk.h"
#include <immintrin.h>
/**
 * Reversal of Coefficients using AVX2  intrinsics
 * Called once in MWDSPCRL_avx2_discrete_fir_init_mask
 * Called iteratively in MWDSPCRL_avx2_discrete_fir_port
 * @param coeffs : Coefficients vector
 * @param coeffs_rev : Reversed Coefficients vector
 * @param coeffs_length : Filter Order
 * Output
 * @return void
 **/

static void ReversalOfCoefficients(const float* coeffs,float* coeffs_rev,size_t coeffs_length)
{
    /* Reversal of the Coefficients Array - AVX2 SIMD Implementation */
    /* Reversed Coefficients to be used for SIMD Implementation */
    __m256 load_coeffs;
    size_t start_indx=0;
    size_t end_indx = coeffs_length - 1;
    size_t remainder = coeffs_length & 7;                                        /* Scalar loop run count*/
    size_t loopcount = coeffs_length >> 3;                                        /* Vector loop run count*/
    size_t s = 0;
    /* Reversal of the Coefficients Array - AVX2 SIMD Implementation */
    for (s = 0; s <loopcount; s++)
    {
        load_coeffs = _mm256_loadu_ps(coeffs + start_indx);                                              /* Load coefficients array */
        load_coeffs = _mm256_permutevar8x32_ps(load_coeffs, _mm256_set_epi32(0, 1, 2, 3, 4, 5, 6, 7));   /* Reverses the elements in load_coeffs vector */
        start_indx += 8;
        _mm256_storeu_ps((coeffs_rev + end_indx-7), load_coeffs);                                         /* Stores the load_coeffsrev vector in coeffs_rev array */
        end_indx -= 8;
    }

    /* Reversing the remaining array data not divisible by 8 */
    for (s = 0; s < remainder; s++)
    {
        coeffs_rev[end_indx--] = coeffs[start_indx++];
    }
    /* End of Reversal of the Coefficients Array - AVX2 SIMD Implementation */
}

/**
 * AVX2 implementation of Discrete FIR
 * Calculates the output data based on the filter coefficients and input data
 * @param coeffs : Coefficients vector
 * @param coeffs : Reversed coefficients vector
 * @param coeffs_length : Filter order
 * @param in : Input vector
 * @param frame_length  : Frame length
 * @param num_chan : Number of num_chan
 * @param states : States vector
 * Output
 * @param out : Output vector
 * @return void
 **/

static void DiscreteFIR(const float* coeffs,float* coeffs_rev,size_t coeffs_length , const float* in, size_t frame_length , size_t num_chan, float* states, float* out)
{
    __m256 setone = _mm256_set_ps(1, 1, 1, 1, 1, 1, 1, 1);                        /* Sets all the elements in the vector with 1 */
    int ch;
    /* Handling the case when input length or coefficient length is zero */
    if (frame_length == 0 || coeffs_length== 0)
    {
        size_t s;
        for ( s = 0; s < num_chan*frame_length ; s++)
            out[s] = 0.0;
    }

    /* AVX2 SIMD Implementation when Frame length > Filter order */
        /* Initialization C Loop */

        /*
         * for (i = 0; i < (coeffs_length - 1); i++)
         * {
         * acc = 0.0;
         * for (j = 0; j < i + 1; j++)
         * {
         * acc += in[i - j] * b[j];
         * }
         *
         * for (j = 0; j < (coeffs_length - 1) - i; j++)
         * {
         * acc += b[(i + j) + 1] *
         * states[j];
         * }
         *
         * out[i] = acc;
         * }*/

        /* Initialization Loop - AVX2 SIMD Implementation */

        /* Consume delay line and beginning of input samples */
        for (ch = 0; ch < num_chan; ch++)
        {
            size_t coef_looplen = coeffs_length - 1;
            size_t j;
            int statesOffset = (int)(ch*coef_looplen);
            int samplesOffset = (int)(ch*frame_length);
            int i;
            size_t numIterations=coef_looplen;
            if(frame_length<coef_looplen)
                numIterations=frame_length;

            for (i = 0; i < numIterations; i++)
            {
                /* Loop 1 - operations from zero to (i+1) */
                __m256 sum;
                __m256 permuted_sum;
                __m256 final_vector_sum;
                /* Initializing the accumulator to Zero */
                __m256 accumulator = _mm256_setzero_ps();
                float scalarSum = 0;
                size_t coeffs_indx = coef_looplen - i;                                /* Coefficient offset*/
                size_t in_indx = samplesOffset;                                        /* Inputs offset*/
                size_t incr = i + 1;
                size_t loopCount = (incr) >> 3;                                     /* Vector loop run count */
                size_t remLoopCount = (incr) & 7;                                   /* Scalar loop run count */
                size_t remaining_coeffs;
                size_t states_index1;

                for (j = 0; j < loopCount; j++)
                {
                    __m256 load_In1 = _mm256_loadu_ps(in + in_indx);                    /* Loading eight values from &in[in_indx] */
                    __m256 load_coeffs1 = _mm256_loadu_ps(coeffs_rev + coeffs_indx);    /* Loading eight values from &coeffs_rev[coeffs_indx] */
                    accumulator = _mm256_fmadd_ps(load_In1, load_coeffs1,accumulator);  /* acc1=acc1+(load_In1*load_coeffs1) */
                    in_indx += 8;
                    coeffs_indx += 8;
                }
                /* Scalar operations for data not divisible by 8 */
                for (j= 0; j < remLoopCount; j++)
                    scalarSum += in[in_indx++] * coeffs_rev[coeffs_indx++];


                /* Loop 2 -operations from (i+1) to coef_looplen */
                remaining_coeffs = coef_looplen - i;
                coeffs_indx = i + 1;                                     /* Coefficient offset*/
                states_index1 = statesOffset;                     /* State offset*/
                loopCount = remaining_coeffs >> 3;
                remLoopCount = remaining_coeffs & 7;
                for (j = 0; j < loopCount; j++)
                {
                    __m256 load_coeffs2 = _mm256_loadu_ps(coeffs + coeffs_indx);          /* Loading eight values from &coeffs[coeffs_indx] */
                    __m256 load_states = _mm256_loadu_ps(states + states_index1);          /* Loading eight values from &states[states_index1] */
                    accumulator = _mm256_fmadd_ps(load_coeffs2, load_states,accumulator); /* acc2=acc2+(load_coeffs2*load_states) */
                    coeffs_indx += 8;
                    states_index1 += 8;
                }
                /* Scalar operations for data not divisible by 8 */
                for (j = 0; j < remLoopCount; j++)
                {
                    scalarSum += coeffs[coeffs_indx++] * states[states_index1++];

                }
                sum = _mm256_dp_ps(accumulator, setone, 0xf1);                                             /* Finding the sum of lower 4 and upper 4 elements and store at index 0 and 4 */
                permuted_sum = _mm256_permutevar8x32_ps(sum, _mm256_set_epi32(0, 1, 2, 3, 5, 6, 4, 0));     /* Relocate the element at index 4 to index 1 */
                final_vector_sum = _mm256_hadd_ps(permuted_sum, permuted_sum);                            /* Horizontally add the elements */
                out[samplesOffset+i] = scalarSum + _mm_cvtss_f32(_mm256_extractf128_ps(final_vector_sum, 0));    /* Extract lower 32 bits and perform scalar addition */

            }
            /* End Of Initialization Loop - AVX2 SIMD Implementation */


            /* Main Loop */
            /*
             * for (i = (coeffs_length - 1); i < frame_length ; i++) {
             * acc1 = 0.0;
             * for (j = 0; j < coeffs_length; j++) {
             * acc1 += in[i - j] * b[j];
             * }
             *
             * out[i] = acc1;
             * }
             */

            /* Core Loop of FIR - AVX2 SIMD Implementation */
            /* Loop Unrolled Twice */
            /* Consume remaining input samples */
            for (i = (int)(samplesOffset + coef_looplen); i < samplesOffset + frame_length; i++)
            {

                __m256 load_in1;
                __m256 load_in2;
                __m256 load_coeffsrev1;
                __m256 load_coeffsrev2;
                __m256 sum;
                __m256 permuted_sum;
                __m256 final_vector_sum;
                /* Initializing the accumulators to Zero */
                __m256 acc1 = _mm256_setzero_ps();
                __m256 acc2 = _mm256_setzero_ps();
                float scalarSum = 0;
                size_t in_index = i - coef_looplen;                                         /* Inputs offset*/
                size_t coeffs_index = 0;                                                 /* Coefficient offset*/
                size_t loopcount1 = coeffs_length >> 4;                                  /* Divide by 16 as loop unrolling twice is done */
                size_t remainder1 = coeffs_length & 15;                                  /* coeffs_length%16 as loop unrolling twice is done */

                for (j = 0; j < loopcount1; j++)
                {
                    load_in1 = _mm256_loadu_ps(in + in_index);
                    load_in2 = _mm256_loadu_ps(in + in_index + 8);
                    load_coeffsrev1 = _mm256_loadu_ps(coeffs_rev + coeffs_index);
                    load_coeffsrev2 = _mm256_loadu_ps(coeffs_rev + coeffs_index + 8);
                    in_index += 16;
                    coeffs_index += 16;
                    acc1 = _mm256_fmadd_ps(load_in1, load_coeffsrev1, acc1);          /* acc1=acc1+(load_in1*load_coeffsrev1) */
                    acc2 = _mm256_fmadd_ps(load_in2, load_coeffsrev2, acc2);          /* acc1=acc1+(load_in2*load_coeffsrev2) */
                }
                /* Scalar operations for data not divisible by 8 */
                for (j = 0; j < remainder1; j++)
                {
                    scalarSum += in[in_index++] * coeffs_rev[coeffs_index++];

                }
                sum = _mm256_dp_ps(_mm256_add_ps(acc1, acc2), setone, 0xf1);                                   /* Finding the sum of lower 4 and upper 4 elements */
                permuted_sum = _mm256_permutevar8x32_ps(sum, _mm256_set_epi32(0, 1, 2, 3, 5, 6, 4, 0));     /* Relocate the element at index 4 to index 1 */
                final_vector_sum = _mm256_hadd_ps(permuted_sum, permuted_sum);                              /* Horizontally add the elements */
                out[i] = scalarSum + _mm_cvtss_f32(_mm256_extractf128_ps(final_vector_sum, 0));                    /* Extract lower 32 bits and perform scalar addition */
            }
            /* End Of Core Loop of FIR - AVX2 SIMD Implementation */

            /* Update the Delay Line for the Next Frame */
            /*    for (i = 0; i < (coeffs_length - 1); i++) {
             * states[coeffs_length - 2 - i] = in[i + frame_length  - (coeffs_length - 1)];
             * }
             */

            if(frame_length>=coef_looplen)
            {
                for (j = 0; j < coef_looplen; j++)
                    states[statesOffset+j] = in[samplesOffset+frame_length - 1 - j];
            }
            else
            {
                size_t loopvalue = coeffs_length - 2 - frame_length;
                for (i = (int)(statesOffset + loopvalue); i >= statesOffset; i--)
                {
                states[frame_length + i] = states[i];
                }

                for (i = 0; i < frame_length; i++)
                {
                    states[frame_length - 1+ (statesOffset - i)] = in[samplesOffset + i];
                }
            }


            /* End of DiscreteFir: '<Root>/Discrete FIR Filter' */

        }
}

/**
 * Copies the initial states into the states array and does the Reversal of Coefficients
 * @param coeffs : Coefficients vector
 * @param coeffs_rev : Reversed Coefficients vector
 * @param coeffs_length : Filter Order
 * @param initiaStates : Initial States vector
 * @param states : States vector
 * @param stateSize : States Size
 * Output
 * @return void
 **/

void MWDSPCRL_avx2_fir_sgl_real_init_mask(float* coeffs,float* coeffs_rev,size_t coeffs_length,float* initialStates, float* states, size_t stateSize,size_t initStateSize,size_t num_chan)
{
    ReversalOfCoefficients(coeffs,coeffs_rev,coeffs_length);
       
    /* Copying the InitialStates into a states buffer */
    if((initStateSize!=1) && (stateSize) && (initStateSize!=stateSize))
    {
        size_t i,tmp;
        for (i = 0; i < num_chan; i++) {
            tmp=i * initStateSize;
            memcpy(&states[tmp],&initialStates[0],initStateSize*sizeof(float));
        }
    }
    else if(initStateSize==stateSize)
    {
        memcpy(&states[0],&initialStates[0],stateSize*sizeof(float));
    }
    else
    {
        int i;
        float stateVal = *initialStates;
        for(i = 0; i < stateSize; i++)
            states[i] = stateVal;
    }
}

/**
 * Copies the initial states into the states array
 * @param initiaStates : Initial States vector
 * @param states : States vector
 * @param stateSize : States Size
 * Output
 * @return void
 **/

void MWDSPCRL_avx2_fir_sgl_real_init_port(float* initialStates, float* states, size_t stateSize,size_t initStateSize,size_t num_chan)
{
    /* Copying the InitialStates into a states buffer */
    if((initStateSize!=1) && (stateSize) && (initStateSize!=stateSize))
    {
        size_t i,tmp;
        for (i = 0; i < num_chan; i++) {
            tmp=i * initStateSize;
            memcpy(&states[tmp],&initialStates[0],initStateSize*sizeof(float));
        }
    }
    else if(initStateSize==stateSize)
    {
        memcpy(&states[0],&initialStates[0],stateSize*sizeof(float));
    }
    else
    {
        int i;
        float stateVal = *initialStates;
        for(i = 0; i < stateSize; i++)
            states[i] = stateVal;
    }
}

/**
 *AVX2 implementation of Discrete FIR when Coefficient Source is Dialog Parameters
 *
 **/

void MWDSPCRL_avx2_fir_sgl_real_mask(float* coeffs,float* coeffs_rev,size_t coeffs_length , const float* in, size_t frame_length , size_t num_chan, float* states, float* out)
{
    DiscreteFIR(coeffs,coeffs_rev,coeffs_length , in, frame_length , num_chan, states,out);
}

/**
 *AVX2 implementation of Discrete FIR when Coefficient Source is Input Port
 *
 **/

void MWDSPCRL_avx2_fir_sgl_real_port(const float* coeffs,float* coeffs_rev,size_t coeffs_length , const float* in, size_t frame_length , size_t num_chan, float* states, float* out)
{
    ReversalOfCoefficients(coeffs,coeffs_rev,coeffs_length);
    DiscreteFIR(coeffs,coeffs_rev,coeffs_length , in, frame_length , num_chan, states,out);
}
