/* Copyright 2020 The MathWorks, Inc. */

#include "fir_sgl_cmplx_avx2_blk.h"
#include <immintrin.h>
#define VECTOR_UNITS_FOR_256 (256/32)
#define UF 2*VECTOR_UNITS_FOR_256
/**
 * Reversal of Coefficients using AVX2 intrinsics
 * Called once in MWDSPCRL_avx2_discrete_fir_init_mask
 * Called iteratively in MWDSPCRL_avx2_discrete_fir_port
 * @param coeffs : Coefficients vector
 * @param reversedCoeffs : Reversed Coefficients vector
 * @param coeffsLength : Filter Order
 * @param transCoeffs : Coefficients vector to store duplicated coefficients
 * @param reversedtransCoeffs : Coefficients vector to store duplicated reversed coefficients
 * Output
 * @return void
 **/

static void ReversalOfCoefficients(const float* coeffs, float* reversedCoeffs, size_t coeffsLength, float* transCoeffs,float* reversedtransCoeffs)
{
    /* Reversal of the Coefficients Array - AVX2 SIMD Implementation */
    /* Reversed Coefficients to be used for SIMD Implementation */
    __m256 loadCoefficients;
    size_t startIndex = 0;
    size_t endIndex = coeffsLength - 1;
    size_t coefCount = coeffsLength >> 3;                                         /* Vector Loop run count for Reversal of coefficients*/
    size_t scalarCount = coeffsLength & 7;                                        /* Scalar Loop run count for Reversal of coefficients*/
    size_t idx,m=0;

    for (idx = 0; idx < coefCount; idx++)
    {
        loadCoefficients = _mm256_loadu_ps(coeffs + startIndex);
        loadCoefficients = _mm256_permutevar8x32_ps(loadCoefficients, _mm256_set_epi32(0, 1, 2, 3, 4, 5, 6, 7)); /* Rearrange the coefficients in reverse order */
        startIndex += VECTOR_UNITS_FOR_256;
        _mm256_storeu_ps((reversedCoeffs + endIndex - 7), loadCoefficients);
        endIndex -= VECTOR_UNITS_FOR_256;
    }

    for (idx = 0; idx < scalarCount; idx++)
    {
        reversedCoeffs[endIndex--] = coeffs[startIndex++];
    }

    /*Duplicating the coefficents for computing MAC with the interleaved input data*/
    for(idx=0; idx < coeffsLength * 2;idx +=2)
    {
        transCoeffs[idx] = coeffs[m];
        transCoeffs[idx+1] = coeffs[m];
        reversedtransCoeffs[idx] = reversedCoeffs[m];
        reversedtransCoeffs[idx + 1] = reversedCoeffs[m];
        m++;
    }
}  /* End of Reversal of the Coefficients Array - AVX2 SIMD Implementation*/

/**
 * Reversal of complex Coefficients using AVX2 intrinsics
 * Called once in MWDSPCRL_avx2_discrete_fir_init_mask
 * Called iteratively in MWDSPCRL_avx2_discrete_fir_cmplx_port
 * @param coeffs : Coefficients vector
 * @param reversedtransCoeffs : Reversed Coefficients vector
 * @param coeffsLength : Filter Order
 * Output
 * @return void
 **/
static void ReversalOfComplexCoefficients(float* coeffs, float* reversedtransCoeffs, size_t coeffsLength)
{
    /* Reversal of the Coefficients Array - AVX2 SIMD Implementation */
    /* Reversed Coefficients to be used for SIMD Implementation */
    __m256 loadCoefficients;
    size_t startIndex = 0;
    size_t endIndex = coeffsLength - 1;
    size_t coefCount = coeffsLength >> 3;                                         /* Vector Loop run count for Reversal of coefficients*/
    size_t scalarCount = coeffsLength & 7;                                        /* Scalar Loop run count for Reversal of coefficients*/
    size_t idx;

    for (idx = 0; idx < coefCount; idx++)
    {
        loadCoefficients = _mm256_loadu_ps(coeffs + startIndex);
        loadCoefficients = _mm256_permutevar8x32_ps(loadCoefficients, _mm256_set_epi32(1,0,3,2,5,4,7,6)); /* Rearrange the coefficients in reverse order */
        startIndex += VECTOR_UNITS_FOR_256;
        _mm256_storeu_ps((reversedtransCoeffs + endIndex - 7), loadCoefficients);
        endIndex -= VECTOR_UNITS_FOR_256;
    }

    for (idx = 0; idx < scalarCount; idx+=2)
    {
        reversedtransCoeffs[endIndex--] = coeffs[startIndex+1];
        reversedtransCoeffs[endIndex--] = coeffs[startIndex];
        startIndex += 2;
    }

}  /* End of Reversal of the Coefficients Array - AVX2 SIMD Implementation*/

/**
 * AVX2 implementation of Discrete FIR
 * Calculates the output data based on real filter coefficients and complex input data
 * @param S : structure sDiscreteFIR
 * @param transCoeffs : Coefficients vector
 * @param reversedtransCoeffs : Reversed coefficients vector
 * @param in : Input vector
 * Output
 * @param out : Output vector
 * @return void
 **/

static void DiscreteFIR_realCoeffs(sDiscreteFIRSglCmplx *S,float* transCoeffs, float* reversedtransCoeffs, const float* in, float* out)
{
    size_t coeffsLength = S->coeffs_length;
    float* states = S->states;
    int ch;
    size_t frameLength = S->frame_length;
    size_t num_chan = S->num_chan;
    size_t stateLength = coeffsLength - 2;
    size_t numIterations = stateLength;

    if (frameLength < stateLength)
        numIterations = frameLength;

    for (ch = 0; ch < num_chan; ch++)
    {
        size_t j;
        int statesOffset = (int)(ch * stateLength);
        int samplesOffset = (int)(ch * frameLength);
        int i;
        int numIter;
        size_t ScalarLoopCount;
        size_t VectLoopCount;
        for (i = 0; i < numIterations; i += 2)
        {
            __m256 loadIn;
            __m256 loadCoeffs;
            __m256 loadState;
            __m256 accumulator = _mm256_setzero_ps();
            __m128 high128;
            __m128 low128;
            __m128 summation;
            float scalarSum_Im = 0.0;
            float scalarSum_Re = 0.0;
            size_t incr = (i >> 1) + 1;
            size_t vectorLoopCount = ((incr) >> 3)<<1;
            size_t scalarLoopCount = (incr) & 7;
            size_t coefIdx = stateLength - i;                                    /* Coefficient offset */
            size_t inIdx = samplesOffset;                                        /* Input offset */
            size_t stateIdx;
            size_t oIdx = (i + samplesOffset);
            for (j = 0; j < vectorLoopCount; j++) {
                loadIn = _mm256_loadu_ps(&in[inIdx]);
                loadCoeffs = _mm256_loadu_ps(&reversedtransCoeffs[coefIdx]);
                accumulator = _mm256_fmadd_ps(loadIn, loadCoeffs, accumulator);
                inIdx += VECTOR_UNITS_FOR_256;
                coefIdx += VECTOR_UNITS_FOR_256;
            }
            for (j = 0; j < scalarLoopCount; j++)
            {
                scalarSum_Re += in[inIdx++] * reversedtransCoeffs[coefIdx++];
                scalarSum_Im += in[inIdx++] * reversedtransCoeffs[coefIdx++];
            }
            incr = (stateLength - i) >> 1;
            vectorLoopCount = (incr >> 3)<<1;
            scalarLoopCount = incr & 7;
            stateIdx = statesOffset;                                            /* States offset */
            coefIdx = i + 2;                                                    /* Coefficient offset */
            for (j = 0; j < vectorLoopCount; j++)
            {
                loadState = _mm256_loadu_ps(&states[stateIdx]);
                loadCoeffs = _mm256_loadu_ps(&transCoeffs[coefIdx]);
                accumulator = _mm256_fmadd_ps(loadCoeffs, loadState, accumulator);
                coefIdx += VECTOR_UNITS_FOR_256;
                stateIdx += VECTOR_UNITS_FOR_256;
            }
            for (j = 0; j < scalarLoopCount; j++)
            {
                scalarSum_Re += transCoeffs[coefIdx++] * states[stateIdx++];
                scalarSum_Im += transCoeffs[coefIdx++] * states[stateIdx++];
            }

            high128 = _mm256_extractf128_ps(accumulator, 1);
            low128 = _mm256_extractf128_ps(accumulator, 0);
            summation = _mm_add_ps(high128, low128);
            summation=_mm_permute_ps(summation, _MM_SHUFFLE(3,1,2,0));
            summation=_mm_hadd_ps(summation,summation);
            out[oIdx] = _mm_cvtss_f32(summation) + scalarSum_Re;
            summation = _mm_permute_ps(summation,_MM_SHUFFLE(0,1,2,3));
            out[++oIdx] = _mm_cvtss_f32(summation)+scalarSum_Im;

        }
        /* Discrete FIR - AVX2 SIMD Implementation for calculating output when frameLength > stateLength*/
        numIter = (int)(samplesOffset+frameLength);
        ScalarLoopCount = coeffsLength & 15;
        VectLoopCount = coeffsLength >> 4;
        for (i = (int)(samplesOffset + stateLength); i < numIter; i += 2) {
            __m256 accumulator = _mm256_setzero_ps();
            __m256 accumulator1 = _mm256_setzero_ps();
            __m256 loadIn;
            __m256 loadIn1;
            __m256 loadCoeffs;
            __m256 loadCoeffs1;
            __m128 high128;
            __m128 low128;
            __m128 summation;
            float scalarSum_Re = 0.0;
            float scalarSum_Im = 0.0;
            size_t inIndex = i - stateLength;                                /*Input offset*/
            size_t coefIndex = 0;                                            /*Coefficients offset*/

            for (j = 0; j < VectLoopCount; j++)
            {
                loadIn = _mm256_loadu_ps(&in[inIndex]);
                loadIn1 = _mm256_loadu_ps(&in[inIndex + VECTOR_UNITS_FOR_256]);
                loadCoeffs = _mm256_loadu_ps(&reversedtransCoeffs[coefIndex]);
                loadCoeffs1 = _mm256_loadu_ps(&reversedtransCoeffs[coefIndex + VECTOR_UNITS_FOR_256]);
                inIndex += UF;
                coefIndex += UF;
                accumulator = _mm256_fmadd_ps(loadIn, loadCoeffs, accumulator);
                accumulator1 = _mm256_fmadd_ps(loadIn1, loadCoeffs1, accumulator1);

            }
            for (j = 0; j < ScalarLoopCount; j+=2)
            {
                scalarSum_Re += in[inIndex++] * reversedtransCoeffs[coefIndex++];
                scalarSum_Im += in[inIndex++] * reversedtransCoeffs[coefIndex++];
            }

            accumulator = _mm256_add_ps(accumulator, accumulator1);
            high128 = _mm256_extractf128_ps(accumulator, 1);
            low128 = _mm256_extractf128_ps(accumulator, 0);
            summation = _mm_add_ps(high128, low128);
            summation=_mm_permute_ps(summation, _MM_SHUFFLE(3,1,2,0));
            summation=_mm_hadd_ps(summation,summation);
            out[i] = _mm_cvtss_f32(summation) + scalarSum_Re;
            summation = _mm_permute_ps(summation,_MM_SHUFFLE(0,1,2,3));
            out[i+1] = _mm_cvtss_f32(summation)+scalarSum_Im;

        }

        /* Update delay line for next frame */
        if (frameLength < stateLength)
        {
            size_t loopvalue = coeffsLength - 4 - frameLength;
            size_t x;
            size_t y;

            for (i = (int)(statesOffset + loopvalue); i >= statesOffset; i-=2)
            {
                x = (frameLength + i) ;
                y = i;
                states[x] = states[y];
                states[++x] = states[++y];
            }

            for (i = 0; i < frameLength; i+=2)
            {
                x = (frameLength - 2 + (statesOffset - i)) ;
                y = (samplesOffset + i);
                states[x] = in[y];
                states[++x] = in[++y];
            }
        }
        else
        {

            for (j = 0; j < stateLength; j+=2) {

                size_t x = (j + statesOffset) ;
                size_t y = (frameLength - 2 - j + samplesOffset);
                states[x] = in[y];
                states[++x] = in[++y];

            }
        }
    }
}

/**
 * AVX2 implementation of Discrete FIR
 * Calculates the output data based on complex filter coefficients and complex input data
 * @param S : structure sDiscreteFIR
 * @param coeffs : Coefficients vector
 * @param reversedCoeffs : Reversed coefficients vector
 * @param in : Input vector
 * Output
 * @param out : Output vector
 * @return void
 **/
static void DiscreteFIR_cmplxCoeffs(sDiscreteFIRSglCmplx *S,float* coeffs, float* reversedCoeffs, const float* in, float* out)
{
    float* states = S->states;
    int ch;
    size_t coeffsLength = S->coeffs_length;
    size_t frameLength = S->frame_length;
    size_t num_chan = S->num_chan;
    size_t stateLength = coeffsLength - 2;
    size_t numIterations = stateLength;

    if (frameLength < stateLength)
        numIterations = frameLength;

    for (ch = 0; ch < num_chan; ch++)
    {
        __m256 negFactor = _mm256_setr_ps(1,-1,1,-1,1,-1,1,-1);
        size_t j;
        int statesOffset = (int)(ch * stateLength);
        int samplesOffset = (int)(ch * frameLength);
        int i;
        int numIter = (int)(samplesOffset + frameLength);
        size_t ScalarLoopCount = coeffsLength & 15;
        size_t VectLoopCount = coeffsLength >> 4;        

        for (i = 0; i < numIterations; i += 2)
        {
            __m256 loadIn;
            __m256 loadCoeffs;
            __m256 loadState;
            __m256 accumulator = _mm256_setzero_ps();
            __m256 result = _mm256_setzero_ps();
            __m256 realMul;
            __m256 imagMul;
            __m128 high128;
            __m128 low128;
            __m128 summation;
            float scalarSum_Im = 0.0;
            float scalarSum_Re = 0.0;
            size_t incr = (i >> 1) + 1;
            size_t vectorLoopCount = ((incr) >> 3) << 1;
            size_t scalarLoopCount = incr & 7;
            size_t coefIdx = stateLength - i;                                /* Coefficient offset */
            size_t inIdx = samplesOffset;                                    /* Input offset */
            size_t stateIdx;
            size_t oIdx = i + samplesOffset;
            for (j = 0; j < vectorLoopCount; j++) {
                loadIn = _mm256_loadu_ps(&in[inIdx]);
                loadCoeffs = _mm256_loadu_ps(&reversedCoeffs[coefIdx]);
                inIdx += VECTOR_UNITS_FOR_256;
                coefIdx += VECTOR_UNITS_FOR_256;
                realMul = _mm256_mul_ps(loadIn,loadCoeffs);
                loadCoeffs = _mm256_permute_ps(loadCoeffs,0xb1);
                loadCoeffs = _mm256_mul_ps(loadCoeffs,negFactor);
                imagMul = _mm256_mul_ps(loadIn,loadCoeffs);
                result = _mm256_hsub_ps(realMul,imagMul);
                accumulator = _mm256_add_ps(accumulator,result);
            }
            for (j = 0; j < scalarLoopCount; j++)
            {
                scalarSum_Re += in[inIdx] * reversedCoeffs[coefIdx] - (in[inIdx+1] * reversedCoeffs[coefIdx+1]);
                scalarSum_Im += in[inIdx] * reversedCoeffs[coefIdx + 1] + (in[inIdx+1] * reversedCoeffs[coefIdx]);
                coefIdx += 2;
                inIdx += 2;
            }
            incr = (stateLength - i) >> 1;
            vectorLoopCount = (incr >> 3) << 1;
            scalarLoopCount = incr & 7;
            stateIdx = statesOffset;                                            /* States offset */
            coefIdx = i + 2;                                                    /* Coefficient offset */
            for (j = 0; j < vectorLoopCount; j++)
            {
                loadState = _mm256_loadu_ps(&states[stateIdx]);
                loadCoeffs = _mm256_loadu_ps(&coeffs[coefIdx]);
                coefIdx += VECTOR_UNITS_FOR_256;
                stateIdx += VECTOR_UNITS_FOR_256;
                realMul = _mm256_mul_ps(loadState,loadCoeffs);
                loadCoeffs = _mm256_permute_ps(loadCoeffs, 0xb1);
                loadCoeffs = _mm256_mul_ps(loadCoeffs, negFactor);
                imagMul = _mm256_mul_ps(loadState, loadCoeffs);
                result = _mm256_hsub_ps(realMul, imagMul);
                accumulator = _mm256_add_ps(accumulator,result);
            }
            for (j = 0; j < scalarLoopCount; j++)
            {
                scalarSum_Re += coeffs[coefIdx] * states[stateIdx] - (coeffs[coefIdx + 1] * states[stateIdx + 1]);
                scalarSum_Im += coeffs[coefIdx] * states[stateIdx + 1] + (coeffs[coefIdx + 1] * states[stateIdx]);
                coefIdx += 2;
                stateIdx += 2;
            }

            high128 = _mm256_extractf128_ps(accumulator, 1);
            low128 = _mm256_extractf128_ps(accumulator, 0);
            summation = _mm_add_ps(high128, low128);
            summation=_mm_hadd_ps(summation,summation);
            out[oIdx] = _mm_cvtss_f32(summation) + scalarSum_Re;
            summation = _mm_permute_ps(summation,_MM_SHUFFLE(0,1,2,3));
            out[++oIdx] = _mm_cvtss_f32(summation)+scalarSum_Im;
        }
        /* Discrete FIR - AVX2 SIMD Implementation for calculating output when frameLength > stateLength*/
        for (i = (int)(samplesOffset + stateLength); i < numIter; i += 2) {
            __m256 accumulator = _mm256_setzero_ps();
            __m256 accumulator_1 = _mm256_setzero_ps();
            __m256 result;
            __m256 result_1;
            __m256 loadIn;
            __m256 loadIn_1;
            __m256 loadCoeffs;
            __m256 loadCoeffs_1;
            __m256 realMul;
            __m256 imagMul;
            __m128 high128;
            __m128 low128;
            __m128 summation;
            float scalarSum_Re = 0.0;
            float scalarSum_Im = 0.0;
            size_t inIndex = i - stateLength;                                /*Input offset*/
            size_t coefIndex = 0;                                            /*Coefficients offset*/

            for (j = 0; j < VectLoopCount; j++)
            {
                loadIn = _mm256_loadu_ps(&in[inIndex]);
                loadCoeffs = _mm256_loadu_ps(&reversedCoeffs[coefIndex]);
                realMul = _mm256_mul_ps(loadIn, loadCoeffs);
                loadCoeffs = _mm256_permute_ps(loadCoeffs, 0xb1);
                loadCoeffs = _mm256_mul_ps(loadCoeffs, negFactor);
                imagMul = _mm256_mul_ps(loadIn, loadCoeffs);
                result = _mm256_hsub_ps(realMul, imagMul);
                accumulator = _mm256_add_ps(accumulator, result);

                loadIn_1 = _mm256_loadu_ps(&in[inIndex + VECTOR_UNITS_FOR_256]);
                loadCoeffs_1 = _mm256_loadu_ps(&reversedCoeffs[coefIndex + VECTOR_UNITS_FOR_256]);
                inIndex += UF;
                coefIndex += UF;
                realMul = _mm256_mul_ps(loadIn_1, loadCoeffs_1);
                loadCoeffs_1 = _mm256_permute_ps(loadCoeffs_1, 0xb1);
                loadCoeffs_1 = _mm256_mul_ps(loadCoeffs_1, negFactor);
                imagMul = _mm256_mul_ps(loadIn_1, loadCoeffs_1);
                result_1 = _mm256_hsub_ps(realMul, imagMul);
                accumulator_1 = _mm256_add_ps(accumulator_1, result_1);

            }
            for (j = 0; j < ScalarLoopCount; j += 2)
            {
                scalarSum_Re += in[inIndex] * reversedCoeffs[coefIndex] - (in[inIndex + 1] * reversedCoeffs[coefIndex + 1]);
                scalarSum_Im += in[inIndex] * reversedCoeffs[coefIndex + 1] + (in[inIndex + 1] * reversedCoeffs[coefIndex]);
                coefIndex += 2;
                inIndex += 2;
            }
            accumulator = _mm256_add_ps(accumulator, accumulator_1);
            high128 = _mm256_extractf128_ps(accumulator, 1);
            low128 = _mm256_extractf128_ps(accumulator, 0);
            summation = _mm_add_ps(high128, low128);
            summation=_mm_hadd_ps(summation,summation);
            out[i] = _mm_cvtss_f32(summation) + scalarSum_Re;
            summation = _mm_permute_ps(summation,_MM_SHUFFLE(0,1,2,3));
            out[i+1] = _mm_cvtss_f32(summation)+scalarSum_Im;
        }

        /* Update delay line for next frame */
        if (frameLength < stateLength)
        {
            size_t loopvalue = coeffsLength - 4 - frameLength;
            size_t x;
            size_t y;

            for (i = (int)(statesOffset + loopvalue); i >= statesOffset; i -= 2)
            {
                x = (frameLength + i);
                y = i;
                states[x] = states[y];
                states[++x] = states[++y];
            }

            for (i = 0; i < frameLength; i += 2)
            {
                x = (frameLength - 2 + (statesOffset - i));
                y = (samplesOffset + i);
                states[x] = in[y];
                states[++x] = in[++y];
            }
        }
        else
        {
            for (j = 0; j < stateLength; j += 2) {
                size_t x = (j + statesOffset);
                size_t y = (frameLength - 2 - j + samplesOffset);
                states[x] = in[y];
                states[++x] = in[++y];
            }
        }
    }
}

/**
 * Copies the initial states into the states array and does the Reversal of Coefficients
 * @param S : structure sDiscreteFIR
 * @param coeffs : Coefficients vector
 * @param reversedCoeffs : Reversed Coefficients vector
 * @param isRealCoeffs : Flag for coefficients
 * @param coeffs_length : Filter Order
 * @param initialStates : Initial States vector
 * @param states : States vector
 * @param stateSize : States Size
 * @param initStateSize : Initial StateSize
 * @param num_chan : Number of channels
 * @param frame_length : FrameSize
 * @param transCoeffs : Coefficients vector to store duplicated coefficients
 * @param reversedtransCoeffs : Coefficients vector to store duplicated reversed coefficients
 * Output
 * @return void
 **/
void MWDSPCRL_avx2_fir_sgl_cmplx_init_mask(sDiscreteFIRSglCmplx *S,float* coeffs,float* reversedCoeffs,size_t isRealCoeffs,size_t coeffs_length,float* initialStates,float* states,size_t stateSize,size_t initStateSize,size_t num_chan,size_t frame_length,float* transCoeffs,float* reversedtransCoeffs)
{
    if(isRealCoeffs)
    {
        ReversalOfCoefficients(coeffs,reversedCoeffs,coeffs_length,transCoeffs,reversedtransCoeffs);
    }
    else
    {
        ReversalOfComplexCoefficients(coeffs,reversedtransCoeffs,coeffs_length*2);
    }
    /* Copying the InitialStates into a states buffer */
    if((initStateSize!=2) && (stateSize) && (initStateSize!=stateSize)){
        size_t i,tmp;
        for (i = 0; i < num_chan; i++) {
            tmp = i * initStateSize;
            memcpy(&states[tmp],&initialStates[0],initStateSize*sizeof(float));
        }
    }
    else if(initStateSize==stateSize)
    {
        memcpy(&states[0],&initialStates[0],stateSize*sizeof(float));
    }
    else{
        int i;
        for(i = 0; i < stateSize; i+=2){
            states[i] = initialStates[0];
            states[i+1] = initialStates[1];
        }
    }
    S->frame_length=frame_length;
    S->num_chan=num_chan;
    S->coeffs_length=coeffs_length*2;
    S->states=states;
}

/**
 * Copies the initial states into the states array
 * @param S : structure sDiscreteFIR
 * @param coeffs_length : Coefficients length
 * @param initiaStates : Initial States vector
 * @param states : States vector
 * @param stateSize : States Size
 * @param initStateSize : Initial StateSize
 * @param num_chan : Number of channels
 * @param frame_length : FrameSize
 * Output
 * @return void
 **/

void MWDSPCRL_avx2_fir_sgl_cmplx_init_port(sDiscreteFIRSglCmplx *S,size_t coeffs_length,float* initialStates,float* states,size_t stateSize,size_t initStateSize,size_t num_chan,size_t frame_length)
{
    /* Copying the InitialStates into a states buffer */
    if((initStateSize !=2) && (stateSize) && (initStateSize!=stateSize)){
        size_t i,tmp;
        for (i = 0; i < num_chan; i++) {
            tmp = i * initStateSize;
            memcpy(&states[tmp],&initialStates[0],initStateSize*sizeof(float));
        }
    }
    else if(initStateSize==stateSize)
    {
        memcpy(&states[0],&initialStates[0],stateSize*sizeof(float));
    }
    else{
        int i;
        for(i = 0; i < stateSize; i +=2){
            states[i] = initialStates[0];
            states[i+1] = initialStates[1];
        }
    }

    S->frame_length=frame_length;
    S->num_chan=num_chan;
    S->coeffs_length=coeffs_length*2;
    S->states=states;
}

/**
 *AVX2 implementation of Discrete FIR when Coefficient Source is Dialog Parameters
 *
 **/

void MWDSPCRL_avx2_fir_sgl_cmplx_mask(sDiscreteFIRSglCmplx *S,float* transCoeffs,float* reversedtransCoeffs, const float* in,float* out,size_t isRealCoeffs,float* coeffs)
{
    if(isRealCoeffs){
        DiscreteFIR_realCoeffs(S,transCoeffs,reversedtransCoeffs,in,out);
    }
    else
    {
        DiscreteFIR_cmplxCoeffs(S,coeffs,reversedtransCoeffs,in,out);
    }

}

/**
 *AVX2 implementation of Discrete FIR when Coefficient Source is Input Port and Coefficients are REAL valued
 *
 **/

void MWDSPCRL_avx2_fir_sgl_cmplx_port(sDiscreteFIRSglCmplx *S,float* transCoeffs,float* reversedtransCoeffs , const float* in, float* out,const float* coeffs,float* reversedCoeffs)
{
    ReversalOfCoefficients(coeffs, reversedCoeffs, S->coeffs_length/2,transCoeffs,reversedtransCoeffs);
    DiscreteFIR_realCoeffs(S,transCoeffs, reversedtransCoeffs, in, out);
}

/**
 *AVX2 implementation of Discrete FIR when Coefficient Source is Input Port and Coefficients are COMPLEX valued
 *
 **/
void MWDSPCRL_avx2_fir_sgl_cmplx_port_coefficients(sDiscreteFIRSglCmplx *S,float* reversedtransCoeffs, const float* in, float* out, float* coeffs)
{
    ReversalOfComplexCoefficients(coeffs, reversedtransCoeffs, S->coeffs_length);
    DiscreteFIR_cmplxCoeffs(S, coeffs, reversedtransCoeffs, in, out);
}
