/* Copyright 2019-2020 The MathWorks, Inc. */

#include "interp_dbl_avx2_blk.h"
#include <immintrin.h>
#define VECTOR_UNITS_FOR_256 (256/64)
#define UF (2*VECTOR_UNITS_FOR_256)

/*-----------------Error codes-------------------*/
#define ERR_NULL_POINTER  (-1)
#define NO_ERR (0)

#define NULL_PTR_CHECK(ptr, str) \
    if(ptr == NULL) {                 \
        printf("NULL pointer received in %s()\n", str); \
        return (ERR_NULL_POINTER); \
    }

/**
 * Reversal of Coefficients using AVX2 intrinsics
 * Called once in MWDSPCRL_avx2_fir_interpolator_init_mask
 * Called iteratively in MWDSPCRL_avx2_fir_interpolator_port
 * @param coeffs : Coefficients vector
 * @param reversedCoeffs : Reversed Coefficients vector
 * @param coeffs_length : Filter Order
 * Output
 * @return void
 **/
static void reversalOfCoefficients(const double *coeffs, double *reversedCoeffs, size_t coef_length)
{
    /* Reversal of the Coefficients Array - AVX2 SIMD Implementation */
    /* Reversed Coefficients to be used for SIMD Implementation */
    size_t j;
    size_t startIndex = 0;
    size_t endIndex = coef_length - 1;
    size_t coefCount = (coef_length) >> 2;                                      /* Vector Loop run count for Reversal of coefficients*/
    size_t scalarCount = (coef_length) & 3;                                     /* Scalar Loop run count for Reversal of coefficients*/

    for (j = 0; j < coefCount; j++)
    {
        __m256d loadCoefficients = _mm256_loadu_pd(&coeffs[startIndex]);
        loadCoefficients = _mm256_permute4x64_pd(loadCoefficients, _MM_SHUFFLE(0, 1, 2, 3));  /* Rearrange the coefficients in reverse order */
        startIndex += VECTOR_UNITS_FOR_256;
        _mm256_storeu_pd((reversedCoeffs + endIndex - 3), loadCoefficients);
        endIndex -= VECTOR_UNITS_FOR_256;
    }

    for (j = 0; j < scalarCount; j++)
    {
        reversedCoeffs[endIndex--] = coeffs[startIndex++];
    }

}

/**
 * AVX2 implementation of FIR Interpolator
 * Calculates the output data by interpolating the values between the input data using polyphase implementation
 * @param coeffs : Coefficients vector
 * @param coeffs_length : Filter order
 * @param iFactor : Interpolation factor
 * @param in : Input vector
 * @param frameSize : Frame length
 * @param states : States vector
 * @param nChans : Number of Channels
 * Output
 * @param out : Output vector
 * @return void
 **/
static int firInterpolator(InterpDblReal *S, const double *coeffs, double *reversedCoeffs, const double *in, double *out)
{
    size_t j,mPolyphase, kChans;  /*Loop Iterators*/
    size_t iFactor = S->iFactor;
    size_t frameSize = S->frameSize;
    size_t nChans = S->nChans;
    size_t coef_length=S->coef_length;
    double* states = S->states;
    size_t polyphaseFiltLen = coef_length / iFactor;
    size_t nStates = polyphaseFiltLen - 1;
    size_t numIterations=nStates;
    int i;
    size_t oIdx;

    NULL_PTR_CHECK(coeffs, "firInterpolator");
    NULL_PTR_CHECK(S, "firInterpolator");
    if (nStates > 0) {
        NULL_PTR_CHECK(states, "firInterpolator");
    }

    if (frameSize < nStates)
        numIterations = frameSize;

    for (kChans = 0; kChans < nChans; kChans++) /* Loop for multichannel inputs */
    {
        int delayBuffIdx = (int)(kChans * nStates);
        size_t dataInArrayIdx = kChans * frameSize;
        size_t dataOutArrayIdx = dataInArrayIdx * iFactor;
        for (mPolyphase = 0; mPolyphase < iFactor; mPolyphase++) /* Polyphase Implementation Loop */
        {
            size_t coefArrayIdx = mPolyphase * polyphaseFiltLen;
            size_t nStateVectLoopCount = polyphaseFiltLen >> 3;
            size_t nStateScalarLoopCount = polyphaseFiltLen & 7;
            int numIteration =(int)( frameSize - nStates);
            oIdx = dataOutArrayIdx + mPolyphase;

            /* Reference C Loop
             * for (i = 0; i < numIterations; i++) {
             *     acc1 = 0.0;
             *     for (j = 0; j < i + 1; j++) {
             *         acc1 += in[dataInArrayIdx + i - j] * coeffs[coefArrayIdx + j];
             *     }
             *
             *     for (j = 0; j < nStates - i; j++) {
             *         acc1 += coeffs[coefArrayIdx + (i + j) + 1] *states[delayBuffIdx + j];
             *     }
             *
             *     out[oIdx] = acc1;
             *     oIdx += iFactor;
             * }
             *
             */

            for (i = 0; i < numIterations; i++) /* FIR Interpolator- AVX2 SIMD Implementation */
            {
                __m256d accumulator = _mm256_setzero_pd();
                __m256d loadIn;
                __m256d loadCoeffs;
                __m256d loadState;
                __m256d vectorAddition;
                double vectorSum = 0.0;
                double scalarSum = 0.0;
                size_t coefIndex = (coef_length - 1) - i - coefArrayIdx;
                size_t inIndex = dataInArrayIdx;
                size_t vectorLoopCount = (i+1) >> 2;
                size_t scalarLoopCount = (i+1) & 3;
                size_t statesIndex=delayBuffIdx;
                
                for (j = 0; j < vectorLoopCount; j++)
                {
                    loadIn = _mm256_loadu_pd(&in[inIndex]);                     /* Vector Load */
                    loadCoeffs = _mm256_loadu_pd(&reversedCoeffs[coefIndex]);
                    accumulator = _mm256_fmadd_pd(loadIn, loadCoeffs, accumulator);
                    inIndex += VECTOR_UNITS_FOR_256;
                    coefIndex += VECTOR_UNITS_FOR_256;
                }

                for (j = 0; j < scalarLoopCount; j++)
                    scalarSum += in[inIndex++] * reversedCoeffs[coefIndex++];

                coefIndex = coefArrayIdx + i + 1;
                vectorLoopCount = (nStates - i) >> 2;
                scalarLoopCount = (nStates - i) & 3;

                for (j = 0; j < vectorLoopCount; j++)
                {
                    loadCoeffs = _mm256_loadu_pd(&coeffs[coefIndex]);
                    loadState = _mm256_loadu_pd(&states[statesIndex]);
                    accumulator = _mm256_fmadd_pd(loadCoeffs, loadState, accumulator);
                    coefIndex += VECTOR_UNITS_FOR_256;
                    statesIndex += VECTOR_UNITS_FOR_256;
                }
                for (j = 0; j < scalarLoopCount; j++)
                {
                    scalarSum += coeffs[coefIndex++] * states[statesIndex++];
                }
                /* Computing sum of individual elements in a vector */
                vectorAddition = _mm256_hadd_pd(accumulator, accumulator);
                vectorAddition = _mm256_permute4x64_pd(vectorAddition, 0xd8);
                vectorAddition = _mm256_hadd_pd(vectorAddition, vectorAddition);
                vectorSum = _mm_cvtsd_f64(_mm256_castpd256_pd128(vectorAddition));

                out[oIdx] = vectorSum + scalarSum;
                oIdx += iFactor;
            }

            /* Reference C Loop for iterations when Framesize > nStates
             * for (i = 0; i < frameSize-nStates; i++) {
             *     acc1 = 0.0;
             *     for (j = 0; j < nStates+1; j++) {
             *        acc1 += in[dataInArrayIdx + i - j] * coeffs[coefArrayIdx + j];
             *     }
             *
             *     out[oIdx] = acc1;
             *     oIdx += iFactor;
             * }
             *
             */

            /* Loop Unrolled Twice */

            for (i = 0; i < numIteration; i++)    /* FIR Interpolator- AVX2 SIMD Implementation for calculating output when Framesize > nStates*/
            {
                __m256d loadIn;
                __m256d loadIn1;
                __m256d loadCoeffs;
                __m256d loadCoeffs1;
                __m256d vectorAddition;
                /*  Initializing the accumulators to Zero */
                __m256d accumulator = _mm256_setzero_pd();
                __m256d accumulator1 = _mm256_setzero_pd();
                double scalarSum = 0.0;
                double vectorSum = 0.0;
                size_t inIdx = i + dataInArrayIdx;
                size_t coefIdx = (coef_length - 1) - nStates - coefArrayIdx;

                for (j = 0; j < nStateVectLoopCount; j++)
                {

                    loadIn = _mm256_loadu_pd(&in[inIdx]);                           /* Vector load */
                    loadIn1 = _mm256_loadu_pd(&in[inIdx + VECTOR_UNITS_FOR_256]);
                    loadCoeffs = _mm256_loadu_pd(&reversedCoeffs[coefIdx]);
                    loadCoeffs1 = _mm256_loadu_pd(&reversedCoeffs[coefIdx + VECTOR_UNITS_FOR_256]);

                    inIdx += UF;                                                     /* Auto-increment by 8 since loop unrolling takes place twice */
                    coefIdx += UF;
                    accumulator = _mm256_fmadd_pd(loadIn, loadCoeffs, accumulator);
                    accumulator1 = _mm256_fmadd_pd(loadIn1, loadCoeffs1, accumulator1);

                }

                for (j = 0; j < nStateScalarLoopCount; j++)
                {
                    scalarSum += in[inIdx++] * reversedCoeffs[coefIdx++];
                }

                /* Summing up the MUL-Accumulation results */
                accumulator = _mm256_add_pd(accumulator, accumulator1);
                /* Computing sum of individual elements in a vector */
                vectorAddition = _mm256_hadd_pd(accumulator, accumulator);
                vectorAddition = _mm256_permute4x64_pd(vectorAddition, 0xd8);
                vectorAddition = _mm256_hadd_pd(vectorAddition, vectorAddition);
                vectorSum = _mm_cvtsd_f64(_mm256_castpd256_pd128(vectorAddition));
                out[oIdx] =  vectorSum + scalarSum;
                oIdx += iFactor;

            } /* FIR Interpolator- AVX2 SIMD Implementation for calculating output when Framesize > nStates*/

        } /* End of Polyphase Implementation Loop */

        /* Update delay line for next frame */
        if (frameSize >= nStates)
        {
            for (i = 0; i < nStates; i++)
            {
                states[delayBuffIdx + i] = in[frameSize - 1 - i + dataInArrayIdx];
            }
        }

        else
        {
            int d;
            for (d = (int)(delayBuffIdx + nStates - frameSize - 1); d >= delayBuffIdx; d--)
                states[frameSize + d] = states[d];
            for (j = 0; j < frameSize; j++)
                states[delayBuffIdx + frameSize - 1 - j] = in[dataInArrayIdx + j];
        }

    } /* End of Loop for multichannel inputs */
    return NO_ERR;
}

/**
 * Reversal of Coefficients for coefficients from dialog parameters and assignment of the structure elements is done in MWDSPCRL_avx2_fir_interpolator_init_mask
 * @param S : structure fir_Interpolator
 * @param coeffs : Coefficients vector
 * @param reversedCoeffs : Reversed Coefficients vector
 * @param coeffs_length : Filter Order
 * @param iFactor : Interpolation factor
 * @param frameSize : Frame length
 * @param nChans : Number of channels
 * @param states : States vector
 * Output
 * @return void
 **/

void MWDSPCRL_avx2_interp_dbl_real_init(const double *coeffs, double *reversedCoeffs, size_t coef_length, size_t iFactor, size_t nChans, double *states)
{
    size_t polyphaseFiltLen;
    polyphaseFiltLen = coef_length/iFactor;
    if(polyphaseFiltLen-2 > 0){
        size_t stateLength;
        stateLength=((coef_length/iFactor)-1)*nChans;
        memset(states,0,stateLength*sizeof(states[0]));
    }
    reversalOfCoefficients(coeffs,reversedCoeffs,coef_length);
}

/**
 *AVX2 implementation of Discrete FIR when Coefficient Source is Input Port
 *
 **/
void MWDSPCRL_avx2_interp_dbl_real_port(InterpDblReal *S, const double *initialCoeffs,double *reversedCoeffs, const double *in, double *out)
{
    size_t indx,m,j;  /*Loop iterators*/
    size_t iFactor = S->iFactor;
    size_t coef_length=S->coef_length;
    size_t polyphaseFiltLen=(coef_length+iFactor-1)/iFactor;
    size_t coeffLen=polyphaseFiltLen*iFactor;
    double *coeffs=NULL;
    coeffs=(double*)calloc(coeffLen, sizeof(double));
    for(m = 0; m < iFactor; m++)
    {
        indx = m*polyphaseFiltLen;
        for(j = m; j < coef_length;j+=iFactor)
        {
            coeffs[indx]=initialCoeffs[j];
            indx++;
        }
    }
    S->coef_length=coeffLen;
    reversalOfCoefficients(coeffs,reversedCoeffs,coeffLen);
    firInterpolator(S,coeffs,reversedCoeffs,in,out);
    S->coef_length=coef_length;
    free(coeffs);
}

/**
 *AVX2 implementation of Discrete FIR when Coefficient Source is Dialog Parameters
 *
 **/
void MWDSPCRL_avx2_interp_dbl_real_process( InterpDblReal *S, const double *coeffs, double *reversedCoeffs, const double *in, double *out)
{
    firInterpolator(S,coeffs,reversedCoeffs,in,out);
}
