/* Copyright 2020 The MathWorks, Inc. */

#include "interp_sgl_cmplx_avx2_blk.h"
#include <immintrin.h>
#define VECTOR_UNITS_FOR_256 (256/32)
#define UF (2*VECTOR_UNITS_FOR_256)
/**
 * Reversal of Coefficients using AVX2 intrinsics
 * Called once in MWDSPCRL_avx2_fir_interpolator_init_mask
 * Called iteratively in MWDSPCRL_avx2_fir_interpolator_port
 * @param coeffs : Coefficients vector
 * @param reversedCoeffs : Reversed Coefficients vector
 * @param coeffsLength : Filter Order
 * @param transCoeffs : Coefficients vector to store duplicated coefficients
 * @param reversedtransCoeffs : Coefficients vector to store duplicated reversed coefficients
 * Output
 * @return void
 **/

static void reversalOfCoefficients(const float *coeffs, float *reversedCoeffs, size_t coeffsLength, float* transCoeffs,float* reversedtransCoeffs)
{
    /* Reversal of the Coefficients Array - AVX2 SIMD Implementation */
    /* Reversed Coefficients to be used for SIMD Implementation */
    size_t j,m=0;
    size_t startIndex = 0;
    size_t endIndex = coeffsLength - 1;
    size_t coefCount = (coeffsLength) >> 3;                                      /* Vector Loop run count for Reversal of coefficients*/
    size_t scalarCount = (coeffsLength) & 7;                                     /* Scalar Loop run count for Reversal of coefficients*/
    size_t twiceCoeffLen = coeffsLength << 1;
    for (j = 0; j < coefCount; j++)
    {
        __m256 loadCoefficients = _mm256_loadu_ps(&coeffs[startIndex]);
        loadCoefficients = _mm256_permutevar8x32_ps(loadCoefficients, _mm256_set_epi32(0, 1, 2, 3, 4, 5, 6, 7)); /* Rearrange the coefficients in reverse order */
        startIndex += VECTOR_UNITS_FOR_256;
        _mm256_storeu_ps((reversedCoeffs + endIndex - 7), loadCoefficients);
        endIndex -= VECTOR_UNITS_FOR_256;
    }

    for (j = 0; j < scalarCount; j++)
    {
        reversedCoeffs[endIndex--] = coeffs[startIndex++];
    }
    /*Duplicating the coefficents for computing MAC with the interleaved input data*/
    for (j = 0; j < twiceCoeffLen; j += 2)
    {
        transCoeffs[j] = coeffs[m];
        transCoeffs[j + 1] = coeffs[m];
        reversedtransCoeffs[j] = reversedCoeffs[m];
        reversedtransCoeffs[j + 1] = reversedCoeffs[m];
        m++;
    }
}

/**
 * Reversal of complex Coefficients using AVX2 intrinsics
 * Called once in MWDSPCRL_avx2_fir_interpolator_init_mask
 * Called iteratively in MWDSPCRL_avx2_fir_interpolator_cmplx_port
 * @param coeffs : Coefficients vector
 * @param reversedtransCoeffs : Reversed Coefficients vector
 * @param coeffsLength : Filter Order
 * Output
 * @return void
 **/
static void reversalOfCmplxCoefficients(const float *coeffs, float *reversedCoeffs, size_t coeffsLength)
{
    /* Reversal of the Coefficients Array - AVX2 SIMD Implementation */
    /* Reversed Coefficients to be used for SIMD Implementation */
    size_t j;
    size_t startIndex = 0;
    size_t endIndex = coeffsLength - 1;
    size_t coefCount = (coeffsLength) >> 3;                                      /* Vector Loop run count for Reversal of coefficients*/
    size_t scalarCount = (coeffsLength) & 7;                                     /* Scalar Loop run count for Reversal of coefficients*/

    for (j = 0; j < coefCount; j++)
    {
        __m256 loadCoefficients = _mm256_loadu_ps(&coeffs[startIndex]);
        loadCoefficients = _mm256_permutevar8x32_ps(loadCoefficients, _mm256_set_epi32(1,0,3,2,5,4,7,6)); /* Rearrange the coefficients in reverse order */
        startIndex += VECTOR_UNITS_FOR_256;
        _mm256_storeu_ps((reversedCoeffs + endIndex - 7), loadCoefficients);
        endIndex -= VECTOR_UNITS_FOR_256;
    }

    for (j = 0; j < scalarCount; j+=2)
    {
        reversedCoeffs[endIndex--] = coeffs[startIndex+1];
        reversedCoeffs[endIndex--] = coeffs[startIndex];
        startIndex += 2;
    }
}  /* End of Reversal of the Coefficients Array - AVX2 SIMD Implementation*/

/**
 * AVX2 implementation of FIR Interpolator
 * Calculates the output data by interpolating the values between the complex input data using polyphase implementation with real coefficients
 * @param S : structure fir_Interpolator
 * @param transCoeffs : Coefficients vector to store duplicated coefficients
 * @param reversedtransCoeffs : Coefficients vector to store duplicated reversed coefficients
 * @param in : Input vector
 * Output
 * @param out : Output vector
 * @return void
 **/
static void firInterpolator_RealCoeffs(FIRInterpSglCmplx *S, float* states, float* transCoeffs,float* reversedtransCoeffs, const float* in, float* out)
{    
    int i;
    size_t iFactor = S->iFactor;
    size_t frameSize = S->frameSize;
    size_t nChans = S->nChans;
    size_t coeffsLength=(S->coeffsLength)<<1;
    size_t polyphaseFiltLen = coeffsLength / iFactor;
    size_t nStates = polyphaseFiltLen - 2;
    size_t numIterations = nStates;
    size_t j, mPolyphase, kChans;  /*Loop Iterators*/
    size_t oIdx;

    if (frameSize < nStates)
        numIterations = frameSize;

    for (kChans = 0; kChans < nChans; kChans++) /* Loop for multichannel inputs */
    {
        int delayBuffIdx;
        size_t dataInArrayIdx;
        size_t dataOutArrayIdx;
        delayBuffIdx = (int)(kChans * nStates);
        dataInArrayIdx = kChans * frameSize;
        dataOutArrayIdx = dataInArrayIdx * iFactor;
        for (mPolyphase = 0; mPolyphase < iFactor; mPolyphase++) /* Polyphase Implementation Loop */
        {
            int numIteration;
            size_t coefArrayIdx;
            size_t nStateVectLoopCount;
            size_t nStateScalarLoopCount;
            numIteration = (int)(frameSize - nStates);
            coefArrayIdx = mPolyphase * polyphaseFiltLen;
            nStateVectLoopCount = polyphaseFiltLen >> 4;
            nStateScalarLoopCount = polyphaseFiltLen & 15;
            oIdx = dataOutArrayIdx + (mPolyphase << 1);
            for (i = 0; i < numIterations; i += 2) /* FIR Interpolator- AVX2 SIMD Implementation */
            {
                __m256 accumulator = _mm256_setzero_ps();
                __m256 loadIn;
                __m256 loadCoeffs;
                __m256 loadState;
                __m128 high128;
                __m128 low128;
                __m128 summation;
                float scalarSum_Re = 0.0;
                float scalarSum_Im = 0.0;
                size_t coefIndex;
                size_t incr;
                size_t vectorLoopCount;
                size_t scalarLoopCount;
                size_t statesIndex = delayBuffIdx;
                size_t inIndex = dataInArrayIdx;
                coefIndex = (coeffsLength - 2) - i - coefArrayIdx;
                incr = (i >> 1) + 1;
                vectorLoopCount = (incr >> 3) << 1;;
                scalarLoopCount = incr & 7;
                

                for (j = 0; j < vectorLoopCount; j++)
                {
                    loadIn = _mm256_loadu_ps(&in[inIndex]);
                    loadCoeffs = _mm256_loadu_ps(&reversedtransCoeffs[coefIndex]);
                    accumulator = _mm256_fmadd_ps(loadIn, loadCoeffs, accumulator);
                    inIndex += VECTOR_UNITS_FOR_256;
                    coefIndex += VECTOR_UNITS_FOR_256;
                }

                for (j = 0; j < scalarLoopCount; j++)
                {
                    scalarSum_Re += in[inIndex++] * reversedtransCoeffs[coefIndex++];
                    scalarSum_Im += in[inIndex++] * reversedtransCoeffs[coefIndex++];
                }

                coefIndex = coefArrayIdx + i + 2;
                incr = (nStates - i) >> 1;
                vectorLoopCount = (incr >> 3) << 1;
                scalarLoopCount = incr & 7;

                for (j = 0; j < vectorLoopCount; j++)
                {
                    loadCoeffs = _mm256_loadu_ps(&transCoeffs[coefIndex]);
                    loadState = _mm256_loadu_ps(&states[statesIndex]);
                    accumulator = _mm256_fmadd_ps(loadCoeffs, loadState, accumulator);
                    coefIndex += VECTOR_UNITS_FOR_256;
                    statesIndex += VECTOR_UNITS_FOR_256;
                }
                for (j = 0; j < scalarLoopCount; j++)
                {
                    scalarSum_Re += transCoeffs[coefIndex++] * states[statesIndex++];
                    scalarSum_Im += transCoeffs[coefIndex++] * states[statesIndex++];
                }
                /* Computing sum of individual elements in a vector */
                high128 = _mm256_extractf128_ps(accumulator, 1);
                low128 = _mm256_extractf128_ps(accumulator, 0);
                summation = _mm_add_ps(high128, low128);
                summation=_mm_permute_ps(summation, _MM_SHUFFLE(3,1,2,0));
                summation=_mm_hadd_ps(summation,summation);
                out[oIdx] = _mm_cvtss_f32(summation) + scalarSum_Re;
                summation = _mm_permute_ps(summation,_MM_SHUFFLE(0,1,2,3));
                out[oIdx+1] = _mm_cvtss_f32(summation)+scalarSum_Im;
                oIdx += iFactor << 1;
            }

            /* Loop Unrolled Twice */
            for (i = 0; i < numIteration; i += 2)    /* FIR Interpolator- AVX2 SIMD Implementation for calculating output when Framesize > nStates*/
            {
                __m256 loadIn;
                __m256 loadIn1;
                __m256 loadCoeffs;
                __m256 loadCoeffs1;
                __m128 high128;
                __m128 low128;
                __m128 summation;
                /*  Initializing the accumulators to Zero */
                __m256 accumulator = _mm256_setzero_ps();
                __m256 accumulator1 = _mm256_setzero_ps();
                float scalarSum_Re = 0.0;
                float scalarSum_Im = 0.0;
                size_t inIdx;
                size_t coefIdx;
                inIdx = i + dataInArrayIdx;
                coefIdx = (coeffsLength - 2) - nStates - coefArrayIdx;

                for (j = 0; j < nStateVectLoopCount; j++)
                {
                    loadIn = _mm256_loadu_ps(&in[inIdx]);                           /* Vector load */
                    loadIn1 = _mm256_loadu_ps(&in[inIdx + VECTOR_UNITS_FOR_256]);
                    inIdx += UF;
                    loadCoeffs = _mm256_loadu_ps(&reversedtransCoeffs[coefIdx]);
                    loadCoeffs1 = _mm256_loadu_ps(&reversedtransCoeffs[coefIdx + VECTOR_UNITS_FOR_256]);
                    coefIdx += UF;
                    accumulator = _mm256_fmadd_ps(loadIn, loadCoeffs, accumulator);
                    accumulator1 = _mm256_fmadd_ps(loadIn1, loadCoeffs1, accumulator1);
                }

                for (j = 0; j < nStateScalarLoopCount; j+=2)
                {
                    scalarSum_Re += in[inIdx++] * reversedtransCoeffs[coefIdx++];
                    scalarSum_Im += in[inIdx++] * reversedtransCoeffs[coefIdx++];
                }
                /* Summing up the MUL-Accumulation results */
                accumulator = _mm256_add_ps(accumulator, accumulator1);
                high128 = _mm256_extractf128_ps(accumulator, 1);
                low128 = _mm256_extractf128_ps(accumulator, 0);
                summation = _mm_add_ps(high128, low128);
                summation=_mm_permute_ps(summation, _MM_SHUFFLE(3,1,2,0));
                summation=_mm_hadd_ps(summation,summation);
                out[oIdx] = _mm_cvtss_f32(summation) + scalarSum_Re;
                summation = _mm_permute_ps(summation,_MM_SHUFFLE(0,1,2,3));
                out[oIdx+1] = _mm_cvtss_f32(summation)+scalarSum_Im;
                oIdx += iFactor << 1;
            } /* FIR Interpolator- AVX2 SIMD Implementation for calculating output when Framesize > nStates*/
        } /* End of Polyphase Implementation Loop */

        /* Update delay line for next frame */
        if (frameSize >= nStates)
        {
            for (i = 0; i < nStates; i+=2)
            {
                size_t x = delayBuffIdx + i;
                size_t y = frameSize - 2 - i + dataInArrayIdx;
                states[x] = in[y];
                states[++x] = in[++y];
            }
        }
        else
        {
            size_t x;
            size_t y;
            for (i = (int)(delayBuffIdx + nStates - frameSize - 2); i >= delayBuffIdx; i -= 2) {
                x = frameSize + i;
                y = i;
                states[x] = states[y];
                states[++x] = states[++y];
            }
            for (j = 0; j < frameSize; j += 2) {
                x = delayBuffIdx + frameSize - 2 - j;
                y = dataInArrayIdx + j;
                states[x] = in[y];
                states[++x] = in[++y];
            }
        }
    } /* End of Loop for multichannel inputs */
}

/**
 * AVX2 implementation of FIR Interpolator
 * Calculates the output data by interpolating the values between the complex input data using polyphase implementation with complex coefficients
 * @param S : structure fir_Interpolator
 * @param coeffs : Coefficients vector
 * @param reversedcoeffs : Reversed coefficients vector
 * @param in : Input vector
 * Output
 * @param out : Output vector
 * @return void
 **/
static void firInterpolator_CmplxCoeffs(FIRInterpSglCmplx *S, float* states, const float *coeffs, float *reversedCoeffs, const float *in, float *out)
{    
    int i;
    size_t iFactor = S->iFactor;
    size_t frameSize = S->frameSize;
    size_t nChans = S->nChans;
    size_t coeffsLength=S->coeffsLength;
    size_t polyphaseFiltLen = coeffsLength/ iFactor;
    size_t nStates = polyphaseFiltLen - 2;
    size_t numIterations = nStates;
    size_t j, mPolyphase, kChans;  /*Loop Iterators*/
    size_t oIdx;

    if (frameSize < nStates)
        numIterations = frameSize;

    for (kChans = 0; kChans < nChans; kChans++) /* Loop for multichannel inputs */
    {
        __m256 negFactor = _mm256_setr_ps(1,-1,1,-1,1,-1,1,-1);
        int delayBuffIdx;
        size_t dataInArrayIdx;
        size_t dataOutArrayIdx;
        delayBuffIdx = (int)(kChans * nStates);
        dataInArrayIdx = kChans * frameSize;
        dataOutArrayIdx = dataInArrayIdx * iFactor;
        for (mPolyphase = 0; mPolyphase < iFactor; mPolyphase++) /* Polyphase Implementation Loop */
        {
            int numIteration;
            size_t coefArrayIdx;
            size_t nStateVectLoopCount;
            size_t nStateScalarLoopCount;
            numIteration = (int)(frameSize - nStates);
            coefArrayIdx = mPolyphase * polyphaseFiltLen;
            nStateVectLoopCount = polyphaseFiltLen >> 4;
            nStateScalarLoopCount = polyphaseFiltLen & 15;
            oIdx = dataOutArrayIdx + (mPolyphase*2);
            for (i = 0; i < numIterations; i += 2) /* FIR Interpolator- AVX2 SIMD Implementation */
            {
                __m256 accumulator = _mm256_setzero_ps();
                __m256 loadIn;
                __m256 loadCoeffs;
                __m256 loadState;
                __m256 realMul;
                __m256 imagMul;
                __m256 result;
                __m128 high128;
                __m128 low128;
                __m128 summation;
                float scalarSum_Re = 0.0;
                float scalarSum_Im = 0.0;
                size_t coefIndex;
                size_t incr;
                size_t vectorLoopCount;
                size_t scalarLoopCount;
                size_t inIndex = dataInArrayIdx;
                size_t statesIndex = delayBuffIdx;
                coefIndex = (coeffsLength - 2) - i - coefArrayIdx;
                incr = (i >> 1) + 1;
                vectorLoopCount = (incr >> 3) << 1;;
                scalarLoopCount = incr & 7;

                for (j = 0; j < vectorLoopCount; j++)
                {
                    loadIn = _mm256_loadu_ps(&in[inIndex]);                     /* Vector Load */
                    loadCoeffs = _mm256_loadu_ps(&reversedCoeffs[coefIndex]);
                    realMul = _mm256_mul_ps(loadIn,loadCoeffs);
                    loadCoeffs = _mm256_permute_ps(loadCoeffs,0xb1);
                    loadCoeffs = _mm256_mul_ps(loadCoeffs,negFactor);
                    imagMul = _mm256_mul_ps(loadIn,loadCoeffs);
                    result = _mm256_hsub_ps(realMul,imagMul);
                    accumulator = _mm256_add_ps(accumulator,result);
                    inIndex += VECTOR_UNITS_FOR_256;
                    coefIndex += VECTOR_UNITS_FOR_256;
                }

                for (j = 0; j < scalarLoopCount; j++)
                {
                    scalarSum_Re += in[inIndex] * reversedCoeffs[coefIndex]-(in[inIndex+1]* reversedCoeffs[coefIndex+1]);
                    scalarSum_Im += in[inIndex] * reversedCoeffs[coefIndex+1]+ (in[inIndex+1] * reversedCoeffs[coefIndex]);
                    coefIndex+=2;
                    inIndex+=2;
                }

                coefIndex = coefArrayIdx + i + 2;
                incr = (nStates - i) >> 1;
                vectorLoopCount = (incr >> 3) << 1;
                scalarLoopCount = incr & 7;

                for (j = 0; j < vectorLoopCount; j++)
                {
                    loadCoeffs = _mm256_loadu_ps(&coeffs[coefIndex]);
                    loadState = _mm256_loadu_ps(&states[statesIndex]);
                    realMul = _mm256_mul_ps(loadState,loadCoeffs);
                    loadCoeffs = _mm256_permute_ps(loadCoeffs, 0xb1);
                    loadCoeffs = _mm256_mul_ps(loadCoeffs, negFactor);
                    imagMul = _mm256_mul_ps(loadState, loadCoeffs);
                    result = _mm256_hsub_ps(realMul, imagMul);
                    accumulator = _mm256_add_ps(accumulator,result);
                    coefIndex += VECTOR_UNITS_FOR_256;
                    statesIndex += VECTOR_UNITS_FOR_256;
                }
                for (j = 0; j < scalarLoopCount; j++)
                {
                    scalarSum_Re += coeffs[coefIndex] * states[statesIndex] - (coeffs[coefIndex+1] * states[statesIndex+1]);
                    scalarSum_Im += coeffs[coefIndex] * states[statesIndex+1]+(coeffs[coefIndex+1]*states[statesIndex]);
                    coefIndex+=2;
                    statesIndex+=2;
                }
                /* Computing sum of individual elements in a vector */
                high128 = _mm256_extractf128_ps(accumulator, 1);
                low128 = _mm256_extractf128_ps(accumulator, 0);
                summation = _mm_add_ps(high128, low128);
                summation=_mm_hadd_ps(summation,summation);
                out[oIdx] = _mm_cvtss_f32(summation) + scalarSum_Re;
                summation = _mm_permute_ps(summation,_MM_SHUFFLE(0,1,2,3));
                out[oIdx+1] = _mm_cvtss_f32(summation)+scalarSum_Im;
                oIdx += iFactor * 2;
            }
             /* Loop Unrolled Twice */

            for (i = 0; i < numIteration; i += 2)    /* FIR Interpolator- AVX2 SIMD Implementation for calculating output when Framesize > nStates*/
            {
                __m256 loadIn;
                __m256 loadIn_1;
                __m256 loadCoeffs;
                __m256 loadCoeffs_1;
                __m256 realMul;
                __m256 imagMul;
                __m256 result;
                __m256 result_1;
                __m128 high128;
                __m128 low128;
                __m128 summation;
                /*  Initializing the accumulators to Zero */
                __m256 accumulator = _mm256_setzero_ps();
                __m256 accumulator_1 = _mm256_setzero_ps();
                float scalarSum_Re = 0.0;
                float scalarSum_Im = 0.0;
                size_t inIdx;
                size_t coefIdx;
                inIdx = i + dataInArrayIdx;
                coefIdx = (coeffsLength - 2) - nStates - coefArrayIdx;

                for (j = 0; j < nStateVectLoopCount; j++)
                {
                    loadIn = _mm256_loadu_ps(&in[inIdx]);                           /* Vector load */
                    loadCoeffs = _mm256_loadu_ps(&reversedCoeffs[coefIdx]);
                    realMul = _mm256_mul_ps(loadIn, loadCoeffs);
                    loadCoeffs = _mm256_permute_ps(loadCoeffs, 0xb1);
                    loadCoeffs = _mm256_mul_ps(loadCoeffs, negFactor);
                    imagMul = _mm256_mul_ps(loadIn, loadCoeffs);
                    result = _mm256_hsub_ps(realMul, imagMul);
                    accumulator = _mm256_add_ps(accumulator, result);
                    loadIn_1 = _mm256_loadu_ps(&in[inIdx + VECTOR_UNITS_FOR_256]);
                    loadCoeffs_1 = _mm256_loadu_ps(&reversedCoeffs[coefIdx + VECTOR_UNITS_FOR_256]);
                    inIdx += UF;                                                     /* Auto-increment by 8 since loop unrolling takes place twice */
                    coefIdx += UF;
                    realMul = _mm256_mul_ps(loadIn_1, loadCoeffs_1);
                    loadCoeffs_1 = _mm256_permute_ps(loadCoeffs_1, 0xb1);
                    loadCoeffs_1 = _mm256_mul_ps(loadCoeffs_1, negFactor);
                    imagMul = _mm256_mul_ps(loadIn_1, loadCoeffs_1);
                    result_1 = _mm256_hsub_ps(realMul, imagMul);
                    accumulator_1 = _mm256_add_ps(accumulator_1, result_1);
                }

                for (j = 0; j < nStateScalarLoopCount; j+=2)
                {
                    scalarSum_Re += in[inIdx] * reversedCoeffs[coefIdx]-(in[inIdx+1]*reversedCoeffs[coefIdx+1]);
                    scalarSum_Im += in[inIdx] * reversedCoeffs[coefIdx+1]+(in[inIdx+1]*reversedCoeffs[coefIdx]);
                    inIdx+=2;
                    coefIdx+=2;
                }

                /* Summing up the MUL-Accumulation results */
                accumulator = _mm256_add_ps(accumulator, accumulator_1);

                high128 = _mm256_extractf128_ps(accumulator, 1);
                low128 = _mm256_extractf128_ps(accumulator, 0);
                summation = _mm_add_ps(high128, low128);
                summation=_mm_hadd_ps(summation,summation);
                out[oIdx] = _mm_cvtss_f32(summation) + scalarSum_Re;
                summation = _mm_permute_ps(summation,_MM_SHUFFLE(0,1,2,3));
                out[oIdx+1] = _mm_cvtss_f32(summation)+scalarSum_Im;

                oIdx += iFactor * 2;
            } /* FIR Interpolator- AVX2 SIMD Implementation for calculating output when Framesize > nStates*/

        } /* End of Polyphase Implementation Loop */

        /* Update delay line for next frame */
        if (frameSize >= nStates)
        {
            for (i = 0; i < nStates; i+=2)
            {
                size_t x = delayBuffIdx + i;
                size_t y = frameSize - 2 - i + dataInArrayIdx;
                states[x] = in[y];
                states[++x] = in[++y];
            }
        }

        else
        {
            int d;
            size_t x;
            size_t y;
            for (d = (int)(delayBuffIdx + nStates - frameSize - 2); d >= delayBuffIdx; d -= 2) {
                x = frameSize + d;
                y = d;
                states[x] = states[y];
                states[++x] = states[++y];
            }
            for (j = 0; j < frameSize; j += 2) {
                x = delayBuffIdx + frameSize - 2 - j;
                y = dataInArrayIdx + j;
                states[x] = in[y];
                states[++x] = in[++y];
            }
        }

    } /* End of Loop for multichannel inputs */
}


/**
 * Reversal of Coefficients for coefficients from dialog parameters and assignment of the structure elements is done in MWDSPCRL_avx2_fir_interpolator_init_mask
 * @param S : structure fir_Interpolator
 * @param coeffs : Coefficients vector
 * @param reversedCoeffs : Reversed Coefficients vector
 * @param coeffLength : Filter Order
 * @param iFactor : Interpolation factor
 * @param frameSize : Frame length
 * @param nChans : Number of channels
 * @param states : States vector
 * @param isRealCoeffs : Flag for coefficients
 * Output
 * @return void
 **/
void MWDSPCRL_avx2_interp_sgl_cmplx_init(const float *coeffs, float *reversedCoeffs, size_t coeffsLength,float* transCoeffs,float* reversedtransCoeffs, size_t iFactor, size_t nChans, float* states)
{
    size_t polyphaseFiltLen;
    polyphaseFiltLen = coeffsLength/iFactor;
    if(polyphaseFiltLen-2 > 0) {
        size_t stateLength;
        stateLength =((coeffsLength/iFactor)-1)*nChans*2;
        memset(states,0,stateLength*sizeof(states[0]));
    }
    reversalOfCoefficients(coeffs,reversedCoeffs,coeffsLength,transCoeffs,reversedtransCoeffs);
}

void MWDSPCRL_avx2_interp_sgl_cmplx_init_coefficients(const float *coeffs, size_t coeffsLength, float* reversedtransCoeffs, size_t iFactor, size_t nChans, float* states)
{
    size_t polyphaseFiltLen;
    polyphaseFiltLen = coeffsLength/iFactor;
    if(polyphaseFiltLen-2 > 0) {
        size_t stateLength;
        stateLength=((coeffsLength/iFactor)-1)*nChans*2;
        memset(states,0,stateLength*sizeof(states[0]));
    }
    reversalOfCmplxCoefficients(coeffs,reversedtransCoeffs,coeffsLength);
}


/**
 *AVX2 implementation of FIR Interpolator when Coefficient Source is Input Port and coefficents are REAL Valued
 *
 **/
void MWDSPCRL_avx2_interp_sgl_cmplx_port(FIRInterpSglCmplx *S, float *states, const float *initialCoeffs,float *reversedCoeffs, float* transCoeffs,float* reversedtransCoeffs,const float *in, float *out)
{
    size_t indx,m,j;  /*Loop iterators*/
    size_t iFactor = S->iFactor;
    size_t coeffsLength=S->coeffsLength;
    size_t polyphaseFiltLen=(coeffsLength+iFactor-1)/iFactor;
    size_t coeffLen=polyphaseFiltLen*iFactor;
    /*Polyphase Conversion of coefficents*/
    float *coeffs=NULL;
    coeffs=(float*)calloc(coeffLen, sizeof(float));
    for(m = 0; m < iFactor; m++)
    {
        indx = m*polyphaseFiltLen;
        for(j = m; j < coeffsLength;j+=iFactor)
        {
            coeffs[indx]=initialCoeffs[j];
            indx++;
        }
    }
    S->coeffsLength=coeffLen;    
    reversalOfCoefficients(coeffs,reversedCoeffs,coeffLen,transCoeffs, reversedtransCoeffs);
    firInterpolator_RealCoeffs(S, states, transCoeffs, reversedtransCoeffs, in, out);    
    S->coeffsLength=coeffsLength;    
    free(coeffs);
}

/**
 *AVX2 implementation of FIR Interpolator when Coefficient Source is Dialog Parameters
 *
 **/
void MWDSPCRL_avx2_interp_sgl_cmplx_process(FIRInterpSglCmplx *S, float *states, float* transCoeffs,float* reversedtransCoeffs, const float* in, float* out)
{
    firInterpolator_RealCoeffs(S, states, transCoeffs, reversedtransCoeffs, in, out);
}

void MWDSPCRL_avx2_interp_sgl_cmplx_process_coefficients(FIRInterpSglCmplx *S, float* states, float* reversedtransCoeffs, const float* in, float* out, const float* coeffs)
{    
    firInterpolator_CmplxCoeffs(S,states,coeffs,reversedtransCoeffs,in,out);
}

/**
 *AVX2 implementation of FIR Interpolator when Coefficient Source is Input Port and Coefficients are COMPLEX valued
 *
 **/
void MWDSPCRL_avx2_interp_sgl_cmplx_port_coefficients(FIRInterpSglCmplx *S, float* states, float *initialCoeffs,float* transCoeffs,float* reversedtransCoeffs,const float* in, float* out)
{
    size_t indx,m,j,arrIdx;  /*Loop iterators*/
    size_t iFactor = S->iFactor;
    size_t coeffsLength=S->coeffsLength;
    size_t polyphaseFiltLen=(coeffsLength+iFactor-1)/iFactor;
    size_t coeffLen=polyphaseFiltLen*iFactor;
    /*Polyphase Conversion of coefficents*/
    for(m = 0; m < iFactor; m++)
    {
        indx = (m * polyphaseFiltLen) * 2;
        for(j = m; j < coeffsLength;j+=iFactor)
        {
            arrIdx = j * 2;
            transCoeffs[indx++]=initialCoeffs[arrIdx];
            transCoeffs[indx++]=initialCoeffs[arrIdx + 1];
        }
    }    
    S->coeffsLength=coeffLen*2;
    reversalOfCmplxCoefficients(transCoeffs, reversedtransCoeffs, S->coeffsLength);
    firInterpolator_CmplxCoeffs(S, states, transCoeffs, reversedtransCoeffs, in, out);    
    S->coeffsLength=coeffsLength;    
}
