/* Copyright 2019 The MathWorks, Inc. */

#include "lms_dbl_avx2_fcn.h"

#include <immintrin.h>

/* LMS with adapt input, double precision */
void MWSPCGlmsa_D_avx2(const real_T x[], const real_T d[], real_T mu,
        uint32_T *startIdx, real_T xBuf[], real_T wBuf[], int32_T wLen,
        real_T leakFac, int32_T xLen, real_T y[], real_T eY[],
        boolean_T NeedAdapt)
{
    int32_T i;
    int32_T j;
    int32_T j1;
    real_T prodMuErr;
    
    int32_T bStartIdx = (int32_T)*startIdx;
    
    for (i = 0; i < xLen; i++) {
        /* Copy the current sample at the END of the circular buffer */
        xBuf[bStartIdx] = x[i];
        bStartIdx++;
        if (bStartIdx == wLen) {
            bStartIdx = 0U;
        }
        
        /* Multiply wgtBuff_vector (not yet updated) and inBuff_vector */
        j1 = 0;
        
        __m256d s1 = _mm256_setzero_pd();
        __m256d s2 = _mm256_setzero_pd();
        for (j = bStartIdx; j + 7 < wLen; j += 8) {
            __m256d t1 = _mm256_loadu_pd(&wBuf[j1]);
            __m256d t2 = _mm256_loadu_pd(&xBuf[j]);
            __m256d t3 = _mm256_loadu_pd(&wBuf[j1+4]);
            __m256d t4 = _mm256_loadu_pd(&xBuf[j+4]);
            s1 = _mm256_fmadd_pd(t1, t2, s1);
            s2 = _mm256_fmadd_pd(t3, t4, s2);
            j1 += 8;
        }
        
        y[i] = 0.0;
        for ( ; j < wLen; j ++) {
            y[i] += wBuf[j1] * xBuf[j];
            j1++;
        }
        
        for (j = 0; j + 7 < bStartIdx; j += 8) {
            __m256d t1 = _mm256_loadu_pd(&wBuf[j1]);
            __m256d t2 = _mm256_loadu_pd(&xBuf[j]);
            __m256d t3 = _mm256_loadu_pd(&wBuf[j1 + 4]);
            __m256d t4 = _mm256_loadu_pd(&xBuf[j + 4]);
            s1 = _mm256_fmadd_pd(t1, t2, s1);
            s2 = _mm256_fmadd_pd(t3, t4, s2);
            j1 += 8;
        }
        
        /* Horizontal summation */
        s1 = _mm256_add_pd(s1, s2);
        __m128d hsum = _mm_add_pd(_mm256_castpd256_pd128(s1), _mm256_extractf128_pd(s1, 1));
        hsum = _mm_add_sd(hsum, _mm_unpackhi_pd(hsum, hsum));
        y[i] += _mm_cvtsd_f64(hsum);
        
        for (; j < bStartIdx; j++) {
            y[i] += wBuf[j1] * xBuf[j];
            j1++;
        }
        
        /* Get error for the current sample */
        eY[i] = d[i] - y[i];
        prodMuErr = mu * eY[i];
        
        if (NeedAdapt) {
            /* Update weight-vector for next input sample */
            j1 = 0;
            __m256d leakFac256 = _mm256_broadcast_sd(&leakFac);
            __m256d prodMuErr256 = _mm256_broadcast_sd(&prodMuErr);
            
            for (j = bStartIdx; j+7 < wLen; j += 8) {
                __m256d t1 = _mm256_loadu_pd(&wBuf[j1]);
                __m256d t2 = _mm256_loadu_pd(&xBuf[j]);
                __m256d t3 = _mm256_loadu_pd(&wBuf[j1 + 4]);
                __m256d t4 = _mm256_loadu_pd(&xBuf[j + 4]);
                __m256d t5 = _mm256_mul_pd(t1, leakFac256);
                __m256d t6 = _mm256_fmadd_pd(t2, prodMuErr256, t5);
                _mm256_storeu_pd(&wBuf[j1], t6);
                __m256d t7 = _mm256_mul_pd(t3, leakFac256);
                __m256d t8 = _mm256_fmadd_pd(t4, prodMuErr256, t7);
                _mm256_storeu_pd(&wBuf[j1+4], t8);
                j1 += 8;
            }
            
            for ( ; j < wLen; j++) {
                wBuf[j1] = prodMuErr * xBuf[j] + leakFac * wBuf[j1];
                j1++;
            }
            
            for (j = 0; j+7 < bStartIdx; j += 8) {
                __m256d t1 = _mm256_loadu_pd(&wBuf[j1]);
                __m256d t2 = _mm256_loadu_pd(&xBuf[j]);
                __m256d t3 = _mm256_loadu_pd(&wBuf[j1 + 4]);
                __m256d t4 = _mm256_loadu_pd(&xBuf[j + 4]);
                __m256d t5 = _mm256_mul_pd(t1, leakFac256);
                __m256d t6 = _mm256_fmadd_pd(t2, prodMuErr256, t5);
                _mm256_storeu_pd(&wBuf[j1], t6);
                __m256d t7 = _mm256_mul_pd(t3, leakFac256);
                __m256d t8 = _mm256_fmadd_pd(t4, prodMuErr256, t7);
                _mm256_storeu_pd(&wBuf[j1 + 4], t8);
                j1 += 8;
            }
            
            for ( ; j < bStartIdx; j++) {
                wBuf[j1] = prodMuErr * xBuf[j] + leakFac * wBuf[j1];
                j1++;
            }
        }
    }
    *startIdx = bStartIdx;
}

/* LMS, double precision */
void MWSPCGlmsn_D_avx2(const real_T x[], const real_T d[], real_T mu,
        uint32_T *startIdx, real_T xBuf[], real_T wBuf[], int32_T wLen,
        real_T leakFac, int32_T xLen, real_T y[], real_T eY[])
{
    MWSPCGlmsa_D_avx2(x,d,mu,startIdx,xBuf,wBuf,wLen,leakFac,xLen,y,eY,1);
}

/* LMS with weights output, double precision */
void MWSPCGlmsw_D_avx2(const real_T x[], const real_T d[], real_T mu,
        uint32_T *startIdx, real_T xBuf[], real_T wBuf[], int32_T wLen,
        real_T leakFac, int32_T xLen, real_T y[], real_T eY[], real_T wY[])
{
    int32_T j;
    int32_T j1;
    
    MWSPCGlmsa_D_avx2(x,d,mu,startIdx,xBuf,wBuf,wLen,leakFac,xLen,y,eY,1);
    
    j1 = wLen;
    for (j = 0; j < wLen; j++) {
        wY[j] = wBuf[j1 - 1];
        j1--;
    }
}

/* LMS with both adapt input and weights output, double precision */
void MWSPCGlmsb_D_avx2(const real_T x[], const real_T d[], real_T mu,
        uint32_T *startIdx, real_T xBuf[], real_T wBuf[], int32_T wLen,
        real_T leakFac, int32_T xLen, real_T y[], real_T eY[], real_T wY[],
        boolean_T NeedAdapt)
{
    int32_T j;
    int32_T j1;
    
    MWSPCGlmsa_D_avx2(x,d,mu,startIdx,xBuf,wBuf,wLen,leakFac,xLen,y,eY,NeedAdapt);
    
    j1 = wLen;
    for (j = 0; j < wLen; j++) {
        wY[j] = wBuf[j1 - 1];
        j1--;
    }
}


/* Normalized LMS with adapt input, double precision */
void MWSPCGlmsna_D_avx2(const real_T x[], const real_T d[], real_T mu,
        uint32_T *startIdx, real_T xBuf[], real_T wBuf[], int32_T wLen,
        real_T leakFac, int32_T xLen, real_T y[], real_T eY[],
        boolean_T NeedAdapt)
{
    int32_T i;
    int32_T j;
    int32_T j1;
    real_T prodMuErr;
    
    int32_T bStartIdx = (int32_T)*startIdx;
    
    for (i = 0; i < xLen; i++) {
        real_T bufEnergy = 0.0;
        
        /* Copy the current sample at the END of the circular buffer */
        xBuf[bStartIdx] = x[i];
        bStartIdx++;
        if (bStartIdx == wLen) {
            bStartIdx = 0U;
        }
        
        /* Multiply wgtBuff_vector (not yet updated) and inBuff_vector */
        /* Get the energy of the signal in updated buffer */
        j1 = 0;
        
        __m256d s1 = _mm256_setzero_pd();
        __m256d s2 = _mm256_setzero_pd();
        __m256d s3 = _mm256_setzero_pd();
        __m256d s4 = _mm256_setzero_pd();
        for (j = bStartIdx; j + 7 < wLen; j += 8) {
            __m256d t1 = _mm256_loadu_pd(&wBuf[j1]);
            __m256d t2 = _mm256_loadu_pd(&xBuf[j]);
            __m256d t3 = _mm256_loadu_pd(&wBuf[j1+4]);
            __m256d t4 = _mm256_loadu_pd(&xBuf[j+4]);
            s1 = _mm256_fmadd_pd(t1, t2, s1);
            s2 = _mm256_fmadd_pd(t3, t4, s2);
            s3 = _mm256_fmadd_pd(t2, t2, s3);
            s4 = _mm256_fmadd_pd(t4, t4, s4);
            j1 += 8;
        }
        
        y[i] = 0.0;
        for ( ; j < wLen; j ++) {
            y[i] += wBuf[j1] * xBuf[j];
            bufEnergy += xBuf[j] * xBuf[j];
            j1++;
        }
        
        for (j = 0; j + 7 < bStartIdx; j += 8) {
            __m256d t1 = _mm256_loadu_pd(&wBuf[j1]);
            __m256d t2 = _mm256_loadu_pd(&xBuf[j]);
            __m256d t3 = _mm256_loadu_pd(&wBuf[j1 + 4]);
            __m256d t4 = _mm256_loadu_pd(&xBuf[j + 4]);
            s1 = _mm256_fmadd_pd(t1, t2, s1);
            s2 = _mm256_fmadd_pd(t3, t4, s2);
            s3 = _mm256_fmadd_pd(t2, t2, s3);
            s4 = _mm256_fmadd_pd(t4, t4, s4);
            j1 += 8;
        }
        
        /* Horizontal summations */
        s1 = _mm256_add_pd(s1, s2);
        __m128d hsum = _mm_add_pd(_mm256_castpd256_pd128(s1), _mm256_extractf128_pd(s1, 1));
        hsum = _mm_add_sd(hsum, _mm_unpackhi_pd(hsum, hsum));
        y[i] += _mm_cvtsd_f64(hsum);
        
		s3 = _mm256_add_pd(s3, s4);
        hsum = _mm_add_pd(_mm256_castpd256_pd128(s3), _mm256_extractf128_pd(s3, 1));
        hsum = _mm_add_sd(hsum, _mm_unpackhi_pd(hsum, hsum));
        bufEnergy += _mm_cvtsd_f64(hsum);
        
        for (; j < bStartIdx; j++) {
            y[i] += wBuf[j1] * xBuf[j];
            bufEnergy += xBuf[j] * xBuf[j];
            j1++;
        }
        
        /* Get error for the current sample */
        eY[i] = d[i] - y[i];
        prodMuErr = mu * eY[i] / (bufEnergy + 2.2204460492503131E-16);
        
        if (NeedAdapt) {
            /* Update weight-vector for next input sample */
            j1 = 0;
            __m256d leakFac256 = _mm256_broadcast_sd(&leakFac);
            __m256d prodMuErr256 = _mm256_broadcast_sd(&prodMuErr);
            
            for (j = bStartIdx; j+7 < wLen; j += 8) {
                __m256d t1 = _mm256_loadu_pd(&wBuf[j1]);
                __m256d t2 = _mm256_loadu_pd(&xBuf[j]);
                __m256d t3 = _mm256_loadu_pd(&wBuf[j1 + 4]);
                __m256d t4 = _mm256_loadu_pd(&xBuf[j + 4]);
                __m256d t5 = _mm256_mul_pd(t1, leakFac256);
                __m256d t6 = _mm256_fmadd_pd(t2, prodMuErr256, t5);
                _mm256_storeu_pd(&wBuf[j1], t6);
                __m256d t7 = _mm256_mul_pd(t3, leakFac256);
                __m256d t8 = _mm256_fmadd_pd(t4, prodMuErr256, t7);
                _mm256_storeu_pd(&wBuf[j1+4], t8);
                j1 += 8;
            }
            
            for ( ; j < wLen; j++) {
                wBuf[j1] = prodMuErr * xBuf[j] + leakFac * wBuf[j1];
                j1++;
            }
            
            for (j = 0; j+7 < bStartIdx; j += 8) {
                __m256d t1 = _mm256_loadu_pd(&wBuf[j1]);
                __m256d t2 = _mm256_loadu_pd(&xBuf[j]);
                __m256d t3 = _mm256_loadu_pd(&wBuf[j1 + 4]);
                __m256d t4 = _mm256_loadu_pd(&xBuf[j + 4]);
                __m256d t5 = _mm256_mul_pd(t1, leakFac256);
                __m256d t6 = _mm256_fmadd_pd(t2, prodMuErr256, t5);
                _mm256_storeu_pd(&wBuf[j1], t6);
                __m256d t7 = _mm256_mul_pd(t3, leakFac256);
                __m256d t8 = _mm256_fmadd_pd(t4, prodMuErr256, t7);
                _mm256_storeu_pd(&wBuf[j1 + 4], t8);
                j1 += 8;
            }
            
            for ( ; j < bStartIdx; j++) {
                wBuf[j1] = prodMuErr * xBuf[j] + leakFac * wBuf[j1];
                j1++;
            }
        }
    }
    *startIdx = bStartIdx;
}

/* Normalized LMS, double precision */
void MWSPCGlmsnn_D_avx2(const real_T x[], const real_T d[], real_T mu,
        uint32_T *startIdx, real_T xBuf[], real_T wBuf[], int32_T wLen,
        real_T leakFac, int32_T xLen, real_T y[], real_T eY[])
{
    MWSPCGlmsna_D_avx2(x,d,mu,startIdx,xBuf,wBuf,wLen,leakFac,xLen,y,eY,1);
}

/* Normalized LMS with weights output, double precision */
void MWSPCGlmsnw_D_avx2(const real_T x[], const real_T d[], real_T mu,
        uint32_T *startIdx, real_T xBuf[], real_T wBuf[], int32_T wLen,
        real_T leakFac, int32_T xLen, real_T y[], real_T eY[], real_T wY[])
{
    int32_T j;
    int32_T j1;
    
    MWSPCGlmsna_D_avx2(x,d,mu,startIdx,xBuf,wBuf,wLen,leakFac,xLen,y,eY,1);
    
    j1 = wLen;
    for (j = 0; j < wLen; j++) {
        wY[j] = wBuf[j1 - 1];
        j1--;
    }
}

/* Normalized LMS with adapt input and weights output, double precision */
void MWSPCGlmsnb_D_avx2(const real_T x[], const real_T d[], real_T mu,
        uint32_T *startIdx, real_T xBuf[], real_T wBuf[], int32_T wLen,
        real_T leakFac, int32_T xLen, real_T y[], real_T eY[], real_T wY[],
        boolean_T NeedAdapt)
{
    int32_T j;
    int32_T j1;
    
    MWSPCGlmsna_D_avx2(x,d,mu,startIdx,xBuf,wBuf,wLen,leakFac,xLen,y,eY,NeedAdapt);
    
    j1 = wLen;
    for (j = 0; j < wLen; j++) {
        wY[j] = wBuf[j1 - 1];
        j1--;
    }
}



