/* Copyright 2019 The MathWorks, Inc. */

#include "lms_sgl_avx2_fcn.h"

#include <immintrin.h>

/* LMS with adapt input, single precision */
void MWSPCGlmsa_S_avx2(const real32_T x[], const real32_T d[], real32_T mu,
        uint32_T *startIdx, real32_T xBuf[], real32_T wBuf[], int32_T wLen,
        real32_T leakFac, int32_T xLen, real32_T y[], real32_T eY[],
        boolean_T NeedAdapt)
{
    int32_T i;
    int32_T j;
    int32_T j1;
    real32_T prodMuErr;
    
    int32_T bStartIdx = (int32_T)*startIdx;
    
    for (i = 0; i < xLen; i++) {
        /* Copy the current sample at the END of the circular buffer */
        xBuf[bStartIdx] = x[i];
        bStartIdx++;
        if (bStartIdx == wLen) {
            bStartIdx = 0U;
        }
        
        /* Multiply wgtBuff_vector (not yet updated) and inBuff_vector */
        j1 = 0;
        
        __m256 s1 = _mm256_setzero_ps();
        for (j = bStartIdx; j + 7 < wLen; j += 8) {
            __m256 t1 = _mm256_loadu_ps(&wBuf[j1]);
            __m256 t2 = _mm256_loadu_ps(&xBuf[j]);
            s1 = _mm256_fmadd_ps(t1, t2, s1);
            j1 += 8;
        }
        
        y[i] = 0.0F;
        for ( ; j < wLen; j ++) {
            y[i] += wBuf[j1] * xBuf[j];
            j1++;
        }
        
        for (j = 0; j + 7 < bStartIdx; j += 8) {
            __m256 t1 = _mm256_loadu_ps(&wBuf[j1]);
            __m256 t2 = _mm256_loadu_ps(&xBuf[j]);
            s1 = _mm256_fmadd_ps(t1, t2, s1);
            j1 += 8;
        }
        
        /* Horizontal summation */
        __m128 hsum = _mm_add_ps(_mm256_castps256_ps128(s1), _mm256_extractf128_ps(s1, 1));
        hsum = _mm_add_ps(hsum, _mm_movehl_ps(hsum, hsum));
        hsum = _mm_add_ss(hsum, _mm_shuffle_ps(hsum, hsum, 0x1));
        y[i] += _mm_cvtss_f32(hsum);
        
        for (; j < bStartIdx; j++) {
            y[i] += wBuf[j1] * xBuf[j];
            j1++;
        }
        
        /* Get error for the current sample */
        eY[i] = d[i] - y[i];
        prodMuErr = mu * eY[i];
        
        /* Update weight-vector for next input sample */
        if (NeedAdapt) {
            j1 = 0;
            __m256 leakFac256 = _mm256_broadcast_ss(&leakFac);
            __m256 prodMuErr256 = _mm256_broadcast_ss(&prodMuErr);
            for (j = bStartIdx; j+7 < wLen; j += 8) {
                __m256 t1 = _mm256_loadu_ps(&wBuf[j1]);
                __m256 t2 = _mm256_loadu_ps(&xBuf[j]);
                __m256 t5 = _mm256_mul_ps(t1, leakFac256);
                __m256 t6 = _mm256_fmadd_ps(t2, prodMuErr256, t5);
                _mm256_storeu_ps(&wBuf[j1], t6);
                j1 += 8;
            }
            for ( ; j < wLen; j++) {
                wBuf[j1] = prodMuErr * xBuf[j] + leakFac * wBuf[j1];
                j1++;
            }
            
            for (j = 0; j+7 < bStartIdx; j += 8) {
                __m256 t1 = _mm256_loadu_ps(&wBuf[j1]);
                __m256 t2 = _mm256_loadu_ps(&xBuf[j]);
                __m256 t5 = _mm256_mul_ps(t1, leakFac256);
                __m256 t6 = _mm256_fmadd_ps(t2, prodMuErr256, t5);
                _mm256_storeu_ps(&wBuf[j1], t6);
                j1 += 8;
            }
            for ( ; j < bStartIdx; j++) {
                wBuf[j1] = prodMuErr * xBuf[j] + leakFac * wBuf[j1];
                j1++;
            }
        }
    }
    *startIdx = bStartIdx;
}

/* LMS, single precision */
void MWSPCGlmsn_S_avx2(const real32_T x[], const real32_T d[], real32_T mu,
        uint32_T *startIdx, real32_T xBuf[], real32_T wBuf[], int32_T wLen,
        real32_T leakFac, int32_T xLen, real32_T y[], real32_T eY[])
{
    MWSPCGlmsa_S_avx2(x,d,mu,startIdx,xBuf,wBuf,wLen,leakFac,xLen,y,eY,1);
}

/* LMS with weights output, single precision */
void MWSPCGlmsw_S_avx2(const real32_T x[], const real32_T d[], real32_T mu,
        uint32_T *startIdx, real32_T xBuf[], real32_T wBuf[], int32_T wLen,
        real32_T leakFac, int32_T xLen, real32_T y[], real32_T eY[], real32_T wY[])
{
    int32_T j;
    int32_T j1;
    
    MWSPCGlmsa_S_avx2(x,d,mu,startIdx,xBuf,wBuf,wLen,leakFac,xLen,y,eY,1);
    
    j1 = wLen;
    for (j = 0; j < wLen; j++) {
        wY[j] = wBuf[j1 - 1];
        j1--;
    }
}

/* LMS with both adapt input and weights output, single precision */
void MWSPCGlmsb_S_avx2(const real32_T x[], const real32_T d[], real32_T mu,
        uint32_T *startIdx, real32_T xBuf[], real32_T wBuf[], int32_T wLen,
        real32_T leakFac, int32_T xLen, real32_T y[], real32_T eY[], real32_T wY[],
        boolean_T NeedAdapt)
{
    int32_T j;
    int32_T j1;
    
    MWSPCGlmsa_S_avx2(x,d,mu,startIdx,xBuf,wBuf,wLen,leakFac,xLen,y,eY,NeedAdapt);
    
    j1 = wLen;
    for (j = 0; j < wLen; j++) {
        wY[j] = wBuf[j1 - 1];
        j1--;
    }
}


/* Normalized LMS with adapt input, single precision */
void MWSPCGlmsna_S_avx2(const real32_T x[], const real32_T d[], real32_T mu,
        uint32_T *startIdx, real32_T xBuf[], real32_T wBuf[], int32_T wLen,
        real32_T leakFac, int32_T xLen, real32_T y[], real32_T eY[],
        boolean_T NeedAdapt)
{
    int32_T i;
    int32_T j;
    int32_T j1;
    real32_T prodMuErr;
    
    int32_T bStartIdx = (int32_T)*startIdx;
    
    for (i = 0; i < xLen; i++) {
        real32_T bufEnergy = 0.0F;
        
        /* Copy the current sample at the END of the circular buffer */
        xBuf[bStartIdx] = x[i];
        bStartIdx++;
        if (bStartIdx == wLen) {
            bStartIdx = 0U;
        }
        
        /* Multiply wgtBuff_vector (not yet updated) and inBuff_vector */
        /* Get the energy of the signal in updated buffer */
        j1 = 0;
        __m256 s1 = _mm256_setzero_ps();
        __m256 s3 = _mm256_setzero_ps();
        for (j = bStartIdx; j + 7 < wLen; j += 8) {
            __m256 t1 = _mm256_loadu_ps(&wBuf[j1]);
            __m256 t2 = _mm256_loadu_ps(&xBuf[j]);
            s1 = _mm256_fmadd_ps(t1, t2, s1);
            s3 = _mm256_fmadd_ps(t2, t2, s3);
            j1 += 8;
        }
        
        y[i] = 0.0F;
        for ( ; j < wLen; j ++) {
            y[i] += wBuf[j1] * xBuf[j];
            bufEnergy += xBuf[j] * xBuf[j];
            j1++;
        }
        
        for (j = 0; j + 7 < bStartIdx; j += 8) {
            __m256 t1 = _mm256_loadu_ps(&wBuf[j1]);
            __m256 t2 = _mm256_loadu_ps(&xBuf[j]);
            s1 = _mm256_fmadd_ps(t1, t2, s1);
            s3 = _mm256_fmadd_ps(t2, t2, s3);
            j1 += 8;
        }
        
        /* Horizontal summations */
        __m128 hsum = _mm_add_ps(_mm256_castps256_ps128(s1), _mm256_extractf128_ps(s1, 1));
        hsum = _mm_add_ps(hsum, _mm_movehl_ps(hsum, hsum));
        hsum = _mm_add_ss(hsum, _mm_shuffle_ps(hsum, hsum, 0x1));
        y[i] += _mm_cvtss_f32(hsum);
        
        hsum = _mm_add_ps(_mm256_castps256_ps128(s3), _mm256_extractf128_ps(s3, 1));
        hsum = _mm_add_ps(hsum, _mm_movehl_ps(hsum, hsum));
        hsum = _mm_add_ss(hsum, _mm_shuffle_ps(hsum, hsum, 0x1));
        bufEnergy += _mm_cvtss_f32(hsum);
        
        for (; j < bStartIdx; j++) {
            y[i] += wBuf[j1] * xBuf[j];
            bufEnergy += xBuf[j] * xBuf[j];
            j1++;
        }
        
        /* Get error for the current sample */
        eY[i] = d[i] - y[i];
        prodMuErr = mu * eY[i] / (bufEnergy + 1.1920929E-7F);
        
        if (NeedAdapt) {
            /* Update weight-vector for next input sample */
            j1 = 0;
            __m256 leakFac256 = _mm256_broadcast_ss(&leakFac);
            __m256 prodMuErr256 = _mm256_broadcast_ss(&prodMuErr);
            for (j = bStartIdx; j+7 < wLen; j += 8) {
                __m256 t1 = _mm256_loadu_ps(&wBuf[j1]);
                __m256 t2 = _mm256_loadu_ps(&xBuf[j]);
                __m256 t5 = _mm256_mul_ps(t1, leakFac256);
                __m256 t6 = _mm256_fmadd_ps(t2, prodMuErr256, t5);
                _mm256_storeu_ps(&wBuf[j1], t6);
                j1 += 8;
            }
            for ( ; j < wLen; j++) {
                wBuf[j1] = prodMuErr * xBuf[j] + leakFac * wBuf[j1];
                j1++;
            }
            
            for (j = 0; j+7 < bStartIdx; j += 8) {
                __m256 t1 = _mm256_loadu_ps(&wBuf[j1]);
                __m256 t2 = _mm256_loadu_ps(&xBuf[j]);
                __m256 t5 = _mm256_mul_ps(t1, leakFac256);
                __m256 t6 = _mm256_fmadd_ps(t2, prodMuErr256, t5);
                _mm256_storeu_ps(&wBuf[j1], t6);
                j1 += 8;
            }
            for ( ; j < bStartIdx; j++) {
                wBuf[j1] = prodMuErr * xBuf[j] + leakFac * wBuf[j1];
                j1++;
            }
        }
    }
    *startIdx = bStartIdx;
}

/* Normalized LMS, single precision */
void MWSPCGlmsnn_S_avx2(const real32_T x[], const real32_T d[], real32_T mu,
        uint32_T *startIdx, real32_T xBuf[], real32_T wBuf[], int32_T wLen,
        real32_T leakFac, int32_T xLen, real32_T y[], real32_T eY[])
{
    MWSPCGlmsna_S_avx2(x,d,mu,startIdx,xBuf,wBuf,wLen,leakFac,xLen,y,eY,1);
}

/* Normalized LMS with weights output, single precision */
void MWSPCGlmsnw_S_avx2(const real32_T x[], const real32_T d[], real32_T mu,
        uint32_T *startIdx, real32_T xBuf[], real32_T wBuf[], int32_T wLen,
        real32_T leakFac, int32_T xLen, real32_T y[], real32_T eY[], real32_T wY[])
{
    int32_T j;
    int32_T j1;
    
    MWSPCGlmsna_S_avx2(x,d,mu,startIdx,xBuf,wBuf,wLen,leakFac,xLen,y,eY,1);
    
    j1 = wLen;
    for (j = 0; j < wLen; j++) {
        wY[j] = wBuf[j1 - 1];
        j1--;
    }
}

/* Normalized LMS with adapt input and weights output, single precision */
void MWSPCGlmsnb_S_avx2(const real32_T x[], const real32_T d[], real32_T mu,
        uint32_T *startIdx, real32_T xBuf[], real32_T wBuf[], int32_T wLen,
        real32_T leakFac, int32_T xLen, real32_T y[], real32_T eY[], real32_T wY[],
        boolean_T NeedAdapt)
{
    int32_T j;
    int32_T j1;
    
    MWSPCGlmsna_S_avx2(x,d,mu,startIdx,xBuf,wBuf,wLen,leakFac,xLen,y,eY,NeedAdapt);
    
    j1 = wLen;
    for (j = 0; j < wLen; j++) {
        wY[j] = wBuf[j1 - 1];
        j1--;
    }
}



