function audioblkAudioDeviceIO(blkh, command, varargin)
% AUDIOBLKCOMPRESSION Mask initialization function for Audio Device Reader
% and Audio Device Writer blocks

% Copyright 2015-2021 The MathWorks, Inc.

model = bdroot(blkh);
isCalledFromModel = strcmp(get_param(model, 'BlockDiagramType'), 'model');
isInputDevice = strcmp(get_param(blkh, 'MaskType'), 'Audio Device Reader');

if isCalledFromModel
    % This function modifies settings of blocks in the subsystem. This
    % should not dirty the model.
    % Let openfcn and initfcn dirty the model if a device or driver has
    % been changed for the user, so that they can save it.
    dirtyFlag = get_param(model,'Dirty');
    c = onCleanup(@()set_param(model,'Dirty',dirtyFlag));
end

switch command
    
    case 'stopfcn'
        % Make sure refresh button is enabled
        MaskObject = get_param(blkh,'MaskObject');
        buttonObject = MaskObject.getDialogControl('RefreshButton');
        buttonObject.Enabled = 'on';
    
    case 'openfcn'
        % Executed on double-clicking the block

        %  a. If Device was changed during load time by Forwarding Table's
        % transformation function (audioDeviceIOTransform), then throw a
        % warning dialog
        oldDevice = get_param(blkh, 'DeviceWarnFlag');
        if ~strcmp(oldDevice, '''''')
            uiwait(warndlg(getString(message('dsp:audioDeviceIO:invalidDeviceValidDriver', ...
                oldDevice, get_param(blkh, 'Driver'))), ...
                getString(message('dsp:audioDeviceIO:invalidDevice_title')), 'modal'));
            set_param(blkh, 'DeviceWarnFlag', '''''');
        end
        
        %  b. Open the mask dialog
        open_system(blkh, 'mask');
        
    case 'initfcn'
        % Executed on beginning a simulation
        % These steps are similar to 'openfcn', only difference is that
        % here we throw a warning instead of a warndlg. Also, we do not
        % open the mask dialog.
        
        %  a. If Device was changed during load time by Forwarding Table's
        % transformation function (audioDeviceIOTransform), then throw a
        % warning dialog
        oldDevice = get_param(blkh, 'DeviceWarnFlag');
        if ~strcmp(oldDevice, '''''')
            noInputDeviceStr = getString(message('dsp:audioDeviceIO:noInputDevice'));
            noOutputDeviceStr = getString(message('dsp:audioDeviceIO:noOutputDevice'));
            if ~any(strcmp(oldDevice, {noInputDeviceStr, noOutputDeviceStr}))
                % Warn only when model was not saved with 'No audio ...
                % device'
                blockPath = strrep(getfullname(blkh), newline, ' ');
                blockType = get_param(blkh, 'MaskType');
                currDriver = get_param(blkh, 'Driver');
                warning(message('dsp:audioDeviceIO:invalidDevice_blockInit', ...
                    oldDevice, blockPath, blockType, currDriver));
            end
            set_param(blkh, 'DeviceWarnFlag', '''''');
        end
        
        %  b. Error out if no audio devices were found on the system
        if getpref('dsp','portaudioHostApi') ~= -1
            devNameStr = get_param(blkh, 'Device');
            noInputDeviceStr = getString(message('dsp:audioDeviceIO:noInputDevice'));
            noOutputDeviceStr = getString(message('dsp:audioDeviceIO:noOutputDevice'));
            if any(strcmp(devNameStr, {noInputDeviceStr, noOutputDeviceStr}))
                error(message('dsp:audioDeviceIO:noInputDevice'));
            end
        else
            % -1 should give silence irrespective of device
            set_param(blkh, 'DriverIdx', '-1');
        end
        
        % Disable the refresh button while it's running
        if isCalledFromModel
            MaskObject = get_param(blkh,'MaskObject');
            buttonObject = MaskObject.getDialogControl('RefreshButton');
            buttonObject.Enabled = 'off';
        end
        
    case 'loadfcn'
        % Executed on loading model or library containing the block
        
        if strcmp(get_param(model, 'Name'),'audiosinks')
            % audiosinks will load dspsnks4. Do not run this code twice.
            return;
        end
        maskStyles = get_param(blkh, 'MaskStyles');
        maskNames = get_param(blkh, 'MaskNames');
        maskVisibilities = get_param(blkh, 'MaskVisibilities');
        %  a. Populate the list of drivers. If it is a model, populate the
        %  list with only the drivers available on current platform. If it
        %  is a library, add all drivers to this list even if they may not
        %  be available on current platform. This is so that a model saved
        %  on a different platform does not throw any error.
        if isCalledFromModel
            availableDrivers = dsp.internal.getAudioIODriversAndDevices('drivers');
            [maskStyles, maskVisibilities] = setDriverList(availableDrivers, maskStyles, maskVisibilities, maskNames);
        else
            availableDrivers = dsp.internal.getAudioIODriversAndDevices('allDrivers');
            [maskStyles, maskVisibilities] = setDriverList(availableDrivers, maskStyles, maskVisibilities, maskNames);
            set_param(blkh, 'MaskStyles', maskStyles);
            %  b. If it is loaded from a library, set the driver to its default value for this platform.
            set_param(blkh, 'Driver', dsp.internal.getAudioIODriversAndDevices('defaultDriver'));
        end
        
        % c. Populate list of available devices
        if isInputDevice
            deviceText = 'inputDevices';
            noDevices = unique({ getString(message('dsp:audioDeviceIO:noInputDevice')), ...
                                 'No audio input device detected' });
        else
            deviceText = 'outputDevices';
            noDevices = unique({ getString(message('dsp:audioDeviceIO:noOutputDevice')), ...
                                 'No audio output device detected' });
        end
        if isCalledFromModel
            availableDevices = dsp.internal.getAudioIODriversAndDevices(deviceText, get_param(blkh, 'Driver'));
        else
            availableDevices = cell(0);
            for ii = 1:numel(availableDrivers)
                devs = dsp.internal.getAudioIODriversAndDevices(deviceText, availableDrivers{ii});
                if numel(devs) == 1 && any(strcmpi(noDevices,devs{1}))
                    continue % Do not add 'No audio X device detected' to the list
                end
                availableDevices(end+1:end+numel(devs)) = devs;
            end
            if isempty(availableDevices)
                availableDevices = noDevices(1);
            else
                % If there is a "Default" device, make it the first in the list
                if any(strcmp('Default',availableDevices))
                    availableDevices = ['Default',availableDevices];
                end
                availableDevices = unique(availableDevices,'stable');
            end
        end
        maskStyles = setDeviceList(availableDevices, maskStyles, maskNames);
        set_param(blkh, 'MaskStyles', maskStyles);
        set_param(blkh, 'MaskVisibilities', maskVisibilities);
        
    case 'maskinit'
        % Executed on mask init
        
        maskStyles = get_param(blkh, 'MaskStyles');
        maskNames = get_param(blkh, 'MaskNames');
        maskVisibilities = get_param(blkh, 'MaskVisibilities');
        %  a. Populate the list of drivers supported on current platform,
        %  if called from a model. Need to do this again because the list
        %  in library has all possible drivers, not just ones that are
        %  supported, so if the block is dragged from its library it will
        %  also show all possible drivers.
        if isCalledFromModel
            availableDrivers = dsp.internal.getAudioIODriversAndDevices('drivers');
            [maskStyles, maskVisibilities] = setDriverList(availableDrivers, maskStyles, maskVisibilities, maskNames);
            set_param(blkh, 'MaskStyles', maskStyles);
        end
        
        % b. Check if driver has changed. If yes, execute its callback code
        maskStyles = driverCallback(blkh, isInputDevice, maskStyles, maskNames);
        
        % c. Set visibility of sample rate edit box.
        if ~isInputDevice
            maskVisibilities = inheritSampleRateCallback(blkh, maskVisibilities, maskNames);
        end
        
        %  d. Set visibility of channel mapping edit box
        driverIdx = dsp.internal.getAudioIODriversAndDevices('driverIndex', get_param(blkh, 'Driver'));
        maskVisibilities = channelMappingCallback(blkh, isInputDevice, driverIdx, maskVisibilities, maskNames);
        
        set_param(blkh, 'MaskStyles', maskStyles, 'MaskVisibilities', maskVisibilities);
        
        updateInfoBox(blkh, isInputDevice);
        
    case 'deviceRefresh'
        % Executed on device list refresh
        
        % Clear cache of audio devices
        multimedia.internal.audio.device.DeviceInfo.reset;
        
        % Update Device List
        
        currentDriver = get_param(blkh, 'Driver');
        currentDevice = get_param(blkh, 'Device');

        %  Reset the list of available devices
        if isInputDevice
            availableDevices = dsp.internal.getAudioIODriversAndDevices('inputDevices', currentDriver);
        else
            availableDevices = dsp.internal.getAudioIODriversAndDevices('outputDevices', currentDriver);
        end

        maskStyles = get_param(blkh, 'MaskStyles');
        maskNames = get_param(blkh, 'MaskNames');
        maskStyles = setDeviceList(availableDevices, maskStyles, maskNames);
        set_param(blkh, 'MaskStyles', maskStyles);

        % Set selection to the same device as before
        % unless that device is no longer available.
        if ~contains(currentDevice, availableDevices)
            currentDevice = availableDevices{1};
        end
        set_param(blkh, 'Device', currentDevice);
        
    case 'deviceCallback'
        % Executed when device is changed
        updateInfoBox(blkh, isInputDevice);

    case 'driverCallback'
        % Executed when the Driver is changed
        maskStyles = get_param(blkh, 'MaskStyles');
        maskNames = get_param(blkh, 'MaskNames');
        maskStyles = driverCallback(blkh, isInputDevice, maskStyles, maskNames);
        set_param(blkh, 'MaskStyles', maskStyles);
        
    case 'inheritSampleRateCallback'
        % Executed when the inherit sample rate checkbox is changed
        
        maskNames = get_param(blkh, 'MaskNames');
        maskVisibilities = get_param(blkh, 'MaskVisibilities');
        
        % a. Set visibility of sample rate edit box
        if ~isInputDevice
            maskVisibilities = inheritSampleRateCallback(blkh, maskVisibilities, maskNames);
        end
        set_param(blkh, 'MaskVisibilities', maskVisibilities);
        
    case 'defaultChannelMappingCallback'
        % Executed when the default channel mapping checkbox is changed
        
        maskNames = get_param(blkh, 'MaskNames');
        maskVisibilities = get_param(blkh, 'MaskVisibilities');
        
        %  a. Set visibility of channel mapping edit box
        driverIdx = dsp.internal.getAudioIODriversAndDevices('driverIndex', get_param(blkh, 'Driver'));
        maskVisibilities = channelMappingCallback(blkh, isInputDevice, driverIdx, maskVisibilities, maskNames);
        
        set_param(blkh, 'MaskVisibilities', maskVisibilities);
        
    case 'infoButtonCallback'
        % Executed when info button is clicked
        
        if isInputDevice
            infoText = getInfoText_reader(blkh);
        else
            infoText = getInfoText_writer(blkh);
        end
        
        f = [];
        ud = get_param(blkh, 'UserData');
        if isfield(ud, 'InfoHandle')
            f = ud.InfoHandle;
        end
        if isempty(f) || ~ishandle(f)
            blockName = get_param(blkh,'Name');
            ud.InfoHandle = dsp.util.msgbox(infoText,blockName);
            set_param(blkh, 'UserData', ud);
        else
            adjustInfoBoxText(f, infoText);
            figure(f);
        end
        
    case 'deletefcn'
        % Executed when block is deleted
        
        f = [];
        ud = get_param(blkh, 'UserData');
        % Close Info dialog
        if isfield(ud, 'InfoHandle')
            f = ud.InfoHandle;
        end
        if ~isempty(f) && ishandle(f)
            close(f);
        end
        
    case 'copyfcn'
        % Executed when block is copied
        
        % Remove userdata so that info button dialog is detached from new
        % block
        set_param(blkh, 'UserData', []);
end

end

function infoText = getInfoText_reader(blkh)
% Info text for audio device reader

driverName = get_param(blkh, 'Driver');
driverIdx = dsp.internal.getAudioIODriversAndDevices('driverIndex', driverName);
devNameOnBlock = get_param(blkh, 'Device');
if strcmp(devNameOnBlock, 'Default')
    devInfoAll = dspAudioDeviceInfo('defaultInput',driverIdx);
    devName = regexprep(devInfoAll.name, ' \([^\(]*\)$', '');
    if strcmpi(devName, 'Default') && isunix
        % Sometimes a device named 'default' is listed on Linux. In
        % that case, it is renamed to ASLAdefault.
        devName = 'ALSAdefault';
    end
else
    devName = devNameOnBlock;
end
maximumInputChannels = computeMaxChannelsForDevice(devNameOnBlock, 'input', driverIdx);
spacePaddingStr = blanks(22);
driverStr = getString(message('dsp:audioDeviceIO:Driver_MP'));
deviceStr = getString(message('dsp:audioDeviceIO:Device_MP'));
maxChannelsStr = getString(message('dsp:audioDeviceIO:MaxInChannels'));
infoText = sprintf('%s%s %s\n%s%s %s\n%s %d\n', ...
    spacePaddingStr, driverStr, driverName, ...
    spacePaddingStr, deviceStr, devName, ...
    maxChannelsStr, maximumInputChannels);
end

function infoText = getInfoText_writer(blkh)
% Info text for audio device writer

driverName = get_param(blkh, 'Driver');
driverIdx = dsp.internal.getAudioIODriversAndDevices('driverIndex', driverName);
devNameOnBlock = get_param(blkh, 'Device');
if strcmp(devNameOnBlock, 'Default')
    devInfoAll = dspAudioDeviceInfo('defaultOutput',driverIdx);
    devName = regexprep(devInfoAll.name, ' \([^\(]*\)$', '');
    if strcmpi(devName, 'Default') && isunix
        % Sometimes a device named 'default' is listed on Linux. In
        % that case, it is renamed to ASLAdefault.
        devName = 'ALSAdefault';
    end
else
    devName = devNameOnBlock;
end
maximumOutputChannels = computeMaxChannelsForDevice(devNameOnBlock, 'output', driverIdx);
spacePaddingStr = blanks(23);
driverStr = getString(message('dsp:audioDeviceIO:Driver_MP'));
deviceStr = getString(message('dsp:audioDeviceIO:Device_MP'));
maxChannelsStr = getString(message('dsp:audioDeviceIO:MaxOutChannels'));
infoText = sprintf('%s%s %s\n%s%s %s\n%s %d\n', ...
    spacePaddingStr, driverStr, driverName, ...
    spacePaddingStr, deviceStr, devName, ...
    maxChannelsStr, maximumOutputChannels);
end

function adjustInfoBoxText(f, infoText)
% Adjust text in info box display

pos0 = get(f,'Position');
stringObject = findall(f,'Type','Text');
extent0 = get(stringObject,'Extent');
set(stringObject,'String',infoText);
extent1 = get(stringObject,'Extent');
set(f,'Position',[pos0(1:2) pos0(3:4) + (extent1(3:4) - extent0(3:4))]);
end

function updateInfoBox(blkh, isInputDevice)
% Update info box with new information

f = [];
ud = get_param(blkh, 'UserData');
if isfield(ud, 'InfoHandle')
    f = ud.InfoHandle;
end
if ~isempty(f) && ishandle(f)
    if isInputDevice
        infoText = getInfoText_reader(blkh);
    else
        infoText = getInfoText_writer(blkh);
    end
    adjustInfoBoxText(f, infoText);
end
end

function maskStyles = driverCallback(blkh, isInputDevice, maskStyles, maskNames)

%  a. Reset the list of available devices
currentDriver = get_param(blkh, 'Driver');
if isInputDevice
    availableDevices = dsp.internal.getAudioIODriversAndDevices('inputDevices', currentDriver);
else
    availableDevices = dsp.internal.getAudioIODriversAndDevices('outputDevices', currentDriver);
end

maskStyles = setDeviceList(availableDevices, maskStyles, maskNames);
set_param(blkh, 'MaskStyles', maskStyles);

oldDriverIdxStr = get_param(blkh, 'DriverIdx');
currDriverIdx = dsp.internal.getAudioIODriversAndDevices('driverIndex', currentDriver);
hasDriverChanged = ~strcmp(oldDriverIdxStr, num2str(currDriverIdx));
if hasDriverChanged    
    %  b. Compute driver index required by PortAudio.
    %     if driver index is -1, use that instead of current driver (override)
    if ~(strcmp(oldDriverIdxStr, '-1') && getpref('dsp','portaudioHostApi') == -1)
        driverIdx = dsp.internal.getAudioIODriversAndDevices('driverIndex', currentDriver);
        set_param(blkh, 'DriverIdx', num2str(driverIdx));
    end

    %  c. Reset the device to its default
    newDevice = availableDevices{1};
    set_param(blkh, 'Device', newDevice);
end
end

function maskVisibilities = inheritSampleRateCallback(blkh, maskVisibilities, maskNames)
sampleRateIdxOnMask = strcmp(maskNames,'SampleRate');
if strcmp(get_param(blkh, 'InheritSampleRate'), 'on')
    maskVisibilities{sampleRateIdxOnMask} = 'off';
else
    maskVisibilities{sampleRateIdxOnMask} = 'on';
end
end

function maskVisibilities = channelMappingCallback(blkh, isInputDevice, currentDriver, maskVisibilities, maskNames)
if isInputDevice
    ioName = 'input';
    mappingName = 'InputChannelMapping';
    currCheckStatus = get_param(blkh,'DefaultInputChannelMapping');
else
    ioName = 'output';
    mappingName = 'OutputChannelMapping';
    currCheckStatus = get_param(blkh,'DefaultOutputChannelMapping');
end
channelMappingIdxOnMask = strcmp(maskNames,mappingName);
if strcmp(currCheckStatus, 'on')
    maskVisibilities{channelMappingIdxOnMask} = 'off';
elseif strcmp(maskVisibilities{channelMappingIdxOnMask}, 'off')
    maskVisibilities{channelMappingIdxOnMask} = 'on';
    maxNumChannels = computeMaxChannelsForDevice(get_param(blkh,'Device'), ...
                                                 ioName, currentDriver);
    try % get the user's custom channel mapping
        chmap = get_param(blkh,mappingName);
        chmapev = eval(chmap); % an invalid mapping text might through an error
    catch % Use the default mapping if the text was invalid
        chmapev = [];
    end
    % Use the channel mapping if it is valid for the device, otherwise leave as is
    if isempty(chmapev) || ~isnumeric(chmapev) || any(chmapev<1) || any(chmapev>maxNumChannels)
        if (maxNumChannels == 0) || (maxNumChannels == 1)
            set_param(blkh,mappingName,sprintf('%d', maxNumChannels));
        else
            set_param(blkh,mappingName,sprintf('%d:%d', 1, maxNumChannels));
        end
    end
end
end

function maskStyles = setDeviceList(availableDevices, maskStyles, maskNames)
% Set list of available devices
availableDevicesStr = '';
for idx = 1:length(availableDevices)
    availableDevicesStr = [availableDevicesStr, availableDevices{idx}, '|']; %#ok
end

driverIdxOnMask = strcmp(maskNames,'Device');
maskStyles{driverIdxOnMask} = sprintf('popup(%s)', availableDevicesStr(1:end-1));
end

function [maskStyles, maskVisibilities] = setDriverList(availableDrivers, maskStyles, maskVisibilities, maskNames)
% Set list of available drivers
availableDriversStr = '';
for idx = 1:length(availableDrivers)
    availableDriversStr = [availableDriversStr, availableDrivers{idx}, '|']; %#ok
end

driverIdxOnMask = strcmp(maskNames,'Driver');
maskStyles{driverIdxOnMask} = sprintf('popup(%s)', availableDriversStr(1:end-1));
if ~ispc
    % Hide 'Driver' on non-windows platforms
    maskVisibilities{driverIdxOnMask} = 'off';
else
    maskVisibilities{driverIdxOnMask} = 'on';
end
end
