function dspLinkFVTool2Mask(hBlk,mode)
%dspLinkFVTool2Mask Callback for linking FVTool to a block mask.
%   dspLinkFVTool2Mask(block,mode) will launch a new FVTool or re-use an existing
%   FVTool from a block mask. The handle to FVTool will be stored in the
%   UserData of the block in a field of a structure called 'fvtool', e.g.,
%   ud.fvtool.
%
%   This function calls fvtool using a filter object assumed to be cached in
%   the UserData of the block as a field called 'filter', e.g. ud.filter.
%
%   This function must also be called from the 'init' portion of the mask
%   helper function so that FVTool will be updated based on changes to the
%   filter parameters.
%
%   Valid modes are:
%   'create': Launch a new FVTool (if one doesn't already exist for the block)
%   'update': Update the linked FVTool plot (do nothing if it's closed or doesn't exist)
%   'close':  Close the linked FVTool plot (do nothing if it's already closed)

%   Copyright 1988-2021 The MathWorks, Inc.

% Since this function may modify the UserData of the block however no other
% actual block settings may have changed, do not set the model 'Dirty' flag
% due to simply pressing the "View Filter Response" button. Instead, handle
% this by caching the model 'Dirty' setting and restoring at end.
rootModel = bdroot(hBlk);
origDirty = get_param(rootModel,'Dirty');

if nargin < 2
    mode = 'close';
end

filtobj = [];

ud = get_param(hBlk, 'UserData');

% Suppress MFILT deprecation warning
w = warning('off', 'dsp:mfilt:mfilt:Obsolete');
restoreWarn = onCleanup(@() warning(w));

if strcmpi(mode,'close')
    closeFVTool = 1;
else
    closeFVTool = 0;
    if isfield(ud,'filter')
        filtobj = ud.filter;
    else
        if ((isfield(ud,'filterConstructor') && ...
             isfield(ud,'filterConstructorArgs')) && ...
            ~isempty(ud.filterConstructorArgs))
            try
                if strcmpi(ud.filterConstructor,'mfilt.cicdecim') || ...
                        strcmpi(ud.filterConstructor,'dsp.internal.mfilt.cicinterp')
                    filtobj = createCICfiltObj(ud);
                else
                    filtobj = feval(ud.filterConstructor,...
                                ud.filterConstructorArgs{:});
                end
            catch
                filtobj = [];
            end
        end
    end
    if isempty(filtobj)
        error(message('dsp:dspLinkFVTool2Mask:invalidBlkInput'));
    end
end

% Get the Figure & FVTool handles
if dspIsFVToolOpen(hBlk)
    hFig = ud.fvtool;
    if closeFVTool
        close(hFig);
        ud.fvtool = [];
        % Cache UD back in the block, restore model 'Dirty' status
        set_param(hBlk, 'UserData', ud);
        set_param(rootModel, 'Dirty', origDirty);
    else
        hFVT = getappdata(hFig,'FVToolObjectHandle');
        % Adding/Replacing filters
        if strcmpi(get(hFVT, 'LinkMode'), 'replace'),
            hFVT.setfilter(filtobj);
        else
            hFVT.addfilter(filtobj);
        end
        % Bring fvtool to front
        figure(hFig);
    end
elseif strcmpi(mode,'create')
    hFVT = fvtoolwaddnreplace(filtobj);
    hFig = double(hFVT);
    setappdata(hFig, 'FVToolObjectHandle', hFVT);
    % Store the Figure Handle and not the handle to FVTool.
    ud.fvtool = hFig;
    % Cache UD back in the block, restore model 'Dirty' status
    set_param(hBlk, 'UserData', ud);
    set_param(rootModel, 'Dirty', origDirty);
end

% -------------------------------------------------------------------------
function filtobj = createCICfiltObj(ud)

filterInternals = ud.filterConstructorArgs{8};

% create mfilt object
filtobj = feval(ud.filterConstructor,ud.filterConstructorArgs{1:3});

switch filterInternals
    
    case 1  % Full precision
        % filtobj creation is complete
        
    case 2 % MinWordLengths
        filtobj.FilterInternals  = 'MinWordLengths';
        filtobj.OutputWordLength = ud.filterConstructorArgs{6};
        
    case 3 % SpecifyWordLengths
        filtobj.FilterInternals    = 'SpecifyWordLengths';
        filtobj.OutputWordLength   = ud.filterConstructorArgs{6};
        filtobj.SectionWordLengths = ud.filterConstructorArgs{4};
        
    case 4 % SpecifyPrecision
        filtobj.FilterInternals  = 'SpecifyPrecision';
        filtobj.OutputWordLength = ud.filterConstructorArgs{6};
        filtobj.OutputFracLength = ud.filterConstructorArgs{7};
        filtobj.SectionWordLengths = ud.filterConstructorArgs{4};
        filtobj.SectionFracLengths = ud.filterConstructorArgs{5};
end
