function varargout = dspblkbiquad3(action,blkh,varargin)
% DSPBLKBIQUAD Mask dynamic dialog function for IIR Biquad Filter block

% Copyright 2008-2018 The MathWorks, Inc.

if nargin > 1
    if strcmp(action, 'SEAinit')
        block = varargin{2};
    else
        block = get_param(blkh,'object');
    end
else
    blkh = []; block = [];
    if nargin==0, action = 'init'; end
end

switch action
    case 'icon'
        iconStruct   = genFilterIconStruct(block);
        varargout(1) = {iconStruct};
        
    case 'getDefaults'
        argsStruct   = getDefaultArgsStruct(block);
        dtInfo       = getDefaultDataTypeInfo();
        varargout(1) = {argsStruct};
        varargout(2) = {dtInfo};
        
    case 'init'
        argsStruct = getDefaultArgsStruct(block);
        
        if strcmp(block.FilterSource,'Filter object')
            hd = varargin{1};
            
            if isfield(block.UserData,'filterConstructor')
                block.UserData = rmfield(block.UserData,'filterConstructor');
                block.UserData = rmfield(block.UserData,'filterConstructorArgs');
            end
            if isfield(block.UserData,'ICData')
                block.UserData = rmfield(block.UserData,'ICData');
            end
            
            block.UserData.filter = [];
            
            if isempty(hd)
                % Allow empty/undefined filter object parameter at EDIT time
                simStatus = get_param(bdroot(blkh),'SimulationStatus');
                
                if ~(isequal(simStatus, 'updating') || ...
                        isequal(simStatus, 'compiled') || ...
                        isequal(simStatus, 'restarting') || ...
                        isequal(simStatus, 'running'))
                    
                    % Update FVTool plot window, if open
                    if dspIsFVToolOpen(blkh)
                        dspLinkFVTool2Mask(blkh,'update');
                    end
                    
                    % Form default return values
                    dtInfo = getDefaultDataTypeInfo();
                    varargout(1) = {argsStruct};
                    varargout(2) = {dtInfo};
                    return; % EARLY RETURN
                else
                    error(message('dsp:dspblkbiquad:dfiltNotDefined', ...
                        block.FilterObject));
                end
            end
            
            switch class(hd)
                case {  'dfilt.df1sos',  ...
                        'dfilt.df1tsos', ...
                        'dfilt.df2sos',  ...
                        'dfilt.df2tsos'  }
                    filterID = 1;

                case 'dsp.BiquadFilter'
                    if strcmp(hd.SOSMatrixSource,'Property')
                        filterID = 1;
                    else
                        filterID = -1; % invalid dsp.BiquadFilter
                    end

                otherwise
                    filterID = 0;
            end
            
            if filterID == 1
                % Valid filter
                if isa(hd,'dsp.BiquadFilter')
                    hd_clone = clone(hd);
                    release(hd_clone);
                    block.UserData.filter = hd_clone;
                    argsStruct.CoeffSource = 4; % System object
                else
                    block.UserData.filter = hd;
                    argsStruct.CoeffSource = 3; % DFILT
                end
                
                argsStruct = getFilterArgsFromObject(hd,argsStruct);
                dtInfo     = getFixptDataTypeInfoFromObject(block,hd);
                
            elseif filterID == -1
                % Invalid filter - System object with coeffs from port
                error(message('dsp:dspblkbiquad:unsupportedSOSFromPort'));
                
            else
                % Invalid filter - other unsupported filter structure
                error(message( ...
                    'dsp:dspblkbiquad:unsupportedFilterStructure', ...
                    class(hd)));
            end
            
        else
            % -------------------------------------
            % Coeffs from dialog or from input port
            % -------------------------------------
            paramValues = varargin{1};
            
            if isfield(block.UserData,'filter')
                block.UserData = rmfield(block.UserData,'filter');
            end
            
            argsStruct = ...
                genIIRArgsStruct(block.IIRFiltStruct, argsStruct, paramValues);
            
            dtInfo = getFixptDataTypeInfoFromBlock(block);
            
            populateFilterConstructorInfo(block, paramValues);
            
            block.UserData.ICData.ICnum = paramValues.ICnum;
            block.UserData.ICData.ICden = paramValues.ICden;
            block.UserData.ICData.IC    = paramValues.IC;
        end
        
        % Update FVTool plot window, if open
        if dspIsFVToolOpen(blkh)
            dspLinkFVTool2Mask(blkh,'update');
        end
        
        varargout(1) = {argsStruct};
        varargout(2) = {dtInfo};
        
    case 'dynamic'
        handleBlockMaskUI(blkh, block);
end

%--------------------------------------------------------------------------
%      FUNCTIONS TO GENERATE FILTER ICONS
%--------------------------------------------------------------------------
function iconStruct = genFilterIconStruct(block)

iconStruct.posx = 0.5;
iconStruct.posy = 0.5;
iconStruct.i1 = 1;  iconStruct.s1  = '';
iconStruct.i2 = 1;  iconStruct.s2  = '';
iconStruct.i3 = 1;  iconStruct.s3  = '';
iconStruct.in = ''; iconStruct.out = '';

iconStruct.icon = 'Biquad';

if strcmp(block.FilterSource,'Input port(s)')
    iconStruct.in = 'In'; iconStruct.out = 'Out';
    iconStruct.i1 = 2;    iconStruct.s1  = 'Num';
    iconStruct.i2 = 3;    iconStruct.s2  = 'Den';
    if strcmp(block.ScaleValueMode,'Specify via input port (g)')
        iconStruct.i3 = 4;    iconStruct.s3  = 'g';
    end
end

%--------------------------------------------------------------------------
%      FUNCTIONS TO GENERATE FILTER ARGUMENTS
%--------------------------------------------------------------------------
function dtInfo = getDefaultDataTypeInfo

dtInfo.arithmetic = 0;
dtInfo.inputWordLength  = 16;
dtInfo.inputFracLength  = 15;
dtInfo.secondCoeffFracLength = 15;
dtInfo.scaleValueFracLength  = 15;
dtInfo.stageInputMode  = 1; % Default: same as input
dtInfo.stageOutputMode = 1; % Default: same as section input
dtInfo.stageInputWordLength = 32;
dtInfo.stageInputFracLength = 31;
dtInfo.stageOutputWordLength = 32;
dtInfo.stageOutputFracLength = 31;
dtInfo.multiplicandMode = 1; % Default: same as output
dtInfo.multiplicandWordLength = 32;
dtInfo.multiplicandFracLength = 30;
dtInfo.denProdOutputFracLength = 30;
dtInfo.denAccumFracLength = 30;
dtInfo.denStateFracLength = 30;
dtInfo.firstCoeffMode = 2; % Default: same word length as input
dtInfo.firstCoeffWordLength = 16;
dtInfo.firstCoeffFracLength = 15;
dtInfo.prodOutputMode = 2; % Default: same as input
dtInfo.prodOutputWordLength = 32;
dtInfo.prodOutputFracLength = 30;
dtInfo.accumMode = 3; % Default: same as product output
dtInfo.accumWordLength = 32;
dtInfo.accumFracLength = 30;
dtInfo.memoryMode = 4; % Default: same as accumulator
dtInfo.memoryWordLength = 32;
dtInfo.memoryFracLength = 30;
dtInfo.outputMode = 4; % Default: same as accumulator
dtInfo.outputWordLength = 16;
dtInfo.outputFracLength = 15;
dtInfo.overflowMode = 1;
dtInfo.roundingMode = 3; % floor

%--------------------------------------------------------------------------
function argsStruct = getDefaultArgsStruct(block)
switch block.FilterSource
    case 'Dialog parameters'
        argsStruct.CoeffSource = 1;
    case 'Input port(s)'
        argsStruct.CoeffSource = 2;
    otherwise
        argsStruct.CoeffSource = 3;
end
argsStruct.FilterStruct  = 0;
argsStruct.SOSCoeffs     = [1 0.3 0.4 1 0.1 0.2];
argsStruct.ScaleValues   = 1;
argsStruct.OptimizeScaleValues = 1;
argsStruct.ScaleValueMode = 2;
argsStruct.IC = 0;
argsStruct.ICnum = 0;
argsStruct.ICden = 0;

%--------------------------------------------------------------------------
function dtInfo = getFixptDataTypeInfoFromBlock(block)
dtInfo = dspGetFixptDataTypeInfo([],63,block);

dtInfo.inputWordLength  = 16;
dtInfo.inputFracLength  = 15;
dtInfo.arithmetic = 0;

wsv = block.MaskWSVariables;

dtInfo.denAccumFracLength = ...
    double(dspGetEditBoxValFromMaskWSV(wsv,'denAccumFracLength'));

dtInfo.denProdOutputFracLength = ...
    double(dspGetEditBoxValFromMaskWSV(wsv,'denProdOutputFracLength'));

dtInfo.secondCoeffFracLength = ...
    double(dspGetEditBoxValFromMaskWSV(wsv,'secondCoeffFracLength'));

dtInfo.scaleValueFracLength = ...
    double(dspGetEditBoxValFromMaskWSV(wsv,'scaleValueFracLength'));

% used only for DF1T
dtInfo.denStateFracLength = ...
    double(dspGetEditBoxValFromMaskWSV(wsv,'secondMemoryFracLength'));

switch block.stageInputMode
  case 'Same as input'
    dtInfo.stageInputMode        = 1;
    dtInfo.stageInputWordLength  = 0;
    dtInfo.stageInputFracLength  = 0;
  otherwise
    % 'Binary point scaling', 'Slope and bias scaling'
    if strcmpi(block.stageInputMode,'Binary point scaling')
      dtInfo.stageInputMode = 2;
    else
      dtInfo.stageInputMode = 3;
    end
    dtInfo.stageInputWordLength  = ...
        double(dspGetEditBoxValFromMaskWSV(wsv,'stageInputWordLength'));
    dtInfo.stageInputFracLength  = ...
        double(dspGetEditBoxValFromMaskWSV(wsv,'stageInputFracLength'));
end

switch block.stageOutputMode
  case 'Same as section input'
    dtInfo.stageOutputMode        = 1;
    dtInfo.stageOutputWordLength  = 0;
    dtInfo.stageOutputFracLength  = 0;
  otherwise
    % 'Binary point scaling', 'Slope and bias scaling'
    if strcmpi(block.stageOutputMode,'Binary point scaling')
      dtInfo.stageOutputMode = 2;
    else
      dtInfo.stageOutputMode = 3;
    end
    dtInfo.stageOutputWordLength  = ...
        double(dspGetEditBoxValFromMaskWSV(wsv,'stageOutputWordLength'));
    dtInfo.stageOutputFracLength  = ...
        double(dspGetEditBoxValFromMaskWSV(wsv,'stageOutputFracLength'));
end

switch block.multiplicandMode
  case 'Same as output'
    dtInfo.multiplicandMode       = 1;
    dtInfo.multiplicandWordLength = 0;
    dtInfo.multiplicandFracLength = 0;
  otherwise
    % 'Binary point scaling', 'Slope and bias scaling'
    if strcmpi(block.multiplicandMode,'Binary point scaling')
      dtInfo.multiplicandMode = 2;
    else
      dtInfo.multiplicandMode = 3;
    end
    dtInfo.multiplicandWordLength = ...
        double(dspGetEditBoxValFromMaskWSV(wsv,'multiplicandWordLength'));
    dtInfo.multiplicandFracLength = ...
        double(dspGetEditBoxValFromMaskWSV(wsv,'multiplicandFracLength'));
end

%--------------------------------------------------------------------------
function argsStruct = genIIRArgsStruct(filtStruct, argsStruct, paramValues)

[~,warnEnum,errorEnum] = deal(1,2,3);

% Handle ICs
switch filtStruct
    case {'Direct form I', ...
            'Direct form I transposed'}
        argsStruct.ICnum = paramValues.ICnum;
        argsStruct.ICden = paramValues.ICden;
    otherwise
        argsStruct.IC = paramValues.IC;
end

% Setup FilterStruct values according to enum in sdspfilter2.cpp
switch filtStruct
    case {'Direct form I'}
        argsStruct.FilterStruct = 0;
    case {'Direct form I transposed'}
        argsStruct.FilterStruct = 1;
    case {'Direct form II'}
        argsStruct.FilterStruct = 2;
    otherwise
        % 'Direct form II transposed'
        argsStruct.FilterStruct = 3;
end

% Handle coeffs and other info
if (argsStruct.CoeffSource == 2)
    argsStruct.ScaleValueMode = paramValues.ScaleValueMode;
else
    argsStruct.SOSCoeffs   = paramValues.BiQuadCoeffs;
    argsStruct.ScaleValues = paramValues.ScaleValues;
    argsStruct.OptimizeScaleValues = paramValues.OptimizeScaleValues;
    
    [numSections,numCoeffs] = size(argsStruct.SOSCoeffs);
    if ( ~isempty(argsStruct.SOSCoeffs) && ...
            (numSections > 0) && (numCoeffs == 6) )
        if (any(argsStruct.SOSCoeffs(:,4) ~= 0))
            if (any(argsStruct.SOSCoeffs(:,4) ~= 1))
                if (paramValues.a0Flag == errorEnum)
                    error(message('dsp:dspblkbiquad:a0NotEqualOneError'));
                elseif (paramValues.a0Flag == warnEnum)
                    warning(message('dsp:dspblkbiquad:a0NotEqualOneWarn'));
                end
            end
        end
    end
    
end

%--------------------------------------------------------------------------
function populateFilterConstructorInfo(block,params)
% This function populates the .filterConstructor and .filterConstructorArgs
% fields of block.UserData for use by the FVTool button to construct a
% filter object for analysis. Clicking the FVTool button causes the
% function dspLinkFVTool2Mask.m to be called, which uses these two fields
% to construct a filter object at that time.
switch block.IIRFiltStruct
    case 'Direct form I'
        block.UserData.filterConstructor = 'dfilt.df1sos';
        block.UserData.filterConstructorArgs = ...
            {params.BiQuadCoeffs, ...
            params.ScaleValues};
        
    case 'Direct form I transposed'
        block.UserData.filterConstructor = 'dfilt.df1tsos';
        block.UserData.filterConstructorArgs = ...
            {params.BiQuadCoeffs, ...
            params.ScaleValues};
        
    case 'Direct form II'
        block.UserData.filterConstructor = 'dfilt.df2sos';
        block.UserData.filterConstructorArgs = ...
            {params.BiQuadCoeffs, ...
            params.ScaleValues};
        
    otherwise
        % 'Direct form II transposed'
        block.UserData.filterConstructor = 'dfilt.df2tsos';
        block.UserData.filterConstructorArgs = ...
            {params.BiQuadCoeffs, ...
            params.ScaleValues};
end

%--------------------------------------------------------------------------
function dtInfo = getFixptDataTypeInfoFromObject(block,hd)
% Initialize all to defaults
if isa(hd,'dsp.BiquadFilter')
    dtInfo = getFixptDataTypeInfoFromSysObj(hd);
else
    dtInfo = getFixptDataTypeInfoFromDFILT(block,hd);
end

%--------------------------------------------------------------------------
function dtInfo = getFixptDataTypeInfoFromDFILT(block,hd)
% Initialize all to defaults
dtInfo = getDefaultDataTypeInfo();
if isfdtbxinstalled
    switch hd.arithmetic
        case 'double'
            dtInfo.arithmetic = 0;
        case 'single'
            dtInfo.arithmetic = 1;
        case 'fixed'
            dtInfo.arithmetic = 2;
            
            if strcmpi(hd.OverflowMode, 'wrap')
                dtInfo.overflowMode = 1;
            else
                dtInfo.overflowMode = 2;
            end
            
            dfiltName = block.FilterObject;
            if strcmpi(hd.RoundMode, 'ceil')
                dtInfo.roundingMode = 1;
            elseif strcmpi(hd.RoundMode, 'convergent')
                dtInfo.roundingMode = 2;
            elseif strcmpi(hd.RoundMode, 'floor')
                dtInfo.roundingMode = 3;
            elseif strcmpi(hd.RoundMode, 'nearest')
                dtInfo.roundingMode = 4;
            elseif strcmpi(hd.RoundMode, 'round')
                dtInfo.roundingMode = 5;
            elseif strcmpi(hd.RoundMode, 'fix')
                dtInfo.roundingMode = 7;    %zero
            end
            
            if ~hd.Signed
                error(message('dsp:dspblkbiquad:noUnsignedSupport', dfiltName));
            end
            
            if isprop(hd,'CastBeforeSum') && ~hd.CastBeforeSum
                warning(message('dsp:dspblkbiquad:unsupportedCastAfterSum'));
            end
            
            dtInfo.stageInputMode   = 2; % Binary point scaling
            dtInfo.stageOutputMode  = 2; % Binary point scaling
            dtInfo.firstCoeffMode   = 0; % Binary point scaling
            dtInfo.prodOutputMode   = 0; % Binary point scaling
            dtInfo.accumMode        = 0; % Binary point scaling
            dtInfo.memoryMode       = 0; % Binary point scaling
            dtInfo.outputMode       = 0; % Binary point scaling
            dtInfo.multiplicandMode = 2; % Binary point scaling
            
            dtInfo.inputWordLength  = hd.InputWordLength;
            dtInfo.inputFracLength  = hd.InputFracLength;
            
            dtInfo.outputWordLength = hd.OutputWordLength;
            dtInfo.outputFracLength = hd.OutputFracLength;
            
            dtInfo.firstCoeffWordLength = hd.CoeffWordLength;
            dtInfo.prodOutputWordLength = hd.ProductWordLength;
            dtInfo.accumWordLength = hd.AccumWordLength;
            
            dtInfo.firstCoeffFracLength  = hd.NumFracLength;
            dtInfo.secondCoeffFracLength = hd.DenFracLength;
            dtInfo.prodOutputFracLength    = hd.NumProdFracLength;
            dtInfo.denProdOutputFracLength = hd.DenProdFracLength;
            dtInfo.accumFracLength    = hd.NumAccumFracLength;
            dtInfo.denAccumFracLength = hd.DenAccumFracLength;
            dtInfo.scaleValueFracLength = hd.ScaleValueFracLength;
            
            switch class(hd)
              case 'dfilt.df1sos'
                dtInfo.stageInputWordLength  = hd.NumStateWordLength;
                dtInfo.stageInputFracLength  = hd.NumStateFracLength;
                dtInfo.stageOutputWordLength = hd.DenStateWordLength;
                dtInfo.stageOutputFracLength = hd.DenStateFracLength;
                
              case 'dfilt.df1tsos'
                dtInfo.stageInputWordLength  = hd.SectionInputWordLength;
                dtInfo.stageInputFracLength  = hd.SectionInputFracLength;
                dtInfo.stageOutputWordLength = hd.SectionOutputWordLength;
                dtInfo.stageOutputFracLength = hd.SectionOutputFracLength;
                
                dtInfo.memoryWordLength   = hd.StateWordLength;
                dtInfo.memoryFracLength   = hd.NumStateFracLength;
                dtInfo.denStateFracLength = hd.DenStateFracLength;
                
                dtInfo.multiplicandWordLength = hd.MultiplicandWordLength;
                dtInfo.multiplicandFracLength = hd.MultiplicandFracLength;
                
              case {'dfilt.df2sos', 'dfilt.df2tsos'}
                dtInfo.stageInputWordLength  = hd.SectionInputWordLength;
                dtInfo.stageInputFracLength  = hd.SectionInputFracLength;
                dtInfo.stageOutputWordLength = hd.SectionOutputWordLength;
                dtInfo.stageOutputFracLength = hd.SectionOutputFracLength;
                
                dtInfo.memoryWordLength = hd.StateWordLength;
                dtInfo.memoryFracLength = hd.StateFracLength;
            end
            
        otherwise
            error(message('dsp:dspblkbiquad:unsupportedDfiltArith', hd.arithmetic));
    end
end

%--------------------------------------------------------------------------
function dtInfo = getFixptDataTypeInfoFromSysObj(hd)
% Initialize to defaults
dtInfo = getDefaultDataTypeInfo();
dtInfo.arithmetic = 0;
if isfdtbxinstalled
    dtInfoFixpt = hd.getFixedPointInfo;
    dtInfo.secondCoeffFracLength = dtInfoFixpt.DenominatorCoefficientsFracLength;
    dtInfo.scaleValueFracLength = dtInfoFixpt.ScaleValuesFracLength;
    dtInfo. stageInputMode = dtInfoFixpt.SectionInputDataType;
    dtInfo.stageOutputMode = dtInfoFixpt.SectionOutputDataType;
    dtInfo.stageInputWordLength = dtInfoFixpt.SectionInputWordLength;
    dtInfo.stageInputFracLength = dtInfoFixpt.SectionInputFracLength;
    dtInfo.stageOutputWordLength = dtInfoFixpt.SectionOutputWordLength;
    dtInfo.stageOutputFracLength = dtInfoFixpt.SectionOutputFracLength;
    dtInfo.multiplicandMode = dtInfoFixpt.MultiplicandDataType;
    dtInfo.multiplicandWordLength = dtInfoFixpt.MultiplicandWordLength;
    dtInfo.multiplicandFracLength = dtInfoFixpt.MultiplicandFracLength;
    dtInfo.denProdOutputFracLength = dtInfoFixpt.DenominatorProductFracLength;
    dtInfo.denAccumFracLength = dtInfoFixpt.DenominatorAccumulatorFracLength;
    dtInfo.denStateFracLength = dtInfoFixpt.DenominatorStateFracLength;
    dtInfo.firstCoeffMode = dtInfoFixpt.NumeratorCoefficientsDataType;
    dtInfo.firstCoeffWordLength = dtInfoFixpt.NumeratorCoefficientsWordLength;
    dtInfo.firstCoeffFracLength = dtInfoFixpt.NumeratorCoefficientsFracLength;
    dtInfo.prodOutputMode = dtInfoFixpt.NumeratorProductDataType;
    dtInfo.prodOutputWordLength = dtInfoFixpt.NumeratorProductWordLength;
    dtInfo.prodOutputFracLength = dtInfoFixpt.NumeratorProductFracLength;
    dtInfo.accumMode = dtInfoFixpt.NumeratorAccumulatorDataType;
    dtInfo.accumWordLength = dtInfoFixpt.NumeratorAccumulatorWordLength;
    dtInfo.accumFracLength = dtInfoFixpt.NumeratorAccumulatorFracLength;
    dtInfo.memoryMode = dtInfoFixpt.NumeratorStateDataType;
    dtInfo.memoryWordLength = dtInfoFixpt.NumeratorStateWordLength;
    dtInfo.memoryFracLength = dtInfoFixpt.NumeratorStateFracLength;
    dtInfo.outputMode = dtInfoFixpt.OutputDataType;
    dtInfo.outputWordLength = dtInfoFixpt.OutputWordLength;
    dtInfo.outputFracLength = dtInfoFixpt.OutputFracLength;
    dtInfo.overflowMode = dtInfoFixpt.OverflowAction;
    dtInfo.roundingMode = dtInfoFixpt.RoundingMethod;
end

%--------------------------------------------------------------------------
function argsStruct = getFilterArgsFromObject(hdx,argsStruct)
isdfilt = ~isa(hdx,'dsp.BiquadFilter');
if isdfilt
    argsStruct = getFilterArgsFromDFILT(hdx,argsStruct);
else
    argsStruct = getFilterArgsFromSysObj(hdx,argsStruct);
end

%--------------------------------------------------------------------------
function argsStruct = getFilterArgsFromDFILT(hdx,argsStruct)

if isfdtbxinstalled
    hd = reffilter(hdx);
else
    hd = hdx;
end

SOSMatrix = double(hd.sosMatrix);
argsStruct.ScaleValues = double(hd.ScaleValues);
argsStruct.OptimizeScaleValues = double(hdx.OptimizeScaleValues);
argsStruct.SOSCoeffs = SOSMatrix;

hdclass = class(hd);

switch hdclass
    case 'dfilt.df1sos'
        argsStruct.FilterStruct = 0;
    case 'dfilt.df1tsos'
        argsStruct.FilterStruct = 1;
    case 'dfilt.df2sos'
        argsStruct.FilterStruct = 2;
    otherwise
        % 'dfilt.df2tsos'
        argsStruct.FilterStruct = 3;
end

switch hdclass
    case {'dfilt.df1sos', 'dfilt.df1tsos'}
        if hdx.PersistentMemory
            numStates = double(hdx.States.Numerator);
            denStates = double(hdx.States.Denominator);
            if isempty(numStates)
                numStates = 0;
            end
            if isempty(denStates)
                denStates = 0;
            end
            argsStruct.ICnum = numStates;
            argsStruct.ICden = denStates;
        end
        
    otherwise
        % 'dfilt.df2sos', 'dfilt.df2tsos'
        argsStruct.IC = 0;
        if hdx.PersistentMemory
            argsStruct.IC = double(hdx.States);
            if isempty(argsStruct.IC)
                argsStruct.IC = 0;
            end
        end
end

%--------------------------------------------------------------------------
function argsStruct = getFilterArgsFromSysObj(hdx,argsStruct)

Structure = hdx.Structure;

SOSMatrix = double(hdx.SOSMatrix);
argsStruct.ScaleValues = double(hdx.ScaleValues);
argsStruct.OptimizeScaleValues = double(hdx.OptimizeUnityScaleValues);
argsStruct.SOSCoeffs = SOSMatrix;

switch Structure
    case 'Direct form I'
        argsStruct.FilterStruct = 0;
    case 'Direct form I transposed'
        argsStruct.FilterStruct = 1;
    case 'Direct form II'
        argsStruct.FilterStruct = 2;
    otherwise
        % 'Direct form II transposed'
        argsStruct.FilterStruct = 3;
end

switch Structure
    case {'Direct form I', 'Direct form I transposed'}
        argsStruct.ICnum =  double(hdx.NumeratorInitialConditions);
        argsStruct.ICden =  double(hdx.DenominatorInitialConditions);
    otherwise
        %  'Direct form II', 'Direct form II transposed'
        argsStruct.IC = double(hdx.InitialConditions);
end

%--------------------------------------------------------------------------
function handleBlockMaskUI(blkh, blockObj)

% 'Main' tab Parameters
% =====================
blockMaskObj = Simulink.Mask.get(blkh);
filtObjParam = blockMaskObj.getParameter('FilterObject');
fltStrcParam = blockMaskObj.getParameter('IIRFiltStruct');
sosCoefParam = blockMaskObj.getParameter('BiQuadCoeffs');
sclVlMdParam = blockMaskObj.getParameter('ScaleValueMode');
sclValsParam = blockMaskObj.getParameter('ScaleValues');
initCndParam = blockMaskObj.getParameter('IC');
numerICParam = blockMaskObj.getParameter('ICnum');
denomICParam = blockMaskObj.getParameter('ICden');
a0_flagParam = blockMaskObj.getParameter('a0Flag');
optScVlParam = blockMaskObj.getParameter('optimizeScaleValues');

% 'Data Types' tab Parameters
% ===========================
stgInpMdParam = blockMaskObj.getParameter('stageInputMode');
stgInpWLParam = blockMaskObj.getParameter('stageInputWordLength');
stgInpFLParam = blockMaskObj.getParameter('stageInputFracLength');
stgOutMdParam = blockMaskObj.getParameter('stageOutputMode');
stgOutWLParam = blockMaskObj.getParameter('stageOutputWordLength');
stgOutFLParam = blockMaskObj.getParameter('stageOutputFracLength');
mpcndMdParam  = blockMaskObj.getParameter('multiplicandMode');
mpcndWLParam  = blockMaskObj.getParameter('multiplicandWordLength');
mpcndFLParam  = blockMaskObj.getParameter('multiplicandFracLength');
frstCfMdParam = blockMaskObj.getParameter('firstCoeffMode');
frstCfWLParam = blockMaskObj.getParameter('firstCoeffWordLength');
frstCfFLParam = blockMaskObj.getParameter('firstCoeffFracLength');
scndCfFLParam = blockMaskObj.getParameter('secondCoeffFracLength');
sclValFLParam = blockMaskObj.getParameter('scaleValueFracLength');
numPrdMdParam = blockMaskObj.getParameter('prodOutputMode');
numPrdWLParam = blockMaskObj.getParameter('prodOutputWordLength');
numPrdFLParam = blockMaskObj.getParameter('prodOutputFracLength');
denPrdFLParam = blockMaskObj.getParameter('denProdOutputFracLength');
numAccMdParam = blockMaskObj.getParameter('accumMode');
numAccWLParam = blockMaskObj.getParameter('accumWordLength');
numAccFLParam = blockMaskObj.getParameter('accumFracLength');
denAccFLParam = blockMaskObj.getParameter('denAccumFracLength');
numICsMdParam = blockMaskObj.getParameter('memoryMode');
numICsWLParam = blockMaskObj.getParameter('memoryWordLength');
numICsFLParam = blockMaskObj.getParameter('memoryFracLength');
denICsFLParam = blockMaskObj.getParameter('secondMemoryFracLength');
outputMdParam = blockMaskObj.getParameter('outputMode');
outputWLParam = blockMaskObj.getParameter('outputWordLength');
outputFLParam = blockMaskObj.getParameter('outputFracLength');

% Dialog Control UI PROPERTIES (buttons, text strings, etc.)
% ==========================================================
ViewFiltResp   = blockMaskObj.getDialogControl('ViewFilterResponseButton');
DataTypesTab   = blockMaskObj.getDialogControl('DataTypesTab');
FxPtOpGroupBox = blockMaskObj.getDialogControl('FixPtOpParamsGroupBox');
StIOMpGroupBox = blockMaskObj.getDialogControl('stageIOandMultGroupBox');
CfScVlGroupBox = blockMaskObj.getDialogControl('coeffGroupBox');
PrdOutGroupBox = blockMaskObj.getDialogControl('prodOutputGroupBox');
AccumGroupBox  = blockMaskObj.getDialogControl('accumGroupBox');
StatesGroupBox = blockMaskObj.getDialogControl('statesGroupBox');
OutputGroupBox = blockMaskObj.getDialogControl('outputGroupBox');
MpcndTextLbl   = blockMaskObj.getDialogControl('multiplicandLabel');
WLColLblStIOMp = blockMaskObj.getDialogControl('WordLengthColumnLabel');
FLColLblStIOMp = blockMaskObj.getDialogControl('FractionLengthColumnLabel');
WLColLblCofScV = blockMaskObj.getDialogControl('coeffWordLengthLabel');
FLColLblCofScV = blockMaskObj.getDialogControl('coeffFracLengthLabel');
WLColLblPrdOut = blockMaskObj.getDialogControl('prodOutputWordLengthLabel');
FLColLblPrdOut = blockMaskObj.getDialogControl('prodOutputFracLengthLabel');
WLColLblAccum  = blockMaskObj.getDialogControl('accumWordLengthLabel');
FLColLblAccum  = blockMaskObj.getDialogControl('accumFracLengthLabel');
WLColLblStates = blockMaskObj.getDialogControl('statesWordLengthLabel');
FLColLblStates = blockMaskObj.getDialogControl('statesFracLengthLabel');
WLColLblOutput = blockMaskObj.getDialogControl('outputWordLengthColumnLabel');
FLColLblOutput = blockMaskObj.getDialogControl('outputFractionLengthColumnLabel');

if strcmp(blockObj.FilterSource, 'Filter object')
    % ---------------------------------------------
    % SHOW in Parameters group box on 'Main' tab:
    %   (1) Filter object edit box
    %   (2) Input processing drop-down  (no change)
    %   (3) View Filter Response button (no change)
    % ---------------------------------------------
    filtObjParam.Visible = 'on';
    ViewFiltResp.Visible = 'on';
    ViewFiltResp.Enabled = 'on';
    
    % ----------------------------------------------------------
    % HIDE everything else in Parameters group box on 'Main' tab
    % ----------------------------------------------------------
    fltStrcParam.Visible = 'off';
    sosCoefParam.Visible = 'off';
    sclVlMdParam.Visible = 'off';
    sclValsParam.Visible = 'off';
    a0_flagParam.Visible = 'off';
    optScVlParam.Visible = 'off';
    initCndParam.Visible = 'off';
    numerICParam.Visible = 'off';
    denomICParam.Visible = 'off';
    
    % ----------------------------------
    % HIDE entire 'Data Types' tab (N/A)
    % ----------------------------------
    MpcndTextLbl.Visible   = 'off';
    WLColLblStIOMp.Visible = 'off';
    FLColLblStIOMp.Visible = 'off';
    WLColLblCofScV.Visible = 'off';
    FLColLblCofScV.Visible = 'off';
    WLColLblPrdOut.Visible = 'off';
    FLColLblPrdOut.Visible = 'off';
    WLColLblAccum.Visible  = 'off';
    FLColLblAccum.Visible  = 'off';
    WLColLblStates.Visible = 'off';
    FLColLblStates.Visible = 'off';
    WLColLblOutput.Visible = 'off';
    FLColLblOutput.Visible = 'off';
    
    FxPtOpGroupBox.Visible = 'off';
    StIOMpGroupBox.Visible = 'off';
    CfScVlGroupBox.Visible = 'off';
    PrdOutGroupBox.Visible = 'off';
    AccumGroupBox.Visible  = 'off';
    StatesGroupBox.Visible = 'off';
    OutputGroupBox.Visible = 'off';
    DataTypesTab.Visible   = 'off';
    
else
    % =============================================================
    % BIQUAD filter coefficients specified from dialog or from port
    % and the rest of the specification is in block mask parameters
    % =============================================================
    filtObjParam.Visible = 'off';
    fltStrcParam.Visible = 'on';
    
    % SHOW or HIDE the SOSCOEFFS and SCALEVALUES related parameters
    showCoeffsAndScaleValues = ...
        strcmp(blockObj.FilterSource, 'Dialog parameters');
    if showCoeffsAndScaleValues
        sosCoefParam.Visible = 'on';
        sclVlMdParam.Visible = 'off';
        sclValsParam.Visible = 'on';
        a0_flagParam.Visible = 'on';
        optScVlParam.Visible = 'on';
        ViewFiltResp.Visible = 'on';
        ViewFiltResp.Enabled = 'on';
    else
        % HIDE the SOSCOEFFS and SCALEVALUES related parameters
        sosCoefParam.Visible = 'off';
        sclVlMdParam.Visible = 'on';
        sclValsParam.Visible = 'off';
        a0_flagParam.Visible = 'off';
        optScVlParam.Visible = 'off';
        % HIDE and DISABLE the FVTOOL button
        % (input port parameters inaccessible for mask dialog UI)
        ViewFiltResp.Visible = 'off';
        ViewFiltResp.Enabled = 'off';
    end
    
    % SHOW or HIDE other 'Main' tab parameters
    isDF1T = strcmp(blockObj.IIRFiltStruct,'Direct form I transposed');
    isDF1  = strcmp(blockObj.IIRFiltStruct,'Direct form I');
    if isDF1 || isDF1T
        INITCND_DLG_Vis = 'off';
        ICNUM_ICDEN_Vis = 'on';
        DTNUM_DTDEN_Vis = 'on';
    else
        INITCND_DLG_Vis = 'on';
        ICNUM_ICDEN_Vis = 'off';
        DTNUM_DTDEN_Vis = 'off';
    end
    initCndParam.Visible = INITCND_DLG_Vis;
    numerICParam.Visible = ICNUM_ICDEN_Vis;
    denomICParam.Visible = ICNUM_ICDEN_Vis;
    
    % -----------------------------------------------------------------
    % Make 'Data Types' tab VISIBLE and show or hide individual widgets
    % -----------------------------------------------------------------
    
    DataTypesTab.Visible = 'on';
    FxPtOpGroupBox.Visible = 'on';
    
    % COEFFICIENTS AND SCALE VALUES:
    %   Data Types tab 'Mode', 'Word length' and 'Fraction length' params
    if showCoeffsAndScaleValues
        % COEFFS AND SCALE VALUES TEXT PROPERTIES VISIBLE
        % COEFFS AND SCALE VALUES MASK PARAMETERS VISIBLE
        CfScVlGroupBox.Visible  = 'on';
        frstCfMdParam.Visible = 'on';
        if strcmp(blockObj.firstCoeffMode, 'Specify word length')
            % WL is 'on'. All FLs remain 'off' (best precision FLs).
            WLColLblCofScV.Visible = 'on';  % Specified (shown)
            FLColLblCofScV.Visible = 'off'; % Inherited (NOT shown)
            frstCfWLParam.Visible  = 'on';  % Specified (shown)
            frstCfFLParam.Visible  = 'off'; % Inherited (NOT shown)
            scndCfFLParam.Visible  = 'off'; % Inherited (NOT shown)
            sclValFLParam.Visible  = 'off'; % Inherited (NOT shown)
        elseif strcmp(blockObj.firstCoeffMode, 'Binary point scaling')
            WLColLblCofScV.Visible = 'on';  % Specified (shown)
            FLColLblCofScV.Visible = 'on';  % Specified (shown)
            frstCfWLParam.Visible  = 'on';  % Specified (shown)
            frstCfFLParam.Visible  = 'on';  % Specified (shown)
            scndCfFLParam.Visible  = 'on';  % Specified (shown)
            sclValFLParam.Visible  = 'on';  % Specified (shown)
        else
            % WLs and FLs inherited
            WLColLblCofScV.Visible = 'off'; % N/A
            FLColLblCofScV.Visible = 'off'; % N/A
            frstCfWLParam.Visible  = 'off'; % N/A
            frstCfFLParam.Visible  = 'off'; % N/A
            scndCfFLParam.Visible  = 'off'; % N/A
            sclValFLParam.Visible  = 'off'; % N/A
        end
    else
        % COEFFS AND SCALE VALUES TEXT PROPERTIES INVISIBLE
        % COEFFS AND SCALE VALUES MASK PARAMETERS INVISIBLE
        % Hide ENTIRE "Coefficients, Scale Values" group box
        WLColLblCofScV.Visible = 'off';
        FLColLblCofScV.Visible = 'off';
        frstCfWLParam.Visible  = 'off';
        frstCfFLParam.Visible  = 'off';
        scndCfFLParam.Visible  = 'off';
        sclValFLParam.Visible  = 'off';
        CfScVlGroupBox.Visible   = 'off';
    end

    % ---------------------------------------
    % STAGE INPUT, STAGE OUTPUT, MULTIPLICAND
    % ---------------------------------------
    
    StIOMpGroupBox.Visible = 'on';
    showStgMpcdWLFLLabels  = false; % Default (hide WL/FL labels)
    
    stgInpMdParam.Visible  = 'on';
    if strcmp(blockObj.stageInputMode, 'Binary point scaling')
        showStgMpcdWLFLLabels = true;
        stgInpWLParam.Visible = 'on';
        stgInpFLParam.Visible = 'on';
    else
        % WLs and FLs inherited
        stgInpWLParam.Visible = 'off';
        stgInpFLParam.Visible = 'off';
    end
    
    stgOutMdParam.Visible = 'on';
    if strcmp(blockObj.stageOutputMode, 'Binary point scaling')
        showStgMpcdWLFLLabels = true;
        stgOutWLParam.Visible = 'on';
        stgOutFLParam.Visible = 'on';
    else
        % WLs and FLs inherited
        stgOutWLParam.Visible = 'off';
        stgOutFLParam.Visible = 'off';
    end
    
    if isDF1T
        % MULTIPLICAND TEXT LABEL PROPERTY and PARAMETERS VISIBLE
        MpcndTextLbl.Visible = 'on';
        mpcndMdParam.Visible = 'on';
        if strcmp(blockObj.multiplicandMode, 'Binary point scaling')
            showStgMpcdWLFLLabels = true;
            mpcndWLParam.Visible  = 'on';
            mpcndFLParam.Visible  = 'on';
        else
            % WLs and FLs inherited
            mpcndWLParam.Visible = 'off';
            mpcndFLParam.Visible = 'off';
        end
    else
        % MULTIPLICAND TEXT LABEL PROPERTY and PARAMETERS INVISIBLE
        MpcndTextLbl.Visible = 'off';
        mpcndMdParam.Visible = 'off';
        mpcndWLParam.Visible = 'off';
        mpcndFLParam.Visible = 'off';
    end

    % UI widgets 'showStageMpcndWLLabel' and 'showStageMpcndFLLabel'
    % depend on multiple logical computations determined above, for
    % both the StageIO and Multiplicand data type mode settings:
    if showStgMpcdWLFLLabels
        WLColLblStIOMp.Visible = 'on';
        FLColLblStIOMp.Visible = 'on';
    else
        % WLs and FLs inherited
        WLColLblStIOMp.Visible = 'off';
        FLColLblStIOMp.Visible = 'off';
    end
    
    % PRODUCT OUTPUT(S)
    PrdOutGroupBox.Visible = 'on';
    numPrdMdParam.Visible  = 'on';
    if strcmp(blockObj.prodOutputMode, 'Binary point scaling')
        WLColLblPrdOut.Visible = 'on';
        FLColLblPrdOut.Visible = 'on';
        numPrdWLParam.Visible  = 'on';
        numPrdFLParam.Visible  = 'on';
        denPrdFLParam.Visible  = DTNUM_DTDEN_Vis;
    else
        % WLs and FLs inherited
        WLColLblPrdOut.Visible = 'off';
        FLColLblPrdOut.Visible = 'off';
        numPrdWLParam.Visible  = 'off';
        numPrdFLParam.Visible  = 'off';
        denPrdFLParam.Visible  = 'off';
    end
    
    % ACCUMULATOR(S)
    AccumGroupBox.Visible = 'on';
    numAccMdParam.Visible = 'on';
    if strcmp(blockObj.accumMode, 'Binary point scaling')
        WLColLblAccum.Visible = 'on';
        FLColLblAccum.Visible = 'on';
        numAccWLParam.Visible = 'on';
        numAccFLParam.Visible = 'on';
        denAccFLParam.Visible = DTNUM_DTDEN_Vis;
    else
        % WLs and FLs inherited
        WLColLblAccum.Visible = 'off';
        FLColLblAccum.Visible = 'off';
        numAccWLParam.Visible = 'off';
        numAccFLParam.Visible = 'off';
        denAccFLParam.Visible = 'off';
    end
    
    % STATE DATA TYPES (all except DF1 structure)
    if isDF1
        WLColLblStates.Visible = 'off';
        FLColLblStates.Visible = 'off';
        numICsMdParam.Visible  = 'off';
        numICsWLParam.Visible  = 'off';
        numICsFLParam.Visible  = 'off';
        denICsFLParam.Visible  = 'off';
        StatesGroupBox.Visible = 'off';
    else
        % Show ICs DT mode (and possibly other widgets)
        StatesGroupBox.Visible = 'on';
        numICsMdParam.Visible  = 'on';
        if strcmp(blockObj.memoryMode, 'Binary point scaling')
            WLColLblStates.Visible = 'on';
            FLColLblStates.Visible = 'on';
            numICsWLParam.Visible  = 'on';
            numICsFLParam.Visible  = 'on';
            denICsFLParam.Visible  = DTNUM_DTDEN_Vis;
        else
            % WLs and FLs inherited
            WLColLblStates.Visible = 'off';
            FLColLblStates.Visible = 'off';
            numICsWLParam.Visible  = 'off';
            numICsFLParam.Visible  = 'off';
            denICsFLParam.Visible  = 'off';
        end
    end
    
    % OUTPUT
    OutputGroupBox.Visible = 'on';
    outputMdParam.Visible  = 'on';
    if strcmp(blockObj.outputMode, 'Binary point scaling')
        WLColLblOutput.Visible = 'on';
        FLColLblOutput.Visible = 'on';
        outputWLParam.Visible  = 'on';
        outputFLParam.Visible  = 'on';
    else
        % WLs and FLs inherited
        WLColLblOutput.Visible = 'off';
        FLColLblOutput.Visible = 'off';
        outputWLParam.Visible  = 'off';
        outputFLParam.Visible  = 'off';
    end
end
