function dspblkcrossspectrumtfe(blk, action, varargin)
% DSPBLKCROSSSPECTRUMTFE DSP System Toolbox Cross-Spectrum Estimator and
% Discrete Transfer Function Estimator blocks mask helper function

%   Copyright 2015-2022 The MathWorks, Inc.

% This function sometimes changes settings of blocks in the subsystem. this
% should not dirty the model:
model = bdroot(blk);
dirtyFlag = get_param(model,'Dirty');
c = onCleanup(@()set_param(model,'Dirty',dirtyFlag));

if strcmp(get_param(blk,'MaskType'),'Cross-Spectrum Estimator')
    SpectrumBlkName = [blk '/Cross-Spectrum Estimator'];
else
    SpectrumBlkName = [blk '/TFE'];
end

maskNames = get_param(blk,'MaskNames');
maskStr = get_param(blk,'MaskValues');

switch action
    
    case 'dynamic' % parameter changed action
        
        % Get the parameter value
        paramName = varargin{1};
        index = find(ismember(maskNames,paramName));
        val = maskStr{index}; %#ok
        
        visibilities = get_param(blk,'MaskVisibilities');
        original_visibilities = visibilities;
        
        switch paramName
            
            case 'WindowMode'
                WindowLengthIndex  = find(ismember(maskNames,'WindowLength'));
                if strcmp(val,'Same as input frame length')
                    visibilities{WindowLengthIndex} = 'off';
                else
                    visibilities{WindowLengthIndex} = 'on';
                end

            case 'FFTLengthSource'
                FFTIndex  = find(ismember(maskNames,'FFTLength'));
                if strcmp(val,'Auto')
                    visibilities{FFTIndex} = 'off';
                else
                    visibilities{FFTIndex} = 'on';
                end
                
            case 'Window'
                index = find(ismember(maskNames,'SidelobeAttenuation'));
                if strcmp(val,'Kaiser') || strcmp(val,'Chebyshev')
                    visibilities{index} = 'on';
                else
                    visibilities{index} = 'off';
                end

           case 'InheritSampleRate'
               SampleRateIndex  = find(ismember(maskNames,'SampleRate'));
                if strcmp(val,'on')
                    visibilities{SampleRateIndex} = 'off';
                else
                    visibilities{SampleRateIndex} = 'on';
                end
                
            case 'AveragingMethod'
                SpectralAveragesIndex  = find(ismember(maskNames,'SpectralAverages'));
                ForgettingFactorPortIndex = find(ismember(maskNames,'ForgettingFactorPort'));
                ForgettingFactorIndex = find(ismember(maskNames,'ForgettingFactor'));
                if strcmp(val,'Running')
                    visibilities{SpectralAveragesIndex} = 'on';
                    visibilities{ForgettingFactorPortIndex} = 'off';
                    visibilities{ForgettingFactorIndex} = 'off';
                else
                    visibilities{SpectralAveragesIndex} = 'off';
                    visibilities{ForgettingFactorPortIndex} = 'on';
                    visibilities{ForgettingFactorIndex} = 'on';
                end
                
            case 'ForgettingFactorPort'
                averagingMethod = getMaskValue(maskNames,maskStr,'AveragingMethod');
                ForgettingFactorIndex = find(ismember(maskNames,'ForgettingFactor'));
                ForgettingFactorPortIndex = find(ismember(maskNames,'ForgettingFactorPort'));
                if strcmp(val,'on') && strcmp(averagingMethod, 'Exponential')
                   visibilities{ForgettingFactorIndex} = 'off';
                    visibilities{ForgettingFactorPortIndex} = 'on';
                elseif strcmp(val,'off') && strcmp(averagingMethod, 'Exponential')
                   visibilities{ForgettingFactorIndex} = 'on'; 
                   visibilities{ForgettingFactorPortIndex} = 'off';
                end
                
            otherwise
                
        end
        
        if ~isequal(original_visibilities,visibilities)
            set_param(blk,'MaskVisibilities',visibilities),
        end
        
    case 'setup' % Called in the block's initFcn callback        
        % Validate parameter values on dialog
        validateParameters(blk , maskNames, true);
        
        % Setup the buffer parameters
        windowMode = getMaskValue(maskNames,maskStr,'WindowMode');
        if ~strcmp(windowMode,'Same as input frame length')
            set_param([blk '/Buffer_x'],'InheritSize','off'),
            set_param([blk '/Buffer_x'],'N','WindowLength'),
            WindowLength   = slResolve(getMaskValue(maskNames,maskStr,'WindowLength'),blk);
            overlapPercent = slResolve( getMaskValue(maskNames,maskStr,'Overlap'),blk);
            overlapLength  = getNumWelchOverlapSamples(WindowLength,overlapPercent);
            set_param([blk '/Buffer_x'],'V',num2str(overlapLength)),
            set_param([blk '/Buffer_y'],'InheritSize','off'),
            set_param([blk '/Buffer_y'],'N','WindowLength'),
            set_param([blk '/Buffer_y'],'V',num2str(overlapLength)),
        else
            set_param([blk '/Buffer_x'],'InheritSize','on'),
            set_param([blk '/Buffer_x'],'OverlapPercent','Overlap'),
            set_param([blk '/Buffer_y'],'InheritSize','on'),
            set_param([blk '/Buffer_y'],'OverlapPercent','Overlap'),
        end
        
        window = getMaskValue(maskNames,maskStr,'Window');
        if strcmp(window,'Kaiser') || strcmp(window,'Chebyshev')
            sidelobeAttenuation = slResolve(get_param(blk,'SidelobeAttenuation'),blk);
            set_param(SpectrumBlkName,'SidelobeAttenuation',num2str(sidelobeAttenuation)),
        end
        
        SpectralAverages = slResolve(get_param(blk,'SpectralAverages'),blk);
        set_param(SpectrumBlkName,'SpectralAverages',num2str(SpectralAverages)),
        
        FFTsource = getMaskValue(maskNames,maskStr,'FFTLengthSource');
        if strcmp(FFTsource,'Property')
            FFTLength = slResolve(get_param(blk,'FFTLength'),blk);
            set_param(SpectrumBlkName,'FFTLength',num2str(FFTLength)),
        end
            
    case 'init' % Called in the mask initialization commands
        
        % Validate parameter values on dialog
        validateParameters(blk , maskNames, false);

        if strcmp(get_param(blk,'MaskType'),'Cross-Spectrum Estimator')
                inheritFs = get_param(blk,'InheritSampleRate');
                set_param(SpectrumBlkName,'InheritSampleRate',inheritFs);
                if strcmp(inheritFs,'off')
                   Fs = getMaskValue(maskNames,maskStr,'SampleRate');
                   set_param(SpectrumBlkName,'SampleRate',Fs);
                end
        end
        
        averagingMethod = getMaskValue(maskNames,maskStr,'AveragingMethod');
        set_param(SpectrumBlkName,'AveragingMethod',averagingMethod);
        
        switch getMaskValue(maskNames,maskStr,'FrequencyRange')
            case 'One-sided'
                set_param(SpectrumBlkName,'FrequencyRange','onesided');
            case 'Two-sided'
                set_param(SpectrumBlkName,'FrequencyRange','twosided');
            case 'Centered'
                set_param(SpectrumBlkName,'FrequencyRange','centered');
        end
        
        set_param(SpectrumBlkName,'FFTLengthSource',getMaskValue(maskNames,maskStr,'FFTLengthSource'))
        set_param(SpectrumBlkName,'Window', getMaskValue(maskNames,maskStr,'Window'))
        if ~strcmp(get_param(blk,'MaskType'),'Cross-Spectrum Estimator')
            set_param([blk '/TFE'],'OutputCoherence',getMaskValue(maskNames,maskStr,'OutputCoherence'));
        end
        set_param(SpectrumBlkName,'SimulateUsing',getMaskValue(maskNames,maskStr,'SimulateUsing'));
        % Delete/add inports as required
        pos = getInPortPosition(3);
       
        processInputPort(blk, 'lambda','ForgettingFactorPort',3,pos);
        connectInputPort(blk,'lambda','ForgettingFactorPort',3);
        set_param([blk '/x'],'Position', pos{1});
        set_param([blk '/y'],'Position', pos{2});
    
        % Update output ports based on block settings
        % Delete/add port as required
        if ~strcmp(get_param(blk,'MaskType'),'Cross-Spectrum Estimator')
            processOutputPort(blk,'Cxy','OutputCoherence',[510   188   540   202]);
            % Reconnect port
            connectOutputPort(blk,'Cxy','OutputCoherence',2);
        end
        
end
% -------------------------------------------------------------------------

function processOutputPort(blk,outName,paramName,position)
% Frequency output port
inPort = find_system(blk,'MatchFilter',@Simulink.match.allVariants, 'LookUnderMasks', 'on', 'FollowLinks', 'on', 'Name', outName); % look only inside active choice of VSS
if strcmp(get_param(blk,paramName),'on')
    if isempty(inPort)
        add_block('built-in/Outport', [blk '/' outName]);
    else
        lineHandles = get_param([blk '/' outName],'LineHandles');
        if lineHandles.Inport ~= -1
            delete_line(lineHandles.Inport);
        end
    end
    set_param([blk '/' outName],'Position',position)
    set_param([blk '/H'],'Position',[510   158   540   172])
else
    if ~isempty(inPort)
        lineHandles = get_param([blk '/' outName],'LineHandles');
        if lineHandles.Inport ~= -1
            delete_line(lineHandles.Inport);
        end
        delete_block([blk '/' outName]);
    end
    set_param([blk '/H'],'Position',[510   173   540   187])
end

% -------------------------------------------------------------------------
function processInputPort(blk, inputName, paramName, portNumber, position)
% Forgetting factor input port
inPort = find_system(blk,'MatchFilter',@Simulink.match.allVariants, 'LookUnderMasks', 'on', 'FollowLinks', 'on', 'Name', inputName); % look only inside active choice of VSS
if strcmp(get_param(blk,paramName),'on')
    if isempty(inPort)
        add_block('built-in/Inport', [blk '/' inputName]);
    else
        lineHandles = get_param([blk '/' inputName],'LineHandles');
        if lineHandles.Outport ~= -1
            delete_line(lineHandles.Outport);
        end
    end
     set_param([blk '/' inputName],'Position',position{portNumber});
else
    if ~isempty(inPort)
        lineHandles = get_param([blk '/' inputName],'LineHandles');
        if lineHandles.Outport ~= -1
            delete_line(lineHandles.Outport);
        end
        delete_block([blk '/' inputName]);
    end
end

% -------------------------------------------------------------------------
function connectOutputPort(blk,outName,paramName,index)
if  strcmp(get_param(blk,paramName),'on')
    portData  =get_param([blk '/' outName],'PortConnectivity');
    if (portData(1).SrcBlock==-1)
        add_line(blk,['TFE/' num2str(index)],[outName '/1']);
    end
end

% -------------------------------------------------------------------------
function connectInputPort(blk,inputName,paramName,index)
if  strcmp(get_param(blk,paramName),'on')
    portData  = get_param([blk '/' inputName],'PortConnectivity');
    if strcmp(get_param(blk,'MaskType'),'Cross-Spectrum Estimator')
        if (isempty(portData(1).DstBlock))
            add_line(blk,[inputName '/1'],['Cross-Spectrum Estimator/' num2str(index)],'AUTOROUTING','ON');
        end
    else
        if (isempty(portData(1).DstBlock))
            add_line(blk,[inputName '/1'],['TFE/' num2str(index)],'AUTOROUTING','ON');
        end
    end
    set_param([blk '/' inputName],'Port',num2str(index))
end

% -------------------------------------------------------------------------
function pos = getInPortPosition(numPorts)

pos{1} = [];
pos{2} = [];
pos{3} = [];

switch numPorts
    case 1
        pos{1} = [310   162   350   178];
    case 2
        pos{1} = [10   103    40   117];
        pos{2} = [10   259    40   271];
    case 3
        pos{1} = [10  103    40   117];
        pos{2} = [5   182    45   198];
        pos{3} = [5   257    45   273];
end
% -------------------------------------------------------------------------
function validateParameters(blk , MaskNames, forceCheck)
visibilities = get_param(blk,'MaskVisibilities');
% Validate window length
if ~strcmp(get_param(blk,'WindowMode'),'Same as input frame length')
    [L,~] = slResolve(get_param(blk,'WindowLength'),blk);
    if ~isempty(L) || forceCheck
        validateattributes(L,{'double','single'}, {'positive','real','scalar','finite','nonnan'},'','WindowLength');
    end
end

% Validate FFT length
if strcmp(get_param(blk,'FFTLengthSource'),'Property')
    [nfft,~] = slResolve(get_param(blk,'FFTLength'),blk);
    if ~isempty(nfft) || forceCheck
        validateattributes(nfft,{'double'}, {'positive','real','scalar','integer','finite','nonnan'},'','FFTLength');
    end
end

% Validate sample rate
if strcmp(get_param(blk,'MaskType'),'Cross-Spectrum Estimator') && strcmp(get_param(blk,'InheritSampleRate'),'off')
    [Fs,~] = slResolve(get_param(blk,'SampleRate'),blk);
    if ~isempty(Fs) || forceCheck
        validateattributes(Fs,{'double','single'}, {'positive','real','scalar','finite','nonnan'},'','SampleRate');
    end
end

% Validate sidelobe attenuation
if strcmp(get_param(blk,'Window'),'Kaiser') || strcmp(get_param(blk,'Window'),'Chebyshev')
    [R,~] = slResolve(get_param(blk,'SidelobeAttenuation'),blk);
    if ~isempty(R) || forceCheck
        validateattributes(R,{'double','single'}, {'positive','real','scalar','finite','nonnan'},'','SidelobeAttenuation');    

    end
end

if isParameterVisible(MaskNames,visibilities,'SpectralAverages')
    % Validate number of averages
    [L,~] = slResolve(get_param(blk,'SpectralAverages'),blk);
    if ~isempty(L) || forceCheck
        validateattributes(L,{'double'}, {'positive','real','scalar','integer','finite','nonnan'},'','SpectralAverages');
    end
end

if isParameterVisible(MaskNames,visibilities,'ForgettingFactor')
    % Validate forgetting factor
    [L,~] = slResolve(get_param(blk,'ForgettingFactor'),blk);
    if ~isempty(L) || forceCheck
        validateattributes(L,{'double'}, {'real','scalar','finite','nonnan','>=',0,'<=',1},'','ForgettingFactor');
    end
end

% Validate overlap percent
[overlap,~] = slResolve(get_param(blk,'Overlap'),blk);
if ~isempty(overlap)|| forceCheck
    validateattributes(overlap,{'double','single'}, {'real','scalar','finite','nonnan','>=',0,'<',100},'','Overlap');
end

% -------------------------------------------------------------------------

function value = getMaskValue(MaskNames,MaskValues,ParameterName)
index  = find(ismember(MaskNames,ParameterName));
value  = MaskValues{index}; %#ok

% -------------------------------------------------------------------------

function value = isParameterVisible(MaskNames,Visibilities,ParameterName)
index  = find(ismember(MaskNames,ParameterName));
value  = strcmp(Visibilities{index},'on'); %#ok
