function dspblkfblms(action)
% DSPBLKFBLMS Mask dynamic dialog function for fast block lms adaptive filter

% Copyright 1995-2019 The MathWorks, Inc.

blk=gcb;
if nargin==0, action = 'init'; end
cr = newline;
switch action
   case 'init'
     set_param([blk '/BLMS For Iterator' cr 'Subsystem'], 'algo_for', get_param(blk, 'algo'));
     updateStepPort(blk);
     updateAdaptPort(blk);
     updateResetPort(blk);
     load_system('dspobslib');  %for To Frame
     updateweightsoutput(blk);
     CheckParameters(blk);
  case 'addparams' %called when step size input option is changed
      AddParams(blk);
  otherwise % should not come here
      error(message('dsp:dspblkfblms:unhandledCase'));
end

% -----------------------------------------------
function AddParams(blk)
STEPSIZE = 5;
origmaskvis = get_param(blk,'MaskVisibilities');
wantStepPort = strcmp(get_param(blk, 'stepflag'),'Input port');
newflags = {'off', 'on', 'on', 'on', 'on', 'on', 'off', 'on', 'on', 'on', 'on'};
if wantStepPort
    newflags{STEPSIZE} = 'off';
else
    newflags{STEPSIZE} = 'on';
end
if ~isequal(origmaskvis, newflags)
    set_param(blk, 'MaskVisibilities', newflags);
end

% -----------------------------------------------
function updateStepPort(blk)
% Manage the "stepflag" feature
% Make port appear if user wants step-size as input
% Make port disappear if user does not want step size as input.
%
%step port will always be port 3

stepBlk      = [blk '/Step-size'];
isStepPortPresent = strcmp(get_param(stepBlk,'BlockType'),'Inport');
wantStepPort      = strcmp(get_param(blk, 'stepflag'),'Input port');
if wantStepPort && ~isStepPortPresent
    % Change Constant to Port
    pos = get_param(stepBlk,'Position');
    delete_block(stepBlk);
    add_block('built-in/Inport',stepBlk, ...
              'Position', pos, ...
              'Port', '3');

elseif ~wantStepPort && isStepPortPresent
    % Change Port to Constant
    pos = get_param(stepBlk,'Position');
    delete_block(stepBlk);
    add_block('built-in/Constant',stepBlk, ...
              'Position',pos,'Value', 'mu',...
              'OutDataTypeStr', 'Inherit: Inherit via back propagation');
end

%-----------------------------------------------------------
function updateAdaptPort(blk)
% Manage the "adapt" feature
% Make port appear if user wants adaptation-hold control
% Make port disappear if user does not want control over this
%
% Practically, this is performed by swapping a constant block
% for a port, and vice-versa

% The adapt port at the top will be either 3rd or 4th port depending on the
% presence of the step port. 

adaptBlk      = [blk '/Adapt'];
isAdaptPortPresent = strcmp(get_param(adaptBlk,'BlockType'),'Inport');
wantAdaptPort      = strcmp(get_param(blk,'Adapt'),'on');
if wantAdaptPort && ~isAdaptPortPresent
    % Change Constant to Port
    pos = get_param(adaptBlk,'Position');
    delete_block(adaptBlk);
    
    % Note: this can be either the 3rd or 4th depending
    %   on whether the step port is present or not.
    %   You have to specify the port number accordingly.
    %  
    
    stepBlk = [blk '/Step-size'];
    isStepPortPresent = strcmp(get_param(stepBlk,'BlockType'),'Inport');
    portnum = num2str(3+isStepPortPresent);

    add_block('built-in/Inport',adaptBlk, ...
              'Position', pos, ...
              'Port', portnum);

elseif ~wantAdaptPort && isAdaptPortPresent
    % Change Port to Constant
    pos = get_param(adaptBlk,'Position');
    delete_block(adaptBlk);
    add_block('built-in/Constant',adaptBlk, ...
              'Position',pos,'Value', ...
              'boolean(1)');
end

% -----------------------------------------------
function updateResetPort(blk)
% Manage the "reset" feature
% Make port appear if user wants to reset
% Make port disappear if user does not want control over this
%
% Practically, this is performed by swapping a constant block
% for a port, and vice-versa

% reset port will always be the last input port

resetBlk = [blk '/Reset'];
isResetPortPresent = strcmp(get_param(resetBlk,'BlockType'),'Inport');
resetMode = get_param(blk,'resetflag');  % Get popup state
wantResetPort = ~strcmp(resetMode,'None');

% Adjust ports/constants
if wantResetPort && ~isResetPortPresent
    % Change Constant to Port
    pos = get_param(resetBlk,'Position');
    delete_block(resetBlk);

    add_block('built-in/Inport',resetBlk, ...
              'Position',pos);
          
elseif ~wantResetPort && isResetPortPresent
    % Change Port to Constant
    pos = get_param(resetBlk,'Position');
    delete_block(resetBlk);
    add_block('built-in/Constant',resetBlk, ...
              'Position',pos, ...
              'Value','boolean(0)');
end

cr = newline;
% Update reset mode of unit delay block:
delayBlk = [blk '/BLMS For Iterator' cr 'Subsystem/Algorithm/Filter Taps'];
currentResetMode = get_param(delayBlk,'ExternalReset');
% Don't turn off reset mode - in 'None' mode, we leave
% it at "Non-zero sample" with a constant-0 input.
if strcmp(resetMode,'None')
    resetMode='Level hold';
end
if ~strcmp(currentResetMode, resetMode)
    set_param(delayBlk,'ExternalReset',transformResetString(resetMode));
end
% -------------------------------------------------------------------------
function strOut = transformResetString(strIn)
% Map old Reset string to new Reset string
if strcmpi(strIn, 'none')
    strOut = 'None'; % default case
else
    % Consider first character only, case insensitive choices
    charArray = lower(char(strIn));
    switch charArray(1)
        case {'r'}
            strOut = 'Rising';     % 'Rising edge'
        case {'f'}
            strOut = 'Falling';    % 'Falling edge'
        case {'e'}
            strOut = 'Either';     % 'Either edge'
        otherwise
            strOut = 'Level hold'; % 'Non-zero sample'
    end    
end
%----------------------------------------------------------
function updateweightsoutput(blk)

cr = newline;
%update weight output
weights = get_param(blk, 'weights');
if (strcmp(weights, 'off'))
    %first delete at the upper level
    if (exist_block(blk, 'Wts'))
        delete_line(blk, ['BLMS For Iterator' cr 'Subsystem/3'], 'To Sample/1');
        delete_line(blk, 'To Sample/1', 'Wts/1');
        delete_block([blk '/To Sample']);
        delete_block([blk '/Wts']);
        set_param([blk , '/BLMS For Iterator' cr 'Subsystem'], 'weights_for', 'off');
    end
else
    set_param([blk , '/BLMS For Iterator' cr 'Subsystem'], 'weights_for', 'on');
    if ~(exist_block(blk, 'Wts'))
        add_block(['dspobslib/Frame Status' cr 'Conversion'], [blk '/To Sample'],...
            'Position', [500   262   525   298]);
        add_block('built-in/Outport', [blk, '/Wts'], ...
            'Position', [550   273   580   287]);        
        add_line(blk, ['BLMS For Iterator' cr 'Subsystem/3'], 'To Sample/1');
        add_line(blk, 'To Sample/1', 'Wts/1');
    end
end

% -----------------------------------------------
function CheckParameters(blk)

CheckLeakage(blk);
Checkmu(blk);
CheckFilterLength(blk);

% -----------------------------------------------
function CheckLeakage(blk)
%Checks validity of the leakage parameter
errorstatus = 0;
try
	leakage = evalin('base', get_param(blk,'leakage'));  % Get leakage
	if (leakage < 0 || leakage > 1 || ~isreal(leakage))
        errorstatus = 1;
	end
catch
end
if (errorstatus)
    error(message('dsp:dspblkfblms:paramOutOfRange'));
end

% -----------------------------------------------
function Checkmu(blk)
%Checks validity of the mu parameter

noStepPort = strcmp(get_param(blk,'stepflag'),'Dialog');

if (noStepPort)
    %check mu
    errorstatus = 0;
    try    
        mu = evalin('base', get_param(blk,'mu'));  % Get leakage
        if (length(mu) > 1 || mu < 0 || ~isreal(mu))
            errorstatus = 1;
        end
    catch
    end
    if (errorstatus)
        error(message('dsp:dspblkfblms:paramRealScalarError'));
    end
end

% -----------------------------------------------
function CheckFilterLength(blk)
%Checks validity of the filter length parameter

errorstatus = 0;
try
    N = evalin('base', get_param(blk,'N'));  % Get length
    if (N < 0 || (round(N) ~= N))
        errorstatus = 1;
    end
catch
end
if (errorstatus)
    error(message('dsp:dspblkfblms:invalidFilterOrder'));
end
