function varargout = dspblklms(varargin)
% DSPBLKLMS Mask dynamic dialog function for LMS adaptive filter

% Copyright 1995-2018 The MathWorks, Inc.

blkh         = varargin{1};
AlgorithmStr = get_param(blkh, 'Algo');
HasMuInport  = strncmp(get_param(blkh,'stepflag'),'Input port',1);

if nargin > 1
    % Return port labels and block icon
    [si, so]     = getPortLabels(blkh, HasMuInport);
    varargout{1} = si;
    varargout{2} = so;
    varargout{3} = getBlkIconStr(AlgorithmStr);
else
    % Execute dynamic dialog UI code
    handleBlockMaskUI(blkh, HasMuInport, AlgorithmStr);
end

% ------------------------------------------------------------------------
function disp_str = getBlkIconStr(algorithm)
%Draw the algorithm name on the block
lmsSpcStr = '   LMS   '; % extra centering spaces
switch algorithm
    case 'LMS'
        disp_str = 'LMS';
    case 'Normalized LMS'
        disp_str = sprintf('Normalized\n%s', lmsSpcStr);
    case 'Sign-Error LMS'
        disp_str = sprintf('Sign-Error\n%s', lmsSpcStr);
    case 'Sign-Data LMS'
        disp_str = sprintf('Sign-Data\n%s',  lmsSpcStr);
    otherwise % case 'Sign-Sign LMS'
        disp_str = sprintf('Sign-Sign\n%s',  lmsSpcStr);
end

% ------------------------------------------------------------------------
function [si, so] = getPortLabels(blkh, HasMuInport)
si(1).port = 1; si(1).txt='Input';
si(2).port = 2; si(2).txt='Desired';
so(1).port = 1; so(1).txt='Output';
so(2).port = 2; so(2).txt='Error';

HasAdaptInport =  strcmp(get_param(blkh,'Adapt'),'on');
HasResetInport = ~strcmp(get_param(blkh,'resetflag'),'None');
HasWgtOutport  =  strcmp(get_param(blkh,'weights'),'on');

i=3;
if HasMuInport
    si(i).port = i;si(i).txt='Step-size'; i=i+1;
end
if HasAdaptInport
    si(i).port = i;si(i).txt='Adapt'; i=i+1;
end
if HasResetInport
    si(i).port = i;si(i).txt='Reset'; i=i+1;
end
j=3;
if HasWgtOutport
    so(3).port = 3;so(3).txt='Wts';  j=j+1;
end

for m=i:5, si(m)=si(i-1); end
for n=j:3, so(n)=so(j-1); end

% ------------------------------------------------------------------------
function handleBlockMaskUI(blkh, HasMuInport, AlgorithmStr)

% 'Main' tab Parameter objects to set Visible/Enabled
blockMaskObj     = Simulink.Mask.get(blkh);
stepSizeValuePrm = blockMaskObj.getParameter('mu');
if HasMuInport
    stepSizeValuePrm.Visible = 'off';
else
    stepSizeValuePrm.Visible = 'on';
end

% 'Data Types' tab Parameter objects to set Visible/Enabled
handleParametersDTGroupBoxUI(blkh, HasMuInport);
handleWeightsDTGroupBoxUI(blkh);
handlePrdsQuotDTGroupBoxUI(blkh, AlgorithmStr);
handleAccumsDTGroupBoxUI(blkh, AlgorithmStr);

% ------------------------------------------------------------------------
function handleParametersDTGroupBoxUI(blkh, HasMuInport)
% Params DTs have shared Mode and WordLength,
% and individual FracLength mask parameters
blockMaskObj = Simulink.Mask.get(blkh);
prmsWLParam  = blockMaskObj.getParameter('firstCoeffWordLength');
stSzFLParam  = blockMaskObj.getParameter('firstCoeffFracLength');
lkFcFLParam  = blockMaskObj.getParameter('secondCoeffFracLength');
WLColLblPrms = blockMaskObj.getDialogControl('prmsDTWordLengthLabel');
FLColLblPrms = blockMaskObj.getDialogControl('prmsDTFracLengthLabel');

paramsDTGroupMode = get_param(blkh, 'firstCoeffMode');

switch paramsDTGroupMode
    case 'Binary point scaling'
        WLColLblPrms.Visible = 'on';
        FLColLblPrms.Visible = 'on';
        prmsWLParam.Visible  = 'on';
        if HasMuInport
            stSzFLParam.Visible = 'off';
        else
            stSzFLParam.Visible = 'on';
        end
        lkFcFLParam.Visible = 'on';
        
    case 'Specify word length'
        WLColLblPrms.Visible = 'on';
        FLColLblPrms.Visible = 'off';
        prmsWLParam.Visible  = 'on';
        stSzFLParam.Visible  = 'off';
        lkFcFLParam.Visible  = 'off';
        
    otherwise
        % 'Same word length as input'
        % (all WLs/FLs inherited)
        WLColLblPrms.Visible = 'off';
        FLColLblPrms.Visible = 'off';
        prmsWLParam.Visible  = 'off';
        stSzFLParam.Visible  = 'off';
        lkFcFLParam.Visible  = 'off';
end

% ------------------------------------------------------------------------
function handleWeightsDTGroupBoxUI(blkh)
% Weights DTs have individual Mode, WordLength, FracLength
blockMaskObj = Simulink.Mask.get(blkh);
wgtsWLParam  = blockMaskObj.getParameter('memoryWordLength');
wgtsFLParam  = blockMaskObj.getParameter('memoryFracLength');
WLColLblWgts = blockMaskObj.getDialogControl('wgtsDTWordLengthLabel');
FLColLblWgts = blockMaskObj.getDialogControl('wgtsDTFracLengthLabel');

weightsDTGroupMode = get_param(blkh, 'memoryMode');

if strcmp(weightsDTGroupMode, 'Binary point scaling')
    WLColLblWgts.Visible = 'on';
    FLColLblWgts.Visible = 'on';
    wgtsWLParam.Visible  = 'on';
    wgtsFLParam.Visible  = 'on';
else
    % 'Same as first input'
    WLColLblWgts.Visible = 'off';
    FLColLblWgts.Visible = 'off';
    wgtsWLParam.Visible  = 'off';
    wgtsFLParam.Visible  = 'off';
end

% ------------------------------------------------------------------------
function handlePrdsQuotDTGroupBoxUI(blkh, AlgorithmStr)
% Products and quotient DTs have shared Mode and WordLength,
% and individual FracLength mask parameters
blockMaskObj    = Simulink.Mask.get(blkh);
prdQuot_WLParam = blockMaskObj.getParameter('prodOutputWordLength');
prd_uTu_FLParam = blockMaskObj.getParameter('prodOutputFracLength');
prd_WTu_FLParam = blockMaskObj.getParameter('prodOutput2FracLength');
prd_muE_FLParam = blockMaskObj.getParameter('prodOutput3FracLength');
prd_Q_u_FLParam = blockMaskObj.getParameter('prodOutput4FracLength');
quotientFLParam = blockMaskObj.getParameter('quotientFracLength');
WLColLblPrQt = blockMaskObj.getDialogControl('prdsQuotDTWordLengthLabel');
FLColLblPrQt = blockMaskObj.getDialogControl('prdsQuotDTFracLengthLabel');

prodQuotDTGroupMode = get_param(blkh, 'prodOutputMode');

if strcmp(prodQuotDTGroupMode, 'Same as first input')
    WLColLblPrQt.Visible    = 'off';
    FLColLblPrQt.Visible    = 'off';
    prdQuot_WLParam.Visible = 'off';
    prd_uTu_FLParam.Visible = 'off';
    prd_WTu_FLParam.Visible = 'off';
    prd_muE_FLParam.Visible = 'off';
    prd_Q_u_FLParam.Visible = 'off';
    quotientFLParam.Visible = 'off';
else
    % 'Binary point scaling'
    WLColLblPrQt.Visible    = 'on';
    FLColLblPrQt.Visible    = 'on';
    prdQuot_WLParam.Visible = 'on';
    
    switch (AlgorithmStr)
        case 'Normalized LMS'
            prd_uTu_FLParam.Visible = 'on';
            prd_WTu_FLParam.Visible = 'on';
            prd_muE_FLParam.Visible = 'on';
            prd_Q_u_FLParam.Visible = 'on';
            quotientFLParam.Visible = 'on';
            
        case {'LMS', 'Sign-Data LMS'}
            prd_uTu_FLParam.Visible = 'off';
            prd_WTu_FLParam.Visible = 'on';
            prd_muE_FLParam.Visible = 'on';
            prd_Q_u_FLParam.Visible = 'on';
            quotientFLParam.Visible = 'off';
            
        otherwise
            % {'Sign-Error LMS', 'Sign-Sign LMS'}
            prd_uTu_FLParam.Visible = 'off';
            prd_WTu_FLParam.Visible = 'on';
            prd_muE_FLParam.Visible = 'off';
            prd_Q_u_FLParam.Visible = 'on';
            quotientFLParam.Visible = 'off';
    end
end

% ------------------------------------------------------------------------
function handleAccumsDTGroupBoxUI(blkh, AlgorithmStr)
% Accumulators DTs have shared Mode and WordLength,
% and individual FracLength mask parameters
blockMaskObj      = Simulink.Mask.get(blkh);
accs_shr_WL_Param = blockMaskObj.getParameter('accumWordLength');
accum_uTu_FLParam = blockMaskObj.getParameter('accumFracLength');
accum_WTu_FLParam = blockMaskObj.getParameter('accum2FracLength');

WLColLblAccs = blockMaskObj.getDialogControl('accsDTWordLengthLabel');
FLColLblAccs = blockMaskObj.getDialogControl('accsDTFracLengthLabel');

accumsDTGroupMode = get_param(blkh, 'accumMode');

if strcmp(accumsDTGroupMode, 'Same as first input')
    WLColLblAccs.Visible      = 'off';
    FLColLblAccs.Visible      = 'off';
    accs_shr_WL_Param.Visible = 'off';
    accum_uTu_FLParam.Visible = 'off';
    accum_WTu_FLParam.Visible = 'off';
else
    % 'Binary point scaling'
    WLColLblAccs.Visible      = 'on';
    FLColLblAccs.Visible      = 'on';
    accs_shr_WL_Param.Visible = 'on';
    if strcmp(AlgorithmStr, 'Normalized LMS')
        accum_uTu_FLParam.Visible = 'on';
    else
        accum_uTu_FLParam.Visible = 'off';
    end
    accum_WTu_FLParam.Visible = 'on';
end
