function varargout = dspblkupfirdn2(action,FilterSource,varargin)
% DSPBLKUPFIRDN Mask dynamic dialog function for FIR rate conversion block

% Copyright 1995-2022 The MathWorks, Inc.

blkh=gcbh;
switch action
  case 'icon'
    % Generate string used for drawing icon:
    filterDefined = false;
    if FilterSource == 1 || FilterSource == 3
        L = varargin{1};
        M = varargin{2};
        if (~isempty(L) && ~isempty(M))
            filterDefined = true;
        end
    else
        %% Coeffs from filter object:
        hd = varargin{1};
        if ~isempty(hd)
            if isa(hd,'mfilt.firsrc')
                L = hd.RateChangeFactors(1);
                M = hd.RateChangeFactors(2);
                filterDefined = true;
            elseif isa(hd,'dsp.FIRRateConverter') % dsp.FIRRateConverter
                L = hd.InterpolationFactor;
                M = hd.DecimationFactor;
                filterDefined = true;
            end
        end
    end
    if filterDefined
        if isempty(L)  || ~isnumeric(L) || ~isscalar(L) || isfi(L) || ...
                isempty(M)  || ~isnumeric(M) || ~isscalar(M) || isfi(M)
            str = 'x[?]';
        else
            if (L == 1)
                str = ['x[' num2str(M) 'n]'];
            elseif (M == 1)
                str = ['x[n/' num2str(L) ']'];
            else
                str = ['x[' num2str(M) 'n/' num2str(L) ']'];
            end
        end
    else
        str = 'x[?]';
    end
    if FilterSource == 2
        str = [str '\nFilter:\n' get_param(blkh,'FilterObject')];
    end
    varargout{1} = str;

  case 'setup'
    filterDefined = false; %% Default
    h  = [];
    L  = [];
    M  = [];
    block = get_param(blkh,'Object');
    if FilterSource == 1 || FilterSource == 3
        % Coefficients from 'Dialog parameters' or 'Auto'
        % dspblkupfirdn2('setup',FilterSource,h,L,M)
        if isfield(block.UserData,'filter')
            block.UserData = rmfield(block.UserData,'filter');
        end

        dtRows = varargin{1};
        L      = varargin{3};
        M      = varargin{4};
        if ~strcmp(block.rateMode, 'Allow multirate processing')
            AllowArbitraryInputLength = varargin{5};
        end

        if FilterSource == 1
            % Coefficient source: 'Dialog parameters'
            h = varargin{2};
        else
            % Coefficient source: 'Auto'
            if isempty(L) || isempty(M) || ~isnumeric(L) || ~isnumeric(M)
                h = [];
            else
                validateattributes(L, {'numeric',}, ...
                                   {'positive', 'real', 'scalar', 'finite', 'nonnan'}, ...
                                   '', 'Interpolation factor');

                validateattributes(M, {'numeric',}, ...
                                   {'positive', 'real', 'scalar', 'finite', 'nonnan'}, ...
                                   '', 'Decimation factor');

                h = designMultirateFIR(double(L), double(M));
            end
        end

        %dtInfo = dspGetFixptDataTypeInfo(blkh,47);
        dtInfo = dspGetUDTFixptDataTypeInfo(blkh, dtRows, true);
        dtInfo.arithmetic = 0; % Default: double-precision arithmetic
        dtInfo.inputWordLength = 16;
        dtInfo.inputFracLength = 15;

        block.UserData.filterConstructor = 'mfilt.firsrc';
        block.UserData.filterConstructorArgs = {L,M,h};

        if ~(isempty(h) || isempty(L) || isempty(M))
            % Don't attempt any calculations in the mask helper on these
            % parameters if any of them is currently empty or not defined.

            filterDefined = true;

            % Do more checking before using h,D parameters
            if ~isnumeric(h) || isfi(h)
                error(message('dsp:dspblkupfirdn2:invalidCoefficient1'));
            end
            if ~isnumeric(M) || isfi(M)
                error(message('dsp:dspblkupfirdn2:paramDTypeError1'));
            end
            if ~isnumeric(L) || isfi(L)
                error(message('dsp:dspblkupfirdn2:paramDTypeError2'));
            end
            if ~isempty(h)
                if ~isvector(h)
                    error(message('dsp:dspblkupfirdn2:invalidCoefficient2'));
                end
            end
            if ~isempty(L)
                if (L < 1) || (fix(L) ~= L)
                    error(message('dsp:dspblkupfirdn2:paramRealScalarError1'));
                end
            end
            if ~isempty(M)
                if (M < 1) || (fix(M) ~= M)
                    error(message('dsp:dspblkupfirdn2:paramRealScalarError2'));
                end
            end
        end
    else
        % Input arguments when coeffs from filter object:
        % dspblkupfirdn2('setup',FilterSource,FilterObject)
        if isfield(block.UserData,'filterConstructor')
            block.UserData = rmfield(block.UserData,'filterConstructor');
            block.UserData = rmfield(block.UserData,'filterConstructorArgs');
        end
        hd = varargin{1};
        if ~strcmp(block.rateMode, 'Allow multirate processing')
            AllowArbitraryInputLength = varargin{2};
        end
        block.UserData.filter = [];
        if ~isempty(hd)
            if isa(hd,'mfilt.firsrc')
                if strcmpi(block.rateMode, 'Allow multirate processing')
                    % True Multirate mode does not support mfilts
                    error(message('dsp:dspblkupfirdn2:TrueMultirateMfilt', ...
                                  'mfilt.firsrc', 'dsp.FIRRateConverter'));
                end
                FilterSource = 4;
                block.UserData.filter = hd;
                hdx = reffilter(hd); %% Get the reference coeffs, not the quantized ones
                h = hdx.Numerator;
                L = hdx.RateChangeFactors(1);
                M = hdx.RateChangeFactors(2);
                filterDefined = true;
            elseif isa(hd,'dsp.FIRRateConverter')
                hd_clone = clone(hd);
                release(hd_clone);
                block.UserData.filter = hd_clone;
                hdx = hd_clone;
                h = hdx.Numerator;
                L = hdx.InterpolationFactor;
                M = hdx.DecimationFactor;
                filterDefined = true;
            else
                error(message('dsp:dspblkupfirdn2:invalidFilterObject1'));
            end
        end
        dtInfo = getFixptDataTypeInfoFromFilterObject(hd);
    end

    if filterDefined
        %% Update FVTool plot window, if open
        if dspIsFVToolOpen(blkh)
            dspLinkFVTool2Mask(blkh,'update');
        end

        % Convert all mask params to double
        h = double(h); L = double(L); M = double(M);

        % Zero-pad the filter coeffs to a multiple of L
        len = length(h);
        if (rem(len, L) ~= 0)
            nzeros = L - rem(len, L);
            h = [h(:); zeros(nzeros,1)];
        end
        % Reshape into L poly-phases
        h = (reshape(h, L, length(h)/L)).';
    else
        % Close FVTool plot window, if open
        dspLinkFVTool2Mask(blkh,'close');
    end

    % Gather up outputs
    args.CoeffSource = FilterSource;
    args.h = h;
    args.L = L;
    args.M = M;
    if ~strcmp(block.rateMode, 'Allow multirate processing')
        args.AllowArbitraryInputLength = AllowArbitraryInputLength;
    end

    varargout = {args,dtInfo};

  case 'variant'
    block = get_param(blkh,'Object');
    %dspblkupfirdn2('variant',  FilterSource, coreblk, dtInfo)
    % Propagate the write datatype settings to the underlying
    % core block. If the filter object is the source of coeffs, take the
    % datatypes from dtInfo, otherwise just propagate using get_param
    % on the top level mask.

    % Only do this propagation for multirate mode, so we don't throw
    % confusing errors. When the mode is switched to allow multirate,
    % all this will run.

    if strcmp(block.rateMode, 'Allow multirate processing')

        coreblk = varargin{1};
        if (FilterSource == 2)
            % Source is filter object. Go from dtInfo to set_param values.
            % Code that encodes dtInfo is getFixptDataTypeInfo and
            % processParamSection

            dtInfo = varargin{2};

            if dtInfo.firstCoeffMode == 2
                coefdt = "Inherit: Same word length as input";
            else
                coefdt = makeDtString(1, dtInfo.firstCoeffWordLength, ...
                                      dtInfo.firstCoeffFracLength);
            end

            switch dtInfo.prodOutputMode
              case 2 % Same as input
                proddt = "Inherit: Same as input";
              case 5 % Internal rule
                proddt = "Inherit: Inherit via internal rule";
              otherwise
                proddt = makeDtString(1, dtInfo.prodOutputWordLength, ...
                                      dtInfo.prodOutputFracLength);
            end

            switch dtInfo.accumMode
              case 2 % Same as input
                accumdt = "Inherit: Same as input";
              case 3 % Same as product
                accumdt = "Inherit: Same as product output";
              case 5 % Internal rule
                accumdt = "Inherit: Inherit via internal rule";
              otherwise
                accumdt = makeDtString(1, dtInfo.accumWordLength, ...
                                       dtInfo.accumFracLength);
            end

            switch dtInfo.outputMode
              case 2 % Same as input
                outdt = "Inherit: Same as input";
              case 3 % Same as product
                outdt = "Inherit: Same as product output";
              case 4 % Same as accum
                outdt = "Inherit: Same as accumulator";
              otherwise
                outdt = makeDtString(1, dtInfo.outputWordLength, ...
                                     dtInfo.outputFracLength);
            end

            rndArr = {'Ceiling', 'Convergent', 'Floor', 'Nearest', 'Round', 'Simplest', 'Zero'};
            rndmeth = rndArr{dtInfo.roundingMode};
            if dtInfo.overflowMode == 2 %1 - wrap, 2-sat
                satover = 'on';
            else
                satover = 'off';
            end
            coefmin = "[]";
            coefmax = "[]";
            outmax = "[]";
            outmin = "[]";
        else
            coefdt = get_param(gcb, 'firstCoeffLastDataTypeStr');
            proddt = get_param(gcb, 'prodOutputDataTypeStr');
            accumdt = get_param(gcb, 'accumDataTypeStr');
            outdt = get_param(gcb, 'outputDataTypeStr');

            rndmeth = get_param(gcb, 'roundingMode');
            satover = get_param(gcb, 'overflowMode');
            coefmin = get_param(gcb, 'firstCoeffMin');
            coefmax = get_param(gcb, 'firstCoeffMax');
            outmin = get_param(gcb, 'outputMin');
            outmax = get_param(gcb, 'outputMax');
        end

        lock =  get_param(gcb, 'LockScale');
        set_param(coreblk, 'CoefDataTypeStr', coefdt);
        set_param(coreblk, 'ProductDataTypeStr', proddt);
        set_param(coreblk, 'AccumDataTypeStr', accumdt);
        set_param(coreblk, 'OutDataTypeStr',outdt);
        set_param(coreblk, 'CoefMin', coefmin );
        set_param(coreblk, 'CoefMax', coefmax);
        set_param(coreblk, 'OutMin', outmin );
        set_param(coreblk, 'OutMax', outmax);
        set_param(coreblk, 'LockScale',lock);
        set_param(coreblk, 'RndMeth', rndmeth);
        set_param(coreblk, 'SaturateOnIntegerOverflow', satover);
    end



  otherwise
    % Dynamic dialog (visible/enable) control
    dspmltiFIRDynamicDialogUpdate(blkh);
    varargout = {};
end
%------------------------------------------------------------------------------

function s = makeDtString(sign, wl, fl)
s = "fixdt(" + sign + ", " + wl + ", " + fl + ")";


%------------------------------------------------------------------------------
function dtInfo = getFixptDataTypeInfoFromFilterObject(hd)

if isa(hd,'dsp.FIRRateConverter')
    dtInfo = getFIRRateConversionFixptInfoFromSysObj(hd);
else
    dtInfo = getFixptDataTypeInfoFromMFILT(hd);
end

%--------------------------------------------------------------------------
function dtInfo = getFixptDataTypeInfoFromMFILT(hd)

% Initialize all to defaults
dtInfo = initializeFIRRateConversionFixedPointDefaults();

if ~isempty(hd) && isfdtbxinstalled
    switch hd.Arithmetic
      case 'double'
        dtInfo.arithmetic = 0;
      case 'single'
        dtInfo.arithmetic = 1;
      case 'fixed'
        dtInfo.arithmetic = 2;

        if strcmpi(hd.OverflowMode, 'wrap')
            dtInfo.overflowMode = 1;
        else
            dtInfo.overflowMode = 2;
        end

        if strcmpi(hd.RoundMode, 'ceil')
            dtInfo.roundingMode = 1;
        elseif strcmpi(hd.RoundMode, 'convergent')
            dtInfo.roundingMode = 2;
        elseif strcmpi(hd.RoundMode, 'floor')
            dtInfo.roundingMode = 3;
        elseif strcmpi(hd.RoundMode, 'nearest')
            dtInfo.roundingMode = 4;
        elseif strcmpi(hd.RoundMode, 'round')
            dtInfo.roundingMode = 5;
        elseif strcmpi(hd.RoundMode, 'fix')
            dtInfo.roundingMode = 7;    %zero
        end

        if ~hd.Signed
            error(message('dsp:dspblkupfirdn2:invalidFilterObject2'));
        end

        if isprop(hd,'CastBeforeSum') && ~hd.CastBeforeSum
            warning(message('dsp:dspblkupfirdn2:invalidFilterObject3'));
        end

        dtInfo.inputWordLength  = hd.InputWordLength;
        dtInfo.inputFracLength  = hd.InputFracLength;

        dtInfo.firstCoeffMode = 0; % Binary point scaling
        dtInfo.firstCoeffWordLength = hd.CoeffWordLength;
        dtInfo.firstCoeffFracLength = hd.NumFracLength;

        dtInfo.prodOutputMode = 0; % Binary point scaling
        dtInfo.prodOutputWordLength = hd.ProductWordLength;
        dtInfo.prodOutputFracLength = hd.ProductFracLength;

        dtInfo.accumMode = 0; % Binary point scaling
        dtInfo.accumWordLength = hd.AccumWordLength;
        dtInfo.accumFracLength = hd.AccumFracLength;

        dtInfo.outputMode = 0; % Binary point scaling
        dtInfo.outputWordLength = hd.OutputWordLength;
        dtInfo.outputFracLength = hd.OutputFracLength;
    end
end

% end of dspblkupfirdn2.m
