function [b] = dspblkvfdly2(varargin)
% DSPBLKVFDLY Mask dynamic dialog function for variable fractional delay block

% Copyright 1995-2017 The MathWorks, Inc.

b = [];

if (nargin < 1)
    action  = 'dynamic'; % Block mask callback (Dialog UI)
    isBlock = true;
elseif (nargin == 1)
    action  = 'setup';   % Block mask initialization
    isBlock = true;
else % nargin > 1
    action  = 'design';  % Design filter (System object *OR* Block)
    isBlock = ~(matlab.system.isSystemObject(varargin{1}));
end

if isBlock
    blkh = gcbh;
    updateVFDBlockLegacyModeParameters(blkh);
    if strcmp(action, 'design')
        % Design the filter
        b = designVFDFilter(varargin{:});
    elseif strcmp(action,'dynamic')
        % Update the block dialog UI visibilities
        updateVFDBlockDynamicDialogUI(blkh);
    end
    % Update the displayed Dmin, Dmax
    updateBlockDminDmaxText(blkh);
else
    % System object: Design the filter
    b = designVFDFilter(varargin{:});
end

end % dspblkvfdly2

% -------------------------------------------------------------------------
function updateVFDBlockLegacyModeParameters(blkh)
    oldMode = get_param(blkh, 'mode');
    if (~strcmp(oldMode, 'OBSOLETE'))
        % This block lives in an old model (<9a release) that is being
        % opened for the first time in 9a or later release (update params)
        if (strcmp(oldMode,'Linear interpolation'))
            set_param(blkh, 'modeActive', 'Linear');
        elseif (strcmp(oldMode,'FIR interpolation'))
            set_param(blkh, 'modeActive', 'FIR');
            set_param(blkh, 'fallbackFIR', ...
                'Switch to linear interpolation if kernel cannot be centered');
        else
            error(message('dsp:dspblkvfdly2:unhandledCase1'));
        end
        set_param(blkh, 'noDFT', 'off');
        set_param(blkh, 'mode', 'OBSOLETE');
    end
end % updateVFDBlockLegacyModeParameters

% -------------------------------------------------------------------------
function b = designVFDFilter(varargin)
% Design the filter
mode = varargin{2};
if mode==3
    % Farrow filtering
    L = double(varargin{3});
    
    % Lagrange Designs for FD Farrow from
    % http://www.acoustics.hut.fi/~vpv/publications/vesan_vaitos/ch3_pt2_lagrange.pdf
    N = L - 1;
    U = fliplr(vander(0:(L-1)));
    Q = inv(U);
    
    % Modified Farrow - D' = D+1
    if N>1
        T = eye(L);
        for i=1:L
            for j=i:L
                T(i,j) = nchoosek(j-1,i-1)*(floor(N/2))^(j-i);
                [~, lastid] = lastwarn;
                if strcmpi(lastid,'MATLAB:nchoosek:LargeCoefficient')
                    warning('off','MATLAB:nchoosek:LargeCoefficient')
                end
            end
        end
        Q = T * Q; %#ok
    end
    
    % Forcing zeros and ones for those
    % that ought to be zeros and ones,
    % to eliminate the roundoff errors.
    [N1, D1] = rat(Q);
    Q(N1 == 0) = 0;
    Q(N1 == D1) = 1;
    
    b = fliplr(Q.');
    
elseif  mode==2
    % FIR interpolation
    R = double(varargin{3});
    L = double(varargin{4});
    alpha = double(varargin{5});
    
    if isoktocheckparam(R)
        if (~isreal(R)) || (numel(R) ~= 1) || (R<2) || (R>65535) || (floor(R)~=R) || (~isnumeric(R))
            error(message('dsp:dspblkvfdly2:paramRealScalarError1'));
        end
    end
    
    if isoktocheckparam(L)
        if (~isreal(L)) || (numel(L) ~= 1) || (L<=0) || (L>65535) || (floor(L)~=L) || (~isnumeric(L))
            error(message('dsp:dspblkvfdly2:paramRealScalarError2'));
        end
    end
    
    if isoktocheckparam(alpha)
        if (~isreal(alpha)) || (numel(alpha) ~= 1) || isnan(alpha) || (alpha > 1) || (alpha < 0)
            error(message('dsp:dspblkvfdly2:paramRealScalarError3'));
        end
    end
    
    b = [];
    
    if ~isempty(R) && ~isempty(L) && ~isempty(alpha)
        if alpha==1
            % For complete bandwidth (alpha==1), firnyquist provides better
            % coefficients than intfilt. More exactly, the interpolated
            % points will be closer to the original signal.
            b=designMultirateFIR(R,1,L,80);
        else
            b=intfilt(R,L,alpha);
        end
        blen=length(b); bcols = ceil(blen/2/L);
        b = [zeros(bcols*2*L-blen,1);b(:)];
        b = flipud(reshape(b,bcols,2*L)');
    end
    
else
    % Linear interpolation
    b = [];
end

end % designVFDFilter

% -------------------------------------------------------------------------
function updateVFDBlockDynamicDialogUI(blkh)

% Get block mask dialog parameter visibilities
mask_vis_old = get_param(blkh, 'MaskVisibilities');
mask_vis_new = mask_vis_old;

[ LNFrwIdx, LFrwIdx,  RIdx, AlphaIdx, FbFIRIdx, FbFrwIdx, ...
    CofMdIdx, CofWLIdx, CofFLIdx, ... % Coefficients type info
    PrdMdIdx, PrdWLIdx, PrdFLIdx, ... % Product output type info
    AccMdIdx, AccWLIdx, AccFLIdx, ... % Accumulator type info
    OutMdIdx, OutWLIdx, OutFLIdx, ... % Output type info
    PPvMdIdx, PPvWLIdx, PPvFLIdx, ... % Product polyval type info
    APvMdIdx, APvWLIdx, APvFLIdx, ... % Accum polyval type info
    MpcMdIdx, MpcWLIdx, MpcFLIdx  ... % Multiplicand polyval type info
    ] = dspblkGetMaskParamIndicesFromParamNames(blkh, ...
    'L', 'FarrowL', 'R', 'alpha',     ...
    'fallbackFIR', 'fallbackFarrow',  ...
    'firstCoeffMode',  'firstCoeffWordLength',  'firstCoeffFracLength', ...
    'prodOutputMode',  'prodOutputWordLength',  'prodOutputFracLength', ...
    'accumMode',       'accumWordLength',       'accumFracLength',      ...
    'outputMode',      'outputWordLength',      'outputFracLength',     ...
    'interProdMode',   'interProdWordLength',   'interProdFracLength',  ...
    'secondCoeffMode', 'secondCoeffWordLength', 'secondCoeffFracLength',...
    'memoryMode',      'memoryWordLength',      'memoryFracLength');

% -------------------
% Main Tab Parameters
% -------------------

modeStr  = get_param(blkh, 'modeActive');
isLinear = strcmp(modeStr, 'Linear');
isFIR    = strcmp(modeStr, 'FIR');

if isLinear
    % Linear Main Parameters
    mask_vis_new{LNFrwIdx} = 'off';
    mask_vis_new{LFrwIdx}  = 'off';
    mask_vis_new{RIdx}     = 'off';
    mask_vis_new{AlphaIdx} = 'off';
    mask_vis_new{FbFIRIdx} = 'off';
    mask_vis_new{FbFrwIdx} = 'off';
    % Linear Data Type Parameters
    mask_vis_new{CofMdIdx} = 'on';
    mask_vis_new{PrdMdIdx} = 'on';
    mask_vis_new{AccMdIdx} = 'on';
    mask_vis_new{OutMdIdx} = 'on';
    mask_vis_new{PPvMdIdx} = 'off';
    mask_vis_new{APvMdIdx} = 'off';
    mask_vis_new{MpcMdIdx} = 'off';
elseif isFIR
    % FIR Main Parameters
    mask_vis_new{LNFrwIdx} = 'on';
    mask_vis_new{LFrwIdx}  = 'off';
    mask_vis_new{RIdx}     = 'on';
    mask_vis_new{AlphaIdx} = 'on';
    mask_vis_new{FbFIRIdx} = 'on';
    mask_vis_new{FbFrwIdx} = 'off';
else
    % Farrow Main Parameters
    mask_vis_new{LNFrwIdx} = 'off';
    mask_vis_new{LFrwIdx}  = 'on';
    mask_vis_new{RIdx}     = 'off';
    mask_vis_new{AlphaIdx} = 'off';
    mask_vis_new{FbFIRIdx} = 'off';
    mask_vis_new{FbFrwIdx} = 'on';
end

% ------------------------
% Data Type Tab Parameters
% ------------------------

coeffModeStr  = get_param(blkh, 'firstCoeffMode');
prdOutModeStr = get_param(blkh, 'prodOutputMode');
accumModeStr  = get_param(blkh, 'accumMode');
outputModeStr = get_param(blkh, 'outputMode');

allDTParamWLsShownAreInherited = true; % default
allDTParamFLsShownAreInherited = true; % default

% Coefficients
mask_vis_new{CofMdIdx} = 'on';
if strcmp(coeffModeStr, 'Same word length as input')
    mask_vis_new{CofWLIdx} = 'off'; % Inherited
    mask_vis_new{CofFLIdx} = 'off'; % Inherited
elseif strcmp(coeffModeStr, 'Specify word length')
    mask_vis_new{CofWLIdx} = 'on';  % Specified WL
    mask_vis_new{CofFLIdx} = 'off'; % Best precision
    allDTParamWLsShownAreInherited = false;
else
    mask_vis_new{CofWLIdx} = 'on';  % Specified WL
    mask_vis_new{CofFLIdx} = 'on';  % Specified FL
    allDTParamWLsShownAreInherited = false;
    allDTParamFLsShownAreInherited = false;
end

% Product output
mask_vis_new{PrdMdIdx} = 'on';
if strcmp(prdOutModeStr, 'Same as first input')
    mask_vis_new{PrdWLIdx} = 'off'; % Inherited
    mask_vis_new{PrdFLIdx} = 'off'; % Inherited
else
    mask_vis_new{PrdWLIdx} = 'on';  % Specified WL
    mask_vis_new{PrdFLIdx} = 'on';  % Specified FL
    allDTParamWLsShownAreInherited = false;
    allDTParamFLsShownAreInherited = false;
end

% Accumulator
mask_vis_new{AccMdIdx} = 'on';
if ( strcmp(accumModeStr, 'Same as product output') || ...
        strcmp(accumModeStr, 'Same as first input') )
    mask_vis_new{AccWLIdx} = 'off'; % Inherited
    mask_vis_new{AccFLIdx} = 'off'; % Inherited
else
    mask_vis_new{AccWLIdx} = 'on';  % Specified WL
    mask_vis_new{AccFLIdx} = 'on';  % Specified FL
    allDTParamWLsShownAreInherited = false;
    allDTParamFLsShownAreInherited = false;
end

% Output
mask_vis_new{OutMdIdx} = 'on';
if ( strcmp(outputModeStr, 'Same as accumulator') || ...
        strcmp(outputModeStr, 'Same as first input') )
    mask_vis_new{OutWLIdx} = 'off'; % Inherited
    mask_vis_new{OutFLIdx} = 'off'; % Inherited
else
    mask_vis_new{OutWLIdx} = 'on';  % Specified WL
    mask_vis_new{OutFLIdx} = 'on';  % Specified FL
    allDTParamWLsShownAreInherited = false;
    allDTParamFLsShownAreInherited = false;
end

% HIDE or SHOW Farrow Data Type Properties and Parameters

blkMaskObj = Simulink.Mask.get(blkh);
PPvDTLabel = blkMaskObj.getDialogControl('interProdLabel');
APvDTLabel = blkMaskObj.getDialogControl('secondCoeffLabel');
MpcDTLabel = blkMaskObj.getDialogControl('memoryLabel');

if isLinear || isFIR
    
    % HIDE Farrow Data Types
    
    % --- PROPERTIES ---
    PPvDTLabel.Visible = 'off'; % Product output polyval label
    APvDTLabel.Visible = 'off'; % Accumulator polyval label
    MpcDTLabel.Visible = 'off'; % Multiplicand polyval label
    
    % --- PARAMETERS --- 
    % Product output polyval
    mask_vis_new{PPvMdIdx} = 'off';
    mask_vis_new{PPvWLIdx} = 'off';
    mask_vis_new{PPvFLIdx} = 'off';
    % Accumulator polyval
    mask_vis_new{APvMdIdx} = 'off';
    mask_vis_new{APvWLIdx} = 'off';
    mask_vis_new{APvFLIdx} = 'off';
    % Multiplicand polyval
    mask_vis_new{MpcMdIdx} = 'off';
    mask_vis_new{MpcWLIdx} = 'off';
    mask_vis_new{MpcFLIdx} = 'off';
    
else
    
    % SHOW Farrow Data Types
    
    % --- PROPERTIES ---
    PPvDTLabel.Visible = 'on'; % Product output polyval label
    APvDTLabel.Visible = 'on'; % Accumulator polyval label
    MpcDTLabel.Visible = 'on'; % Multiplicand polyval label
    
    % --- PARAMETERS --- 
    % Product output polyval
    mask_vis_new{PPvMdIdx} = 'on';
    prdPlvlModeStr = get_param(blkh, 'interProdMode');
    if strcmp(prdPlvlModeStr, 'Same as first input')
        mask_vis_new{PPvWLIdx} = 'off'; % Inherited
        mask_vis_new{PPvFLIdx} = 'off'; % Inherited
    else
        mask_vis_new{PPvWLIdx} = 'on';  % Specified WL
        mask_vis_new{PPvFLIdx} = 'on';  % Specified FL
    end
    % Accumulator polyval
    mask_vis_new{APvMdIdx} = 'on';
    accPlvlModeStr = get_param(blkh, 'secondCoeffMode');
    if strcmp(accPlvlModeStr, 'Same as first input')
        mask_vis_new{APvWLIdx} = 'off'; % Inherited
        mask_vis_new{APvFLIdx} = 'off'; % Inherited
    else
        mask_vis_new{APvWLIdx} = 'on';  % Specified WL
        mask_vis_new{APvFLIdx} = 'on';  % Specified FL
        allDTParamWLsShownAreInherited = false;
        allDTParamFLsShownAreInherited = false;
    end
    % Multiplicand polyval
    mask_vis_new{MpcMdIdx} = 'on';
    mpcPlvlModeStr = get_param(blkh, 'memoryMode');
    if strcmp(mpcPlvlModeStr, 'Same as first input')
        mask_vis_new{MpcWLIdx} = 'off'; % Inherited
        mask_vis_new{MpcFLIdx} = 'off'; % Inherited
    else
        mask_vis_new{MpcWLIdx} = 'on';  % Specified WL
        mask_vis_new{MpcFLIdx} = 'on';  % Specified FL
        allDTParamWLsShownAreInherited = false;
        allDTParamFLsShownAreInherited = false;
    end
    
end

% Update visibility of 'Word length' and 'Fraction length' column labels
WLColTextLabel = blkMaskObj.getDialogControl('WordLengthLabel');
FLColTextLabel = blkMaskObj.getDialogControl('FracLengthLabel');
if allDTParamWLsShownAreInherited
    WLColTextLabel.Visible = 'off';
else
    WLColTextLabel.Visible = 'on';
end
if allDTParamFLsShownAreInherited
    FLColTextLabel.Visible = 'off';
else
    FLColTextLabel.Visible = 'on';
end

% Update mask parameter visibilities (if changed) on dialog UI
if ~isequal(mask_vis_new, mask_vis_old)
    set_param(blkh, 'MaskVisibilities', mask_vis_new);
end

end % updateVFDBlockDynamicDialogUI

% -------------------------------------------------------------------------
function updateBlockDminDmaxText(blkh)

InputProcessing = get_param(blkh, 'InputProcessing');
modeStr         = get_param(blkh, 'modeActive');
isLinear        = strcmp(modeStr, 'Linear');
isFIR           = strcmp(modeStr, 'FIR');
noDFTOnOff      = get_param(blkh, 'noDFT');
noDFTBool       = strcmp(noDFTOnOff, 'on');

% Form dmintext
% -------------
dmintext = [];
if isLinear
    minval = 0; % fill in dmintext below
elseif isFIR
        fallback = strcmp(get_param(blkh,'fallbackFIR'), ...
            'Switch to linear interpolation if kernel cannot be centered');
        if (fallback)
            minval = 0; % fill in dmintext below
        else
            FIRLStr = get_param(blkh, 'L');
            try
                minval = slResolve(FIRLStr, blkh);
                if ~isempty(minval)
                    minval = minval-1; % fill in dmintext below
                else
                    if (isempty(FIRLStr))
                        dmintext = formDminForFIR(noDFTBool, InputProcessing, 'P');
                    else
                        dmintext = formDminForFIR(noDFTBool, InputProcessing, FIRLStr);
                    end
                end
            catch %#ok<*CTCH>
                if (isempty(FIRLStr))
                    dmintext = formDminForFIR(noDFTBool, InputProcessing, 'P');
                else
                    dmintext = formDminForFIR(noDFTBool, InputProcessing, FIRLStr);
                end
            end
        end
else
    % Farrow
    fallback = strcmp(get_param(blkh, 'fallbackFarrow'), ...
        'Use off-centered kernel');
    if (fallback)
        minval = 0; % fill in dmintext below
    else
        FarrowLStr = get_param(blkh, 'FarrowL');
        try
            minval = slResolve(FarrowLStr, blkh);
            if ~isempty(minval)
                minval = floor(minval/2)-1; % fill in dmintext below
            else
                if (isempty(FarrowLStr))
                    dmintext = formDminForFarrow(noDFTBool, InputProcessing, 'N');
                else
                    dmintext = formDminForFarrow(noDFTBool, InputProcessing, FarrowLStr);
                end
            end
        catch
            if (isempty(FarrowLStr))
                dmintext = formDminForFarrow(noDFTBool, InputProcessing, 'N');
            else
                dmintext = formDminForFarrow(noDFTBool, InputProcessing, FarrowLStr);
            end
        end
    end
end
% Catch all cases for numeric value handling (empty dmintext)
if isempty(dmintext)
    dmintext = formDminForNumeric(noDFTBool, InputProcessing, minval);
end

% Form dmaxtext
% -------------
DmaxPrmValStr = get_param(blkh, 'dmax');
if isempty(DmaxPrmValStr)
    dmaxtext = 'Dmax: Unset';
else
    try
        maxval = slResolve(DmaxPrmValStr, blkh);
        if isempty(maxval)
            % We don't want to error out on [], undefined variables, etc.
            dmaxtext = ['Dmax: ', DmaxPrmValStr];
        else
            dmaxtext = sprintf('Dmax: %d', maxval);
        end
    catch
        % We don't want to error out on defined variables.       
        dmaxtext = ['Dmax: ', DmaxPrmValStr];
    end
end

% Update the block dialog Dmin and Dmax UI text labels
% ----------------------------------------------------
blkMaskObj = Simulink.Mask.get(blkh);
dminTxtObj = blkMaskObj.getDialogControl('DminLabel');
dmaxTxtObj = blkMaskObj.getDialogControl('DmaxLabel');

dminTxtObj.Prompt = dmintext;
dmaxTxtObj.Prompt = dmaxtext;

end % updateBlockDminDmaxText

% -------------------------------------------------------------------------
% Form a dmintext string when dmin is a computed numeric value.
function dmintext = formDminForNumeric(noDFTBool, InputProcessing, minval)
if (noDFTBool)
    if strcmp(InputProcessing, 'Columns as channels (frame based)')            
        if minval > 0
            dmintext = sprintf('Dmin: %d + frame length', minval);
        else
            dmintext = 'Dmin: (frame length)';
        end
    elseif strcmp(InputProcessing, 'Elements as channels (sample based)')
        dmintext = sprintf('Dmin: %d',minval+1);
    else % Inherited (this choice to be removed - see release notes)
        dmintext = sprintf('Dmin: %d  (For frame-based input, add ''frame length - 1'' to Dmin)',minval+1);         
    end
else
    dmintext = sprintf('Dmin: %d', minval);
end
end % formDminForNumeric

% -------------------------------------------------------------------------
% For FIR filter, form a dmintext string when dmin cannot yet be evaluated.
function dmintext = formDminForFIR(noDFTBool, InputProcessing, str)
if (noDFTBool)
    if strcmp(InputProcessing, 'Columns as channels (frame based)')   
        dmintext = ['Dmin: ', '(', str, '-1) + Frame length'];  
    elseif strcmp(InputProcessing, 'Elements as channels (sample based)')    
        dmintext =  ['Dmin: ',str];  
    else % Inherited (this choice to be removed - see release notes)    
        dmintext = sprintf('Dmin: %s  (For frame-based input, add ''frame length - 1'' to Dmin)', str);          
    end
else
    dmintext = ['Dmin: ', '(', str, '-1)'];
end
end % formDminForFIR

% -------------------------------------------------------------------------
% For Farrow filter, form a dmintext string when dmin cannot yet be evaluated.
function dmintext = formDminForFarrow(noDFTBool, InputProcessing, str)
if (noDFTBool)
    if strcmp(InputProcessing, 'Columns as channels (frame based)')   
        dmintext = ['Dmin: ', 'floor(', str, '/2) - 1 + Frame length'];  
    elseif strcmp(InputProcessing, 'Elements as channels (sample based)')    
        dmintext =  ['Dmin: ', 'floor(', str, '/2)'];
    else % Inherited (this choice to be removed - see release notes)    
        dmintext = sprintf('Dmin: floor(%s/2)  (For frame-based input, add ''frame length - 1'' to Dmin)', str);          
    end
else
    dmintext = ['Dmin: ', 'floor(', str, '/2) - 1'];
end 
end % formDminForFarrow
