function [outData] = SPCDelayTransform(inData)
%SPCDelayTransform Forward DSP Delay Block to Simulink Delay Block
%   This is an internal function called by Simulink(R) during model load.

% Copyright 2014-2020 The MathWorks, Inc.

% Do not forward to Simulink Delay if a functionality match is not guaranteed
if ~TransformPossible(inData)
    outData.NewBlockPath = 'dspobslib/Delay';
    outData.NewInstanceData = inData.InstanceData;    
    return
end

% Forward to Simulink Delay
outData.NewBlockPath = 'simulink/Discrete/Delay';
% Set new block parameters
outData.NewInstanceData        = localNVPair('DelayLengthSource', 'Dialog');
outData.NewInstanceData(end+1) = localNVPair('DelayLength', ...
                                 localGetVal(inData.InstanceData, 'delay', '1'));
outData.NewInstanceData(end+1) = localNVPair('InitialConditionSource',...
                                             'Dialog');
outData.NewInstanceData(end+1) = localNVPair('InitialCondition', ...
                                 localGetVal(inData.InstanceData,...
                                 'ic', '0'));
outData.NewInstanceData(end+1) = localNVPair('ExternalReset', 'None');
inputProcessing =                localGetVal(inData.InstanceData, 'InputProcessing', ...
                                 'Inherited (this choice will be removed - see release notes)');
if contains(inputProcessing,'Inherit') % Change the string to match Simulink's string
   inputProcessing = 'Inherited';
end
outData.NewInstanceData(end+1) = localNVPair('InputProcessing',inputProcessing);

delay = localGetVal(inData.InstanceData, 'delay', '1');
[delayVal,status] = safeEval(delay);
if status && delayVal==1 && strcmp(inputProcessing,'Elements as channels (sample based)')
    % Turn on circular buffer. The block goes into rate transition mode
    % otherwise.
    outData.NewInstanceData(end+1) = localNVPair('UseCircularBuffer', 'on');
end

end
%--------------------------------------------------------------------------

function transformOK = TransformPossible(inData)
% This function returns true if a functionality match is not guaranteed
% with forwarding. 

transformOK = true;

delay =  localGetVal(inData.InstanceData, 'delay', '1');
[delayVal,status] =  safeEval(delay);
if ~status
    transformOK = false;
    return;
else
    if ~isscalar(delayVal)
        transformOK = false;
        return;
    end
end

% No transformation if reset port is active (Simulink does not support
% data and reset ports with different rates) 
resetVal = localGetVal(inData.InstanceData, 'reset_popup', 'None');
if ~strcmp(resetVal,'None')
    transformOK = false;
    return;
end

% No support for delay in frame units
delay_units = localGetVal(inData.InstanceData, 'dly_unit', 'Samples');
inputProcessing = localGetVal(inData.InstanceData, 'InputProcessing', 'Inherited (this choice will be removed - see release notes)');
if ~strcmp(delay_units,'Samples') && ~strcmp(inputProcessing,'Elements as channels (sample based)')
    transformOK = false;
    return;
end

% No support for partial or full initial conditions
dif_ic_for_ch  = localGetVal(inData.InstanceData, 'dif_ic_for_ch', 'off');
dif_ic_for_dly = localGetVal(inData.InstanceData, 'dif_ic_for_dly', 'off');
if isPartialIC(dif_ic_for_ch,dif_ic_for_dly) || ...
    isFullIC(dif_ic_for_ch, dif_ic_for_dly)
    transformOK = false;
    return;  
end

end
%--------------------------------------------------------------------------

function entry = localNVPair(name, value)
entry = struct('Name', name, 'Value', value);
end

%--------------------------------------------------------------------------

function flag = isPartialIC(dif_ic_for_ch, dif_ic_for_dly)
% Return true if block is set for partial IC 
flag = (strcmp(dif_ic_for_ch,'on') && strcmp(dif_ic_for_dly,'off')) || ...
       (strcmp(dif_ic_for_ch,'off') && strcmp(dif_ic_for_dly,'on')) ;
end

%--------------------------------------------------------------------------

function flag = isFullIC(dif_ic_for_ch, dif_ic_for_dly)
% Return true if block is set for full IC 
flag = (strcmp(dif_ic_for_ch,'on') && strcmp(dif_ic_for_dly,'on'));
end

%--------------------------------------------------------------------------

function index = localParamIndex(InstanceData, paramName)
% Search paramName for index of paramName
index = -1; % Means not found

for i=1:length(InstanceData)
    if strcmp(InstanceData(i).Name, paramName)
        index = i;
        break;
    end
end
end

%--------------------------------------------------------------------------

function value = localGetVal(InstanceData, paramName, defaultValue)
% Attempt to get value from InstanceData. But if it is not in the
% InstanceData, set it to the library default value.

index = localParamIndex(InstanceData, paramName);
% Use the default value if the parameter is not found in the structure
if index == -1
    value = defaultValue;
else
    value = InstanceData(index).Value;
end

end
