function outData = audioDeviceIOTransform(inData)
% audioDeviceIOTransform Function populates Driver and Device for
% Audio Device Input block, based on the current machine's configuration.
%
%   This is an internal function called by Simulink(R) during model load.

% Copyright 2015-2020 The MathWorks, Inc.

InstanceData = inData.InstanceData;
outData.NewInstanceData = InstanceData;

% Get indices of Driver and Device parameters
DriverIdx = arrayfun(@(x)strcmp(x.Name, 'Driver'),InstanceData);
savedDriverStr = InstanceData(DriverIdx).Value;
DeviceIdx = arrayfun(@(x)strcmp(x.Name, 'Device'),InstanceData);
savedDeviceStr = InstanceData(DeviceIdx).Value;

[DriverStr, driverChangedFlag] = setAudioDriver(savedDriverStr);
if driverChangedFlag
    % Driver needs to be changed. Set Driver to new value.
    outData.NewInstanceData(DriverIdx).Value = DriverStr;
end

[DeviceStr, deviceChangedFlag] = setAudioDevice(savedDeviceStr, ...
            DriverStr, inData.ForwardingTableEntry.('__slOldName__'));
        
DeviceWarnFlagIdx = arrayfun(@(x)strcmp(x.Name, 'DeviceWarnFlag'),InstanceData);

outData.NewInstanceData(DeviceWarnFlagIdx).Value = ''''''; % Reset flag
if deviceChangedFlag
    % Device needs to be changed. Set Device to new value and set
    % DeviceWarnFlag to old device name.
    outData.NewInstanceData(DeviceIdx).Value = DeviceStr;
    
    % Do not warn if model was saved on a system with no audio devices, or
    % if current system has no audio devices
    noInputDeviceStr = 'No audio input device detected';
    noOutputDeviceStr = 'No audio output device detected';
    if ~any(strcmp(savedDeviceStr, {noInputDeviceStr, noOutputDeviceStr})) && ...
            ~any(strcmp(DeviceStr, {noInputDeviceStr, noOutputDeviceStr}))
        outData.NewInstanceData(DeviceWarnFlagIdx).Value = savedDeviceStr;
    end
end

outData.NewBlockPath = inData.ForwardingTableEntry.('__slOldName__');

end

function [newDriver, driverChangedFlag] = setAudioDriver(currentDriver)
    
driverChangedFlag = false;
availableDrivers = dsp.internal.getAudioIODriversAndDevices('drivers');

if ~any(strcmp(currentDriver, availableDrivers))
    newDriver = availableDrivers{1};
    driverChangedFlag = true;
else
    newDriver = currentDriver;
end

end

function [newDevice, deviceChangedFlag] = setAudioDevice(currentDevice, currentDriver, blkPath)

deviceChangedFlag = false;
if contains(blkPath, 'audiosources')
    % Audio Device Input block
    availableDevices = dsp.internal.getAudioIODriversAndDevices('inputDevices', currentDriver);
else
    % Audio Device Output block
    availableDevices = dsp.internal.getAudioIODriversAndDevices('outputDevices', currentDriver);
end

if ~any(strcmp(currentDevice, availableDevices))
    newDevice = availableDevices{1};
    deviceChangedFlag = true;
else
    newDevice = currentDevice;
end

end

