function [outData] = dspFIRMultirateBlockTransform(inData)
% dspFIRMultirateBlockTransform preserves functionality for legacy FIR multirate blocks (dspmlti4).
%
%   This is an internal function called by Simulink(R) during model load.

% Copyright 2015-2022 The MathWorks, Inc.

origBlockPath        = inData.ForwardingTableEntry.('__slOldName__');
outData.NewBlockPath = origBlockPath;
newInstanceData      = inData.InstanceData;

% Obsolete parameters REMOVED (no longer used):
%  (15b) 'additionalParams', 'allowOverrides'
newInstanceData = removeObsoleteParameter(newInstanceData, 'additionalParams');
newInstanceData = removeObsoleteParameter(newInstanceData, 'allowOverrides');

% 'FilterSource' parameter settings:
%  (15b) Map 'Specify via dialog' choice to 'Dialog parameters'
newInstanceData = mapObsoleteParamSetting(newInstanceData, ...
    'FilterSource', 'Specify via dialog', 'Dialog parameters');

if ~contains(origBlockPath, sprintf('FIR Rate\nConversion'))
    % FIR Decimation or FIR Interpolation

    % 'framing' parameter settings:
    %  (15b) Map 'Maintain input frame size' choice to 'Allow multirate processing'
    %  (15b) Map 'Maintain input frame rate' choice to 'Enforce single-rate processing'
    newInstanceData = mapObsoleteParamSetting(newInstanceData, ...
        'framing', 'Maintain input frame size', 'Allow multirate processing');
    newInstanceData = mapObsoleteParamSetting(newInstanceData, ...
        'framing', 'Maintain input frame rate', 'Enforce single-rate processing');

    % 'InputProcessing' parameter settings: Special-case handling for
    %   'Inherited (this choice will be removed - see release notes)'
    [inheritMode, index] = ...
        isInputProcessingSetToInherited(newInstanceData);

    if inheritMode
     newInstanceData(index) =  struct('Name', 'InputProcessing', 'Value', 'Columns as channels (frame based)');
    end
    
end

outData.NewInstanceData = newInstanceData;

if contains(origBlockPath, sprintf('FIR Rate\nConversion')) || contains(origBlockPath, sprintf('FIR\nDecimation'))
    % FIR Rate Conversion or FIR Decimation

    %  (22b) new parameter 'AllowArbitraryInputLength'
    EnableArbitraryIdx = find(ismember({newInstanceData.Name},'AllowArbitraryInputLength'),1);
    % Populate a newly created AllowArbitraryInputLength with false
    if isempty(EnableArbitraryIdx)
        outData.NewInstanceData = [outData.NewInstanceData struct('Name','AllowArbitraryInputLength','Value','off')]; 
    end
end

end % dspFIRMultirateBlockTransform

% -------------------------------------------------------------------------
function newInstanceData = removeObsoleteParameter(newInstanceData, parameterName)
for index=1:length(newInstanceData)
    if strcmp(newInstanceData(index).Name, parameterName)
        % Remove the parameter
        newInstanceData = [newInstanceData(1:index-1) newInstanceData(index+1:end)];
        break;
    end
end
end

% -------------------------------------------------------------------------
function newInstanceData = mapObsoleteParamSetting(newInstanceData, parameterName, oldSetting, newSetting)
for index=1:length(newInstanceData)
    if strcmp(newInstanceData(index).Name, parameterName)
        if strcmp(newInstanceData(index).Value, oldSetting)
            % Update the parameter setting value
            newInstanceData(index).Value = newSetting;
        end
        break;
    end
end
end

% -------------------------------------------------------------------------
function [inheritMode, index] = isInputProcessingSetToInherited(newInstanceData)

inheritMode = true;
for index=1:length(newInstanceData)
    if strcmpi(newInstanceData(index).Name, 'inputprocessing')
        % CAN BE SPELLED inputProcessing OR InputProcessing
        inheritMode = strcmp(newInstanceData(index).Value, ...
            'Inherited (this choice will be removed - see release notes)');
        return;
    end
end
index = index + 1;

end
