function outData = dspVectorScopeTransform(inData)
% dspVectorScopeTransform Vector Scope block transform
%
% This is an internal function called by Simulink(R) during model load.

% Copyright 2017-2022 The MathWorks, Inc.

instanceData = inData.InstanceData;
% Find the Domain property index
domainIdx = ismember({instanceData.Name},'Domain');
% Is the scope in viewer mode
isViewerMode = strcmpi(get_param(gcb,'iotype'), 'viewer');
% Based on the specified domain change the block
switch instanceData(domainIdx).Value
    case 'Time'
        outData = mapVectorScopeParamsToTimeScopeParams(instanceData, isViewerMode);
    case 'Frequency'
        outData = mapVectorScopeParamsToSpectrumAnalyzerParams(instanceData, isViewerMode);
    case 'User-defined'
        outData = mapVectorScopeParamsToArrayPlotParams(instanceData, isViewerMode);
    otherwise
        % Add the vector scope flag to the InstanceData struct
        outData.NewInstanceData = [struct('Name','IsSourceVectorScope','Value','off') instanceData];
        outData.NewBlockPath = inData.ForwardingTableEntry.('__slOldName__');
end
end
%--------------------------------------------------------------------------
function outData = mapVectorScopeParamsToTimeScopeParams(instanceData, isViewerMode)
% Map Vector Scope parameters to the equivalent Time Scope parameters
% Add the vector scope flag to the InstanceData struct
outData = mapVectorScopeParamsToScopeParams(instanceData, {}, {}, false);
if isViewerMode
    outData.NewBlockPath = 'dspviewers/Scope';
else
    outData.NewBlockPath = ['dspsnks4/Time' newline 'Scope'];
end
end
%--------------------------------------------------------------------------
function outData = mapVectorScopeParamsToSpectrumAnalyzerParams(instanceData, isViewerMode)
% Map Vector Scope parameters to the equivalent Spectrum Analyzer parameters
% Add the vector scope flag to the InstanceData struct
outData = mapVectorScopeParamsToScopeParams(instanceData, ...
    {'Domain','XRange','InheritXIncr','XIncr','YUnits','XMin','XMax'}, ...
    {'InputDomain','PlotAsTwoSidedSpectrum','SampleRateSource','SampleRate','SpectrumUnits','StartFrequency','StopFrequency'}, ...
    false);
outData = updateSpectrumAnalyzerParamValues(outData);
if isViewerMode
    outData.NewBlockPath = ['dspviewers/Spectrum' newline 'Analyzer'];
else
    outData.NewBlockPath = ['dspsnks4/Spectrum' newline 'Analyzer'];
end
end
%--------------------------------------------------------------------------
function outData = mapVectorScopeParamsToArrayPlotParams(instanceData, isViewerMode)
% Map Vector Scope parameters to the equivalent Array Plot parameters
% Add the vector scope flag to the InstanceData struct
% Map Parameters
outData = mapVectorScopeParamsToScopeParams(instanceData, ...
    {'XIncr', 'XLabel'}, ...
    {'SampleIncrement','XLabel'}, ...
    true);
% Map Style
if isViewerMode
    outData.NewBlockPath = 'dspviewers/Array Plot';
else
    outData.NewBlockPath = 'dspsnks4/Array Plot';
end
end
%--------------------------------------------------------------------------
function outData = mapVectorScopeParamsToScopeParams(instanceData, vectorScopeParams, scopeParams, iswebscope)
% Map vector scope parameters to the replacement scope parameters
if iswebscope
    outData.NewInstanceData = struct('Name','IsSourceVectorScope','Value','on');
else
    outData.NewInstanceData = [struct('Name','IsSourceVectorScope','Value','on') instanceData];
    return;
end
vectorScopeParams = [vectorScopeParams {'AxisGrid', ...
    'AxisLegend', ...
    'OpenScopeAtSimStart', ...
    'OpenScopeImmediately', ...
    'FigPos', ...
    'YLabel', ...
    'YMin'}];

repScopeParams = [scopeParams {'ShowGrid', ...
    'ShowLegend', ...
    'OpenAtSimulationStart', ...
    'Visible', ...
    'WindowPosition', ...
    'YLabel', ...
    'YLimits'}];

for pIdx = 1:numel(vectorScopeParams)
    propIdx = ismember({instanceData.Name},vectorScopeParams{pIdx});
    if any(propIdx)
        if strcmpi(vectorScopeParams{pIdx},'YMin')
            yMaxIdx = ismember({instanceData.Name},'YMax');
            yMaxValue = instanceData(yMaxIdx).Value;
            value = ['[' instanceData(propIdx).Value ',' yMaxValue ']'];
        else
            value = instanceData(propIdx).Value;
        end
        mapStruct = struct('Name',repScopeParams{pIdx},'Value',value);
        outData.NewInstanceData = [outData.NewInstanceData mapStruct];
    end
end
% PlotType is always a "Line" for Vector Scope
plotType = struct('Name','PlotType','Value','Line');
outData.NewInstanceData = [outData.NewInstanceData plotType];
% Map Style
styleStruct = struct('Style',[]);
lineStyleIdx = ismember({instanceData.Name},'LineStyles');
if any(lineStyleIdx)
    value = strsplit(strrep(instanceData(lineStyleIdx).Value,' ',''),'|');
    styleStruct.Style.LineStyle = value;
end
lineMarkerIdx = ismember({instanceData.Name},'LineMarkers');
if any(lineStyleIdx)
    value = strsplit(strrep(instanceData(lineMarkerIdx).Value,' ',''),'|');
    styleStruct.Style.Marker = value;
end
graphicalSettings = struct('Name','GraphicalSettings','Value',jsonencode(styleStruct));
outData.NewInstanceData = [outData.NewInstanceData graphicalSettings];
end
%--------------------------------------------------------------------------
function outData = updateSpectrumAnalyzerParamValues(outData)
    % Add the FrequencySpan parameter
    outData.NewInstanceData(end + 1) = struct('Name','FrequencySpan','Value','Start and stop frequencies');
    % Add the InputUnits parameter
    outData.NewInstanceData(end + 1) = struct('Name','InputUnits','Value','Watts');
    for idx = 1:numel(outData.NewInstanceData)
        paramStruct = outData.NewInstanceData(idx);
        paramName = paramStruct.Name;
        paramValue = paramStruct.Value;
        switch paramName
            case 'SampleRateSource'
                if strcmpi(paramValue, 'off')
                    paramValue = 'Property';
                else
                    paramValue = 'Inherited';
                end
            case 'SampleRate'
                paramValue = num2str(1/slResolve(paramValue, gcb));
            case 'PlotAsTwoSidedSpectrum'
                paramValue = utils.logicalToOnOff(contains(lower(paramValue), '-fs/2...fs/2' ) || contains(lower(paramValue), '[0...fs]'));
            case 'SpectrumUnits'
                if strcmpi(paramValue, 'dB')
                    paramValue = 'dBW';
                else
                    paramValue = 'Watts';
                end
        end
        % Update the param with new value
        paramStruct.Value = paramValue;
        outData.NewInstanceData(idx) = paramStruct;
    end
end

