function SPCDelayRegisterCompileCheck(block, muObj)
% SPCDELAYREGISTERCOMPILECHECK  Forward DSP Delay block to Simulink delay
% block.

% Copyright 2014-2020 The MathWorks, Inc.

appendCompileCheck(muObj, block, @CollectData, @UpdateBlock);

%--------------------------------------------------------------------------
% Collect block information under compile stage
function param = CollectData(block, ~)

% Get frame-ness, dimensions and datatype of input signal
% This information is required to perform mapping
tp = get_param(block, 'CompiledPortFrameData');

% Detect bus signals
[isBus,isMixedFrameness,frameness] = detectBusSignal(tp);

if ~isBus
    blkInfo.isFrame = (tp.Inport == 1);
    blkInfo.isBus = false;
    blkInfo.isMixedFrameness = false;
else
    blkInfo.isFrame =  frameness;
    blkInfo.isBus = true;
    blkInfo.isMixedFrameness = isMixedFrameness;
end

blkInfo.SampleTime = get_param(block, 'CompiledSampleTime');
DT = get_param(block,'CompiledPortDataTypes');
blkInfo.DT = DT.Inport{1};   % data port datatype

dims = get_param(block, 'CompiledPortDimensions');
inport = dims.Inport;
inputProcessing = get_param(block,'InputProcessing');
if ~blkInfo.isBus
    if (strcmp(inputProcessing,'Columns as channels (frame based)')  && ...
            blkInfo.isFrame(1)) % frame-based processing
        if inport(1)==1 % 1D data
            blkInfo.NumChans = 1;
            blkInfo.FrameSize = inport(2);
        else
            blkInfo.NumChans = prod(inport(3:2+inport(1)-1));
            blkInfo.FrameSize = inport(2);
        end
        blkInfo.dims = [];
    else % sample-based processing
        blkInfo.NumChans = prod(inport(2:2+inport(1)-1));
        blkInfo.FrameSize = [];
        blkInfo.dims = inport(2:2+inport(1)-1);
    end
else
    % Not required for bus signals
    blkInfo.NumChans = [];
    blkInfo.FrameSize = [];
    blkInfo.dims = [];
end

% Transfer block parameters to param structure
blockParamStruct = get_param(block,'DialogParameters');
paramNames = fieldnames(blockParamStruct);
for i=1:length(paramNames)
    param.(paramNames{i}) =  get_param(block,paramNames{i});
end
param.isFrame =  blkInfo.isFrame;
param.SampleTime = blkInfo.SampleTime;
param.NumChans   = blkInfo.NumChans;
param.DT =  blkInfo.DT;
param.FrameSize =  blkInfo.FrameSize;
param.dims =  blkInfo.dims;
param.OutportDims =   dims.Outport(2:2 + dims.Outport(1)-1);
param.isBus = blkInfo.isBus;
param.isMixedFrameness = blkInfo.isMixedFrameness;
    
%--------------------------------------------------------------------------
function UpdateBlock(block, h, param)

if askToReplace(h, block)
    if noTransformPossible(param,block)
        funcSet = '';
        ReplaceBlockReasonStr = '';
    else
        funcSet =  {'muUpdateBlock',block, param};
        if (doUpdate(h))
            muUpdateBlock(block,param);
        end
        ReplaceBlockReasonStr = 'Replace deprecated Delay block';
    end
    
    if ~isempty(funcSet)
        appendTransaction(h, block, ReplaceBlockReasonStr, {funcSet});
    end
end

%--------------------------------------------------------------------------
function flag = noTransformPossible(param,oldBlock)
% Return true of no transformation is possible. This is the case where
% Delay has functionality that is not available in the target
% blocks
flag = false;

% Zero delay
delay = slResolve(param.delay,oldBlock);
if isscalar(delay) && (delay==0)
    flag = true;
    return;
end

% Reset port and data port operating at different rates
if iscell(param.SampleTime)
    flag = true;
    return;
end

ICForEachChan = strcmp(param.dif_ic_for_ch,'on');
IcWithinChan  = strcmp(param.dif_ic_for_dly,'on');
isNotScalarIC = ~(~ICForEachChan && ~IcWithinChan);

% Bus signal - non-scalar initial conditions
if param.isBus && isNotScalarIC
    flag = true;
    return;
end

% Bus signal - delay in frame units.
% when Simulink block relaxes error when input is frame and block is
% sample-based, mapping will be to flip block to sample-based
if param.isBus && strcmp(param.dly_unit,'Frames') && ...
   strcmp(param.InputProcessing , 'Columns as channels (frame based)')
    flag = true;
    return;
end

% Bus signal - non-scalar delay
delay = slResolve(param.delay,oldBlock);
if param.isBus && ~isscalar(delay)
    flag = true;
    return;
end

%--------------------------------------------------------------------------
function  muUpdateBlock(oldBlock, param)

% load simulink
load_system('simulink');

% Map Input Processing
if strcmp(param.InputProcessing,'Inherited (this choice will be removed - see release notes)')
    param.InputProcessing = 'Elements as channels (sample based)';
end
inputProcessing = param.InputProcessing;

% Do not modify input processing for the case where the input is a bus
% signal with mixed frame-ness
if ~(param.isBus && param.isMixedFrameness)
    param.InputProcessing = inputProcessing;
end

% The old block's name and parent are needed for the new block
name  = get_param(oldBlock,'Name');
parent = get_param(oldBlock,'Parent');
param.parent = parent;
param.name = strrep(name,'/','//');

% Create a temp block - if everything works fine replace the old block with
% this one.
tempBlock = [parent '/temp'];

%The decorations must be preserved
param.decorations = getDecorationParams(oldBlock);
param.mask={};

delay = slResolve(param.delay,oldBlock);
% Delay specified in frame units - convert to samples
if strcmp(param.dly_unit,'Frames') && strcmp(param.InputProcessing , 'Columns as channels (frame based)')
    delay = delay * param.FrameSize;
    param.delay = dsp.internal.convertToChar(delay);
end
param.isScalarDelay =  isscalar(delay);
param.icVal = slResolve(param.ic,oldBlock);

if isscalar(delay)
    mapScalarDelay(oldBlock, param, tempBlock);
else
    mapNonScalarDelay(oldBlock, param, tempBlock);
end

% Fix edge cases
fixEdgeCases(param);

%--------------------------------------------------------------------------

function mapScalarDelay(oldBlock, param, tempBlock)
% Replace DSP Delay block by Simulink Delay block
newBlock = 'simulink/Commonly Used Blocks/Delay';
add_block(newBlock, tempBlock);
targetBlock = tempBlock;
delay = param.delay;
DelayLengthSource = 'Dialog';
% Convert Ic to Simulink block format
ic = transformIC(oldBlock,param);

% Set the block parameters
set_param(targetBlock, ...
    'DelayLengthSource',      DelayLengthSource,...
    'DelayLength',            delay, ....
    'ExternalReset',          transformResetString(param.reset_popup), ...
    'InputProcessing',        param.InputProcessing, ...
    'InitialConditionSource', 'Dialog',...
    'InitialCondition',       ic);

% Delete old block and replace with new block
delete_block(oldBlock);
add_block(tempBlock, [param.parent '/' param.name],...
                     param.decorations{:},param.mask{:});
delete_block(tempBlock);

%--------------------------------------------------------------------------

function mapNonScalarDelay(oldBlock, param, tempBlock)
% DSP Delay replaced by iterator subsystem containing Simulink delay
% block

newBlock = 'built-in/SubSystem';
targetBlock = [tempBlock '/Delay'];
add_block(newBlock, tempBlock);

parent = param.parent;
name  = param.name;

% Delay specified on input port
[~ , delaySourceName] = fileparts(tempname);
addBlock(oldBlock,'Pre',40,-30,...
         'dspsrcs4', 'dspsrcs4/Constant',[parent '/' delaySourceName]);

isSampleBased = strcmp(param.InputProcessing,...
                'Elements as channels (sample based)');
delay = slResolve(param.delay,oldBlock);
if ~isSampleBased && ~isrow(delay)
    delayVal = dsp.internal.convertToChar(delay.');
else
    delayVal = param.delay;
end
set_param([parent '/' delaySourceName],'Value',delayVal);

% Ports must be reshaped for sample-based mode to work with For Each
% iterator
if isSampleBased
    [~ , reshapeblk1] = fileparts(tempname);
    addBlock(oldBlock,'Pre',20,0,...
         'simulink', 'built-in/Reshape',[parent '/' reshapeblk1]);
    set_param([parent '/' reshapeblk1],'OutputDimensionality',...
              'Row vector (2-D)')
    [~ , reshapeblk2] = fileparts(tempname);
    addBlock(oldBlock,'Pre',20,-20,...
         'simulink', 'built-in/Reshape',[parent '/' reshapeblk2]);
    set_param([parent '/' reshapeblk2],...
        'OutputDimensionality','Row vector (2-D)')
     [~ , reshapeblk3] = fileparts(tempname);
     addBlock(oldBlock,'Pre',20,-40,...
         'simulink', 'built-in/Reshape',[parent '/' reshapeblk3]);
    numDims = numel(param.OutportDims);
    dimsStr = '';
    for i=1:numDims
        dimsStr = sprintf('%s %s',dimsStr,num2str(param.OutportDims(i)));
    end
    dimsStr = ['[' dimsStr ']'];
    set_param([parent '/' reshapeblk3],'OutputDimensionality','Customize')
    set_param([parent '/' reshapeblk3],'OutputDimensions',dimsStr)
end

icVal = slResolve(param.ic,oldBlock);
% If initial conditions are not scalar, they must be mapped to an input
% port to work with iterator subsystem
if ~isempty(icVal) && ~isscalar(icVal)
    InitialConditionSource = 'Input port';
    [~ , ICInput] = fileparts(tempname);
    addBlock(oldBlock,'Pre',20,-60,...
         'dspsrcs4', 'dspsrcs4/Constant',[parent '/' ICInput]);
    icInputPort = true;
else
    InitialConditionSource = 'Dialog';
    icInputPort = false;
end

param.MaxDelay = num2str(max(reshape(delay,1,numel(delay))));

setupForEachSubsystem(tempBlock,param,icInputPort);

ic = transformIC(oldBlock,param);
if strcmp(InitialConditionSource , 'Input port')
    set_param([parent '/' ICInput],'Value',ic);
    set_param([parent '/' ICInput],'OutDataTypeStr',param.DT)
end

% Set the block parameters
set_param(targetBlock, ...
    'DelayLengthSource',      'Input port',...
    'ExternalReset',          transformResetString(param.reset_popup), ...
    'InputProcessing',        param.InputProcessing, ...
    'InitialConditionSource', InitialConditionSource);
if ~icInputPort
  set_param(targetBlock, 'InitialCondition',param.ic);
end

% Connect blocks
lineHandles = get_param(oldBlock,'LineHandles');
inPorts = lineHandles.Inport;
% Data port
x = get(inPorts(1));
delete_line(x.Handle);
inputblockName = get_param(x.SrcBlockHandle,'Name');
inputblockPort = get_param(x.SrcPortHandle,'PortNumber');

if length(inPorts)==2
    % Reset port
    x = get(inPorts(2));
    delete_line(x.Handle);
    resetblockName = get_param(x.SrcBlockHandle,'Name');
    resetblockPort = get_param(x.SrcPortHandle,'PortNumber');
end

% Out port
outPorts = lineHandles.Outport;
x = get(outPorts(1));
outblockName = get_param(x.DstBlockHandle,'Name');
outblockPort = get_param(x.DstPortHandle,'PortNumber');
delete_line(x.Handle);

% Delete old block and replace with new block
delete_block(oldBlock);
add_block(tempBlock, [param.parent '/' param.name],...
          param.decorations{:},param.mask{:});
delete_block(tempBlock);

if ~isSampleBased
    add_line(parent,[inputblockName '/' num2str(inputblockPort)],...
            [name '/1'],'autorouting','on')
    add_line(parent,[delaySourceName '/1'],[name '/2'],'autorouting','on')
    if strcmp(InitialConditionSource,'Input port')
        if length(inPorts)==2 % reset exists
            add_line(parent,[ICInput '/1'],[name '/4'],'autorouting','on')
        else
            add_line(parent,[ICInput '/1'],[name '/3'],'autorouting','on')
        end
    end
    if length(inPorts)==2
        add_line(parent,[resetblockName '/' num2str(resetblockPort)],...
                [name '/3'],'autorouting','on')
    end
    add_line(parent,[name '/1'],[outblockName '/' num2str(outblockPort)],...
             'autorouting','on')
else
    add_line(parent,[inputblockName '/' num2str(inputblockPort)],...
                    [reshapeblk1 '/1'],'autorouting','on')
    add_line(parent,[reshapeblk1 '/1'],[name '/1'],'autorouting','on')
    
    add_line(parent,[delaySourceName '/1'],[reshapeblk2 '/1'],...
             'autorouting','on')
    add_line(parent,[reshapeblk2 '/1'],[name '/2'],'autorouting','on')
    
    if strcmp(InitialConditionSource,'Input port')
        add_line(parent,[ICInput '/1'],[name '/3'],'autorouting','on')
        resetPort = '4';
    else
        resetPort = '3';
    end
    
    if length(inPorts)==2
        add_line(parent,[resetblockName '/' num2str(resetblockPort)],...
                [name '/' resetPort],'autorouting','on')
    end
    
    add_line(parent,[name '/1'],[reshapeblk3 '/1'],'autorouting','on')
    add_line(parent,[reshapeblk3 '/1'],...
            [outblockName '/' num2str(outblockPort)],'autorouting','on')
end

%--------------------------------------------------------------------------

function setupForEachSubsystem(subsys,param,icInputPort)
% Setup For Each subsystem for the non-scalar Delay case

DataInput = [subsys '/In'];
add_block('built-in/Inport',DataInput,'Position',[50 80 70 100]);
DelayInput = [subsys '/In2'];
add_block('built-in/Inport',DelayInput,'Position',[ 50 165 70 185]);

isReset = ~strcmp(param.reset_popup,'None');
if isReset
    ResetInput = [subsys '/Reset'];
    add_block('built-in/Inport',ResetInput,'Position',[ 50 375 70 395]);
end

if icInputPort
    ICInput = [subsys '/In3'];
    add_block('built-in/Inport',ICInput,'Position',[ 50 555 70 575]);
end

out = [subsys '/out'];
add_block('built-in/Outport',out,'Position',[ 555 305 575 325]);

delay = [subsys '/Delay'];
add_block('simulink/Commonly Used Blocks/Delay', delay,'Position',...
          [355 302 390 338]);
set_param(delay,'DelayLengthSource','Input port')
set_param(delay,'DelayLengthUpperLimit',param.MaxDelay);
set_param(delay,'ExternalReset', transformResetString(param.reset_popup));
 
foreach = [subsys '/For Each Iterator'];
add_block('built-in/ForEach', foreach,'Position',[110 430 130 450]);
if icInputPort
    if ~isReset
        set_param(delay,'InitialConditionSource','Input port')
        set_param(foreach,'InputPartition',{'on','on','on'})
        set_param(foreach,'InputPartitionDimension',{'2','2','2'})
    else
        set_param(delay,'InitialConditionSource','Input port')
        set_param(foreach,'InputPartition',{'on','on','off','on'})
        set_param(foreach,'InputPartitionDimension',{'2','2','2','2'})
    end
else
    if ~isReset
        set_param(foreach,'InputPartition',{'on','on'})
        set_param(foreach,'InputPartitionDimension',{'2','2'})
    else
        set_param(foreach,'InputPartition',{'on','on','off'})
        set_param(foreach,'InputPartitionDimension',{'2','2','2'})
    end
end
set_param(foreach,'OutputConcatenationDimension',{'2'})

add_line(subsys,'In/1','Delay/1','autorouting','on');
add_line(subsys,'In2/1','Delay/2','autorouting','on');
if isReset
    add_line(subsys,'Reset/1','Delay/3','autorouting','on');
end
if icInputPort
    if isReset
        add_line(subsys,'In3/1','Delay/4','autorouting','on');
    else
        add_line(subsys,'In3/1','Delay/3','autorouting','on');
    end
end
add_line(subsys,'Delay/1','out/1','autorouting','on');

%--------------------------------------------------------------------------
function decorations = getDecorationParams(block)
%getDecorationParams Return a cell array with block decorations
%
% Decorations include FontSize, FontWeight, Orientation, etc.
%
decorations = {
    'Position',        [];
    'Orientation',     [];
    'ForegroundColor', [];
    'BackgroundColor', [];
    'DropShadow',      [];
    'NamePlacement',   [];
    'FontName',        [];
    'FontSize',        [];
    'FontWeight',      [];
    'FontAngle',       [];
    'ShowName',        []
    };

for i=1:size(decorations,1)
    decorations{i,2}=get_param(block,decorations{i,1});
end

decorations=reshape(decorations',1,length(decorations(:)));

% -------------------------------------------------------------------------

function strOut = transformResetString(strIn)
% Map old Reset string to new Reset string
switch strIn
    case 'None'
        strOut = 'None';
    case 'Rising edge'
        strOut = 'Rising';
    case 'Falling edge'
        strOut = 'Falling';
    case 'Either edge'
        strOut = 'Either';
    case 'Non-zero sample'
        strOut = 'Level hold';
end

% -------------------------------------------------------------------------

function icOut = transformIC(oldBlock,param)
% Only scalar and "full" IC supported on Simulink Delay block
% Handle case where ic is neither
% On Simulink block, delay is of size numChans x Delay

ic = param.ic;
icVal = slResolve(ic,oldBlock);
if isscalar(icVal) && ~iscell(icVal)
    % Scalar IC - no transformation required
    icOut = ic;
    return;
end

delay = slResolve(param.delay,oldBlock);
ICForEachChan = strcmp(param.dif_ic_for_ch,'on');
IcWithinChan  = strcmp(param.dif_ic_for_dly,'on');

if ~isscalar(delay)
    % for non-scalar delays, we will insert a Variable-Integer Delay
    % Simulink block in a For Each subsystem. The expected full IC format
    % for the Simulink block is maxDelay-by-numchans
    ICSimulink = zeros(eval(param.MaxDelay),param.NumChans);
    
    if ICForEachChan && ~IcWithinChan
        % One scalar IC value for each channel
        % IC from DSP block dimensions:
        % Frame-processing: Vector of length numChans
        % Sample-processing: Same dimensions as input
        icVal = reshape(icVal,1,numel(icVal));
        for i=1:param.NumChans
            ICSimulink(:,i) = icVal(i);
        end
        icOut = dsp.internal.convertToChar(ICSimulink);
    elseif ~ICForEachChan && IcWithinChan
        % All the delays are equal in this case
        % IC is a vector of length delay
        for i=1:length(icVal)
            ICSimulink(i,:) = icVal(i);
        end
         icOut = dsp.internal.convertToChar(ICSimulink);
    else % full IC
        if iscell(icVal)
            for i=1:numel(icVal)
                ICSimulink(end-delay(i)+1:end,i) = icVal{i};
            end
            icOut = dsp.internal.convertToChar(ICSimulink);
        else
            % all the delays are equal (matrix)
            icOut = ic;
        end
    end
    return;
end

% Scalar Delay case
if strcmp(param.InputProcessing,'Columns as channels (frame based)')
    
    % Expected full IC format for frame-based processing
    ICSimulink = zeros(delay,param.NumChans);
    
    if ICForEachChan && ~IcWithinChan
        % One scalar IC value for each channel
        for i=1:length(icVal)
            ICSimulink(:,i) = icVal(i);
        end
        icOut = dsp.internal.convertToChar(ICSimulink);
        return;
    end
    
    if ~ICForEachChan && IcWithinChan
        % IC vector of length Delay
        for i=1:length(icVal)
            ICSimulink(i,:) = icVal(i);
        end
        icOut = dsp.internal.convertToChar(ICSimulink);
        return;
    end
    
    if ICForEachChan && IcWithinChan
        if ~iscell(icVal)
            % IC matrix of size delay x numchans
            ICSimulink = icVal;
        else
            for i=1:length(icVal)
                ICSimulink(:,i) = icVal{i};
            end
        end
        icOut = dsp.internal.convertToChar(ICSimulink);
        return;
    end
    
else % sample based
    
    % Expected full IC format for sample-based processing
    ICSimulink = zeros([param.dims,delay]); %#ok
    
    if ICForEachChan && ~IcWithinChan
        % Scalar IC value per channel
        if isvector(icVal)
            % Use case: IC is a vector. The length of the vector is equal
            % to the number of channels. For example, if the input is 2x2,
            % IC is 1x4 or 4x1. Each value in IC is the initial condition
            % of the whole delay line for each channel, regadless of the
            % length of the delay line. 
            % In the Simulink Delay block, the corresponding IC should be
            % of size IC2 = [dim1, dim2, ... D] where dim# are the input
            % dimensions and D is the delay. IC(:,:...,:,1) values are
            % equal to IC(1), IC(:,:...,:,2) values are
            % equal to IC(2), etc 
            icValReshape = reshape(icVal , param.dims);
            ICSimulink   = repmat(icValReshape , [ones(1,numel(param.dims)) , delay]);
            icOut = dsp.internal.convertToChar(ICSimulink);
            return;
        else
            icOut = sprintf('repmat(%s,[%s %s])',ic,...
                    num2str(ones(1,numel(param.dims))) , num2str(delay) );
        end
        return;
    end
    
    if ~ICForEachChan && IcWithinChan
        % IC is a vector of length Delay
        if ~isrow(icVal)
            icVal = icVal.';
        end
        icTemp = repmat(icVal,prod(param.dims),1);
        ICSimulink = reshape(icTemp,[param.dims delay]);
        icOut = dsp.internal.convertToChar(ICSimulink);
        return;
    end
    
    % Simulink format is dim1-dim2-dim3-...Delay
    if ICForEachChan && IcWithinChan
        if ~iscell(icVal)
            % IC matrix of size delay x numchans.
            ICSimulink =  reshape(permute(icVal,[2 1]),[param.dims delay]);
            icOut = dsp.internal.convertToChar(ICSimulink);
        else
            ICTemp = zeros(prod(param.dims),delay);
            ICCellTemp = reshape(icVal,prod(param.dims),1);
            for i=1:prod(param.dims)
                ICTemp(i,:) = ICCellTemp{i};
            end
            icOut = sprintf('reshape(%s,[%s %s])',...
                             dsp.internal.convertToChar(ICTemp),...
                             num2str(param.dims),num2str(delay));
        end
    end
    
end
%--------------------------------------------------------------------------
function fixEdgeCases(param)

blk = [param.parent '/' param.name];

% For Each Iterator expects a sample-based input.
% Add frame conversion blk at input of For Each subsystem
if param.isFrame(1) && ...
   strcmp(param.InputProcessing,'Columns as channels (frame based)') && ...
   ~param.isScalarDelay
    blkLHandles = get_param(blk, 'LineHandles');
    blkLHInport = get(blkLHandles.Inport(1));
    InsertExtraBlock(blk, blkLHInport, 'Pre', 80, -10,...
        'dspobslib','dspobslib/Frame Conversion',...
        {'OutFrame'},{'Sample-based'});
    blkLHOutport = get(blkLHandles.Outport(1));
    % Restore frame-ness at the output of the subsystem
    InsertExtraBlock(blk, blkLHOutport, 'Post', 20, 5,...
        'dspobslib','dspobslib/Frame Conversion',...
        {'OutFrame'},{'Frame-based'});
end

% Add frame conversion blk if reset port is a frame
if length(param.isFrame)==2 && param.isFrame(2)
    blkLHandles = get_param(blk, 'LineHandles');
    if length(blkLHandles.Inport)==2
       resetPort = 2;  
    else % for Each subsystem
       resetPort = 3;   
    end
    blkLHInport = get(blkLHandles.Inport(resetPort));
    InsertExtraBlock(blk, blkLHInport, 'Pre', 80, -10,...
        'dspobslib','dspobslib/Frame Conversion',...
        {'OutFrame'},{'Sample-based'});
end

% Sample-based mode, scalar output, non-scalar ICs. Insert a reshape block
% to match the output dimensions of the DSP Delay block
if ~strcmp(param.InputProcessing,'Columns as channels (frame based)') && ...
        iscell(param.icVal) || ~isscalar(param.icVal) && ...
        all(param.OutportDims ==1)
    blkLHandles = get_param(blk, 'LineHandles');
    blkLHOutport = get(blkLHandles.Outport(1));
    InsertExtraBlock(blk, blkLHOutport, 'Post', 80, 10,...
        'simulink','built-in/Reshape',...
        {'OutputDimensionality','OutputDimensions'},...
        {'Customize',['[' num2str(param.OutportDims) ']']});
end

%--------------------------------------------------------------------------

function [isBus,isMixedFrameness,frameness] = detectBusSignal(tp)
% Detect bus signals
isMixedFrameness = false;
inport = tp.Inport;
isBus = inport(1)<0;
frameness = [];

if isBus
    numberOfsignals = inport(2);
    hasResetPort = length(inport(3:end))>numberOfsignals;
    
    signalFrameness = inport(3:3 + numberOfsignals-1);
    if length(find(signalFrameness==1))==numberOfsignals || ...
            length(find(signalFrameness==0))==numberOfsignals
        isMixedFrameness = false;
    else
        isMixedFrameness = true;
    end
    
    if hasResetPort
        frameness = [signalFrameness(1) inport(end)];
    else
        frameness = signalFrameness(1);
    end
end
