function SPCVariableIntegerDelayRegisterCompileCheck(block, muObj)
% SPCVARIABLEINTEGERDELAYREGISTERCOMPILECHECK  Forward DSP Variable Integer
% Delay block to Simulink Variable Integer Delay block

% Copyright 2014-2020 The MathWorks, Inc.

appendCompileCheck(muObj, block, @CollectData, @UpdateBlock);

%--------------------------------------------------------------------------
% Collect block information under compile stage
function param  = CollectData(block, ~)

% Get frame-ness, dimensions and datatype of input signal
tp = get_param(block, 'CompiledPortFrameData');
blkInfo.isFrame = (tp.Inport == 1);
blkInfo.SampleTime = get_param(block, 'CompiledSampleTime');
DT = get_param(block,'CompiledPortDataTypes');
blkInfo.DT = DT.Inport{2}; % delay datatype

dims = get_param(block, 'CompiledPortDimensions');
inport = dims.Inport;
InputProcessing = get_param(block,'InputProcessing');
if (strcmp(InputProcessing,'Columns as channels (frame based)')  ...
        && blkInfo.isFrame(1)) % frame-based processing
    if inport(1)==1 % 1D
        blkInfo.NumChans = 1;
        blkInfo.FrameSize = inport(2);
    else
        blkInfo.NumChans = prod(inport(3:2+inport(1)-1));
        blkInfo.FrameSize = inport(2);
    end
    blkInfo.DataSize = dims.Inport(2:2+dims.Inport(1)-1);
    blkInfo.DelaySize = dims.Inport(dims.Inport(1)+3:end);
    blkInfo.dims = [];
else % sample-based processing
    blkInfo.NumChans = prod(inport(2:2+inport(1)-1));
    blkInfo.FrameSize = [];
    blkInfo.dims = inport(2:2+inport(1)-1);
    blkInfo.DataSize = [];
    blkInfo.DelaySize = [];
end

% Get dimensions of delay input
blkInfo.numDelays = prod(inport(inport(1)+3:end));

% Transfer block parameters to param structure
blockParamStruct = get_param(block,'DialogParameters');
paramNames = fieldnames(blockParamStruct);
for i=1:length(paramNames)
    param.(paramNames{i}) =  get_param(block,paramNames{i});
end

param.isFrame =  blkInfo.isFrame;
param.SampleTime = blkInfo.SampleTime;
param.NumChans   = blkInfo.NumChans;
param.DT =  blkInfo.DT;
param.FrameSize =  blkInfo.FrameSize;
param.dims =  blkInfo.dims;
param.numDelays = blkInfo.numDelays;
param.DataSize = blkInfo.DataSize;
param.DelaySize = blkInfo.DelaySize;

%--------------------------------------------------------------------------
function UpdateBlock(block, h, param)

if askToReplace(h, block)

    funcSet =  {'muUpdateBlock',block, param};
    if (doUpdate(h))
        muUpdateBlock(block,param);
    end
    ReplaceBlockReasonStr = ...
        'Replace deprecated Variable Integer Delay block';
    
    if ~isempty(funcSet)
        appendTransaction(h, block, ReplaceBlockReasonStr, {funcSet});
    end
end
%--------------------------------------------------------------------------
function  muUpdateBlock(oldBlock, param)

% load Simulink
load_system('simulink');

% Map Input Processing
inputProcessing = param.InputProcessing;
if strcmp(inputProcessing,...
        'Inherited (this choice will be removed - see release notes)')
    if param.isFrame(1)
        param.InputProcessing = 'Columns as channels (frame based)';
    else
        param.InputProcessing = 'Elements as channels (sample based)';
    end
else
    param.InputProcessing = inputProcessing;
end

% The old block's name and parent are needed for the new block
name  = get_param(oldBlock,'Name');
parent = get_param(oldBlock,'Parent');
param.parent = parent;
param.name = strrep(name,'/','//');

% Create a temp block - if everything works fine replace the old block with
% this one.
tempBlock = [parent '/temp'];

%The decorations must be preserved
param.decorations = getDecorationParams(oldBlock);
param.mask={};

% Max delay of zero not supported in Simulink block. Change it to 1 in that
% case.
if (slResolve(param.dmax,oldBlock)==0)
    param.dmax = '1';
end

if param.numDelays == 1
    mapScalarDelay(oldBlock, param, tempBlock);
else
    % Different delay per channel - For Each Iterator
    mapNonScalarDelay(oldBlock, param, tempBlock);
end

% Fix edge cases
fixEdgeCases(param);

%--------------------------------------------------------------------------
function mapScalarDelay(oldBlock, param, tempBlock)
% Replace DSP Variable Integer Delay block by Simulink Variable Integer
% Delay block
newBlock = 'simulink/Discrete/Variable Integer Delay';
add_block(newBlock, tempBlock);

% Transform ic to Simulink format
ic = transformIC(oldBlock,param);

% Set the block parameters
set_param(tempBlock, ...
    'DelayLengthSource',        'Input port',...
    'DelayLengthUpperLimit',    param.dmax,...
    'InitialConditionSource',   'Dialog', ...
    'InputProcessing',          param.InputProcessing, ...
    'PreventDirectFeedthrough', param.noDFT,...
    'InitialCondition',         ic);

% Delete old block and replace with new block
delete_block(oldBlock);
add_block(tempBlock, [param.parent '/' param.name],...
    param.decorations{:},param.mask{:});
delete_block(tempBlock);

%--------------------------------------------------------------------------

function mapNonScalarDelay(oldBlock, param, tempBlock)

isFrame = strcmp(param.InputProcessing,'Columns as channels (frame based)');
if isFrame && param.DelaySize(1) > 1 % delay varies each sample in a frame
    if length(param.DelaySize)==2 && param.DelaySize(2) > 1
        % Delay varies for each channel and each sample in the frame
        mapDiffDelayPerChannel(oldBlock, param, tempBlock,true);
    else
        % Delay varies for each sample in the frame only
        mapDiffDelayPerSample(oldBlock, param, tempBlock);
    end
else
    % Different delay per channel - For Each Iterator
    mapDiffDelayPerChannel(oldBlock, param, tempBlock , false);
end
%--------------------------------------------------------------------------

function mapDiffDelayPerSample(oldBlock, param, tempBlock)
% Frame-based processing. the delay is a vector with same length as frame.
% The delay changes with each sample in the frame. A For Iterator subsystem
% is required.

newBlock =  'built-in/SubSystem';
add_block(newBlock, tempBlock);

parent = param.parent;
name = param.name;

icVal = slResolve(param.ic,oldBlock);
if ~isempty(icVal) && ~isscalar(icVal)
    % No-scalar IC. IC input port required
    InitialConditionSource = 'Input port';
    icInputPort = true;
    [~ , icSourceName] = fileparts(tempname);
    addBlock(oldBlock,'Pre',80,-10,...
        'dspsrcs4', 'dspsrcs4/Constant',[parent '/' icSourceName]);
else
    InitialConditionSource = 'Dialog';
    icInputPort = false;
end

% Insert For Iterator subsystem
setupForSubsystem(tempBlock,param,icInputPort);

targetblock = [tempBlock '/Delay'];
% Set the block parameters
set_param(targetblock, ...
    'DelayLengthSource',        'Input port',...
    'DelayLengthUpperLimit',    param.dmax,...
    'InitialConditionSource',   InitialConditionSource, ...
    'InputProcessing',          param.InputProcessing, ...
    'PreventDirectFeedthrough', param.noDFT);

ic = transformIC(oldBlock,param);
if icInputPort
    set_param([parent '/' icSourceName],'Value',ic);
end
if strcmp(InitialConditionSource,'Dialog')
    set_param(targetblock, 'InitialCondition',ic);
end

% Connect blocks
lineHandles = get_param(oldBlock,'LineHandles');
inPorts = lineHandles.Inport;
x = get(inPorts(1));
delete_line(x.Handle);
inputblockName = get_param(x.SrcBlockHandle,'Name');
inputblockPort = get_param(x.SrcPortHandle,'PortNumber');

x = get(inPorts(2));
delete_line(x.Handle);
delayblockName = get_param(x.SrcBlockHandle,'Name');
delayblockPort = get_param(x.SrcPortHandle,'PortNumber');

outPorts = lineHandles.Outport;
x = get(outPorts(1));
outblockName = get_param(x.DstBlockHandle,'Name');
outblockPort = get_param(x.DstPortHandle,'PortNumber');
delete_line(x.Handle);

% Delete old block and replace with new block
delete_block(oldBlock);
add_block(tempBlock, [parent '/' name],param.decorations{:},param.mask{:});
delete_block(tempBlock);

add_line(parent,[inputblockName '/' num2str(inputblockPort)],...
    [name '/1'],'autorouting','on')
add_line(parent,[delayblockName '/' num2str(delayblockPort)],...
    [name '/2'],'autorouting','on')
if strcmp(InitialConditionSource,'Input port')
    add_line(parent,[icSourceName '/1'],[name '/3'],'autorouting','on')
end
add_line(parent,[name '/1'],[outblockName '/' num2str(outblockPort)],...
    'autorouting','on')

%--------------------------------------------------------------------------
function mapDiffDelayPerChannel(oldBlock, param, tempBlock,ForIteratorFlag)
% Different delay for each channel. For Each Iterator required.
newBlock =  'built-in/SubSystem';

add_block(newBlock, tempBlock);

parent = param.parent;
name = param.name;

% Sample-based mode, scalar output, non-scalar ICs. Insert a reshape block
% to match the output dimensions of the DSP Delay block
if strcmp(param.InputProcessing,'Elements as channels (sample based)')
    [~ , reshapeblk1] = fileparts(tempname);
    addBlock(oldBlock,'Pre',80,10,...
        'simulink', 'built-in/Reshape',[parent '/' reshapeblk1]);
    set_param([parent '/' reshapeblk1],...
        'OutputDimensionality','Row vector (2-D)')
    [~ , reshapeblk2] = fileparts(tempname);
    addBlock(oldBlock,'Pre',80,-10,...
        'simulink',...
        'built-in/Reshape',[parent '/' reshapeblk2]);
    set_param([parent '/' reshapeblk2],...
        'OutputDimensionality','Row vector (2-D)')
    [~ , reshapeblk3] = fileparts(tempname);
    addBlock(oldBlock,'Pre',80,-30,...
        'simulink', 'built-in/Reshape',...
        [parent '/' reshapeblk3]);
    numDims = length(param.dims);
    dimsStr = '';
    for i=1:numDims
        dimsStr = sprintf('%s %s',dimsStr,num2str(param.dims(i)));
    end
    dimsStr = ['[' dimsStr ']'];
    set_param([parent '/' reshapeblk3],'OutputDimensionality','Customize')
    set_param([parent '/' reshapeblk3],'OutputDimensions',dimsStr)
end

icVal = slResolve(param.ic,oldBlock);
if ~isempty(icVal) && ~isscalar(icVal)
    InitialConditionSource = 'Input port';
    icInputPort = true;
    [~ , icSourceName] = fileparts(tempname);
    addBlock(oldBlock,'Pre',80,-10,...
        'dspsrcs4', 'dspsrcs4/Constant',[parent '/' icSourceName]);
else
    InitialConditionSource = 'Dialog';
    icInputPort = false;
end

setupForEachSubsystem(tempBlock,param,icInputPort,ForIteratorFlag);

ic = transformIC(oldBlock,param);
if icInputPort
    set_param([parent '/' icSourceName],'Value',ic);
end

if ~ForIteratorFlag
    targetblock = [tempBlock '/Delay'];
else
    % A For Iterator subsystem will be inserted inside the For Each
    % iterator - this is the case where the delay is varying for each
    % sample in a frame and for each input channel
    targetblock = [tempBlock '/Delay/Delay'];
end
% Set the block parameters
set_param(targetblock, ...
    'DelayLengthSource',        'Input port',...
    'DelayLengthUpperLimit',    param.dmax,...
    'InitialConditionSource',   InitialConditionSource, ...
    'InputProcessing',          param.InputProcessing, ...
    'PreventDirectFeedthrough', param.noDFT);

if strcmp(InitialConditionSource,'Dialog')
    set_param(targetblock, 'InitialCondition',ic);
end

lineHandles = get_param(oldBlock,'LineHandles');
inPorts = lineHandles.Inport;
x = get(inPorts(1));
delete_line(x.Handle);
inputblockName = get_param(x.SrcBlockHandle,'Name');
inputblockPort = get_param(x.SrcPortHandle,'PortNumber');

x = get(inPorts(2));
delete_line(x.Handle);
delayblockName = get_param(x.SrcBlockHandle,'Name');
delayblockPort = get_param(x.SrcPortHandle,'PortNumber');

outPorts = lineHandles.Outport;
x = get(outPorts(1));
outblockName = get_param(x.DstBlockHandle,'Name');
outblockPort = get_param(x.DstPortHandle,'PortNumber');
delete_line(x.Handle);

% Delete old block and replace with new block
delete_block(oldBlock);
add_block(tempBlock, [parent '/' name],param.decorations{:},param.mask{:});
delete_block(tempBlock);

if ~strcmp(param.InputProcessing,'Elements as channels (sample based)')
    add_line(parent,[inputblockName '/' num2str(inputblockPort)],...
        [name '/1'],'autorouting','on')
    add_line(parent,[delayblockName '/' num2str(delayblockPort)],...
        [name '/2'],'autorouting','on')
    if strcmp(InitialConditionSource,'Input port')
        add_line(parent,[icSourceName '/1'],[name '/3'],'autorouting','on')
    end
    add_line(parent,[name '/1'],[outblockName '/' num2str(outblockPort)],...
        'autorouting','on')
else
    add_line(parent,[inputblockName '/' num2str(inputblockPort)],...
        [reshapeblk1 '/1'],'autorouting','on')
    add_line(parent,[reshapeblk1 '/1'],[name '/1'],'autorouting','on')
    
    add_line(parent,[delayblockName '/' num2str(delayblockPort)],...
        [reshapeblk2 '/1'],'autorouting','on')
    add_line(parent,[reshapeblk2 '/1'],[name '/2'],'autorouting','on')
    
    if strcmp(InitialConditionSource,'Input port')
        add_line(parent,[icSourceName '/1'],[name '/3'],'autorouting','on')
    end
    
    add_line(parent,[name '/1'],[reshapeblk3 '/1'],'autorouting','on')
    add_line(parent,[reshapeblk3 '/1'],...
        [outblockName '/' num2str(outblockPort)],'autorouting','on')
end

%--------------------------------------------------------------------------
function decorations = getDecorationParams(block)
%getDecorationParams Return a cell array with block decorations
%
% Decorations include FontSize, FontWeight, Orientation, etc.
%
decorations = {
    'Position',        [];
    'Orientation',     [];
    'ForegroundColor', [];
    'BackgroundColor', [];
    'DropShadow',      [];
    'NamePlacement',   [];
    'FontName',        [];
    'FontSize',        [];
    'FontWeight',      [];
    'FontAngle',       [];
    'ShowName',        []
    };

for i=1:size(decorations,1)
    decorations{i,2}=get_param(block,decorations{i,1});
end

decorations=reshape(decorations',1,length(decorations(:)));

% -------------------------------------------------------------------------

function icOut = transformIC(oldBlock,param)
% Only scalar and "full" ic supported on Simulink Delay block
% Handle case where ic is neither
% On Simulink block, delay is of size numChans x Delay
ic = param.ic;

icVal = slResolve(ic,oldBlock);
if isscalar(icVal) && ~iscell(icVal)
    % Scalar IC - no transformation required
    icOut = ic;
    return;
end

dmax =slResolve(param.dmax,oldBlock);
if strcmp( param.InputProcessing , 'Columns as channels (frame based)')
    icSimulink = zeros(dmax,param.NumChans);
    if ismatrix(icVal)
        % icVal is a vector of length maxDelay (applies to all channels)
        for i=1: param.NumChans
            icSimulink(:,i) =  icVal;
        end
    else % 3D initial conditions 1-by-NUMCHAN-by-MAXDELAY 
         % (delay varies with each sample in the frame) 
        for i=1: param.NumChans
            icSimulink(:,i) =  icVal(:,i,:);
        end
    end
    icOut = dsp.internal.convertToChar(icSimulink);
else % sample-based processing
    if (size(icVal,1) == 1 && size(icVal,2) == dmax)  || ...
            (size(icVal,1) == dmax && size(icVal,2) == 1)
        % Ic is a vector of length DMAX (partial IC)
        ICTemp = zeros(prod(param.dims),dmax);
        for i=1:prod(param.dims)
            ICTemp(i,:) =  icVal;
        end
        if param.numDelays > 1
            % IC will be a input port for the non-scalar delay case
            % Reshape IC to a DMAX-by-NUMCHANS matrix. For Each Iterator
            % subsystem will select one column for each separate channel
            icOut =dsp.internal.convertToChar(ICTemp.');
        else
            % Convert to dim1-by-dim2-by-...-dimN-by-maxDelay
            icOut = sprintf('reshape(%s,[%s %s])',...
                dsp.internal.convertToChar(ICTemp),num2str(param.dims),num2str(dmax));
        end
    else
        % Size of IC is dim1-by-dim2-by-...-dimN-by-maxDelay
        if param.numDelays > 1
            % IC will be a input port for the non-scalar delay case
            % Reshape IC to a DMAX-by-NUMCHANS matrix. For Each Iterator
            % subsystem will select one column for each separate channel
            icOut = sprintf('reshape(permute(%s,[%s %s]),[%s %s])',...
                ic,num2str(length(param.dims)+1),...
                num2str(1:length(param.dims)),num2str(dmax),...
                num2str(prod(param.dims)));
        else
            % Scalar delay - Ic is already in Simulink format
            icOut = ic;
        end
    end
end
%--------------------------------------------------------------------------
function fixEdgeCases(param)

if param.numDelays > 1
    blk = [param.parent '/' param.name '/Delay'];
else
    blk = [param.parent '/' param.name];
end

if length(param.isFrame)==2 && param.isFrame(2)
    % Add frame conversion blk if delay port is a frame
    blkLHandles = get_param(blk, 'LineHandles');
    blkLHInport = get(blkLHandles.Inport(2));
    InsertExtraBlock(blk, blkLHInport, 'Pre', 80, -10,...
        'dspobslib','dspobslib/Frame Conversion',...
        {'OutFrame'},{'Sample-based'});
end

% Always add a rounding block - since behavior for non-integer
% delays is not the same
if strcmp(param.DT,'double') || strcmp(param.DT,'single')
    blkLHandles = get_param(blk, 'LineHandles');
    blkLHInport = get(blkLHandles.Inport(2));
    
    InsertExtraBlock(blk, blkLHInport, 'Pre', 80, -10,...
        'dspsigattribs',...
        ['dspsigattribs/Data Type ' char(10) 'Conversion'],...
        {'OutDataTypeStr'},{param.DT}); %#ok char(10)
    
    blkLHandles = get_param(blk, 'LineHandles');
    blkLHInport = get(blkLHandles.Inport(2));
    
    InsertExtraBlock(blk, blkLHInport, 'Pre', 80, -10,...
        'simulink',...
        ['simulink/Math Operations/Rounding' char(10) 'Function'],...
        {'Operator'},{'round'}); %#ok char(10)
end
 
if  param.isFrame(2) && param.numDelays > 1
    % Add frame conversion blk at delay input of For Each subsystem
    srcBlk = [param.parent '/' param.name];
    blkLHandles = get_param(srcBlk, 'LineHandles');
    blkLHInport = get(blkLHandles.Inport(2));
    InsertExtraBlock(srcBlk, blkLHInport, 'Pre', 80, -10,...
        'dspobslib','dspobslib/Frame Conversion',...
        {'OutFrame'},{'Sample-based'});
end

if param.numDelays > 1  && ...
        strcmp(param.InputProcessing,'Columns as channels (frame based)')
    if  param.isFrame(1)
        % Add frame conversion blk at input of For Each subsystem
        srcBlk = [param.parent '/' param.name];
        blkLHandles = get_param(srcBlk, 'LineHandles');
        blkLHInport = get(blkLHandles.Inport(1));
        InsertExtraBlock(srcBlk, blkLHInport, 'Pre', 80, -10,...
            'dspobslib','dspobslib/Frame Conversion',...
            {'OutFrame'},{'Sample-based'});
        blkLHOutport = get(blkLHandles.Outport(1));
        % Restore frameness
        InsertExtraBlock(srcBlk, blkLHOutport, 'Post', 20, 5,...
            'dspobslib','dspobslib/Frame Conversion',...
            {'OutFrame'},{'Frame-based'});
    end
end

%--------------------------------------------------------------------------

function setupForSubsystem(subsys,param,icInputPort)

DataInput = [subsys '/In'];
add_block('built-in/Inport',DataInput,'Position',[55 90 75 110]);
DelayInput = [subsys '/In2'];
add_block('built-in/Inport',DelayInput,'Position',[55 205 75   225]);
if icInputPort
    ICInput = [subsys '/In3'];
    add_block('built-in/Inport',ICInput,'Position',[55 305 75 325]);
end

out = [subsys '/out'];
add_block('built-in/Outport',out,'Position',[ 555 145 575 165]);

delay = [subsys '/Delay'];
add_block('simulink/Discrete/Variable Integer Delay', delay);
set_param(delay,'DelayLengthUpperLimit',param.dmax);
foreach = [subsys '/ForIterator'];
add_block('built-in/ForIterator', foreach,'Position',[ 55 165 75 185]);

set_param(foreach,'IterationLimit',num2str(param.FrameSize))

if icInputPort
    set_param(delay,'InitialConditionSource','Input port')
end

% Selector and transpose blocks to extract one set of coefficients/inputs:
selector1 = [subsys '/Selector Data'];
add_block('dspindex/Variable Selector', selector1,...
    'Position',[110 118 205 162]);

selector2 = [subsys '/Selector Delay'];
add_block('dspindex/Variable Selector', selector2,...
    'Position',[110 203 205 247]);

add_line(subsys,'In/1','Selector Data/1','autorouting','on');
add_line(subsys,'ForIterator/1','Selector Data/2','autorouting','on');
add_line(subsys,'Selector Data/1','Delay/1','autorouting','on');

add_line(subsys,'In2/1','Selector Delay/1','autorouting','on');
add_line(subsys,'ForIterator/1','Selector Delay/2','autorouting','on');
add_line(subsys,'Selector Delay/1','Delay/2','autorouting','on');

if icInputPort
    add_line(subsys,'In3/1','Delay/3','autorouting','on');
end

% The assignment filter concatenates outputs from different samples
assignBlk = [subsys '/Assign'];
add_block('built-in/Assignment', assignBlk,'Position',[455 145 515 175]);
set_param(assignBlk,'NumberOfDimensions','2',...
    'IndexOptions','Index vector (port), Assign all')

add_line(subsys,'Delay/1','Assign/2','autorouting','on');
add_line(subsys,'In/1','Assign/1','autorouting','on');
add_line(subsys,'ForIterator/1','Assign/3','autorouting','on');

add_line(subsys,'Assign/1','out/1','autorouting','on');

%--------------------------------------------------------------------------

function setupForEachSubsystem(subsys,param,icInputPort,ForIteratorFlag)

DataInput = [subsys '/In'];
add_block('built-in/Inport',DataInput,'Position',[15 15 35 35]);
DelayInput = [subsys '/In2'];
add_block('built-in/Inport',DelayInput,'Position',[15 95 35 115]);
if icInputPort
    ICInput = [subsys '/In3'];
    add_block('built-in/Inport',ICInput,'Position',[15 165 35 185]);
end

out = [subsys '/out'];
add_block('built-in/Outport',out,'Position',[450 45 470 65]);

delay = [subsys '/Delay'];
if ~ForIteratorFlag
    add_block('simulink/Discrete/Variable Integer Delay', delay);
    set_param(delay,'DelayLengthUpperLimit',param.dmax);
    delayTarget = delay;
else
    % A For Iterator subsystem will be inserted inside the For Each
    % iterator - this is the case where the delay is varying for each
    % sample in a frame and for each input channel
    newBlock =  'built-in/SubSystem';
    add_block(newBlock, delay);
    setupForSubsystem(delay,param,icInputPort);
    delayTarget =  [subsys '/Delay/Delay'];
end

foreach = [subsys '/For Each Iterator'];
add_block('built-in/ForEach', foreach,'Position',[50 250 70 270]);
if icInputPort
    set_param(delayTarget,'InitialConditionSource','Input port')
    set_param(foreach,'InputPartition',{'on','on','on'})
    set_param(foreach,'InputPartitionDimension',{'2','2','2'})
else
    set_param(foreach,'InputPartition',{'on','on'})
    set_param(foreach,'InputPartitionDimension',{'2','2'})
end
set_param(foreach,'OutputConcatenationDimension',{'2'})

add_line(subsys,'In/1','Delay/1','autorouting','on');
add_line(subsys,'In2/1','Delay/2','autorouting','on');
if icInputPort
    add_line(subsys,'In3/1','Delay/3','autorouting','on');
end
add_line(subsys,'Delay/1','out/1','autorouting','on');
