function digitalFilterRegisterCompileCheck(block, muObj)
% DIGITALFILTERREGISTERCOMPILECHECK - Forward Digital Filter block.
%
% The deprecated Digital Filter block is forwarded to one of
% the following blocks, depending on its parameter settings:
% - dsparch4/Discrete FIR Filter
% - dsparch4/Discrete Filter
% - dsparch4/Biquad Filter
% - dsparch4/All pole filter

% Copyright 2014-2019 The MathWorks, Inc.

appendCompileCheck(muObj, block, @CollectData, @UpdateBlock);

%--------------------------------------------------------------------------
% Collect block information under compile stage
function param = CollectData(block, ~)

% Get frame-ness, dimensions and datatype of input signal
% This information is required to perform mapping

tp = get_param(block, 'CompiledPortFrameData');
blkInfo.isFrame = (tp.Inport(1) == 1);
blkInfo.numInputs        = length(tp.Inport);
blkInfo.isCoefPortFrame = zeros(blkInfo.numInputs-1,1);
blkInfo.isCoefColVector = zeros(blkInfo.numInputs-1,1);
blkInfo.CoeffPortSize = zeros(blkInfo.numInputs-1,1);
tpDT  = get_param(block,'CompiledPortDataTypes');
blkInfo.OutputDT = tpDT.Outport{:};
tpCmplx = get_param(block,'CompiledPortComplexSignals');
blkInfo.OutputComplexity = tpCmplx.Inport;

% Determine if coefficient input ports are frame-based
tpDims  = get_param(block,'CompiledPortDimensions');
for i=1:blkInfo.numInputs - 1
    blkInfo.isCoefPortFrame(i) =  (tp.Inport(i+1) == 1);
end

inport = tpDims.Inport;
blkInfo.FrameLength = inport(2);
if inport(1) == 1 % unoriented input
    blkInfo.NumChans = 1;
    inportCoeff = inport(3:end);
else % 2D input
    blkInfo.NumChans = inport(3);
    inportCoeff = inport(4:end);
end

% Coefficients may be 2D or unoriented
if blkInfo.numInputs >= 2
    if inportCoeff(1) == 1 % coef port is unoriented
        blkInfo.isCoefColVector(1) = 0;
        blkInfo.CoeffPortSize(1) = inportCoeff(2);
        inportCoeff = inportCoeff(3:end);
    else
        blkInfo.isCoefColVector(1) = inportCoeff(2) > 1;
        blkInfo.CoeffPortSize(1) = inportCoeff(2)*inportCoeff(3);
        inportCoeff = inportCoeff(4:end);
    end
    if blkInfo.numInputs == 3
        if inportCoeff(1) == 1 % 2nd coef port is unoriented
            blkInfo.isCoefColVector(2) = 0;
            blkInfo.CoeffPortSize(2) = inportCoeff(2);
        else
            blkInfo.isCoefColVector(2) = inportCoeff(2) > 1;
            blkInfo.CoeffPortSize(2) = inportCoeff(2)*inportCoeff(3);
        end
    end
end

% Transfer block parameters to param structure
blockParamStruct = get_param(block,'DialogParameters');
paramNames = fieldnames(blockParamStruct);
for i=1:length(paramNames)
    param.(paramNames{i}) =  get_param(block,paramNames{i});
end
% Move frame-ness, dimensions and datatype info to param structure
param.isCoefPortFrame  = blkInfo.isCoefPortFrame;
param.isCoefColVector  = blkInfo.isCoefColVector;
param.CoeffPortSize    = blkInfo.CoeffPortSize;
param.OutputDT         = blkInfo.OutputDT;
param.OutputComplexity = blkInfo.OutputComplexity;
param.isFrame          = blkInfo.isFrame;
param.FrameLength      = blkInfo.FrameLength ;
param.NumChans         = blkInfo.NumChans ;
param.fixColumnCoeffs  = false;

% Map Input Processing (this applies to all filters/structures)
InputProcessing = param.InputProcessing;
if strcmp(InputProcessing,...
        'Inherited (this choice will be removed - see release notes)')
    if param.isFrame
        param.InputProcessing = 'Columns as channels (frame based)';
    else
        param.InputProcessing = 'Elements as channels (sample based)';
    end
else
    param.InputProcessing = InputProcessing;
end

%--------------------------------------------------------------------------
function UpdateBlock(block, h, param)


if askToReplace(h, block)
    if noTransformPossible(param,block)
        funcSet = '';
        ReplaceBlockReasonStr = '';
    else
        funcSet =  {'muUpdateBlock',block, param};
        if (doUpdate(h))
            muUpdateBlock(block,param);
        end
        ReplaceBlockReasonStr = 'Replace deprecated Digital Filter block';
    end
    
    if ~isempty(funcSet)
        appendTransaction(h, block, ReplaceBlockReasonStr, {funcSet});
    end
end

%--------------------------------------------------------------------------
function flag = noTransformPossible(param,oldBlock)
% Return true if no transformation is possible. This is the case where
% Digital Filter has functionality that is not available in the target
% blocks
flag = false;

% Do not touch blocks that specify the filter as a DFILT.
% Revisit when target blocks support this workflow. 
if isdfilt(param)
    flag = true;
    return;
end

% DFT2 structure + non-unity a0 coefficient from input port
if isDF2TA0NonUnityFromPort(param)
    flag = true;
    return;
end

% FIR DFT state datatype can be set to 'Same as Input' or Binary point
% scaling in DigitalFilter. It is hard coded to Same as Accumulator in FIR
% Filter block
if isFIR(param) && ~isInputFloatingPoint(param) && ...
        strcmp(param.FIRFiltStruct,'Direct form transposed') && ...
        ~strcmp(param.memoryMode,'Same as accumulator')
    flag = true;
    return;
end

% Complex scale values are not allowed on biquad block. There is no way to
% achieve functionality match for fixed-point inputs or for non-zero IC 
% (we'll insert a gain block incorporating scale values for floating point 
% inputs)
if isBiquad(param) && ~isInputFloatingPoint(param) ...
        && ~isreal(slResolve(param.ScaleValues,oldBlock))
    flag = true;
    return;
end
if isBiquad(param) && ~isreal(slResolve(param.ScaleValues,oldBlock)) && ...
   isBiquadNonZeroIC(param,oldBlock)
    flag = true;
    return;  
end

%--------------------------------------------------------------------------
function  muUpdateBlock(oldBlock, param) 

% load libraries
load_system('simulink');
load_system('dsparch4');

% The old block's name and parent are needed for the new block
name  = get_param(oldBlock,'Name');
name = strrep(name,'/','//');
parent = get_param(oldBlock,'Parent');
param.parent = parent;
param.name = name;

% create a temp block - if everything works fine,
% replace the old block with this one.
tempBlock = [parent '/temp'];

%The decorations must be preserved
decorations = getDecorationParams(oldBlock);
mask={};

TypePopup = param.TypePopup;
switch TypePopup
    case 'FIR (all zeros)'
        [tempBlock,param] = upgradeFIR(param, oldBlock, tempBlock);
    case 'IIR (all poles)'
        [tempBlock,param] = upgradeAllPole(param, oldBlock, tempBlock);
    case 'IIR (poles & zeros)'
        % Digital Filter block has IIR and Biquad structures under 'IIR
        % (poles & zeros)'
        if isBiquad(param)
            [tempBlock,param] = upgradeBiquad(param, oldBlock, tempBlock);
        else
            [tempBlock, param] = upgradeIIR(param, oldBlock, tempBlock);
        end
end

% Delete old block and replace with new block
delete_block(oldBlock);
add_block(tempBlock, [parent '/' name],decorations{:},mask{:});
delete_block(tempBlock);

% Fix edge cases by inserting extra blocks
fixEdgeCases(param);

%--------------------------------------------------------------------------
function [tempBlock, param] = upgradeFIR(param, oldBlock, tempBlock)
% Forward Digital Filter to Discrete FIR Filter

% Lattice and DF structures use different parameter name for coefficients
FilterStructure = param.FIRFiltStruct;
if ~strcmp(FilterStructure,'Lattice MA')
    Coefficients = param.NumCoeffs;
else
    Coefficients = param.LatticeCoeffs;
end

% Fix datatype and orientation of coefficients and initial conditions
Coefficients = fixCoefficients(Coefficients,oldBlock);
InitialStates = fixInitialStates(param.IC,oldBlock);

% Map coefficient source
FilterSource = param.FilterSource;
if strcmp(FilterSource,'Input port(s)')
    CoefSource = 'Input port';
else
    CoefSource = 'Dialog parameters';
end

% Set new Block parameters
if ~updateCoefficientsEachSample(param)
    newBlock = 'dsparch4/Discrete FIR Filter';
    add_block(newBlock, tempBlock);
    targetBlock = tempBlock;
    param.fixColumnCoeffs = true;
else
    % When coefficients are updated once per sample in frame-based mode,
    % insert For Iterator subsystem
    newBlock = 'built-in/SubSystem';
    targetBlock = [tempBlock '/Filter'];
    add_block(newBlock, tempBlock);
    paramStruct.BlockType = 'Discrete FIR Filter';
    param = setupOncePerSampleSubsystem(tempBlock,param,paramStruct);
    % The coefficients are a matrix in this case, so there is no need to
    % transpose column-based coefficients into row coefficients. turn off
    % this check.
    param.fixColumnCoeffs = false;
end

firstCoeffMode = get_param(oldBlock,'firstCoeffMode');
% Work around issue in digital Filter. firstCoeffMode sometimes returns 
% 'Same as input' when set to 'Same word length as input' before dialog is
% refreshed
if strcmp(firstCoeffMode,'Same as input')
    param.firstCoeffMode = 'Same word length as input';
end

% Set the block parameters
set_param(targetBlock, ...
    'FilterStructure',FilterStructure,...
    'Coefficients',Coefficients,...
    'InitialStates', InitialStates, ...
    'CoefSource',       CoefSource, ...
    'SaturateOnIntegerOverflow',param.overflowMode,...
    'RndMeth',param.roundingMode,...
    'AccumDataTypeStr',transformFixedPointParameter('accumMode',param),...
    'ProductDataTypeStr',transformFixedPointParameter('prodOutputMode',param),...
    'OutDataTypeStr',transformFixedPointParameter('outputMode',param),...
    'CoefDataTypeStr',transformFixedPointParameter('firstCoeffMode',param),...
    'TapSumDataTypeStr',transformFixedPointParameter('tapSumMode',param),...
    'StateDataTypeStr',transformFixedPointParameter('memoryMode',param),...
    'InputProcessing', param.InputProcessing);

%--------------------------------------------------------------------------
function [tempBlock,param] = upgradeBiquad(param, oldBlock, tempBlock)
% Forward Digital Filter to Biquad Filter

% Biquad coefficients cannot be specified from the input port,
% so turn off the check
param.fixColumnCoeffs = false;

% Map the filter structure
switch param.IIRFiltStruct
    case 'Biquad direct form II transposed (SOS)'
        structure = 'Direct form II transposed';
    case 'Biquad direct form I transposed (SOS)'
        structure = 'Direct form I transposed';
    case 'Biquad direct form II (SOS)'
        structure = 'Direct form II';
    case 'Biquad direct form I (SOS)'
        structure = 'Direct form I';
end

% Fix initial conditions
IC = fixInitialStates(param.IC,oldBlock);
ICnum = fixInitialStates(param.ICnum,oldBlock);
ICden = fixInitialStates(param.ICden,oldBlock);

% stageIO on Digital Filter maps to stageInput and StageOutput on Biquad
% Filter
stageIOMode = param.stageIOMode;
if strcmp(stageIOMode,'Same as input')
    StageInputMode = 'Same as input';
    StageOutputMode = 'Same as section input';
else
    StageInputMode = stageIOMode;
    StageOutputMode = stageIOMode;
end

% Biquad scale values can be complex on Digital Filter but not on Biquad
% Filter. For complex scale values, the scale will be one on the dialog,
% and a gain block incorporating the scale values will be added in
% fixEdgeCases
ScaleValues = param.ScaleValues;
if ~isreal(slResolve(ScaleValues,oldBlock))
    ScaleValuesVal = '1';
else
    ScaleValuesVal = ScaleValues;
end

newBlock = 'dsparch4/Biquad Filter';
add_block(newBlock, tempBlock);
% set new Block param
set_param(tempBlock, ...
    'IIRFiltStruct',structure,...
    'BiQuadCoeffs',param.BiQuadCoeffs,...
    'ScaleValues', ScaleValuesVal, ...
    'IC',   IC, ...
    'ICnum',ICnum,...
    'ICden',ICden,...
    'a0Flag','error',...
    'overflowMode',param.overflowMode,...
    'roundingMode',param.roundingMode,...
    'OutputMode',param.outputMode,...
    'OutputWordLength',param.outputWordLength,...
    'OutputFracLength',param.outputFracLength,...
    'AccumMode',param.accumMode,...
    'AccumWordLength',param.accumWordLength,...
    'AccumFracLength',param.accumFracLength,...
    'denAccumFracLength',param.accumFracLength,...
    'ProdOutputMode',param.prodOutputMode,...
    'ProdOutputWordLength',param.prodOutputWordLength,...
    'ProdOutputFracLength',param.prodOutputFracLength,...
    'denProdOutputFracLength',param.prodOutputFracLength,...
    'firstCoeffMode',param.firstCoeffMode,...
    'firstCoeffWordLength',param.firstCoeffWordLength,...
    'firstCoeffFracLength',param.firstCoeffFracLength,...
    'secondCoeffFracLength',param.secondCoeffFracLength,...
    'StageInputMode',StageInputMode ,...
    'StageOutputMode',StageOutputMode ,...
    'StageInputWordLength', param.stageIOWordLength,...
    'StageInputFracLength', param.stageInFracLength,...
    'StageOutputWordLength',param.stageIOWordLength,...
    'StageOutputFracLength',param.stageOutFracLength,...
    'memoryMode',param.memoryMode,...
    'memoryWordLength',param.memoryWordLength,...
    'memoryFracLength',param.memoryFracLength,...
    'scaleValueFracLength', param.scaleValueFracLength,...
    'secondMemoryFracLength', param.memoryFracLength,...
    'InputProcessing', param.InputProcessing);

%--------------------------------------------------------------------------
function [tempBlock,param] = upgradeDiscreteFilter_a0Dialog(param, oldBlock, tempBlock)
% Forward Digital Filter to Discrete Filter. This function covers the case
% where the leading denominator coefficient on DigitalFilter is non-unity
% on the dialog. Since the All Pole block always treats a0 as unity, we
% forward to a Discrete Filter instead. This only applies to floating-point
% inputs.

% Map structure
structure =  param.AllPoleFiltStruct;
if strcmp(structure,'Direct form')
    structure = 'Direct form II';
else
    structure = 'Direct form II transposed';
end

% Numerator is one
num = '1';
den = param.DenCoeffs;
% Fix coefficient orientation and datatype
den = fixCoefficients(den,oldBlock);

InitialStates = fixInitialStates(param.IC,oldBlock);

% The case covered here is only for coefficients from the dialog (no
% mapping occurs if a0 is not ignored and coefficients come from input
% port)
NumeratorSource = 'Dialog';
DenominatorSource = 'Dialog';
% a0 is non-unity:
a0EqualsOne =  'off';

% Map to new block
newBlock = 'dsparch4/Discrete Filter';
add_block(newBlock, tempBlock);
set_param(tempBlock, ...
    'a0EqualsOne',a0EqualsOne,...
    'FilterStructure',structure,...
    'InputProcessing', param.InputProcessing, ...
    'Numerator',       num, ...
    'Denominator',     den,...
    'InitialStates',InitialStates,...
    'NumeratorSource',NumeratorSource,...
    'DenominatorSource',DenominatorSource);

%--------------------------------------------------------------------------
function [tempBlock, param] = upgradeDiscreteFilter_a0InputPort(param, oldBlock, tempBlock)
% Forward Digital Filter to Discrete Filter. This function covers the case
% where the leading denominator coefficient on DigitalFilter is non-unity
% from an input port. Since the All Pole block always treats a0 as unity,
% we forward to a Discrete Filter instead. This only applies to
% floating-point inputs.

% Map structure, additional blocks will be added in fixEdgeCases

% DFT is not covered by slupdate
structure = 'Direct form II';
% For DF, the input will be scaled by 1/a0
% The numerator is '1' on the dialog (no scaling necessary)
% No scaling necessary for initial conditions
NumeratorSource = 'Dialog';
DenominatorSource = 'Input port';
InitialStatesSource = 'Dialog';

num = '1';
den = param.DenCoeffs;
% Fix coefficient orientation and datatype
den = fixCoefficients(den,oldBlock);

InitialStates = fixInitialStates(param.IC,oldBlock);

if ~updateCoefficientsEachSample(param)
    % Replace by All Pole Filter
    newBlock = 'dsparch4/Discrete Filter';
    add_block(newBlock, tempBlock);
    targetBlock = tempBlock;
    param.fixColumnCoeffs = true;
else
    % Replace by For Iterator Subsystem containing All Pole Filter
    newBlock = 'built-in/SubSystem';
    targetBlock = [tempBlock '/Filter'];
    add_block(newBlock, tempBlock);
    paramStruct.BlockType = 'Discrete Filter';
    paramStruct.wasAllPole = true;
    paramStruct.structure = structure;
    param = setupOncePerSampleSubsystem(tempBlock,param,paramStruct);
    % The coefficients on the input port are a matrix, so turn off the
    % column vector check
    param.fixColumnCoeffs = false;
    param.CoeffPortSize = param.CoeffPortSize /  param.FrameLength;
end
% Map to new block
set_param(targetBlock, ...
    'a0EqualsOne','on',...
    'FilterStructure',structure,...
    'InputProcessing', param.InputProcessing, ...
    'Numerator',       num, ...
    'Denominator',     den,...
    'InitialStates',InitialStates,...
    'Numerator',num,...
    'NumeratorSource',NumeratorSource,...
    'DenominatorSource',DenominatorSource,...
    'InitialStatesSource',InitialStatesSource);

param.fixAllPole = true;

%--------------------------------------------------------------------------
function [tempBlock, param] = upgradeAllPole(param, oldBlock, tempBlock)
% Forward Digital Filter to All Pole Filter

% Map structure
structure = param.AllPoleFiltStruct;
FilterSource = param.FilterSource;
% Coefficient correspond to different parameters on Digital Filter
% depending on structure
if ~strcmp(structure,'Lattice AR')
    den = param.DenCoeffs;
else
    den  = param.LatticeCoeffs;
end
denVal = slResolve(den,oldBlock);

% For Allpole filter (DF and DFT), the leading denominator is treated as 1
% regardless of its value. Digital Filter does not treat it as one.
% Therefore, in this case, we replace by an IIR filter
if ~strcmp(structure,'Lattice AR') && ...
        ~CoefficientsFromInputPort(param) &&...
        denVal(1)~=1 && ...
        isInputFloatingPoint(param)
    [tempBlock, param] = ...
        upgradeDiscreteFilter_a0Dialog(param, oldBlock, tempBlock);
    return;
end

% For all pole structures, leading denominator coefficient may be non-unity
% if "remove a0 term" is unchecked. This is not allowed on newer blocks.
% This only applies for floating-point inputs.
if ~strcmp(structure,'Lattice AR')  && ...
        CoefficientsFromInputPort(param) && ...
        ~ignoreA0(param) && ...
        isInputFloatingPoint(param)
    [tempBlock, param] = ...
        upgradeDiscreteFilter_a0InputPort(param, oldBlock, tempBlock);
    return;
end

% Fix coefficient orientation and datatype
den = fixCoefficients(den,oldBlock);

InitialStates = fixInitialStates(param.IC,oldBlock);

% Map coefficient source
if strcmp(FilterSource,'Input port(s)')
    CoefSource = 'Input port';
else
    CoefSource = 'Dialog parameters';
end

if ~updateCoefficientsEachSample(param)
    % Replace by All Pole Filter
    newBlock = 'dsparch4/Allpole Filter';
    add_block(newBlock, tempBlock);
    targetBlock = tempBlock;
    param.fixColumnCoeffs = true;
else
    % Replace by For Iterator Subsystem containing All Pole Filter
    newBlock = 'built-in/SubSystem';
    targetBlock = [tempBlock '/Filter'];
    add_block(newBlock, tempBlock);
    s.BlockType = 'Allpole Filter';
    param = setupOncePerSampleSubsystem(tempBlock,param,s);
    % The coefficients on the input port are a matrix, so turn off the
    % column vector check
    param.fixColumnCoeffs = false;
end

firstCoeffMode = get_param(oldBlock,'firstCoeffMode');
% Work around issue in digital Filter. firstCoeffMode sometimes returns 
% 'Same as input' when set to 'Same word length as input' before dialog is
% refreshed
if strcmp(firstCoeffMode,'Same as input')
    param.firstCoeffMode = 'Same word length as input';
end

set_param(targetBlock, ...
    'FilterStructure', structure,...
    'InputProcessing', param.InputProcessing, ...
    'Coefficients',    den,...
    'InitialStates',   InitialStates,...
    'CoefSource',      CoefSource,...
    'SaturateOnIntegerOverflow', param.overflowMode,...
    'RndMeth',            param.roundingMode,...
    'AccumDataTypeStr',   transformFixedPointParameter('accumMode',param),...
    'OutDataTypeStr',     transformFixedPointParameter('outputMode',param),...
    'CoefDataTypeStr',    transformFixedPointParameter('firstCoeffMode',param),...
    'StateDataTypeStr',   transformFixedPointParameter('memoryMode',param),...
    'ProductDataTypeStr', transformFixedPointParameter('prodOutputMode',param));

%--------------------------------------------------------------------------
function [tempBlock, param] = upgradeIIR(param, oldBlock, tempBlock)
% Forward Digital Filter to Discrete (IIR) filter

% Map structure
structure =  param.IIRFiltStruct;
% Map and fix coefficient orientation and datatype
num = param.NumCoeffs;
num = fixCoefficients(num,oldBlock);
den = param.DenCoeffs;
den = fixCoefficients(den,oldBlock);
% Map IC
if ~contains(structure,'Direct form II')
    InitialStates = fixInitialStates(param.ICnum,oldBlock);
    InitialDenominatorStates =  fixInitialStates(param.ICden,oldBlock);
else
    InitialStates =  fixInitialStates(param.IC,oldBlock);
end

% Map filter source
FilterSource = param.FilterSource;
if strcmp(FilterSource,'Input port(s)')
    NumeratorSource = 'Input port';
    DenominatorSource = 'Input port';
    a0EqualsOne = 'on';
else
    NumeratorSource = 'Dialog';
    DenominatorSource = 'Dialog';
    denVal = slResolve(den,oldBlock);
    % If possible, turn on "ignore a0" for performance
    % a0 is assumed to be one for fixed-point inputs
    if denVal(1)==1 || ~isInputFloatingPoint(param)
        a0EqualsOne =  'on';
    else
        a0EqualsOne =  'off';
    end
end

% The fixed-point Coefficients parameter on digital Filter maps to the
% Numerator Coefficients and Denominator Coefficients parameters on
% Discrete Filter
coeffMode = param.firstCoeffMode;
switch coeffMode
    case {'Same word length as input','Same as input'}
        NumCoefDataTypeStr = 'Inherit: Inherit via internal rule';
        DenCoefDataTypeStr = 'Inherit: Inherit via internal rule';
    case 'Specify word length'
        wordLength = param.firstCoeffWordLength;
        NumCoefDataTypeStr = sprintf('fixdt(1,%s)',wordLength);
        DenCoefDataTypeStr = sprintf('fixdt(1,%s)',wordLength);
    case {'Binary point scaling','Slope and bias scaling','User-defined'}
        wordLength = param.firstCoeffWordLength;
        fracLength1 = param.firstCoeffFracLength;
        fracLength2 = param.secondCoeffFracLength;
        NumCoefDataTypeStr = ...
            sprintf('fixdt(1,%s,%s)',wordLength,fracLength1);
        DenCoefDataTypeStr = ...
            sprintf('fixdt(1,%s,%s)',wordLength,fracLength2);
end

% Map output mode
outputMode = param.outputMode;
if strcmp(outputMode,'Same as accumulator')
    OutDataTypeStr = 'Inherit: Inherit via internal rule';
else
    OutDataTypeStr = transformFixedPointParameter('outputMode',param);
end


% Digital Filter has a 'Same as Output' option for Multiplicand. Discrete
% Filter does not have that option. If 'Same as Output' was selected, we
% have to extract the output datatype and use it to set the Multiplicand
% datatype on Discrete Filter.
multiplicandMode = param.multiplicandMode;
if strcmp(multiplicandMode,'Same as output')
    % Get the output mode
    outputMode = param.outputMode;
    switch outputMode
        case 'Same as input'
            multiplicandDataTypeStr = 'Inherit: Same as input';
        case 'Same as accumulator'
            accumMode = param.accumMode;
            switch accumMode
                case 'Same as input'
                    multiplicandDataTypeStr = 'Inherit: Same as input';
                case 'Same as product output'
                    prodOutputMode = param.prodOutputMode;
                    switch prodOutputMode
                        case 'Same as input'
                            multiplicandDataTypeStr = 'Inherit: Same as input';
                        otherwise
                            % Custom
                            wordLengthVal = param.prodOutputWordLength;
                            FractionLengthVal = param.prodOutputFracLength;
                            multiplicandDataTypeStr = ...
                                sprintf('fixdt(1,%s,%s)', ...
                                wordLengthVal,FractionLengthVal);
                    end
                otherwise
                    % Custom
                    wordLengthVal = param.accumWordLength;
                    FractionLengthVal = param.accumFracLength;
                    multiplicandDataTypeStr = ...
                        sprintf('fixdt(1,%s,%s)', ...
                        wordLengthVal,FractionLengthVal);
            end
        otherwise
            % Custom
            wordLengthVal = param.outputWordLength;
            FractionLengthVal = param.outputFracLength;
            multiplicandDataTypeStr = ...
                sprintf('fixdt(1,%s,%s)',wordLengthVal,FractionLengthVal);
    end
else
    multiplicandDataTypeStr = ...
        transformFixedPointParameter('multiplicandMode',param);
end

% State on Digital filter can be set to Same as Accumulator. there is no
% such option on discrete Filter. In that case, the accumulator datatype is
% extracted and used to set the state datatype.
memoryMode = param.memoryMode;
if ~strcmp(memoryMode,'Same as accumulator')
    StateDataTypeStr = transformFixedPointParameter('memoryMode',param);
else
    accumMode = param.accumMode;
    switch accumMode
        case 'Same as product output'
            % Need to get product output DT
            prodOutputMode = param.prodOutputMode;
            if strcmp(prodOutputMode,'Same as input')
                StateDataTypeStr =  'Inherit: Same as input';
            else
                % Custom
                wordLengthVal = param.prodOutputWordLength;
                FractionLengthVal = param.prodOutputFracLength;
                StateDataTypeStr =  sprintf('fixdt(1,%s,%s)', ...
                    wordLengthVal,FractionLengthVal);
            end
        case 'Same as input'
            StateDataTypeStr =  'Inherit: Same as input';
        otherwise
            % Custom
            wordLengthVal = param.accumWordLength;
            FractionLengthVal = param.accumFracLength;
            StateDataTypeStr =  sprintf('fixdt(1,%s,%s)', ...
                wordLengthVal,FractionLengthVal);
    end
end

NumAccumDataTypeStr = transformFixedPointParameter('accumMode',param);
DenAccumDataTypeStr = transformFixedPointParameter('accumMode',param);
NumProductDataTypeStr = transformFixedPointParameter('prodOutputMode',param);
DenProductDataTypeStr = transformFixedPointParameter('prodOutputMode',param);

if ~updateCoefficientsEachSample(param)
    % Replace by Discrete Filter
    newBlock = 'dsparch4/Discrete Filter';
    add_block(newBlock, tempBlock);
    targetBlock = tempBlock;
    param.fixColumnCoeffs = true;
else
    % Replace by For Iterator Subsystem containing Discrete Filter
    newBlock = 'built-in/SubSystem';
    targetBlock = [tempBlock '/Filter'];
    add_block(newBlock, tempBlock);
    s.BlockType = 'Discrete Filter';
    s.wasAllPole = false;
    param = setupOncePerSampleSubsystem(tempBlock,param,s);
    % coefficients are a matrix in this case. Shut off row/column check
    param.fixColumnCoeffs = false;
end

set_param(targetBlock, ...
    'a0EqualsOne',a0EqualsOne,...
    'FilterStructure',structure,...
    'InputProcessing', param.InputProcessing, ...
    'Numerator',       num, ...
    'Denominator',     den,...
    'InitialStates',InitialStates,...
    'NumeratorSource',NumeratorSource,...
    'DenominatorSource',DenominatorSource,...
    'SaturateOnIntegerOverflow',param.overflowMode,...
    'RndMeth',param.roundingMode,...
    'NumCoefDataTypeStr',NumCoefDataTypeStr,...
    'DenCoefDataTypeStr',DenCoefDataTypeStr,...
    'NumAccumDataTypeStr',NumAccumDataTypeStr,...
    'DenAccumDataTypeStr',DenAccumDataTypeStr,...
    'NumProductDataTypeStr',NumProductDataTypeStr,...
    'DenProductDataTypeStr',DenProductDataTypeStr,...
    'OutDataTypeStr',OutDataTypeStr,...
    'multiplicandDataTypeStr',multiplicandDataTypeStr,...
    'StateDataTypeStr',StateDataTypeStr);

if ~contains(structure,'Direct form II')
    set_param(targetBlock, 'InitialDenominatorStates',...
                            InitialDenominatorStates);
end

%--------------------------------------------------------------------------
function decorations = getDecorationParams(block)
%getDecorationParams Return a cell array with block decorations
%
% Decorations include FontSize, FontWeight, Orientation, etc.
%

decorations = {
    'Position',        [];
    'Orientation',     [];
    'ForegroundColor', [];
    'BackgroundColor', [];
    'DropShadow',      [];
    'NamePlacement',   [];
    'FontName',        [];
    'FontSize',        [];
    'FontWeight',      [];
    'FontAngle',       [];
    'ShowName',        []
    };

for i=1:size(decorations,1)
    decorations{i,2}=get_param(block,decorations{i,1});
end

decorations=reshape(decorations',1,length(decorations(:)));

%--------------------------------------------------------------------------
function fixEdgeCases(param)
% This function runs after the DigitalFilter block has been replaced by the
% target block. Additional blocks must sometimes be added at the input
% ports of the new block to ensure equivalent functionality

blk = [param.parent '/' param.name];
% Add transpose blk if coef param is a col vector
if param.fixColumnCoeffs
    for i=1:length(param.isCoefColVector)
        if param.isCoefColVector(i)
            blkLHandles = get_param(blk, 'LineHandles');
            blkLHInport = get(blkLHandles.Inport(i+1));
            InsertConversionBlock(blk, blkLHInport, 'Pre', 80, -10,...
                'simulink','simulink/Math Operations/Math Function',...
                'Operator','transpose');
        end
    end
end

% Add frame status conversion blk if coef param is frame-based
for i=1:length(param.isCoefPortFrame)
    if param.isCoefPortFrame(i)
        blkLHandles = get_param(blk, 'LineHandles');
        blkLHInport = get(blkLHandles.Inport(i+1));
        InsertConversionBlock(blk, blkLHInport, 'Pre', 80, -10,...
            'dspobslib','dspobslib/Frame Conversion',...
            'OutFrame','Sample-base');
    end
end

% If the scale values were complex, add a
% gain block incorporating the scale values
if isBiquad(param)
    ScaleValues = slResolve(param.ScaleValues,blk);
    if ~isreal(ScaleValues) && isInputFloatingPoint(param)
        gain = prod(ScaleValues);
        blkLHandles = get_param(blk, 'LineHandles');
        blkLHInport = get(blkLHandles.Inport(1));
        InsertConversionBlock(blk, blkLHInport, 'Pre', 80, -10,...
            'simulink','built-in/Gain',...
            'Gain',num2str(gain));
    end
end

scale_a0 = false;

% Scale coefficients if a0 is not normalized
if (isIIR(param)) && ...
        CoefficientsFromInputPort(param) && ...
        ~ignoreA0(param) && ...
        isInputFloatingPoint(param)
    [~ , fcBlkName] = fileparts(tempname);
    
    scale_a0 = true;
    if ~strcmp(param.IIRFiltStruct,'Direct form II')
        insertCoefficientScaleSubsystem(param,fcBlkName,'DF1',blk);
        ports = [2 3];
    else
        insertCoefficientScaleSubsystem(param,fcBlkName,'DF2',blk);
        ports = [1 3];
    end
end

if isfield(param,'fixAllPole')
    scale_a0 = true;
    [~ , fcBlkName] = fileparts(tempname);
    insertCoefficientScaleSubsystem(param,fcBlkName,'DF2',blk);
    ports = [1 2];
end

if scale_a0
    % Delete the line connecting from/to the port
    blkLHandles = get_param(blk, 'LineHandles');
    
    for i=1:2
        
        blkLHInport = get(blkLHandles.Inport(ports(i)));
        delete_line(blkLHInport.Handle);
        
        fcBlkHandle = [param.parent '/' fcBlkName];
        fromBlkName    = get_param(blkLHInport.SrcBlockHandle, 'Name');
        % If the block's name contains '/', replace it by '//'.
        fromBlkName    = regexprep(fromBlkName, '/', '//');
        fromBlkPortNum = get_param(blkLHInport.SrcPortHandle, 'PortNumber');
        toBlkName      = get_param(fcBlkHandle, 'Name');
        
        blkParentName = get_param(blk, 'Parent');
        add_line(blkParentName, ...
            [fromBlkName '/' int2str(fromBlkPortNum)], ...
            [toBlkName   '/' num2str(i)], ...
            'autorouting', 'on');
        
        fromBlkName    = get_param(fcBlkHandle, 'Name');
        toBlkName      = get_param(blkLHInport.DstBlockHandle, 'Name');
        % If the block's name contains '/', replace it by '//'.
        toBlkName      = regexprep(toBlkName, '/', '//');
        add_line(blkParentName, [fromBlkName '/' int2str(i)], ...
            [toBlkName   '/' int2str(ports(i))], ...
            'autorouting', 'on');
    end
end

%-------------------------------------------------------------------------
function newVal = transformFixedPointParameter(OldParameterName,param)
% Transform old style FI parameters to new unified param style
val = param.(OldParameterName);
if ~strcmp(val,'Binary point scaling') && ...
        ~strcmp(val,'Slope and bias scaling') && ...
        ~strcmp(val,'User-defined') && ...
        ~strcmp(val,'Specify word length')
    newVal = sprintf('Inherit: %s',val);
else
    baseName = strrep(OldParameterName,'Mode','');
    wordLengthStr = sprintf('%sWordLength',baseName);
    wordLengthVal =  param.(wordLengthStr);
    if ~strcmp(val,'Specify word length')
        fractionLengthStr =  sprintf('%sFracLength',baseName);
        FractionLengthVal = param.(fractionLengthStr);
        newVal = sprintf('fixdt(1,%s,%s)',wordLengthVal,FractionLengthVal);
    else
        newVal = sprintf('fixdt(1,%s)',wordLengthVal);
    end
end

%-------------------------------------------------------------------------
function coeffs = fixCoefficients(coeffs,oldBlock)
% Coefficients on new filters must be row vector
coeffVal = slResolve(coeffs,oldBlock);
if size(coeffVal,1) > 1
    coeffs = sprintf('(%s).''',coeffs);
end
% Coefficients on new blocks must be signed
if  isa(coeffVal,'logical') || ...
        isa(coeffVal,'uint8') || ...
        isa(coeffVal,'uint16') || ...
        isa(coeffVal,'uint32')
    coeffs = sprintf('double(%s)',coeffs);
end

%-------------------------------------------------------------------------
function InitialStates = fixInitialStates(InitialStates,oldBlock)
% Digital Filter allows empty IC. FIR Filter does not.
ic = slResolve(InitialStates,oldBlock);
if isempty(ic)
    InitialStates = '0';
end
% IC on new blocks must be signed
if  isa(ic,'logical') || ...
        isa(ic,'uint8') || ...
        isa(ic,'uint16') || ...
        isa(ic,'uint32')
    InitialStates = sprintf('double(%s)',InitialStates);
end

% -------------------------------------------------------------------------
function param = setupOncePerSampleSubsystem(blk,param,s)
% Insert For Iterator subsystem to handle case where coefficients are
% updated once per sample

cmplx = param.OutputComplexity;

% If the coefficients are complex and the input is real, the input
% reference fed to the assignment block must be converted to complex.
% Otherwise, the Assignment block will force the output to real.
if any(cmplx(2:end) ~= cmplx(1)) && cmplx(1)==0
    addComplexConversion = true;
else
    addComplexConversion = false;
end

% Input ports (input data and coefficients)
pos = [ 30    38    60    62];
shift1 = 1;
shift2 = 1;
add_block('built-in/Inport',[blk '/In'],'Position',pos);

if addComplexConversion
    add_block(sprintf('simulink/Math Operations/Real-Imag to\nComplex'),...
        [blk '/Convert'],'Position',pos + [100*shift1 0 100*shift1 0]);
    shift1 = shift1 + 1;
    set_param([blk '/Convert'],'Input','Real')
end

if ~isInputFloatingPoint(param)
    add_block(sprintf('dspsigattribs/Data Type \nConversion'),...
        [blk '/DTConvert'], 'Position', pos + [100*shift1 0 100*shift1 0]);
    shift1 = shift1 + 1;
    origSLOutDTStr = param.OutputDT;
    if ~contains(origSLOutDTStr,'flts')
        set_param([blk '/DTConvert'], ...
            'OutDataTypeStr', tostring(fixdt(origSLOutDTStr)));
    end % integer or fixed-point output data type strings
end % input not floating-point

add_block('built-in/Inport',[blk '/coeff1'],...
    'Position',pos + [0 100*shift2 0 100*shift2]);
add_block('simulink/Math Operations/Reshape',[blk '/reshape1'],...
    'Position',pos + [100 100*shift2 100 100*shift2]);
set_param([blk '/reshape1'],'OutputDimensionality','Customize')
set_param([blk '/reshape1'],'OutputDimensions',sprintf('[%d,%d]',...
    param.CoeffPortSize(1)/param.FrameLength,param.FrameLength))
% Selector and transpose blocks to extract one set of coefficients/inputs:
add_block('dspindex/Variable Selector', [blk '/In Selector'],...
    'Position',pos + [100*shift1 0 100*shift1+50 0]);
add_block('dspindex/Variable Selector', [blk '/Coeff1 Selector'],...
    'Position',pos + [200 100*shift2 200+50 100*shift2]);
set_param([blk '/Coeff1 Selector'],'rowsOrCols','Columns')
add_block('simulink/Math Operations/Math Function', [blk '/Transpose'],...
    'Position',pos + [350 100*shift2 350+50 100*shift2],...
    'Operator','transpose');

% Discrete Filter has two coefficient ports - insert extra blocks in this
% case
if strcmp(s.BlockType,'Discrete Filter')
    if ~s.wasAllPole
        shift2 = shift2 + 1;
        add_block('built-in/Inport',[blk '/coeff2'],...
            'Position',pos + [0 100*shift2 0 100*shift2]);
        add_block('simulink/Math Operations/Reshape',[blk '/reshape2'],...
            'Position',pos + [100 100*shift2 100 100*shift2]);
        set_param([blk '/reshape2'],'OutputDimensionality','Customize')
        set_param([blk '/reshape2'],'OutputDimensions',...
            sprintf('[%d,%d]',...
            param.CoeffPortSize(2)/param.FrameLength,...
            param.FrameLength))
        add_block('dspindex/Variable Selector', ...
            [blk '/Coeff2 Selector'], ...
            'Position', pos + [200 100*shift2 200+50 100*shift2]);
        set_param([blk '/Coeff2 Selector'],'rowsOrCols','Columns')
        add_block('simulink/Math Operations/Math Function', ...
            [blk '/Transpose2'], 'Position', ...
            pos + [350 100*shift2 350+50 100*shift2], ...
            'Operator','transpose');
    end
end

% Insert For Iterator. The number of iterations is equal to the frame
% length
shift2 = shift2 + 1;
add_block('built-in/ForIterator', [blk '/For Iterator'],...
    'Position',pos + [0 100*shift2 50 100*shift2]);
set_param([blk '/For Iterator'],...
    'IterationLimit',num2str(param.FrameLength))

add_block(['dsparch4/' s.BlockType], [blk '/Filter'],...
    'Position',pos + [400 100*shift2 400+50 100*shift2+50]);

% The assignment filter concatenates outputs from different samples
add_block('built-in/Assignment', [blk '/Assignment'],...
    'Position',pos + [600 100*shift2 600+50 100*shift2+50]);
set_param([blk '/Assignment'],'NumberOfDimensions','2',...
    'IndexOptions','Index vector (port), Assign all')

% Output port
add_block('built-in/Outport',[blk '/Out'],...
    'Position',pos + [700 100*shift2 700 100*shift2]);

% Connect blocks
add_line(blk,'In/1','In Selector/1','autorouting','on')
if ~addComplexConversion
    if ~isInputFloatingPoint(param)
        add_line(blk,'In/1','DTConvert/1','autorouting','on')
        add_line(blk,'DTConvert/1','Assignment/1','autorouting','on')
    else
        add_line(blk,'In/1','Assignment/1','autorouting','on')
    end
else
    add_line(blk,'In/1','Convert/1','autorouting','on')
    if ~isInputFloatingPoint(param)
        add_line(blk,'Convert/1','DTConvert/1','autorouting','on')
        add_line(blk,'DTConvert/1','Assignment/1','autorouting','on')
    else
        add_line(blk,'Convert/1','Assignment/1','autorouting','on')
    end
end
add_line(blk,'coeff1/1','reshape1/1','autorouting','on')
add_line(blk,'reshape1/1','Coeff1 Selector/1','autorouting','on')
add_line(blk,'For Iterator/1','In Selector/2','autorouting','on')
add_line(blk,'For Iterator/1','Coeff1 Selector/2','autorouting','on')
add_line(blk,'For Iterator/1','Assignment/3','autorouting','on')
if strcmp(s.BlockType,'Discrete Filter')
    if ~s.wasAllPole
        set_param([blk '/Filter'],'NumeratorSource','Input port');
        set_param([blk '/Filter'],'DenominatorSource','Input port');
    else
        set_param([blk '/Filter'],'DenominatorSource','Input port');
    end
else
    set_param([blk '/Filter'],'CoefSource','Input port');
end
add_line(blk,'In Selector/1','Filter/1','autorouting','on')
add_line(blk,'Coeff1 Selector/1','Transpose/1','autorouting','on')
add_line(blk,'Transpose/1','Filter/2','autorouting','on')
add_line(blk,'Filter/1','Assignment/2','autorouting','on')
add_line(blk,'Assignment/1','Out/1','autorouting','on')
if strcmp(s.BlockType,'Discrete Filter')
    if ~s.wasAllPole
        add_line(blk,'coeff2/1','reshape2/1','autorouting','on')
        add_line(blk,'reshape2/1','Coeff2 Selector/1','autorouting','on')
        add_line(blk,'For Iterator/1','Coeff2 Selector/2',...
            'autorouting','on')
        add_line(blk,'Coeff2 Selector/1','Transpose2/1','autorouting','on')
        add_line(blk,'Transpose2/1','Filter/3','autorouting','on')
    end
end

% this is the new filter location
param.parent = [param.parent '/' param.name];
param.name = 'Filter';


% -------------------------------------------------------------------------

function insertCoefficientScaleSubsystem(param,fcBlkName, form,oldBlock)

% For Direct-Form II, scale the input and den by a0
% For Direct-form I, scale num and den by a0
if strcmp(form,'DF2')
    portStr = 'in';
else
    portStr = 'num';
end

blk = [param.parent '/' fcBlkName];
addBlock(oldBlock,'Pre',40,-30,...
         'simulink', 'built-in/SubSystem',blk);
     
add_block('built-in/Inport',[blk '/' portStr],...
    'Position',[30   177    60   193]);
add_block('built-in/Inport',[blk '/den'],...
    'Position',[30   282    60   298]);

add_block('dspmtrx3/Submatrix',[blk '/Select a0'],...
    'Position', [120   272   170   308]);
set_param([blk '/Select a0'],'RowEndMode','Index')
set_param([blk '/Select a0'],'RowEndIndex','1')
set_param([blk '/Select a0'],'ColEndMode','Index')
set_param([blk '/Select a0'],'ColEndIndex','1')

add_block('simulink/Math Operations/Divide',[blk '/Scale ' portStr],...
    'Position',[240   174   290   216]);
add_block('simulink/Math Operations/Divide',[blk '/Scale Denominator'],...
    'Position',[245   279   295   321]);

add_block('built-in/Outport',[blk '/scaled ' portStr],...
    'Position',[460   177   490   193]);
add_block('built-in/Outport',[blk '/scaled den'],...
    'Position',[460   292   490   308]);

add_line(blk,'den/1','Select a0/1','autorouting','on')
add_line(blk,[portStr '/1'],['Scale ' portStr '/1'],'autorouting','on')
add_line(blk,'Select a0/1',['Scale ' portStr '/2'],'autorouting','on')
add_line(blk,['Scale ' portStr '/1'],['scaled ' portStr '/1'],...
    'autorouting','on')
add_line(blk,'den/1','Scale Denominator/1','autorouting','on')
add_line(blk,'Select a0/1','Scale Denominator/2','autorouting','on')
add_line(blk,'Scale Denominator/1','scaled den/1','autorouting','on')

%--------------------------------------------------------------------------
function InsertConversionBlock(blk, blkLHPort, preOrPost, ...
    tpsBlkHOffset, tpsBlkVOffset,...
    libToLoad,blkToInsert,paramName,Val)

% Get the parent block full name
blkParentName = get_param(blk, 'Parent');

% Get a random name for the Transpose block
[~ , tpsBlkName] = fileparts(tempname);
tpsBlkName = [blkParentName '/' tpsBlkName];

% Delete the line connecting from/to the port.
delete_line(blkLHPort.Handle);

% Orient the Transpose block w.r.t. the instance block being updated
blkPos    = get_param(blk, 'Position');
blkOrient = get_param(blk, 'Orientation');
blkCenter = mean([blkPos(1:2); blkPos(3:4)]);
blkWidth  = blkPos(3) - blkPos(1);
blkHeight = blkPos(4) - blkPos(2);

if strcmpi(blkOrient, 'left') || strcmpi(blkOrient, 'right')
    sign = (strcmpi(blkOrient, 'left')  && strcmpi(preOrPost, 'Pre')) ||...
        (strcmpi(blkOrient, 'right') && strcmpi(preOrPost, 'Post'));
    sign = sign * 2 - 1; % Map 0 to -1 and 1 to 1
    tpsBlkCenter = [blkCenter(1) + sign * (blkWidth/2 + tpsBlkHOffset),...
        blkCenter(2) - tpsBlkVOffset];
    tpsBlkWidth  = 20;
    tpsBlkHeight = blkHeight;
elseif (strcmpi(blkOrient, 'up') || strcmpi(blkOrient, 'down'))
    sign = (strcmpi(blkOrient, 'up')   && strcmpi(preOrPost, 'Pre')) ||...
        (strcmpi(blkOrient, 'down') && strcmpi(preOrPost, 'Post'));
    sign = sign * 2 - 1; % Map 0 to -1 and 1 to 1
    tpsBlkCenter = [blkCenter(1) - tpsBlkVOffset,...
        blkCenter(2) + sign * (blkHeight/2 + tpsBlkHOffset)];
    tpsBlkWidth  = blkWidth;
    tpsBlkHeight = 20;
end

tpsBlkPos = [tpsBlkCenter(1) - tpsBlkWidth  / 2, ...
    tpsBlkCenter(2) - tpsBlkHeight / 2, ...
    tpsBlkCenter(1) + tpsBlkWidth  / 2, ...
    tpsBlkCenter(2) + tpsBlkHeight / 2];

% Load dspmtrx3 library that contains the Transpose block.
load_system(libToLoad);

% Add the Transpose block and hide its name
tpsBlkHandle = add_block(blkToInsert, tpsBlkName, ...
    'Position',           tpsBlkPos, ...
    'Orientation',        blkOrient, ...
    'ShowName',           'off');
if ~isempty(paramName)
    set_param(tpsBlkHandle,paramName,Val);
end

tpsBlkPortH = get_param(tpsBlkHandle, 'PortHandles');

% Reconnect lines from the original source block to the Transpose block,
% and from the Transpose block to the original destination block.
fromBlkName    = get_param(blkLHPort.SrcBlockHandle, 'Name');
% If the block's name contains '/', replace it by '//'.
fromBlkName    = regexprep(fromBlkName, '/', '//');
fromBlkPortNum = get_param(blkLHPort.SrcPortHandle, 'PortNumber');
toBlkName      = get_param(tpsBlkHandle, 'Name');
toBlkPortNum   = get_param(tpsBlkPortH.Inport, 'PortNumber');
add_line(blkParentName, [fromBlkName '/' int2str(fromBlkPortNum)], ...
    [toBlkName   '/' int2str(toBlkPortNum)], ...
    'autorouting', 'on');

fromBlkName    = get_param(tpsBlkHandle, 'Name');
fromBlkPortNum = get_param(tpsBlkPortH.Outport, 'PortNumber');
toBlkName      = get_param(blkLHPort.DstBlockHandle, 'Name');
% If the block's name contains '/', replace it by '//'.
toBlkName      = regexprep(toBlkName, '/', '//');
toBlkPortNum   = get_param(blkLHPort.DstPortHandle, 'PortNumber');
add_line(blkParentName, [fromBlkName '/' int2str(fromBlkPortNum)], ...
    [toBlkName   '/' int2str(toBlkPortNum)], ...
    'autorouting', 'on');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Utility Functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------------------------------------------------------------------
function flag = updateCoefficientsEachSample(oldBlock)
flag = strcmp(oldBlock.FiltPerSampPopup,'One filter per sample') && ...
    CoefficientsFromInputPort(oldBlock) && ...
    strcmp(oldBlock.InputProcessing,'Columns as channels (frame based)');
%-------------------------------------------------------------------------
function flag = isIIR(oldBlock)
flag = strcmp(oldBlock.TypePopup,'IIR (poles & zeros)') && ...
    isempty(strfind(oldBlock.IIRFiltStruct,'SOS'));
%-------------------------------------------------------------------------
function flag = isBiquad(oldBlock)
flag = strcmp(oldBlock.TypePopup,'IIR (poles & zeros)') && ...
    ~isempty(strfind(oldBlock.IIRFiltStruct,'SOS'));
%-------------------------------------------------------------------------
function flag = isAllPole(oldBlock)
flag = strcmp(oldBlock.TypePopup,'IIR (all poles)');
%-------------------------------------------------------------------------
function flag = isFIR(oldBlock)
flag = strcmp(oldBlock.TypePopup,'FIR (all zeros)');
%-------------------------------------------------------------------------
function flag = CoefficientsFromInputPort(oldBlock)
flag = strcmp(oldBlock.FilterSource,'Input port(s)');
%-------------------------------------------------------------------------
function flag = ignoreA0(oldBlock)
flag = strcmp(oldBlock.denIgnore,'on');
%-------------------------------------------------------------------------
function flag = isInputFloatingPoint(oldBlock)
flag = strcmp(oldBlock.OutputDT,'double') || ...
    strcmp(oldBlock.OutputDT,'single');
%-------------------------------------------------------------------------
function flag = isdfilt(oldBlock)
flag = strcmp(oldBlock.FilterSource,'dfilt object');
%-------------------------------------------------------------------------
function flag = isDF2TA0NonUnityFromPort(param)
flag = ( (isIIR(param)&& ...
    strcmp(param.IIRFiltStruct,'Direct form II transposed'))...
    || (isAllPole(param) && ...
    strcmp(param.AllPoleFiltStruct,'Direct form transposed'))) && ...
    strcmp(param.denIgnore,'off') ...
    && strcmp( param.CoeffSource,'Input port(s)');
%-------------------------------------------------------------------------
function flag = isBiquadNonZeroIC(param,oldBlock)
if strcmp(param.IIRFiltStruct,'Biquad direct form II transposed (SOS)') ...
        || strcmp(param.IIRFiltStruct,'Biquad direct form II (SOS)')
    IC = fixInitialStates(param.IC,oldBlock);
    if any(slResolve(IC,oldBlock))
        flag = true;
    else
        flag = false;
    end
else
    ICnum = fixInitialStates(param.ICnum,oldBlock);
    ICden = fixInitialStates(param.ICden,oldBlock);
    flag  = ...
        any(slResolve(ICnum,oldBlock)) || ...
        any(slResolve(ICden,oldBlock));
end
