function varargout = plotDCBlockerFilter(block)
% PLOTDCBLOCKERFILTER Plot response of filter used in DC Blocker block
%   PLOTDCBLOCKERFILTER(BLOCK) plots the response of the lowpass filter
%   specified by the DC Blocker block BLOCK.  BLOCK is a string
%   that specifies the name of the DC Blocker block.  The function uses
%   FVTOOL to visualize the filter response
%
%   H = PLOTDCBLOCKERFILTER(BLOCK) plots the filter response and creates a
%   handle, H, to an FVTOOL object.
%
%   Example:
%   % Visualize the filter in the DC Blocker in the model
%   % docDCBlockerCICMode.
%   open_system('docDCBlockerCICMode');
%   block = [bdroot '/DC Blocker'];
%   plotDCBlockerFilter(block);
%
%   See also FVTOOL.

%   Copyright 2014-2021 The MathWorks, Inc.

narginchk(1,1)
nargoutchk(0,1)

if (~strcmp(get_param(block,'masktype'), 'dsp.simulink.DCBlocker'))
    error(message('dsp:system:DCBlocker:WrongBlock'));
end

% Instantiate a System object with the appropriate parameters, and execute
% the fvtool method on it.
alg = get_param(block, 'Algorithm');

if strcmpi(alg, 'IIR')
  normBW = str2double(get_param(block, 'NormalizedBandwidth'));
  order = str2double(get_param(block, 'Order'));
  hDCBlocker = dsp.DCBlocker(...
    'Algorithm', 'IIR', ...
    'NormalizedBandwidth', normBW, ...
    'Order', order);
elseif strcmpi(alg, 'FIR')
  length = str2double(get_param(block, 'Length'));
  hDCBlocker = dsp.DCBlocker(...
    'Algorithm', 'FIR', ...
    'Length', length);
elseif strcmpi(alg, 'CIC')
  normBW = str2double(get_param(block, 'NormalizedBandwidth'));
  hDCBlocker = dsp.DCBlocker(...
    'Algorithm', 'CIC', ...
    'NormalizedBandwidth', normBW);
elseif strcmpi(alg, 'Subtract mean')
  error(message('dsp:system:DCBlocker:DCBlocker_novisualization'));
end

if nargout > 0
  varargout{1} = fvtool(hDCBlocker);
else
  fvtool(hDCBlocker);
end

% EOF
