classdef(CaseInsensitiveProperties=true) cicinterp < mfilt.abstractcicinterp
    %CICINTERP Cascaded Integrator-Comb Interpolator.
    %   MFILT.CICINTERP will be removed in a future release. Use
    %   dsp.CICInterpolator instead.


    %   Hm = MFILT.CICINTERP(R,M,N,IWL,OWL,WLPS) constructs a Cascaded
    %   Integrator-Comb (CIC) interpolation filter. This structure has a
    %   latency of N (number of sections).  The construction of this object
    %   requires the Fixed-Point Designer.
    %
    %   Inputs:
    %   R is the interpolation factor
    %   M is the differential delay
    %   N is the number of sections
    %   IWL is the wordlength of the input signal
    %   OWL is the wordlength of the output signal
    %   WLPS can be a scalar or vector (of length 2*N). WLPS defines the word
    %        lengths per section used during either the accumulation of the
    %        data in the integrator sections or the subtraction of the data
    %        performed by the comb section (using 'wrap' arithmetic). If WLPS
    %        is a scalar, that value is applied to each filter section.
    %
    %   % EXAMPLE: Interpolation by a factor of 2 (used to convert from 22.05kHz
    %   % to 44.1kHz)
    %   R = 2;                    % Interpolation factor
    %   Hm = mfilt.cicinterp(R);  % Use default NumberOfSections & DifferentialDelay
    %   Fs = 22.05e3;             % Original sampling frequency: 22.05kHz
    %   n = 0:5119;               % 5120 samples, 0.232 second long signal
    %   x  = sin(2*pi*1e3/Fs*n);  % Original signal, sinusoid at 1kHz
    %
    %   y_fi = filter(Hm,x); % 5120 samples, still 0.232 seconds
    %
    %   % Scale the output to overlay plots
    %   x = double(x); y = double(y_fi); y = y/max(abs(y));
    %   stem(n(1:22)/Fs,x(1:22),'filled'); hold on; % Plot original sampled at 22.05kHz
    %   stem(n(1:44)/(Fs*R),y(4:47),'r');           % Plot interpolated signal (44.1kHz) in red
    %   xlabel('Time (sec)');ylabel('Signal value');
    %
    %   See also MFILT/STRUCTURES.
   
    %   Copyright 2019 The MathWorks, Inc.    

    %mfilt.cicinterp class
    %   mfilt.cicinterp extends mfilt.abstractcicinterp.
    %
    %    mfilt.cicinterp properties:
    %       PersistentMemory - Property is of type 'bool'
    %       NumSamplesProcessed - capture (read only)
    %       FilterStructure - Property is of type 'ustring'  (read only)
    %       States - Property is of type 'mxArray'
    %       DifferentialDelay - Property is of type 'posint user-defined'
    %       NumberOfSections - Property is of type 'posint user-defined'
    %       Arithmetic - Property is of type 'filterdesign_cic_arith enumeration: {'fixed'}'
    %       InterpolationFactor - Property is of type 'posint user-defined'
    %
    %    mfilt.cicinterp methods:
    %       blocklib - Returns the library and source block
    %       createhdlfilter - Returns the corresponding hdlfiltercomp object
    %       dgdfgen - generate the dg_dfilt structure from a specified filter structure
    %       thisdisp -   Display this object.
    %       filt2struct -   Return the structure representation of the filter.
    %       getblockraterestrictions - Get rate restrictions for the block method.
    %       secfilter - Filter this section.
    %       thisblockparams -   Return the block params.
    %       thisfiltquant_plugins - FILTQUANT_PLUGINS Filterquantizer plugin for MFILT.CICINTERP.
    %       tocicinterp -   Convert to a CIC Interpolator.
    %       tosysobj - Convert mfilt CIC interpolator structure to System object
    
    
    
    methods  % constructor block
        function Hm = cicinterp(varargin)
            if ~isfixptinstalled
                error(message('dsp:mfilt:cicinterp:cicinterp:fixptTbxRq'));
            end
            Hm.privfq = dfilt.filterquantizer;
            Hm.privfilterquantizer = dfilt.filterquantizer;
            % Set Object Default States
            factNofSections = 2;
            factDiffDelay = 1;
            zi = filtstates.cic(zeros(factNofSections,1),zeros(factNofSections,factDiffDelay));
            Hm.States = zi;
            Hm.FilterStructure = 'Cascaded Integrator-Comb Interpolator';
            
            set_arith(Hm, 'fixed');
            Hm.InterpolationFactor = 2;
            if nargin > 0
                Hm.InterpolationFactor = varargin{1};
            end
            
            % Initialization utility.
            cic_init(Hm,varargin{2:end});
        end  % cicinterp
    end  % constructor block
    
    methods  % public methods
        [lib,srcblk,hasInputProcessing,hasRateOptions] = blocklib(~,~)
        hF = createhdlfilter(this)
        DGDF = dgdfgen(Hd,hTar,doMapCoeffsToPorts)
        thisdisp(this)
        s = filt2struct(this)
        r = getblockraterestrictions(~,inputProcessing)
        [y,zf] = secfilter(Hm,x,zi)
        p = thisblockparams(this)
        constr = thisfiltquant_plugins(h,arith)
        Hm = tocicinterp(this)
        Hs = tosysobj(this,returnSysObj)
    end  % public methods
    
    methods (Hidden) % possibly private or hidden
        s = objblockparams(this,varname)
    end  % possibly private or hidden
    
end  % classdef

