classdef(CaseInsensitiveProperties=true) fftfirinterp < mfilt.abstractfirinterp
%FFTFIRINTERP Overlap-Add FIR Polyphase Interpolator.

%   Copyright 2017-2020 The MathWorks, Inc.



    %   MFILT.FFTFIRINTERP will be removed in a future release. Use
    %   dsp.FIRInterpolator instead.
    %
    %   Hd = MFILT.FFTFIRINTERP(L,NUM,BL) constructs an overlap-add FIR
    %   polyphase interpolator.
    %
    %   L is the interpolation factor. It must be an integer. If not specified,
    %   it defaults to 2.
    %
    %   NUM is a vector containing the coefficients of the FIR lowpass filter
    %   used for interpolation. If omitted, a lowpass Nyquist filter of gain L
    %   and cutoff frequency of Pi/L is used by default.
    %
    %   BL is the length of each block of input data used in the filtering. It
    %   must be an integer. If not specified, it defaults to 100.
    %
    %   The number of FFT points is given by BL+ceil(length(NUM)/L)-1. It may be
    %   advantageous to choose BL such that the number of FFT points is a power
    %   of two.
    %
    %   EXAMPLE:
    %     %Interpolation by a factor of 8 (notice the removal of spectral replicas)
    %     L = 8;                                % Interpolation factor
    %     Hm = mfilt.fftfirinterp(L);           % We use the default filter
    %     N = 8192;                             % Number of points
    %     Hm.BlockLength = N;                   % Set block length to number of points
    %     Fs = 44.1e3;                          % Original sampling frequency: 44.1kHz
    %     n = 0:N-1;                            % 0.1858 secs of data
    %     x = sin(2*pi*n*22e3/Fs);              % Original signal, sinusoid at 22kHz
    %     y = filter(Hm,x);                     % Interpolated sinusoid
    %     xu = L*upsample(x,8);                 % Upsample to compare (spectrum doesn't change)
    %     [Px,f]=periodogram(xu,[],65536,L*Fs); % Power spectrum of original
    %     [Py,f]=periodogram(y,[],65536,L*Fs);  % Power spectrum of interpolated
    %     plot(f,10*log10(([Fs*Px,L*Fs*Py])))
    %     legend('22 kHz sinusoid sampled at 44.1 kHz',...
    %     '22 kHz sinusoid sampled at 352.8 kHz')
    %     xlabel('Frequency (Hz)'); ylabel('Power spectrum');
    %
    %   See also MFILT/STRUCTURES.

    
    %mfilt.fftfirinterp class
    %   mfilt.fftfirinterp extends mfilt.abstractfirinterp.
    %
    %    mfilt.fftfirinterp properties:
    %       PersistentMemory - Property is of type 'bool'
    %       NumSamplesProcessed - capture (read only)
    %       FilterStructure - Property is of type 'ustring'  (read only)
    %       States - Property is of type 'mxArray'
    %       Numerator - Property is of type 'DFILTNonemptyVector user-defined'
    %       InterpolationFactor - Property is of type 'posint user-defined'
    %       BlockLength - Property is of type 'posint user-defined'
    %       NonProcessedSamples - Property is of type 'MATLAB array'  (read only)
    %
    %    mfilt.fftfirinterp methods:
    %       blockparams - Returns the parameters for BLOCK
    %       thisdisp - Object display.
    %       fftcoeffs -  Get the FFT coefficients used for the filtering.
    %       filt2struct -   Returns a structure representation of the filter.
    %       genmcode -   Generate an MATLAB code object for this object.
    %       getstates -  Get the filter states.
    %       isfixedptable -   True if the object is fixedptable.
    %       thisisrealizable -   True if the object is realizable.
    %       loadpublicinterface -   Load the public interface.
    %       multirate_info - THISINFO
    %       polyorder -  Get the order of the polyphase filters.
    %       savepublicinterface -   Save the public interface.
    %       secfilter - Filter this section.
    %       set2int - Scales the coefficients to integer numbers.
    %       setblocklength - Overloaded set on the blocklength property.
    %       setnum - Overloaded set for the Numerator property.
    %       setpolyphasematrix -   Set the polyphasematrix.
    %       thisnstates - NSTATES  Number of states in discrete-time filter.
    %       thisreset - Reset the non processed samples.
    %       thissetstates - Overloaded set for the States property.
    %       tofftfirinterp -   Convert to an FFT FIR Interpolator.
    %       tofirinterp -   Convert to an FIR Interpolator.
    %       tosysobj - Convert to a System object
    %       ziexpand - Expand initial conditions for multiple channels when necessary
    %       ziscalarexpand - Expand empty or scalar initial conditions to a vector.
    
    
    properties (Access=protected, SetObservable, GetObservable)
        %FFTCOEFFS Property is of type 'MATLAB array'
        pfftcoeffs = [];
    end
    
    properties (SetAccess=protected, SetObservable, GetObservable)
        %NONPROCESSEDSAMPLES Property is of type 'MATLAB array'  (read only)
        NonProcessedSamples = [];
    end
    
    properties (SetObservable, GetObservable)
        %BLOCKLENGTH Property is of type 'posint user-defined'
        BlockLength = [];
    end
    
    
    methods  % constructor block
        function Hm = fftfirinterp(L,num,BL)
            deprecationMessage(Hm,'dsp.FIRInterpolator'); % Deprecated in R2016a.
            Hm.privfq = dfilt.filterquantizer;
            Hm.privfilterquantizer = dfilt.filterquantizer;
            Hm.FilterStructure = 'Overlap-Add FIR Polyphase Interpolator';
            set_privarith(Hm,'double');
            Hm.InterpolationFactor = 2;
            Hm.BlockLength = 100;
            if nargin>0
                Hm.InterpolationFactor = L;
            end
            
            if nargin < 2
                Hm.Numerator = defaultfilter(Hm,Hm.InterpolationFactor,1);
            elseif isa(num,'dfilt.singleton') && isfir(num)
                [b,~] = tf(num);
                Hm.Numerator = b;
            else
                num = num(:).';
                Hm.Numerator = num;
            end
            
            if nargin < 3
                % Set a default blocklength
                % Don't use factoryValue so overload set runs
                BL = 100;
            end
            Hm.BlockLength = BL;
        end  % fftfirinterp
    end  % constructor block
    
    methods
        function set.BlockLength(obj,value)
            % User-defined DataType = 'posint user-defined'
            obj.BlockLength = setblocklength(obj,value);
        end
        
    end   % set and get functions
    
    methods  % public methods
        [lib,srcblk,s] = blockparams(Hd,mapstates,varargin)
        thisdisp(this)
        c = fftcoeffs(Hm)
        s = filt2struct(this)
        hcode = genmcode(this,varname,place)
        z = getstates(Hm,dummy)
        b = isfixedptable(this)
        b = thisisrealizable(this)
        loadpublicinterface(this,s)
        [p,v] = multirate_info(Hm)
        np = polyorder(Hm)
        s = savepublicinterface(this)
        [y,z] = secfilter(Hm,x,z)
        varargout = set2int(this,coeffWL,inWL)
        BL = setblocklength(Hm,BL)
        num = setnum(Hm,num)
        polym = setpolyphasematrix(this,polym)
        n = thisnstates(Hm)
        thisreset(Hm)
        S = thissetstates(Hm,S)
        Hm = tofftfirinterp(this)
        Hm = tofirinterp(this)
        Hs = tosysobj(this,returnSysObj)
        zi = ziexpand(Hm,x,zi)
        S = ziscalarexpand(Hm,S)
        info = qtoolinfo(this)
    end  % public methods
    
    
    methods (Hidden) % possibly private or hidden
        BL = blocklength(Hm)
        c = evalcost(this)
        xi = getnonprocessedsamples(Hm)
        loadr14(H,s)
        setnonprocessedsamples(Hm,xf)
    end  % possibly private or hidden
    
end  % classdef

