classdef(CaseInsensitiveProperties=true) firinterp < mfilt.abstractfirinterp
%FIRINTERP Direct-Form FIR Polyphase Interpolator.

%   Copyright 2017-2020 The MathWorks, Inc.


%   MFILT.FIRINTERP will be removed in a future release. Use
%   dsp.FIRInterpolator instead.
%
%   Hm = MFILT.FIRINTERP(L,NUM) returns a direct-form FIR polyphase
%   interpolator Hm.
%
%   L is the interpolation factor. It must be an integer. If not specified,
%   it defaults to 2.
%
%   NUM is a vector containing the coefficients of the FIR lowpass filter
%   used for interpolation. If omitted, a low-pass Nyquist filter of gain L
%   and cutoff frequency of Pi/L is designed by default.
%
%   EXAMPLE:
%      %Interpolation by a factor of 2 (used to convert from 22.05kHz
%      %to 44.1kHz)
%      L = 2;                               % Interpolation factor
%      Hm = mfilt.firinterp(L);             % We use the default filter
%      Fs = 22.05e3;                        % Original sampling frequency: 22.05kHz
%      n = 0:5119;                          % 5120 samples, 0.232 second long signal
%      x  = sin(2*pi*1e3/Fs*n);             % Original signal, sinusoid at 1kHz
%      y = filter(Hm,x);                    % 10240 samples, still 0.232 seconds
%      stem(n(1:22)/Fs,x(1:22),'filled')    % Plot original sampled at 22.05kHz
%      hold on                              % Plot interpolated signal (44.1kHz) in red
%      stem(n(1:44)/(Fs*L),y(25:68),'r')
%      xlabel('Time (sec)');
%      ylabel('Signal value')
%
%   See also MFILT/STRUCTURES.


%mfilt.firinterp class
%   mfilt.firinterp extends mfilt.abstractfirinterp.
%
%    mfilt.firinterp properties:
%       PersistentMemory - Property is of type 'bool'
%       NumSamplesProcessed - capture (read only)
%       FilterStructure - Property is of type 'ustring'  (read only)
%       States - Property is of type 'mxArray'
%       Numerator - Property is of type 'DFILTNonemptyVector user-defined'
%       InterpolationFactor - Property is of type 'posint user-defined'
%       Arithmetic - Property is of type 'filterdesign_arith enumeration: {'double','single','fixed'}'
%
%    mfilt.firinterp methods:
%       blocklib - Returns the library and source block
%       blockparams - Returns the parameters for BLOCK
%       createhdlfilter - Returns the corresponding hdlfiltercomp object
%       dgdfgen - generate the dg_dfilt structure from a specified filter structure
%       thisdisp - Object display.
%       filt2struct -   Returns a structure representation of the filter.
%       genmcode -   Generate an MATLAB code object for this object.
%       getbestprecision - Return best precision for Product and Accumulator
%       getstates - Overloaded get for the States property.
%       isblockable - True if the object supports the block method
%       ishdlable - True if HDL can be generated for the filter object.
%       loadpublicinterface -   Load the public interface.
%       naddp1 -   Returns the number of additions + 1.
%       secfilter - Polyphase FIR Interpolator filter
%       set_tapindex -   PreSet function for the 'tapindex' property.
%       thisfiltquant_plugins - FILTQUANT_PLUGINS Table of filterquantizer plugins
%       thisnstates - NSTATES  Number of states in discrete-time filter.
%       thissetstates - Overloaded set for the States property.
%       tofftfirinterp -   Convert to an Overlap-Add Interpolator.
%       tofirinterp -   Convert to an FIR Interpolator.


properties (SetObservable, GetObservable)
    %ARITHMETIC Property is of type 'filterdesign_arith enumeration: {'double','single','fixed'}'
    Arithmetic = 'double';
end


methods  % constructor block
    function Hm = firinterp(L,num)
        narginchk(0,2);
        Hm.privfq = dfilt.filterquantizer;
        Hm.privfilterquantizer = dfilt.filterquantizer;
        Hm.FilterStructure = 'Direct-Form FIR Polyphase Interpolator';
        Hm.Arithmetic = 'double';
        Hm.InterpolationFactor = 2;
        
        if nargin>0
            Hm.InterpolationFactor = L;
        end
        
        if nargin < 2
            Hm.Numerator = defaultfilter(Hm,Hm.InterpolationFactor,1);
        elseif isa(num,'dfilt.singleton') && isfir(num)
            [b,~] = tf(num);
            Hm.Numerator = b;
        else
            num = num(:).';
            Hm.Numerator = num;
        end
    end  % firinterp
end  % constructor block

methods
    function value = get.Arithmetic(obj)
        value = get_arith(obj,obj.Arithmetic);
    end
    function set.Arithmetic(obj,value)
        % Enumerated DataType = 'filterdesign_arith enumeration: {'double','single','fixed'}'
        value = validatestring(value,{'double','single','fixed'},'','Arithmetic');
        obj.Arithmetic = set_arith(obj,value);
    end
    
end   % set and get functions

methods  % public methods
    [lib,srcblk,hasInputProcessing,hasRateOptions] = blocklib(~,~)
    s = blockparams(Hd,mapstates,varargin)
    hF = createhdlfilter(this)
    DGDF = dgdfgen(Hd,hTar,doMapCoeffsToPorts)
    thisdisp(this)
    s = filt2struct(this)
    hcode = genmcode(this,varname,place)
    s = getbestprecision(h)
    S = getstates(Hm,S)
    b = isblockable(~)
    [result,errstr,errorObj] = ishdlable(filterobj)
    loadpublicinterface(this,s)
    n = naddp1(h)
    [y,z] = secfilter(Hm,x,z)
    tapindex = set_tapindex(H,tapindex)
    constr = thisfiltquant_plugins(h,arith)
    n = thisnstates(Hm)
    S = thissetstates(Hd,S)
    Hm = tofftfirinterp(this)
    Hm = tofirinterp(this)
end  % public methods


methods (Hidden) % possibly private or hidden
    loadr14(H,s)
    [p,v] = multirate_info(Hm)
    s = objblockparams(this,varname)
    specifyall(this,flag)
    thisreset(H)
end  % possibly private or hidden
end  % classdef

