classdef(CaseInsensitiveProperties=true) holdinterp < dsp.internal.mfilt.linearinterp
%HOLDINTERP FIR Hold Interpolator

%   Copyright 2017-2021 The MathWorks, Inc.


%   Hm = MFILT.HOLDINTERP(L) returns an FIR hold interpolator Hm with an
%   interpolation factor L. L must be an integer. If not specified,
%   it defaults to 2.
%
%   EXAMPLE:
%      %Interpolation by a factor of 2 (used to convert from 22.05kHz
%      %to 44.1kHz)
%      L = 2;                               % Interpolation factor
%      Hm = dsp.internal.mfilt.holdinterp(L);
%      Fs = 22.05e3;                        % Original sampling frequency: 22.05kHz
%      n = 0:5119;                          % 5120 samples, 0.232 second long signal
%      x  = sin(2*pi*1e3/Fs*n);             % Original signal, sinusoid at 1kHz
%      y = filter(Hm,x);                    % 10240 samples, still 0.232 seconds
%      stem(n(1:22)/Fs,x(1:22),'filled')    % Plot original sampled at 22.05kHz
%      hold on                              % Plot interpolated signal (44.1kHz) in red
%      stem(n(1:44)/(Fs*L),y(1:44),'r')
%      xlabel('Time (sec)');ylabel('Signal value')
%
%   See also MFILT/STRUCTURES.


%mfilt.holdinterp class
%   mfilt.holdinterp extends mfilt.linearinterp.
%
%    mfilt.holdinterp properties:
%       PersistentMemory - Property is of type 'bool'
%       NumSamplesProcessed - capture (read only)
%       FilterStructure - Property is of type 'ustring'  (read only)
%       States - Property is of type 'mxArray'
%       Numerator - Property is of type 'DFILTNonemptyVector user-defined'
%       InterpolationFactor - Property is of type 'posint user-defined'
%       Arithmetic - Property is of type 'filterdesign_arith enumeration: {'double','single','fixed'}'
%
%    mfilt.holdinterp methods:
%       blocklib - Returns the library and source block
%       blockparams - Returns the parameters for BLOCK
%       createhdlfilter - Returns the corresponding hdlfiltercomp object
%       getRateOptionParamName - Get the rateOptionParamName.
%       getstates -   Get the states.
%       ishdlable - True if HDL can be generated for the filter object.
%       secfilter - Polyphase FIR Interpolator filter
%       setfilterstructure -   Set the filterstructure.
%       setinterp - Overloaded set for the InterpolationFactor property.
%       thisfiltquant_plugins - FILTQUANT_PLUGINS Table of filterquantizer plugins
%       thisnstates - NSTATES  Number of states in discrete-time filter.
%       thissetstates -   Set the states.
%       toholdinterp -   Convert to a hold interpolator.
%       tosysobj - Convert to a System object

methods  % constructor block
    function Hm = holdinterp(L)
        narginchk(0,1);
        Hm.privfq = dfilt.filterquantizer;
        Hm.privfilterquantizer = dfilt.filterquantizer;
        Hm.FilterStructure = 'FIR Hold Interpolator';
        Hm.Arithmetic = 'double';
        Hm.InterpolationFactor = 2;

        if nargin>0
            Hm.InterpolationFactor = L;
        end
    end  % holdinterp
end  % constructor block

methods  % public methods
    [lib,srcblk,hasInputProcessing,hasRateOptions] = blocklib(~,~)
    s = blockparams(Hd,mapstates,varargin)
    hF = createhdlfilter(this)
    rateOptionParamName = getRateOptionParamName(~)
    S = getstates(this,S)
    [result,errstr,errorObj] = ishdlable(filterobj)
    [y,z] = secfilter(Hm,x,z)
    setfilterstructure(H)
    L = setinterp(Hm,L)
    constr = thisfiltquant_plugins(h,arith)
    n = thisnstates(Hm)
    S = thissetstates(Hd,S)
    Hm = toholdinterp(this)
    Hs = tosysobj(this,returnSysObj)
    info = qtoolinfo(this)
end  % public methods


methods (Hidden) % possibly private or hidden
    [targs,strs] = convertchoices(this)
    [out,coeffnames,variables] = mapcoeffstoports(this,varargin)
end  % possibly private or hidden

end  % classdef

