classdef(CaseInsensitiveProperties=true) iirinterp < mfilt.abstractiirinterp
    %IIRINTERP   Construct an IIR interpolator filter object.
    %   MFILT.IIRINTERP will be removed in a future release. Use
    %   dsp.IIRHalfbandInterpolator instead.
    %
    %   Hm = MFILT.IIRINTERP(C1,C2,...) constructs an IIR interpolator filter
    %   given the coefficients specified in the cell arrays C1, C2, etc.
    %
    %   The IIR interpolator is a polyphase IIR filter where each phase is a
    %   cascade allpass IIR filter.
    %
    %   Each cell array Ci, contains a set of vectors representing a cascade of
    %   allpass sections. Each element in the cell array is one section. For
    %   more information on the contents of each cell array, see the help for
    %   DFILT.CASCADEALLPASS. The contents of the cell array are the same as
    %   the ones specified in the constructor of DFILT.CASCADEALLPASS and have
    %   the same interpretation. However, the following exception applies: if
    %   one of the cell arrays contains only one vector which consists of a
    %   series of zeros and a unique element equal to one, that cell array
    %   represent a DFILT.DELAY with latency equal to the number of zeros
    %   rather than a DFILT.CASCADEALLPASS. This case occurs with quasi linear
    %   phase IIR interpolators.
    %
    %   Note that one usually does not construct IIR interpolators explicitly as
    %   discussed so far. Instead, one obtains these filters as a result from a
    %   design of a halfband interpolator. An example of this is shown below.
    %
    %   % Example # 1: Design an elliptic halfband interpolator with a interpolation
    %   % factor of 2.
    %   TW  = 100;  % Transition width of filter to be designed, 100 Hz
    %   Ast = 80;   % Stopband attenuation of filter to be designed, 80 db
    %   Fs  = 2000; % Sampling frequency of the filter
    %   L   = 2;    % Interpolation factor
    %   f = fdesign.interpolator(L,'halfband','TW,Ast',TW,Ast,Fs); % Store interpolator design specs
    %   % Now perform actual design. Hm results in an mfilt.iirinterp
    %   Hm = design(f,'ellip','FilterStructure','iirinterp');
    %   realizemdl(Hm) % Requires Simulink; build a Simulink model for the filter
    %
    %   % Example # 2: Design an linear phase halfband interpolator with an interpolation
    %   % factor of 2.
    %   TW  = 100;  % Transition width of filter to be designed, 100 Hz
    %   Ast = 60;   % Stopband attenuation of filter to be designed, 60 db
    %   Fs  = 2000; % Sampling frequency of the filter
    %   L   = 2;    % Interpolation factor
    %   f = fdesign.interpolator(L,'halfband','TW,Ast',TW,Ast,Fs); % Store interpolator design specs
    %   % Now perform actual design. Hm results in an mfilt.iirinterp
    %   Hm = design(f,'iirlinphase','FilterStructure','iirinterp');
    %   realizemdl(Hm) % Requires Simulink; build a Simulink model for the filter
    %
    %   See also MFILT/STRUCTURES.

%   Copyright 2017-2022 The MathWorks, Inc.
    
    %mfilt.iirinterp class
    %   mfilt.iirinterp extends mfilt.abstractiirinterp.
    %
    %    mfilt.iirinterp properties:
    %       PersistentMemory - Property is of type 'bool'
    %       NumSamplesProcessed - capture (read only)
    %       FilterStructure - Property is of type 'ustring'  (read only)
    %       States - Property is of type 'mxArray'
    %       Polyphase - Property is of type 'mxArray'
    %       InterpolationFactor - Property is of type 'posint user-defined'  (read only)
    %
    %    mfilt.iirinterp methods:
    
    %   Author(s): R. Losada
    
    methods  % constructor block
        function this = iirinterp(varargin)
            this.privfq = dfilt.filterquantizer;
            this.privfilterquantizer = dfilt.filterquantizer;
            this.FilterStructure = 'IIR Polyphase Interpolator';
            set_privarith(this,'double');
            this.InterpolationFactor = 2;
            for n = 1:nargin
                fn{n} = sprintf('Phase%d',n); %#ok<AGROW>
                for k = 1:length(varargin{n})
                    phasefn{k} = sprintf('Section%d',k); %#ok<AGROW>
                end
                fv{n} = cell2struct(varargin{n},phasefn,2); %#ok<AGROW>
                clear phasefn;
            end
            
            
            if nargin > 0
                this.Polyphase = cell2struct(fv,fn,2);
            else
                % Suppress the deprecation warning on calling design method without
                % "SystemObject" flag set to true.
                w = warning('off','dsp:fdesign:basecatalog:UseSystemObjectForMultirateDesigns');
                wCleanup = onCleanup(@() warning(w));
                % Design a filter
                f = fdesign.interpolator(2,'halfband');
                h = ellip(f, AllowLegacyFilters=true);
                this.Polyphase = h.polyphase;
            end
            
            
        end  % iirinterp
        
    end  % constructor block
    
    methods (Hidden) % possibly private or hidden
        constr = dfiltconstructor(this)
        [out,coeffnames,variables] = mapcoeffstoports(this,varargin)
    end  % possibly private or hidden
    
end  % classdef

