classdef (CaseInsensitiveProperties=true) linearinterp < mfilt.abstractfirinterp
%LINEARINTERP FIR Linear Interpolator.


%   Copyright 2017-2021 The MathWorks, Inc.


%   MFILT.LINEARINTERP will be removed in a future release. Use
%   dsp.CICInterpolator instead.
%
%   Hm = MFILT.LINEARINTERP(L) returns an FIR linear interpolator Hm with
%   an interpolation factor L. L must be an integer. If not specified,
%   it defaults to 2.
%
%   EXAMPLE:
%      %Interpolation by a factor of 2 (used to convert from 22.05kHz
%      %to 44.1kHz)
%      L = 2;                               % Interpolation factor
%      Hm = mfilt.linearinterp(L);
%      Fs = 22.05e3;                        % Original sampling frequency: 22.05kHz
%      n = 0:5119;                          % 5120 samples, 0.232 second long signal
%      x  = sin(2*pi*1e3/Fs*n);             % Original signal, sinusoid at 1kHz
%      y = filter(Hm,x);                    % 10240 samples, still 0.232 seconds
%      stem(n(1:22)/Fs,x(1:22),'filled')    % Plot original sampled at 22.05kHz
%      hold on                              % Plot interpolated signal (44.1kHz) in red
%      stem(n(1:44)/(Fs*L),y(2:45),'r')
%      xlabel('Time (sec)');ylabel('Signal value')
%
%   See also MFILT/STRUCTURES.


%mfilt.linearinterp class
%   mfilt.linearinterp extends mfilt.abstractfirinterp.
%
%    mfilt.linearinterp properties:
%       PersistentMemory - Property is of type 'bool'
%       NumSamplesProcessed - capture (read only)
%       FilterStructure - Property is of type 'ustring'  (read only)
%       States - Property is of type 'mxArray'
%       Numerator - Property is of type 'DFILTNonemptyVector user-defined'
%       InterpolationFactor - Property is of type 'posint user-defined'
%       Arithmetic - Property is of type 'filterdesign_arith enumeration: {'double','single','fixed'}'
%
%    mfilt.linearinterp methods:
%       blocklib - Returns the library and source block
%       blockparams - Returns the parameters for BLOCK
%       coefficient_info -   Get the coefficient information for this filter.
%       createhdlfilter - Returns the corresponding hdlfiltercomp object
%       dgdfgen - generate the dg_dfilt structure from a specified filter structure
%       thisdisp - Object display.
%       filt2struct -   Returns a structure representation of the filter.
%       genmcode -   Generate an MATLAB code object for this object.
%       getbestprecision - Return best precision for Product and Accumulator
%       getstates - Overloaded get for the States property.
%       isblockable - True if the object supports the block method
%       ishdlable - True if HDL can be generated for the filter object.
%       loadmetadata -   Load the meta data.
%       loadreferencecoefficients -   Load the reference coefficients.
%       naddp1 -   Returns the number of additions + 1.
%       secfilter - Polyphase FIR Interpolator filter
%       set2int - Scales the coefficients to integer numbers.
%       set_tapindex -   PreSet function for the 'tapindex' property.
%       setfilterstructure -   Set the filterstructure.
%       setinterp - Overloaded set for the InterpolationFactor property.
%       setnum - Overloaded set for the Numerator property.
%       thisfiltquant_plugins - FILTQUANT_PLUGINS Table of filterquantizer plugins
%       thisismaxphase -   True if the object is maxphase.
%       thisisminphase -   True if the object is minphase.
%       thisnstates - NSTATES  Number of states in discrete-time filter.
%       thissetstates - Overloaded set for the States property.
%       tolinearinterp -   Convert to a linear interpolator.


properties (SetObservable, GetObservable)
    %ARITHMETIC Property is of type 'filterdesign_arith enumeration: {'double','single','fixed'}'
    Arithmetic = 'double';
end


methods  % constructor block
    function Hm = linearinterp(L)
        narginchk(0,1);
        Hm.privfq = dfilt.filterquantizer;
        Hm.privfilterquantizer = dfilt.filterquantizer;
        Hm.FilterStructure = 'Direct-Form FIR Polyphase Linear Interpolator';
        Hm.Arithmetic = 'double';
        Hm.InterpolationFactor = 2;
        if nargin>0
            Hm.InterpolationFactor = L;
        end
    end  % linearinterp
end  % constructor block

methods
    function value = get.Arithmetic(obj)
        value = get_arith(obj,obj.Arithmetic);
    end
    function set.Arithmetic(obj,value)
        % Enumerated DataType = 'filterdesign_arith enumeration: {'double','single','fixed'}'
        value = validatestring(value,{'double','single','fixed'},'','Arithmetic');
        obj.Arithmetic = set_arith(obj,value);
    end
end   % set and get functions

methods  % public methods
    [lib,srcblk,hasInputProcessing,hasRateOptions] = blocklib(~,~)
    s = blockparams(Hd,mapstates,varargin)
    [p,v] = coefficient_info(this)
    hF = createhdlfilter(this)
    DGDF = dgdfgen(Hd,hTar,doMapCoeffsToPorts)
    thisdisp(this)
    s = filt2struct(this)
    hcode = genmcode(this,varname,place)
    s = getbestprecision(h)
    S = getstates(Hm,S)
    b = isblockable(~)
    [result,errstr,errorObj] = ishdlable(filterobj)
    loadmetadata(this,s)
    loadreferencecoefficients(this,s)
    n = naddp1(h)
    [y,z] = secfilter(Hm,x,z)
    varargout = set2int(this,coeffWL,inWL)
    tapindex = set_tapindex(H,tapindex)
    setfilterstructure(H)
    L = setinterp(Hm,L)
    num = setnum(Hm,num)
    constr = thisfiltquant_plugins(h,arith)
    b = thisismaxphase(h)
    b = thisisminphase(h,tol)
    n = thisnstates(Hm)
    S = thissetstates(Hd,S)
    Hm = tolinearinterp(this)
end  % public methods

methods (Hidden) % possibly private or hidden
    [targs,strs] = convertchoices(this)
    loadr14(H,s)
    specifyall(this,flag)
    thisreset(H)
end  % possibly private or hidden

end  % classdef

