function varargout = ifir(this, varargin)
%IFIR   Design an interpolated FIR filter.

%   Copyright 2005-2022 The MathWorks, Inc.
if nargin > 1
    [varargin{:}] = convertStringsToChars(varargin{:});
end


% Store current design flags state
df = getcurrentspecs(this).getDesignFlags();
restoredf = onCleanup(@() getcurrentspecs(this).setDesignFlags(df));

varargin = parseDesignFlags(this,varargin);
sysObjFlag = getcurrentspecs(this).getDesignFlags.SystemObject;

% This function operates on mfilt object. If System object is requested, 
% first design mfilt then defer convertion to System object (if possible at all)
sysObjIdx = find(cellfun(@(x)strcmpi('SystemObject',x),varargin),1);
if ~isempty(sysObjIdx)
    if varargin{sysObjIdx+1} == 1
        varargin(sysObjIdx:sysObjIdx+1) = []; % remove the property so that result is mfilt at first
    end
end

% Suppress MFILT deprecation warning on calling ifir
w = warning('off', 'dsp:mfilt:mfilt:Obsolete');
restoreWarn = onCleanup(@() warning(w));

% Make sure that IFIR is valid for this set of specifications.
if ~isdesignmethod(this, 'ifir')
    error(message('dsp:fdesign:interpolator:ifir:invalidMethod', 'IFIR', this.SpecificationTypeListeners));
end

% Parse the inputs for the multirate filter structure.
[filtstruct, varargin] = parsestruct(this, 'firinterp', 'ifir', varargin{:});

% Map to internal object if applicable
AllowMfilt = getcurrentspecs(this).getDesignFlags.allowDFilts();
filtstruct = dsp.internal.mfilt.mapInternalmfilt(['mfilt.', filtstruct], AllowMfilt);

ifactor = this.InterpolationFactor;

[Hd, upfactor] = ifir(this.CurrentFDesign, varargin{:}, 'rcf', ifactor);

fm = getfmethod(Hd);

if isa(Hd, 'dfilt.cascade')
    % Integrate noble identities in Cascade
    [Hm,final_upfactor] = cascadenoble(Hd,upfactor,ifactor,filtstruct);
    if sysObjFlag
        Hm = sysobj(Hm);  % return dsp.FilterCascade
    end
elseif isa(Hd, 'dfilt.parallel')
    if sysObjFlag
        % Error out because parallel structure is not supported by System objects
        error(message('dsp:fdesign:decimator:ifir:ParallelNotSupportedBySystemObjects'));
    end
    % Find cascade branch within parallel
    if isa(Hd.Stage(1),'dfilt.cascade')
        s1 = 1;
        s2 = 2;
    else
        s1 = 2;
        s2 = 1;
    end
    [Hm1,final_upfactor] = cascadenoble(Hd.Stage(s1),upfactor,ifactor,filtstruct);
    Hvec = interpdelaynoble(this,Hd.Stage(s2),ifactor);
    Hvec = interpmergedelays(this,Hvec);
    Hm2 = cascade(Hvec);
    Hm = parallel(Hm1,Hm2);
else

    % If we are returned a single section, the design was simple enough
    % that we can just use a single interpolator.
    Hm = feval(filtstruct, ifactor, ifactor*Hd.Numerator);
    final_upfactor = 1;
    
    if sysObjFlag
        Hm = sysobj(Hm);  % return dsp.FilterCascade
    end
end

if isa(Hm, 'dsp.internal.FilterAnalysis') % if returning a System object
    setMetaData(Hm,this,fm);
else
    Hm.setfdesign(this);
    Hm.setfmethod(fm);
end

if nargout
    varargout = {Hm, final_upfactor};
else
    if this.NormalizedFrequency
        inputs = {'NormalizedFrequency', 'On'};
    else
        inputs = {'Fs', this.Fs};
    end

    fvtool(Hm, inputs{:});
end
%--------------------------------------------------------------------------
function [Hm,final_upfactor] = cascadenoble(Hd,upfactor,ifactor,filtstruct)

%  /\ LK -- H1(z) -- H2(z^L)
%
%  /\ LK -- H2(z^L) -- H1(z)
%
%  /\ K -- H2(z) -- /\L -- H1(z)
%
%  H2_interp_K(z) -- H1_interp_L(z)
%
%  If Upfactor is greater than InterpolationFactor
%
%  /\ L -- H1(z) -- H2(z^LK)
%
%  /\ L -- H2(z^LK) -- H1(z)
%
%  H2(z^K) -- /\L -- H1(z)
%
%  H2(z^K) -- H1_interp_L(z)
%
%  K is often 1, in which case we don't need a multirate.

% Divide the interpolation factor into two parts.
L = gcd(upfactor, ifactor);
K = ifactor/L;  % Usually 1
final_upfactor = upfactor/L;

% Remove the extra ones from the numerator.  We will get these
% automatically when we use the interpolating structure.
b_h2 = Hd.Stage(2).Numerator(1:L:end);

% If there is "left over" interpolation put it into the first stage.
if K == 1
    Hm1 = Hd.Stage(2);
    Hm1.Numerator = b_h2;
else
    Hm1 = feval(filtstruct, K, K*b_h2);
end

% Create the 2nd stage.
if L == 1
    Hm2 = Hd.Stage(1);
else
    Hm2 = feval(filtstruct, L, L*Hd.Stage(1).Numerator);
end

Hm = mfilt.cascade(Hm1, Hm2); %#ok<MCASCADE>


% [EOF]
