classdef equirippleNstage < filterdesign.internal.nstageDesigner
%

%   Copyright 2018-2019 The MathWorks, Inc.

    properties
        Fpass = 1e3;    
        Algorithm = 'firceqrip'
    end
    
    methods
        function set.Fpass(obj,Fp)
            validateattributes(Fp,{'numeric'},{'scalar','real',...
                'finite','positive'},...
                'set.Fpass','Fpass',2);
            obj.Fpass = Fp;
        end
        
        function set.Algorithm(obj,alg)
            algstr = validatestring(alg,{'firceqrip','firgr'},'set.Algorithm','Algorithm',2);
            obj.Algorithm = algstr;
        end
    end
    
    methods (Access = protected)
        function c = cost(obj,M1,M2,TW,Ap)
            % M1 - Decimation from first stage up to this stage
            % M2 - Decimation for this stage
            % TW - Transition width
            % Ap - Passband ripple
                        
            FsStage = obj.SampleRate/M1;
            switch obj.CostMeasure
                case 'design'
                    c.Filter = designStage(obj,M2,TW,Ap,FsStage);
                    N = nnz(c.Filter.Numerator);
                case 'estimate'
                    Ast = obj.Astop;
                    Fp  = obj.Fpass;
                    Fst = Fp + TW;                    
                    Rpass = convertmagunits(Ap,'db','linear','pass');
                    Rstop = convertmagunits(Ast,'db','linear','stop');
                    
                    N = firpmord([Fp Fst],[1, 0],[Rpass Rstop],FsStage);
                    if N < 3
                        % firpmord can return 0
                        N = 3;
                    end                    
            end
            % Cost including polyphase decimation
            c.Cost    = N/(M1*M2);
            c.NCoeffs = N;
            c.Apass   = Ap;
            c.FsStage = FsStage;
        end
        
        function c = stagecost(obj,t,Mvec)
            
            M  = obj.RateChangeFactor;
            
            % Compute decimation up to this stage
            M1 = M/prod(Mvec);
            
            % Compute decimation for this stage
            r = Mvec(1);                                                  

            % Determine transition width for this stage
            idx = find(all(t.Path(:,end-length(Mvec)+1:end)==Mvec,2),1,'first');
            TW = t.Fstop(idx,end-length(Mvec)+2) - obj.Fpass;
                        
            % Determine passband ripple for this stage
            Ap = t.Apass(idx,end-length(Mvec)+1); 
            
            % Determine the cost for this stage
            c = cost(obj,M1,r,TW,Ap);
            c.TW = TW;
        end
        
        function c = dynProgStageCost(obj,t,k,j,r)
            
            M  = obj.RateChangeFactor;
            
            % Compute decimation up to this stage
            M1 = M/t.Node(k);

            % Determine transition width for this stage
            TW = t.FstopReplica(j) - obj.Fpass;
            
            % Determine passband ripple for this stage. Divide by 2 because
            % firgr expects peak values rather than peak-to-peak
            if t.Node(k) == M
                Ap = t.Apass(j);
            else
                Ap = t.Apass(j)/2;
            end
            
            c = cost(obj,M1,r,TW,Ap);
            c.TW = TW;
        end
                        
        function setStageFstop(obj,N)
            
            t = obj.costTable;
            Fs = obj.SampleRate;
            M  = obj.RateChangeFactor;
            TW = obj.TransitionWidth;
            Fp = obj.Fpass;
            Ap = obj.Apass;
            
            for k = 1:size(t,1)               
                t.TW(k,end) = TW;
            end
            
            for k = 1:size(t,1)
                for m = 1:N
                    M2 = prod(t.Path(k,m:end));
                    M1 = M/M2;
                    FsStage = Fs/M1;
                    t.Fstop(k,m) = FsStage-(Fp+TW);
                    if m == 1
                        % First stage has same ripple as second stage
                        t.Apass(k,m) = Ap/2^(N-1);
                    else
                        t.Apass(k,m) = Ap/2^(N-m+1);
                    end
                end
            end
            
            obj.costTable = t;
        end        
        
        function setLastStageFstop(obj)
            
            t = obj.costTable;
            Fs = obj.SampleRate;
            M  = obj.RateChangeFactor;
            TW = obj.TransitionWidth;
            Fp = obj.Fpass;
            
            for k = 1:size(t,1)
                M2 = t.Node(k);
                M1 = M/M2;
                FsStage = Fs/M1;
                t.FstopReplica(k) = FsStage-(Fp+TW);
            end
            
            obj.costTable = t;
        end
        
        function setMetaData(obj,c)
           fd = fdesign.decimator(obj.RateChangeFactor,'Lowpass',...
               obj.Fpass, obj.Fpass + obj.TransitionWidth,...
               obj.Apass,obj.Astop,...
               obj.SampleRate);
           
           fm = fdfmethod.eqriplpmin;
           
           m = measure(c,fd);
           
           setMetaData(c,fd,fm,m);
        end
        
        function b = designStage(obj,M2,TW,Ap,FsStage)
            
            Ast = obj.Astop;
            Fp  = obj.Fpass;
            Fst = Fp + TW;            
            
            Rpass = convertmagunits(Ap,'db','linear','pass');            
            Rstop = convertmagunits(Ast,'db','linear','stop');                        
            Fnyq = FsStage/2;
            
            switch obj.Algorithm
                case 'firgr'
                    h = firgr('minorder',[0 Fp/Fnyq, Fst/Fnyq 1],[1 1 0 0],...
                        [Rpass Rstop]);
                case 'firceqrip'
                    h = firceqrip('minorder',[Fp/Fnyq, Fst/Fnyq],[Rpass Rstop]);
            end
            
            if strcmpi(obj.DecimatorStructure,'firdecim')
                filtstruct = 'Direct form';
            else
                filtstruct = 'Direct form transposed';
            end
            b = dsp.FIRDecimator(M2,h,'Structure',filtstruct);                          
        end
    end
end