function [Fpass, F3dB, F6dB, Fstop, Apass, Astop] = lowpassmeasure(B, tw, maxLM, varargin)
% LOWPASSMEASURE Measure the following for a lowpass filter
%   - Fpass: Frequency at the start of the passband.
%   - F3dB:  Cutoff frequency for the 3 dB point.
%   - F6dB:  Cutoff frequency for the 6 dB point.
%   - Fstop: Frequency at the end of the stopband.
%   - Apass: Amount of ripple in the passband.
%   - Astop: Attenuation in the stopband.

% Copyright 2019-2022 The MathWorks, Inc.

%#codegen
narginchk(3,4);

% Provide optional argument as "Stopband" if only Astop and Fstop are
% required and other outputs are set to ~

Fstop = measurefstop(tw,maxLM);
Astop = measuremattenuation(B, Fstop, 1);

if nargin==4
    validatestring(varargin{1},"Stopband");
    % Only compute required measurements and set others to nan.
    Fpass = nan;
    F3dB = nan;
    F6dB = nan;
    Apass = nan;   
else
    Fpass = measurefpass(tw,maxLM);
    F3dB  = findfrequency(B,1/sqrt(2), false, true);
    F6dB  = findfrequency(B,1/2, false, true);
    Apass = measureripple(B, 0, Fpass);
end
end
%--------------------------------------------------------------------------
% Measure the start of the passband
function fpass = measurefpass(tw,band)
Fs = 2;
fpass = Fs/2/band-tw/2;
end
%--------------------------------------------------------------------------
% Measure the end of the stopband
function Fstop = measurefstop(tw,band)
Fs = 2;
Fstop = Fs/2/band+tw/2;
end
%--------------------------------------------------------------------------
% Measure the ripple in the passband
function rip = measureripple(num, Fstart, Fend)
Fs = 2;
N = 2^10;
[h, ~] = filterdesign.internal.freqz(num, 1, linspace(Fstart, Fend, N), N,Fs);
h      = abs(h);
% The ripple is defined as the amplitude (dB) variation between the two
% specified frequency points.
rip = 20*log10(max(h)) - 20*log10(min(h));
end
%--------------------------------------------------------------------------
% Measure the attenuation in the stopband
function atten = measuremattenuation(num, Fstart, Fend)
N = 2^12;
Fs = 2;
h = abs(filterdesign.internal.freqz(num, 1, linspace(Fstart, Fend, N), N, Fs));
ngain = 1;
atten = 20*log10(ngain) - 20*log10(max(h));
end
%--------------------------------------------------------------------------
function F = findfrequency(num, A, up, first)
% Find the 3dB or the 6dB frequency values
Fs = 2;
ngain = 1;
A = ngain*A;
N = 2^10;
h = complex(zeros(1,N),zeros(1,N));
[h(1,:),w] = filterdesign.internal.freqz(num, 1, zeros(1,N), N, Fs);
h_mag = abs(h);

[w_lo, w_hi] = lclfindfrequency(w, h_mag, A, up, first);

[F, htest] = getF(num, [w_lo w_hi], Fs);
count = 1;

while abs(A-htest) > 1e-5 && count < 4
    % Do a second refinement if we did not hit the point exactly.
    [h(1,:), w] = filterdesign.internal.freqz(num, 1, linspace(w_lo, w_hi, N), N, Fs);
    h_mag = abs(h);
    [w_lo, w_hi] = lclfindfrequency(w, h_mag, A, up, first);
    [F, htest] = getF(num, [w_lo w_hi], Fs);
    count = count+1;
end
end
%--------------------------------------------------------------------------
function [w_lo, w_hi] = lclfindfrequency(w, h, A, ~, first)
w_lo = zeros(1,1);
w_hi = zeros(1,1);
if first
    indx = find(h <= A, 1, 'first');
    if isempty(indx)
        indx = length(w);
    end
    w_lo(1) = w(max(1, indx-1));
    w_hi(1) = w(indx);
end
end
%--------------------------------------------------------------------------
function [F, htest] = getF(num, w, Fs)
F = mean(w);
[htest,~] = filterdesign.internal.freqz(num, 1, [0 F], 2,Fs);
htest = abs(htest(2));
end
