classdef minordIterator < handle
    %minordIterator - class used to find minimum order FIR filter that
    %                 meets specs
    
    %   Copyright 2018-2019 The MathWorks, Inc.
    
    properties
        current
        increment = 1
        direction = 1
        constraint = 'minorder' % 'minorder', 'mineven', 'minodd'
        
        lowNfilt  = -1
        highNfilt = -1
        
        validFilt
        
        designFCN
        testFCN
        
        designArgs
        testArgs
        
        isError = false
        
        MaxIter = 20;
    end
    
    methods
        function obj = minordIterator(constraint,initOrd)
            
            if nargin < 1
                constraint = 'minorder';
            end
            
            if nargin < 2
                initOrd = 100;
            end
            
            switch constraint
                case 'mineven'
                    if rem(initOrd,2)
                        initOrd = initOrd + 1;
                    end
                case 'minodd'
                    if ~rem(initOrd,2)
                        initOrd = initOrd + 1;
                    end
            end
            
            obj.constraint = constraint;
            obj.current    = initOrd;
            
            obj.increment = ceil(initOrd*.1);
        end
        
        function set.increment(obj,increment)
            obj.increment = increment;
            if ~isequal(obj.constraint,'minorder') && rem(obj.increment,2) %#ok<MCSUP>
                % Force even increment
                obj.increment = obj.increment + 1;
            end
        end
        
        function set.constraint(obj,constr)
            constr = validatestring(constr,{'minorder','mineven','minodd'});
            obj.constraint = constr;
        end
        
        function [filt,WARN] = findMinOrd(obj,varargin)
            WARN = init(obj,varargin{:});
            if ~WARN
               WARN = bisect(obj,varargin{:});
            end
            filt = obj.validFilt;            
        end
    end
    
    methods (Access = private)
        
        function WARN = init(obj,varargin)
            % Bracket the minimum order
            isBracketed = false;
            MaxIt = obj.MaxIter;
            count = 1;
            WARN = false;
            while ~isBracketed
                try
                    [filt,failed] = obj.designFCN(obj.current,obj.designArgs{:});
                    if failed
                        pass = false;
                    else
                        pass = obj.testFCN(filt,obj.testArgs{:});
                    end
                catch
                    failed = true;
                    pass = false;
                end
                
                if pass
                    % Found an upper bound
                    obj.direction = -1;
                    obj.highNfilt = obj.current;
                    obj.validFilt = filt;
                    if obj.highNfilt == 3
                        % Done
                        obj.lowNfilt = 2;
                    end
                else
                    if count == MaxIt
                        %Unable to brake after MaxIt iterations.
                        WARN = true;
                        break
                    end
                    % Need to increase order
                    if ~failed
                        % Don't change direction if failed to converge
                        obj.direction = 1;
                        obj.lowNfilt = obj.current;
                    end
                end
                
                count = count + 1;
                
                if (obj.highNfilt > 0) && (obj.lowNfilt > 0)
                    isBracketed = true;
                    
                    obj.increment = ceil(abs(obj.highNfilt - obj.lowNfilt)/2);
                end
                
                obj.current = max(obj.current + ...
                    obj.direction*obj.increment,3); % Don't go below order 3
                if isequal(obj.constraint,'mineven') && (obj.current==3)
                    obj.current = 4;
                end
            end
        end
        
        function WARN = bisect(obj,varargin)
            
            isDone = false;
            if obj.highNfilt == obj.lowNfilt + 1
                % We found min order in init, end early
                isDone = true;
            end
            
            MaxIt = obj.MaxIter;
            count = 1;
            WARN = false;
            while ~isDone
                try
                    [filt,failed] = obj.designFCN(obj.current,obj.designArgs{:});
                    pass = obj.testFCN(filt,obj.testArgs{:});
                catch
                    pass = false;
                end
                
                if pass
                    % Found an upper bound
                    obj.direction = -1;
                    obj.highNfilt = obj.current;
                    obj.validFilt = filt;
                    WARN = failed;
                else
                    if count == MaxIt
                        % Unable to brake after MaxIt iterations.
                        WARN = true;
                        break;
                    end
                    
                    % Need to increase order
                    obj.direction = 1;
                    obj.lowNfilt = obj.current;
                end
                
                count = count + 1;
                
                obj.increment = ceil(abs(obj.highNfilt - obj.lowNfilt)/2);
                inc = obj.direction*obj.increment;
                new = obj.current + inc;
                if obj.direction > 0
                    if new >= obj.highNfilt
                        obj.current = obj.highNfilt;
                        isDone = true;
                    else
                        obj.current = new;
                    end
                else
                    if new <= obj.lowNfilt
                        isDone = true;
                    else
                        obj.current = max(new,3);
                    end
                end
            end
        end
    end
end


