classdef (Abstract = true) nstageDesigner < handle
%

%   Copyright 2018-2021 The MathWorks, Inc.

properties    
    SampleRate           = 48000
    TransitionWidth      = 1e3;  
    Apass                = 1e-1 % Ignored for Nyquist designs
    Astop                = 80
    RateChangeFactor     = 8          
    CostTol              = 1e-6;
    CostTiebreaker       = 'NumCoeffs';
    MinTotalCoeffs       = false;
    HalfbandDesignMethod = 'equiripple';        
    DecimatorStructure   = 'firdecim';
    MultirateType        = 'decimator';
    CostMeasure          = 'estimate';
end

properties (Access = protected)        
    mutistageOptions 
    costTable
end

methods (Abstract, Access = protected)
    c = cost(obj)
    setStageFstop(obj)
    setLastStageFstop(obj)
    setMetaData(obj)
    c = stagecost(obj)
    c = dynProgStageCost(obj)
    b = designStage(obj);
end
        
methods          
    function c = dynprogDesign(obj)
        
        initializeDynProgTable(obj);
        setLastStageFstop(obj);     
        minimizeCostUsingDynProg(obj);
        c = buildDynProgCascade(obj);
        setMetaData(obj,c);
    end
    
    function c = design(obj,NumStages)
        
        if nargin < 2
            NumStages = 1;
        end

        MaxStages = length(factor(obj.RateChangeFactor));            
        
        validateattributes(NumStages,{'numeric'},{'scalar','real',...
            'finite','positive','integer','<=',MaxStages},...
            'design','NumStages');
        
        % Design for a specific number of stages
        if isequal(obj.CostMeasure,'estimate')
            % Try dynamic programming
            cdyn = dynprogDesign(obj);
            N = getNumStages(cdyn);
            if N == NumStages
                % Return design from dynamic programming, it may be
                % different from c = designNumStages(obj,NumStages).
                % Since it is based on an estimate either way, it is
                % best to match dynamic programming answer
                c = cdyn;
            else
                c = designNumStages(obj,NumStages);
            end
        else
            % This design should always match the design using dynamic
            % programming if the number of stages coincides with the
            % resulting number of stages from the dynamic programming
            % design
            c = designNumStages(obj,NumStages);
        end
        
        setMetaData(obj,c);
    end
    
    function tout = designAll(obj,NumStages)
        % Design all possible combinations for a given number of stages            
        
        if nargin < 2
            NumStages = 1;
        end
        
        MaxStages = length(factor(obj.RateChangeFactor));
        validateattributes(NumStages,{'numeric'},{'scalar','real',...
                'finite','positive','integer','<=',MaxStages},...
                'design','NumStages',2);

        initializeCostTable(obj,NumStages);
        setStageFstop(obj,NumStages);
        minimizeCost(obj);
        t = obj.costTable;
        
        tout = t(:,1:3);
        
        Nfilts = size(t,1);
        c = cell(Nfilts,1);
        TotalCost   = zeros(Nfilts,1);
        TotalCoeffs = zeros(Nfilts,1);            
                                
        for k = 1:Nfilts
            if isequal(obj.CostMeasure,'estimate')
                % Need to design the filters
                h = cell(1,NumStages);
                for m = 1:NumStages
                    M  = t.Path(k,m);
                    TW = t.TW(k,m);
                    Fs = t.FsStage(k,m);
                    Ap = t.Apass(k,m);
                    b  = designStage(obj,M,TW,Ap,Fs);                        
                    h{m} = b;
                end
            else
                % Filters are designed already
                h = t.Filter(k,:);
            end
            
            if strcmpi(obj.MultirateType,'interpolator')
                % Convert to interpolation cascade. Reverse order of
                % filters
                [c{k},tt] = convertToInterpolatorCascade(h);
                

                tout.Cost(k,:) = tt.Cost;
                tout.NCoeffs(k,:) = tt.NCoeffs;
                
                % Path needs to be reversed
                tout.Path(k,:) = fliplr(tout.Path(k,:));
            else
                % Decimator
                c{k} = dsp.FilterCascade(h{:},CloneStages=false);
                
            end
            
            TotalCost(k)   = sum(tout.Cost(k,:));
            TotalCoeffs(k) = sum(tout.NCoeffs(k,:));
        end
        
        tout.TotalCost   = TotalCost;
        tout.TotalCoeffs = TotalCoeffs;
        tout.Filter      = c;
                    
        tout = tout(:,[1,2,4,3,5,6]);            
    end                   
    
    function set.SampleRate(obj,Fs)
        validateattributes(Fs,{'numeric'},{'scalar','real',...
            'finite','positive'},...
            'set.SampleRate','Fs');
        obj.SampleRate = Fs;
    end
    
    function set.TransitionWidth(obj,TW)
        validateattributes(TW,{'numeric'},{'scalar','real',...
            'finite','positive'},...
            'set.TransitionWidth','TW');
        obj.TransitionWidth = TW;
    end
    
    function set.Apass(obj,Ap)
        validateattributes(Ap,{'numeric'},{'scalar','real',...
            'finite','positive'},...
            'set.Apass','Apass');
        obj.Apass = Ap;
    end
    
    function set.Astop(obj,Ast)
        validateattributes(Ast,{'numeric'},{'scalar','real',...
            'finite','positive'},...
            'set.Astop','Astop');
        obj.Astop = Ast;
    end
    
    function set.CostTol(obj,Tol)
        validateattributes(Tol,{'numeric'},{'scalar','real',...
            'finite','nonnegative'},...
            'set.CostTol','CostTol');
        obj.CostTol = Tol;
    end
    
    function set.RateChangeFactor(obj,M)
        validateattributes(M,{'numeric'},{'scalar','real',...
            'finite','positive','integer','>',1},...
            'set.RateChangeFactor','RateChangeFactor');
        obj.RateChangeFactor = M;
    end
    
    function set.HalfbandDesignMethod(obj,st)
        s = validatestring(st,{'equiripple','kaiserwin','butter','ellip',...
            'iirlinphase'},'set.HalfbandDesignMethod','DesignMethod');            
        obj.HalfbandDesignMethod = s;
    end
    
    function set.CostTiebreaker(obj,st)
        s = validatestring(st,{'NumStages','NumCOeffs'},...
            'set.CostTiebreaker','CostTiebreaker');            
        obj.CostTiebreaker = s;
    end
    
    function set.DecimatorStructure(obj,st)
        s = validatestring(st,{'firdecim'  'firtdecim'},...
            'set.DecimatorStructure','Structure');            
        obj.DecimatorStructure = s;
    end
    
    function set.MultirateType(obj,st)
        s = validatestring(st,{'decimator'  'interpolator'},...
            'set.MultirateType','MultirateType');            
        obj.MultirateType = s;
    end
    
    function set.CostMeasure(obj,st)
        s = validatestring(st,{'estimate','design'},...
            'set.CostMeasure','CostMeasure');            
        obj.CostMeasure = s;
    end
end

methods (Access=private)
    function c = designNumStages(obj,NumStages)   
        
        tout = designAll(obj,NumStages);
        
        c = pickOptimal(obj,tout);
                                
        setMetaData(obj,c);
    end
    
    function c = pickOptimal(obj,t)
                             
        
        if obj.MinTotalCoeffs
            % Focus on total number of coefficients. If tie, choose the
            % one with minimum cost
            m = min(t.TotalCoeffs);
            
            % Find all cases with min coeffs
            k = find(m == t.TotalCoeffs);
            
            % Determine the one with minimum cost
            mc = min(t.TotalCost(k));
            
            % Find all cases with cost within tol
            kc = find(abs(mc-t.TotalCost(k)) <= obj.CostTol);
            
            % If more than one option, pick the one with fewest
            % coefficients for its first stage
            [~,kopt] = min(t.NCoeffs(k(kc),1));
            
            idx = k(kc(kopt));
            
        else
            % Focus on multiplications per input sample. If tie, choose
            % the one with fewest coefficients
            mc = min(t.TotalCost);
            
            % Find all cases with cost within tol
            kc = find(abs(mc-t.TotalCost) <= obj.CostTol);
            
            % Determine the one with minimum coefficients
            m = min(t.TotalCoeffs(kc));
            
            % Find all cases with min coeffs
            k = find(m == t.TotalCoeffs(kc));
            
            % If more than one option, pick the one with fewest
            % coefficients for its first stage
            [~,kopt] = min(t.NCoeffs(kc(k),1));
            
            idx = kc(k(kopt));
        end
        
        c = t.Filter{idx,:};            
    end
    
     function initializeDynProgTable(obj)            
        M  = obj.RateChangeFactor;
        TW = obj.TransitionWidth;
        Ap = obj.Apass;
        
        % Determine all factors of M
        Mvec = unique([cumprod(factor(M)),cumprod(fliplr(factor(M)))],'sorted');                        
        
        % Initialize
        t = table(Mvec', [Mvec' ones(size(Mvec'))],zeros(size(Mvec')),zeros(size(Mvec')),...
            ones(size(Mvec')),zeros(size(Mvec')),TW*ones(size(Mvec')),zeros(length(Mvec),1),...
            zeros(length(Mvec),1),cell(size(Mvec')),...
            'VariableNames',{'Node','Path','Cost','NumCoeffs','NumStages',...
            'Fstop','TW','Apass','FsStage','Filter'});
        
        % Initialize with last stage costs            
        for k = 1:size(t,1)               
            if k ~= size(t,1)
                Ap1 = Ap/2;
            else
                Ap1 = Ap;
            end
           
            c = cost(obj,M/Mvec(k),Mvec(k),TW,Ap1);
            
            t.Cost(k)      = c.Cost;
            t.NumCoeffs(k) = c.NCoeffs;
            if isfield(c,'Filter')
                t.Filter(k)    = {c.Filter};
            end
            t.Apass(k) = c.Apass;                
            t.FsStage(k)  = c.FsStage;
        end
        
        obj.costTable = t;            
     end
    
    function initializeCostTable(obj,N)
        M  = obj.RateChangeFactor;
        TW = obj.TransitionWidth;            
        
        % Find all ways of factoring M in N factors
        f = factorin(M,N);
        obj.mutistageOptions = f;
        
        [Mf,Nf] = size(f);
        
        % Initialize
        t = table(f,zeros(Mf,Nf),zeros(Mf,Nf),zeros(Mf,Nf),zeros(Mf,Nf),...
            zeros(Mf,Nf),zeros(Mf,Nf),cell(Mf,Nf),...
            'VariableNames',{'Path','Cost','NCoeffs','Fstop','TW','Apass','FsStage','Filter'});
        
        % Initialize with last stage costs
        Mvec = unique(f(:,Nf));
        if Nf == 1
            % Single stage
            Ap = obj.Apass; 
        else
            Ap = obj.Apass/2;
        end
            
        for k = 1:length(Mvec)                
            c                 = cost(obj,M/Mvec(k),Mvec(k),TW,Ap);                  
            idx               = find(t.Path(:,end)==Mvec(k));
            t.Cost(idx,Nf)    = c.Cost; 
            t.NCoeffs(idx,Nf) = c.NCoeffs; 
            if isfield(c,'Filter')
                t.Filter(idx,Nf)  = {c.Filter};
            end
            t.FsStage(idx,Nf) = c.FsStage;                
        end
        
        obj.costTable = t;
    end
    
    function minimizeCost(obj)
        
        f = obj.mutistageOptions;
        t = obj.costTable;
        
        [~,Nf] = size(f);
        
        for n = Nf-1:-1:1
            % Design only once for all repeated cases
            Mmat = unique(f(:,n:Nf),'row');
            for k = 1:size(Mmat,1)
                c                = stagecost(obj,t,Mmat(k,:));
                idx              = find(all(t.Path(:,n:Nf)==Mmat(k,:),2));
                
                t.Cost(idx,n)    = c.Cost; 
                t.NCoeffs(idx,n) = c.NCoeffs; 
                t.TW(idx,n)      = c.TW;
                if isfield(c,'Filter')
                    t.Filter(idx,n)  = {c.Filter};
                end
                t.FsStage(idx,n) = c.FsStage;                    
            end
        end
        
        obj.costTable = t;
    end
    
    function minimizeCostUsingDynProg(obj)
        t = obj.costTable;
        
        for k = find(t.Node>=4,1,'first'):size(t,1)
            % Find all possible paths
            if ~isprime(t.Node(k))
                jopt = 1;
                j = k-1;
                while j >= 1
                    r = t.Node(k)/t.Node(j);
                    if abs(r - round(r)) < sqrt(eps(1))                            
                        c = dynProgStageCost(obj,t,k,j,r);                            
                        minCost   = t.Cost(k);
                        minCoeffs = t.NumCoeffs(k);
                        mc = t.Cost(j) + c.Cost;
                        nc = t.NumCoeffs(j) + c.NCoeffs;
                        if obj.MinTotalCoeffs
                            if (nc < minCoeffs)
                                % Less coefficients, replace
                                t = replaceFilter(t,c,mc,nc,j,k);                                                                        
                                jopt = j;
                            elseif nc == minCoeffs
                                % Same number of coefficients. Decide
                                % by multiplications per input sample 
                                if (mc < minCost) && (norm(minCost-mc,inf) > obj.CostTol)
                                    % Clearly better, replace
                                    t = replaceFilter(t,c,mc,nc,j,k);
                                    jopt = j;
                                end
                            end
                        else
                            % Minimize Multiplications Per Input Sample
                            if (mc < minCost) && (norm(minCost-mc,inf) > obj.CostTol)
                                % Clearly better, replace
                                t = replaceFilter(t,c,mc,nc,j,k);
                                jopt = j;
                            elseif norm(minCost-mc,inf) <= obj.CostTol
                                % Cost is within tolerance. Use tiebraker
                                if isequal(obj.CostTiebreaker,'NumStages')
                                    % Do nothing. Keep one with fewer stages
                                    
                                else
                                    % Keep one with fewer coefficients
                                    if nc < minCoeffs
                                        t = replaceFilter(t,c,mc,nc,j,k);
                                        jopt = j;
                                    end
                                end
                            end
                        end
                    end
                    j = j - 1;
                end
                t.NumStages(k) = t.NumStages(k) + t.NumStages(jopt);
            end
        end
        
        obj.costTable = t;
    end
    
    function c = buildDynProgCascade(obj)
        t = obj.costTable;
        
        isDone = false;
        n = 0;
        currentNodeIdx = size(t,1);
        h = cell(1,length(factor(obj.RateChangeFactor)));
        
        while ~isDone
            n = n + 1;
            nextNodeIdx = find(t.Node == t.Path(currentNodeIdx,2));
            if t.Path(currentNodeIdx,2) == 1
                isDone = true;
            end
            if isequal(obj.CostMeasure,'estimate')
                % Need to design the filters
                M  = t.Path(currentNodeIdx,1)/t.Path(currentNodeIdx,2);
                TW = t.TW(currentNodeIdx);
                Fs = t.FsStage(currentNodeIdx);
                Ap = t.Apass(currentNodeIdx);
                b  = designStage(obj,M,TW,Ap,Fs);                    
                h{n} = b;
            else
                % Filters are designed already
                b = t.Filter(currentNodeIdx);
                h{n} = b{1};
            end
           
            currentNodeIdx = nextNodeIdx;
        end
        h = h(1:n);
        if isequal(obj.MultirateType,'interpolator')
            c = convertToInterpolatorCascade(h);
        else
            c = dsp.FilterCascade(h{:},CloneStages=false);
        end
    end                      
end
end %classdef

%%
% Helper functions

function [c,tout] = convertToInterpolatorCascade(h)
% Convert a cell array of decimators to a cell array of interpolators
% in reverse order. Adjust passband gain, costs, and metadata

    N = length(h);
    temp = cell(1,N);
    Cost = zeros(1,N);
    NCoeffs = zeros(1,N);
    R = zeros(1,N);

    % Construct interpolator stages in reverse order of decimation stages 
    for l = 1:N

        % The interpolation factor is the same as the decimation factor
        R(l) = h{N-l+1}.DecimationFactor;

        % Passband gain is the interpolation factor
        num = R(l)*h{N-l+1}.Numerator; 

        % Construct an interpolator
        temp{l} = dsp.FIRInterpolator(R(l),num);

        % Copy metadata
        fd = h{N-l+1}.getfdesign;

        % Currently support conversion of Nyquist response metadata only 
        switch lower(fd.Response)
        case 'nyquist'
            fdout = fdesign.interpolator(R(l), 'Nyquist',...
                    R(l), 'Tw,Ast', fd.TransitionWidth, fd.Astop,...
                    fd.Fs);
            fmethod      = h{N-l+1}.getfmethod;
            designmethod = h{N-l+1}.getdesignmethod;
            measurements = [];
            
            % Set metadata
            temp{l}.setMetaData(fdout, fmethod, measurements, designmethod);
        otherwise
            % This warning is left for future development. Currently
            % the decimators designs always have Nyquist response type. In
            % case this changes in the future, this warning should alert
            % the developer that whatever new response is given has to be
            % dealth with.
            coder.internal.warning('dsp:designMultistageRateConversion:responsenotsupported',fd.Response);
        end
    
        % Cost was computed for decimation, so recompute
        ct = cost(temp{l});
        Cost(l) = prod(R(1:l-1))*ct.MultiplicationsPerInputSample;
        NCoeffs(l) = ct.NumCoefficients;
    end
    
    tout.Cost = Cost;
    tout.NCoeffs = NCoeffs;
    c = dsp.FilterCascade(temp{:}, CloneStages=false);
end %convertToInterpolatorCascade

function t = replaceFilter(t,c,mc,nc,j,k)
    t.Cost(k)      = mc;
    t.Path(k,2)    = t.Node(j);
    if isfield(c,'Filter')
        t.Filter(k)    = {c.Filter};
    end
    t.NumCoeffs(k) = nc;
    t.TW(k)        = c.TW;
    t.Apass(k)     = c.Apass;
    t.FsStage(k)   = c.FsStage;
end %replaceFilter

function f = factorin(N,L)

    if L == 1
        f = N;
        return;
    end
    
    f = [];
    
    f2 = factorin2(N);
    if L == 2
        f = f2;
        return;
    end
    
    for k = 1:size(f2,1)
        tmp1 = factorin(f2(k,1),L-1);
        tmp2 = factorin(f2(k,2),L-1);
        f = [f;... 
            [tmp1 f2(k,2)*ones(size(tmp1,1),min(size(tmp1,2),1))];...
            [f2(k,1)*ones(size(tmp2,1),min(size(tmp2,2),1)) tmp2]...
            ]; %#ok<AGROW>
    end
    
    f = unique(f,'rows');
end % factorin


function funiq = factorin2(N)

    if isprime(N)
        funiq = [];
        return;
    end
    
    facts = factor(N);
    
    u = unique(perms(facts),'rows');
    [Mu,Nu] = size(u);
    
    f = zeros(Mu*(Nu-1),2);
    
    for k = 1:Mu
        for j = 1:Nu-1
            f((k-1)*(Nu-1)+j,:) = [prod(u(k,1:j),2),prod(u(k,j+1:Nu),2)];
        end
    end
    
    funiq = unique(f,'rows');

end %factorin2
