classdef nyquistNstage < filterdesign.internal.nstageDesigner
    %

    %   Copyright 2018-2019 The MathWorks, Inc.
    
    properties
        NyquistDesignMethod  = 'equiripple';
    end
    
    methods
        function set.NyquistDesignMethod(obj,st)
            s = validatestring(st,{'equiripple','kaiserwin'},...
                'set.NyquistDesignMethod','DesignMethod');
            obj.NyquistDesignMethod = s;
        end
    end
    
    methods (Access = protected)
        function c = cost(obj,M1,M2,TW,Ap)
            % M1 - Decimation from first stage up to this stage
            % M2 - Decimation for this stage
            % TW - Transition width
            % Ap - Ignored for Nyquist
            
            Astop   = obj.Astop;
            FsStage = obj.SampleRate/M1;
            
            switch obj.CostMeasure
                case 'design'
                    c.Filter = designStage(obj,M2,TW,Ap,FsStage);                    
                    ct = cost(c.Filter);
                    
                    % Cost including polyphase decimation
                    c.Cost    = ct.MultiplicationsPerInputSample/M1;
                    c.NCoeffs = ct.NumCoefficients;                    
                case 'estimate'
                    Rst = convertmagunits(Astop,'db','linear','stop');
                    Fc  = FsStage/(2*M2);
                    Fp  = Fc - TW/2;
                    Fst = Fc + TW/2;
                    if (M2 == 2 && isequal(obj.HalfbandDesignMethod,'equiripple')) || ...
                            isequal(obj.NyquistDesignMethod,'equiripple')
                        N = firpmord([Fp Fst], [1 0], [Rst Rst], FsStage);
                    else
                        % Estimate not implemented for IIR halfbands!
                        N = kaiserord([Fp Fst], [1 0], [Rst Rst], FsStage);
                    end
                    c.NCoeffs = N;
                    c.Cost = N/(M2*M1);                            
            end
            c.Apass = Ap;
            c.FsStage = FsStage;
        end
        
        function c = stagecost(obj,t,Mvec)
            M  = obj.RateChangeFactor;
            Fs = obj.SampleRate;
            
            Ap = obj.Apass;
            
            % Compute decimation up to this stage
            M1 = M/prod(Mvec);
            
            % Compute decimation for this stage
            r = Mvec(1);
            
            % Sample rate for this stage
            FsStage = Fs/M1;
            
            % Determine transition width for this stage
            idx = find(all(t.Path(:,end-length(Mvec)+1:end)==Mvec,2),1,'first');
            TW = min(2*(FsStage/(2*r) - (Fs/(2*M)-t.TW(idx,end-length(Mvec)+2)/2)),...
                2*(t.Fstop(idx,end-length(Mvec)+2) - FsStage/(2*r)));
            
            % Determine the cost for this stage
            c = cost(obj,M1,r,TW,Ap);
            c.TW = TW;
        end
        
        function c = dynProgStageCost(obj,t,k,j,r)
            
            Fs = obj.SampleRate;
            M  = obj.RateChangeFactor;
            
            % Compute decimation up to this stage
            M1 = M/t.Node(k);
            
            % Sample rate for this stage
            FsStage = Fs/M1;
            
            % Determine transition width for this stage
            TW = min(2*(FsStage/(2*r) - (Fs/(2*M)-t.TW(j)/2)),...
                2*(t.Fstop(j) - FsStage/(2*r)));
            
            % Determine the cost for this stage
            c = cost(obj,M1,r,TW,0);
            c.TW = TW;
        end
        
        function setStageFstop(obj,N)
            
            t = obj.costTable;
            Fs = obj.SampleRate;
            M  = obj.RateChangeFactor;
            TW = obj.TransitionWidth;
            
            
            for k = 1:size(t,1)
                t.TW(k,end) = TW;
            end
            
            for k = 1:size(t,1)
                for m = 1:N
                    M2 = prod(t.Path(k,m:end));
                    M1 = M/M2;
                    FsStage = Fs/M1;
                    t.Fstop(k,m) = FsStage-FsStage/(2*M2)-TW/2;
                end
            end
            
            obj.costTable = t;
        end
        
        function setLastStageFstop(obj)
            
            t = obj.costTable;
            Fs = obj.SampleRate;
            M  = obj.RateChangeFactor;
            TW = obj.TransitionWidth;
            
            for k = 1:size(t,1)
                M2 = t.Node(k);
                M1 = M/M2;
                FsStage = Fs/M1;
                t.Fstop(k) = FsStage-FsStage/(2*M2)-TW/2;
            end
            
            obj.costTable = t;
        end
        
        function setMetaData(obj,c)
            fd = fdesign.decimator(obj.RateChangeFactor,'Nyquist',...
                obj.RateChangeFactor,obj.TransitionWidth,obj.Astop,...
                obj.SampleRate);
            
            fm = fdfmethod.kaisernyqmin;
            
            m  = measure(c,fd);
            
            setMetaData(c,fd,fm,m);
        end
        
        function b = designStage(obj,M,TW,~,FsStage)
           
            Astop   = obj.Astop;
            f = fdesign.decimator(M,'Nyquist',M,TW,Astop,FsStage);
            if M == 2
                % Use halfband design method for halfbands
                switch obj.HalfbandDesignMethod
                    case {'equiripple','kaiserwin'}
                        b = feval(obj.HalfbandDesignMethod,f,...
                            'SystemObject',true,'FilterStructure',obj.DecimatorStructure);
                    otherwise
                        b = feval(obj.HalfbandDesignMethod,f,...
                            'SystemObject',true);
                end
            else
                if isequal(obj.NyquistDesignMethod,'equiripple')
                    try
                        % Try equiripple designs first. This results in fewer
                        % coefficients when it converges
                        b = equiripple(f,'SystemObject',true,...
                            'FilterStructure',obj.DecimatorStructure);
                    catch
                        % Revert to Kaiser window when equiripple fails
                        b = kaiserwin(f,'SystemObject',true,...
                            'FilterStructure',obj.DecimatorStructure);
                    end
                else
                    b = kaiserwin(f,'SystemObject',true,...
                        'FilterStructure',obj.DecimatorStructure);
                end
            end
        end
    end
end
