function Hs = tosysobj(this,returnSysObj)
%TOSYSOBJ Convert mfilt iir multirate structure to System object

%   Copyright 2016 The MathWorks, Inc.

if ~returnSysObj
    % If returnSysObj is false, then it means that we want to know if the
    % System object conversion is supported for the class at hand. Return a
    % flag as an output instead of returning the filter System object.
    Hs = true;
    return;
end

% Construct the equivalent System object for the current structure
structString = lower(this.FilterStructure);
wdfFlag = false;
switch(structString)
    case {'iirdecim', 'iir polyphase decimator'}
        Hs = dsp.IIRHalfbandDecimator('Specification','Coefficients', ...
            'Structure','Minimum multiplier');
    case {'iirwdfdecim', 'iir wave digital filter polyphase decimator'}
        Hs = dsp.IIRHalfbandDecimator('Specification','Coefficients', ...
            'Structure','Wave Digital Filter');
        wdfFlag = true;
    case {'iirinterp', 'iir polyphase interpolator'}
        Hs = dsp.IIRHalfbandInterpolator('Specification','Coefficients', ...
            'Structure','Minimum multiplier');
    case {'iirwdfinterp', 'iir wave digital filter polyphase interpolator'}
        Hs = dsp.IIRHalfbandInterpolator('Specification','Coefficients', ...
            'Structure','Wave Digital Filter');
        wdfFlag = true;
end

% Copy the coefficients to the System object
nphases = length(this.privphase);
assert(nphases==2);
for k = 1:nphases
    currPhase = this.privphase(k);
    if k==1 && isa(currPhase,'dfilt.delay')
        Hs.HasPureDelayBranch = true;
        Hs.Delay = currPhase.Latency;
    else
        coef = coefficients(currPhase);
        % Check for trailing first order section in the second phase
        if k==2 && (numel(coef{1})==2) && (numel(coef{end})==1)
            if numel(coef)>2 && numel(coef{end-1})==1
                % Last two sections are first order, covert them to one
                % second order section
                lastSectionCoef = [coef{end}+coef{end-1}, coef{end}*coef{end-1}];
                coef{end-1} = lastSectionCoef;
                coef(end) = [];
            else
                % Only the last section is first order. Enable the trailing
                % first order section flag
                Hs.HasTrailingFirstOrderSection = true;
                coef{end} = repmat(coef{end},1,2);
            end
        end
        
        % Assign coefficients to the System object. Covert Allpass
        % coefficients to Wave Digital Filter coefficients before assigning
        % them to the System object
        if wdfFlag
            wdfcoeff = allpass2wdf(coef);
            Hs.(['WDFCoefficients', num2str(k)]) = cell2mat(wdfcoeff);
        else
            Hs.(['AllpassCoefficients', num2str(k)]) = cell2mat(coef);
        end
    end
end

if this.PersistentMemory
    warning(message('signal:dfilt:basecatalog:PersistentMemory',class(this),'sysobj'));
end
