classdef(CaseInsensitiveProperties=true) cascade < dfilt.cascade
%

%   Copyright 2017-2021 The MathWorks, Inc.

    %CASCADE  Cascade DFILT and MFILT filters.
    %   MFILT.CASCADE will be removed in a future release. Use
    %   dsp.FilterCascade instead.
    %
    %   Hm = MFILT.CASCADE(Hm1, Hm2) connects the filters Hm1 and Hm2 in series.
    %   The block diagram of this cascade looks like:
    %
    %      x ---> Hm1 ---> Hm2 ---> y
    %
    %   Note that one usually does not construct CASCADE filters explicitly.
    %   Instead, one obtains these filters as a result from a design using
    %   <a href="matlab:help fdesign">FDESIGN</a>. For more information on generating CASCADE objects from FDESIGN,
    %   see the <docid:dsp_ug#example-multistagerateconv Multistage Rate Conversion> example.
    %
    %   % EXAMPLE #1: Direct instantiation
    %   hm(1) = mfilt.firdecim(2);
    %   hm(2) = dfilt.scalar(2);
    %   Hcas = mfilt.cascade(hm(1),hm(2))
    %   realizemdl(Hcas)  % Requires Simulink
    %
    %   % EXAMPLE #2: Design an multistage decimator filter
    %   Hcas = design(fdesign.decimator(8,'lowpass',.11,.12,.2,60), 'multistage')
    %   fvtool(Hcas)        % Analyze filter
    %   x = randn(100,1); % Input signal
    %   y = filter(Hcas,x); % Apply filter to input signal
    %
    %   See also MFILT/STRUCTURES
    
    %mfilt.cascade class
    %   mfilt.cascade extends dfilt.cascade.
    %
    %    mfilt.cascade properties:
    %       PersistentMemory - Property is of type 'bool'
    %       NumSamplesProcessed - capture (read only)
    %       FilterStructure - Property is of type 'ustring'  (read only)
    %       Stage - Property is of type 'dfilt.basefilter vector'
    %
    %    mfilt.cascade methods:
    %       computezerophase - Compute the zerophase response of a multirate/multistage filter.
    %       createhdlfilter - CREATHDLFILTER <short description>
    %       dispatch - Returns the contained DFILT objects.
    %       filter - Filter method for multirate/multistage cascades.
    %       get_ratechangefactors -   PreGet function for the 'ratechangefactors' property.
    %       getallratechangefactors - Get the rate change factors between sections and
    %       getratechangefactors -   Get the ratechangefactors.
    %       iirxform - IIR Transformations
    %       validateratechangefactors -   Validate the rate change factors for analysis.
    %       zpk -   Multirate filter zero-pole-gain conversion.
    
    methods  % constructor block
        function Hm = cascade(varargin)
            if nargin == 0
                varargin = {mfilt.firdecim, dsp.internal.mfilt.firinterp}; %#ok<MFDECIM>
            end
            
            % MFILT objects are deprecated in R2016a.
            warning(message('dsp:mfilt:mfilt:Obsolete',class(Hm),'dsp.FilterCascade'));
            
            Hm.FilterStructure = 'Cascade';
            
            % Check that all are dfilts or mfilts before starting to set parameters.
            for k=1:length(varargin)
                if isnumeric(varargin{k})
                    g = squeeze(varargin{k});
                    if isempty(g) || length(g)>1
                        error(message('dsp:mfilt:cascade:cascade:MFILTErr'));
                    end
                    varargin{k} = dfilt.scalar(g);
                end
                % Do not attempt varargin{k}(end) for scalar varargin{k}
                % If varargin{k} is a System object, this will fail (function
                % notation)
                L = numel(varargin{k});
                if L > 1
                    v =  varargin{k}(end);
                else
                    v =  varargin{k};
                end
                if ~(isa(v,'dfilt.abstractfilter') || isa(v,'dfilt.multistage'))
                    error(message('dsp:mfilt:cascade:cascade:InvalidEnum'));
                end
            end
            
            % Set the rate change factor and the sections of the cascade
            allStages = [];
            
            for k=1:length(varargin)
                newsec = varargin{k}(:);
                allStages = [allStages; newsec]; %#ok<AGROW>
            end
            Hm.Stage = allStages;
        end  % cascade
        
    end  % constructor block
    
    methods  % public methods
        [hz,wz,phiz,opts] = computezerophase(Hm,varargin)
        hF = createhdlfilter(this)
        Hd = dispatch(Hm)
        y = filter(Hm,x,dim)
        rcf = get_ratechangefactors(this,rcf)
        [R,noblefactors] = getallratechangefactors(Hm)
        rcf = getratechangefactors(this)
        [Ht,anum,aden] = iirxform(Ho,fun,varargin)
        id = validateratechangefactors(this)
        [z,p,k] = zpk(Hm)
    end  % public methods
    
    
    methods (Hidden) % possibly private or hidden
        checkifspblksisneeded(this)
        n = thisimpzlength(this)
    end  % possibly private or hidden
    
end  % classdef

