classdef(CaseInsensitiveProperties=true) cicdecim < mfilt.abstractcicdecim
    %CICDECIM Cascaded Integrator-Comb Decimator.
    %   MFILT.CICDECIM will be removed in a future release. Use
    %   dsp.CICDecimator instead.
    %
    %   Hm = MFILT.CICDECIM(R,M,N,IWL,OWL,WLPS)constructs a Cascaded
    %   Integrator-Comb (CIC) decimation filter object. This structure
    %   has a latency of N (number of sections).  The construction of this
    %   object requires the Fixed-Point Designer.
    %
    %   Inputs:
    %   R is the decimation factor
    %   M is the differential delay
    %   N is the number of sections
    %   IWL is the wordlength of the input signal
    %   OWL is the wordlength of the output signal
    %   WLPS can be a scalar or vector (of length 2*N). WLPS defines the word
    %        lengths per section used during either the accumulation of the
    %        data in the integrator sections or the subtraction of the data
    %        performed by the comb section (using 'wrap' arithmetic). If WLPS
    %        is a scalar, that value is applied to each filter section.
    %
    %   % EXAMPLE: Decimation by a factor of 2 (used to convert from 44.1kHz
    %   % to 22.05kHz)
    %   R = 2;                       % Decimation factor
    %   Hm = mfilt.cicdecim(R);      % Use default NumberOfSections & DifferentialDelay
    %   Fs = 44.1e3;                 % Original sampling frequency: 44.1kHz
    %   n = 0:10239;                 % 10240 samples, 0.232 second long signal
    %   x  = sin(2*pi*1e3/Fs*n);     % Original signal, sinusoid at 1kHz
    %
    %   y_fi = filter(Hm,x); % 5120 samples, still 0.232 seconds
    %
    %   % Scale output to overlay plots
    %   x = double(x); y = double(y_fi); y = y/max(abs(y));
    %   stem(n(1:44)/Fs,x(2:45)); hold on;           % Plot original sampled at 44.1kHz
    %   stem(n(1:22)/(Fs/R),y(3:24),'r','filled');   % Plot decimated signal (22.05kHz) in red
    %   xlabel('Time (sec)');ylabel('Signal value');
    %
    %   See also MFILT/STRUCTURES.

%   Copyright 2017-2020 The MathWorks, Inc.
    
    %mfilt.cicdecim class
    %   mfilt.cicdecim extends mfilt.abstractcicdecim.
    %
    %    mfilt.cicdecim properties:
    %       PersistentMemory - Property is of type 'bool'
    %       NumSamplesProcessed - capture (read only)
    %       FilterStructure - Property is of type 'ustring'  (read only)
    %       States - Property is of type 'mxArray'
    %       DifferentialDelay - Property is of type 'posint user-defined'
    %       NumberOfSections - Property is of type 'posint user-defined'
    %       Arithmetic - Property is of type 'filterdesign_cic_arith enumeration: {'fixed'}'
    %       DecimationFactor - Property is of type 'posint user-defined'
    %       InputOffset - Property is of type 'spt_uint32 user-defined'
    %
    %    mfilt.cicdecim methods:
    %       blocklib - Returns the library and source block
    %       createhdlfilter - Returns the corresponding hdlfiltercomp object
    %       dgdfgen - generate the dg_dfilt structure from a specified filter structure
    %       thisdisp -   Display this object.
    %       filt2struct -   Returns the structure representation of the filter.
    %       getRateOptionParamName - Get the rateOptionParamName.
    %       getblockinputprocessingrestrictions - Get input processing restrictions for
    %       getblockraterestrictions - Get rate restrictions for the block method.
    %       secfilter - Filter this section.
    %       thisblockparams -   Return the block params.
    %       thisfiltquant_plugins - FILTQUANT_PLUGINS Filterquantizer plugin for MFILT.CICDECIM.
    %       tosysobj - Convert mfilt CIC decimator structure to System object
    methods  % constructor block
        function Hm = cicdecim(varargin)
            if ~isfixptinstalled
                error(message('dsp:mfilt:cicdecim:cicdecim:fixptTbxRq'));
            end
            Hm.privfq = dfilt.filterquantizer;
            Hm.privfilterquantizer = dfilt.filterquantizer;
            % Set Object Default States
            factNofSections = 2;
            factDiffDelay = 1;
            zi = filtstates.cic(zeros(factNofSections,1),zeros(factNofSections,factDiffDelay));
            Hm.States = zi;
            deprecationMessage(Hm,'dsp.CICDecimator'); % Deprecated in R2016a.
            Hm.FilterStructure = 'Cascaded Integrator-Comb Decimator';
            
            set_arith(Hm, 'fixed');
            Hm.DecimationFactor = 2;
            
            if nargin > 0
                set(Hm,'DecimationFactor',varargin{1});
            end
            
            % Initialization utility used in both CIC classes.
            cic_init(Hm,varargin{2:end});
            
            
            
        end  % cicdecim
        
    end  % constructor block
    
    methods  % public methods
        [lib,srcblk,hasInputProcessing,hasRateOptions] = blocklib(~,~)
        hF = createhdlfilter(this)
        DGDF = dgdfgen(Hd,hTar,doMapCoeffsToPorts)
        thisdisp(this)
        s = filt2struct(this)
        rateOptionParamName = getRateOptionParamName(~)
        r = getblockinputprocessingrestrictions(~)
        r = getblockraterestrictions(~,inputProcessing)
        [y,zf] = secfilter(this,x,zi)
        p = thisblockparams(this)
        constr = thisfiltquant_plugins(h,arith)
        Hs = tosysobj(this,returnSysObj)
    end  % public methods
    
    
    methods (Hidden) % possibly private or hidden
        s = objblockparams(this,varname)
    end  % possibly private or hidden
    
end  % classdef

