classdef(CaseInsensitiveProperties=true) farrowsrc < mfilt.abstractfarrowsrc
    %FARROWSRC Farrow Sample-Rate Converter
    %   MFILT.FARROWSRC will be removed in a future release. Use
    %   dsp.FarrowRateConverter instead.
    %
    %   Hm = MFILT.FARROWSRC(L,M,COEFFS) constructs a Farrow sample-rate
    %   converter.
    %
    %   L is the interpolation factor. It must be real and positive. If not
    %   specified, it defaults to 3.
    %
    %   M is the decimation factor. It must be real and positive. If not
    %   specified, it defaults to 2.
    %
    %   COEFFS is a matrix containing the coefficients of the filter. If
    %   omitted, it defaults to [-1 1;1 0], a linear interpolation filter.
    %
    %   % Example #1
    %      %Sample-rate conversion to convert from 44.1kHz to 48kHz with a
    %      %linear interpolation filter.
    %      [L,M] = rat(48/44.1);
    %      Hm = mfilt.farrowsrc(L,M);           % We use the default filter
    %      Fs = 44.1e3;                         % Original sampling frequency
    %      n = 0:9407;                          % 9408 samples, 0.213 seconds long
    %      x  = sin(2*pi*1e3/Fs*n);             % Original signal, sinusoid at 1kHz
    %      y = filter(Hm,x);                    % 10241 samples, still 0.213 seconds
    %      stem(n(1:45)/Fs,x(1:45))             % Plot original sampled at 44.1kHz
    %      hold on
    %      % Plot fractionally interpolated signal (48kHz) in red
    %      stem((n(2:50)-1)/(Fs*L/M),y(2:50),'r','filled')
    %      xlabel('Time (sec)');ylabel('Signal value')
    %      legend('44.1 kHz sample rate','48kHz sample rate')
    %
    %   % Example #2
    %      %Design sample-rate converter that uses a 3rd order Lagrange
    %      %interpolation filter to convert from 44.1kHz to 48kHz.
    %      f = fdesign.polysrc(L,M,'Fractional Delay','Np',3);
    %      Hm(2) = design(f,'lagrange');
    %      hfvt = fvtool(Hm,'NormalizeMagnitudeto1','on');
    %      axis([0 .06 -125 10])
    %      legend(hfvt, 'Linear Interpolation', 'Cubic Lagrange Interpolation')
    %
    %   % Example #3
    %      %Convert a polyphase sample-rate converter to a Farrow sample-rate
    %      %converter and compare their respective implementation costs.
    %      fr = fdesign.rsrc(L,M);
    %      Hpm = design(fr); % Polyphase SRC
    %      p=polyphase(Hpm); % Polyphase decomposition
    %      N = 5;            % Polynomial order
    %      Np = size(p,2);   % Length of each polyphase filter
    %      X = 0:1/L:(L-1)/L;
    %      for i=1:Np,
    %         fp(Np-i+1,:)=polyfit(X,p(:,i).',N);
    %      end
    %      Hfm = mfilt.farrowsrc(L,M,fp); % Farrow SRC
    %      cost(Hpm)
    %      cost(Hfm)
    %
    %   For more information about Farrow SRCs, see the
    %   <a href="matlab:helpview('dsp','efficientsrcdemo')">Efficient Sample Rate Conversion between Arbitrary Factors</a> demo.
    %'
    %   See also MFILT/STRUCTURES.

%   Copyright 2018-2020 The MathWorks, Inc.
    
    %mfilt.farrowsrc class
    %   mfilt.farrowsrc extends mfilt.abstractfarrowsrc.
    %
    
    %    mfilt.farrowsrc properties:
    %       PersistentMemory - Property is of type 'bool'
    %       NumSamplesProcessed - capture (read only)
    %       FilterStructure - Property is of type 'ustring'  (read only)
    %       States - Property is of type 'mxArray'
    %       Arithmetic - Property is of type 'filterdesign_arith enumeration: {'double','single','fixed'}'
    %       Coefficients - Property is of type 'mxArray'
    %       InterpolationFactor - Property is of type 'posdouble user-defined'
    %       DecimationFactor - Property is of type 'posdouble user-defined'
    %
    %    mfilt.farrowsrc methods:
    %       createhdlfilter - Returns the corresponding hdlfiltercomp for HDL Code
    %       dgdfgen - generate the dg_dfilt structure from a specified filter structure
    %       getbestprecision - Get the bestprecision.
    %       ishdlable - True if HDL can be generated for the filter object.
    %       thisisrealizable - True if the object is realizable
    %       nadd - Returns the number of adders
    %       nmult - Returns the number of multipliers
    %       parse_coeffstoexport - Store coefficient names and values into hTar for
    %       set_interpolationfactor - PreSet function for the 'interpolationfactor' property
    %       thisfiltquant_plugins - FILTQUANT_PLUGINS Table of filterquantizer plugins
    %       thisreset - <short description>
    
    
    properties (Access=protected, SetObservable)
        %TNEXT Property is of type 'mxArray'
        Tnext = 0;
    end
    
    methods  % constructor block
        function this = farrowsrc(varargin)
            narginchk(0,3);
            deprecationMessage(this,'dsp.FarrowRateConverter'); % Deprecated in R2016a.
            this.privfq = dfilt.filterquantizer;
            this.privfilterquantizer = dfilt.filterquantizer;
            this.FilterStructure = 'Farrow Sample-Rate Converter';
            this.Arithmetic = 'double';
            this.Coefficients = [ -1, 1; 1, 0 ];
            if nargin>0
                this.InterpolationFactor = varargin{1};
            end
            if nargin>1
                this.DecimationFactor =  varargin{2};
            end
            
            if nargin>2
                this.Coefficients =  varargin{3};
            end
        end  % farrowsrc
    end  % constructor block
    
    methods  % public methods
        hF = createhdlfilter(this)
        DGDF = dgdfgen(Hd,hTar,doMapCoeffsToPorts)
        s = getbestprecision(h)
        [result,errstr,errorObj] = ishdlable(Hb)
        b = thisisrealizable(this)
        n = nadd(this)
        n = nmult(this,optimones,optimnegones)
        [hTar,domapcoeffstoports] = parse_coeffstoexport(Hd,hTar)
        L = set_interpolationfactor(this,L)
        constr = thisfiltquant_plugins(h,arith)
        thisreset(this)
    end  % public methods
    
    
    methods (Hidden) % possibly private or hidden
        loadprivatedata(this,s)
        [out,coeffnames,variables] = mapcoeffstoports(this,varargin)
        s = saveprivatedata(this)
        [y,z] = secfilter(this,x,z)
        [NMult,NAdd,NStates,MPIS,APIS] = thiscost(this,M)
        Hs = tosysobj(this,returnSysObj)

    end  % possibly private or hidden
    
end  % classdef

