classdef(CaseInsensitiveProperties=true) firdecim < mfilt.abstractfirdecim
    %FIRDECIM Direct-Form FIR Polyphase Decimator.
    %   MFILT.FIRDECIM will be removed in a future release. Use
    %   dsp.FIRDecimator instead.
    %
    %   Hm = MFILT.FIRDECIM(M) returns an FIR-based decimator object Hm with an
    %   decimation factor of M. A low-pass Nyquist filter of gain 1 and cutoff
    %   frequency of Pi/M is designed by default. This filter allows some
    %   aliasing in the transition band but it's been chosen for its efficiency
    %   since it's first polyphase component is a pure delay.
    %
    %   Hm = MFILT.FIRDECIM(M,NUM) returns a direct-form FIR polyphase decimator Hm.
    %
    %   M is the decimation factor. It must be an integer. If not specified, it
    %   defaults to 2.
    %
    %   NUM is a vector containing the coefficients of the FIR lowpass filter
    %   used for decimation. If omitted, a low-pass Nyquist filter of gain 1
    %   and cutoff frequency of Pi/M is designed by default.
    %
    %   EXAMPLE:
    %      %Decimation by a factor of 2 (used to convert from 44.1kHz to 22.05kHz)
    %      M = 2;                               % Decimation factor
    %      Hm = mfilt.firdecim(M);              % We use the default filter
    %      Fs = 44.1e3;                         % Original sampling frequency: 44.1kHz
    %      n = 0:10239;                         % 10240 samples, 0.232 second long signal
    %      x  = sin(2*pi*1e3/Fs*n);             % Original signal, sinusoid at 1kHz
    %      y = filter(Hm,x);                    % 5120 samples, still 0.232 seconds
    %      stem(n(1:44)/Fs,x(1:44))             % Plot original sampled at 44.1kHz
    %      hold on                              % Plot decimated signal (22.05kHz) in red
    %      stem(n(1:22)/(Fs/M),y(13:34),'r','filled')
    %      xlabel('Time (sec)');ylabel('Signal value')
    %
    %   See also MFILT/STRUCTURES.
    
    %   Copyright 2019 The MathWorks, Inc.    

    %mfilt.firdecim class
    %   mfilt.firdecim extends mfilt.abstractfirdecim.
    %
    %    mfilt.firdecim properties:
    %       PersistentMemory - Property is of type 'bool'
    %       NumSamplesProcessed - capture (read only)
    %       FilterStructure - Property is of type 'ustring'  (read only)
    %       States - Property is of type 'mxArray'
    %       Numerator - Property is of type 'DFILTNonemptyVector user-defined'
    %       DecimationFactor - Property is of type 'posint user-defined'
    %       Arithmetic - Property is of type 'filterdesign_arith enumeration: {'double','single','fixed'}'
    %       PolyphaseAccum - Property is of type 'mxArray'
    %       InputOffset - Property is of type 'spt_uint32 user-defined'
    %
    %    mfilt.firdecim methods:
    %       blocklib - Returns the library and source block
    %       blockparams - Returns the parameters for BLOCK
    %       createhdlfilter - Returns the corresponding hdlfiltercomp object
    %       dgdfgen - generate the dg_dfilt structure from a specified filter structure
    %       getbestprecision - Return best precision for Product and Accumulator
    %       getblockraterestrictions - Get rate restrictions for the block method.
    %       getrealizemdlraterestrictions - Get rate restrictions for the realizemdl
    %       getstates -   Get the states.
    %       ishdlable - True if HDL can be generated for the filter object.
    %       loadpublicinterface -   Load the public interface.
    %       naddp1 - Returns the number of additions + 1.
    %       reset - Reset the filter.
    %       secfilter - Polyphase FIR Interpolator filter
    %       thisfiltquant_plugins - FILTQUANT_PLUGINS Table of filterquantizer plugins
    %       thisnstates - NSTATES  Number of states in discrete-time filter.
    %       thispolyaccscalarexpand - Returns the number of rows of the PolyphaseAccum vector
    %       thissetstates - Overloaded set for the States property.
    %       tofirdecim -   Convert to an FIRDECIM object.
    %       tofirtdecim -   Convert to an FIRTDECIM.
    %       validatestates -   Validate the states.
    methods  % constructor block
        function Hm = firdecim(M,num)
            narginchk(0,2);
            
            deprecationMessage(Hm,'dsp.FIRDecimator'); % Deprecated in R2016a.
            Hm.privfq = dfilt.filterquantizer;
            Hm.privfilterquantizer = dfilt.filterquantizer;
            Hm.FilterStructure = 'Direct-Form FIR Polyphase Decimator';
            Hm.Arithmetic = 'double';
            Hm.DecimationFactor = 2;
            if nargin>0
                Hm.DecimationFactor = M;
            end
            
            if nargin < 2
                Hm.Numerator = defaultfilter(Hm,1,Hm.DecimationFactor);
            elseif isa(num,'dfilt.singleton') && isfir(num)
                [b,~] = tf(num);
                Hm.Numerator = b;
            else
                num = num(:).';
                Hm.Numerator = num;
            end
        end  % firdecim
        
    end  % constructor block
    
    methods  % public methods
        [lib,srcblk,hasInputProcessing,hasRateOptions] = blocklib(~,~)
        s = blockparams(Hd,mapstates,varargin)
        hF = createhdlfilter(this)
        DGDF = dgdfgen(Hd,hTar,doMapCoeffsToPorts)
        s = getbestprecision(h)
        r = getblockraterestrictions(~,inputProcessing)
        r = getrealizemdlraterestrictions(~,inputProcessing)
        S = getstates(Hm,S)
        [result,errstr,errorObj] = ishdlable(filterobj)
        loadpublicinterface(this,s)
        n = naddp1(h)
        reset(Hm)
        [y,zf] = secfilter(Hm,x,zi)
        constr = thisfiltquant_plugins(h,arith)
        n = thisnstates(Hm)
        nacc = thispolyaccscalarexpand(h)
        S = thissetstates(Hd,S)
        Hm = tofirdecim(this)
        Hm = tofirtdecim(this)
        validatestates(this)
    end  % public methods
    
    
    methods (Hidden) % possibly private or hidden
        s = objblockparams(this,varname)
        thisreset(Hm)
    end  % possibly private or hidden
    
    methods(Static,Hidden)
        function props = getDisplayPropertiesImpl()
            props = {...
                'FilterStructure', ...
                'Arithmetic', ...
                'Numerator',...
                'DecimationFactor',...
                'PersistentMemory'};
        end
    end
    
end  % classdef

