classdef(CaseInsensitiveProperties=true)firsrc < mfilt.abstractsrc
    %FIRSRC Direct-Form FIR Polyphase Sample-Rate Converter.
    %   MFILT.FIRSRC will be removed in a future release. Use
    %   dsp.FIRRateConverter instead.
    %
    %   Hm = MFILT.FIRSRC(L,M,NUM) constructs a direct-form FIR polyphase
    %   sample-rate converter.
    %
    %   L is the interpolation factor. It must be an integer. If not specified,
    %   it defaults to 2.
    %
    %   M is the decimation factor. It must be an integer. If not specified, it
    %   defaults to 1. If L is also not specified, M defaults to 3.
    %
    %   NUM is a vector containing the coefficients of the FIR lowpass filter
    %   used for sample-rate conversion. If omitted, a lowpass Nyquist filter
    %   of gain L and cutoff frequency of Pi/max(L,M) is used by default.
    %
    %   EXAMPLE:
    %      %Sample-rate conversion by a factor of 147/160 (used to
    %      %downconvert from 48kHz to 44.1kHz)
    %      L  = 147; M = 160;                   % Interpolation/decimation factors.
    %      Hm = mfilt.firsrc(L,M);              % We use the default filter
    %      Fs = 48e3;                           % Original sampling frequency: 48kHz
    %      n = 0:10239;                         % 10240 samples, 0.213 seconds long
    %      x  = sin(2*pi*1e3/Fs*n);             % Original signal, sinusoid at 1kHz
    %      y = filter(Hm,x);                    % 9408 samples, still 0.213 seconds
    %      stem(n(1:49)/Fs,x(1:49))             % Plot original sampled at 48kHz
    %      hold on
    %      % Plot fractionally decimated signal (44.1kHz) in red
    %      stem(n(1:45)/(Fs*L/M),y(13:57),'r','filled')
    %      xlabel('Time (sec)');ylabel('Signal value')
    %
    %   See also MFILT/STRUCTURES.

%   Copyright 2017-2020 The MathWorks, Inc.
    
    %mfilt.firsrc class
    %   mfilt.firsrc extends mfilt.abstractsrc.
    %
    %    mfilt.firsrc properties:
    %       PersistentMemory - Property is of type 'bool'
    %       NumSamplesProcessed - capture (read only)
    %       FilterStructure - Property is of type 'ustring'  (read only)
    %       States - Property is of type 'mxArray'
    %       Numerator - Property is of type 'DFILTNonemptyVector user-defined'
    %       RateChangeFactors - Property is of type 'posint_vector user-defined'
    %       InputOffset - Property is of type 'mxArray'
    %       Arithmetic - Property is of type 'filterdesign_arith enumeration: {'double','single','fixed'}'
    %
    %    mfilt.firsrc methods:
    %       blocklib - Returns the library and source block
    %       blockparams - Returns the parameters for BLOCK
    %       createhdlfilter - Returns the corresponding hdlfiltercomp object
    %       dgdfgen - Generate the dg_dfilt structure from a specified filter structure
    %       thisdisp - Object display.
    %       getbestprecision - Return best precision for product and accumulator.
    %       getblockinputprocessingrestrictions - Get input processing restrictions for
    %       getblockraterestrictions - Get rate restrictions for the block method.
    %       getstates - Overloaded get for the States property.
    %       isblockable - True if the object supports the block method
    %       ishdlable - True if HDL can be generated for the filter object.
    %       thisisrealizable - True if the object is realizable
    %       loadpublicinterface -   Load the public interface.
    %       naddp1 -   Returns the number of additions + 1.
    %       resetpolym - Reset the polyphase matrix.
    %       secfilter - Polyphase FIR sampling-rate converter filter.
    %       set_polyselect -   PreSet function for the 'polyselect' property.
    %       set_tapindex -   PreSet function for the 'tapindex' property.
    %       setnum - Overloaded set for the Numerator property.
    %       setratechangefactor - SETRATE Overloaded set for the RateChangeFactor property.
    %       thischeckratefactors - Check for valid L and M.
    %       thisfiltquant_plugins - FILTQUANT_PLUGINS Table of filterquantizer plugins
    %       thisnstates - NSTATES  Number of states in discrete-time filter.
    %       thissetstates - Overloaded set for the States property.
    %       tofirsrc -   Convert to an FIR SRC
    %       tosysobj - Convert mfilt SRC structure to System object
    
    
    properties (Access=protected, SetObservable, GetObservable)
        %POLYPHASESELECTOR Property is of type 'MATLAB array'
        PolyphaseSelector = uint32( 0 );
    end
    
    properties (SetObservable, GetObservable)
        %ARITHMETIC Property is of type 'filterdesign_arith enumeration: {'double','single','fixed'}'
        Arithmetic = 'double';
    end
    
    
    methods  % constructor block
        function Hm = firsrc(L,M,num)
            narginchk(0,3);
            
            if nargin==0
                % Default values
                L=2;M=3;
            elseif nargin==1
                M=1;
            end
            
            if length(L)>1, error(message('dsp:mfilt:firsrc:firsrc:MFILTErr'));end
            
            deprecationMessage(Hm,'dsp.FIRRateConverter'); % Deprecated in R2016a.
            Hm.privfq = dfilt.filterquantizer;
            Hm.privfilterquantizer = dfilt.filterquantizer;
            Hm.FilterStructure = 'Direct-Form FIR Polyphase Sample-Rate Converter';
            Hm.Arithmetic = 'double';
            Hm.RateChangeFactors = [L,M];
            
            if nargin < 3
                Hm.Numerator = defaultfilter(Hm,L,M);
            elseif isa(num,'dfilt.singleton') && isfir(num)
                [b,~] = tf(num);
                Hm.Numerator = b;
            else
                num = num(:).';
                Hm.Numerator = num;
            end
        end  % firsrc
        
    end  % constructor block
    
    methods
        function set.PolyphaseSelector(obj,value)
            obj.PolyphaseSelector = set_polyselect(obj,value);
        end
        
        function value = get.Arithmetic(obj)
            value = get_arith(obj,obj.Arithmetic);
        end
        function set.Arithmetic(obj,value)
            % Enumerated DataType = 'filterdesign_arith enumeration: {'double','single','fixed'}'
            value = validatestring(value,{'double','single','fixed'},'','Arithmetic');
            obj.Arithmetic = set_arith(obj,value);
        end
        
    end   % set and get functions
    
    methods  % public methods
        [lib,srcblk,hasInputProcessing,hasRateOptions] = blocklib(~,~)
        s = blockparams(Hd,mapstates,varargin)
        hF = createhdlfilter(this)
        DGDF = dgdfgen(Hd,hTar,doMapCoeffsToPorts)
        thisdisp(this)
        s = getbestprecision(h)
        r = getblockinputprocessingrestrictions(~)
        r = getblockraterestrictions(~,~)
        S = getstates(Hm,dummy)
        b = isblockable(~)
        [result,errstr,errorObj] = ishdlable(Hb)
        b = thisisrealizable(this)
        loadpublicinterface(this,s)
        n = naddp1(h)
        resetpolym(Hm,num)
        [y,z] = secfilter(Hm,x,z)
        polyselect = set_polyselect(H,polyselect)
        tapindex = set_tapindex(H,tapindex)
        num = setnum(Hm,num)
        R = setratechangefactor(Hm,R)
        thischeckratefactors(Hm,L,M)
        constr = thisfiltquant_plugins(h,arith)
        n = thisnstates(Hm)
        S = thissetstates(Hm,S)
        Hm = tofirsrc(this)
        Hs = tosysobj(this,returnSysObj)
    end  % public methods
    
    
    methods (Hidden) % possibly private or hidden
        s = objblockparams(this,varname)
        specifyall(this,flag)
        thisreset(H)
    end  % possibly private or hidden
    
end  % classdef

