classdef (CaseInsensitiveProperties,Abstract) abstractoneptlp < matlab.mixin.SetGet & matlab.mixin.Copyable
    %mpr.abstractoneptlp class
    %    mpr.abstractoneptlp properties:
    %       M - Property is of type 'mxArray'  (read only)
    %       fgrid - Property is of type 'mxArray'  (read only)
    %       fext - Property is of type 'mxArray'  (read only)
    %       Agrid - Property is of type 'mxArray'  (read only)
    %       Aext - Property is of type 'mxArray'  (read only)
    %       Ggrid - Property is of type 'mxArray'  (read only)
    %       Gext - Property is of type 'mxArray'  (read only)
    %       Wgrid - Property is of type 'mxArray'  (read only)
    %       Wext - Property is of type 'mxArray'  (read only)
    %       errgrid - Property is of type 'mxArray'  (read only)
    %       errext - Property is of type 'mxArray'  (read only)
    %       analyticerr - Property is of type 'mxArray'  (read only)
    %       measurederr - Property is of type 'mxArray'  (read only)
    %       DensityFactor - Property is of type 'mxArray'  (read only)
    %       iter - Property is of type 'mxArray'  (read only)
    %
    %    mpr.abstractoneptlp methods:
    %       disp -   Display this object.
    %       iterate -   Perform one iteration of the MPR algorithm
    %       modifiedlagrange -   Implements the modified lagrange interpolation formula
    %       mpr -   Implement the MPR algorithm
    
    % Copyright 2018-2019 The MathWorks, Inc.
    
    properties (SetAccess=protected, SetObservable)
        %M Property is of type 'mxArray'  (read only)
        M = [];
        %FGRID Property is of type 'mxArray'  (read only)
        fgrid = [];
        %FEXT Property is of type 'mxArray'  (read only)
        fext = [];
        %AGRID Property is of type 'mxArray'  (read only)
        Agrid = [];
        %AEXT Property is of type 'mxArray'  (read only)
        Aext = [];
        %GGRID Property is of type 'mxArray'  (read only)
        Ggrid = [];
        %GEXT Property is of type 'mxArray'  (read only)
        Gext = [];
        %WGRID Property is of type 'mxArray'  (read only)
        Wgrid = [];
        %WEXT Property is of type 'mxArray'  (read only)
        Wext = [];
        %ERRGRID Property is of type 'mxArray'  (read only)
        errgrid = [];
        %ERREXT Property is of type 'mxArray'  (read only)
        errext = [];
        %ANALYTICERR Property is of type 'mxArray'  (read only)
        analyticerr = 1e6;
        %MEASUREDERR Property is of type 'mxArray'  (read only)
        measurederr = 1e5;
        %DENSITYFACTOR Property is of type 'mxArray'  (read only)
        DensityFactor = 16;
        %ITER Property is of type 'mxArray'  (read only)
        iter = 0;
    end
    
    
    methods
    end   % set and get functions
    
    methods  % public methods
        %----------------------------------------
        function disp(this)
            %DISP   Display this object.
            
            %   Author(s): R. Losada
            
            disp(get(this));
            
            
        end  % disp
        
        %----------------------------------------
        function iterate(this)
            %ITERATE   Perform one iteration of the MPR algorithm
            
            %   Author(s): R. Losada
            
            
            fextVal = this.fext;
            AgridVal = this.Agrid;
            AextVal = this.Aext;
            fgridVal = this.fgrid;
            WgridVal = this.Wgrid;
            
            % Compute error analytically
            compute_err(this);
            
            % Evaluate approximating polynomial at grid points
            GgridVal = evalapprox(this,fgridVal); % Need to pass grid as input
            
            errgridVal = WgridVal.*(AgridVal - GgridVal);
            
            % Find new M extremals (keep the edges)
            [new_ext,newidx] = extremals(this,fgridVal,errgridVal);
            
            % Update the mag of the error
            this.measurederr = norm(errgridVal,inf);
            
            if (length(new_ext) > length(fextVal))
                % Number of extremals increased, reject superfluous extremals
                new_ext = reject_ext(this,new_ext,newidx);
            end
            
            % Compute polynomial at new extremals
            GextVal = evalapprox(this,new_ext); % Takes a different grid as input
            
            % Set extremals to new extremals
            this.fext = new_ext;
            
            % Determine weights at extremals
            weights_at_ext(this);
            
            % Compute error at extremals
            errextVal = this.Wext.*(AextVal-GextVal);
            
            this.errgrid = errgridVal;
            this.errext = errextVal;
            this.Ggrid = GgridVal;
            this.Gext = GextVal;
            this.Aext = AextVal;
            
            this.iter = this.iter+1;
            
            
            
        end  % iterate
        
        %----------------------------------------
        function Ggrid= modifiedlagrange(this,Ck,grid)
            %MODIFIEDLAGRANGE   Implements the modified lagrange interpolation formula
            %   For a reference see: www.maths.man.ac.uk/~nareports/narep440.pdf
            %   or IMA Journal of Numerical Analysis (2004) 24, 547 556 The numerical
            %   stability of barycentric Lagrange interpolation.
            
            %   Author(s): R. Losada
            
            m = this.M;
            fExt = this.fext(1:end-1); % Notice last point not used
            
            beta = lagrangecoeff(this,m+1,fExt);
            
            %
            Ggrid = zeros(size(grid));
            for n = 1:length(grid)
                GExt = Ck(grid(n)==fExt);
                if isempty(GExt)
                    ssum = sum(Ck.*beta./(cos(grid(n)*pi)-cos(fExt*pi)));
                    % If grid(n) is very close to an element of fext, ssum may become
                    % infinite. Use Ck in this case
                    if isinf(ssum)
                        [~,i] = min(abs(grid(n)-fExt));
                        Ggrid(n) = Ck(i);
                    else
                        lx = prod(cos(grid(n)*pi)-cos(fExt*pi));
                        Ggrid(n) = lx.*ssum;
                    end
                else
                    Ggrid(n) = GExt;
                end
            end
            
            
            
            
        end  % modifiedlagrange
        
        %----------------------------------------
        function b = mpr(this)
            %MPR   Implement the MPR algorithm
            
            %   Author(s): R. Losada
            
            % Here's an interesting case when looking at number of iterations vs
            % density factor vs accuracy of convergence (relative difference threshold)
            % Suppose rel_diff_thresh is changed to 1e-8 below (in the code).
            % N=99;
            % ws=19;
            % H=mpr.oneptlp_t2(N,ws,16);b=mpr(H); % Design doesn't converge relative
            %                                     % difference reaches rock bottom at
            %                                     % 8.7942e-05. Even allowing more
            %                                     % iterations would not help. 1e-8 is
            %                                     % too small a threshold for convergence
            % H=mpr.oneptlp_t2(N,ws,32);b=mpr(H); % By increasing the density factor,
            %                                     % the design converges in 12
            %                                     % iterations ans the relative
            %                                     % difference is 1.3445e-16
            
            
            maxiter = 30; % Maximum number of iterations allowed
            
            % Relative difference threshold. This value is influenced by the density
            % factor. The reason has to do with example above
            rel_diff_thresh = str2double(['1e-',num2str(floor(log2(this.DensityFactor)))]);
            
            % In this check we could use the following suggested in Antoniou instead:
            % while((rel_difference(this)>1e-8)&&this.iter<maxiter)
            % but it is equivalent and requires a little more computation
            while (norm((this.analyticerr-this.measurederr)/this.measurederr,inf) > rel_diff_thresh) &&...
                    (this.iter < maxiter)
                iterate(this);
            end
            
            if (norm((this.analyticerr-this.measurederr)/this.measurederr,inf) > rel_diff_thresh) &&...
                    (this.iter == maxiter)
                warning(message('dsp:mpr:abstractoneptlp:mpr:nonConvergence'));
            end
            
            b = determine_impresp(this);
            
            
            
        end  % mpr
        
    end  % public methods
    
    
    methods (Hidden) % possibly private or hidden
        %----------------------------------------
        function Ggrid = barycentriclagrange(this,Ck,grid)
            %BARYCENTRICLAGRANGE
            
            %   Author(s): R. Losada
            
            % Evaluate the approximating polynomial at all grid points by using the
            % Lagrange interpolation formula in barycentric form
            m = this.M;
            fExt = this.fext(1:end-1); % Notice last point not used
            
            beta = lagrangecoeff(this,m+1,fExt);
            
            %
            Ggrid = zeros(size(grid));
            for n = 1:length(grid)
                Ggrid(n) = evalG(Ck,beta,fExt,grid(n));
            end
        end  % barycentriclagrange
        
        
        %----------------------------------------
        
        %----------------------------------------
        function compute_err(this)
            %COMPUTE_ERR
            
            %   Author(s): R. Losada
            
            MVal = this.M;
            WextVal = this.Wext;
            fextVal = this.fext;
            
            % Compute b_k
            b = lagrangecoeff(this,MVal+2,fextVal);
            
            k = 1:MVal+2;
            sgn = (-1).^(k+1);
            this.analyticerr = b(1)/sum(b.*sgn./WextVal);
            
            
        end  % compute_err
        
        %----------------------------------------
        function desired_resp(this)
            %DESIRED_RESP
            
            %   Author(s): R. Losada
            
            % Determine desired function at extremals
            this.Aext = [1, zeros(1,length(this.fext)-1)];
            
            % Determine desired response on grid
            this.Agrid = [1, zeros(1,length(this.fgrid)-1)];
            
            
            
        end  % desired_resp
        
        %----------------------------------------
        function Ggrid = evalapprox(this,grid)
            %EVALAPPROX
            
            %   Author(s): R. Losada
            
            m = this.M;
            aExt = this.Aext;
            wExt = this.Wext;
            err = this.analyticerr;
            
            % Determine Ck, the value the approx poly takes at the extremals
            k = 1:m+1;
            sgn = (-1).^k;
            Ck = aExt(1:end-1) + sgn*err./wExt(1:end-1);
            
            % Evaluate the approximating polynomial at all grid points by using the
            % Lagrange interpolation formula in barycentric form
            
            %Ggrid = barycentriclagrange(this,Ck,grid); % The barycentric form was used in the original MPR
            Ggrid = modifiedlagrange(this,Ck,grid); % try this alternative; it's faster and seems to produce better results
            
            
        end  % evalapprox
        
        %----------------------------------------
        function [new_ext,newidx] = extremals(~,fgrid,errgrid)
            %EXTREMALS
            
            %   Author(s): R. Losada
            
            % Compute extremals
            
            
            % Ensure that band edges are retained
            max_ext = local_max(errgrid(2:end));
            min_ext = local_max(-errgrid(2:end));
            
            newidx = sort([max_ext,min_ext]);
            
            new_ext = [0,fgrid(newidx+1)];
        end  % extremals
        
        
        %----------------------------------------
        
        %----------------------------------------
        function init(this,Fst,wstruct)
            %INIT
            
            %   Author(s): R. Losada
            
            m = this.M;
            
            % Setup search grid
            dF = this.DensityFactor; % Use either factory or specified value
            dens = dF*(m+2); % grid density
            
            this.fgrid = [0,linspace(Fst,1,dens)];
            
            % Initialize guess for extremals
            initialize_extremals(this);
            
            % Determined desired response at extremals and on entire grid
            desired_resp(this);
            
            % Determine weights on entire grid
            determine_weights(this,wstruct);
            
            % Determine weights at extremals
            weights_at_ext(this);
            
            % Initialize value of approx polynomial on grid
            this.Ggrid = zeros(size(this.Agrid));
            
            % Initialize value of approx polynomial at extremals
            this.Gext = zeros(size(this.Aext));
            
            % Initialize error on grid
            this.errgrid = this.Agrid;
            
            % Initialize error at extremals
            this.errext = this.Aext;
            
            
            
        end  % init
        
        %----------------------------------------
        function initialize_extremals(this)
            %INITIALIZE_EXTREMALS
            
            %   Author(s): R. Losada
            
            MVal = this.M;
            
            % Initialize extremals, they must be on the grid
            fgridVal = this.fgrid;
            
            Lgrid = length(fgridVal);
            
            mextidx = round(linspace(2,max(2,Lgrid-floor((Lgrid-1)/MVal)),MVal));
            
            this.fext = [0,fgridVal(mextidx),fgridVal(end)];
            
            
            
            
        end  % initialize_extremals
        
        %----------------------------------------
        function b = lagrangecoeff(~,L,fext)
            %LAGRANGECOEFF
            
            %   Author(s): R. Losada
            
            % Compute lagrange coefficient
            b = zeros(1,L); % preallocate
            for k = 1:L
                % Extract kth extremal
                extmk = [fext(1:k-1),fext(k+1:end)];
                b(k) = prod(1./(cos(fext(k)*pi)-cos(extmk*pi)));
            end
            
            
        end  % lagrangecoeff
        
        %----------------------------------------
        function nyquist_weights(this,wstruct)
            %NYQUIST_WEIGHTS
            
            %   Author(s): R. Losada
            
            hp = wstruct.iresp;
            nneg = wstruct.nneg;
            Decay = wstruct.decay;
            
            GF = this.fgrid;
            
            DW = zeros(size(GF)); % Preallocate
            
            DW(1) = 1;
            
            % Compute the zerophase response of BF(z^L) on the freq. grid
            Hz = zerophase(hp,1,GF*pi);
            
            if nneg
                DW(2:end) = sqrt(abs(Hz(2:end)));
            else
                DW(2:end) = abs(Hz(2:end));
            end
            
            if Decay ~= 0
                % Determine deltay from Decay
                deltax = 1-GF(2);
                deltay = Decay*deltax;
                
                % Weight must increase exponentially for linear dB slope
                DW(2:end) = DW(2:end).*10.^(linspace(0,deltay/20,length(GF)-1));
            end
            
            this.Wgrid = DW;
            
            
            
        end  % nyquist_weights
        
        %----------------------------------------
        function new_ext = reject_ext(this,new_ext,newidx)
            %REJECT_EXT
            
            %   Author(s): R. Losada
            
            % Reject superfluous potential extremals
            fExt = this.fext;
            errGrid = this.errgrid;
            
            % Retain extremals from previous iterations
            % Find indices of extremals to retain
            retain_idx = ismember(new_ext,fExt);
            
            % Make sure we always retain the first two extremals since they correspond
            % to the band edges.
            retain_idx(1:2) = true;
            
            % Compute new error at new extremals
            errExt = errGrid(newidx);
            
            % Compute max error
            maxerr = norm(errExt,inf);
            
            % Artificially increase error at extremals to retain so they are not
            % deleted
            alterrext = errExt;
            alterrext(retain_idx) = maxerr+eps;
            
            while (length(new_ext) > length(fExt))
                [~,minidx] = min(abs(alterrext)); % Don't include band-edge
                new_ext = [new_ext(1:minidx-1), new_ext(minidx+1:end)];
                alterrext = [alterrext(1:minidx-1), alterrext(minidx+1:end)];
            end
            
            
        end  % reject_ext
        
        %----------------------------------------
        function Q = rel_difference(this)
            %REL_DIFFERENCE
            
            %   Author(s): R. Losada
            
            
            % Determine relative difference between extremals
            errextVal = this.errext(2:end);
            n = norm(errextVal,inf);
            Q = (n-min(abs(errextVal)))/n;
            
            
        end  % rel_difference
        
        %----------------------------------------
        function weights_at_ext(this)
            %WEIGHTS_AT_EXT
            
            %   Author(s): R. Losada
            
            % Determine weights at extremals
            this.Wext = this.Wgrid(ismember(this.fgrid,this.fext));
            
            
            
        end  % weights_at_ext
        
    end  % possibly private or hidden
    
end  % classdef

function Ggrid_x = evalG(Ck,beta,fext,x)

Ggrid_x = Ck(x==fext);
if isempty(Ggrid_x)
    betaprime = beta./(cos(x*pi)-cos(fext*pi));
    Ggrid_x = sum(Ck.*betaprime)/sum(betaprime);
end
end  % evalG


function k = local_max(x)
% k = local_max(x)
% finds location of local maxima
%
% Modified version of the one in signal/private

% Don't include maxima if value is negative
x(x<=0) = 0;

M = length(x);
x1 = x(1:M-1);
x2 = x(2:M);
b1 = (x1<=x2); b2 = (x1>x2);
k = find(b1(1:M-2)&b2(2:M-1))+1;
if x(1)>x(2), k = [k, 1]; end
if x(M)>x(M-1), k = [k, M]; end
k = sort(k);
end  % local_max


