classdef (CaseInsensitiveProperties) oneptlp_t2 < mpr.abstractoneptlp
    %mpr.oneptlp_t2 class
    %   mpr.oneptlp_t2 extends mpr.abstractoneptlp.
    %
    
    %    mpr.oneptlp_t2 properties:
    %       M - Property is of type 'mxArray'  (read only)
    %       fgrid - Property is of type 'mxArray'  (read only)
    %       fext - Property is of type 'mxArray'  (read only)
    %       Agrid - Property is of type 'mxArray'  (read only)
    %       Aext - Property is of type 'mxArray'  (read only)
    %       Ggrid - Property is of type 'mxArray'  (read only)
    %       Gext - Property is of type 'mxArray'  (read only)
    %       Wgrid - Property is of type 'mxArray'  (read only)
    %       Wext - Property is of type 'mxArray'  (read only)
    %       errgrid - Property is of type 'mxArray'  (read only)
    %       errext - Property is of type 'mxArray'  (read only)
    %       analyticerr - Property is of type 'mxArray'  (read only)
    %       measurederr - Property is of type 'mxArray'  (read only)
    %       DensityFactor - Property is of type 'mxArray'  (read only)
    %       iter - Property is of type 'mxArray'  (read only)
    %
    %    mpr.oneptlp_t2 methods:
    
    % Copyright 2018 The MathWorks, Inc.
    
    methods  % constructor block
        function this = oneptlp_t2(N,Fst,dF,wstruct)
            %ONEPTLP_T2   Construct an ONEPTLP_T2 object.
            %   H = ONEPTLP_T2(N,Fst,DF) constructs a type II, one-passband-point
            %   lowpass MPR object H given a filter order N, a stopband-edge frequency
            %   Fst, and a density factor DF.
            %
            %   N must be odd (type II lowpass only). If not specified, it defaults to
            %   99.
            %
            %   Fst must be greater than zero and less than one. If not specified, it
            %   defaults to 0.2.
            %
            %   DF must be a positive integer. If not specified, it defaults to 16.
            %
            %   H = ONEPTLP_T2(N,Fst,DF,Wstruct) specifies a structure with parameters to
            %   determine the weights. Wstruct has the following fields:
            %       iresp - impulse response of filter to shape weights (used for
            %               Nyquist designs but can be set to 1)
            %       nneg  - boolean indicating whether the response should be
            %               non-negative  (also used for Nyquist designs)
            %       decay - stopband decay rate in dB.
            
            %   Author(s): R. Losada
            
            
            % Set up defaults
            if nargin < 1, N = 99; end
            if nargin < 2, Fst = 0.2; end
            if nargin > 2, this.DensityFactor = dF; end
            if nargin < 4, wstruct.nneg = false; wstruct.iresp = 1; wstruct.decay = 0; end
            
            % The algorithm works with half the length
            if rem(N,2) == 0
                error(message('dsp:mpr:oneptlp_t2:oneptlp_t2:evenOrder'));
            end
            M = (N-1)/2;
            this.M = M;
            
            % Initialize all properties
            init(this,Fst,wstruct);
            
            
        end  % oneptlp_t2
        
    end  % constructor block
    
    methods (Hidden) % possibly private or hidden
        %----------------------------------------
        function b = determine_impresp(this)
            %DETERMINE_IMPRESP
            
            %   Author(s): R. Losada
            
            M = this.M;
            N = M*2;
            
            % Evaluate approximating polynomial at N+1 evenly spaced points
            w = linspace(0,2-2/(N+1),N+1); % % Don't include factor of pi
            Gw = evalapprox(this,w);
            
            % Can evaluate as this
            % bb = zeros(1,M+2);
            % for n=0:M,
            %     bb(n+1) = (Gw(1)+2*sum(Gw(2:M+1).*cos(2*pi*(1:M)*n/(N+1))))/(N+1);
            % end
            %
            % bc = .5*(bb(1:end-1)+bb(2:end));
            
            % But use ifft instead to be faster
            hz = ifft(Gw, 'symmetric'); % These are the modified coeffs
            bc = .5*(hz(1:M+1)+[hz(2:M+1),0]);
            
            b = [bc(end:-1:1),bc];
            
            
        end  % determine_impresp
        
        %----------------------------------------
        function determine_weights(this,wstruct)
            %DETERMINE_WEIGHTS
            
            %   Author(s): R. Losada
            
            % For now, the weights are determined in a way necessary for Nyquist
            % designs
            nyquist_weights(this,wstruct);
            
            % Since this is type II, the weights need to be modified accordingly
            this.Wgrid = cos(this.fgrid*pi/2).*this.Wgrid;
            
            
        end  % determine_weights
        
    end  % possibly private or hidden
    
end  % classdef

