classdef(CaseInsensitiveProperties=true) quantizer < quantum.cquantizer
    %QUANTIZER   Constructor for QUANTIZER object.
    %   Q = QUANTIZER creates a quantizer with all default values.
    %
    %   Q = QUANTIZER(Value1, Value2, ... ) creates a QUANTIZER object with
    %   values Value1, Value2, ....  If two values conflict, then the last
    %   value in the list is the one that is set.
    %
    %   Q = QUANTIZER(a) where a is a structure whose field names are object
    %   property names, sets the properties named in each field name with
    %   the values contained in the structure.
    %
    %   Q = QUANTIZER(pn,pv) sets the named properties specified in the cell
    %   array of strings pn to the corresponding values in the cell array
    %   pv.
    %
    %   Q = QUANTIZER('Property1',Value1, 'Property2',Value2,...) assigns
    %   values associated with named properties.
    %
    %   It is permissible to use property/value string pairs, structures,
    %   and property/value cell array pairs in the same call to SET.
    %
    %   Values sorted by property name:
    %   'Mode'
    %      'none'   -  No quantization is done.  Pass through input to output.
    %      'double' - Double-precision mode.  Override all other parameters.
    %      'float'  - Custom-precision floating-point mode.
    %      'single' - Single-precision mode.  Override all other parameters.
    %      'fixed'  - Signed fixed-point mode.
    %      'ufixed' - Unsigned fixed-point mode.
    %
    %   'Roundmode'
    %      'ceil'       - Round towards positive infinity.
    %      'convergent' - Convergent rounding.
    %      'fix'        - Round towards zero.
    %      'floor'      - Round towards negative infinity.
    %      'round'      - Round towards nearest.
    %
    %   'Overflowmode' (fixed-point only)
    %      'saturate' - Saturate at max value on overflow.
    %      'wrap'     - Wrap on overflow.
    %
    %   'Format'
    %      [wordlength  fractionlength] - The format for fixed and ufixed mode.
    %      [wordlength  exponentlength] - The format for float mode.
    %
    %
    %   Besides properties, a QUANTIZER object also has states 'max', 'min',
    %   'noverflows', 'nunderflows', 'noperations'.  They can be accessed
    %   through QUANTIZER/GET or q.max, q.min, q.noverflows, q.nunderflows,
    %   but they cannot be set.  They are updated during the
    %   QUANTIZER/QUANTIZE method, and are reset by the QUANTIZER/RESET
    %   method.
    %
    %   States:
    %     'max'         - Maximum value before quantizing.
    %     'min'         - Minimum value before quantizing.
    %     'noverflows'  - Number of overflows.
    %     'nunderflows' - Number of underflows.
    %     'noperations' - Number of elements quantized.
    %
    %   Help is available for all the properties by typing at the command line:
    %     HELP QUANTIZER/<PROPERTY>
    %   or
    %     HELPWIN QUANTIZER/<PROPERTY>
    %   where <PROPERTY> is one of the properties.  For
    %   example
    %     help quantizer/mode
    %     help quantizer/roundmode
    %     help quantizer/overflowmode
    %     help quantizer/format
    %
    %   Example:
    %     q = quantizer('fixed', 'ceil', 'saturate', [5 4])
    %
    %   See also FILTERDESIGN, QUANTIZER/GET, QUANTIZER/SET, QUANTIZER/MODE,
    %   QUANTIZER/ROUNDMODE, QUANTIZER/OVERFLOWMODE, QUANTIZER/FORMAT,
    %   QUANTIZER/QUANTIZE, QUANTIZER/RESET.

%   Copyright 2018-2020 The MathWorks, Inc.
    
    %quantum.quantizer class
    %   quantum.quantizer extends quantum.cquantizer.
    %
    %    quantum.quantizer properties:
    %       mode - Property is of type 'quantizer Mode'
    %       datamode - Property is of type 'quantizer Mode'
    %       roundmode - Property is of type 'quantizer Round Mode'
    %       overflowmode - Property is of type 'quantizer Overflow Mode'
    %       format - Property is of type 'real point'
    %       wordlength - Property is of type 'double'
    %       fractionlength - Property is of type 'double'
    %       precision - Property is of type 'double'
    %       exponentlength - Property is of type 'double'
    %       max - Property is of type 'double'
    %       min - Property is of type 'double'
    %       maxlog - Property is of type 'double'
    %       minlog - Property is of type 'double'
    %       noverflows - Property is of type 'double'
    %       nunderflows - Property is of type 'double'
    %       noperations - Property is of type 'double'
    %       denormalmax - Property is of type 'double'
    %       denormalmin - Property is of type 'double'
    %       eps - Property is of type 'double'
    %       exponentbias - Property is of type 'double'
    %       exponentmax - Property is of type 'double'
    %       exponentmin - Property is of type 'double'
    %       realmax - Property is of type 'double'
    %       realmin - Property is of type 'double'
    %       version - Property is of type 'int'
    %       Tag - Property is of type 'string'
    
    %   Thomas A. Bryan
     
    methods  % constructor block
        function q = quantizer(varargin)
            if nargin > 0
                setquantizer(q,varargin{:});
            end
        end  % quantizer
    end  % constructor block
end  % classdef

