function [h, varargout] = designFracDelayFIR(FD, CP)
%designFracDelayFIR Design a band-limited fractional delay FIR filter.
%   The function designs an FIR approximation of an ideal shift filter
%   with a specified fractional (non-integer) delay value FD within [0,1].
%
%   The ideal filter has the frequency response exp(-j*w*(FD+i0)), modeling
%   a band-limited interpolator followed by shifted uniform sampling. It
%   has a flat unit gain, and a constant group delay of FD+i0. Here, i0 is
%   the integer latency, which is the center index of the FIR window,
%   approximately half its length.
%
%   The FIR approximation is obtained by truncating the ideal filter and
%   weighting the truncated filter by a Kaiser window. The Kaiser window
%   is designed to optimize the FIR frequency response, maximizing the
%   combined bandwidths of both gain response and group delay response.
%
%   The function supports two design modes: one to achieve a target FIR
%   length, and the other to achieve a target bandwidth. The design mode
%   is determined by the second function argument, referred to as the
%   control parameter CP.
%
%   To design an FIR of length N>1, call the function with CP set to N.
%
%   Alternatively, to design an FIR whose combined bandwidth is BW<1
%   (specified in normalized frequency units), call the function with CP
%   set to BW. The function then automatically determines the appropriate
%   FIR length for which the combined bandwidth is at least BW.
%
%   h = designFracDelayFIR(FD) designs a fractional delay FIR filter of
%   delay FD and default length 50, giving approximately 0.9 bandwidth.
%
%   h = designFracDelayFIR(FD,N) designs a fractional delay FIR filter
%   of delay FD and length N.
%
%   h = designFracDelayFIR(FD,BW) designs a fractional delay FIR filter
%   of delay FD and a combined bandwidth of at least BW (where BW<0.999).
%
%   [h,i0] = designFracDelayFIR(...) returns the center index of the
%   designed FIR filter. The nominal group delay of the filter is i0+FD.
%
%   [h,i0,bw] = designFracDelayFIR(...) returns the combined bandwidth
%   of the filter h.
%
% % Example 1: Design a fractional delay filter with a delay of 1/3 and
% % length 6. The filter is then tested by shifting the samples of a
% % known function, and then compare the filter's output against the
% % shifted function.
%   FD = 1/3
%   [h,i0] = designFracDelayFIR(FD,6)
%   fdf = dsp.FIRFilter(h);
%   % Generate a sampled sequence
%   f = @(t) 0.1*t.^2+cos(1.3*t);
%   n = (0:9)'; t = linspace(0,9,512);
%   x = f(n);
%   % Plot the filtered sequence against the shifted-function samples,
%   % which are approximately the same
%   hold on
%   plot(t,f(t),"k:",n,x,"ko","LineWidth",1)
%   plot(n,fdf(x),"rx",t,f(t-i0-FD),"r:","LineWidth",1)
%   xlabel('Time')
%   legend(["f(t)","Samples","Filter output","True shifted f(t-i0-FD)"])
%   hold off
%
% % Example 2: Design a fractional delay filter with a delay of 0.786 and
% % combined bandwidth coverage of 80% of the frequency domain. Plot
% % resulting frequency responses (gain and group delay) using fvtool.
%   FD = 0.786
%   [h,i0,bw] = designFracDelayFIR(FD, 0.8);
%   hfv = fvtool(h,"Analysis","magnitude","OverlayedAnalysis","grpdelay");
%   ax = findall(hfv,"type","axes");
%   xline(ax(1),bw, "LineStyle",":", "Color","b","LineWidth",2)
%   yline(ax(1),i0+FD, "LineStyle",":","Color","r","LineWidth",1)
%   yticks(ax(1),[i0, i0+FD,i0+1:i0+9]); yticklabels(ax(1),'auto')
%   legend(ax(1),{'Gain','Bandwidth','Group Delay','Nominal Group Delay'})
%   grid(ax(1),'off');
%
%   See also  dsp.VariableFractionalDelay, dsp.FIRFilter, dsp.Delay,
%   designMultirateFIR.

% Copyright 2020 The MathWorks, Inc.

%#codegen

% Parse and validate arguments
narginchk(1,2);

coder.varsize('bw');

if nargin<2
    % Default FIR length 50, gives about 90% bandwidth coverage
    CP = cast(50,'like',FD);
end

% Maximal allowed FIR length
maxLength = 2^16;

% Maximal allowed bandwidth in BW design mode
maxBW = 0.999;

validateInput(FD,CP,maxLength,maxBW);
CP = cast(CP,'like',FD);

FD = FD(:); % Enforce column vector

% Add some range assertions, to prevent codegen errors when DMA is disabled
assert(CP<=maxLength);
assert(length(FD)<=2^11);

bw = cast([],'like',FD);

% Design the FIR
if CP<1
    % Design by bandwidth
    [h,i0,bw] = designDelayFIRByBandwidth(FD,CP,maxLength);
else
    % Design by length
    [h, i0] = designDelayFIRByLength(FD,CP);
end

if nargout>=2 % Return center index if need be
    varargout{1} = i0;
end

if nargout>=3 % Return combined bandwidth if requested
    if isempty(bw)
        varargout{2} = combinedGDAndGainBandwidth(h, i0+FD);
    else
        varargout{2} = bw;
    end
end
end


function [h,i0,bw] = designDelayFIRByBandwidth(FD, BWmin,maxLength)
    %designDelayFIRByBandwidth Design a fractional delay FIR filter
    % that has a combined bandwidth of at least BWmin.

    % Get an estimated FIR length for the desired minimal combined
    % bandwidth
    N = lengthForMinimalBandwidth(BWmin);

    % For codegen's sake
    coder.varsize('h');

    % Design an FIR for that length
    [h, i0] = designDelayFIRByLength(FD,N);

    % Verify the design, and stretch the length up if needed
    bw = combinedGDAndGainBandwidth(h, i0+FD);
    maxIterations = 10^3; % Maximal iteration cap
    iterations = 1;
    while any(bw<BWmin) && (iterations<= maxIterations) && (N<=maxLength)
        if N>10^2
            N = N+10;
        else
            N = N+2;
        end

        iterations = iterations+1;
        [h, i0] = designDelayFIRByLength(FD,N);
        bw = combinedGDAndGainBandwidth(h,i0+FD);
    end

    if isempty(coder.target)
        % Emit a maximum iteration limit warning
        coder.internal.warningIf(iterations>maxIterations || N>maxLength, 'dsp:designFracDelayFIR:maxiterations');
    end
end

function N = lengthForMinimalBandwidth(bw)
%lengthForMinimalBandwidth Determine the FIR length required for a minimal bandwidth
% Those values were derived empirically.

% For small bandwidth, there are predetermined appropriate lengths

    smallBWcutoffs = [0.0904,  0.3377,   0.4484,    0.5900,  0.6435];
    smallBWlengths = [2, 4, 6, 8, 10];

    smallBWcutoffs = cast(smallBWcutoffs,'like',bw);
    smallBWlengths = cast(smallBWlengths,'like',bw);

    % Determine if bw is in the small bandwidth table
    idx = find(bw<smallBWcutoffs,1,'first');

    if ~isempty(idx)
        % Bandwidth is either in the table or smaller than its first element.
        N = smallBWlengths(idx(1));
    else
        % For a bandwidth over the lookup table, use a rational approximation
        margin = 1-bw;

        p=[-10.39840 6.36660 -0.05257];
        q=[1.00000 -0.00861 0.00000];

        p = cast(p, 'like', bw);
        q = cast(q, 'like', bw);

        N = ceil(polyval(p,margin)/polyval(q,margin));

        % Round down to nearest even, cap at maximum length
        N = min(N-mod(N,2),12000);
    end
    assert(N>=2);
end

function [h,i0] = designDelayFIRByLength(FD, N)
    %designDelayFIRByLength Design a fractional delay convolution filter
    % of a specified length N. The resulting filter has an optimal
    % combined bandwidth across all values of FD.

    % In this Kaiser-based design method, filters of even lengths perform
    % better than filter of lengths. Therefore, if N is odd we design a
    % filter of length N-1 and pad the resulting FIR with a trailing zero.

    h = zeros(1,N,'like',FD);   % Preallocation helps codegen

    N = N-mod(N,2);             % Always design an even length FIR

    i0 = ceil(N/2)-1;           % Center index
    wIndex = (0:N-1)-i0;        % Symmetric window

    beta  = betaByLength(N);    % Kaiser shape parameter optimal for N
    window = kaiser(N,beta)';   % Obtain Kaiser window
    hIdeal = sinc(wIndex-FD);   % Truncated ideal shift filter
    h(:,1:N) = hIdeal.*window;
    h = h./sum(h,2);            % Trim DC gain
end

function beta=betaByLength(N)
    %betaByLength Approximates the optimal Kaiser shape parameter for a
    %   given filter length N.
    %   The resulting Kaiser window gives maximal combined bandwidth
    %   (gain and group delay) across all delay values.
    %   The map of optimal beta values has been obtained empirically.
    %   Values for short filters (N<=4) are obtained by a lookup table.
    %   For longer filters (N>=6, of even lengths) beta is obtained by
    %   a rational expression p(N)/q(N).

    shortFirTable = [...
                    0,...       % 2
                    2.21];      % 4

    p=[11.01299 2395.00455 -6226.46055];
    q=[1.00000 326.73886 1094.40241];

    % Cast to the same type of N (codegen idiosyncrasy)
    shortFirTable = cast(shortFirTable,'like',N);
    p = cast(p,'like',N);
    q = cast(q,'like',N);

    if N<=4 % For N=2,4
        beta = shortFirTable(N/2);
    else % For N>=6
        beta = polyval(p,N)/polyval(q,N);
    end
end

function validateInput(FD, CP,maxLength,maxBW)
    validateattributes(FD, {'single','double'},...
        {'nonempty','finite','real','scalar','nonnegative'...
        '<=',1,},...
        '','FD (fractional delay value)');

    validateattributes(CP, {'numeric'},...
        {'nonempty','finite','real','scalar','positive'},...
        '','CP (control parameter)');

    % Validate CP based on design mode
    if CP>1
        % Validate CP is an integer>=2
        validateattributes(CP, {'numeric'},...
            {'integer','>=',2,'<=',maxLength},...
            '','CP (length)');
    else
        % Validate CP is a fraction up to maximum allowed bandwidth
        validateattributes(CP, {'numeric'},...
            {'real','<',maxBW},...
            '','CP (transition width)');
    end

end
