function B = designMultirateFIR(L,M, varargin)
%designMultirateFIR Design multirate FIR filter.
%   B = designMultirateFIR(L,M) designs a multirate FIR filter with
%   interpolation factor L and decimation factor M. B is the vector of
%   designed FIR coefficients.
%
%   B = designMultirateFIR(L,M,P) designs a multirate FIR filter with
%   interpolation factor L, decimation factor M, and half-polyphase length
%   P (the default is 12). If the first FIR coefficient is zero (B(1) = 0), 
%   the resulting polyphase length is 2P, otherwise the polyphase length 
%   is 2P + 1.
%
%   B = designMultirateFIR(L,M,TW) designs a multirate FIR filter with
%   interpolation factor L, decimation factor M, and normalized transition
%   width TW. TW must be less than 1.
%
%   B = designMultirateFIR(L,M,P,ASTOP) designs a multirate FIR filter
%   with interpolation factor L, decimation factor M, half-polyphase length
%   P, and stopband attenuation Astop in dB (the default is 80 dB).
%
%   B = designMultirateFIR(L,M,TW,ASTOP) designs a multirate FIR filter
%   with interpolation factor L, decimation factor M, normalized transition
%   width TW, and stopband attenuation Astop in dB (the default is 80 dB).
%
%   B = designMultirateFIR(...,'SystemObject',FLAG) returns a
%   vector of filter coefficients B if FLAG is false or a multirate System
%   object if FLAG is true.
%
% % EXAMPLE 1: Design coefficients for an FIR interpolator with
% % interpolation factor equal to 5:
%      b = designMultirateFIR(5,1);
%      fvtool(b,1);
%
% % EXAMPLE 2: Design coefficients for an FIR decimator with decimation
% % factor equal to 3 and half-polyphase length equal to 14:
%      b = designMultirateFIR(1,3,14);
%      fvtool(b,1);
%
% % EXAMPLE 3: Design coefficients for an FIR rate converter with
% % interpolation factor equal to 3, decimation factor equal to 4,
% % transition width equal to 0.02, and stopband attenuation equal to
% % 90 dB:
%      b = designMultirateFIR(3,4,0.02,90);
%      fvtool(b,1);
%
% % EXAMPLE 4: Create dsp.FIRInterpolator System object with interpolation
% % factor equal to 5, transition width equal to 0.01, and stopband
% % attenuation equal to 60 dB:
%      firInterp = designMultirateFIR(5,1,0.01,60,'SystemObject',true);
%      fvtool(firInterp);
%
%   See also FIRHALFBAND, FIRNYQUIST, designMultistageDecimator,
%   designMultistageInterpolator, dsp.SampleRateConverter,
%   dsp.ComplexBandpassDecimator, designFracDelayFIR.

%   Copyright 2015-2021 The MathWorks, Inc.

%#codegen

coder.extrinsic('filterdesign.internal.designNyquistMinOrder');
narginchk(2,6);

% Validate interpolation factor L
validateattributes(L,{'numeric'},...
    {'finite','real','scalar','positive','integer'},...
    '','L');
% Validate decimation factor M
validateattributes(M,{'numeric'},...
    {'finite','real','scalar','positive','integer'},...
    '','M');

L = double(L);
M = double(M);
maxLM = max(L,M);

% Number of polyphase branches to be used
if L > 1
    NPolyBranches = L;
else
    NPolyBranches = M;
end

if nargin <= 2
    % Default value for the number of polyphase branches if not specified,
    % stopband attenuation and system object flag
    P = 12;         % Half the polyphase length
    Astop = 80;     % Stopband Attenuation
    sysobj = false; % Return a system object
else
    % Parse and validate varargin
    [P, Astop, sysobj] = parseAndValidateInput(varargin{:});
end

% return early if both the decimation factor and the interpolation factor
% are equal to 1
if maxLM == 1
    B = 1;
    if isempty(coder.target) && sysobj
        B = dsp.FIRInterpolator(maxLM,B);
    end
    return;
end

% If P input provided is less than one that means Transition
% width (TW) is specified which requires min order design.
% To perform min order design in codegen, inputs must be constant.
if P < 1 && coder.internal.isConst(P)
    % Fold the function expression into constants in the generated code
    R = P * maxLM / 2;
    coder.internal.errorIf((R <= 0 || R >= 1),'dsp:designMultirateFIR:invalidRolloffFactor');
    b = coder.const(@filterdesign.internal.designNyquistMinOrder,maxLM,P,Astop);
    B = L.*b;
else
    % If the half polyphase length is specified.

    N = 2 * P * NPolyBranches;
    if ~isempty(coder.target)
        % Generate error in codegen if half polyphase length provided is less 
        % than 1
        validateattributes(P,{'numeric'},{'>=',1,'integer','finite','real','scalar','positive'},'','P');
        % Maximal allowed FIR filter order
        maxLength = 2^16;
        % Add range assertions, to prevent codegen errors when DMA is disabled
        assert(N <= maxLength);
    end

    if Astop > 50
        beta = 0.1102*(Astop-8.71);
    elseif Astop < 21
        beta = 0;
    else
        beta = (0.5842*(Astop-21)^0.4) +  0.07886*(Astop-21);
    end
    num   = L .* lowpasslband(N,maxLM,kaiser(N+1,beta)); % Passband gain = L
    
    % Determine if the design configuration has a trailing zero 
    if ~(L>1 && M>L && mod(P*L,M)~=0)
        % In this case, the first and last elements of the design are zero. 
        % Remove trailing zero. This makes the FIR length 2*P*NPolyBranches 
        % divisible by L (since NPolyBranches=L). 
        % The polyphase length in this case is 2*P.
        B = num(1:end-1); 
    else
        % In case the first and last elements of the design are nonzero, 
        % leave it as is. The overall length is 2*P*NPolyBranches+1. 
        % The polyphase length in this case is 2*P+1.
        B = num;
    end
end
% return System object if requested
if isempty(coder.target) && sysobj
    multiType = getMultirateFilterType(L,M);
    if P > 1
        switch multiType
            case "interp"
                B    = dsp.FIRInterpolator(L,B);
                fdes = fdesign.interpolator(L,'Nyquist',L,'N,Ast',N,Astop);
            case "decim"
                B    = dsp.FIRDecimator(M,B);
                fdes = fdesign.decimator(M,'Nyquist',M,'N,Ast',N,Astop);
            case "src"
                B    = dsp.FIRRateConverter(L,M,B);
                fdes = fdesign.rsrc(L,M,'Nyquist',maxLM,'N,Ast',N,Astop);
        end
        fmethod      = fdfmethod.kaiserhbastop;
        measurements = [];
        designmethod = 'kaiserwin';
        % set meta data on object to enable fvtool to plot the
        % specification mask
        setMetaData(B,fdes, fmethod, measurements, designmethod);
    else
        switch multiType
            case "interp"
                f = fdesign.interpolator(L,'Nyquist',L,P,Astop);
            case "decim"
                f = fdesign.decimator(M,'Nyquist',M,P,Astop);
            case "src"
                f = fdesign.rsrc(L,M,'Nyquist',maxLM,P,Astop);
        end
        B = design(f,'SystemObject',true);
    end
end

end
%--------------------------------------------------------------------------
function [P, Astop, sysobjValue] = parseAndValidateInput(varargin)
% default values
P = 12;        % Half the polyphase length
Astop = 80;    % Stopband Attenuation
sysobjProp  = 'SystemObject';
sysobjValue = false; % Return a system object
% Number of varargin inputs
numInputs = numel(varargin);

% Parse input values
if(numInputs == 1)
    % If the number of varargin arguments is 1, then the inputs is P/TW
    P = double(varargin{1});    
end

if(numInputs == 2)
    % If the number of varargin arguments is 2, then the inputs are either
    % P/TW and Astop or 'SystemObject' and flag 
    if(ischar(varargin{1}) || isstring(varargin{1}))
        sysobjProp = varargin{1};
        sysobjValue = varargin{2};
    else
        P = double(varargin{1});
        Astop = double(varargin{2});
    end
end

if (numInputs == 3)
    % If the number of varargin arguments is 3, then the inputs are P/TW
    % 'SystemObject', flag 
    P = double(varargin{1});
    sysobjProp = varargin{2};
    sysobjValue = varargin{3};
end

if (numInputs == 4)
    % If the number of varargin arguments is 4, then the inputs are P/TW,
    % Astop, 'SystemObject', flag
    P    = double(varargin{1});
    Astop = double(varargin{2});
    sysobjProp = varargin{3};
    sysobjValue = varargin{4};
end
% validate system object property value pair
coder.internal.errorIf(~strcmpi(sysobjProp,'SystemObject'),...
    'dsp:designMultirateFIR:invalidPropValueArgs');

% validate that specified P/TW are positive scalars
validateattributes(P,{'numeric'},...
    {'finite','real','scalar','positive'},...
    '','P');
if(P >= 1)
    % If the provided P >= 1, validate P is an integer
    validateattributes(P,{'numeric'},...
        {'integer'},...
        '','P');
end
% validate that specified Astop is nonnegative scalar
validateattributes(Astop,{'numeric'},...
    {'finite','real','scalar','nonnegative'},...
    '','Astop');

if isempty(coder.target)
    validateattributes(sysobjValue,{'logical','numeric'}, {'scalar'},'','Flag');
else
    % In codegen mode, returning a system object is not supported
    coder.internal.errorIf(sysobjValue, ...
        'dsp:designMultirateFIR:sysObjCodeGenUnsupported');
end
end
%--------------------------------------------------------------------------
function multiType = getMultirateFilterType(L,M)
if L > 1
    if M > 1
        multiType = "src";
    else
        multiType = "interp";
    end
else
    multiType = "decim";
end
end

