function d = designMultistageDecimator(M,varargin)
%designMultistageDecimator Design multistage decimator
%   C = designMultistageDecimator(M) designs a multistage decimator that
%   has an overall decimation M. M is a positive integer. In order for C to
%   be multistage, M must not be a prime number.
%
%   Note: the design can take a while if M has many factors.
%
%   C = designMultistageDecimator(M,Fs,TW) designs a multistage
%   decimator with sampling rate Fs and transition width TW. If not
%   specified, Fs defaults to 48000 Hz and TW defaults to 0.2*Fs/M. TW
%   must be less than Fs/M. Sampling rate in this case refers to the input
%   sampling rate, i.e. the sampling rate of the signal prior to the
%   multistage decimator.
%
%   The multistage decimator will have a cutoff frequency of Fs/(2*M).
%
%   C = designMultistageDecimator(M,Fs,TW,Astop) specifies a minimum
%   attenuation of Astop decibels for the resulting design.
%
%   C = designMultistageDecimator(...,'NumStages',N) designs a decimator
%   with N stages. M must be able to factor into at least N factors (not
%   counting 1 or M as factors). If NumStages is not specified or if N is
%   set to 'auto', the design algorithm determines the number of stages
%   that result in the lowest number of multiplications per input sample.
%
%   C = designMultistageDecimator(...,'MinTotalCoeffs',Flag) specifies
%   whether to choose the configuration that minimizes the total number of
%   coefficients instead of the one that minimizes multiplications per
%   input sample. Flag is a logical and it defaults to false.
%
%   C = designMultistageDecimator(...,'CostMethod',Method) specifies
%   whether to 'estimate' the filter order required for each stage or
%   to 'design' each stage and compute the filter order. Estimates are much
%   faster but may lead to suboptimal designs. Method defaults to
%   'estimate'.  
%
%   C = designMultistageDecimator(...,'CostTolerance',Tol) specifies a
%   tolerance Tol as a positive scalar. The tolerance is used to determine
%   the multistage configuration. When multiple configurations result in
%   the same lowest number of multiplications per input sample (within the
%   tolerance specified), the configuration that yields the lowest number
%   of coefficients overall is chosen. Tol defaults to 1e-6.
%
% % EXAMPLE 1: Design a multistage decimator that decimates by 12.
% % Design another decimator that decimates by 12 using 2 stages.
%      c3 = designMultistageDecimator(12); % Results in 3 stages
%      c2 = designMultistageDecimator(12,'NumStages',2);
%      cost(c3) % 3 stages results in fewer multiplications per sample
%      cost(c2)
%
% % EXAMPLE 2: Design a multistage decimator that decimates by 8,
% % has a transition width of 100 Hz and an input sampling rate of 100 kHz.
% % The design is set to minimize the total number of coefficients. 
%      c = designMultistageDecimator(8,100e3,100,'MinTotalCoeffs',true);
%      fvtool(c);
%
%   See also designMultistageInterpolator, designMultirateFIR,
%   dsp.SampleRateConverter, dsp.ComplexBandpassDecimator,
%   dsp.FilterCascade

%   Copyright 2021 The MathWorks, Inc.

validateattributes(M,{'numeric'},{'scalar','real',...
    'finite','positive','integer','>',1},...
    'designMultistageDecimator','Decimation Factor',1);

[nobj,NumStages] = filterdesign.internal.designMultistageParser(M,varargin{:});

if (ischar(NumStages) || isstring(NumStages)) && strcmpi(NumStages,'auto')
    d = dynprogDesign(nobj);
else
    d = design(nobj,NumStages);
end
