function d = designMultistageInterpolator(L,varargin)
%designMultistageInterpolator Design multistage interpolator
%   C = designMultistageInterpolator(L) designs a multistage interpolator
%   that has an overall interpolation L. L is a positive integer. In order
%   for C to be multistage, L must not be a prime number.
%
%   Note: the design can take a while if L has many factors.
%
%   C = designMultistageInterpolator(L,Fs,TW) designs a multistage
%   interpolator with sampling rate Fs and transition width TW. If not
%   specified, Fs defaults to 48000 Hz and TW defaults to 0.2*Fs/L. TW
%   must be less than Fs/L. Sampling rate in this case refers to the output
%   sampling rate, i.e. the sampling rate of the signal after the
%   multistage interpolator.
%
%   The multistage interpolator will have a cutoff frequency of Fs/(2*L).
%
%   C = designMultistageInterpolator(L,Fs,TW,Astop) specifies a minimum
%   attenuation of Astop decibels for the resulting design.
%
%   C = designMultistageInterpolator(...,'NumStages',N) designs an
%   interpolator with N stages. L must be able to factor into at least N
%   factors (not counting 1 or L as factors). If NumStages is not specified
%   or if N is set to 'auto', the design algorithm determines the number of
%   stages that result in the lowest number of multiplications per input
%   sample.
%
%   C = designMultistageInterpolator(...,'MinTotalCoeffs',Flag) specifies
%   whether to choose the configuration that minimizes the total number of
%   coefficients instead of the one that minimizes multiplications per
%   input sample. Flag is a logical and it defaults to false.
%
%   C = designMultistageInterpolator(...,'CostMethod',Method) specifies
%   whether to 'estimate' the filter order required for each stage or
%   to 'design' each stage and compute the filter order. Estimates are much
%   faster but may lead to suboptimal designs. Method defaults to
%   'estimate'.
%
%   C = designMultistageInterpolator(...,'CostTolerance',Tol) specifies a
%   tolerance Tol as a positive scalar. The tolerance is used to determine
%   the multistage configuration. When multiple configurations result in
%   the same lowest number of multiplications per input sample (within the
%   tolerance specified), the configuration that yields the lowest number
%   of coefficients overall is chosen. Tol defaults to 1e-6.
%
% % EXAMPLE 1: Design a multistage interpolator that interpolates by 12.
% % Design another interpolator that interpolates by 12 using 2 stages.
%      c3 = designMultistageInterpolator(12); % Results in 3 stages
%      c2 = designMultistageInterpolator(12,'NumStages',2);
%      cost(c3) % 3 stages results in fewer multiplications per sample
%      cost(c2)
%
% % EXAMPLE 2: Design a multistage interpolator that interpolates by 8,
% % has a transition width of 100 Hz and an output sampling rate of 10 kHz.
% % The design is set to minimize the total number of coefficients. 
%      c = designMultistageInterpolator(8,10e3,100,'MinTotalCoeffs',true);
%      fvtool(c);
%
%   See also designMultistageDecimator, designMultirateFIR,
%   dsp.SampleRateConverter, dsp.ComplexBandpassDecimator,
%   dsp.FilterCascade

%   Copyright 2021 The MathWorks, Inc.

validateattributes(L,{'numeric'},{'scalar','real',...
    'finite','positive','integer','>',1},...
    'designMultistageInterpolator','Interpolation Factor',1);

[nobj,NumStages] = filterdesign.internal.designMultistageParser(L,varargin{:});
nobj.MultirateType        = 'interpolator';

if (ischar(NumStages) || isstring(NumStages)) && strcmpi(NumStages,'auto')
    d = dynprogDesign(nobj);
else
    d = design(nobj,NumStages);
end
