function [h,WARN] = firceqrip(N,Fo,DEV,varargin)
%FIRCEQRIP  Constrained equiripple FIR filter design.
%   B = FIRCEQRIP(N,Fo,DEV) designs length N+1 filter coefficients for a
%   lowpass linear-phase FIR filter with cut-off frequency Fo. The
%   2-element vector DEV = [d1 d2] specifies the maximum deviation or
%   ripple (in linear units) in the pass-band and stop-band. At the cut-off
%   frequency Fo the frequency response magnitude is 0.5. Fo should be
%   between 0 and 1 (0 < Fo < 1).
%
%   This function produces the same equiripple lowpass filters as the FIRPM
%   (Parks-McClellan) algorithm, however, the specifications are supplied
%   differently.
%
%   B = FIRCEQRIP(...,'slope',R) designs a filter with a nonequiripple
%   stop-band. R is the attenuation slope dB/(pi rad/sample) (R > 0) [try R
%   = 10].
%
%   B = FIRCEQRIP('minorder',[Fp Fst],DEV) designs filter with the minimum
%   number of coefficients required to meet the deviations in DEV = [d1 d2]
%   while having a transition width no greater than Fst-Fp (the stopband
%   and passband edges respectively). You can specify 'mineven' or 'minodd'
%   instead of 'minorder' to design minimum even order (odd length) or
%   minimum odd order (even length) filters respectively.
%
%   B = FIRCEQRIP(...,'passedge') designs a filter where Fo is the passband
%   edge. B = FIRCEQRIP(...,'stopedge') designs a filter where Fo is the
%   stopband edge.
%
%   B = FIRCEQRIP(...,'high') designs a highpass filter.
%
%   B = FIRCEQRIP(...,'min') designs a minimum-phase FIR filter. Minimum
%   phase designs do not support minimum order.
%
%   B = FIRCEQRIP(...,'invsinc',C) designs a lowpass filter with a
%   pass-band having the shape of an inverse-sinc function, 1/sinc(C*f). If
%   C is a two-element vector, C = [c p], then the shape of the pass-band
%   is 1/sinc(c*f)^p. [c < 1/wo]. Inverse sinc designs do not support 
%   minimum order.
%
%   B = FIRCEQRIP(...,'invdiric',C) designs a lowpass filter with a
%   pass-band having the shape of an inverse-Dirichlet-sinc function
%   C*sin(f/2)/sin(C*f/2). If C is a two-element vector, C = [c p], then
%   the shape of the pass-band is [c*sin(f/2)/sin(c*f/2)]^p. If C is a
%   three-element vector, C = [c p r], then the shape of the pass-band is
%   [r*c*sin(f/(2*r))/sin(c*f/2)]^p. Inverse-Dirichlet-sinc designs do not 
%   support minimum order.
%
%   Several trailing arguments can be used simultaneously. The order of the
%   optional trailing arguments is not important.
%
%   Note: The linear-phase lowpass [highpass] filters will be Type I or II
%   [Type I or IV] depending on the parameter N specified by the user.
%
%   [B,F] = FIRCEQRIP(...) returns F set to true if the design fails to
%   converge. If Fo is too small or too large, or if c or p is too large,
%   then the supplied specifications will be unfeasible or the design
%   algorithm may fail.
%
%   % Examples
%   b = firceqrip(30,0.4,[0.05 1e-3]);                 fvtool(b) %  60 dB
%   b = firceqrip('minorder',[0.35 0.45],[0.02 1e-4]); fvtool(b) %  80 dB
%   b = firceqrip(50,0.4,[0.05 1e-5]);                 fvtool(b) % 100 dB
%   b = firceqrip(100,0.3,[0.05 1e-4],'passedge');           fvtool(b)
%   b = firceqrip(70,0.4,[0.05 1e-3],'slope',20,'stopedge'); fvtool(b)
%   b = firceqrip(40,0.4,[0.05 1e-2],'stopedge','min');      fvtool(b)
%   b = firceqrip(70,0.4,[0.05 1e-4],'invsinc',[2 1.5]);     fvtool(b)
%   b = firceqrip(70,0.4,[0.05 1e-4],'invdiric',[2 1.5 4]);  fvtool(b)
%
%   See also FIRGR, FIRCLS, FIRLS, IIRLPNORM, IIRLPNORMC, IIRGRPDELAY,
%   IFIR, FIRNYQUIST, FIRHALFBAND.

%   Author(s): Ivan Selesnick
%   Copyright 1999-2018 The MathWorks, Inc.


narginchk(3,11);

[N,Fo,DEV,isHighpass,isMinOrd,s] = parsefirceqrip(N,Fo,DEV,varargin{:});

if isMinOrd

    % Estimate initial order
    if isHighpass
        M = firpmord( Fo, [0 1], DEV, 2 );
    else
        M = firpmord( Fo, [1 0], DEV, 2 );
    end        
    if M < 3
        % firpmord can return 0
        M = 3;
    end
    
    if strcmpi(N,'mineven') && rem(M,2)
        M = M + 1;
    end
    
    if strcmpi(N,'minodd') && ~rem(M,2)
        M = M + 1;
    end
        
    it            = filterdesign.internal.minordIterator;
    it.designFCN  = @firceqripDesign;
    it.testFCN    = @firceqripMeasure;
    if isHighpass
        it.designArgs = [Fo(1),DEV,'stopedge',varargin];
    else
        it.designArgs = [Fo(1),DEV,'passedge',varargin];
    end
    it.testArgs   = {s};
    it.current    = M;
    it.constraint = N; % Set before increment
    it.increment  = ceil(.1*M);
    
                
    [h,WARN] = findMinOrd(it);
else
    [h,WARN] = firceqripDesign(N,Fo,DEV,varargin{:});
end
%--------------------------------------------------------------------------
function pass = firceqripMeasure(h,s)

s.hd.Numerator = h;
m  = measure(s.hd,s.fd);

% Pass if measurements are with 1e-3 dB
tol = 1e-3;
X1 = s.Ap;
X2 = s.Ast;
Y1 = m.Apass;
Y2 = m.Astop;
if ((abs(X1-Y1)/min(abs(X1), abs(Y1)) < tol) || (Y1 < X1)) && ...
        ((abs(X2-Y2)/min(abs(X2), abs(Y2)) < tol) || (Y2 > X2))
    pass = true;
else
    pass = false;
end
%--------------------------------------------------------------------------
function [N,Fo,DEV,isHighpass,isMinOrd,s] = parsefirceqrip(N,Fo,DEV,varargin)

isMinOrd = false;
if ischar(N) || isstring(N)
    N = validatestring(N,{'minorder','mineven','minodd'});
    isMinOrd = true;
else
    validateattributes(N,{'numeric'},{'real','positive','finite','integer',...
        'scalar'},'firceqrip','N',1);
    N = double(N);
end

if ischar(N) || isstring(N)
    % Fo must have both passband and stopband frequencies
    validateattributes(Fo,{'numeric'},{'real','positive','finite','<',1,'2d','numel',2},...
        'firceqrip','Fo',2);
else
    validateattributes(Fo,{'numeric'},{'real','positive','finite','<',1,'scalar'},...
        'firceqrip','Fo',2);
end
Fo = double(Fo);

validateattributes(DEV,{'numeric'},{'real','positive','finite','<',1,'2d','numel',2},...
    'firceqrip','DEV',3);
DEV = double(DEV);

isHighpass = false;
if any(strncmp('high',varargin,4))
    isHighpass = true;
end

if isMinOrd
    s.hd = dsp.FIRFilter;
    
    if isHighpass
        Rp  = DEV(2);
        Rst = DEV(1);
    else
        Rp  = DEV(1);
        Rst = DEV(2);
    end
    
    s.Ap  = convertmagunits(Rp,'linear','db','pass');
    s.Ast = convertmagunits(Rst,'linear','db','stop');
    
    if isHighpass
        s.fd  = fdesign.highpass(Fo(1),Fo(2),s.Ast,s.Ap);
    else
        s.fd  = fdesign.lowpass(Fo(1),Fo(2),s.Ap,s.Ast);
    end
else
    s.hd = [];
    s.fd = [];
end

if any(strncmp('min',varargin,3))    
    if isMinOrd
        error(message('dsp:firceqrip:NoMinOrderMinPhase'));
    end
end

if any(strncmp('inv',varargin,3))
    if isMinOrd
        error(message('dsp:firceqrip:NoMinOrderInvSinc'));
    end
end


