function b = fireqint(N,L,alpha,W)
%FIREQINT Equiripple interpolation FIR filter design.
%   B = FIREQINT(N,L,ALPHA) designs an FIR equiripple filter useful for
%   interpolation purposes.
%
%   N is the filter order (must be an integer). L is the interpolation
%   factor (must be an integer). ALPHA is the bandlimitedness factor, same
%   as in INTFILT. The signal to be interpolated is assumed to have zero
%   (negligible) power in the frequency band between ALPHA*Pi and Pi. ALPHA
%   must therefore be a positive scalar between 0 and 1. FIREQINT will
%   treat such band as a "don't care" band.
%
%   B = FIREQINT(N,L,ALPHA,W) allows for the specification of weights in
%   vector W. The number of weights is given by 1 + FLOOR(L/2). The use of
%   weights enable having different attenuations in different parts of the
%   stopband as well as the ability to adjust the compromise between
%   passband ripple and stopband attenuation.
%
%   B = FIREQINT('minorder',L,ALPHA,DEV) allows for the design of a filter
%   of minimum-order that meets the design specifications. DEV is a vector
%   of maximum deviations or ripples (in linear units) from the ideal
%   response. The length of DEV is given by 1 + FLOOR(L/2).
%
%   B = FIREQINT({'minorder',INITORD},L,ALPHA,DEV) allows for the
%   specification of an initial estimate of the filter order.
%
%   EXAMPLE: Design a minimum order interpolation filter for L = 6 and
%            % ALPHA = 0.8. A vector of ripples must be supplied.
%            b = fireqint('minorder',6,.8,[0.01 .1 .05 .02]);
%            firinterp = dsp.FIRInterpolator('InterpolationFactor',6,...
%               'Numerator',b); % Create a polyphase interpolator filter
%            zerophase(firinterp); 
%
%   See also FIRNYQUIST, FIRGR, FIRHALFBAND, INTFILT, FIRLS, MFILT.

%   Author(s): R. Losada
%   Copyright 1999-2020 The MathWorks, Inc.

% Check for arguments data types and number of arguments
narginchk(3,4);

validateattributes(L,'double', {'scalar','nonempty','real','nonnan','finite'}, ...
    'fireqint','L');

validateattributes(alpha,'double', {'scalar','nonempty','real','nonnan','finite'}, ...
    'fireqint','alpha');

% Some error checking
if (iscell(N) || dspIsChar(N)) && nargin < 4
    error(message('dsp:fireqint:nodevs'));
end

if dspIsChar(N)
    N = char(N);
end

if iscell(N)
    if dspIsChar(N{1})
        N{1} = char(N{1});
    end
    % Check intial order is an integer
    if N{2} ~= round(N{2})
        error(message('dsp:fireqint:nonIntInitOrd'));
    end
end

if ~iscell(N)
    if N ~= round(N)
        error(message('dsp:fireqint:nonIntOrd'));
    end
end

if L ~= round(L)
	error(message('dsp:fireqint:nonIntL'));
end

if alpha <= 0  || alpha >= 1
	error(message('dsp:fireqint:alpha0t1'));
end

if nargin > 3 
    validateattributes(W,'double',{'nonempty','real','nonnan','finite'}, ...
    'fireqint','W');
    if(length(W) ~= floor(L/2) + 1)
        error(message('dsp:fireqint:wronglengthw'));
    end
end

% Determine length of F
lf = L+2;
lastf = 1;
if rem(L,2)
    lastf=[];
end

fpartialedges = (alpha+0:2:lf-2)/L;
fpartialedges2 = (-alpha+2:2:lf-2)/L;

% Determine frequency vector
F = sort([0,fpartialedges,fpartialedges2,lastf]);

% Determine magnitude vector
A = [1 1 zeros(1,length(F)-2)];

if nargin < 4
    W = ones(1,length(A)/2); 
end

% If L is odd, estimate the order with the previous even factor first
if rem(L,2) && dspIsChar(N)
    btemp = fireqint(char(N),L-1,alpha,W);
    N = {'minorder',length(btemp)-1};
end
    
b = L*firgr(N,F,A,W/L);

