function [num, den, allpassnum, allpassden] = iirlp2mbc(b, a, wo, wt)
%IIRLP2MBC IIR lowpass to complex M-band frequency transformation.
%   [Num,Den,AllpassNum,AllpassDen] = IIRLP2MBC(B,A,Wo,Wc) returns
%   numerator and denominator coefficients, NUM and DEN respectively, of
%   the transformed complex M-band digital filter. It applies an Mth-order
%   real lowpass to complex multi-bandpass frequency transformation, where
%   M is the number of times an original feature is replicated in the
%   target filter. It also returns the numerator, ALLPASSNUM, and the
%   denominator, ALLPASSDEN, of the allpass mapping filter. The prototype
%   lowpass filter is given with a numerator specified by B and the
%   denominator specified by A.
%
%   Inputs:
%     B          - Numerator of the prototype lowpass filter provided as a
%                  vector, or a as a matrix of coefficients for filter   
%                  implemented using second or higher order sections.
%     A          - Denominator of the prototype lowpass filter provided as 
%                  a vector, or a as a matrix of coefficients for filter   
%                  implemented using second or higher order sections.
%     Wo         - Frequency value to be transformed from the prototype 
%                  filter. The frequency should be normalized to be between
%                  0 and 1, with 1 corresponding to half the sample rate.
%     Wc         - Desired frequency location in the transformed target 
%                  filter. The frequency should be normalized to be between
%                  -1 and 1, with 1 corresponding to half the sample rate.
%   Outputs:
%     Num        - Numerator of the target filter
%     Den        - Denominator of the target filter
%     AllpassNum - Numerator of the mapping filter
%     AllpassDen - Denominator of the mapping filter
%
%   Example:
%        [b, a]     = ellip(3, 0.1, 30, 0.409); 
%        [num1, den1] = iirlp2mbc(b, a, 0.5, [2 4 6 8]/10);
%        ss = tf2sos(b,a);
%        [num2, den2] = iirlp2mbc(ss(:,1:3),ss(:,4:6), 0.5, -[8 6 4 2]/10);
%        fos = dsp.FourthOrderSectionFilter('Numerator',num2,...
%           'Denominator',den2);
%        fvtool(b, a, num1, den1,fos);
%
%   See also IIRFTRANSF, ALLPASSLP2MBC and ZPKLP2MBC.

%   References:
%     [1] Krukowski A. and I. Kale, "High-order complex frequency transformations",
%         Internal report No. 27/2001, Applied DSP and VLSI Research Group,
%         University of Westminster.

%   Author(s): Dr. Artur Krukowski, University of Westminster, London, UK.
%   Copyright 1999-2021 The MathWorks, Inc.

%#codegen
% --------------------------------------------------------------------

% Check for number of input arguments
narginchk(4,4);

% Calculate the mapping filter
[allpassnum, allpassden] = allpasslp2mbc(wo, wt);

% Perform the transformation
[num, den]               = iirftransf(b, a, allpassnum, allpassden);