function [num, den, allpassnum, allpassden] = iirlp2xc(b, a, wo, wt)
%IIRLP2XC IIR lowpass to complex N-point frequency transformation.
%   [Num,Den,AllpassNum,AllpassDen] = IIRLP2XC(B,A,Wo,Wt) returns the
%   numerator and denominator coefficients, NUM and DEN respectively, of
%   the transformed complex N-point digital filter. It applies an Nth-order
%   real lowpass to complex multipoint frequency transformation, where N is
%   the number of features being mapped. It also returns the numerator,
%   ALLPASSNUM , and the denominator, ALLPASSDEN, of the allpass mapping
%   filter.  The prototype lowpass filter is given with the numerator
%   specified by B and the denominator specified by A.
%
%   Inputs:
%     B          - Numerator of the prototype lowpass filter provided as a
%                  vector, or a as a matrix of coefficients for filter   
%                  implemented using second or higher order sections.
%     A          - Denominator of the prototype lowpass filter provided as 
%                  a vector, or a as a matrix of coefficients for filter   
%                  implemented using second or higher order sections.
%     Wo         - Frequency value to be transformed from the prototype 
%                  filter. The frequency should be normalized to be between
%                  0 and 1, with 1 corresponding to half the sample rate.
%     Wt         - Desired frequency location in the transformed target 
%                  filter. The frequency should be normalized to be between
%                  -1 and 1, with 1 corresponding to half the sample rate.
%   Outputs:
%     Num        - Numerator of the target filter
%     Den        - Denominator of the target filter
%     AllpassNum - Numerator of the mapping filter
%     AllpassDen - Denominator of the mapping filter
%
%   Example:
%        [b, a]     = ellip(3, 0.1, 30, 0.409); 
%        [num1, den1] = iirlp2xc(b, a, [-0.5 0.5], [-0.25 0.25]);
%        ss = tf2sos(b,a);
%        [num2, den2] = iirlp2xc(ss(:,1:3),ss(:,4:6),...
%           [-0.5 0.5], [-0.15 0.15]);
%        fvtool(b, a, num1, den1,[num2 den2]);
%
%   See also IIRFTRANSF, ALLPASSLP2XC and ZPKLP2XC.

%   References:
%     [1] Krukowski A. and I. Kale, "High-order complex frequency transformations",
%         Internal report No. 27/2001, Applied DSP and VLSI Research Group,
%         University of Westminster.

%   Author(s): Dr. Artur Krukowski, University of Westminster, London, UK.
%   Copyright 1999-2021 The MathWorks, Inc.

%#codegen
% --------------------------------------------------------------------

% Check for number of input arguments
narginchk(4,4);

% Calculate the mapping filter
[allpassnum, allpassden] = allpasslp2xc(wo, wt);

% Perform the transformation
[num, den]               = iirftransf(b, a, allpassnum, allpassden);
