function [s,g,B,A] = iirparameq(varargin)
%IIRPARAMEQ IIR biquad digital parametric equalizer filter design.
%   [SOS,SV] = IIRPARAMEQ(N,G,Wo,BW) designs an Nth order IIR parametric
%   equalizer with gain G, center frequency Wo, and bandwidth BW. The
%   filter is designed in biquad form and its coefficients are returned in
%   the second-order section matrix SOS. SV is a vector of scale values
%   between each biquad stage.
%
%   N must be an even positive integer. G is the equalizer gain in dB and
%   must be a real scalar.
%
%   Wo and BW must real scalars and lie in the closed interval [0,1], with
%   1.0 corresponding to pi radians/sample.
%
%   The bandwidth is defined at 10*log10((1 + G2)/2), where G2 is related
%   to the filter gain by G2 = 10^(G/10). For example, if the filter gain
%   is 3 dB, the bandwidth specified will occur at 1.754 dB.
%
%   The bandwidth BW is related to the Q-factor of a filter by BW = Wo/Q.
%
%   [SOS,SV,B,A] = IIRPARAMEQ(N,G,Wo,BW) return additionally a matrix B of
%   numerator fourth-order sections and a matrix A of denominator
%   fourth-order sections. These can be used in lieu of a biquad
%   implementation and are particularly useful for the case Wo = 0.5.
% 
%   EXAMPLE 1:
%      % Design two equalizers centered at 100 Hz and 1000 Hz respectively,
%      % both with a Gain of 5 dB and Q-factor of 1.5 for a system running
%      % at 48 kHz. Plot the magnitude response of both filters using a log
%      % scale.
%      Fs  = 48e3;
%      N   = 6;
%      G   = 5;
%      Q   = 1.5;
%      Wo1 = 100/(Fs/2);
%      Wo2 = 1000/(Fs/2);
%      BW1 = Wo1/Q;
%      BW2 = Wo2/Q;
%      [SOS1,SV1] = iirparameq(N,G,Wo1,BW1);  
%      [SOS2,SV2] = iirparameq(N,G,Wo2,BW2);  
%      BQ1 = dsp.BiquadFilter('SOSMatrix',SOS1,'ScaleValues',SV1);
%      BQ2 = dsp.BiquadFilter('SOSMatrix',SOS2,'ScaleValues',SV2);
%      fvtool(BQ1,BQ2,'Fs',Fs,'FrequencyScale','Log');
%
%   EXAMPLE 2:
%      % Design a higher order notch filter and compare to a traditional
%      % second-order notch filter designed with IIRNOTCH. Filters
%      % intersect at the -3 dB point.
%      Fs  = 44.1e3;
%      N   = 8;
%      G   = -inf;
%      Q   = 1.8;
%      Wo  = 60/(Fs/2); % Notch at 60 Hz
%      BW  = Wo/Q; % Bandwidth will occur at -3 dB for this special case
%      [SOS1,SV1] = iirparameq(N,G,Wo,BW);  
%      [NUM,DEN]  = iirnotch(Wo,BW); 
%      SOS2 = [NUM,DEN];
%      BQ1 = dsp.BiquadFilter('SOSMatrix',SOS1,'ScaleValues',SV1);
%      BQ2 = dsp.BiquadFilter('SOSMatrix',SOS2);
%      hfv = fvtool(BQ1,BQ2,'Fs',Fs,'FrequencyScale','Log');
%      ax = findobj(hfv,'type','axes');
%      legend(ax,'8th order notch filter','2nd order notch filter');
%
%   See also IIRPEAK, IIRNOTCH, IIRCOMB

%   Copyright 2014-2021 The MathWorks, Inc.

%   References:
%     [1] Sophocles J. Orfanidis, High-Order Digital Parametric Equalizer
%         Design. J. Audio Eng. Soc., vol.53, pp. 1026-1046, November 2005.

%#codegen

% Input validation
narginchk(1, 4);
if isempty(coder.target)
    warning(message('dsp:iirparameq:iirparameq_NotSupported'));
end
[s,g,B,A] = iirparameqDesign(varargin{:});
