function bw = combinedGDAndGainBandwidth(h,nominalGD)
%combinedGDAndGainBandwidth Compute the combined bandwidth of an FIR h.
% The combined bandwidth is defined as the minimum between group delay
% bandwidth and the gain bandwidth.

% Copyright 2020 The MathWorks, Inc.

%#codegen

    % Default tolerance of 1%
    tol = cast(0.01,'like',nominalGD);

    nominalGD = nominalGD(:);  % Enforce column vector
    assert(size(h,1)==length(nominalGD));
    L = size(h,2); % FIR Length

    % Compute the resolution of the frequency domain, which has to be at
    % least as much as length(h).
    wres = max(4096, 2^(ceil(log2(L))));

    % Cap lengths (in case dynamic memory allocation is disabled)
    assert(wres<=2^20);
    [H, g] = gainAndGroupDelay(h, wres);

    % Group delay bandwidth is defined as the lowest frequency at which
    % the group delay error is larger than tol.
    bwGD  = band(g-nominalGD,tol);

    % Gain delay bandwidth is computed similarly to the group delay
    % bandwidth
    bwGain = band(H-1,tol);

    % The combined bandwidth is the minimum of the two bandwidths
    bw = min(bwGain, bwGD);
end

function bw = band(u,tol)
    %band Find the first normalized index for which |u(w)| >= tol.
    % The returned value, bw, is essentially the bandwidth of u.
    % In other words, we are guaranteed that |u(w)|<tol for w in [0,bw]

    N = size(u,2);      % Length
    nChns = size(u,1);  % number of channels

    bw = zeros(nChns,1,'like',u);

    for k=1:nChns
        idx = find(abs(u(k,:))>=tol,1,'first');

        if isempty(idx)
            % If |u(w)|<tol for all indices, return 1.
            bw(k) = 1; % Full bandwidth
        else
            % Otherwise, return the normalized index (between 0..1)
            bw(k) = (idx-1)/(N-1);
        end
    end
end

function [H,g] = gainAndGroupDelay(h,N)
    %gainAndGroupDelay Compute the gain response and group delay of an
    % FIR filter h with N-points frequency granularity. Multichannel FIR
    % is supported (each row corresponds to a channel)

    L = size(h,2); % FIR Length

    assert(L<=N); % Ensure sufficient frequency resolution

    % Approximate DTFT using FFT
    H = fft(h,2*N,2);

    % Group delay approximation by Smith's method
    rh = h.*(0:L-1);    % Ramp-weighted sequence
    rH = fft(rh,2*N,2);
    g = real(rH./H);

    % Trim half the FFT (the rest is a symmetric image)
    H = abs(H(:,1:N));
    g = g(:,1:N);
end
