function [num,den] = polyallpasssub(b,a,allpassnum,allpassden)
%POLYALLPASSSUB   Substitute delays in polynomials with allpass filters.

%   Author(s): R. Losada
%   Copyright 1999-2021 The MathWorks, Inc.

%#codegen

% Remove possible trailing zeros and get polynomial lengths
M = find(b~=0, 1, 'last' );
N = find(a~=0, 1, 'last');

coder.internal.errorIf(isempty(M) || isempty(N) ,'dsp:iirftransf:InvalidCoefficients');

% Codegen expects scalar 
b = b(1:M(1)); 
a = a(1:N(1)); 

% Compute temporary numerator
tempnum = newpoly(b,allpassnum,allpassden,M(1),a);

% Compute temporary denominator
tempden = newpoly(a,allpassnum,allpassden,N(1),a);

% Now include common denominators
num = conv(tempnum,polypow(allpassden,N(1)-M(1)));
den = conv(tempden,polypow(allpassden,M(1)-N(1)));

num = num/den(1);
den = den/den(1);

%-------------------------------------------------------------------
function temppoly = newpoly(b,allpassnum,allpassden,M,den)
% Compute new polynomial after substitution


% For each coefficient, we will have a resulting polynomial after we
% substitute, form each polynomial and then add them all up to compute
% the new numerator or denominator.

% Compute length of new polynomial
temppolyLength = (length(allpassnum)-1)*(M-1) + 1;

% Obtain dataType
if isreal(b) && isreal(allpassnum) && isreal(allpassden) && isreal(den)
    pCoeff = 1;
else
    pCoeff = 1i;
end         
if isa(b,'single') || isa(allpassnum,'single') || isa(allpassden,'single')|| isa(den,'single')
    pCoeffType = cast(pCoeff,'single');
else
    pCoeffType = pCoeff;
end

temppolyMat = zeros(M,temppolyLength,'like',pCoeffType);
for n = 1:M
	temppolyMat(n,:) = b(n).*conv(polypow(allpassden,M-n),...
		polypow(allpassnum,n-1)); 
end

temppoly = sum(temppolyMat,1);

%-------------------------------------------------------------------
function p = polypow(q,N)
%POLYPOW   Evaluate polynomial to power of N.

% Return early if order is zero or negative
if N <= 0
    p = 1;
	return
end

% Initialize recursion
L = length(q);
p = [cast(1,class(q)) zeros(1,(L-1)*N,'like',q)];

% Multiply polynomial by q, N times
lenOld = 1;
lenNew = 1 + L -1;
for n = 1:N
    p(1:lenNew) = conv(p(1:lenOld),q);
    lenOld = lenNew;
    lenNew = lenNew + L - 1;
end
